      subroutine cbond(ngr,nrp,x,f,nf,eb,iprint)
C     B. P. van Eijck, February 1998.
C     Calculates energies of bonds: sum of 0.5 * CB * (length - B0)**2.
C     Is based on the GROMOS procedure COBOND by W. F. van Gunsteren.

C     Input through parameters:
C        NGR: number of groups
C        NRP: number of atoms
C        X(1..): atom coordinates
C        NF rules the calculation of derivatives:
C           NF=0: no derivatives
C           NF=1: calculates F
C           NF=2: calculates F and VIRA
C        IPRINT>1: bond distances and bond energies to screen and unit 16
C     Input through common block /BIND/:
C           NBOND: number of bonds
C           IB, JB (1..): atom numbers in pairs for each bond
C           CB, B0 (1..): force constants and reference bond lengths
C     Input through common block /ATTP/:
C        PANM(1..): atom names in topology file, used if IFIL>0

C     Output through parameters:
C        F(1..): increased with contributions from bond forces on atoms
C        EB: total bond energy
C     Output through common block /VIRA/:
C           VIRA(1:3,1:3): increased with contribution to virial

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cvira.i'
      double precision x,f,eb,rij2,rij,rij1,db,df,ebh,xh
      integer ngr,nrp,nf,iprint,k,n,ibn,jbn,i3,j3,m,m1
      dimension x(*),f(*),rij(3)

      eb=0
      if (nbond.eq.0) return
      if (iprint.gt.1) print 20
      if (iprint.gt.0) write(16,20)
      do k=1,ngr
         do n=1,nbond
            ibn=ib(n)+(k-1)*nrp
            jbn=jb(n)+(k-1)*nrp
            i3=3*ibn-3
            j3=3*jbn-3
            rij2=0
            do m=1,3
               rij(m)=x(i3+m)-x(j3+m)
               rij2=rij2+rij(m)**2
            enddo
            rij1=sqrt(rij2)
            DB=RIJ1-B0(n)
            DF=CB(n)*DB
            EBH=DF*DB/2
            EB=EB+EBH
            if (iprint.gt.1)
     +           print 30,ibn,jbn,panm(ibn),panm(jbn),rij1,
     +              ibcod(n),type(ibn),type(jbn),b0(n),cb(n),ebh
            if (iprint.gt.0)
     +           write(16,30) ibn,jbn,panm(ibn),panm(jbn),rij1,
     +              ibcod(n),type(ibn),type(jbn),b0(n),cb(n),ebh
            if (nf.gt.0) then
               do m=1,3
                  xh=rij(m)*df/rij1
                  f(i3+m)=f(i3+m)-xh
                  f(j3+m)=f(j3+m)+xh
                  if (nf.eq.2) then
                     do m1=1,3
                        vira(m,m1)=vira(m,m1)+xh*rij(m1)/2
                     enddo
                  endif
               enddo
            endif
         enddo
      enddo
 20   format('   Bonds   (kJ, nm)',25x,'Value     Topology',
     +     24x,'r0        kb        eb')
 30   format(2i4,13x,2a5,10x,f8.4,i8,3x,2a5,10x,f8.4,f10.0,f10.3)
      end




      subroutine choek(ngr,nrp,x,f,nf,eba,iprint)
C     B. P. van Eijck, February 1998.
C     Calculates energies of angles: sum of 0.5 * CT * (angle - T0)**2
C     Is based on the GROMOS procedure ANGLE by W. F. van Gunsteren.

C     Input through parameters:
C        NGR: number of groups
C        NRP: number of atoms
C        X(1..): atom coordinates
C        NF rules the calculation of derivatives:
C           NF=0: no derivatives
C           NF=1: calculates F
C           NF=2: calculates F and VIRA
C        IPRINT>1: angles and angle energies to screen and unit 16
C     Input through common block /HOEK/:
C           NHOEK: number of angles
C           IT, JT, KT (1..): atom numbers for each angle
C           CT, T0 (1..): force constants and reference angles
C     Input through common block /ATTP/:
C        PANM(1..): atom names in topology file, used if IFIL>0

C     Output through parameters:
C        F(1..): increased with contributions from angle forces on atoms
C        EBA: total angle energy
C     Output through common block /VIRA/:
C           VIRA(1:3,1:3): increased with contribution to virial

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cvira.i'
      integer ngr,nrp,nf,iprint,k,n,itn,jtn,ktn,i3,j3,k3,m,m1
      double precision x,f,eba,pi,rad,rij2,rkj2,rijk,rij,rkj,rij1,rkj1,
     +     cosa,at,da,df,ebah,vi2,vk2,vi,vk,dti,dtk
      dimension x(*),f(*),rij(3),rkj(3),vi(3),vk(3),dti(3),dtk(3)

      eba=0
      if (nhoek.eq.0) return
      if (iprint.gt.1) print 20
      if (iprint.gt.0) write(16,20)
      pi=4*atan(1.0d0)
      rad=180/pi
      do k=1,ngr
         do n=1,nhoek
            itn=it(n)+(k-1)*nrp
            jtn=jt(n)+(k-1)*nrp
            ktn=kt(n)+(k-1)*nrp
            i3=3*itn-3
            j3=3*jtn-3
            k3=3*ktn-3
            rij2=0
            rkj2=0
            rijk=0
            do m=1,3
               rij(m)=x(i3+m)-x(j3+m)
               rkj(m)=x(k3+m)-x(j3+m)
               rij2=rij2+rij(m)**2
               rkj2=rkj2+rkj(m)**2
               rijk=rijk+rij(m)*rkj(m)
            enddo
            rij1=sqrt(rij2)
            rkj1=sqrt(rkj2)
            cosa=rijk/(rij1*rkj1)
            at=acos(cosa)
            if (cosa.gt.0.99999) then
               if (iprint.gt.0) then
                  print*,' Cosine too large:', cosa
                  write(16,*) ' Cosine too large:', cosa
               endif
               at=0
               cosa=0.99999
            endif
            if (cosa.lt.-0.99999) then
               if (iprint.gt.0) then
                  print*,' Cosine too small:', cosa
                  write(16,*) ' Cosine too small:', cosa
               endif
               at=180/rad
               cosa=-0.99999
            endif
            DA=AT-T0(n)
            DF=CT(n)*DA
            EBAH=DF*DA/2
            EBA=EBA+EBAH
            if (iprint.gt.1)
     +           print 30,itn,jtn,ktn,panm(itn),panm(jtn),panm(ktn),
     +              rad*at,ihcod(n),type(itn),type(jtn),type(ktn),
     +              rad*t0(n),ct(n),ebah
            if (iprint.gt.0)
     +           write(16,30) itn,jtn,ktn,panm(itn),panm(jtn),panm(ktn),
     +              rad*at,ihcod(n),type(itn),type(jtn),type(ktn),
     +              rad*t0(n),ct(n),ebah 
            if (nf.gt.0) then
               vi2=0
               vk2=0
               do m=1,3
                  vi(m)=rkj(m)/rkj1-cosa*rij(m)/rij1
                  vk(m)=rij(m)/rij1-cosa*rkj(m)/rkj1
                  vi2=vi2+vi(m)**2
                  vk2=vk2+vk(m)**2
               enddo
               do m=1,3
                  dti(m)=-df*vi(m)/sqrt(rij2*vi2)
                  dtk(m)=-df*vk(m)/sqrt(rkj2*vk2)
               enddo
               do m=1,3
                  f(i3+m)=f(i3+m)-dti(m)
                  f(j3+m)=f(j3+m)+dti(m)+dtk(m)
                  f(k3+m)=f(k3+m)-dtk(m)
                  if (nf.eq.2) then
                     do m1=1,3
                        vira(m,m1)=vira(m,m1)+
     +                       (dti(m)*rij(m1)+dtk(m)*rkj(m1))/2
                     enddo
                  endif
               enddo
            endif
         enddo
      enddo
 20   format('   Angles   (kJ)',28x,'Value     Topology',
     +     24x,'t0        kt        et')
 30   format(3i4,9x,3a5,5x,f8.1,i8,3x,3a5,5x,f8.1,f10.2,f10.3)
      end




      subroutine cdiha(ngr,nrp,x,f,nf,impr,eda,iprint)
C     B. P. van Eijck, February 1998.
C     Calculates energies of dihedral angles (proper or improper).
C     Is based on the GROMOS procedure DIHANG by W. F. van Gunsteren.

C     Input through parameters:
C        NGR: number of groups
C        NRP: number of atoms
C        X(1..): atom coordinates
C        NF rules the calculation of derivatives:
C           NF=0: no derivatives
C           NF=1: calculates F
C           NF=2: calculates F and VIRA
C        IMPR decides what to calculate:
C           IMPR=1: improper dihedrals (harmonic potential):
C                   sum of 0.5 * CQ * (improper - Q0)[modulo 2 pi]**2
C           IMPR=2: proper dihedrals (cosine potential):
C                   sum of abs(CP) - CP * cos(NP * dihedral)
C        IPRINT>1: dihedrals and dihedral energies to screen and unit 16
C     Input through common blocks:
C        /IMPR/
C           NIMPR: number of impropers
C           IQ, JQ, KQ, LQ (1..): atom numbers for each dihedral
C           CQ, Q0 (1..): force constants and reference bond lengths
C        /TORS/
C           NTORS: number of propers
C           IP, JP, KP, LP (1..): atom numbers for each dihedral
C           CP, NP (1..): force constants and periodicities
C     Input through common block /ATTP/:
C        PANM(1..): atom names in topology file, used if IFIL>0

C     Output through parameters:
C        F(1..): increased with contributions from dihedral forces on atoms
C        EDA: total (im)proper dihedral energy
C     Output through common block /VIRA/:
C           VIRA(1:3,1:3): increased with contribution to virial

C     Calls subroutine PROD

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cvira.i'
      integer ngr,nrp,nf,impr,iprint,nda,k,n,iqn,jqn,kqn,lqn,
     +     i3,j3,k3,l3,ipn,jpn,kpn,lpn,m,narg,m1,ilin
      double precision x,f,eda,pi,rad,pyetwo,rij,rkj,rkl,sj,sk,sjsk,sj2,
     +     sk2,rkj2,cosa,sina,sj1,sk1,rkj1,ap,arg,sp,edah,rijk,rjkl,rlj,
     +     aj,ak,dpi,dpj,dpk,dpl
      dimension x(*),f(*),rij(3),rkj(3),rkl(3),rlj(3),
     +     sj(3),sk(3),sjsk(3),dpi(3),dpj(3),dpk(3),dpl(3)

      eda=0
      nda=0
      ilin=0
      if (impr.eq.1) then
         nda=nimpr
         if (iprint.gt.1) print 20
         if (iprint.gt.0) write(16,20)
      endif
      if (impr.eq.2) then
         nda=ntors
         if (iprint.gt.1) print 30
         if (iprint.gt.0) write(16,30)
      endif
      if (nda.eq.0) return

      pi=4*atan(1.0d0)
      rad=180/pi
      pyetwo=2*pi
      do k=1,ngr
         do n=1,nda
            if (impr.eq.1) then
               iqn=iq(n)+(k-1)*nrp
               jqn=jq(n)+(k-1)*nrp
               kqn=kq(n)+(k-1)*nrp
               lqn=lq(n)+(k-1)*nrp
               i3=3*iqn-3
               j3=3*jqn-3
               k3=3*kqn-3
               l3=3*lqn-3
            else
               ipn=ip(n)+(k-1)*nrp
               jpn=jp(n)+(k-1)*nrp
               kpn=kp(n)+(k-1)*nrp
               lpn=lp(n)+(k-1)*nrp
               i3=3*ipn-3
               j3=3*jpn-3
               k3=3*kpn-3
               l3=3*lpn-3
            endif

            do m=1,3
               rij(m)=x(i3+m)-x(j3+m)
               rkj(m)=x(k3+m)-x(j3+m)
               rkl(m)=x(k3+m)-x(l3+m)
            enddo

            call prod(rij,rkj,sj)
            call prod(rkj,rkl,sk)
            call prod(sj,sk,sjsk)
            sj2=0
            sk2=0
            rkj2=0
            cosa=0
            sina=0
            do m=1,3
               sj2=sj2+sj(m)**2
               sk2=sk2+sk(m)**2
               rkj2=rkj2+rkj(m)**2
               cosa=cosa+sj(m)*sk(m)
               sina=sina+rkj(m)*sjsk(m)
            enddo
            if (sj2.lt.1.0d-12) then
C              Division by',sj2,' in CDIHA'
               sj2=0.0000001
               ilin=1
            endif
            if (sk2.lt.1.0d-12) then
C              Division by',sk2,' in CDIHA'
               sk2=0.0000001
               ilin=1
            endif
            sj1=sqrt(sj2)
            sk1=sqrt(sk2)
            rkj1=sqrt(rkj2)
            cosa=cosa/(sj1*sk1)
            sina=sina/(sj1*sk1*rkj1)
            ap=atan2(sina,cosa)
            if (impr.eq.1) then
               ARG=AP-q0(n)
               NARG=NINT(ARG/pyetwo)
               ARG=ARG-NARG*PYETWO
               SP=cq(n)*ARG
               EDAH=SP*ARG/2
               EDA=EDA+EDAH
               if (ilin.eq.1) then
                  ap=0
                  edah=0
                  print 60,ipn,jpn,kpn,'improper dihedral'
                  write(16,60) ipn,jpn,kpn,'improper dihedral'
               endif
               if (iprint.gt.1)
     +              print 40,iqn,jqn,kqn,lqn,panm(iqn),panm(jqn),
     +                 panm(kqn),panm(lqn),ap*rad,
     +                 iicod(n),type(iqn),type(jqn),type(kqn),type(lqn),
     +                 q0(n)*rad,cq(n),edah
               if (iprint.gt.0)
     +              write(16,40) iqn,jqn,kqn,lqn,panm(iqn),panm(jqn),
     +                 panm(kqn),panm(lqn),ap*rad,
     +                 iicod(n),type(iqn),type(jqn),type(kqn),type(lqn),
     +                 q0(n)*rad,cq(n),edah
            else
               ARG=NP(n)*AP

C              OPLS reference energy
               if (np(n).eq.2) then 
                  EDAH=-CP(n)*(1-COS(ARG))
               else
                  EDAH=CP(n)*(1+COS(ARG))
               endif
C              UPACK formalism
               edah=abs(cp(n))+cp(n)*cos(arg)

               EDA=EDA+EDAH
               SP=-CP(n)*NP(n)*SIN(ARG)
               if (ilin.eq.1) then
                  ap=0
                  edah=0
                  print 60,ipn,jpn,kpn,'torsion'
                  write(16,60) ipn,jpn,kpn,'torsion'
               endif
               if (iprint.gt.1)
     +              print 50,ipn,jpn,kpn,lpn,panm(ipn),panm(jpn),
     +                 panm(kpn),panm(lpn),ap*rad,
     +                 itcod(n),type(ipn),type(jpn),type(kpn),type(lpn),
     +                 np(n),cp(n),edah
               if (iprint.gt.0)
     +              write(16,50) ipn,jpn,kpn,lpn,panm(ipn),panm(jpn),
     +                 panm(kpn),panm(lpn),ap*rad,
     +                 itcod(n),type(ipn),type(jpn),type(kpn),type(lpn),
     +                 np(n),cp(n),edah
            endif
cc            if (ilin.eq.1) return
            
            if (nf.gt.0 .and. ilin.ne.1) then
               rijk=0
               rjkl=0
               do m=1,3
                  rlj(m)=x(l3+m)-x(j3+m)
                  rijk=rijk+rij(m)*rkj(m)
                  rjkl=rjkl+rkj(m)*rkl(m)
               enddo
               aj=sp/(sj2*rkj1)
               ak=sp/(sk2*rkj1)
               do m=1,3
                  dpi(m)= aj*sj(m)*rkj2
                  dpj(m)=-aj*sj(m)*(rkj2-rijk) + ak*sk(m)*rjkl
                  dpk(m)=-aj*sj(m)*rijk + ak*sk(m)*(rkj2-rjkl)
                  dpl(m)=-ak*sk(m)*rkj2
               enddo
               do m=1,3
                  f(i3+m)=f(i3+m)-dpi(m)
                  f(j3+m)=f(j3+m)-dpj(m)
                  f(k3+m)=f(k3+m)-dpk(m)
                  f(l3+m)=f(l3+m)-dpl(m)
                  if (nf.eq.2) then
                     do m1=1,3
                        vira(m,m1)=vira(m,m1)+(dpi(m)*rij(m1)+
     +                       dpk(m)*rkj(m1)+dpl(m)*rlj(m1))/2
                     enddo
                  endif
               enddo
            endif
         enddo
      enddo
 20   format('   Impropers   (kJ)',25x,'Value     Topology',          
     +     24x,'q0        kq        eq')
 30   format('   Dihedrals   (kJ)',25x,'Value     Topology',          
     +     24x,' n        kq        eq')
 40   format(4i4,5x,4a5,f8.1,i8,3x,4a5,f8.1,f10.2,f10.3)
 50   format(4i4,5x,4a5,f8.1,i8,3x,4a5,i8,2f10.3)
 60   format(' ***** Linear chain in DIHA:' 3i4,'  Problem in ',a)
      end
