      subroutine cart(nrptot,x,iprint)

C     Constructs cartesian coordinates from internal coordinates.
C     In a first call a file CART.OUT is made. Every line contains indices of 
C     four atoms. This file must be copied to CART.IN and on each line the 
C     value of the corresponding dihedral angle (degrees) must be inserted.

C     If the file CART.IN is found, this procedure calculates the cartesian 
c     coordinates using these dihedrals, combined with bond lengths and bond 
C     angles from the topology file.

C     Input through parameters:
C        NRPTOT: number of atoms (possibly in several molecules)
C        X(1..): coordinates (rectangular, nm)
C        IPRINT regulates output to the screen.

C     Input from common block /BIND/:
C        NBOND: number of bonds
C        IB,JB(1..NBOND): code numbers for bonded atoms
C     Input through common block /HOEK/:
C        NHOEK: number of angles
C        IT, JT, KT (1..): atom numbers for each angle
C        T0 (1..): force constants and reference angles
C     Input through common block /ATTP/:
C        PANM(1..): atom names in topology file

C     Output parameters:
C        X(1..): coordinates (rectangular, nm)

      implicit none
      include 'params.i'
      include 'cbond.i'

      integer i,i1,i2,i3,i4,i5,iats,ibound,idone,inext,ios,iprint,inoh,
     +     iring,j,k,k1,k2,k3,k4,m,n,n4,nbound,ncyc,nfound,nrptot,nword
      double precision bondl,d,dih,dihan1,hoekl,pi,r,rad,theta,x,xprev
      dimension iats(4),x(maxxco),xprev(3),
     +     ibound(maxnrp,6),nbound(maxnrp),idone(maxnrp)
      character regel*80

      inoh=1
   50 continue
      do i=1,nrptot
         idone(i)=0
         if (inoh.eq.1 .and. panm(i)(1:1).eq.'H') idone(i)=100
      enddo
      do i=1,3*nrptot
         x(i)=0
      enddo
      ncyc=0
      nfound=0
      inext=0
      pi=4*atan(1.0d0)
      rad=180/pi

      open(41,file='cart.in',status='OLD',iostat=ios)
      if (ios.eq.0) goto 300

C     First part of the calculation. Prepare file CART.OUT
      open(42,file='cart.out')

C     Construct array IBOUND(i,j), where atoms j=1...NBOUND(i) are 
C        bound to atom i.
      do i=1,nrptot
         n=0
         do j=1,nbond
            k=0
            if (ib(j).eq.i) k=jb(j)
            if (jb(j).eq.i) k=ib(j)
            if (k.ne.0) then
               n=n+1
               ibound(i,n)=k
            endif
            nbound(i)=n
         enddo
         if (iprint.gt.3) print*,i,(ibound(i,k),k=1,nbound(i))
      enddo

      ncyc=0
      nfound=0
      inext=0
C     Elements of IDONE are set to 1 if its coordinates are known

  100 continue
C     Select four linearly connected unknown atoms
      do i1=1,nrptot
         if (idone(i1).eq.0) then
            do k1=1,nbound(i1)
               i2=ibound(i1,k1)
               if (idone(i2).eq.0) then
                  do k2=1,nbound(i2)
                     i3=ibound(i2,k2)
                     if (idone(i3).eq.0 .and. i3.ne.i1) then
                        do k3=1,nbound(i3)
                           i4=ibound(i3,k3)
                           if (idone(i4).eq.0 .and. i4.ne.i2) then
                              print 22,i1,i2,i3,i4,
     +                             panm(i1),panm(i2),panm(i3),panm(i4)
                              write(42,21) i1,i2,i3,i4,
     +                             panm(i1),panm(i2),panm(i3),panm(i4)
                              idone(i1)=1
                              idone(i2)=1
                              idone(i3)=1
                              idone(i4)=1
                              nfound=nfound+4
                              goto 110
                           endif
                        enddo
                     endif
                  enddo
               endif
            enddo
         endif
      enddo
      print*,' ***** No four atoms found!'
      if (nfound.eq.0 .and. inoh.eq.1) then
         print*,' Try again, this time with H atoms'
         inoh=0
         goto 50
      endif
      stop

  110 continue
      ncyc=ncyc+1
      if (iprint.gt.3) print*,' ====================== CYCLE',ncyc,
     +     ' =========================='
      n4=0
      do i4=1,nrptot
C        Look for connections with unknown atom I4
         if (idone(i4).eq.0) then
            do i3=1,nrptot
               if (idone(i3).eq.1) then
                  do k1=1,nbound(i3)
                     if (ibound(i3,k1).eq.i4) then
                        do k2=1,nbound(i3)
                           i2=ibound(i3,k2)
                           if (idone(i2).eq.1) then
                              do k3=1,nbound(i2)
                                 i1=ibound(i2,k3)
                                 if (idone(i1).eq.1 .and. i1.ne.i3) then
                                    print 21,i4,i3,i2,i1,panm(i1),
     +                                   panm(i2),panm(i3),panm(i4)
                                    write(42,21) i4,i3,i2,i1,panm(i1),
     +                                   panm(i2),panm(i3),panm(i4)
C                                   A connection has been found
                                    idone(i4)=idone(i4)-1
                                    n4=n4+1
c                                   Check for ring clusure
                                    do k4=1,nbound(i4)
                                       i5=ibound(i4,k4)
                                       if (idone(i5).ne.0 .and. 
     +                                      i5.ne.i3) then
                                          print 27,i4,i5,
     +                                         panm(i4),panm(i5)
                                          write(42,27) i4,i5,
     +                                         panm(i4),panm(i5)
                                       endif
                                    enddo
                                    goto 120
                                 endif
                              enddo
                           endif
                        enddo
                     endif
  120                continue
                  enddo
               endif
            enddo
         endif
      enddo
      do i=1,nrptot
         if (idone(i).lt.0) nfound=nfound+1
         if (idone(i).lt.-1) then
            print*,' Ring closure at atom',i
         endif
         idone(i)=abs(idone(i))
      enddo
      print*,n4,' new connections, total atoms found',nfound
      if (n4.ne.0) goto 110

      inext=inext+1
      print*,' Molecule', inext,' totalling',nfound,' atoms found'
      if (nfound.lt.nrptot) then
         print*,' There is another independent molecule'
         goto 100
      endif
      if (nfound.gt.nrptot) then
         print*,' Too many atoms found:', nfound,' rather than',nrptot
         stop
      endif
      print*,' *** File CART.OUT has been prepared, copy to CART.IN.'
      print*,' *** Insert here the dihedral values after each 4 indices'
      print*,' *** And rerun the program'
      stop

  300 continue
C     Second part of the calculation. Read dihedral values from CART.IN
      read(41,25,err=800,end=500) regel
      if (nword(regel).lt.9) goto 810
      call intstr(regel,1,i4)
      call intstr(regel,2,i3)
      call intstr(regel,3,i2)
      call intstr(regel,4,i1)
      call realst(regel,9,dih)
      if (iprint.gt.2) print 24,i4,i3,i2,i1,dih

  320 continue
C     I1 is the first atom. A subsequent molecule is shifted by 10 A.
      print*,' Starting new molecule',inext+1
      x(3*i1-2)=x(3*i1-2)+inext
      idone(i1)=1
C     I2 is the second atom, on the x-axis
      x(3*i2-2)=x(3*i1-2)+bondl(i1,i2)
      idone(i2)=1
C     I3 is the third atom, in the x-y plane
      idone(i3)=1
      theta=hoekl(i1,i2,i3)
      r=bondl(i2,i3)
      x(3*i3-2)=x(3*i2-2)-r*cos(theta)
      x(3*i3-1)=r*sin(theta)
      if (iprint.gt.2) then
         print*,i1,(x(3*i1-3+m),m=1,3)
         print*,i2,(x(3*i2-3+m),m=1,3)
         print*,i3,(x(3*i3-3+m),m=1,3)
      endif
      nfound=nfound+3

  340 continue
C     I4 is the next atom, from dihedral I1-I2-I3-I4
      iring=0
      if (idone(i4).eq.1) then
         do m=1,3
            xprev(m)=x(3*i4-3+m)
         enddo
         iring=iring+1
      else
         nfound=nfound+1
      endif
      idone(i4)=1
      r=bondl(i3,i4)
      theta=hoekl(i2,i3,i4)
c     print*,i1,i2,i3,i4,r,theta*rad,dih
      call addatom(i1,i2,i3,i4,r,theta,dih/rad,x)
      if (iring.eq.1) then
         d=0
         do m=1,3
            d=d+(x(3*i4-3+m)-xprev(m))**2
            x(3*i4-3+m)=(x(3*i4-3+m)+xprev(m))/2
         enddo
         print 26,i4,panm(i4),sqrt(d/3)
         write(16,26) i4,panm(i4),sqrt(d/3)
      endif
      if (iring.gt.1) then
         print*,' ***** Atom',i4,' found',iring,' times. Error?'
         print*,' Type ENTER if you wish to continue anyhow'
         read 25, regel
      endif

      if (iprint.gt.2) then
         iats(1)=i1
         iats(2)=i2
         iats(3)=i3
         iats(4)=i4
         print*,nfound,i4,(x(3*i4-3+m),m=1,3),dihan1(x,iats)*rad
      endif

      if (nfound.ne.nrptot) then
         read(41,25,err=800,end=500) regel
         if (nword(regel).lt.9) goto 810
         call intstr(regel,1,i4)
         call intstr(regel,2,i3)
         call intstr(regel,3,i2)
         call intstr(regel,4,i1)
         call realst(regel,9,dih)
         if (iprint.gt.2) print 24,i1,i2,i3,i4,dih
         if (idone(i1).eq.0 .and. idone(i2).eq.0 .and. idone(i3).eq.0
     +        .and. idone(i4).eq.0) then
            inext=inext+1
            goto 320
         endif
         goto 340
      endif

  500 continue
      print*,'***** Premature end of CART.IN'
      n=0
      do i=1,nrptot
         if (idone(i).lt.100) then
            n=n+1
            naam(n)=panm(i)
            do m=1,3
               x(3*n-3+m)=x(3*i-3+m)
            enddo
         endif
      enddo
      nrptot=n
      if (iprint.gt.3) print*,' Geometry construction in YCART ready'
      return
  800 print*,'***** Error reading file CART.IN'
      stop
  810 print*,'***** Insufficient data on line:'
      print*,regel
      stop

c   20 format(i5,3f10.4)
   21 format(4i4,3x,4a4,'   ')
   22 format(' Starting dihedral',4i4,3x,4a4)
c   23 format(4i4,19x,f40.1)
   24 format(4i4,f10.1)
   25 format(a)
   26 format('Atom',i4,3x,a4,' already done, probably ring closure.',
     +     ' RMS =',f6.3)
   27 format(2i4,2x,2a4,' Check ring closure data (remove this line!)')
      end



      subroutine addatom(i1,i2,i3,i4,r,theta,omega,x)
C     Adds atom ID to atom IC in the chain IA-IB-IC. Coordinates in array X.
C     Distance IC-ID is R, angle IB-IC-ID is theta, dihedral is omega
C     Angles in radians

      implicit none
      integer i1,i2,i3,i4,m
      double precision arot,brot,co,omega,r,rc,rs,si,theta,v1,v2,v3,x
      dimension x(*),v1(3),v2(3),v3(3),arot(3),brot(3)
      do m=1,3
         v1(m)=x(3*i3-3+m)-x(3*i2-3+m)
         v3(m)=x(3*i2-3+m)-x(3*i1-3+m)
      enddo
      call norm(v1)
      call prod(v1,v3,v2)
      call prod(v1,v2,v3)
      call norm(v3)
      rs=r*sin(theta)
      rc=r*cos(theta)
c      print*,i4,rs,rc,v3,v1
      do m=1,3
         x(3*i4-3+m)=x(3*i3-3+m)+v3(m)*rs-v1(m)*rc
      enddo
      co=cos(omega)-1
      si=sin(omega)
      call introt(3*i4-3,3*i3-3,3*i2-3,x,arot,brot)
c      print*,i4,co,si,arot,brot
      do m=1,3
         x(3*i4-3+m)=x(3*i4-3+m)+si*brot(m)+co*arot(m)
      enddo
      end




      function bondl(i1,i2)
C     Delivers bond length reference value I1-I2
C     Stops if this bond is not defined
C     Input from common block /BIND/:
C        NBOND: number of bonds
C        IB,JB(1..NBOND): code numbers for bonded atoms
C        B0(1..NBOND): distances to bonded H-atoms

      implicit none
      include 'params.i'
      include 'cbond.i'

      double precision bondl
      integer i1,i2,j
      bondl=-1
      do j=1,nbond
         if  ((ib(j).eq.i1 .and. jb(j).eq. i2) .or.
     +        (ib(j).eq.i2 .and. jb(j).eq. i1)) then
            bondl=b0(j)
            return
         endif
      enddo
      if (bondl.lt.0) then
         print*,' ***** Bond',i1,i2,' unknown'
         stop
      endif
      end




      function hoekl(i1,i2,i3)
C     Delivers bond angle reference value I1-I2-I3 in radians
C     Stops if this angle is not defined
C     Input through common block /HOEK/:
C           NHOEK: number of angles
C           IT, JT, KT (1..): atom numbers for each angle
C           T0 (1..): force constants and reference angles

      implicit none
      include 'params.i'
      include 'cbond.i'

      double precision hoekl
      integer i1,i2,i3,j
      hoekl=-1
      do j=1,nhoek
         if  ((it(j).eq.i1 .and. jt(j).eq. i2 .and. kt(j).eq.i3) .or.
     +        (it(j).eq.i3 .and. jt(j).eq. i2 .and. kt(j).eq.i1)) then
            hoekl=t0(j)
            return
         endif
      enddo
      if (hoekl.lt.0) then
         print*,' ***** Angle',i1,i2,i3,' unknown'
         stop
      endif
      end




      function dihan1(x,iats)
C     Returns the value (in radians) of a dihedral angle
C     Is extracted from procedure CDIHA.

C     Input through parameters:
C        X: array of atom coordinates
C        IATS(1..4): absolute values give the atom numbers of the dihedral

C     Calls subroutine PROD.

      implicit none
      integer i3,iats,j3,k3,l3,m
      double precision cosa,dihan1,rij,rkj,rkj1,rkj2,rkl,sina,
     +     sj,sj1,sj2,sjsk,sk,sk1,sk2,x

      dimension x(*),iats(*),rij(3),rkj(3),rkl(3),sj(3),sk(3),sjsk(3)

      i3=3*abs(iats(1))-3
      j3=3*abs(iats(2))-3
      k3=3*abs(iats(3))-3
      l3=3*abs(iats(4))-3

      do m=1,3
         rij(m)=x(i3+m)-x(j3+m)
         rkj(m)=x(k3+m)-x(j3+m)
         rkl(m)=x(k3+m)-x(l3+m)
      enddo

      call prod(rij,rkj,sj)
      call prod(rkj,rkl,sk)
      call prod(sj,sk,sjsk)
      sj2=0
      sk2=0
      rkj2=0
      cosa=0
      sina=0
      do m=1,3
         sj2=sj2+sj(m)**2
         sk2=sk2+sk(m)**2
         rkj2=rkj2+rkj(m)**2
         cosa=cosa+sj(m)*sk(m)
         sina=sina+rkj(m)*sjsk(m)
      enddo
      sj1=sqrt(sj2)
      sk1=sqrt(sk2)
      rkj1=sqrt(rkj2)
      cosa=cosa/(sj1*sk1)
      sina=sina/(sj1*sk1*rkj1)
      dihan1=atan2(sina,cosa)
      end
