      subroutine conn(nfound,nrp,ngr1,naam,x,cosa,cosb,cosc,iprint)

C     Finds connectivity from coordinates and calls IDENT
C     A complete group is assumed: no space group operations.      

C     Input through parameters:
C        NFOUND: number of atoms on coordinate file
C        NRP: number of atoms in the topology file
C        NAAM(1..): atom names
C        X(1..): coordinates (nm, non-rectangular)
C        IPRINT regulates the quantity of output.

C     Output through parameters:
C        NGR1: the number of topology groups
C        X(1..): reshuffeled coordinates (nm, non-rectangular)

      implicit none
      include 'params.i'

      integer nfound,nrp,ngr1,iprint
      integer i,igr,index,inew,j,k,k0,m,nconn,nind,nmol,at1,at2,omr,nomr
      double precision x,dx,dy,dz,rij,xr,dist,
     +  cosa,cosb,cosc
      dimension at1(maxat),at2(maxat),index(maxat),dist(2*maxat),
     +     igr(maxat),x(*),xr(maxxco),nomr(maxat),omr(maxat,12)
      character*8 naam(*),naam1(0:maxat)
      data inew /0/
      save inew

C     Determine connectivity from coordinates
      if (inew.eq.0) then
         print 37,nfound,nrp
         write(16,37) nfound,nrp
      endif
      inew=1

      do i=1,nfound
         do m=1,3
            xr(3*i-3+m)=x(3*i-3+m)
         enddo
         nomr(i)=0
         do m=1,12
            omr(i,m)=0
         enddo
c         print*,i,(xr(3*i-3+m),m=1,3)
      enddo

      nconn=0
      do i=1,nfound
         naam1(i)=naam(i)
c        print*,'Checking atom',i
         do j=i+1,nfound
C           Determine shortest distance i,j
            dx=xr(3*i-2)-xr(3*j-2)
            dy=xr(3*i-1)-xr(3*j-1)
            dz=xr(3*i  )-xr(3*j  )
            rij=sqrt(dx**2+dy**2+dz**2+
     +           2*(dx*dy*cosc+dx*dz*cosb+dy*dz*cosa))
            if (naam(i)(1:1).eq.'H' .and. naam(j)(1:1).eq.'H')
     +           goto 190
            if ((naam(i)(1:1).eq.'H' .or. naam(j)(1:1).eq.'H') .and. 
     +           rij.gt.0.130) goto 190
c            if ((naam(i)(1:1).eq.'F' .or. naam(j)(1:1).eq.'F') .and. 
c     +           rij.gt.0.160) goto 190
c            if ((naam(i)(1:1).eq.'I' .or. naam(j)(1:1).eq.'I') .and. 
c     +           rij.lt.0.215) goto 180
            if (rij.gt.0.25) goto 190
            nconn=nconn+1
            at1(nconn)=i
            at2(nconn)=j
            nomr(i)=nomr(i)+1
            nomr(j)=nomr(j)+1
            omr(i,nomr(i))=j
            omr(j,nomr(j))=i
            dist(nconn)=rij
            if (iprint.gt.2) print 43,nconn,naam(i),naam(j),i,j,rij
  190       continue
         enddo
      enddo
      print*,nconn,' bonds (inclusive 1..3) from coordinates'
      write(16,*) nconn,' bonds (inclusive 1..3) from coordinates'

C     Eliminate 1..3 (and perhaps also 1..4) interactions
      k0=0
      do k=1,nconn
         if (dist(k).gt.0.160) then
            if (iprint.gt.1) print*,k,dist(k),' between',at1(k),at2(k)
            do i=1,nfound
               do m=1,nomr(i)
                  if (omr(i,m).eq.at1(k))then
                     do j=1,nomr(i)
                        if (omr(i,j).eq.at2(k)) then
                           dist(k)=0
                           if (iprint.gt.1) print*,'Eliminated by atom',
     +                           i,' bonds',m,j
                        endif
                     enddo
                  endif
               enddo
            enddo
         endif
      enddo

C     Remove 1...3 interacties from distance list
      print*,' After elimination'
      write(16,*) ' After elimination'
      k0=0
      do k=1,nconn
         if (dist(k).gt.0.01) then
            k0=k0+1
            index(k0)=k
            at1(k0)=at1(k)
            at2(k0)=at2(k)
         endif
      enddo
      nconn=k0
      print*,nconn,' bonds found, 1..3 eliminated'
      write(16,*) nconn,' bonds found, 1..3 eliminated'
c      do i=1,nconn
c         print*,i,at1(i),at2(i)
c      enddo
      
C     Check for separate molecules
      do i=1,nfound
         igr(i)=0
      enddo
      nmol=1
      k=0
      igr(1)=1
  200 continue
      k0=k
      do k=1,nconn
         do i=1,nfound
            if (igr(i).eq.nmol) then
C              Search for atoms j to be attached to i
               j=0
               if (at1(k).eq.i) j=at2(k)
               if (at2(k).eq.i) j=at1(k)
               if (j.gt.0) igr(j)=nmol
            endif
         enddo
      enddo
      k=0
      do i=1,nfound
         if (igr(i).eq.nmol) k=k+1
      enddo
      if (k.gt.k0) goto 200
      if (iprint.gt.1) print*,'Molecule ',nmol,' found with',k,' atoms'
      do i=1,nfound
c         print*,i,igr(i)
         if (igr(i).eq.0) then
            k=0
            nmol=nmol+1
            igr(i)=nmol
            goto 200
         endif
      enddo
      print*,nmol,' separate molecules found'
      write(16,*) nmol,' separate molecules found'

C     Find connectivity and shells from topology
      call ident(nrp,nfound,nconn,naam1,igr,at1,at2,
     +     ngr1,nmol,index,iprint)
      if (ngr1.eq.0) then
         print*,' CONN found error in IDENT'
         write(16,*) ' CONN found error in IDENT'
         return
      endif
      nind=ngr1*nrp
      if (iprint.gt.1) then
         print*,nfound,' atoms found'
         print*,nmol,' molecules found'
         print*,ngr1,' topology groups found'
         print*,nind,' atoms should be present'
         write(16,*) nfound,' atoms found'
         write(16,*) nmol,' molecules found'
         write(16,*) ngr1,' topology groups found'
         write(16,*) nind,' atoms should be present'
      endif

C     Check presence of H atoms and rearrange coordinates
      naam1(0)='absent'
      do i=1,nind
         j=index(i)
         naam(i)=naam1(j)
         if (j.eq.0) then
            if (iprint.gt.1) print*,' H atom',i,' absent'
            if (iprint.gt.1) write(16,*) ' H atom',i,' absent'
            do m=1,3
               x(3*i-3+m)=0
            enddo
         else
            do m=1,3
               x(3*i-3+m)=xr(3*j-3+m)
            enddo
         endif
      enddo
               
c      do i=1,nind
c         print*,i,'   ',naam(i),(x(3*i-3+m),m=1,3)
c      enddo

      print*,'END CONN'
      write(16,*) 'END CONN'

   37 format(' CONN determines the connectivity for',2i5,' coordinates')
   43 format(i4,4x,2a5,2i4,f8.3)
      end
