      program crysmd

C     Performs molecular dynamics run.
C     Adapted from the GROMOS procedure PROMD, for crystal simulation only.
C     Thanks are due to W. F. van Gunsteren for his permission to do so.

C     Only atomic virial, no restraining of positions, distances
C     or dihedrals, no solvent, no free energy calculations,
C     no switch or twin-range in energy.
C     Only one independent molecule, expanded with equivalent positions into
C     one unit cell, can be studied. This is, of course, a very unrealistic
C     simulation, only meant to "shake up" the system to leave a possible
C     saddle point in the potential energy.

C     For NGR>1 the topology is expanded for NGR molecules.
C     Input of space group number through ISPGR.
C     For explanationm of other parameters see manual and RUNMD.

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cbond.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cpola.i'
      include 'cnbkl.i'
      include 'cwald.i'
      include 'color.i'

      integer i,i3,icont,iexp,ig,inpt,ios,iprint,iref,ispgr,itest,
     +     ix,iy,iz,j,k,k3,m,n,nfound,ngr,ngr0,nop3,nrp,nrps,nrun,
     +     nscm,nsnb,nsporg,nstlim,ntc,ntp,ntpr,ntpw,ntt,ntwe,ntwv,
     +     ntwx,nx,ny,nz
      double precision alf6md,alfcmd,ax,bb,box,boxav,bx,by,cc,comp,
     +     cosa,cosb,cosc,cut6md,cutcmd,cx,cy,cz,dipo,dt,epot,etot,
     +     pi,pres0,rad,rcutmd,t,taup,tautp,temp,temp0,tempi,
     +     tol,tolemd,tolkmd,v,x,xav
      dimension x(maxxco),v(maxxco),
     +     xav(maxxco),boxav(6),iref(12),itest(3),dipo(3),box(6)
      character*8 title(16),spname
      character titel*80, regel*200

      open(unit=21,file='in.xvi',status='OLD',iostat=ios)
      if (ios.ne.0) then
         print*,' ***** No coordinate file IN.XVI'
         stop
      endif
      open(unit=14,file='out.ave')
      open(unit=16,file='out.pri')
      open(unit=19,file='out.xvo')
      print*,geel,' C R Y S M D',wit
      write(16,*) ' '
      write(16,*) ' C R Y S M D'

C     READ DATA CHARACTERIZING THE MD-RUN
      open(unit=13,file='in.md',status='OLD',iostat=ios)
      regel=' '
      if (ios.eq.0) n=inpt('INPUT ',regel)
      close(13)

      call leest(nrp,1,1)
      n=inpt('CUTOFF',regel)
      if (n.ge.2) call realst(regel,2,cutoff)
      if (n.ge.3) then
         if (n.lt.8) then
            print*,' ***** Wrong number of items in CUTOFF',n
            stop
         endif
         call realst(regel,3,alphac)
         call realst(regel,4,alpha6)
         call realst(regel,5,hmaxc)
         call realst(regel,6,hmax6)
         call realst(regel,7,tole)
         call realst(regel,8,tolk)
      endif
      rcutmd=cutoff
      alfcmd=alphac
      alf6md=alpha6
      cutcmd=hmaxc
      cut6md=hmax6
      tolemd=tole
      tolkmd=tolk
      ngr0=1
      ispgr=1
      pres0=0
      iprint=0
      iexp=0
      nx=1
      ny=1
      nz=1

      n=inpt('SPGR  ',regel)
      spname='noname'
      if (n.ge.2) call intstr(regel,2,ispgr)
      n=inpt('NGR   ',regel)
      if (n.ge.2) call intstr(regel,2,ngr0)
      n=inpt('PRINT ',regel)
      if (n.ge.2) call intstr(regel,2,iprint)
      n=inpt('EXPAND',regel)
      if (n.gt.0) iexp=1
      if (n.ge.4) then
         call intstr(regel,2,nx)
         call intstr(regel,3,ny)
         call intstr(regel,4,nz)
      endif

      write(16,*) '     ngr0     ispgr       npp    iprint'
      write(16,51) ngr0,ispgr,npp,iprint
      if (npp.gt.1) then
         write(16,*) '    npp  poltol  schaal   gamma     nnn     mmm',
     +        '     nte    nttt'
         write(16,29) npp,poltol,schaal,gampol,nnn,mmm,nte,nttt
      endif
      call leesmd(rcutmd,alfcmd,alf6md,cutcmd,cut6md,tolemd,tolkmd,
     +     tempi,temp0,tautp,pres0,comp,taup,t,dt,tol,
     +     ig,nrun,ntt,ntp,nscm,nstlim,ntc,nsnb,
     +     ntpr,ntwx,ntwv,ntwe,ntpw)
      i=inpt('CHECK ',regel)

      inner=1
      call leesr(ispgr,spname,iref,itest,msom,inner)

      print*,' Number of atoms',nrp
      write(16,*) ' Number of atoms',nrp

      ebig=1000000
      noint=.false.
      inner=1
      cutoff=rcutmd
      alphac=alfcmd
      alpha6=alf6md
      hmaxc=cutcmd
      hmax6=cut6md
      tole=tolemd
      tolk=tolkmd
      doewc=(alphac.gt.1.0d-9)
      doew6=(alpha6.gt.1.0d-9)
      doew=(doewc .or. doew6)
      if (doewc) inner=0
      nsporg=0

C     READ COORDINATES AND VELOCITIES
      pi=4*atan(1.0d0)
      rad=180/pi
      read(21,11,err=430) titel
      print*,' Coordinate file: ',titel(1:50)
      write(16,*) ' Coordinate file: ',titel(1:50)
      read(titel,12) title
      read(21,72,err=430) nfound
      do i=1,nfound
         read(21,72,err=430) k,panm(i),k,
     +        (x(3*(i-1)+m),m=1,3),(v(3*(i-1)+m),m=1,3)
      enddo
      read(21,*,end=440,err=440) (box(m),m=1,6)
      goto 450
 430  continue
      print*,' Error reading MDI file'
      stop
 440  continue
      print*,' No cell given in MDI file'
      stop
 450  continue
      close(21)
      icont=0
      if (nfound.ne.nrp*ngr0) then
         if (iexp.eq.1 .and. nfound.ne.nrp*ngr0*(nspgr+1)*nx*ny*nz) then
            print*,' N unequal NRP*NGR0*EXPAND',nfound,nrp,ngr0,
     +           nspgr,nx,ny,nz
            stop
         else
c            icont=1
            print*,' ***** N unequal NRP*NGR0',nfound,nrp,ngr0
            ngr0=nfound/nrp
            if (nfound.eq.nrp*ngr0) then
               print*,' Changing NGR0 to',ngr0
            else
               stop
            endif
         endif
      endif
C     Convert from (a,b,c,alfa,beta,gamma) to (ax,by,cz,bx,cx,cy)
      bb=box(2)
      cc=box(3)
      cosa=cos(box(4)/rad)
      cosb=cos(box(5)/rad)
      cosc=cos(box(6)/rad)
      ax=box(1)
      cx=cc*cosb
      bx=bb*cosc
      by=sqrt(bb*bb-bx*bx)
      cy=(bb*cc*cosa-bx*cx)/by
      cz=sqrt(cc*cc-cx*cx-cy*cy)
      box(1)=ax
      box(2)=by
      box(3)=cz
      box(4)=bx
      box(5)=cx
      box(6)=cy
      ngr=ngr0*nx*ny*nz
      if (icont.eq.1 .and. iexp.gt.0) then
         ngr=ngr*(nspgr+1)
         nrps=nrp*ngr0*(nspgr+1)
         nsporg=nspgr
         nspgr=0
         ispgr=1
         spname='P1'
         do m=1,3
            msom(m)=1
            iref(m+3)=1
            iref(m+6)=0
         enddo
      endif

C     Expand equivalent positions to P1
      if (icont.eq.0 .and. iexp.gt.0) then
         print*,' Expand equivalent positions to P1' 
         write(16,*) ' Expand equivalent positions to P1' 
         box(1)=ax*nx
         box(2)=by*ny
         box(3)=cz*nz
         box(4)=bx*ny
         box(5)=cx*nz
         box(6)=cy*nz
         ngr=ngr*(nspgr+1)
         if (ngr0*nrp.gt.maxat) then
            print*,' ***** Increase MAXAT',ngr0,nrp,maxat
            stop
         endif
         do nop=1,nspgr
            nop3=3*nop-3
            do i=1,nrp
               j=3*i-3
               k=j+3*nop*nrp
               x(k+1)=msp(nop3+1)*x(j+1)+
     +              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               x(k+2)=msp(nop3+2)*x(j+2)+
     +              tsp(nop3+2)*by+tsp(nop3+3)*cy
               x(k+3)=msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
            enddo
         enddo
         nrps=nrp*ngr0*(nspgr+1)
         nsporg=nspgr
         nspgr=0
         ispgr=1
         spname='P1'
         do m=1,3
            msom(m)=1
            iref(m+3)=1
            iref(m+6)=0
         enddo

         k=nrps
C        Expand equivalent cells
         print*,' Expand equivalent cells:',nx,ny,nz
         write(16,*) ' Expand equivalent cells:',nx,ny,nz
         do ix=0,nx-1
            do iy=0,ny-1
               do iz=0,nz-1
                  if (ix.ne.0 .or. iy.ne.0 .or. iz.ne.0) then
                     do i=1,nrps
                        k=k+1
                        i3=3*i-3
                        k3=3*k-3
                        x(k3+1)=x(i3+1)+ix*ax+iy*bx+iz*cx
                        x(k3+2)=x(i3+2)+iy*by+iz*cy
                        x(k3+3)=x(i3+3)+iz*cz
c                        print*,ix,iy,iz,i,k,(x(k3+m),m=1,3)
                     enddo
                  endif
               enddo
            enddo
         enddo
      endif

      finner=16*atan(1.0d0)/(3*(nspgr+1))
      druk=pres0/(nspgr+1)

C     CALCULATE DIPOLE MOMENTS
      do m=1,3
         dipo(m)=0
      enddo
      j=0
      do k=1,ngr0
         do i=1,nrp
            do m=1,3
               j=j+1
               dipo(m)=dipo(m)+cg(i)*x(j)
            enddo
c            print*,k,i,cg(i),j,x(j-2),x(j-1),x(j)
         enddo
      enddo
      print 66,(dipo(m),m=1,3)
      write(16,66) (dipo(m),m=1,3)

C     WRITE TITLE TO TAPES WHEN REQUIRED (16a5 if unformatted!)
      if (ntwx.ne.0) then
         if (ntpw.eq.0) then
            open(unit=17,file='out.co',form='UNFORMATTED')
            write(17) title
         else
            open(unit=17,file='out.co')
            write(17,11) titel
         endif
      endif
      if (ntwv.ne.0) then
         if (ntpw.eq.0) then
            open(unit=18,file='out.ve',form='UNFORMATTED')
            write(18) title
         else
            open(unit=18,file='out.ve')
            write(18,11) titel
         endif
      endif
      if (ntwe.ne.0) then
         open(unit=15,file='out.en')
         write(15,11) titel
      endif

C     PERFORM MD-RUN(S)
      call runmd(ngr,nrp,nbond,box,x,v,panm,
     +     temp,epot,etot,boxav,xav,iprint,nx,ny,nz,nsporg,
     +     tempi,temp0,tautp,pres0,comp,taup,t,dt,tol,
     +     ig,nrun,ntt,ntp,nscm,nstlim,ntc,nsnb,
     +     ntpr,ntwx,ntwv,ntwe,ntpw)

C     WRITE TITLE, FINAL COORDINATES AND VELOCITIES, AND BOX
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      write(19,11) titel
      write(19,95) nrp*ngr
      do j=1,nrp*ngr
         k=mod(j-1,nrp)+1
         write(19,72) 1,panm(k),j,
     +        (x(3*j-3+m),m=1,3),(v(3*j-3+m),m=1,3)
      enddo
      write(19,93) ax,bb,cc,rad*acos(cosa),
     +     rad*acos(cosb),rad*acos(cosc)
C     Write MDI file for average independent coordinates and box
      ax=boxav(1)/nx
      by=boxav(2)/ny
      cz=boxav(3)/nz
      bx=boxav(4)/ny
      cx=boxav(5)/nz
      cy=boxav(6)/nz
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      write(14,11) titel
      write(14,95) nrp*ngr0
      do j=1,nrp*ngr0
         k=mod(j-1,nrp)+1
         write(14,72) 1,panm(k),j,(xav(3*j-3+m),m=1,3)
      enddo
      write(14,93) ax,bb,cc,rad*acos(cosa),
     +     rad*acos(cosb),rad*acos(cosc)

 11   format(a80)
 12   format(16a5)
c 23   format('  Ewald method for Coulomb energy is used; ',
c     +     'alphac =',f7.2,' hmaxc =',f7.2)
c 24   format('  Ewald method for L-J R-6 energy is used; ',
c     +     'alpha6 =',f7.2,' hmax6 =',f7.2)
 29   format(i8,3f8.4,4i8)
c 31   format(1x,a79)
 51   format(5i10)
 66   format('  Dipole moment asymm. unit (* 4.0745 for Debye): ',3f8.3)
 72   format(i5,5x,a5,i5,6f8.4)
 93   format(3f10.5,3f10.3)
c 94   format(3e14.7)
 95   format(i5,2x,a40)
      end
