      subroutine dclus(icount,istruc,nindex,ener,vol,dener,rvol,
     +     import,isel,izoek,ihit,iprint)

C     Subroutine to perform clustering on basis of interatomic distances.
C     Relies upon list of distances made in DLIST
C     Written bij W.T.M. Mooij, adapted by B.P. van Eijck

C     Input through parameters:
C     ICOUNT: the number of lists with structures
C     ISTRUC: the total number of the structures
C     NINDEX(ISTRUC): indicates the place where the structure ISTRUC
C         can be found when it must be ordered to energy
C     ENER(ISTRUC): energy of the structure ISTRUC
C     VOL(ISTRUC): molecular volume of the structure ISTRUC
C     IMPORT(ISTRUC): the structure with lowest IMPORTance is removed
C     DENER: allowed energy difference between two structures
C     RVOL: allowed relative volume difference between two structures
C     ISEL = 0: discard structure with highest energy
C            1: discard structure from list with lowest number
C            2: discard structure from list with highest number
C     IZOEK = 0: do clustering
C             1: do search. Then ISTRUC = 2.
C     IPRINT regulates output to unit 16 and to screen.

C     Input through common blocks:
C     /NAAM/ gives various constants:
C        MPOW:   distances are muliplied by MPOW to make them integer.
C        MDIFF:  tolerance DIFF (nm) for equivalence of distances,
C                multiplied by MPOW to make it integer
C        MCUT:   [cutoff distance (nm) - 1.1 * dist],
C                multiplied by MPOW to make it integer
C            The distance list is based on CUTOFF, the actual comparison on
C                this slightly smaller distance.
C        MARGIN: number of contacts per molecule that are allowed not to be
C                found (zero or small).
C     /CLUS/ gives various integer arrays:
C        STRLST(ISTRUC): sequence code number of the structure ISTRUC
C        ZFLAG(ISTRUC): the number of unequivalent molecules
C        SLIST(ISTRUC): the position in the array MDIST before the one where
C                 the first distance of the structure ISTRUC is placed.
C        TELLER(ISTRUC,mol=1...ZFLAG(ISTRUC)): the position of the last
C                distance of molecule MOL in the list of distances.
C        MDIST(1...): the array of distances, multiplied by MPOW. The last
C                distance of molecule MOL in the structure ISTRUC can be found
C                in the position SLIST(ISTRUC)+TELLER(ISTRUC,MOL) in this arra
C        ISTART(1...ICOUNT): number of first structure of a list
C        IEND(1...ICOUNT):   number of last structure of a list
C        NCLUS(ISTRUC): the number of equivalent structures already collected

C     Output through parameters (only in search mode):
C     IHIT = 1 if ISTRUC corresponds to the experimental structure
C          = 0 otherwise

C     Output through common block /CLUS/ (only in clustering mode):
C     NCLUS(ISTRUC): the new number of structures that are equivalent to ISTRUC
C     STRFLG(ISTRUC) = 1 if structure ISTRUC is removed
C                    = 0 otherwise

      implicit none
      include 'cdist.i'
      integer i,i1,i2,icount,idents,ihit,ilist1,ilist2,
     +     ilmin,imp1,imp2,import,iprint,irem,is1,is2,
     +     isel,istruc,izoek,j,m1,mdd,mindd,
     +     n,n1,n2,niclus,nindex,ns1,ns2,nst,nst1,nst2
      double precision d,d1,d2,dd,de,dener,ener,rvol,vol
      dimension nindex(maxstr),ener(maxstr),vol(maxstr),import(*)
      integer molflg(maxmol),atflg(maxcon),faults,hits

      if (izoek.eq.0 .or. iprint.gt.1) then
         print*,'   icount    istruc      mpow     mdiff      mcut',
     +        '    margin      isel'
         print 16,icount,istruc,mpow,mdiff,mcut,margin,isel
         write(16,*) '   icount    istruc     mpow      mdiff',
     +        '      mcut    margin      isel'
         write(16,16) icount,istruc,mpow,mdiff,mcut,margin,isel
      endif
      if (iprint.gt.3) then
         print*,'i,nindex(i),zflag(i),strflg(i),strlst(i)'
         do i=1,istruc
            print*,i,nindex(i),zflag(i),strflg(i),strlst(i)
         enddo
         print*,'i,(teller(i,j),j=1,zflag(i))'
         do i=1,istruc
            print*,i,(teller(i,j)-teller(i,j-1),j=1,zflag(i))
         enddo
         print*,'i,istart(i),iend(i)'
         do i=1,icount
            print*,i,istart(i),iend(i)
         enddo
      endif
      if (iprint.gt.4) then
         print*,'i,j,k,from(i,j,k),to(i,j,k),d(i,j,k)'
         do n=1,istruc
            do j=1,zflag(n)
               print*,j,teller(n,j-1)+1,teller(n,j)
               do i=teller(n,j-1)+1,teller(n,j)
                  nst=slist(n)
                  d=mod(mdist(i+nst),mpow)/dfloat(mpow)
                  print 14,n,j,i-teller(n,j-1),d
               enddo
            enddo
         enddo
      endif

C     Check for identical surroundings for the different structures.
      irem=0
      ihit=0
      niclus=0
      do i=1,istruc
         strflg(i)=0
         iclus(i)=0
c         nclus(i)=1
      enddo

C     Do loop in increasing energy over structures
      do is1=1,istruc-1
         ns1=nindex(is1)
         if (iclus(ns1).eq.0) then
            niclus=niclus+1
            iclus(ns1)=niclus
c            print*,strlst(ns1), ' creating cluster',niclus
         endif
c         print*,'NCLUS',strlst(ns1),iclus(ns1)
         if (strflg(ns1).eq.1) goto 985
C        Determine which input file this came from
         if (izoek.eq.0) then
            do i=1,icount
               if (ns1.ge.istart(i) .and. ns1.le.iend(i)) ilist1=i
            enddo
         endif
         if (iprint.gt.2) then
c            if (abs(ener(ns1)).lt.1.0e8) then
            print 22, strlst(ns1),ener(ns1),vol(ns1)
            write(16,22) strlst(ns1),ener(ns1),vol(ns1)
c            else
c               print 22, strlst(ns1),0,vol(ns1)
c               write(16,22) strlst(ns1),0,vol(ns1)
c            endif
         endif

C        Do loop in increasing energy over structures
         do is2=is1+1,istruc
            ns2=nindex(is2)
            if ((strflg(ns2).eq.1).or.(zflag(ns1).ne.
     +           zflag(ns2))) goto 984
C           Determine which input file this came from
            if (izoek.eq.0) then
               do i=1,icount
                  if (ns2.ge.istart(i) .and. ns2.le.iend(i)) ilist2=i
               enddo
            else
               ilist2=icount
            endif

C           MOLFLAG is needed when more than one independent molecule is
C           present. It keeps track of identicals in the different molecules
C           of the two structures.
            do i=1,zflag(ns1)
               molflg(i)=0
            enddo
            idents=0
            if (iprint.gt.2) then
c               if (abs(ener(ns2)).lt.1.0e8) then
               print 23, strlst(ns2),ener(ns2),vol(ns2)
               write(16,23) strlst(ns2),ener(ns2),vol(ns2)
c               else
c                  print 23, strlst(ns2),0,vol(ns2)
c                  write(16,23) strlst(ns2),0,vol(ns2)
c               endif
            endif
            de=abs(ener(ns1)-ener(ns2))
            if (de.gt.dener .and. de.lt.1.0e8) goto 984
            if (abs(vol(ns1)-vol(ns2))/vol(ns2).gt.rvol) goto 984
C           Do loop over the independent molecules of structure1
            do n1=1,zflag(ns1)
C              Do loop over the independent molecules of structure2
               do n2=1,zflag(ns2)
C                 Check if this molecule is already found:
                  if (molflg(n2).eq.1) goto 982
                  faults=0
C                 Atom flag ATFLG keeps track of 'used' contact
                  do i=1,teller(ns2,n2)
                     atflg(i)=0
                  enddo
C                 Do loop over all the contacts of molecules n1 in structure1
C                 contacts within diff from distance cut-off are disregarded
                  do i1=teller(ns1,n1-1)+1,teller(ns1,n1)
                     nst1=slist(ns1)
                     m1=mdist(i1+nst1)
                     if (mod(m1,mpow).gt.mcut) goto 981
                     hits=0
                     mindd=100000000
C                    Select the closest corresponding distance in structure 2.
                     do i2=teller(ns2,n2-1)+1,teller(ns2,n2)
                        if (atflg(i2).eq.0) then
                           nst2=slist(ns2)
                           mdd=abs(m1-mdist(i2+nst2))
                           if (iprint.gt.5) then
                              d1=mod(mdist(i1+nst1),mpow)/dfloat(mpow)
                              d2=mod(mdist(i2+nst2),mpow)/dfloat(mpow)
                              dd=abs(d1-d2)
                              print 15,n1,i1,d1,n2,i2,d2,dd
                           endif
                           if (mdd.lt.mdiff) then
                              hits=hits+1
                              if (mdd.lt.mindd) then
                                 mindd=mdd
                                 ilmin=i2
                              endif
                           endif
                        endif
                     enddo
                     if (hits.eq.0) then
                        if (faults.ge.margin) goto 982
                        faults=faults+1
                        goto 981
                     else
                        atflg(ilmin)=1
C                       This contact is now marked as used
                     endif
  981                continue
                  enddo
C                 If the program gets here all the contacts in the molecule on
C                 are found in molecule two:
                  molflg(n2)=1
                  idents=idents+1
  982             continue
               enddo
            enddo
            if (idents.eq.zflag(ns1)) then
               if (izoek.eq.1) then
C                 Another hit was found in the search
                  ihit=1
                  print 20,strlst(ns2),ilist2,zflag(ns2),
     +                 ener(ns2)-ener(ns1),(vol(ns2)-vol(ns1))/vol(ns1)
                  write(16,20) strlst(ns2),ilist2,zflag(ns2),
     +                 ener(ns2)-ener(ns1),(vol(ns2)-vol(ns1))/vol(ns1)
               else
C                 The clustering removes one structure; note E(ns1) < E(ns2).
                  imp1=import(ilist1)
                  imp2=import(ilist2)
C                 Create clusters
                  if (iclus(ns2).eq.0) then
                     iclus(ns2)=iclus(ns1)
                  else
c                     print*,'Clusters equivalent',iclus(ns1),iclus(ns2)
                     do i=1,istruc
                        if (iclus(i).eq.iclus(ns2)) iclus(i)=iclus(ns1)
                     enddo
                  endif
                  if  ((isel.eq.1 .and. imp2.gt.imp1) .or.
     +                 (isel.eq.2 .and. imp2.lt.imp1)) then
C                    Discard structure NS1
C                    ISEL = 1: this structure has the lowest importance
C                    ISEL = 2: this structure has the highest importance
                     if (isel.ne.3) strflg(ns1)=1
                     nclus(ns2)=nclus(ns2)+nclus(ns1)
                     irem=irem+1
                     if (iprint.gt.0) then
                        print 12, strlst(ns1),ilist1,strlst(ns2),
     +                       ilist2,zflag(ns2),ener(ns1)-ener(ns2),
     +                       (vol(ns1)-vol(ns2))/vol(ns2)
                        write(16,12) strlst(ns1),ilist1,strlst(ns2),
     +                       ilist2,zflag(ns2),ener(ns1)-ener(ns2),
     +                       (vol(ns1)-vol(ns2))/vol(ns2)
                     endif
                     goto 985
                  else
C                    Discard structure NS2
C                    ISEL = 0: this structure has the highest energy
C                    ISEL = 1: this structure has the lowest importance
C                    ISEL = 2: this structure has the highest importance
                     if (isel.ne.3) strflg(ns2)=1
                     nclus(ns1)=nclus(ns1)+nclus(ns2)
                     irem=irem+1
                     if (iprint.gt.0) then
                        print 12, strlst(ns2),ilist2,strlst(ns1),
     +                       ilist1,zflag(ns1),ener(ns2)-ener(ns1),
     +                       (vol(ns2)-vol(ns1))/vol(ns1)
                        write(16,12) strlst(ns2),ilist2,strlst(ns1),
     +                       ilist1,zflag(ns1),ener(ns2)-ener(ns1),
     +                       (vol(ns2)-vol(ns1))/vol(ns1)
                     endif
                  endif
               endif
            else
c               print*,'No hit:',strlst(ns1),strlst(ns2)
               if (iclus(ns2).eq.0) then
                  niclus=niclus+1
                  iclus(ns2)=niclus
               endif
            endif
  984       continue
         enddo
  985    continue
      enddo
      if (izoek.eq.0) then
         print 13,irem,istruc
         write(16,13) irem,istruc
      endif
      do i=1,istruc
         if (iclus(i).eq.0) then
            niclus=niclus+1
            iclus(i)=niclus
         endif
      enddo

   12 format(i6,' from list',i3,' removed against',i6,
     +     ' from list',i3,' Z" =',i3,
     +     '   Energy diff. ',f8.3,'     Rel.vol.diff.',f10.5)
   13 format(i8,' structures removed out of',i6)
   14 format(3i4,f8.3)
   15 format(2i4,f8.3,4x,2i4,2f8.3)
   16 format(10i10)
   20 format(' Experimental structure corresponds to structure',i6,
     +     ' from list',i3,' Z" =',i3,
     +     '   Energy diff. ',f8.3,'     Rel.vol.diff.',f10.5)
   22 format(' Structure A:               ',i6,1x,f10.3,f10.5)
   23 format('              Structure B:  ',i6,1x,f10.3,f10.5)
      end
