      subroutine diff(box1,box2,a1r1,r2a2,iref,tol,iprint,
     +     ierr,dif1,dif2,box1pp,wrot)

C     B.P. van Eijck, december 1994.
C     Compares two crystal structures, BOX1 en BOX2.
C     Transforms BOX1 into BOX1PP which should be equal to BOX2.

C     Input through parameters:
C        BOX1, BOX2(1..6): cel-dimensions ax,by,cz,bx,cx,cy
C        BOX1, BOX2(7..9): centers of gravity
C        A1R1 = A1(inv).R1
C        R2A2 = R2(inv).A2
C        IREF: indicates whether parameter can be refined (see LEESR)
C        TOL: maximum allowable deviation from integer in N
C        IPRINT: output if IPRINT > 5, larger IPRINT gives more output
C     Input through common block:
Cccc        /SPGR/ gives space group information:
Cccc           NSPGR, MSP, TSP: see subroutine LEESR

C     Output parameters:
C        IERR: error confition:
C           IERR=0: transformation possible, no problems
C                 (except that DIF2 may be large, to be tested externally)
C           IERR=1: elements of N deviate too much from TOL
C           IERR=2: det N not equal 1
C           IERR=3: M1, M2 not equivalent
C           IERR=4: s' not integer
C              The following should not occur:
C           IERR=5: inconsistent results for tau
C           IERR=6: shift should not be free in this space group
C           IERR=7: shift should be free in this space group
C           IERR=8: angle should be 90 degrees in this space group
C           IERR=9: volumes different after transformation
C        DIF1: largest deviation from integer in N (within TOL)
C        DIF2: largest difference between BOX2 en BOX1PP
C        BOX1PP(1..9): the transformed box
C        WROT(1..3,1..3): the rotation matrix W

C     Calls subroutine TRANSF.

      implicit none
      include 'params.i'
      integer i,icond,ierr,iprint,iref,j,k,m,n,n1,n2,n3,nin
      double precision a,a1r1,ax,box1,box1pp,box2,bx,by,cx,cy,cz,
     +     d1,d2,d3,dif1,dif2,r2a2,s,scr,tol,wrot

      dimension box2(*),box1(*),box1pp(*),iref(*),wrot(3,3),
     +     nin(3,3),icond(4),a1r1(3,3),r2a2(3,3),scr(3)

      if (iprint.gt.5) then
         do i=1,3
            do j=1,3
               scr(j)=0
               do k=1,3
                  scr(j)=scr(j)+a1r1(i,k)*r2a2(k,j)
               enddo
            enddo
            print 90,(scr(j),j=1,3),(nint(scr(j)),j=1,3)
         enddo
      endif

      dif1=0
      do i=1,3
         do j=1,3
            a=0
            do k=1,3
               a=a+a1r1(i,k)*r2a2(k,j)
            enddo
            n=nint(a)
            s=abs(n-a)
            if (s.gt.dif1) then
               dif1=s
               if (s.gt.tol) then
                  ierr=1
                  return
               endif
            endif
            nin(i,j)=n
         enddo
      enddo
C     NIN = [A1(inv).R1] . [R2(trans).A2] = N

C     Do the transformation, calculate WROT
      call transf(box1,nin,box1pp,icond,iref,wrot,iprint)
      ierr=icond(4)
      if (iprint.gt.5) print 70,ierr
      if (ierr.ne.0) return

C     Check whether translations over half the cell axes are needed
      ax=box1pp(1)
      by=box1pp(2)
      cz=box1pp(3)
      bx=box1pp(4)
      cx=box1pp(5)
      cy=box1pp(6)
      d1=2*(box2(7)-box1pp(7))
      d2=2*(box2(8)-box1pp(8))
      d3=2*(box2(9)-box1pp(9))
      n3=nint(d3/cz)
      n2=nint((d2-n3*cy)/by)
      n1=nint((d1-n2*bx-n3*cx)/ax)
      box1pp(7)=box1pp(7)+(n1*ax+n2*bx+n3*cx)/2
      box1pp(8)=box1pp(8)+(n2*by+n3*cy)/2
      box1pp(9)=box1pp(9)+(n3*cz)/2

C     Determine the largest deviation
      dif2=0
      do m=1,9
         if (iref(m).ne.0) then
            s=abs(box2(m)-box1pp(m))
            if (s.gt.dif2) dif2=s
         endif
      enddo

      if (iprint.gt.5) then
         print 80,'Box1    ',(box1(m),m=1,9)
         print 80,'Box2    ',(box2(m),m=1,9)
         print 80,'Box1pp  ',(box1pp(m),m=1,9)
      endif

 70   format(25x,'Status after transformation:',i5)
 80   format(25x,a8,3(3f8.4,3x),3f12.3)
 90   format(25x,'N - matrix    ',3f8.3,5x,3i5)
      end
