      function dihan(n,x)

C     Returns the value (in radians) of dihedral angle numbered N.
C     Is extracted from procedure CDIHA.

C     Input through parameters:
C        N: number of the dihedral
C        X: array of atom coordinates
C     Input through common block /INDS/:
C        IOME, JOME, KOME, LOME: arrays containing the atom numbers
C            of the dihedral angles

C     Calls subroutine PROD.

      implicit none
      include 'params.i'
      include 'cinds.i'
      double precision cosa,dihan,rij,rkj,rkj1,rkj2,rkl,sina,
     +     sj,sj1,sj2,sjsk,sk,sk1,sk2,x
      integer i3,j3,k3,l3,m,n
      dimension x(*),rij(3),rkj(3),rkl(3),sj(3),sk(3),sjsk(3)

      i3=3*iome(n)-3
      j3=3*jome(n)-3
      k3=3*kome(n)-3
      l3=3*lome(n)-3

      do m=1,3
         rij(m)=x(i3+m)-x(j3+m)
         rkj(m)=x(k3+m)-x(j3+m)
         rkl(m)=x(k3+m)-x(l3+m)
      enddo

      call prod(rij,rkj,sj)
      call prod(rkj,rkl,sk)
      call prod(sj,sk,sjsk)
      sj2=0
      sk2=0
      rkj2=0
      cosa=0
      sina=0
      do m=1,3
         sj2=sj2+sj(m)**2
         sk2=sk2+sk(m)**2
         rkj2=rkj2+rkj(m)**2
         cosa=cosa+sj(m)*sk(m)
         sina=sina+rkj(m)*sjsk(m)
      enddo
      if (sj2.lt.1.0d-12) then
         print*,' Division by',sj2,' in DIHAN'
         goto 100
      endif
      if (sk2.lt.1.0d-12) then
         print*,' Division by',sk2,' in DIHAN'
         goto 100
      endif
      sj1=sqrt(sj2)
      sk1=sqrt(sk2)
      rkj1=sqrt(rkj2)
      cosa=cosa/(sj1*sk1)
      sina=sina/(sj1*sk1*rkj1)
      dihan=atan2(sina,cosa)
      return

 100  continue
C     Error status
      print 60,jome(n),kome(n),kome(n),lome(n)
      print 70,iome(n),(x(i3+m),m=1,3)
      print 70,jome(n),(x(j3+m),m=1,3)
      print 70,kome(n),(x(k3+m),m=1,3)
      print 70,lome(n),(x(l3+m),m=1,3)
c     stop
      dihan=0
 60   format(' ***** Parallel bonds ',i4,' --',i4,' and',i4,' --',i4)
 70   format(i5,3f10.5)
      end
