      subroutine expsym(nrgls,regels,nop,mspg,tspg,iprint)

C     Interprets symmetry generators and expand these to the full group

C     Input through parameters:
C     NRGLS: number of "cards" with symmetry generators
C     REGELS(1...NRGLS): cards (a80) containing e.g. SYMM x+1/2, y-x+3/4,z+3/4
C     IPRINT regulates output to unit 16 and to screen.

C     Output through parameters:
C     NOP: the total number of generated operations (E included)
C     MSPG: the rotation components
C     TSPG: the translation components

      implicit none
      integer i,i0,i9,ic,iklaar,il,inum,iprint,is,isl,itel,itr,j,
     +     m,m1,m2,m3,me,mn,mscr,mspg,mspgin,ngen,nop,nprev,nrgls,ntal
      double precision a,tn,tscr,tspg,tspgin
      dimension mspg(1000,3,3),tspg(1000,3),
     +     mspgin(3,3),tspgin(3),mn(3,3),tn(3),me(3,3),tscr(3),mscr(3,3)
      character*10 um(3)
      character*80 regel, regels(*)
      character code*4,ch*1

      i0=ichar('0')
      i9=ichar('9')
      nop=1
      do m1=1,3
         tspg(1,m1)=0
         do m2=1,3
            me(m1,m2)=0
            mspg(1,m1,m2)=0
         enddo
         mspg(1,m1,m1)=1
         me(m1,m1)=1
      enddo

      do ngen=1,nrgls
         regel=regels(ngen)
         code=regel(1:4)
         if (code.ne.'symm' .and. code.ne.'SYMM') then
            print*,' ***** Wrong symmetry code transmitted to EXPSYM'
            stop
         endif
         do m1=1,3
            tspgin(m1)=0
            do m2=1,3
               mspgin(m1,m2)=0
            enddo
         enddo
         m=1
         il=4
         is=1
         itr=0
         isl=0
 130     continue
C        Read next character
         il=il+1
         if (il.eq.80) goto 140
         if (il.gt.80) then
            print*,' ***** Error in generator in EXPSYM',ngen
            print*,regel
            stop
         endif
         ch=regel(il:il)
         if (ch.eq.' ') goto 130
         if (ch.eq.'-') is=-1
         if (ch.eq.'+') is=1
         if (ch.eq.'x' .or. ch.eq.'X') mspgin(m,1)=is
         if (ch.eq.'y' .or. ch.eq.'Y') mspgin(m,2)=is
         if (ch.eq.'z' .or. ch.eq.'Z') mspgin(m,3)=is
         if (ch.eq.'/') isl=1
         if (ch.eq.',') goto 140
         ic=ichar(ch)
         if (ic.ge.i0 .and. ic.le.i9) read(ch,12) itr
         if (isl.eq.0) then
            itel=itr*is
         else
            inum=itr
         endif
         goto 130

 140     continue
         if (isl.eq.0 .or. inum.eq.0) then
            tspgin(m)=itel
         else
            tspgin(m)=dfloat(itel)/dfloat(inum)
         endif
         if (m.lt.3) then
            m=m+1
            is=1
            isl=0
            itr=0
            goto 130
         endif

         if (iprint.gt.1) then
            print 13, ngen
            write(16,13) ngen
         endif
         if (iprint.gt.3) then
            do m1=1,3
               print 23,(mspgin(m1,m2),m2=1,3),tspgin(m1)
               write(16,23) (mspgin(m1,m2),m2=1,3),tspgin(m1)
            enddo
         endif
         nprev=nop
C        Check whether this is a new one
         do i=1,nop
            do m1=1,3
               do m2=1,3
                  if (mspgin(m1,m2).ne.mspg(i,m1,m2)) goto 185
               enddo
               a=tspgin(m1)-tspg(i,m1)
               if (abs(a-nint(a)).gt.0.01d0) goto 185
            enddo
            if (iprint.gt.1) then
               print 18,ngen
               write(16,18) ngen
            endif
            goto 700
 185        continue
         enddo

C        Evaluate a new symmetry generator
         ntal=0
         do m1=1,3
            tn(m1)=0
            do m2=1,3
               mn(m1,m2)=me(m1,m2)
            enddo
         enddo

C        Do the next operation NTAL of this generator
 255     ntal=ntal+1
         iklaar=1
C        Calculate M(n+1) = M(1).M(n) and T(n+1) = M(1).T(n)+T(1)
         do m1=1,3
            tscr(m1)=tn(m1)
            tn(m1)=tspgin(m1)
            do m2=1,3
               mscr(m1,m2)=mn(m1,m2)
               mn(m1,m2)=0
            enddo
         enddo
         do m1=1,3
            do m2=1,3
               tn(m1)=tn(m1)+mspgin(m1,m2)*tscr(m2)
               do m3=1,3
                  mn(m1,m2)=mn(m1,m2)+mspgin(m1,m3)*mscr(m3,m2)
               enddo
               if (mn(m1,m2).ne.me(m1,m2)) iklaar=0
            enddo
            if (abs(tn(m1)).gt.0.95) tn(m1)=tn(m1)-nint(tn(m1))
            if (abs(tn(m1)).gt.0.1) iklaar=0
         enddo
         if (iprint.gt.3) then
            do m1=1,3
               print 23,(mn(m1,m2),m2=1,3),tn(m1)
            enddo
         endif

C        The last operation has been done if IKLAAR=1
         if (iklaar.eq.1) then
            if (iprint.gt.1) then
               print 16,ngen,ntal
               write(16,16) ngen,ntal
            endif
            goto 700
         endif

C        Apply this operator to a known operation I
         do i=1,nprev
            nop=nop+1
            do m1=1,3
               tspg(nop,m1)=tn(m1)
               do m2=1,3
                  mspg(nop,m1,m2)=0
               enddo
            enddo
            do m1=1,3
               do m2=1,3
                  tspg(nop,m1)=tspg(nop,m1)+mn(m1,m2)*tspg(i,m2)
                  do m3=1,3
                     mspg(nop,m1,m2)=
     +                    mspg(nop,m1,m2)+mn(m1,m3)*mspg(i,m3,m2)
                  enddo
               enddo
               if (abs(tspg(nop,m1)).gt.0.99)
     +              tspg(nop,m1)=tspg(nop,m1)-nint(tspg(nop,m1))
            enddo

C           Check whether the result was already known
            do j=1,nop-1
               do m1=1,3
                  do m2=1,3
                     if (mspg(j,m1,m2).ne.mspg(nop,m1,m2)) goto 435
                  enddo
                  a=tspg(j,m1)-tspg(nop,m1)
                  if (abs(a-nint(a)).gt.0.01d0) goto 435
               enddo
               if (iprint.gt.1) then
                  print 14,ngen,ntal,i
                  write(16,14) ngen,ntal,i
               endif
               nop=nop-1
               goto 495
 435           continue
            enddo

            if (iprint.gt.1) then
               print 15,ngen,ntal,i,nop
               write(16,15) ngen,ntal,i,nop
            endif
            if (iprint.gt.3) then
               do m1=1,3
                  print 23,(mspg(nop,m1,m2),m2=1,3),tspg(nop,m1)
               enddo
            endif
 495        continue

         enddo
         goto 255

 700     continue
      enddo
      if (iprint.gt.1) then
         print 17,nop
         write(16,17) nop
      endif

C     Write results to file, if desired for unknown reason
      if (iprint.gt.5) then
         open(unit=20,file='expsym.out')
         do i=1,nop
            do m=1,3
               um(m)='          '
               if (mspg(i,m,1).gt.0) um(m)(1:2)='+x'
               if (mspg(i,m,1).lt.0) um(m)(1:2)='-x'
               if (mspg(i,m,2).gt.0) um(m)(3:4)='+y'
               if (mspg(i,m,2).lt.0) um(m)(3:4)='-y'
               if (mspg(i,m,3).gt.0) um(m)(5:6)='+z'
               if (mspg(i,m,3).lt.0) um(m)(5:6)='-z'
               a=tspg(i,m)
               if (abs(a-0.5000d0) .lt. 0.0002) um(m)(7:10)='+1/2'
               if (abs(a-0.2500d0) .lt. 0.0002) um(m)(7:10)='+1/4'
               if (abs(a-0.7500d0) .lt. 0.0002) um(m)(7:10)='+3/4'
               if (abs(a-0.3333d0) .lt. 0.0002) um(m)(7:10)='+1/3'
               if (abs(a-0.6667d0) .lt. 0.0002) um(m)(7:10)='+2/3'
               if (abs(a-0.1667d0) .lt. 0.0002) um(m)(7:10)='+1/6'
               if (abs(a-0.8333d0) .lt. 0.0002) um(m)(7:10)='+5/3'
               if (abs(a+0.5000d0) .lt. 0.0002) um(m)(7:10)='+1/2'
               if (abs(a+0.2500d0) .lt. 0.0002) um(m)(7:10)='+3/4'
               if (abs(a+0.7500d0) .lt. 0.0002) um(m)(7:10)='+1/4'
               if (abs(a+0.3333d0) .lt. 0.0002) um(m)(7:10)='+2/3'
               if (abs(a+0.6667d0) .lt. 0.0002) um(m)(7:10)='+1/3'
               if (abs(a+0.1667d0) .lt. 0.0002) um(m)(7:10)='+5/6'
               if (abs(a+0.8333d0) .lt. 0.0002) um(m)(7:10)='+1/3'
            enddo
            write(20,24) (um(m),m=1,3)
         enddo
      endif

 12   format(i1)
 13   format('   Symmetry generator',i3,' interpreted')
 14   format('   Symmetry generator',i3,' multiplicity',i3,
     +     ' acting on operation',i3,' creates no new one')
 15   format('   Symmetry generator',i3,' multiplicity',i3,
     +     ' acting on operation',i3,' creates new operation',i3)
 16   format('   Symmetry generator',i3,' multiplicity',i3,' finished')
 17   format('   Symmetry analysis finished:',i4,' operations')
 18   format('   Symmetry generator',i3,' is already known')
 23   format(3i5,f10.5)
 24   format('symm  ',a10,',   ',a10,',   ',a10)
      end
