      subroutine files(iout1,nummer,filtit,ener,spname,nrp,panm,x,box)
C     Make coordinate file for one structure, format MDI or SPF.
C     In both formats output cell data are a, b, c, alpha, beta, gamma,
C     but units are nanometers (MDI) and Angstrom (SPF). Output atomic
C     coordinates are Cartesian (MDI, nm) or oblique and fractional (SPF).
      
C     Input through parameters:
C     NUMMER: name of the file to be made:
C        IOUT=1: Make file NUMMER.MDI.
C        IOUT=2: Make file NUMMER.SPF.
C        IOUT=5: Make file NUMMER.XYZ
C     FILTIT: title (first line) of the file to be made
C     ENER: real number (e.g. the energy) written to the SPF output file
C     SPNAME: name of the space group written to the SPF output file 
C     NRP: number of atoms
C     PANM(1..): names of atoms
C     X(1..): coordinates
C     BOX(1..6): ax, by, cz, bx, cx, cy

C     Input through common block /SPGR/: space group information:
C        NSPGR, MSP, TSP: see subroutine LEESR
C     Input through common block /NONB/: charge information:
C        CG(1..)

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cnonb.i'
      integer i,icel,iout,iout1,iwr,j,k,m,mloc,n,n1,n1min,n2,n2min,
     +     n3,n3min,natm,nop3,nopm,nrp,ntit,nummer,nword
      double precision alpha,ax,bb,beta,box,bx,by,cc,cosa,cosb,cosc,
     +     cx,cy,cz,d,d1,d2,d3,dmin,ener,gamma,pi,rad,t,tloc,x,x1,x2,x3,
     +     facq
      dimension x(*),box(*),mloc(0:3),tloc(0:3)
      character*(*) filtit
      character*7 filenm*10,titel*80,ch*1,num*5,symx,symy,symz
      character*8 panm(maxat),spname

      iout=iout1
      iwr=5
      if (iout1.ge.10) then
         iout=iout1-10
         iwr=4
      endif
      ntit=nword(filtit)
      if (ntit.eq.0) then
C        Change spname to lower case, omit slashes, and add file number
         n=0
         do i=1,8
            ch=spname(i:i)
            if (ch.ne.' ' .and. ch.ne.'/') then
               k=ichar(ch)
               if (ch.ge.'A' .and. ch.le.'Z') k=k+32
               n=n+1
               titel(n:n)=char(k)
            endif
         enddo
         titel(n+1:n+1)='-'
         write(num,53) nummer
         titel=titel(1:n+1)//num
c         print*,titel
c         pause
      else
         titel=filtit
      endif

      pi=4*atan(1.0d0)
      rad=180/pi
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
c      if (abs(ax-0.1d0).gt.0.0001 .or. abs(bx).gt.0.0001 .or.
c     +     abs(by-0.1d0).gt.0.0001 .or. abs(cx).gt.0.0001 .or.
c     +     abs(cz-0.1d0).gt.0.0001 .or. abs(cy).gt.0.0001) then
      if (abs(ax-0.1d0).gt.0.0001d0 .or. abs(by-0.1d0).gt.0.0001d0 .or.
     +     abs(cz-0.1d0).gt.0.0001d0) then
         icel=1
         bb=sqrt(bx**2+by**2)
         cc=sqrt(cx**2+cy**2+cz**2)
         cosa=(bx*cx+by*cy)/(bb*cc)
         cosb=cx/cc
         cosc=bx/bb
      else
         icel=0
         bb=0.1d0
         cc=0.1d0
         cosa=0
         cosb=0
         cosc=0
      endif
c      print*,'FILES',icel,(box(m),m=1,6),ax,by,cz,bx,cx,cy
      alpha=rad*acos(cosa)
      beta=rad*acos(cosb)
      gamma=rad*acos(cosc)

c      irfat1=11
c      irfat2=18
      if (irfat1.ne.0 .and. (nspgr.gt.0 .or. spname.eq.'P1')) then
C        Find equivalent postions that give the shortest distance between
C        atom IRFAT1 and all atoms in the "molecule" starting from IRFAT2
         dmin=10000
         do nop=0,nspgr
            do m=1,3
               if (nop.eq.0) then
                  mloc(m)=1
                  tloc(m)=0
               else
                  mloc(m)=msp(3*nop-3+m)
                  tloc(m)=tsp(3*nop-3+m)
               endif
             enddo
             do n1=-3,3
               do n2=-3,3
                  do n3=-3,3
                     do i=irfat2,nrp
                        j=3*i-3
                        x1=mloc(1)*x(j+1)+(tloc(1)+n1)*ax+
     +                       (tloc(2)+n2)*bx+(tloc(3)+n3)*cx
                        d1=x1-x(3*irfat1-2)
                        x2=mloc(2)*x(j+2)+
     +                       (tloc(2)+n2)*by+(tloc(3)+n3)*cy
                        d2=x2-x(3*irfat1-1)
                        x3=mloc(3)*x(j+3)+(tloc(3)+n3)*cz
                        d3=x3-x(3*irfat1)
                        d=sqrt(d1**2+d2**2+d3**2)
                        if (d.lt.dmin) then
                           dmin=d
                           nopm=nop
                           natm=i
                           n1min=n1
                           n2min=n2
                           n3min=n3
c               print*,dmin,nopm,natm,n1min,n2min,n3min,
c     +                          x(j+2)/by,x2/by,x(3*irfat1-1)/by
                        endif
                     enddo
                  enddo
               enddo
            enddo
         enddo
C        The operator NOPM gives the shortest distance: DMIN to atom NATM.
C        The corresponding cell shifts are N1MIN1, N2MIN, N3MIN.
C        Transform atoms IRFAT2 ... NRP:
         do m=1,3
            if (nopm.eq.0) then
               mloc(m)=1
               tloc(m)=0
            else
               mloc(m)=msp(3*nopm-3+m)
               tloc(m)=tsp(3*nopm-3+m)
            endif
         enddo
         do i=irfat2,nrp
            j=3*i-3
            x(j+1)=mloc(1)*x(j+1)+(tloc(1)+n1min)*ax+
     +           (tloc(2)+n2min)*bx+(tloc(3)+n3min)*cx
            x(j+2)=mloc(2)*x(j+2)+(tloc(2)+n2min)*by+
     +           (tloc(3)+n3min)*cy
            x(j+3)=mloc(3)*x(j+3)+(tloc(3)+n3min)*cz
         enddo
C        Check distance:
         i=3*irfat1-3
         j=3*natm-3
         d=0
         do m=1,3
            d=d+(x(i+m)-x(j+m))**2
         enddo
         print 57,nopm,n1min,n2min,n3min,irfat1,natm,sqrt(d)
         write(16,57) nopm,n1min,n2min,n3min,irfat1,natm,sqrt(d)
         irfat1=0
         irfat2=0
      endif

      if (iout.eq.1) then
C     Make .MDI file. It contains expanded equivalent positions 
C        (but not NRP; to use the expanded file change NRP and remove 
C        the first output line giving BOX)
         if (iwr.eq.4) write(filenm,24) nummer,'.mdi'
         if (iwr.eq.5) write(filenm,25) nummer,'.mdi'
         open(17,file=filenm)
         write(17,27) titel
         write(17,72) nrp
         do i=1,nrp
            write(17,72) 1,panm(i)(1:5),i,(x(3*i-3+m),m=1,3)
         enddo
         if (icel.eq.1) write(17,73) box(1),bb,cc,alpha,beta,gamma
         if (nspgr.gt.0) then
            do nop=1,nspgr
               nop3=3*nop-3
               do i=1,nrp
                  j=3*i-3
                  k=j+3*nop*nrp
                  x(k+1)=msp(nop3+1)*x(j+1)+
     +                 tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
                  x(k+2)=msp(nop3+2)*x(j+2)+
     +                 tsp(nop3+2)*by+tsp(nop3+3)*cy
                  x(k+3)=msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
                  write(17,72) nop+1,panm(i)(1:5),
     +                 i+nop*nrp,(x(k+m),m=1,3)
               enddo
            enddo
         endif
         if (icel.eq.1) write(17,73) box(1),bb,cc,alpha,beta,gamma
         close(17)
      endif

      if (iout.eq.5) then
C     Make XYZ file
         facq=sqrt(332.0636d0*0.4184d0)
         if (iwr.eq.4) write(filenm,24) nummer,'.xyz'
         if (iwr.eq.5) write(filenm,25) nummer,'.xyz'
         open(17,file=filenm)
         write(17,72) nrp
         write(17,30) 10.0*ax,0.0,0.0,10.0*bx,10.0*by,
     +        0.0,10.0*cx,10.0*cy,10.0*cz
         do i=1,nrp
            j=ichar(panm(i)(2:2))
            if (j.ge.ichar('0') .and. j.le.ichar('9')) then
               write(17,28) panm(i)(1:1),
     +              (10.0*x(3*i-3+m),m=1,3),cg(i)/facq
            else
               write(17,29) panm(i)(1:2),
     +              (10.0*x(3*i-3+m),m=1,3),cg(i)/facq
            endif
         enddo
      endif

      if (iout.ge.2) then
C        Make .SPF file for PLUTON and PLATON.
C        Transform X to oblique coordinates
         call tri(nrp,0,x,1,cosa,cosb,cosc)
         if (iwr.eq.4) write(filenm,24) nummer,'.spf'
         if (iwr.eq.5) write(filenm,25) nummer,'.spf'
         if (iout.lt.4) then
            open(17,file=filenm)
            write(17,26) titel
         endif
         if (icel.eq.1) then
            if (iout.lt.4) write(17,50)
     +           10*box(1),10*bb,10*cc,alpha,beta,gamma
         endif
         if (ener.gt.99999.0d0) ener=99999.0d0
         if (iout.lt.4) write(17,55) ener
c         if (spname(1:2).ne.'no' .and. spname(1:1).ne.'0') then
c            if (iout.lt.4) write(17,54) spname
c         endif
         if (spname(1:1).ge.'A' .and. spname(1:1).le.'Z') then
            if (iout.lt.4) write(17,54) spname
         else
            if (iout.lt.4) write(17,56) spname
         endif

         do j=1,nspgr
C           Write space group symmetry operations
            i=3*j-2
            symx='  x    '
            if (msp(i).eq.-1) symx(2:2)= '-'
            t=tsp(i)-0.5d0
            t=t-nint(t)
            if (abs(t).lt.0.002) symx(4:7)='+1/2'
            i=3*j-1
            symy='  y    '
            if (msp(i).eq.-1) symy(2:2)= '-'
            t=tsp(i)-0.5d0
            t=t-nint(t)
            if (abs(t).lt.0.002) symy(4:7)='+1/2'
            i=3*j
            symz='  z    '
            if (msp(i).eq.-1) symz(2:2)= '-'
            t=tsp(i)-0.5d0
            t=t-nint(t)
            if (abs(t).lt.0.002) symz(4:7)='+1/2'
            if (iout.lt.4) write(17,63) symx,symy,symz
         enddo

         do i=1,nrp
            if (iout.lt.4) write(17,52) panm(i),x(3*i-2)/box(1),
     +           x(3*i-1)/bb,x(3*i)/cc
         enddo
         if (iout.lt.4) write(17,70)

         close(17)
C        Transform X back to rectangular coordinates
C        This introduces minor changes in the coordinates X!
         call tri(nrp,0,x,-1,cosa,cosb,cosc)
      endif

   24 format(i4.4,a4)
   25 format(i5.5,a4)
   26 format('TITL',2x,a60)
   27 format(2x,a65)
   28 format(1x,a1,2x,3(f9.5,1x),f9.5)
   29 format(1x,a2,1x,3(f9.5,1x),f9.5)
   30 format('Lattice=',9f9.5)
   50 format('CELL ',3f10.5,3f10.3)
c   51 format('TITL',2x,a8,i7.5,2x,a55)
   52 format('ATOM ',a8,3x,3f10.6)
   53 format(i5.5)
   54 format('SPGR ',a8)
   55 format('ENER ',f10.3)
   56 format('--COMMENT SPGR ',a8)
   57 format('Applied operator',i4,2x,'Shifts',3i5,
     +     4x,'New distance between atoms',2i4,f10.5)
   63 format('SYMM ',a7,' , ',a7,' , ',a7)
   70 format('ENDS')
   72 format(i5,5x,a5,i5,6f8.4)
   73 format(3f10.5,3f10.3)
      end




      subroutine fdat(iout1,ispgr,
     +     nummer,filtit,ener,spname,nrp,ngr,x,box)
C     Make coordinate file for one structure, format FDAT or CSSR
C     Written by W. T. M. Mooy.
C     Addition by B. P. van Eijck: format CIF enabled.

C     Input through parameters:
C     IOUT: 1: write CIF file
C           2: write CSSR file, without charges
C           3: write CSSR file, with charges
C           4: write FDAT file
C     ISPGR: space group number
C     NUMMER: make file NUMMER.DAT
C     NRP: number of atoms
C     ENER: real number (e.g. the energy) written to the output file
C     SPNAME: name of the space group written to the output file 
C     X(1..): coordinates
C     BOX(1..6): ax, by, cz, bx, cx, cy

C     Input through common block /SPGR/: space group information:
C        NSPGR, MSP, TSP: see subroutine LEESR
C     Input through common block /NONB/: charge information:
C        CG(1..)
C     Input through common block /BOND/: connectivity information: 
C        NBOND, IB(1..), JB(1..)
C     Input through common block /ATTP/:
C        PANM(1..): atom names in topology file

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cbond.i'
      include 'cnonb.i'

      integer i,i12,ii,imod,iout,iout1,ispgr,iwr,j,jj,k,m,n12,nconv,
     +     ngr,nrp,nspuit,nummer,ioutsp
      double precision alpha,bb,beta,box,bx,by,cc,charge,cosa,cosb,cosc,
     +     cx,cy,cz,ener,facq,gamma,pi,rad,t,x
      dimension n12(maxat),i12(4,maxat)
      integer typenr(maxat)

      dimension x(*),box(*)
      character*10 filenm
      character*(*) filtit
      character*8 spname
      character*7 symx,symy,symz,alls1*50,alls2*50
      character*2 atname

      iout=iout1
      iwr=5
      if (iout1.ge.10) then
         iout=iout1-10
         iwr=4
      endif
c      ntit=nword(filtit)
c      print*,'fdat',ntit,'---',filtit
c      pause
      pi=4*atan(1.0d0)
      rad=180/pi
c      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      alpha=rad*acos(cosa)
      beta=rad*acos(cosb)
      gamma=rad*acos(cosc)
C     Transform X to oblique coordinates
      call tri(nrp,0,x,1,cosa,cosb,cosc)

      nconv=10*10000
      nspuit=(icentr*nspgr/2)+1
      if (iout.eq.4) then
C        Write out FDAT file:
         if (iwr.eq.4) write(filenm,24) nummer,'.dat'
         if (iwr.eq.5) write(filenm,25) nummer,'.dat'
         open(17,file=filenm)
         write(17,12) icryst,3,1111,5,0,27,0,0,nspuit,0,nrp,0,0,0,
     +        1,0,2,icentr,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,96
         write(17,13) nint(nconv*box(1)),nint(nconv*bb),nint(nconv*cc),
     +        nint(1000*alpha),nint(1000*beta),nint(1000*gamma),4,4,4,
     +        3,3,3,0,0,0,0,0,0,0,0,mod(ispgr,1000),spname,nspgr+1,0
         write(17,14) nummer,ener
         write(17,10) (mspuit(3*j-2)+1,1,1,nint(tspuit(3*j-2)*12),
     +        1,mspuit(3*j-1)+1,1,nint(tspuit(3*j-1)*12),
     +        1,1,mspuit(3*j)+1,nint(tspuit(3*j)*12),j=1,nspuit)

         imod=nrp/3
         do i=1,3*imod,3
            write(17,11) (panm(i+j),nint(100000*x(3*(i+j)-2)/box(1)),
     +           nint(100000*x(3*(i+j)-1)/bb),
     +           nint(100000*x(3*(i+j))/cc),j=0,2)
c         print*, (panm(i+j),100000*x(3*(i+j)-2)/box(1),
c     +        100000*x(3*(i+j)-1)/bb,100000*x(3*(i+j))/cc,j=0,2)
         enddo
         if ((imod*3-nrp).ne.0) write(17,11)
     +        (panm(i),nint(100000*x(3*i-2)/box(1)),
     +        nint(100000*x(3*i-1)/bb),nint(100000*x(3*i)/cc),
     +        i=3*imod+1,nrp)

      else if (iout.eq.2 .or. iout.eq.3) then
C        Write out CSSR file:
         facq=sqrt(332.0636d0*0.4184d0)
C        Work out connectivity
         do i=1,maxat
            n12(i)=0
            typenr(i)=0
            do j=1,4
               i12(j,i)=0
            enddo
         enddo
         do k=1,ngr
            do i=1,nbond
               ii=ib(i)+(k-1)*nrp/ngr
               jj=jb(i)+(k-1)*nrp/ngr
               n12(ii)=n12(ii)+1
               n12(jj)=n12(jj)+1
               i12(n12(ii),ii)=jj
               i12(n12(jj),jj)=ii
            enddo
         enddo
C        MM3-like atom typing
         do i=1,nrp
C           Extended typing, exclusively based on atom type names:
            if (type(i)(1:2).eq.'HC') typenr(i)=5
            if (type(i)(1:2).eq.'HR') typenr(i)=20
            if (type(i)(1:2).eq.'HO') typenr(i)=21
            if (type(i)(1:2).eq.'HA') typenr(i)=21
            if (type(i)(1:2).eq.'HN') typenr(i)=23
            if (type(i)(1:2).eq.'HZ') typenr(i)=24
            if (type(i)(1:2).eq.'HW') typenr(i)=44
            if (type(i)(1:2).eq.'CS') typenr(i)=1
            if (type(i)(1:2).eq.'CP') typenr(i)=1
            if (type(i)(1:2).eq.'CE') typenr(i)=1
            if (type(i)(1:2).eq.'CA') typenr(i)=1
            if (type(i)(1:2).eq.'C ') typenr(i)=3
            if (type(i)(1:2).eq.'CD') typenr(i)=3
            if (type(i)(1:2).eq.'CR') typenr(i)=50
            if (type(i)(1:2).eq.'N ') typenr(i)=9
            if (type(i)(1:2).eq.'NH') typenr(i)=9
            if (type(i)(1:2).eq.'ND') typenr(i)=9
            if (type(i)(1:2).eq.'NR') typenr(i)=37
            if (type(i)(1:2).eq.'NC') typenr(i)=8
            if (type(i)(1:2).eq.'NP') typenr(i)=8
            if (type(i)(1:2).eq.'OA') typenr(i)=6
            if (type(i)(1:2).eq.'OE') typenr(i)=6
            if (type(i)(1:2).eq.'OS') typenr(i)=6
            if (type(i)(1:2).eq.'O ') typenr(i)=7
            if (type(i)(1:2).eq.'OD') typenr(i)=7
            if (type(i)(1:2).eq.'OZ') typenr(i)=75
            if (type(i)(1:2).eq.'OW') typenr(i)=43
            if (type(i)(1:2).eq.'F ') typenr(i)=11
            if (type(i)(1:2).eq.'S ') typenr(i)=15
            if (type(i)(1:2).eq.'SD') typenr(i)=74
            if (type(i)(1:2).eq.'SR') typenr(i)=42
            if (type(i)(1:2).eq.'Cl') typenr(i)=12
            if (type(i)(1:2).eq.'Br') typenr(i)=13
            if (type(i)(1:2).eq.'I ') typenr(i)=14
         enddo

         if (iwr.eq.4) write(filenm,26) nummer,'.cssr'
         if (iwr.eq.5) write(filenm,27) nummer,'.cssr'
         open(17,file=filenm)
         write (17,59) 10*box(1),10*bb,10*cc
   59    format (38x,3f8.3)
         write (17,511)  alpha,beta,gamma,mod(ispgr,1000),spname
  511    format(21x,3f8.3,'    SPGR=',i3,1x,a11)
c         if (ntit.eq.0) then
         write (17,512) nrp,0,filtit(1:40)
c         else
c            write(515) filtit
c         endif
  512    format (2i4,1x,a60)
c  515    format(a62)

C        Undocumented option: write symmetry elements for unknown space groups.
         if (spname(1:1).lt.'A' .or. spname(1:1).gt.'Z'
     +        .or. spname(1:2).eq.'P4') then
            print*,spname
            write(16,*) spname
            do j=1,nspgr
               i=3*j-2
               symx='  x    '
               if (msp(i).eq.-1) symx(2:2)= '-'
               t=tsp(i)-0.5d0
               t=t-nint(t)
               if (abs(t).lt.0.002) symx(4:7)='+1/2'
               i=3*j-1
               symy='  y    '
               if (msp(i).eq.-1) symy(2:2)= '-'
               t=tsp(i)-0.5d0
               t=t-nint(t)
               if (abs(t).lt.0.002) symy(4:7)='+1/2'
               i=3*j
               symz='  z    '
               if (msp(i).eq.-1) symz(2:2)= '-'
               t=tsp(i)-0.5d0
               t=t-nint(t)
               if (abs(t).lt.0.002) symz(4:7)='+1/2'
               write(17,63) symx,symy,symz
            enddo
         endif
   63    format('SYMM ',a7,' , ',a7,' , ',a7)

         write (17,514) nummer,ener
  514    format(i10.5,f12.4)

         do i=1,nrp
            charge=0
            if (iout.eq.3) charge=cg(i)/facq
            write (17,513) i,panm(i),x(3*i-2)/box(1),
     +           x(3*i-1)/bb,x(3*i)/cc,(i12(j,i),j=1,4),0,0,0,0,
     +           charge,typenr(i)
         enddo
  513    format(i4,1x,a4,2x,f9.5,1x,f9.5,1x,f9.5,1x,8i4,1x,f7.3,1x,i3)
      else
C        Write out CIF file:
         if (iwr.eq.4) write(filenm,24) nummer,'.cif'
         if (iwr.eq.5) write(filenm,25) nummer,'.cif'
         open(17,file=filenm)
         write(17,101) nummer,filtit
         write(17,109) ener
         i=mod(ispgr,1000)
         if (i.lt.3) then
            write(17,116)
         elseif (i.gt.15) then
            write(17,118)
         else
            write(17,117)
         endif
         write(17,110) spname(1:1), spname(2:8)
         write(17,115) i

C        Write space group symmetry operations
         ioutsp=1
         if (ioutsp.eq.1) then
            write(17,119)
            write(17,113)
            write(17,*) '   x,y,z'
            do j=1,nspgr
               i=3*j-2
               symx='  x    '
               if (msp(i).eq.-1) symx(2:2)= '-'
               t=tsp(i)-0.5d0
               t=t-nint(t)
               if (abs(t).lt.0.002) symx(4:7)='+1/2'
               i=3*j-1
               symy='  y    '
               if (msp(i).eq.-1) symy(2:2)= '-'
               t=tsp(i)-0.5d0
               t=t-nint(t)
               if (abs(t).lt.0.002) symy(4:7)='+1/2'
               i=3*j
               symz='  z    '
               if (msp(i).eq.-1) symz(2:2)= '-'
               t=tsp(i)-0.5d0
               t=t-nint(t)
               if (abs(t).lt.0.002) symz(4:7)='+1/2'
               write(alls1,64) symx,symy,symz
               m=0
               alls2=' '
               do k=1,50
                  if (alls1(k:k).ne.' ') then
                     m=m+1
                     alls2(m:m)=alls1(k:k)
                  endif
               enddo
               write(17,65) alls2
            enddo
         endif

         write(17,102) 10*box(1)
         write(17,103) 10*bb
         write(17,104) 10*cc
         write(17,105) alpha
         write(17,106) beta
         write(17,107) gamma
         write(17,114) 1000*box(1)*box(2)*box(3)
         write(17,108) ngr*(nspgr+1)

C        Write atom coordinates
         write(17,119)
         write(17,120)
         write(17,121)
         write(17,111)
         do i=1,nrp
            atname='  '
            atname(1:1)=panm(i)(1:1)
            j=ichar(panm(i)(2:2))
            if (j.lt.48 .or. j.gt.57) atname(2:2)=panm(i)(2:2)
            write (17,112) panm(i),atname,
     1            x(3*i-2)/box(1),x(3*i-1)/bb,x(3*i)/cc
         enddo
      endif

C     Transform X back to rectangular coordinates
C     This introduces minor changes in the coordinates X!
      call tri(nrp,0,x,-1,cosa,cosb,cosc)
      close(17)

   10 format(5(3i1,i2,3i1,i2,3i1,i2),5x)
   11 format(A5,3i7,1x,a5,3i7,1x,a5,3i7)
   12 format(1H#,'PACKNB96',2i1,i6,6x,11i3,22i1,i2)
   13 format(6i6,6i1,6i2,2i3,i3,a8,i3,i2,4x)
   14 format('HITNR.',2x,i5,2x,'EN.',f9.2)
   24 format(i4.4,a4)
   25 format(i5.5,a4)
   26 format(i4.4,a5)
   27 format(i5.5,a5)
   64 format(4x,a7,' , ',a7,' , ',a7)
   65 format(4x,a)
  101 format('data_',i4.4,3x,a)
  102 format('_cell_length_a',f10.5)
  103 format('_cell_length_b',f10.5)
  104 format('_cell_length_c',f10.5)
  105 format('_cell_angle_alpha',f10.3)
  106 format('_cell_angle_beta',f10.3)
  107 format('_cell_angle_gamma',f10.3)
  108 format('_cell_formula_units_Z',i4)
  109 format('# UPACK energy: ',f10.3)
  110 format('_symmetry_space_group_name_H-M ''',a1,1x,a7,'''')
  111 format('_atom_site_fract_x',
     +     /,'_atom_site_fract_y',/,'_atom_site_fract_z')
  112 format(a4,3x,a2,2x,3f10.6)
 113  format('_symmetry_equiv_pos_as_xyz')
 114  format('_cell_volume',f11.3)
 115  format('_space_group_IT_number',i5)
 116  format('_space_group_crystal_system       triclinic')
 117  format('_space_group_crystal_system       monoclinic')
 118  format('_space_group_crystal_system       orthorhombic')
 119  format('loop_')
 120  format('_atom_site_label')
 121  format('_atom_site_type_symbol')
      end
