      subroutine func0(npar,par,der,epot,isubr)

C     Energy calculation for a free molecule.
C     Fills array X with atom coordinates, calls energy routine.
C     Calculates energy EPOT and forces DER.

C     Input through parameters:
C        NPAR: number of parameters
C        PAR(1..NPAR): the atomic coordinates
C        ISUBR=0: calculation through NONBE (no pair list, co derivatives)
C             =1: calculation through NONBE (make pair list)
C             =2: calculation through LIST (use pair list)
C     Input through common block /PARS/:
C        NRP: number of atoms

C     Output through parameters:
C        EPOT: nonbonded energy
C        DER(1..NPAR): negative derivatives of EPOT to PAR
C     Output through common block:
C        X(1..3*NRP): optimized coordinates

C     Calls subroutines CBOND, CHOEK, CDIHA, NONBE and LIST.

      implicit none
      include 'params.i'
      include 'cpars.i'
      include 'caidt.i'
      dimension der(*),par(*),box(12),uit(12),f(maxxco)
      double precision box,der,eb,eh,ei,enb,ep,epot,f,par,uit
      integer i,isubr,nf,npar,nr3

      nr3=3*nrp*ngr
      do i=1,3
         box(i)=1000
         box(i+3)=0
      enddo
      do i=1,nr3
         x(i)=par(i)
      enddo
      if (npar.ne.nr3) then
         print*,' ***** error in FUNC0:',nrp,npar,nr3
         stop
      endif

      nf=0
      if (isubr.gt.0) nf=1
      if (isubr.eq.0) call nonbe(ngr,nrp,enb,0,uit,0,box,x,f,0)
      if (isubr.eq.1) call nonbe(ngr,nrp,enb,1,uit,1,box,x,f,0)
      if (isubr.eq.2) call list(ngr,nrp,enb,1,uit,box,x,f)
      call cbond(ngr,nrp,x,f,nf,eb,0)
      call choek(ngr,nrp,x,f,nf,eh,0)
      call cdiha(ngr,nrp,x,f,nf,1,ei,0)
      call cdiha(ngr,nrp,x,f,nf,2,ep,0)
      epot=enb+eb+eh+ei+ep

      do i=1,nr3
         der(i)=f(i)
      enddo
      end
