      subroutine func1(npar,par,der,enb,isubr)

C     Energy calculation for a crystal of rigid molecules.
C     Put a reference molecule (coordinates XR) into a crystal, applying
C     rotations and translations. The crystallographic parameters
C     are put into BOX, the atomic coordinates into X.
C     Calls energy routine, calculates energy ENB and forces DER.

C     Input through parameters:
C        NPAR: number of parameters
C        PAR(1..NPAR): cell parameters, centers of gravity, Euler angles.
C        ISUBR=1: calculation through NONBE (make pair list)
C             =2: calculation through LIST (use pair list)
C     Input through common block /PARS/:
C        NRP: number of atoms
C        NGR: number of groups of molecules
C        XR(1..3*NRP): reference coordinates for the free molecule
C        CEL(1..12): ax, by, cz, bx, cx, cy, X, Y, Z, phi, theta, psi
C        IREF(1..12)=0: take BOX from CEL and do not refine
C                   =1: take BOX from PAR and do refine
C     Input through common block /AIDT/:
C        NMOLS: the number of discrete molecules
C        IMOLS(1..NMOLS): the first atom of molecule I is IMOLS(I)

C     Output through parameters:
C        ENB: nonbonded energy
C        DER(1..NPAR): negative derivatives of ENB to PAR
C     Output through common block:
C        X(1..3*NRP): optimized coordinates

C     Calls subroutines NONBE and LIST.

      implicit none
      include 'params.i'
      include 'cpars.i'
      include 'caidt.i'

      dimension der(*),par(*),euler(3,3),deuler(3,3,3),box(6),uit(12),
     +     pos(3),f(maxxco)
      double precision box,ca,cb,cg,der,deuler,enb,euler,f,
     +     par,phi,pos,psi,sa,sb,sg,theta,uit
      integer i,i1,i2,i3,imol,isubr,j,j3,k,kpar,m,npar

      kpar=0
      do i=1,6
         if (iref(i).gt.0) then
            kpar=kpar+1
            box(i)=par(kpar)
         else
            box(i)=cel(i)
         endif
      enddo
c      print*,'FUNC1',nmols,nrp,npar,kpar
      do imol=1,nmols
         i1=imols(imol)
         i2=imols(imol+1)-1
c         print*,' Molecule',imol,' from', i1,' to',i2
         if (imol.eq.1) then
            do m=1,3
               if (iref(m+6).gt.0) then
                  kpar=kpar+1
                  pos(m)=par(kpar)
               else
                  pos(m)=cel(m+6)
               endif
            enddo
         else
            pos(1)=par(kpar+1)
            pos(2)=par(kpar+2)
            pos(3)=par(kpar+3)
            kpar=kpar+3
         endif
         phi=par(kpar+1)
         theta=par(kpar+2)
         psi=par(kpar+3)
         kpar=kpar+3
         ca=cos(phi)
         sa=sin(phi)
         cb=cos(theta)
         sb=sin(theta)
         cg=cos(psi)
         sg=sin(psi)
         euler(1,1)=    cg*ca-sg*sa*cb
         euler(1,2)=    cg*sa+sg*ca*cb
         euler(1,3)=    sg*sb
         euler(2,1)=   -sg*ca-cg*sa*cb
         euler(2,2)=   -sg*sa+cg*ca*cb
         euler(2,3)=    cg*sb
         euler(3,1)=    sa*sb
         euler(3,2)=   -ca*sb
         euler(3,3)=    cb
         do i=i1,i2
            j=3*(i-1)
            do m=1,3
               x(j+m)=pos(m)
               do k=1,3
                  x(j+m)=x(j+m)+euler(m,k)*xr(j+k)
               enddo
            enddo
         enddo
      enddo
      if (npar.ne.kpar) then
         print*,' ***** error in FUNC1',npar,kpar
         stop
      endif

      if (isubr.eq.1) call nonbe(ngr,nrp,enb,1,uit,1,box,x,f,0)
      if (isubr.eq.2) call list(ngr,nrp,enb,1,uit,box,x,f)

      do i=1,npar
         der(i)=0
      enddo
      kpar=0
      do i=1,6
         if (iref(i).gt.0) then
            kpar=kpar+1
            der(kpar)=uit(i)
         endif
      enddo
      do imol=1,nmols
         i1=imols(imol)
         i2=imols(imol+1)-1
         if (imol.eq.1) then
            do m=1,3
               if (iref(m+6).gt.0) then
                  kpar=kpar+1
                  do i=i1,i2
                     j=3*(i-1)
                     der(kpar)=der(kpar)+f(j+m)
                  enddo
               endif
            enddo
         else
            do m=1,3
               kpar=kpar+1
               do i=i1,i2
                  j=3*(i-1)
                  der(kpar)=der(kpar)+f(j+m)
               enddo
            enddo
         endif

         phi=par(kpar+1)
         theta=par(kpar+2)
         psi=par(kpar+3)
         ca=cos(phi)
         sa=sin(phi)
         cb=cos(theta)
         sb=sin(theta)
         cg=cos(psi)
         sg=sin(psi)
         deuler(1,1,1)=-cg*sa-sg*ca*cb
         deuler(1,1,2)=       sg*sa*sb
         deuler(1,1,3)=-sg*ca-cg*sa*cb
         deuler(1,2,1)= cg*ca-sg*sa*cb
         deuler(1,2,2)=      -sg*ca*sb
         deuler(1,2,3)=-sg*sa+cg*ca*cb
         deuler(1,3,1)= 0
         deuler(1,3,2)= sg*cb
         deuler(1,3,3)= cg*sb
         deuler(2,1,1)= sg*sa-cg*ca*cb
         deuler(2,1,2)=       cg*sa*sb
         deuler(2,1,3)=-cg*ca+sg*sa*cb
         deuler(2,2,1)=-sg*ca-cg*sa*cb
         deuler(2,2,2)=      -cg*ca*sb
         deuler(2,2,3)=-cg*sa-sg*ca*cb
         deuler(2,3,1)= 0
         deuler(2,3,2)= cg*cb
         deuler(2,3,3)=-sg*sb
         deuler(3,1,1)= ca*sb
         deuler(3,1,2)= sa*cb
         deuler(3,1,3)= 0
         deuler(3,2,1)= sa*sb
         deuler(3,2,2)=-ca*cb
         deuler(3,2,3)= 0
         deuler(3,3,1)= 0
         deuler(3,3,2)=-sb
         deuler(3,3,3)= 0
         do m=1,3
            kpar=kpar+1
            do i=i1,i2
               j=3*(i-1)
               do i3=1,3
                  do j3=1,3
                     der(kpar)=der(kpar)+
     +                    f(j+i3)*deuler(i3,j3,m)*xr(j+j3)
                  enddo
               enddo
            enddo
         enddo
      enddo

      if (npar.ne.kpar) then
         print*,' ***** error in FUNC1',npar,kpar
         stop
      endif
      end
