      subroutine func3(npar,par,der,epot,isubr)

C     Energy calculation for a crystal of flexible molecules.
C     The crystallographic parameters are put into BOX,
C     the atomic coordinates into X.
C     Calls energy routine, calculates energy EPOT and forces DER.

C     Input through parameters:
C        NPAR: number of parameters
C        PAR(1..NPAR): the cell parameters and atomic coordinates.
C        ISUBR=0: calculation through NONBE (no pair list, no derivatives)
C             =1: calculation through NONBE (make pair list)
C             =2: calculation through LIST (use pair list)
C     Input through common block /PARS/:
C        NGR: number of molgroups
C        NRP: number of atoms in one molecule
C        CEL(1..6): ax, by, cz, bx, cx, cy
C        IREF(1..6)=0: take BOX from CEL and do not refine
C                  =1: take BOX from PAR and do refine
C        IREF(7..9)=0: do not refine this coordinate for the first atom
C                      (symmetry allows one coordinate to be freely chosen)
C                  =1: take this coordinate from PAR and do refine

C     Output through parameters:
C        EPOT: nonbonded energy
C        DER(1..NPAR): negative derivatives of EPOT to PAR
C     Output through common block:
C        X(1..3*NRP): optimized coordinates

C     Calls subroutines NONBE, LIST, CBOND, CHOEK and CDIHA.

      implicit none
      include 'params.i'
      include 'cpars.i'
      include 'cinds.i'
      dimension der(*),par(*),box(12),uit(12),f(maxxco)
      double precision box,der,eb,eh,ei,enb,ep,epot,f,par,uit
      integer i,isubr,kpar,nf,npar,nr3

      kpar=0
      nr3=3*nrp*ngr
      do i=1,6
         box(i)=cel(i)
         if (iref(i).gt.0) then
            kpar=kpar+1
            box(i)=par(kpar)
         endif
      enddo
      do i=1,3
         if (iref(i+6).gt.0) then
            kpar=kpar+1
            x(i)=par(kpar)
         endif
      enddo
      do i=4,nr3
         kpar=kpar+1
         x(i)=par(kpar)
      enddo
      if (npar.ne.kpar) then
         print*,' ***** error in FUNC3:',nrp,npar,kpar
         stop
      endif

      if (isubr.eq.0) call nonbe(ngr,nrp,enb,0,uit,0,box,x,f,0)
      if (isubr.eq.1) call nonbe(ngr,nrp,enb,1,uit,1,box,x,f,0)
      if (isubr.eq.2) call list(ngr,nrp,enb,1,uit,box,x,f)
      nf=0
      if (isubr.gt.0) nf=1
      call cbond(ngr,nrp,x,f,nf,eb,0)
      call choek(ngr,nrp,x,f,nf,eh,0)
      call cdiha(ngr,nrp,x,f,nf,1,ei,0)
      call cdiha(ngr,nrp,x,f,nf,2,ep,0)
      epot=enb+eb+eh+ei+ep

C     End of calculation
      kpar=0
      do i=1,6
         if (iref(i).gt.0) then
            kpar=kpar+1
            der(kpar)=uit(i)
         endif
      enddo
      do i=1,3
         if (iref(i+6).gt.0) then
            kpar=kpar+1
            der(kpar)=f(i)
         endif
      enddo
      do i=4,nr3
         kpar=kpar+1
         der(kpar)=f(i)
      enddo
      end
