      subroutine func3g(npar,par,der,epot,isubr)

C     Energy calculation for a crystal of flexible molecules.
C     The crystallographic parameters are put into BOX,
C     the atomic coordinates into X.
C     Calls energy routine, calculates energy EPOT and forces DER.

C     Input through parameters:
C        NPAR: number of parameters
C        PAR(1..NPAR): the cell parameters and atomic coordinates.
C        ISUBR=0: calculation through NONBE (no pair list, no derivatives)
C             =1: calculation through NONBE (make pair list)
C             =2: calculation through LIST (use pair list)
C     Input through common block /PARS/:
C        NGR: number of molgroups
C        NRP: number of atoms in one molecule
C        CEL(1..6): ax, by, cz, bx, cx, cy
C        IREF(1..6)=0: take BOX from CEL and do not refine
C                  =1: take BOX from PAR and do refine
C        IREF(7..9)=0: do not refine this coordinate for the first atom
C                      (symmetry allows one coordinate to be freely chosen)
C                  =1: take this coordinate from PAR and do refine
C     Input through common block /AIDT/:
C        IAI=0: calculate intramolecular energy from default force field
C           =1: calculate intramolecular energy and derivatives from GAMESS.
C           =2: calculation through previous call with IAI=1
C        IDFTB=1: calculate intramolecular energy, derivatives 
C                and charges from DFTB.
C        NCYCGA: the number of the ab-initio cycle
C        MAXCYC: maximum number of ab-initio cycles
C        NMOLS: the number of discrete molecules
C        BTER2, BTRA2: basis sets for inter- and intramolecular calculations
C        NAIB, NAIH, NAID: numbers of bond lengths, bond angles, 
C              and dihedrals, respectively.
C        SAIB, SAIH, SAID: maximum allowed shifts in these parameters
C        IAIB ... LAID: identifies corresponding atoms (for output only)
C        IPREM: regulates output for energy minimization

C     Output through parameters:
C        EPOT: nonbonded energy
C        DER(1..NPAR): negative derivatives of EPOT to PAR
C     Output through common block /PARS/:
C        X(1..3*NRP): starting coordinates from parameters

C     Output through common block /AIDT/:
C        IREDO>0: shift too large for parameter IREDO
C             <0: MAXCYC exceeded

      implicit none
      include 'params.i'
      include 'cpars.i'
      include 'caidt.i'
      include 'cinds.i'
      include 'chadd.i'
      include 'cbond.i'
      double precision a,akar1,box,damp,der,diff,dihan2,eb,ebond,egam,
     +     eh,ei,ei1,enb,ep,epot,f,f1,fgam,grad1,par,pi,rad,refpar,
     +     scr1,scr2,shfac,shfacm,snew,sref,uit
      integer i,inew,iredop,iredos,isubr,j,k,kpar,mldcyc,mold,nf,nneg,
     +     npar,nr3,ns,ipr
      dimension der(*),par(*),box(12),uit(12),f(maxxco),fgam(maxxco),
     +     f1(maxxco)
      dimension sref(maxpar),snew(maxpar),diff(maxpar),refpar(maxpar),
     +     scr1(1),scr2(1,1)
      character*10 regel

C     NCYCGA is the number of the GAMESS-iteration
      data inew /0/
      save sref,refpar,iredop

c      ipr=iprem
      ipr=0
      if (ipr.gt.0) then
         print*,' Starting FUNC3G; isubr,ncycga,iredo,iai:',
     +        isubr,ncycga,iredo,iai
         print*,naib,naih,naid,saib,saih,said
         print*,' Starting FUNC3G with parameters:'
         print*,(par(i),i=1,npar)
      endif

      iredo=0
      pi=4*atan(1.0d0)
      rad=180/pi
      nf=0
      if (isubr.gt.0) nf=1
      if (ncycga.gt.maxcyc) then
C        Return to calling program and stop minimization
         print*,' Cycle',ncycga,' already exceeds MAXCYC =',maxcyc
         iredo=-1000
         return
      endif
      do i=1,maxxco
         fgam(i)=0
      enddo
C???  Explain MLDCYC
      mldcyc=3

C     Determine the optimization parameters PAR(1..NPAR)
      kpar=0
      nr3=3*nrp*ngr
      ei1=0
      do i=1,nr3
         f(i)=0
         f1(i)=0
      enddo
      do i=1,6
         box(i)=cel(i)
         if (iref(i).gt.0) then
            kpar=kpar+1
            box(i)=par(kpar)
         endif
      enddo
      do i=1,3
         if (iref(i+6).gt.0) then
            kpar=kpar+1
            x(i)=par(kpar)
         endif
      enddo
      do i=4,nr3
         kpar=kpar+1
         x(i)=par(kpar)
      enddo
      if (npar.ne.kpar) then
         print*,' ***** Error in set-up FUNC3G:',nrp,ngr,npar,kpar
         stop
      endif
      if (bter2.eq.'NONE' .and. btra2.eq.'NONE') goto 600

C     Determine internal coordinates (SNEW) for ab initio calculations.
      ns=nr3-6*nmols
      call intcar(scr1,scr2,scr1,0,nneg,snew,akar1,grad1)
      if (inew.eq.0) then
C        This occurs only in the first call
C???        Derivatives were already found in PREP
         if (naimpr.gt.0) then
            print*,' Possible artificial improper dihedrals'
            write(16,*) ' Possible artificial improper dihedrals'
            do i=nimpr+1,nimpr+naimpr
               a=dihan2(iq(i),jq(i),kq(i),lq(i),x)
               print*,i,iq(i),jq(i),kq(i),lq(i),cq(i),a*rad
               write(16,*) i,iq(i),jq(i),kq(i),lq(i),cq(i),a*rad
               q0(i)=a
            enddo
         endif
      endif
      if (iai.eq.1 .or. inew.eq.0) then
         inew=1
C        Set reference internal coordinates
         do i=1,ns
            sref(i)=snew(i)
         enddo
         do i=1,npar
            refpar(i)=par(i)
         enddo
         if (sref(1).gt.0.001d0) then
            print 74,(sref(i),i=1,naib)
            print 75,(sref(i),i=naib+1,naib+naih)
            print 76,(sref(i),i=naib+naih+1,naib+naih+naid)
            write(16,74) (snew(i),i=1,naib)
            write(16,75) (snew(i),i=naib+1,naib+naih)
            write(16,76) (snew(i),i=naib+naih+1,naib+naih+naid)
         endif
      else
C        Determine deviations from the reference
         do i=1,ns
            diff(i)=snew(i)-sref(i)
            if (ipr.gt.0) print*,'DIFF',i,diff(i),snew(i),sref(i)
            if (diff(i).gt. 180) diff(i)=diff(i)-360
            if (diff(i).lt.-180) diff(i)=diff(i)+360
         enddo
C        Check that shifts do not become too large
         k=0
         shfacm=1
         do i=1,naib
            k=k+1
ccc            diff(k)=10*diff(k)
C           Conversion to Angstroms
            shfac=abs(diff(k))/saib
            if (shfac.gt.shfacm) then
               iredo=k
               shfacm=shfac
            endif
         enddo
         do i=1,naih
            k=k+1
cc            diff(k)=rad*diff(k)
            shfac=abs(diff(k))/saih
            if (shfac.gt.shfacm) then
               iredo=k
               shfacm=shfac
            endif
         enddo
         do i=1,naid
            k=k+1
cc            diff(k)=rad*diff(k)
            shfac=abs(diff(k))/said
            if (shfac.gt.shfacm) then
               iredo=k
               shfacm=shfac
            endif
         enddo
ccc         if (k.ne.ns .and. iai.gt.0) then
ccc            print*,' ***** Error in number of ',
ccc     +           'internal coordinates',k,ns,iai
ccc            write(16,*) ' ***** Error in number of ',
ccc     +           'internal coordinates',k,ns
ccc            stop
ccc         endif
         if (iredo.gt.0) then
C           Shift of coordinate too large, redo GAMESS'
            print 73,iredo,diff(iredo)
            write(16,73) iredo,diff(iredo)
            if (iredo.le.naib) then
               k=iredo
               print 44,k,iaib(k),jaib(k),sref(iredo),snew(iredo)
               write(16,44) k,iaib(k),jaib(k),sref(iredo),snew(iredo)
            elseif (iredo.le.naib+naih) then
               k=iredo-naib
               print 45,k,iaih(k),jaih(k),kaih(k),
     +              sref(iredo),snew(iredo)
               write(16,45) k,iaih(k),jaih(k),kaih(k),
     +              sref(iredo),snew(iredo)
            else
               k=iredo-naib-naih
               print 46,k,iaid(k),jaid(k),kaid(k),laid(k),
     +              sref(iredo),snew(iredo)
               write(16,46) k,iaid(k),jaid(k),kaid(k),laid(k),
     +              sref(iredo),snew(iredo)
            endif
            if (ipr.gt.1) then
               print*,' Coordinates after undamped shifts would be:'
               write(16,*)' Coordinates after undamped shifts would be:'
               print 74,(snew(i),i=1,naib)
               print 75,(snew(i),i=naib+1,naib+naih)
               print 76,(snew(i),i=naib+naih+1,naib+naih+naid)
               write(16,74) (snew(i),i=1,naib)
               write(16,75) (snew(i),i=naib+1,naib+naih)
               write(16,76) (snew(i),i=naib+naih+1,naib+naih+naid)
            endif
         endif
C        IREDO is the number of the offending coordinate. Its deviation,
C              DIFF(IREDO), is larger than allowed by a factor SHFACM.

         if (iredo.gt.0) then
C           A new GAMESS calculation may be necessary.
C           Determine damping factor for parameter shifts
ccccc            iredos=nint(sign(float(iredo),diff(iredo)))
            iredos=iredo
            if (diff(iredo).lt.0) iredos=-iredo
            if (ncycga.eq.1) then 
               damp=shfacm
               print 71,damp
               write(16,71) damp
            else
               if (iredo.ne.abs(iredop)) then 
C                   Another parameter has exceeded the limit
                  damp=shfacm
                  print 71,damp
                  write(16,71) damp
               else
C                 The same parameter has exceeded the limit
                  if (iredos.eq.-iredop) then
C                    Try to prevent endless oscillation:
                     damp=2
                     print 72,damp,iredo
                     write(16,72) damp,iredo
                     if (iredo.gt.naib+naih) then
C    Nog uitzoeken!!!
C                          Add artificial improper term to stabilize torsion
c                           i=niprop+iredo-naib-naih
c                           kprop(i)=100
c                           vprop(i)=snew(iredo)*rad-diff(iredo)/damp
c                           print 71,i,iredo,kprop(i),vprop(i)
                     endif
                  endif
               endif
            endif
            iredop=iredos
ccc            ncycga=ncycga+1
ccc            if (ncycga.gt.maxcyc) then
C              Tell the calling program to stop minimalization
ccc               print*,' ***** Exceeding MAXCYC =',maxcyc
ccc               iredo=-iredo
ccc            endif
C           Calculate new parameters
            do i=1,npar
c                  print*,i,refpar(i),par(i),
c     +                 refpar(i)+(par(i)-refpar(i))/damp
ccccc  !!!!!             par(i)=refpar(i)+(par(i)-refpar(i))/damp
            enddo
         endif
      endif

CCCCC     Probeer voorlopig maar eens:
      mold=0
      if (bter2.eq.btra2 .and. mod(ncycga-1,mldcyc).eq.0) then
         mold=1
         if (ipr.gt.0) print*,
     +        'A combined GAMESS / MOLDEN calculation will be done'
      endif
      mold=1

C     Bonded terms from ab-initio calculation
      mold=0
      if (bter2.eq.btra2) mold=1
      if (btra2.ne.'NONE') then
C        Ab-initio intramolecular potential
         if (iai.eq.1) then
C           Start new GAMESS calculation, creating a hessian matrix
            if (iprem.gt.0) print*,'   Starting GAMESS-intra'
            call games(fgam,ebond,ichad,mold,btra2,iprem)
            if (iprem.gt.0) print*,'   GAMESS-intra done: ',ebond
         else
C           Find energies and derivatives from previous GAMESS calculation
            if (iprem.gt.5 ) print*,'   Recalculating E-intra'
            call games(fgam,ebond,ichad,0,btra2,iprem)
            if (iprem.gt.5) print*,'   Recalculating done: ',ebond
         endif
      endif

C     Nonbonded terms from ab-initio calculation
      if (bter2.ne.'NONE' .and. iai.eq.1) then
c     +     .and. mod(ncycga-1,mldcyc).eq.0) then
C        Ab-initio intermolecular potential
         if (bter2.ne.btra2) then
C           Start new GAMESS calculation, for MOLDEN only
            iai=0
            if (iprem.gt.0) print*,
     +           'Starting a special GAMESS / MOLDEN calculation'
            call games(scr1,egam,ichad,1,bter2,iprem)
            if (iprem.gt.0) print*,'   GAMESS-inter done: ',egam
         endif

C        Remove artificial improper dihedrals
ccc         do i=niprop+1,niprop+naid
ccc            kprop(i)=0
ccc         enddo
      endif
      if (iai.ne.2) then
         ichad=-abs(ichad)
         iai=2
      endif

C     Normal continuation of energy minimization

      if (naimpr.gt.0) then
C        Calculate genuine improper energy
         call cdiha(ngr,nrp,x,f,nf,1,ei,0)
C        Calculate improper energy, genuine + artificial
         nimpr=nimpr+naimpr
         call cdiha(ngr,nrp,x,f1,nf,1,ei1,0)
         nimpr=nimpr-naimpr
C        Find artificial contribution
         ei1=ei1-ei
         do i=1,nr3
            f1(i)=f1(i)-f(i)
         enddo
      endif

ccc      do i=1,n
ccc         do j=1,3
ccc            deid(j,i)=0
ccc            derivs2(j,i)=0
ccc         enddo
ccc      enddo
ccc      eid=0
ccc      if (btra2.eq.'NONE') then
ccc         call eimprop1
ccc         eid0=eid
ccc         do i=1,n
ccc            do j=1,3
ccc               derivs2(j,i)=deid(j,i)
ccc            enddo
ccc         enddo
ccc      endif
C     Compare with genuine + artificial improper energy 
ccc      eid=0
ccc      niprop=niprop+naid
ccc      call eimprop1
ccc      niprop=niprop-naid
ccc      eid=eid-eid0
C     Retain the artificial improper dihedral energy
ccc      ebond=ebond+eid
ccc      do i=1,n
ccc         do j=1,3
ccc            deid(j,i)=deid(j,i)-derivs2(j,i)
ccc         enddo
ccc      enddo
ccc      if (iprem.gt.-1 .and. abs(eid).gt.1.0e-3) print 70,eid
ccc      if (iprem.gt.5) then
ccc         print*,' Derivatives for artificial improper dihedrals'
ccc         do i=1,n
ccc            print 43,i,(deid(j,i),j=1,3)
ccc         enddo
ccc      endif
      if (iprem.gt.5) then
         print*,' Derivatives for other bonded interactions'
         do i=1,nrp*ngr
            print 43,i,(fgam(3*i-3+j),j=1,3)
         enddo
      endif
C     End of GAMESS calculations

  600 continue
      if (idftb.gt.0) then
ccc         noint=.true.
cc         ichad=-1
ccc         iai=10
         print*,'Calling DFTB'
         call dftb(fgam,ebond,isubr,0)
c         print*,ebond
c         pause
      endif
C     Calculate nonbonded energy
      if (isubr.eq.0) then
         print*,' Calling nonbe with ISUBR=0, how come? Please check.'
         print*,' Type ENTER if you wish to continue anyhow'
         read 42, regel
         call nonbe(ngr,nrp,enb,0,uit,0,box,x,f,0)
      endif
      if (isubr.eq.1) then
C        Create a (new) atom pair list
         if (iprem.gt.5) print*,' Calling NONBE'
         call nonbe(ngr,nrp,enb,1,uit,1,box,x,f,0)
      endif
      if (isubr.eq.2) then
C        Use a previously created pair list
         if (iprem.gt.5) print*,' Calling LIST'
         call list(ngr,nrp,enb,1,uit,box,x,f)
      endif
C     Calculate bonded energy
      if (btra2.eq.'NONE' .and. idftb.eq.0) then
         call cbond(ngr,nrp,x,f,nf,eb,0)
         call choek(ngr,nrp,x,f,nf,eh,0)
         call cdiha(ngr,nrp,x,f,nf,1,ei,0)
         call cdiha(ngr,nrp,x,f,nf,2,ep,0)
         ebond=eb+eh+ei+ep
      endif
      ebond=ebond+ei1
      epot=ebond+enb

      if (iprem.gt.3) print*,' FUNC done, E-bond, E-nonbe, E-tot: ',
     +     ebond,enb,epot
      if (iprem.gt.5) then
         print*,' Derivatives for nonbonded interactions'
         do i=1,nrp*ngr
            print 43,i,(f(3*i-3+j),j=1,3)
         enddo
         print*,' Sum of these derivatives'
         do i=1,nrp*ngr
            print 43,i,(f(3*i-3+j)+ fgam(3*i-3+j),j=1,3)
         enddo
      endif

C     Combine derivatives
      do i=1,nr3
c         print*,fgam(i),f(i),fgam(i)+f(i)
         f(i)=f(i)+fgam(i)+f1(i)
c         f(i)=f(i)+fgam(i)
      enddo

C     End of calculation
      kpar=0
      do i=1,6
         if (iref(i).gt.0) then
            kpar=kpar+1
            der(kpar)=uit(i)
         endif
      enddo
      do i=1,3
         if (iref(i+6).gt.0) then
            kpar=kpar+1
            der(kpar)=f(i)
         endif
      enddo
      do i=4,nr3
         kpar=kpar+1
         der(kpar)=f(i)
      enddo
      if (npar.ne.kpar) then
         print*,' ***** Error in derivatives FUNC3G:',nrp,ngr,npar,kpar
         stop
      endif

      if (iprem.gt.3) print*,'End of func3g, energy:',epot
c      if (iprem.gt.2) print*,'FUNC3G forces:',(der(i),i=1,npar)
c      pause

   42 format(a)
   43 format(i5,4x,3f11.5,4x,3f11.2)
   44 format('This is bond',i4,' between atoms',2i4,2f8.3)
   45 format('This is angle',i4,' between atoms',3i4,2f8.1)
   46 format('This is dihedral',i4,' between atoms',4i4,2f8.3)
   71 format(' Shift damping set to shift / maximum:',f8.3)
   72 format(' Shifts damped with factor ',f8.2,
     +     ' to suppress oscillations in parameter ',2i4)
   73 format(' Shift of internal coordinate',i5,' too large:',f10.4)
c     +     '     Redo GAMESS after cycle',i5)
   74 format(' Bonds (A): ',50f7.3)
   75 format(' Angles   : ',50f7.1)
   76 format(' Dihedrals: ',50f7.1)
      end



      function dihan2(n1,n2,n3,n4,x)

C     Returns the value (in radians) of dihedral angle
C     Is a slight modification of function DIHAN.

C     Input through parameters:
C        n1,n2,n3,n4: numbers of the 4 constituting atoms
C        X: array of atom coordinates

C     Calls subroutine PROD.

      implicit none
      include 'params.i'
      double precision cosa,dihan2,rij,rkj,rkj1,rkj2,rkl,sina,
     +     sj,sj1,sj2,sjsk,sk,sk1,sk2,x
      integer i3,j3,k3,l3,m,n1,n2,n3,n4
      dimension x(*),rij(3),rkj(3),rkl(3),sj(3),sk(3),sjsk(3)

      i3=3*n1-3
      j3=3*n2-3
      k3=3*n3-3
      l3=3*n4-3

      do m=1,3
         rij(m)=x(i3+m)-x(j3+m)
         rkj(m)=x(k3+m)-x(j3+m)
         rkl(m)=x(k3+m)-x(l3+m)
      enddo

      call prod(rij,rkj,sj)
      call prod(rkj,rkl,sk)
      call prod(sj,sk,sjsk)
      sj2=0
      sk2=0
      rkj2=0
      cosa=0
      sina=0
      do m=1,3
         sj2=sj2+sj(m)**2
         sk2=sk2+sk(m)**2
         rkj2=rkj2+rkj(m)**2
         cosa=cosa+sj(m)*sk(m)
         sina=sina+rkj(m)*sjsk(m)
      enddo
      if (sj2.lt.1.0d-12) then
         print*,' Division by',sj2,' in DIHAN'
         goto 100
      endif
      if (sk2.lt.1.0d-12) then
         print*,' Division by',sk2,' in DIHAN'
         goto 100
      endif
      sj1=sqrt(sj2)
      sk1=sqrt(sk2)
      rkj1=sqrt(rkj2)
      cosa=cosa/(sj1*sk1)
      sina=sina/(sj1*sk1*rkj1)
      dihan2=atan2(sina,cosa)
      return

 100  continue
C     Error status
      print 60,n2,n3,n3,n4
      print 70,n1,(x(i3+m),m=1,3)
      print 70,n2,(x(j3+m),m=1,3)
      print 70,n3,(x(k3+m),m=1,3)
      print 70,n3,(x(l3+m),m=1,3)
c     stop
      dihan2=0
 60   format(' ***** Parallel bonds ',i4,' --',i4,' and',i4,' --',i4)
 70   format(i5,3f10.5)
      end




      subroutine dftb(f,egames,isubr,iprint)
C     Delivers ab-initio energy, forces and charges for a free molecule.
C     Program DFTB is called through a non-Fortran system call.
C     This program can be obtained from the SCM company, Amsterdam.
C     It was tested in 2016 and was then found not very satisfactory.

C     Input through parameters:
C        ISUBR=0: calculation with charges through NONBE
C             =1: calculation with charges through NONBE
C             =2: calculation without charges through LIST (use pair list)
C        IPRINT regulates output to screen.
C     Input through common block /PARS/:
C        NGR: number of molgroups
C        NRP: number of atoms in one molecule
C        X(1..): coordinates
C     Input through common block /AIDT/:
C        AIOFST: offset (atomic units) for reasonable energies in kJ/mol
C        NMOLS: the number of discrete molecules
C        IMOLS(1..NMOLS): the first atom of molecule I is IMOLS(I)
C     Input through common block /ATTP/:
C        TYPE(1..): atom types
C        PANM(1..): atom names

C     Output through parameters:
C        EGAMES: energy (kJ/mol)
C        F(1..): the negative derivatives of EGAMES to X(1..).

C     Output through common block /CNONB/:
C        CG(1..): the DFTB charges

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'caidt.i'
      include 'cpars.i'
      include 'cnonb.i'

      integer i,i1,i2,iprint,ipr,istat,m,isubr,mold,nco,nm,nrpt,ndftb
      double precision au2kj,bohr,chg,egames,cgt,
     +     egtot,eg1,f,grad,fact,scr,facq,som
      dimension f(*),cgt(maxxco),grad(maxpar),scr(3)
      character regel*200,atname*2,job*100
      integer system
      data ndftb /0/
      save

      ipr=iprint
      mold=0
      if (isubr.lt.2) mold=1
      ndftb=ndftb+1
      au2kJ=2625.5d0
      bohr=0.0529177d0
      fact=au2kj/bohr
      facq=sqrt(332.0636d0*0.4184d0/epsil)
      nrpt=nrp*ngr
      nco=3*nrpt
      som=0

c      ipr=10

      do i=1,maxpar
         grad(i)=0
         cgt(i)=0
      enddo
      egtot=0

c     Call DFTB and collect data from the output
ccc      open(26,file='molden.xyz')
      do nm=1,nmols
         i1=imols(nm)
         i2=imols(nm+1)-1
C           Specifically for ionic dimer in CPR19!!!!!
c            if (nmols.eq.2 .and. nrp.eq.28) then
c               if (nm.eq.1) write(23,10) 'CHARGE 1'
c               if (nm.eq.2) write(23,10) 'CHARGE -1'
c            endif
C           Specifically for ionic dimer in CPR24!!!!!
c            if (nmols.eq.3 .and. nrp.eq.20) then
c               if (nm.eq.1) write(23,10) 'CHARGE 1'
c               if (nm.eq.2) write(23,10) 'CHARGE -1'
c            endif
         write(23,30) i2-i1+1
         write(23,10) 'Voor DFTB'
ccc         write(26,30) i2-i1+1
ccc         write(26,10) 'Voor DFTB'
         do i=i1,i2
            chg=0
            atname=type(i)(1:1)
            if (atname(1:1).eq.'H') chg=1
            if (atname(1:1).eq.'C') chg=6
            if (atname(1:1).eq.'N') chg=7
            if (atname(1:1).eq.'O') chg=8
            if (atname(1:1).eq.'F') chg=9
            if (atname(1:1).eq.'S') chg=16
            if (type(i)(1:2).eq.'Cl') then
               atname='Cl'
               chg=17
            endif
            if (type(i)(1:2).eq.'Br') then
               atname='Br'
               chg=35
            endif
            if (chg.lt.0.5) then
               print*,' ***** Unknown atom type:',
     +              i,'   ',type(i),'   ',panm(i)
               stop
            endif
            write(23,15) atname,(10*x(3*i-3+m),m=1,3),chg
         enddo
         close(23)

         if (mold.eq.0) write(job,*) 'runscm fort.23 no'
         if (mold.gt.0) write(job,*) 'runscm fort.23 yes'
         istat=system(job)
         if (istat.ne.0) then
            print*,istat,' Error running DFTB-job'
            print*,job
            stop
         endif
         open(unit=25,file='adfrep.out',status='OLD',err=140)
         read(25,10)regel
         read (regel(27:39),*,end=150,err=150) eg1
         if (ipr.gt.0) print*,'  DFTB energy (a.u.) ',eg1

C        Get derivatives
         read(25,10) regel
         read(25,10) regel
         do i=i1,i2
            read(25,10) regel
            read(regel( 2:14),*,end=150,err=150) grad(3*i-2)
            read(regel(15:27),*,end=150,err=150) grad(3*i-1)
            read(regel(28:40),*,end=150,err=150) grad(3*i)
         enddo
         if (ipr.gt.2) then
            do i=i1,i2
               if (ipr.gt.3) print 21,i,(grad(3*i-3+m),m=1,3)
               do m=1,3
                  scr(m)=scr(m)+grad(3*i-3+m)
               enddo
            enddo
            print 61,(scr(m),m=1,3)
         endif

         if (mold.gt.0) then
C           Get charges
            read(25,10) regel
            read(25,10) regel
            do i=i1,i2
               read(25,10)regel
               read(regel(10:18),*,end=150,err=150) cgt(i)
ccc               write(26,15) atname,(10*x(3*i-3+m),m=1,3),cgt(i)
               som=som+cgt(i)
            enddo
            if (iprint.gt.2) print 62,som
         endif
         egtot=egtot+eg1
         close(25)
      enddo
ccc      close(26)

C     All input data from DFTB are available.
      egames=(egtot-aiofst)*au2kJ
      print 70,ndftb,egtot,aiofst,egames/ngr
      write(16,70) ndftb,egtot,aiofst,egames/ngr
      do i=1,nco
         f(i)=-grad(i)*fact
      enddo

      if (mold.gt.0) then
         som=0
         do i=1,nrpt
            som=som+cgt(i)
         enddo
         if (abs(som).gt.0.0001 .and. ipr.gt.1) then
            print 22,som
            write(16,22) som
         endif
         do i=1,nrpt
            cgt(i)=cgt(i)-som/nrpt
         enddo
         print 63,ndftb
         print 64,(cgt(i),i=1,nrpt)
         write(16,63) ndftb
         write(16,64) (cgt(i),i=1,nrpt)
         do m=0,maxat/nrpt-2
            do i=1,nrp
               cg(i+m*nrp)=cgt(i)*facq
            enddo
         enddo
      endif
      return

  140 print*,'***** Error opening file ADFREP.OUT'
      stop
  150 print*,'***** Error while reading ADFREP.OUT'
      stop

   10 format(a)
   15 format(a5,2x,3f13.6,f9.4)
   21 format(i5,5x,3f14.8,f12.3)
   22 format(' ***** Total charge not zero:',f8.3,'. Reset to zero!')
   30 format(i5,3f19.11)
   61 format(' Check sum rule for gradients: ',4f20.12)
   62 format(' Check sum rule for charges:   ',f10.5)
   63 format(' New DFTB charges found in cycle',i4,':')
   64 format(12f10.4)
   70 format(' DFTB call',i4,'   Total DFTB energy (a.u.)',f16.6,
     +     '   After offset',f12.3,' in kJ/mol:',f12.3)
      end
