      subroutine games(f,egames,ichad,mold,basis,iprint)
C     Delivers ab-initio energy and forces for a free molecule.
C     Program GAMESS-UK is called through a non-Fortran system call.

C     Input through parameters:
C        BASIS: basis set (character*30)
C        MOLD>0: obtain charges through calling MOLDEN
C            =0: do not do so
C        IPRINT regulates output to unit 16 and to screen.
C     Input through common block /PARS/:
C        NGR: number of molgroups
C        NRP: number of atoms in one molecule
C        X(1..): coordinates
C     Input through common block /AIDT/:
C        IAI=0: calculation of energy only (no derivatives)
C           =1: calculation of energy, 1st and 2nd derivatives
C           =2: calculation through previous call with IAI=1
C        NCYCGA: the number of the ab-initio cycle
C        AIOFST: offset (atomic units) for reasonable energies in kJ/mol
C        NAIB, NAIH, NAID: numbers of bond lengths, bond angles, 
C              and dihedrals, respectively (from call of INTCAR).
C        SAIB, SAIH, SAID: maximum allowed shifts in these parameters
C        NMOLS: the number of discrete molecules
C        IMOLS(1..NMOLS): the first atom of molecule I is IMOLS(I)
C     Input through common block /ATTP/:
C        TYPE(1..): atom types
C        PANM(1..): atom names

C     Output through parameters:
C        EGAMES: energy (kJ/mol)
C        ICHAD<0: new charges have been produced
C             >0: old values remain unchanged
C        F(1..): the negative derivatives of EGAMES to X(1..).

C     Output through saved arrays, created with IAI=1:
C        XSAVE: coordinates as produced by GAMESS
C        GRAD(1..3*NRP) Vector of first derivatives
C        AKAR(1..3*NRP,1..3*NRP) Matrix of second derivatives
C        EREF: last recalculated GAMESS energy

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'caidt.i'
      include 'cpars.i'

      integer i,i1,i2,ichad,iprint,istat,j,k,m,mold,nco,nm,nneg,nrpt
      double precision akar,akar1,amas,au2kj,bohr,chg,eg2,egames,
     +     egtot,eg1,eref,f,grad,grad1,rad,rms,rrms,s,s1,s2,scr,xsave
      dimension f(*),xsave(maxxco),
     +     akar(maxpar,maxpar),grad(maxpar),amas(maxat),scr(maxpar),
     +     akar1(maxpar,maxpar),grad1(maxpar),s1(maxpar),s2(maxpar)
      character regel*200,atname*2,job*100,filename*20,filenm*9,
     +     basis*30,title*30,basis1*30
      integer system
      save
ccccc      data ncycga /0/

      au2kJ=2625.5d0
      bohr=0.0529177d0
      rad=45/atan(1.0d0) 
      nrpt=nrp*ngr
      nco=3*nrpt

      if (ncycga.eq.0) then
         ncycga=1
C        Read data from file IN.INT
         if (iai.ne.0) then
            print*,'Calling intcar',1,ncycga,iai
            call intcar(amas,akar,grad,0,nneg,s1,akar1,grad1)
         endif
      endif
      
      if (iai.eq.1) then
         do i=1,nco
            do j=1,nco
               akar(i,j)=0
            enddo
         enddo
      endif
      egtot=0

      if (iai.le.1) then
C        Call GAMESS and collect data from the output
         if (basis(1:4).eq.'    ') then
            print*,' ***** Error in basis: ',basis
            stop
         endif
         do nm=1,nmols
            i1=imols(nm)
            i2=imols(nm+1)-1
C           Specifically for ionic dimer in CPR19!!!!!
            if (nmols.eq.2 .and. nrp.eq.28) then
               if (nm.eq.1) write(23,10) 'CHARGE 1'
               if (nm.eq.2) write(23,10) 'CHARGE -1'
            endif
C           Specifically for ionic dimer in CPR24!!!!!
            if (nmols.eq.3 .and. nrp.eq.20) then
               if (nm.eq.1) write(23,10) 'CHARGE 1'
               if (nm.eq.2) write(23,10) 'CHARGE -1'
            endif
C           Specifically for ionic dimer in CPR33!!!!!
            if (nmols.ge.2 .and. nrp.eq.43) then
               if (2*(nm/2).eq.nm-1) write(23,10) 'CHARGE -1'
               if (2*(nm/2).eq.nm) write(23,10) 'CHARGE 1'
            endif
            write(23,10) 'GEOMETRY  ANGSTROM'
            do i=i1,i2
               chg=0
               atname=type(i)(1:1)
               if (atname(1:1).eq.'H') chg=1
               if (atname(1:1).eq.'C') chg=6
               if (atname(1:1).eq.'N') chg=7
               if (atname(1:1).eq.'O') chg=8
               if (atname(1:1).eq.'F') chg=9
               if (atname(1:1).eq.'S') chg=16
               if (atname(1:1).eq.'I') then
                  chg=53
                  if (nrp.eq.100) then
C                    Specific for cpr27, I cannot be used in 6-31G**
                     atname='Cl'
                     chg=17
                     print*,'*****   Iodine changed to chlorine!'
                  endif
               endif
               if (type(i)(1:2).eq.'Si') then
                  atname='Si'
                  chg=14
               endif
               if (type(i)(1:2).eq.'Cl') then
                  atname='Cl'
                  chg=17
               endif
               if (type(i)(1:2).eq.'Br') then
                  atname='Br'
                  chg=35
               endif
               if (type(i)(1:2).eq.'Cu') then
                  atname='Cu'
                  chg=29
               endif
               if (type(i)(1:2).eq.'C ' .and.
     +              basis(1:7).eq.'DZPP-CP') atname='CP'
               if (atname(1:1).eq.'H' .and. 
     +              basis(1:9).eq.'B3LYP-DFT') then
                  if (type(i)(1:2).eq.'HC') then
                     atname='HC'
                  else
                     atname='HO'
                  endif
               endif
               if (chg.lt.0.5) then
                  print*,' ***** Unknown atom type in GAMES:',
     +                 i,'   ',type(i),'   ',panm(i)
                  stop
               endif
               write(23,11) (10*x(3*i-3+m),m=1,3),chg,atname
            enddo
            write(23,10) 'END'
            close(23)

            basis1=basis
  200       continue
C           Call GAMESS through system call of GAMES4UPACK
            open(unit=83,file='filename')
            read(83,10) filename
            close(83)
            i=iai
            if (basis1(1:6).eq.'IMPORT' .and. ncycga.eq.1) i=i+10
            write(job,12) i, basis1, filename(1:20), ncycga, nm
            if (basis1(1:6).eq.'IMPORT') then
               print*,' Importing old GAMESS-output'
               write(16,*) ' Importing old GAMESS-output'
            else
               print*,' Starting GAMESS-calculation, basis ',basis1
               write(16,*) ' Starting GAMESS-calculation, basis ',basis1
            endif
            print*,job
            istat=system(job)
            if (istat.ne.0) then
               print*,istat,' Error running Gamess-job'
               print*,job
               stop
            endif

C           Analyze GAMESS-output
            call leesg(iai,nm,i1,i2,iprint,
     +           xsave,amas,x,grad,akar,eg1,title)
            close(25)

            if (basis1.eq.'IMPORT') then
               print*,' ***** IMPORT file says: ',title
               write(16,*) ' ***** IMPORT file says: ',title
               if (iai.eq.0 .and. title.ne.bter2) then
                  print*,'Redo with basis: ',bter2
                  write(16,*) 'Redo with basis: ',bter2
                  basis1=bter2
                  goto 200
               endif
               if (iai.gt.0 .and. title.ne.btra2) then
                  print*,'Redo with basis: ',btra2
                  write(16,*) 'Redo with basis: ',btra2
                  basis1=btra2
                  goto 200
               endif
            else
               if (basis1.ne.title) then
                  print*,' ***** Error reading GAMESS output: ',title
                  stop
               endif
            endif

C           Run MOLDEN to obtain multipole moments
            if (mold.gt.0) then
               m=nmols/ngr
               if (m.ne.1) m=mod(nm-1,m)+1
               write(filenm,27) 'mol',m
   27          format(a3,i2.2,'.def')
               write(job,13) filenm,nm,abs(ichad)
c               print*,ngr,m,nm,filenm,ichad
c               print*, job
c               pause
               print*,'Running MOLDEN for molecule',nm
               write(16,*) 'Running MOLDEN for molecule',nm
               istat=system(job)
               if (istat.ne.0) then
                  print*,istat,' ***** Error running Molden-job'
                  stop
               endif
C              Print RMS and RRMS deviations
               open(unit=53,file='rms.dat',err=260)
               read(53,10,err=260,end=260) regel
               call realst(regel,5,rms)
               read(53,10,err=260,end=260) regel
               call realst(regel,5,rrms)
               print 14,rms,rrms
               write(16,14) rms,rrms
               close(53)
  260          continue
            endif
            if (eg1.gt.-0.01) then
               print*,' ***** Something is very wrong in GAMESS!'
               print*,'       The energy is not negative: ',eg1
               stop
            endif
            egtot=egtot+eg1
         enddo
         egames=(egtot-aiofst)*au2kJ
         print 70,ncycga,egtot,aiofst,egames/ngr
         write(16,70) ncycga,egtot,aiofst,egames/ngr
C        All input data from GAMESS are available.

C        Stop here if no derivatives are desired
         if (iai.eq.0) return
         eref=egames

C        First check of sum rules for gradient and force constant matrix
         if (iprint.gt.0) then
            do m=1,3
               scr(m)=0
               do i=1,nrpt
                  scr(m)=scr(m)+grad(3*i-3+m)
               enddo
            enddo
            print 61,(scr(m),m=1,3)
         endif
         if (iprint.gt.3) then
            print*,'Check sum rule for second derivatives'
            do i=1,nco
               k=0
               do m=1,3
                  scr(m)=0
                  do j=m,nco,3
                     scr(m)=scr(m)+akar(i,j)
                  enddo
                  if (abs(scr(m)).gt.0.00002001) k=1
               enddo
               if (k.eq.1) print 30,i,(scr(m),m=1,3)
            enddo
         endif

C        Scale results (after symmetrizing)
c         nco=3*nrpt
         do i=1,nco
            grad(i)=grad(i)*au2kJ/bohr
         enddo
         do i=1,nco
            do j=1,i
               s=(akar(i,j)+akar(j,i))*au2kJ/(2*bohr**2)
               akar(j,i)=s
               akar(i,j)=s
            enddo
         enddo
C        Write first and second derivatives to files 46 and hessian-cyc....
c         write(filename,15) ncycga
c   15 format('hessian-cyc.',i3.3)
c         print*,filename
c         open(47,file=filename)
         open(47,file='hessian')
         open(unit=46,form='UNFORMATTED')
         do i=1,nco
            write(47,*) i
            write(47,*) -grad(i)
            write(47,7777) (akar(i,j),j=1,nco)
 7777       format(15f9.0)
            write(46) i
            write(46) -grad(i)
            write(46) (akar(i,j),j=1,nco)
         enddo
         close(47)
         close(46)

c         print*,nco,au2kJ,bohr,au2kJ/bohr,au2kJ/(2*bohr**2)
c         do i=1,nco
c            print*,'AKAR  ',i,'    GRAD '
c            print*,grad(i)
c            print*,(akar(i,j),j=1,nco)
c         enddo
c         pause

C        Project out translations and rotations, calculate frequencies
         print*,'Calling intcar',2,ncycga,iai
         call intcar(amas,akar,grad,1,nneg,s1,akar1,grad1)
         do i=1,nco
            grad(i)=grad1(i)
            do j=1,nco
               akar(i,j)=akar1(i,j)
            enddo
         enddo
C        Now AKAR = H' and GRAD = G' for the reference structure
c         print*,' Calling Intcar 3 ',n,nco

c         do i=1,nco
c            print*,i,grad(i)
c            print 411,(akar(i,j),j=1,nco)
c  411       format(10f12.2)
c         enddo

         print*,'Calling intcar',3,ncycga,iai
         call intcar(amas,akar,grad,3,nneg,s1,akar1,grad1)
         do i=1,nco
            f(i)=-grad1(i)
         enddo
cc         print 74,(s1(i),i=1,naib)
cc         print 75,(s1(i),i=naib+1,naib+naih)
cc         print 76,(s1(i),i=naib+naih+1,naib+naih+naid)
cc         write(16,74) (s1(i),i=1,naib)
cc         write(16,75) (s1(i),i=naib+1,naib+naih)
cc         write(16,76) (s1(i),i=naib+naih+1,naib+naih+naid)

C        Output of forces
c         print*,' Negative derivatives, free molecule and total'
c         do i=1,nrpt
c            print 42,i,(-grad1(3*i-3+m),m=1,3),(f(3*i-3+m),m=1,3)
c   42 format(i5,4x,3f11.2,4x,3f11.2)

c         enddo

C        Second check of sum rules for gradient and force constant matrix
         if (iprint.gt.0) then
            do m=1,3
               scr(m)=0
               do i=1,nrpt
                  scr(m)=scr(m)+grad1(3*i-3+m)
               enddo
            enddo
            print 61,(scr(m),m=1,3)
         endif
         if (iprint.gt.3) then
            print*,'Check sum rule for second derivatives'
            do i=1,nco
               k=0
               do m=1,3
                  scr(m)=0
                  do j=m,nco,3
                     scr(m)=scr(m)+akar1(i,j)
                  enddo
                  if (abs(scr(m)).gt.1.0d-9) k=1
               enddo
               if (k.eq.1) print 30,i,(scr(m),m=1,3)
            enddo
         endif
      endif

      if (iai.eq.2) then
C        Calculate energy and forces in the harmonic approximation
C        from the results obtained in the calculation with IAI = 1
c         nco=3*nrpt
c         print*,' Calling Intcar 2 ',n
         print*,'Calling intcar',4,ncycga,iai
         call intcar(amas,akar,grad,2,nneg,s2,akar1,grad1)
         do i=1,nco
            scr(i)=s2(i)-s1(i)
            if (scr(i).gt. 180) scr(i)=scr(i)-360
            if (scr(i).lt.-180) scr(i)=scr(i)+360
         enddo

c         do i=1,nco
c            print*,i,scr(i),grad(i),grad1(i)
c            print 411,(akar(i,j),j=1,nco)
c  411       format(10f12.2)
c         enddo

         eg1=0
         eg2=0
         do i=1,nco
            if (i.le.naib) then
               scr(i)=scr(i)/10
            else
               scr(i)=scr(i)/rad
            endif
         enddo
c         print 777,(nint(10**7*scr(i)),i=1,nco)
c  777    format(7(3i6,3x))
         do i=1,nco
            eg1=eg1+grad(i)*scr(i)
            f(i)=-grad1(i)
            do j=1,nco
               eg2=eg2+akar(i,j)*scr(i)*scr(j)/2
               f(i)=f(i)-akar1(i,j)*scr(j)
            enddo
         enddo
         egames=eref+eg1+eg2

         if (iprint.gt.5) then
            print 31,eref,eg1,eg2,egames
            write(16,31) eref,eg1,eg2,egames
         endif
         if (iprint.gt.6) then
            print*,' Coordinates and negative derivatives'
            do i=1,nrpt
               print 43,i,(x(3*i-3+m),m=1,3),
     +              (-grad1(3*i-3+m),m=1,3),(f(3*i-3+m),m=1,3)
            enddo
         endif
      endif
c      do i=1,5
c         print*,i,grad(i),(akar(i,j),j=1,6)
c      enddo
c      do i=1,5
c         print*,i,grad1(i),(akar1(i,j),j=1,6)
c      enddo
c      print*,'GAMESS Forces:',(f(i),i=1,nco)
c   74 format(' Bonds (A): ',50f7.3)
c   75 format(' Angles   : ',50f7.1)
c   76 format(' Dihedrals: ',50f7.1)

   10 format(a)
   11 format(3f13.6,f9.1,2x,a5)
   12 format('games4upack ',i3,2x,a30,2x,a20,4i5)
   13 format('molden4upack ',a,2i4)
   14 format(' The RMS and RRMS deviations are',2f9.4)
   30 format(i5,3f19.11)
   31 format('    Extrapolated Gamess-energy (order 0, 1, 2, total) ',
     +     /,4f15.6)
   43 format(i5,4x,3f11.5,4x,3f11.2,4x,3f11.2)
   61 format(' Check sum rule for gradients: ',3f20.12)
   70 format(' Cycle',i4,'   Total Gamess energy (a.u.)',f16.6,
     +     '   After offset',f12.3,' in kJ/mol:',f12.3)
      end
