      subroutine getsym(ispgr,spname,nscrd,regels,iprint)

C     GETSYM is called by LEESX and calls LEESR.      
C     Reads space group data for triclinic, monoclinic, or orthorhombic
C     space groups and selects data for the required space group.
C     Alternatively, use information from SYMM cards.

C     Input through parameters:
C        ISPGR: space group number
C        SPNAME: space group name.
C        NSCRD: number of cards with symmetry elements
C        REGELS(1..NSCRD): symmetry information regels
C        IPRINT regulates the quantity of output.

C     Output through parameters:
C        NSCRD set to zero: no calling of SPEPOS and IDENT necessary.

C     Input from file IN.SPG (unit 11) for every implemented space group.
C     See procedure LEESR for structure of this file
C        IREF(1..12) =  0/1: disables/enables optimization of parameters:
C        ISCR: not used
C        MSOM(m) = sum over MSP(m), compares with IREF.
C        INNER = 0 if the cell has no dipole moment, otherwise it is unchanged
C        ISPGR and SPNAME: changed from input to consistent values, if
C            necessary.

C---------------------------------------------------------------------------

C     Output through parameters:
C        ISPGR > 0: space group number
C              < 0: space group not found in file ALL.SPG
C        SPNAME: space group name. Special possibilities:
C               'unknown' if only symmetry operators are found
C               'r-spname' if symmetry has been artificially reduced
C                          for special positions or high symmetry

C     Output to common block /SPGR/ (see LEESR for details): 
C        NSPGR: the number of non-E low-symmetry operators
C        MSP, TSP: these data, see LEESR
C        NOP: the total number of complete transformation operators
C        MSPG(1000,3,3), TSPG(1000,3): these data
C        Various information for FDAT
C     Output to common block /NONB/:
C        INNER = 0 if the cell has no dipole moment, otherwise it is unchanged
C        MSOM(m) = sum over MSP(m), compares with IREF.
C     Output to common block /CPARS/:
C        IREF: through LEESR
C---------------------------------------------------------------------------

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cnonb.i'
      include 'cpars.i'

      integer i,i0,i9,ic,ihisym,iklaar,il,inum,iprint,is,iscr,isl,
     +     ispgr,itel,itr,j,m,m1,m2,m3,me,mn,mscr,mspgin,ngen,nosp,
     +     nprev,nscrd,nspr,ntal
      double precision a,t,tn,tscr,tspgin

      dimension iscr(6),
     +     mspgin(3,3),tspgin(3),mn(3,3),tn(3),me(3,3),tscr(3),mscr(3,3)
      character*10 um(3)
      character*80 regel, regels(*)
      character code*4,ch*1

      character*8 spname

      print 11,ispgr,spname,nscrd
      write(16,11) ispgr,spname,nscrd
      nosp=0
  100 continue
      nspgr=0
      nop=1

C     Get symmetry information from space group number or name
      if (spname.ne.'nospgr' .and. spname(1:2).ne.'r-') then
         call leesr(ispgr,spname,iref,iscr,msom,inner)
         nspr=nspgr
C        Delivers:
C           NSPGR: the number of symmetry operators (E excluded)
C           MSP: the rotation components
C           TSP: the translation components
C        LEESR may set INNER to zero

c         print*,'ispgr',ispgr,nspgr,nscrd
         if (ispgr.gt.-3) then
            nop=nspgr+1
            print*,nspgr,' non-E operators found from LEESR'
            write(16,*) nspgr,' non-E operators found from LEESR'
            do i=1,nspgr
c               print*,i,(msp(3*i-3+m),m=1,3),(tsp(3*i-3+m),m=1,3)
               do m=1,3
                  do m1=1,3
                     mspg(i+1,m,m1)=0
                  enddo
                  j=3*i-3+m
                  mspg(i+1,m,m)=msp(j)
                  tspg(i+1,m)=tsp(j)
               enddo
            enddo
         else
            nosp=1
            print*,'***** Unknown space group name found'
            write(16,*) '***** Unknown space group name found'
            print 19,nscrd
            print*,' Type GO if you accept this and wish to continue'
ccc            read*, regel
ccc            if (regel.ne.'GO' .and. regel.ne.'go') stop
C           These two sets ccc were originally absent.That is safer but
C              inconvenient when doing many calculations in runppall.spf.
C           But STOP here is always safe, and necessary if an unknown space
C              group has been given without SYMM information. 
C           But when all SYMM cards are present OR this structure has been
C              completely expanded to P1, then the following will apply:
               print 19,nscrd
               write(16,19) nscrd
               spname='P1'
               goto 100
         endif
      elseif (nscrd.eq.0) then
         nosp=1
         print*,'***** No space group symmetry information found'
         write(16,*)
     +        ' ***** No space group symmetry information found'
         print 19,nscrd
         print*,' Type GO if you accept this and wish to continue'
         read*, regel
         if (regel.ne.'GO' .and. regel.ne.'go') stop
C        STOP here is always recommended, unless P1 is accepted.
         print 19,nscrd
         write(16,19) nscrd
         spname='P1'
         goto 100
      endif

C     Get symmetry information from SYMM cards
      i0=ichar('0')
      i9=ichar('9')
ccc      nop=1
      do m1=1,3
         tspg(1,m1)=0
         do m2=1,3
            me(m1,m2)=0
            mspg(1,m1,m2)=0
         enddo
         mspg(1,m1,m1)=1
         me(m1,m1)=1
      enddo

      do ngen=1,nscrd
         regel=regels(ngen)
         code=regel(1:4)
         if (code.ne.'symm' .and. code.ne.'SYMM') then
            print*,' ***** Wrong symmetry code transmitted to EXPSYM'
            stop
         endif
         do m1=1,3
            tspgin(m1)=0
            do m2=1,3
               mspgin(m1,m2)=0
            enddo
         enddo
         m=1
         il=4
         is=1
         itr=0
         isl=0
  130    continue
C        Read next character
         il=il+1
         if (il.eq.80) goto 140
         if (il.gt.80) then
            print*,' ***** Error in generator in EXPSYM',ngen
            print*,regel
            stop
         endif
         ch=regel(il:il)
         if (ch.eq.' ') goto 130
         if (ch.eq.'-') is=-1
         if (ch.eq.'+') is=1
         if (ch.eq.'x' .or. ch.eq.'X') mspgin(m,1)=is
         if (ch.eq.'y' .or. ch.eq.'Y') mspgin(m,2)=is
         if (ch.eq.'z' .or. ch.eq.'Z') mspgin(m,3)=is
         if (ch.eq.'/') isl=1
         if (ch.eq.',') goto 140
         ic=ichar(ch)
         if (ic.ge.i0 .and. ic.le.i9) read(ch,12) itr
         if (isl.eq.0) then
            itel=itr*is
         else
            inum=itr
         endif
         goto 130

  140    continue
         if (isl.eq.0 .or. inum.eq.0) then
            tspgin(m)=itel
         else
            tspgin(m)=dfloat(itel)/dfloat(inum)
         endif
         if (m.lt.3) then
            m=m+1
            is=1
            isl=0
            itr=0
            goto 130
         endif

c         if (iprint.gt.1) then
            print 13,ngen
            write(16,13) ngen
c         endif
c         if (iprint.gt.3) then
            print 23,((mspgin(m1,m2),m2=1,3),tspgin(m1),m1=1,3)
            write(16,23) ((mspgin(m1,m2),m2=1,3),tspgin(m1),m1=1,3)
c         endif
C        The card has been read and interpreted

         nprev=nop
C        Check whether this is a new one
         do i=1,nop
            do m1=1,3
               do m2=1,3
                  if (mspgin(m1,m2).ne.mspg(i,m1,m2)) goto 185
               enddo
               a=tspgin(m1)-tspg(i,m1)
               if (abs(a-nint(a)).gt.0.01) goto 185
            enddo
c            if (iprint.gt.1) then
               print 18,ngen
               write(16,18) ngen
c            endif
            goto 700
  185       continue
         enddo

C        Evaluate a new symmetry generator
         ntal=0
         do m1=1,3
            tn(m1)=0
            do m2=1,3
               mn(m1,m2)=me(m1,m2)
            enddo
         enddo

C        Do the next operation NTAL of this generator
  255    ntal=ntal+1
         iklaar=1
C        Calculate M(n+1) = M(1).M(n) and T(n+1) = M(1).T(n)+T(1)
         do m1=1,3
            tscr(m1)=tn(m1)
            tn(m1)=tspgin(m1)
            do m2=1,3
               mscr(m1,m2)=mn(m1,m2)
               mn(m1,m2)=0
            enddo
         enddo
         do m1=1,3
            do m2=1,3
               tn(m1)=tn(m1)+mspgin(m1,m2)*tscr(m2)
               do m3=1,3
                  mn(m1,m2)=mn(m1,m2)+mspgin(m1,m3)*mscr(m3,m2)
               enddo
               if (mn(m1,m2).ne.me(m1,m2)) iklaar=0
            enddo
            if (abs(tn(m1)).gt.0.95) tn(m1)=tn(m1)-nint(tn(m1))
            if (abs(tn(m1)).gt.0.1) iklaar=0
         enddo
         if (iprint.gt.3) then
            print*,'  Acting',ntal,' times on itself delivers:'
            print 23,((mn(m1,m2),m2=1,3),tn(m1),m1=1,3)
         endif

C        The last operation has been done if IKLAAR=1
         if (iklaar.eq.1) then
c            if (iprint.gt.1) then
               print 16,ngen,ntal
               write(16,16) ngen,ntal
c            endif
            goto 700
         endif

C        Apply this operator to a known operation I
         do i=1,nprev
            nop=nop+1
            do m1=1,3
               tspg(nop,m1)=tn(m1)
               do m2=1,3
                  mspg(nop,m1,m2)=0
               enddo
            enddo
            do m1=1,3
               do m2=1,3
                  tspg(nop,m1)=tspg(nop,m1)+mn(m1,m2)*tspg(i,m2)
                  do m3=1,3
                     mspg(nop,m1,m2)=
     +                    mspg(nop,m1,m2)+mn(m1,m3)*mspg(i,m3,m2)
                  enddo
               enddo
               if (abs(tspg(nop,m1)).gt.0.99d0)
     +              tspg(nop,m1)=tspg(nop,m1)-nint(tspg(nop,m1))
            enddo

C           Check whether the result was already known
            do j=1,nop-1
               do m1=1,3
                  do m2=1,3
                     if (mspg(j,m1,m2).ne.mspg(nop,m1,m2)) goto 435
                  enddo
                  a=tspg(j,m1)-tspg(nop,m1)
                  if (abs(a-nint(a)).gt.0.01) goto 435
               enddo
c               if (iprint.gt.1) then
                  print 14,ngen,ntal,i
                  write(16,14) ngen,ntal,i
c               endif
               nop=nop-1
               goto 495
  435          continue
            enddo

c            if (iprint.gt.1) then
               print 15,ngen,ntal,i,nop
               write(16,15) ngen,ntal,i,nop
c            endif
            if (iprint.gt.3) then
               print 23,((mspg(nop,m1,m2),m2=1,3),tspg(nop,m1),m1=1,3)
            endif
  495       continue

         enddo
         goto 255

  700    continue
      enddo

cc      nop=nspgr+1
      print 17,nop
      write(16,17) nop
      if (iprint.gt.3) then
         do j=1,nop
            print*,' Operator',j
            print 23,((mspg(j,m1,m2),m2=1,3),tspg(j,m1),m1=1,3)
         enddo
      endif

C     Write results to file, if desired for unknown reason
      if (iprint.gt.5) then
         open(unit=20,file='expsym.out')
         do i=1,nop
            do m=1,3
               um(m)='          '
               if (mspg(i,m,1).gt.0) um(m)(1:2)='+x'
               if (mspg(i,m,1).lt.0) um(m)(1:2)='-x'
               if (mspg(i,m,2).gt.0) um(m)(3:4)='+y'
               if (mspg(i,m,2).lt.0) um(m)(3:4)='-y'
               if (mspg(i,m,3).gt.0) um(m)(5:6)='+z'
               if (mspg(i,m,3).lt.0) um(m)(5:6)='-z'
               a=tspg(i,m)
               if (abs(a-0.5000d0) .lt. 0.0002d0) um(m)(7:10)='+1/2'
               if (abs(a-0.2500d0) .lt. 0.0002d0) um(m)(7:10)='+1/4'
               if (abs(a-0.7500d0) .lt. 0.0002d0) um(m)(7:10)='+3/4'
               if (abs(a-0.3333d0) .lt. 0.0002d0) um(m)(7:10)='+1/3'
               if (abs(a-0.6667d0) .lt. 0.0002d0) um(m)(7:10)='+2/3'
               if (abs(a-0.1667d0) .lt. 0.0002d0) um(m)(7:10)='+1/6'
               if (abs(a-0.8333d0) .lt. 0.0002d0) um(m)(7:10)='+5/3'
               if (abs(a+0.5000d0) .lt. 0.0002d0) um(m)(7:10)='+1/2'
               if (abs(a+0.2500d0) .lt. 0.0002d0) um(m)(7:10)='+3/4'
               if (abs(a+0.7500d0) .lt. 0.0002d0) um(m)(7:10)='+1/4'
               if (abs(a+0.3333d0) .lt. 0.0002d0) um(m)(7:10)='+2/3'
               if (abs(a+0.6667d0) .lt. 0.0002d0) um(m)(7:10)='+1/3'
               if (abs(a+0.1667d0) .lt. 0.0002d0) um(m)(7:10)='+5/6'
               if (abs(a+0.8333d0) .lt. 0.0002d0) um(m)(7:10)='+1/3'
            enddo
            write(20,24) (um(m),m=1,3)
         enddo
      endif

C     Check for high symmetry
      nspgr=0
      do i=1,nop
         ihisym=0
         do m1=1,3
c               print*,i,m1,'   ',(mspg(i,m1,m2),m2=1,3)
            if (abs(mspg(i,m1,m1)).ne.1) ihisym=1
            t=2*tspg(i,m1)
            if (abs(t-nint(t)).gt.0.001d0) ihisym=1
            do m2=1,3
               if ((m1.ne.m2) .and. mspg(i,m1,m2).ne.0) ihisym=1
            enddo
         enddo
         if (ihisym.eq.1) then
            if (iprint.gt.2) print*,' ***** Symmetry operator',i,
     +           ' higher than orthorhombic'
         else
            if (i.gt.1) then
               do m=1,3
                  msp(3*nspgr+m)=mspg(i,m,m)
                  tsp(3*nspgr+m)=tspg(i,m)
               enddo
               nspgr=nspgr+1
            endif
         endif
      enddo
      if (nosp.eq.1) ispgr=-10
      if (spname(1:6).eq.'nospgr' .and. nscrd.eq.1) then
         ispgr=1
         spname='P1'
         goto 100
      endif
      print 20,ispgr,spname,nscrd
      write(16,20) ispgr,spname,nscrd
      if (iprint.gt.1) then
         print*,nspr,' operators found from LEESR'
         print*,nop,' operators found in total'
         print*,nspgr,' non-E low-symmetry operators found'
      endif
      if (nspgr.eq.nspr) nscrd=0

   11 format(' GETSYM gets symmetry information for space group',i5,
     +     ' named ',a,' and', i4,' symmetry cards')
   20 format(' END GETSYM for space group',i5,
     +     ' named ',a,' and', i4,' symmetry cards')
   12 format(i1)
   13 format('    SYMM generator',i3,' interpreted')
   14 format('    Symmetry generator',i3,' multiplicity',i3,
     +     ' acting on operation',i3,' creates no new one')
   15 format('    Symmetry generator',i3,' multiplicity',i3,
     +     ' acting on operation',i3,' creates new operation',i3)
   16 format('    Symmetry generator',i3,' multiplicity',i3,' finished')
   17 format('  Symmetry analysis finished:',i4,' operations')
   18 format('    Symmetry generator',i3,' is already known')
   19 format('  Assuming space group P1 with',i4,' symmetry operators')
   23 format(3(3i4,f8.4,4x))
   24 format('symm  ',a10,',   ',a10,',   ',a10)
      end
