      subroutine ident(nrp,nfound,nconn,naam1,igr,at1,at2,
     +     ngr,nmol,index,iprint)

C     Tries to find the correct labels for a set of connected atoms.

C     Input through parameters:
C        Regarding the topology data:
C           NRP: number of atoms in the topology file
C        Regarding the coordinate data:
C           NFOUND: number of atoms
C           NCONN: number of bond connections
C           NAAM1(1..): atom names
C           IGR(1..): molecule numbers for each atom
C           AT1, AT2 (1..NCONN): numbers of connected atoms as found from
C              coordinates by subroutine SPEPOS or CONN
C        IPRINT regulates the quantity of output.

C     Input from common block /BIND/:
C        IB,JB(1..NBOND): code numbers for bonded atoms
C     Input from common block /ATTP/:
C        PANM(1..): atom names in topology file

C     Output parameters:
C        NGR:  number of topology groups in the coordinate file
C        NMOL: number of separate molecules in the coordinate file
C        INDEX(1..): connects topology atoms with coordinate atoms

      implicit none
      include 'params.i'
      include 'cbond.i'

      integer nrp,nfound,nconn,igr,ngr,nmol,index,iprint,at1,at2,
     +     nat,ntot,nsh,natsh,ncorr,nchar,maxfrg,maxnma,
     +     tgrp,cgrp,tshell,cshell,table,corr,frag,
     +     i,j,j1,jj,k,k1,l,m,mm,m1,m2,m4,m5,n,n1,n2,
     +     ig,ilink,ingr,ink,iprerr,iprsav,isav,itry,ic,ifr,ifrat,
     +     iass,iass1,iatt1,iatt2,icorr,inoh,icshell,itop,itshell,
     +     jngr,mma,mt,mfr,natf,nadd,nma,nsol,ntgrp,ntshell,ncshell,
     +     natgr,ncgrp,nconnt,nfrag,ngrat,nki,nkies,inew,ipr1
      parameter(nat=200,ntot=1600,nsh=50,natsh=30,ncorr=50,
     +     nchar=200,maxfrg=10,maxnma=20)
      character*1 atom,att(nat)
      character*8 naam1(0:maxat)
      character*200 formul,form1,form2,xxxxx,frgfrm(nat),grpfrm(nat)
C     Number of characters here must be at least equal to nchar
      dimension at1(*),at2(*),index(*),igr(0:maxat),
     +     iass(ncorr),iass1(ncorr),natgr(ncorr),ilink(maxat),
     +     iatt1(maxat),iatt2(maxat),nkies(40,8),
     +     tshell(nat,nsh,natsh),itshell(nat,nsh),ntshell(nat),
     +     cshell(ntot,nsh,natsh),icshell(ntot,nsh),ncshell(ntot),
     +     table(maxat,ncorr),icorr(maxat),tgrp(maxat,20),ntgrp(maxat),
     +     corr(maxnma,nat),frag(maxfrg,nat),natf(nat),isav(nat),
     +     cgrp(maxat,20),ncgrp(maxat)
      data inew /0/
      save inew

      ipr1=iprint
      if (inew.eq.1) ipr1=-1
      inew=1
      if (ipr1.ge.0) print*,'Start IDENT, trying to identify atoms'
      if (ipr1.ge.0) write(16,*)'Start IDENT, trying to identify atoms'
c      ipr1=4
      if (ipr1.gt.2) then
         do i=1,nmol
            print*,' Molecule',i
            do j=1,nfound
               if (igr(j).eq.i) print*,j
            enddo
         enddo
      endif
         
      inoh=0
c     print*,' give INOH'
c      read*,inoh

  100 continue
      iprerr=1
      naam1(0)=' '
c      print*,' Restart with inoh =',inoh,ngr,nmol
c      pause

      do i=1,nat
         index(i)=0
         natf(i)=0
         ntshell(i)=0
         do j=1,nsh
            itshell(i,j)=0
            do k=1,natsh
               tshell(i,j,k)=0
            enddo
         enddo
         do j=1,maxfrg
            frag(j,i)=0
         enddo
      enddo
      do i=1,ntot
         ncshell(i)=0
         do j=1,nsh
            icshell(i,j)=0
            do k=1,natsh
               cshell(i,j,k)=0
            enddo
         enddo
      enddo
      do i=1,16
         do j=1,8
            nkies(i,j)=0
         enddo
      enddo

C     Determine connectivity from topology file
      if (ipr1.gt.0) print*,
     +     'Determine connectivity from topology file'
      nconnt=0
      do i=1,nrp
         do j=i+1,nrp
            if (panm(i)(1:1).eq.'H' .and. panm(j)(1:1).eq.'H')
     +           goto 102
            do k=1,nbond
               if ((i.eq.ib(k) .and. j.eq.jb(k)) .or.
     +              (j.eq.ib(k) .and. i.eq.jb(k))) then
                  nconnt=nconnt+1
                  iatt1(nconnt)=i
                  iatt2(nconnt)=j
                  if (ipr1.gt.4) print 44,nconnt,i,j
               endif
            enddo
  102       continue
         enddo
      enddo
      if (ipr1.gt.0) then
         print*,nrp,' atoms found from connectivity'
         print*,nconnt,' bonds found from connectivity'
      endif
C     For bond NCONNT, the atoms are ATT1(nconnt) and ATT2(nconnt)

C     Make groups of surrounding atoms from topology
      if (ipr1.gt.0) print*,' Atom surroundings from topology'
      do i=1,nrp
         ntgrp(i)=0
         do j=1,nconnt
            k=0
            if (iatt1(j).eq.i) k=iatt2(j) 
            if (iatt2(j).eq.i) k=iatt1(j)
            if (k.gt.0) then
               if (panm(k)(1:1).eq.'H' .and. inoh.gt.0) goto 105
               ntgrp(i)=ntgrp(i)+1
               tgrp(i,ntgrp(i))=k
            endif
  105       continue
         enddo
         if (ipr1.gt.3) 
     +        print 40,panm(i),(panm(tgrp(i,j)),j=1,ntgrp(i))
      enddo
C     For atom i, TGRP(i, ntgrp(i)) contains all surrounding atoms

C     Create shells from topology
      if (ipr1.gt.3) print*,' Shells for each atom from topology'
      do i=1,nrp
         k=1
         itshell(i,k)=1
         tshell(i,k,1)=i
  110    continue
         k=k+1
         n=0
         do j=1,itshell(i,k-1)
            do m=1,ntgrp(tshell(i,k-1,j))
               l=tgrp(tshell(i,k-1,j),m)
               do k1=1,k
                  do j1=1,itshell(i,k1)
                     if (l.eq.tshell(i,k1,j1)) goto 120
                  enddo
               enddo
               n=n+1
               if (i.gt.nat) then
                  print*,' ***** Increase NAT',i
                  goto 500
               endif
               if (k.gt.nsh) then
                  print*,' ***** Increase NSH',k
                  goto 500
               endif
               if (n.gt.natsh) then
                  print*,' ***** Increase NATSH',n
                  goto 500
               endif
               itshell(i,k)=n
               tshell(i,k,n)=l
  120          continue
            enddo
         enddo
         if (n.ne.0) then
C           Sort for atom types in the same shell
            do m=1,n
               isav(m)=tshell(i,k,m)
               att(m)=panm(isav(m))(1:1)
            enddo
            do m1=1,n
               atom=' '
               do m2=1,n
                  if (att(m2).ge.atom) then
                     atom=att(m2)
                     mm=m2
                  endif
               enddo
               att(mm)=' '
               tshell(i,k,m1)=isav(mm)
            enddo
            goto 110
         endif
         ntshell(i)=k-1
         do k=1,ntshell(i)
            if (ipr1.gt.5) print*,i,k,itshell(i,k),'   ',
     +           (panm(tshell(i,k,m)),m=1,itshell(i,k))
         enddo
      enddo
      do i=1,nrp
         if (ipr1.gt.3) print 60,
     +        i,panm(i),ntshell(i),(itshell(i,k),k=1,ntshell(i))
         do j=1,i-1
            if (ntshell(i).ne.ntshell(j)) goto 160
            do k=1,ntshell(i)
               if (itshell(i,k).ne.itshell(j,k)) goto 160
               do m=1,itshell(i,k)
                  if (panm(tshell(i,k,m))(1:1).ne.
     +                 panm(tshell(j,k,m))(1:1)) goto 160
               enddo
            enddo
            if (ipr1.gt.3) print*,
     +           ' Possible equivalent atoms within topology', i,j
  160       continue
         enddo
      enddo

C     Check for separate fragments
      nfrag=0
      do i=1,nrp
         if (panm(i)(1:1).eq.'H' .and. inoh.gt.0) goto 180
         do n=1,nfrag
            do m=1,natf(nfrag)
               if (i.eq.frag(nfrag,m)) goto 180
            enddo
         enddo
         nfrag=nfrag+1
         if (nfrag.gt.maxfrg) then
            print*,' ***** Increase maxfrg',nfrag
            goto 500
         endif
         n=0
         do k=1,ntshell(i)
            do m=1,itshell(i,k)
               n=n+1
               frag(nfrag,n)=tshell(i,k,m)
            enddo
         enddo
         natf(nfrag)=n
  180    continue
      enddo

C     Find bruto formula for topology fragments
      do ifr=1,nfrag
         if (ipr1.gt.1) then
            print 61,ifr,natf(ifr)
            print 62,(frag(ifr,m),m=1,natf(ifr))
         endif            
         j=0
         formul=' '
         do ifrat=1,natf(ifr)
            i=frag(ifr,ifrat)
            atom=panm(i)(1:1)
            if (atom.ne.'H' .or. inoh.eq.0) then
               j=j+1
               formul=formul(1:j)//atom
            endif
         enddo
         formul=formul(2:j+1)
         frgfrm(ifr)=formul
         if (ipr1.gt.1) then
            print*,'Bruto molecule for fragment',ifr
            print 41,frgfrm(ifr)
         endif
      enddo
C     End of topology analysis

      
C     Start coordinate analysis
C     Make groups of surrounding atoms from coordinates
      if (ipr1.gt.0) print*,' Atom surroundings from coordinates'
c      print*,(igr(k),k=1,nfound)
      do ig=1,nmol
         if (ipr1.gt.0) print*,' Analyzing molecule', ig
         ngrat=0
         formul=' '
         do i=1,nfound
            if (naam1(i)(1:1).eq.'H' .and. inoh.gt.0) goto 200
            if (igr(i).eq.ig) then
               atom=naam1(i)(1:1)
               ngrat=ngrat+1
               formul=formul(1:ngrat)//atom
               ncgrp(i)=0
               do j=1,nconn
                  k=0
                  if (at1(j).eq.i) k=at2(j) 
                  if (at2(j).eq.i) k=at1(j)
                  if (k.gt.0) then
                     if (naam1(k)(1:1).eq.'H' .and. inoh.gt.0)
     +                    goto 190
                     ncgrp(i)=ncgrp(i)+1
                     cgrp(i,ncgrp(i))=k
                  endif
  190             continue
               enddo
               if (ipr1.gt.3)
     +              print 40,naam1(i),(naam1(cgrp(i,j)),j=1,ncgrp(i))
            endif
  200       continue
         enddo
         grpfrm(ig)=formul(2:ngrat+1)
         if (ipr1.gt.1) then
            print*,' Bruto formula for molecule',ig,
     +            ' with',ngrat,'atoms:'
            print 41,grpfrm(ig)
         endif

C        Create shells from coordinates
         if (ipr1.gt.3) print*,
     +        ' Shells for each atom from coordinates'
         do i=1,nfound
            if (naam1(i)(1:1).eq.'H' .and. inoh.gt.0) goto 230
            if (igr(i).eq.ig) then
               k=1
               icshell(i,k)=1
               cshell(i,k,1)=i
  210          continue
               k=k+1
               n=0
               do j=1,icshell(i,k-1)
                  do m=1,ncgrp(cshell(i,k-1,j))
                     l=cgrp(cshell(i,k-1,j),m)
                     do k1=1,k
                        do j1=1,icshell(i,k1)
                           if (l.eq.cshell(i,k1,j1)) goto 220
                        enddo
                     enddo
                     n=n+1
                     if (i.gt.ntot) then
                        print*,' ***** Increase NTOT',i
                        goto 500
                     endif
                     if (k.gt.nsh) then
                        print*,' ***** Increase NSH',k
                        goto 500
                     endif
                     if (n.gt.natsh) then
                        print*,' ***** Increase NATSH',n
                        goto 500
                     endif
                     icshell(i,k)=n
                     cshell(i,k,n)=l
  220                continue
                  enddo
               enddo
               if (n.ne.0) then
C                 Sort for atom types in the same shell
                  do m=1,n
                     isav(m)=cshell(i,k,m)
                     att(m)=naam1(isav(m))(1:1)
                  enddo
                  do m1=1,n
                     atom=' '
                     do m2=1,n
                        if (att(m2).ge.atom) then
                           atom=att(m2)
                           mm=m2
                        endif
                     enddo
                     att(mm)=' '
                     cshell(i,k,m1)=isav(mm)
                  enddo
                  goto 210
               endif
               ncshell(i)=k-1
               do k=1,ncshell(i)
                  if (ipr1.gt.5) print*,i,k,icshell(i,k),'   ',
     +                 (naam1(cshell(i,k,m)),m=1,icshell(i,k))
               enddo
            endif
  230       continue
         enddo
         do i=1,nfound
            if (igr(i).eq.ig) then
               if (ipr1.gt.3) print 60,i,naam1(i),
     +              ncshell(i),(icshell(i,k),k=1,ncshell(i))
               do j=1,i-1
                  if (ncshell(i).ne.ncshell(j)) goto 240
                  do k=1,ncshell(i)
                     if (icshell(i,k).ne.icshell(j,k)) goto 240
                     do m=1,icshell(i,k)
                        if (naam1(cshell(i,k,m))(1:1).ne.
     +                       naam1(cshell(j,k,m))(1:1)) goto 240
                     enddo
                  enddo
                  if (ipr1.gt.4) print*,
     +                 ' Possible equivalence within coordinates', i,j
  240             continue
               enddo
            endif
         enddo
      enddo

C     Check connection between the two sets
      ngr=0
      if (ipr1.gt.0) then
         print*,' '
         print*,'Investigate connection between coordinates and ',
     +        'topology'
      endif
      do m=1,nchar
         xxxxx(m:m)='x'
      enddo
      do i=1,ncorr
         iass(i)=0
         iass1(i)=0
      enddo

      do ig=1,nmol
         do m2=1,nchar
            if (grpfrm(ig)(m2:m2).eq.' ') goto 260
         enddo
  260    continue
         natgr(ig)=m2-1
      enddo

      ingr=0
  270 continue
      if (ipr1.gt.0) print*,'Searching for topology set',ngr+1
      do ifr=1,nfrag
         form1=frgfrm(ifr)
         if (ipr1.gt.2) print 30,ifr,form1
         do m1=1,nchar
            atom=form1(m1:m1)
            if (atom.eq.' ') goto 272
         enddo
  272    continue
         m1=m1-1

         do ig=1,nmol
            if (natgr(ig).eq.m1 .and. iass(ig).eq. 0) then
               form2=grpfrm(ig)
               formul=form1
               if (ipr1.gt.3) print 31,ig,form2
               do m2=1,nchar
                  atom=form2(m2:m2)
                  if (atom.eq.' ') goto 276
                  do m=1,m1
                     if (atom.eq.formul(m:m)) then
                        formul(m:m)='x'
                        goto 274
                     endif
                  enddo
  274             continue
               enddo
  276          continue
               m2=m2-1
               if (formul(1:m1).eq.xxxxx(1:m1)) then
                  if (ipr1.gt.3) print 32,ig,ifr
                  ingr=ingr+1
                  iass(ig)=ifr
                  iass1(ingr)=ig
                  goto 280
               endif
            endif
         enddo
         if (ipr1.ge.0) then
            print*,' ***** No identification of atoms found!'
            write(16,*)' ***** No identification of atoms found!'
         endif
         if (inoh.eq.0) then
            if (ipr1.ge.0) then
               print 43
               write(16,43)
            endif
            inoh=inoh+1
            goto 100
         endif
         goto 500
  280    continue
      enddo

C     One fragment of the topology file is identified
      ngr=ngr+1
      if (ipr1.gt.0) then
         print 34,ngr
         print 41,'       Molecule  ',(ig,ig=1,nmol)
         print 41,'       Fragment  ',(iass(ig),ig=1,nmol)
      endif
      do ig=1,nmol
         if (iass(ig).eq.0) goto 270
      enddo
      if (ipr1.gt.0) print*,ngr,
     +     ' complete topology set(s) definitively identified'

      ingr=0
      ink=0
C     Investigate possibilities for connecting atoms

      do jngr=1,ngr
         do ifr=1,nfrag
            ink=ink+1
            nma=0
            ingr=ingr+1
            ig=iass1(ingr)
  305       continue
            if (ipr1.gt.3) print 35,ifr,ig
            do mfr=1,natf(ifr)
               i=frag(ifr,mfr)
               if (ipr1.gt.3) 
     +              print 45,i,panm(i),(itshell(i,k),k=1,ntshell(i))
               n=0
               if (ipr1.gt.4) print*,' Trying:'
               do j=1,nfound
                  if (igr(j).eq.ig) then
                     if (ipr1.gt.4) print 45,j,naam1(j),
     +                    (icshell(j,k),k=1,ncshell(j))
                     if (ntshell(i).ne.ncshell(j)) goto 310
                     do k=1,ntshell(i)
                        if (itshell(i,k).ne.icshell(j,k)) goto 310
                        do m=1,itshell(i,k)
                           if (panm(tshell(i,k,m))(1:1).ne.
     +                          naam1(cshell(j,k,m))(1:1)) goto 310
                        enddo
                     enddo
                     n=n+1
                     table(i,n)=j
  310                continue
                  endif
               enddo
               icorr(i)=n
               if (n.eq.0) then
                  if (iprerr.eq.1 .and. inoh.gt.0) then
                     print*,' ***** No equivalence found for atom',i,
     +                    ',  printing details once:'
                     iprerr=0
                     iprsav=ipr1
                     ipr1=10
                     goto 305
                  endif
                  print*,' ***** No equivalence found for atom',i
                  write(16,*) ' ***** No equivalence found for atom',i
                  ipr1=iprsav
                  if (inoh.eq.0) then
                     print 43
                     write(16,43)
                     inoh=inoh+1
                     goto 100
                  endif
                  goto 500
               endif
               if (ipr1.gt.3) print 42,i,panm(i),
     +              (table(i,j),naam1(table(i,j)),j=1,icorr(i))
               if (icorr(i).gt.nma .and. panm(i)(1:1).ne.'H') then
                  nma=icorr(i)
                  if (nma.gt.maxnma) then
                     print*,' ***** Increase maxnma',nma
                     goto 500
                  endif
                  itry=i
               endif
               do n=1,nma
                  corr(n,i)=0
               enddo
            enddo

C           Find acceptable correspondence(s) for the first non-H atom
C           with the most possibilities

            do n=1,nma
               if (ipr1.gt.2) print*,'Possible correspondence set',n
               do mfr=1,natf(ifr)
                  j=frag(ifr,mfr)
                  do k=1,nsh
                     do m=1,natsh
                        cshell(j,k,m)=abs(cshell(j,k,m))
                     enddo
                  enddo
               enddo
               j=table(itry,n)
               corr(n,itry)=j
               if (ipr1.gt.2) print 51,itry,panm(itry),j,naam1(j)
               do k=2,ntshell(itry)
C                 Consider the next shell (K)
                  do mt=1,itshell(itry,k)
C                    Consider the atoms (MT) in that shell
                     itop=tshell(itry,k,mt)
C                    ITOP is the topology atom number of MT
                     if (ipr1.gt.5) then
                        print*,'TOP',itop,'   ',(tgrp(itop,jj),
     +                       jj=1,ntgrp(itop))
                        print*,'COR',(corr(n,tgrp(itop,jj)),
     +                       jj=1,ntgrp(itop))
                     endif
                     nsol=0
                     do l=1,icorr(itop)
                        ic=table(itop,l)
C                       IC is a possible coordinate number corresponding to ITOP
                        if (ipr1.gt.5) print*,'COR   ',ic,
     +                       '     ',(cgrp(abs(ic),m5),
     +                       m5=1,ncgrp(abs(ic)))
                        if (ipr1.gt.3) print 46,k,mt,itop,ic,
     +                       (cshell(j,k,m),m=1,icshell(j,k))
                        do m=1,icshell(j,k)
                           if (ic.eq.cshell(j,k,m)) then
C                             Check compatibility of first shells
                              nadd=0
                              do m4=1,ntgrp(itop)
                                 do m5=1,ncgrp(ic)
                                    if (corr(n,tgrp(itop,m4)).eq.
     +                                   cgrp(abs(ic),m5)) then
                                       nadd=1
                                       m2=m
                                    endif
                                 enddo
                              enddo
                              if (ipr1.gt.3 .and. nadd.eq.1)
     +                             print 47,itop,panm(itop),ic,naam1(ic)
                              nsol=nsol+nadd
                           endif
                        enddo
                     enddo
                     if (nsol.eq.0) then
                        if (ipr1.gt.1) print*, '    No solutions for',
     +                       ' atom',itop,'  ',panm(itop)
                        goto 340
                     endif
                     if (nsol.gt.1) then
                        if (ipr1.gt.3) print*, nsol,
     +                       ' solutions for atom', 
     +                       itop,'   ',panm(itop),'  Choose ',
     +                       cshell(j,k,m2),'  ',naam1(cshell(j,k,m2))
                     endif
                     corr(n,itop)=cshell(j,k,m2)
                     cshell(j,k,m2)=-cshell(j,k,m2)
                  enddo
               enddo
  340          continue
            enddo

            if (ipr1.gt.2) then
               print*,' There are',nma,' possible sets of solutions:' 
               do mfr=1,natf(ifr)
                  i=frag(ifr,mfr)
                  print 44,i,(corr(n,i),n=1,nma)
               enddo
            endif
            mma=0
            do nki=1,nma
               do mfr=1,natf(ifr)
                  i=frag(ifr,mfr)
                  if (corr(nki,i).eq.0) then
                     if (ipr1.gt.3)
     +                    print*,'Solution', nki,' is unacceptable'
                     goto 356
                  endif
               enddo
               mma=mma+1
               do j=1,natf(ifr)
                  i=frag(ifr,j)
                  corr(mma,i)=corr(nki,i)
               enddo
  356          continue
            enddo
            nma=mma
            if (nma.eq.0) then
               print*, ' ***** No acceptable solutions in IDENT'
               goto 500
            endif
            if (ipr1.gt.2)
     +           print*,' There are',nma,' possible solutions left:'

C           Make final correspondence index
            if (ipr1.gt.2) then
               print*,'  Correspondence table(s) for fragment', ifr
               do mfr=1,natf(ifr)
                  i=frag(ifr,mfr)
                  print 44,i,(corr(n,i),n=1,nma)
               enddo
            endif
            nki=1
  130       continue
            if (nma.gt.1) then
               if (nkies(jngr,ifr).lt.1) then
                  print 52,nma,ig
                  write(16,52) nma,ig
c                  print*,(icn(i),i=1,10)
                  if (icn(ig).eq.0) then
                     print*,'Type the desired number:'
                     read*,nki
                  else
                     nki=icn(ig)
                  endif
                  print*,'Chosen:',nki
                  write(16,*) 'Chosen:',nki
                  if (nki.lt.1 .or. nki.gt.nma) goto 130
C                 Remove next 3 lines if the order differs between groups
                  do i=1,ngr
                     nkies(i,ifr)=nki
                  enddo
               else
                  nki=nkies(jngr,ifr)
               endif
            endif

            do mfr=1,natf(ifr)
               i=frag(ifr,mfr)
               j=nrp*(jngr-1)+i
               index(j)=corr(nki,i)
               if (j.gt.maxat) then
                  print*,' ***** Increase MAXAT'
                  goto 500
               endif
               if (index(j).eq.0) then
                  print*,' ***** Error 4 in YIDENT',
     +                 mfr,i,j,jngr,ifr,ngr,nfrag,inoh
                  goto 500
               endif
            enddo
         enddo
      enddo

      if (inoh.eq.0) goto 400
C     Searching atoms bound to H atoms
      if (ipr1.ge.0) then
         print*,'  Searching atoms bound to H atoms'
         write(16,*)'  Searching atoms bound to H atoms'
      endif
      do i=1,nfound
         ilink(i)=0
         if (naam1(i)(1:1).eq.'H') then
            do j=1,nconn
               m=0
               if (at1(j).eq.i) m=at2(j)
               if (at2(j).eq.i) m=at1(j)
               if (m.gt.0) then
                  if (ipr1.gt.1) print 36,i,naam1(i),m,naam1(m)
                  ilink(i)=m
               endif
            enddo
         endif
      enddo

C     Introducing these H atoms in the topology
      do n1=1,ngr
         do n2=1,nrp
            i=(n1-1)*nrp+n2
            k=index(i)
c            print*,i,panm(i),k,naam1(k)
            if (k.eq.0) then
               if (panm(i)(1:1).ne.'H') then
                  print*,' ***** Error 3 in YIDENT',k,naam1(k)
                  goto 500
               endif
C              Search connection of H atom in topology
               do j=1,nconnt
                  m=0
                  if (iatt1(j).eq.n2) m=iatt2(j)
                  if (iatt2(j).eq.n2) m=iatt1(j)
                  if (m.gt.0) then
                     n=(n1-1)*nrp+m
                     do l=1,nfound
                        if (ilink(l).eq.index(n)) then
                           index(i)=l
                           ilink(l)=0
                           if (ipr1.gt.1) print 37,i,panm(i),
     +                          n,panm(m),
     +                          index(n),naam1(index(n)),l,naam1(l)
                           goto 370
                        endif
                     enddo
                     if (ipr1.gt.0) print 38,i,panm(i),n,panm(m),
     +                    index(n),naam1(index(n))
  370                continue
                  endif
               enddo
            endif
         enddo
      enddo

  400 continue
      if (ipr1.gt.2) then
         print*,' Final correspondence index'
         do i=1,ngr*nrp
            print 39,i,panm(i),index(i),naam1(index(i))
         enddo
      endif
c      if (ipr1.gt.0) print*,'End IDENT.'
      print*,'IDENT found: nfound, nmol, ngr =',nfound,nmol,ngr
      write(16,*) 'IDENT found: nfound, nmol, ngr =',nfound,nmol,ngr
      return

  500 continue
      print*,'IDENT failed: nfound, nmol, ngr =',nfound,nmol,ngr
      write(16,*) 'IDENT failed: nfound, nmol, ngr =',nfound,nmol,ngr
      ngr=0
      return
c      stop

   30 format(4x,'Searching for molecules corresponding to ',
     +     'topology fragment',i4,2x,/,a)
   31 format(8x,'Analyzing molecule',i4,2x,a)
   32 format(8x,'Molecule',i4,
     +     ' may correspond to this fragment',i4)
   34 format(i4,' complete topology set(s) tentatively identified:')
   35 format(/,'Analyzing fragment',i4,' with molecule',i4)
   36 format(i4,2x,a,'is close to',i4,2x,a)
   37 format('H atom',i4,2x,a,'linked to',i4,2x,a,
     +     '   - searching',i4,2x,a,
     +     '   - finding',i4,2x,a)
   38 format('H atom',i4,2x,a,'linked to',i4,2x,a,3x,
     +     '- searching',i4,2x,a,' but finding nothing!')
   39 format(i4,2x,a,i8,2x,a)
   40 format(a5,5x,20a5)
   41 format(a,40i3)
   42 format(' Topolopy atom',i4,2x,a4,
     +     ' with coordinate atom(s)',10(i4,2x,a4))
   43 format(/,' Trying a second search, without H atoms:')
   44 format(i6,3x,20i4)
   45 format(i5,4x,a4,40i3)
   46 format(4i5,4x,20i4)
   47 format(' An assignment for topology atom',i4,2x,a4,
     +     ' can be',i4,2x,a4)
   51 format(' Searching correspondences for topology atom',
     +     i4,2x,a4,' assigned to',i4,2x,a4)
   52 format('***** Note',i3,
     +     ' possible correspondence sets for molecule',i3,'!')
   60 format (i5,2x,a5,' connected to',i4,' shells',30i4)
 61   format(' Fragment',i4,' contains',i4,' atoms:')
 62   format(200i4)
      end
