      subroutine intcar(amas,akar,grad,iwerk,
     +     nneg,sref,akar1,grad1)
C     Transforms first and second energy derivatives (Cartesian coordinates) 
C        to internal coordinates. 
C     Removes translational and rotational parts and returns cleaned matrices.
C     B. P. van Eijck, January 2000.

C     On first call, a file IN.INT is read. This should contain:
C        NAIB, number of bonds
C              For each bond a line with atom numbers: IAIB,JAIB
C        NAIH, number of angles
C              For each angle a line with atom numbers: IAIH,JAIH,KAIH
C        NAID, number of dihedrals
C              For each dihedral a line with atom numbers: IAID,JAID,KAID,LAID

C     Input through parameters:
C     AMAS(1..NRPT): atomic masses as read from GAMESS output.
C        This ensures comparable vibrational frequencies.
C        These masses probably differ from AMASS in CNONB.I, which may refer
C        to isotopic mixtures. For the transformation procedure,
C        each choice for AMAS (e.g. all set to 1) should be all right.
C     AKAR(1..3*NRPT,1..3*NRPT) Matrix of second derivatives (H)
C     GRAD(1..3*NRPT) Vector of first derivatives (G)
C     IWERK: regulates activities:
C        IWERK=0: Only preliminary work: read file IN.INT if not done earlier.
C
C        IWERK=1: Input AKAR and GRAD from GAMESS.
C                    Is Cartesian, but needs to be cleaned
C                    because it is not invariant for rotation (and sometimes 
C                    not even for translation, because of roundoff errors).
C                 Output in internal coordinates:
C                    AKAR1 contains A(T).H.A = H'(where A = B^-1)
C                    GRAD1 contains A(T).G = G'
C        IWERK=2: Input AKAR (H') and GRAD (G') from previous call
C                 of INTCAR with IWERK=1.
C                 Output prepared for GAMES:
C                    AKAR1 contains B(T).H'
C                    GRAD1 contains B(T).G'
C        IWERK=3: As IWERK=2, but also calculation of vibrational frequencies:
C                 Constructs AKAR1.B = B(T).A(T).H.A.B
C                 Finds eigenvalues of this matrix
C     Input through common block /PARS/:
C        NGR: number of molgroups
C        NRP: number of atoms in one molecule
C        X(1..): coordinates
C     Input through common block /AIDT/:
C        NMOLS: the number of discrete molecules

C     Output through parameters:
C     NNEG (only if IWERK=3): number of unexpected frequencies < 10 cm-1
C     SREF(1..3*NRPT): internal coordinates at geometry X
C     AKAR1(1..3*NRPT,1..3*NRPT) Transformed matrix of second derivatives
C     GRAD1(1..3*NRPT) Transformed vector of first derivatives

      implicit none
      include 'params.i'
      include 'caidt.i'
      include 'cpars.i'
      include 'cbond.i'

      integer i,j,k,m,n,nrpt,nr3,ios,nait,nm,i1,i2,m1,m2,inew,
     +     ierr,iwerk,nneg,nspec
      double precision c,bmat,xcm,tmass,trmat,xtr,tmu,scr,ama,amat,hulp,
     +     ew,scr1,scr2,scr3,pi,clight,amas,akar,sref,grad,akar1,grad1
      dimension amas(*),grad(*),akar(maxpar,maxpar),
     +     trmat(3,3),tmu(3,3),xtr(3),xcm(3),scr(3,3),
     +     bmat(maxpar,maxpar),amat(maxpar,maxpar),hulp(maxpar,maxpar),
     +     akar1(maxpar,maxpar),grad1(maxpar),sref(maxpar),
     +     ew(maxpar),scr1(maxpar),scr2(maxpar),scr3(2,maxpar)
      
      data inew /0/
      save

      nrpt=nrp*ngr
      nr3=3*nrpt
c      print*, 'Starting INTCAR with',nrpt,' atoms'
      if (inew.eq.0) then
C        On first call, read information on internal coordinates
ccc         inew=1
         naib=0
         naih=0
         naid=0
         open(unit=21,file='in.int',status='OLD',iostat=ios)
         if (ios.ne.0) then
            if (inew.eq.0) then
               print*,' ***** INTCAR: No file IN.INT present'
               write(16,*) ' ***** INTCAR: No file IN.INT present'
            endif
            inew=1
            return

C           Previous version:
ccc         print*,' ***** No file IN.INT present'
ccc         write(16,*) ' ***** No file IN.INT present'
ccc         stop
C           All parameters are a distance 0!!!!
ccc         naib=3*nrpt-6*nmols
ccc         naih=0
ccc         naid=0
ccc         do k=1,naib
ccc            iaib(k)=1
ccc            jaib(k)=1
ccc         enddo
         else
            read(21,*,end=900,err=900) naib
            do k=1,naib
               read(21,*,end=900,err=900) iaib(k),jaib(k)
            enddo
            read(21,*,end=900,err=900) naih
            do k=1,naih
               read(21,*,end=900,err=900) iaih(k),jaih(k),kaih(k)
            enddo
            read(21,*,end=900,err=900) naid
            do k=1,naid
               read(21,*,end=900,err=900) 
     +              iaid(k),jaid(k),kaid(k),laid(k)
            enddo
            print 30,naib,naih,naid
            write(16,30) naib,naih,naid

C           Check for artificial improper dihedrals
            naimpr=0
            read(21,*,end=200,err=200) naimpr
            if (naimpr.gt.0) then
               print*,' Reading',naimpr,' artificial improper dihedrals'
               do m=nimpr+1,nimpr+naimpr
                  if (m.gt.maximp) then
                     print*,'***** Too many artificial dihedrals',naimpr
                     stop
                  endif
                  read(21,*,end=910,err=910) iq(m),jq(m),kq(m),lq(m)
                  print*,iq(m),jq(m),kq(m),lq(m)
                  cq(m)=100
               enddo
            endif            
  200       continue
            close(21)
         endif

         nait=naib+naih+naid
         print 42,nait,nrpt
         write(16,42) nait,nrpt
         m=3*nrpt-6*nmols
         i2=0
         nspec=1000
         do i=1,nmols
            nm=imols(i+1)-imols(i)
C?????            if (nm.eq.2) i2=i2+1
            if (nm.eq.2) STOP 'Diatomic molecule not implemented'
            if (nm.gt.2) i2=i2+3*nm-6
            if (nm.lt.nspec) nspec=nm
            print*,imols(i),imols(i+1),nait,m,i2
         enddo
c         ngr=m/nait
c         if (nait*ngr.ne.m) then
         if (nait*ngr.ne.i2) then
            print*,' ***** Error in number of ',
     +           'internal coordinates',nait,m,i2,nmols
            stop
         endif
         if (ngr*(nmols/ngr).ne.nmols) then
            print*,' ***** Error in numbers of groups or atoms',
     +           nait,m,nmols
            stop
         endif
         print 43,ngr,nmols/ngr
         write(16,43) ngr,nmols/ngr
         if (nspec.lt.3) then
            print*,' Small molecule found with',nspec,' atoms'
            print*,' B-matrix following publication'
            write(16,*) ' B-matrix following publication'
            write(16,*) ' Small molecule found with',nspec,' atoms'
         else
            print*,' Mass-dependent B-matrix for vibrations'
            write(16,*)' Mass-dependent B-matrix for vibrations'
         endif

C        Enable use of additional improper dihedrals
         do m=1,ngr
            j=(m-1)*nrpt/ngr
            do k=1,naid
C  Dit is eventjes onbegrijpelijk.
c               iiprop(1,niprop+(m-1)*naid+k)=iaid(k)+j
c               iiprop(2,niprop+(m-1)*naid+k)=jaid(k)+j
c               iiprop(3,niprop+(m-1)*naid+k)=kaid(k)+j
c               iiprop(4,niprop+(m-1)*naid+k)=laid(k)+j
c               kprop(niprop+(m-1)*naid+k)=0
c               vprop(niprop+(m-1)*naid+k)=0
C  Waarschijnlijk als volgt:
c               iq(nimpr+(m-1)*naid+k)=iaid(k)+j
c               jq(nimpr+(m-1)*naid+k)=jaid(k)+j
c               kq(nimpr+(m-1)*naid+k)=kaid(k)+j
c               lq(nimpr+(m-1)*naid+k)=laid(k)+j
c               cq(nimpr+(m-1)*naid+k)=100
c               q0(nimpr+(m-1)*naid+k)=0
ccc               nimpr=nimpr+1
            enddo
         enddo
ccc         print*,nimpr,' possible improper dihedrals in INTCAR'

C        Expand for more than one molecule
         do m=2,ngr
            j=(m-1)*nrpt/ngr
            do k=1,naib
               iaib((m-1)*naib+k)=iaib(k)+j
               jaib((m-1)*naib+k)=jaib(k)+j
            enddo
            do k=1,naih
               iaih((m-1)*naih+k)=iaih(k)+j
               jaih((m-1)*naih+k)=jaih(k)+j
               kaih((m-1)*naih+k)=kaih(k)+j
            enddo
            do k=1,naid
               iaid((m-1)*naid+k)=iaid(k)+j
               jaid((m-1)*naid+k)=jaid(k)+j
               kaid((m-1)*naid+k)=kaid(k)+j
               laid((m-1)*naid+k)=laid(k)+j
            enddo
         enddo
         naib=ngr*naib
         naih=ngr*naih
         naid=ngr*naid
      endif

      do i=1,nr3
         sref(i)=0
         do m=1,nr3
            bmat(m,i)=0
         enddo
      enddo
C     Create the intramolecular part of the B-matrix
C     In the publication the intramolecular terms come last, here first.

      k=0
      do i=1,naib
         k=k+1
         call bbond(k,iaib(i),jaib(i),x,bmat,sref)
      enddo
      do i=1,naih
         k=k+1
         call bhoek(k,iaih(i),jaih(i),kaih(i),x,bmat,sref)
      enddo
      do i=1,naid
         k=k+1
         call bdiha(k,iaid(i),jaid(i),kaid(i),laid(i),x,bmat,sref)
      enddo
      if (inew.eq.0) then
         inew=1
         if (sref(1).gt.0.001d0) then
            print 74,(sref(i),i=1,naib)
            print 75,(sref(i),i=naib+1,naib+naih)
            print 76,(sref(i),i=naib+naih+1,naib+naih+naid)
            write(16,74) (sref(i),i=1,naib)
            write(16,75) (sref(i),i=naib+1,naib+naih)
            write(16,76) (sref(i),i=naib+naih+1,naib+naih+naid)
         endif
         return
      endif

      if (iwerk.gt.0) then
C        Create the part of the B-matrix for translation and rotation
C        of the separate molecules. 
         do nm=1,nmols
            i1=imols(nm)
            i2=imols(nm+1)-1
c            print*,'molecule',nm,i1,i2

C           Calculate distances to center of mass
            do m=1,3
               xcm(m)=0
            enddo
            tmass=0
            do i=i1,i2
               tmass=tmass+amas(i)
               do m=1,3
                  xcm(m)=xcm(m)+x(3*(i-1)+m)*amas(i)
               enddo
            enddo
            do m=1,3
               xcm(m)=xcm(m)/tmass
               do n=1,3
                  trmat(m,n)=0
               enddo
            enddo

C           Construct translational and rotational parts of the B matrix
C           If molecules with < 3 atoms are present, follow the publication.
            if (nspec.lt.3) then
cccc           k=nr3-nm*6
c              print*,'Molecule',nm,' from',i1,i2,'   end from:',k
               do i=i1,i2
                  j=3*i-3
                  do m=1,3
                     xtr(m)=x(j+m)-xcm(m)
                  enddo
                  ama=amas(i)
                  bmat(k+1,j+1)=1
                  bmat(k+2,j+2)=1
                  bmat(k+3,j+3)=1
                  m=3
                  if (i2-i1.gt.1) then
                     bmat(k+4,j+1)=0
                     bmat(k+4,j+2)=-xtr(3)
                     bmat(k+4,j+3)= xtr(2)
                     bmat(k+5,j+1)= xtr(3)
                     bmat(k+5,j+2)=0
                     bmat(k+5,j+3)=-xtr(1)
                     bmat(k+6,j+1)=-xtr(2)
                     bmat(k+6,j+2)= xtr(1)
                     bmat(k+6,j+3)=0
                     m=6
                  endif
               enddo
               k=k+m
               goto 250
            endif

C           Otherwise, use the following mass-dependent algorithm which is
C           only chosen to produce correct vibrational frequencies.
            do i=i1,i2
c               print*,'atom',i,amas(i),(x(3*i-3+m),m=1,3)
               c=0
               do m=1,3
                  xtr(m)=x(3*(i-1)+m)-xcm(m)
                  c=c+xtr(m)**2*amas(i)
               enddo

C              Calculate the inertial matrix TRMAT
               do m=1,3  
                  trmat(m,m)=trmat(m,m)+c
                  do n=1,3
                     trmat(m,n)=trmat(m,n)-xtr(m)*xtr(n)*amas(i)
                  enddo
               enddo
            enddo

c            print*,i1,i2,tmass,amas(i1)
c            do m=1,3
c               print*,xcm(m),m,(trmat(m,n),n=1,3)
c            enddo

C           Invert the inertial matrix TRMAT to MU
            call matinv(trmat,tmu,3,3,scr)
C           Check that the product is the unit matrix
            do m1=1,3
               do m2=1,3
                  c=0
                  if (m1.eq.m2) c=-1
                  do i=1,3
                     c=c+trmat(m1,i)*tmu(i,m2)
                  enddo
                  if (abs(c).gt.1.0d-15) print*,'TRINV-ERROR',m1,m2,c
               enddo
            enddo

C           Construct translational and rotational parts of the B matrix
ccc            k=nr3-nm*6
c            print*,'Molecule',nm,' from',i1,i2,'   end from:',k
            do i=i1,i2
               j=3*i-3
               do m=1,3
                  xtr(m)=x(j+m)-xcm(m)
               enddo
               ama=amas(i)
               bmat(k+1,j+1)=ama/tmass
               bmat(k+2,j+2)=ama/tmass
               bmat(k+3,j+3)=ama/tmass
               bmat(k+4,j+1)= ama*(tmu(1,2)*xtr(3)-tmu(1,3)*xtr(2))
               bmat(k+4,j+2)= ama*(tmu(1,3)*xtr(1)-tmu(1,1)*xtr(3))
               bmat(k+4,j+3)= ama*(tmu(1,1)*xtr(2)-tmu(1,2)*xtr(1))
               bmat(k+5,j+1)= ama*(tmu(2,2)*xtr(3)-tmu(2,3)*xtr(2))
               bmat(k+5,j+2)= ama*(tmu(2,3)*xtr(1)-tmu(2,1)*xtr(3))
               bmat(k+5,j+3)= ama*(tmu(2,1)*xtr(2)-tmu(2,2)*xtr(1))
               bmat(k+6,j+1)= ama*(tmu(3,2)*xtr(3)-tmu(3,3)*xtr(2))
               bmat(k+6,j+2)= ama*(tmu(3,3)*xtr(1)-tmu(3,1)*xtr(3))
               bmat(k+6,j+3)= ama*(tmu(3,1)*xtr(2)-tmu(3,2)*xtr(1))
            enddo
            k=k+6
            
  250       continue
         enddo
c         print*,k,'coordinates found'
      endif

c      do i=1,nr3
c         print*,'BMAT row',i
c         print 41,(bmat(i,j),j=1,nr3)
c      enddo
c         do i=1,nr3
c            print*,'AKAR  ',i,'    GRAD ',grad(i)
c            print*,(akar(i,j),j=1,nr3)
c         enddo
c         pause

      if (iwerk.eq.1) then
C        Calculate A = B^(-1)
C        print*,' Inverting the B-matrix'
         call matinv(bmat,amat,nr3,maxpar,hulp)
         c=1
         do i=1,nr3
            do j=1,nr3
               if (abs(amat(i,j)).gt.c) then
                  c=abs(amat(i,j))
                  print*,'LARGE AMAT ',i,j,c
               endif
            enddo
         enddo
         print*,'LARGEST AMAT ',c
         write(16,*)'LARGEST AMAT ',c

C        Check that B.A is the unit matrix
         do m1=1,6
            do m2=1,6
               c=0
               if (m1.eq.m2) c=-1
               do i=1,nr3
                  c=c+bmat(m1,i)*amat(i,m2)
               enddo
               if (abs(c).gt.1.0d-12) print*,'BA-ERROR',m1,m2,c
            enddo
         enddo

C        Construct matrix AKAR1 = A(T).H.A = H'

c      do i=1,nr3
c         print*,'AMAT row',i
c         print 41,(amat(i,j),j=1,nr3)
c      enddo
c      pause


         do i=1,nr3
            do j=1,nr3
               hulp(i,j)=0
               do m=1,nr3
                  hulp(i,j)=hulp(i,j)+akar(i,m)*amat(m,j)
               enddo
            enddo
         enddo
         do i=1,nr3
            do j=1,nr3
               akar1(i,j)=0
               do m=1,nr3
                  akar1(i,j)=akar1(i,j)+amat(m,i)*hulp(m,j)
               enddo
            enddo
         enddo

C        Construct GRAD1 = A(T).G = G'
         do i=1,nr3
            grad1(i)=0
            do m=1,nr3
               grad1(i)=grad1(i)+amat(m,i)*grad(m)
            enddo
         enddo

c         do i=1,nr3
c            print*,'AKAR1  ',i,'    GRAD1 ',grad(i)
c            print*,(akar1(i,j),j=1,nr3)
c         enddo
c         pause

C        Remove unphysical entries from H' and G'
         do i=nr3-6*nmols+1,nr3
            grad1(i)=0
            do j=1,nr3
               akar1(i,j)=0
               akar1(j,i)=0
            enddo
         enddo


c         do i=1,nr3
c            print*,'Cleaned AKAR1  ',i,'    GRAD1 ',grad(i)
c            print*,(akar1(i,j),j=1,nr3)
c         enddo
c         pause

         return
      endif

      if (iwerk.gt.1) then
C        Now input AKAR = H' for reference structure
C        Construct AKAR1 = B(T).H'

c         do i=1,nr3
c            print*,'Input AKAR  ',i,'    GRAD ',grad(i)
c            print*,(akar(i,j),j=1,nr3)
c         enddo
c         pause

         do i=1,nr3
            do j=1,nr3
               akar1(i,j)=0
               do m=1,nr3
ccccc                  akar1(i,j)=akar1(i,j)+bmat(m,j)*akar(m,i)
                  akar1(i,j)=akar1(i,j)+bmat(m,i)*akar(m,j)
               enddo
            enddo
         enddo
C        Now input GRAD = G' for reference structure
C        Construct GRAD1 = B(T).G'
         do i=1,nr3
            grad1(i)=0
            do j=1,nr3
               grad1(i)=grad1(i)+bmat(j,i)*grad(j)
            enddo
         enddo

c         do i=1,nr3
c            print*,'AKAR1  ',i,'    GRAD1 ',grad1(i)
c            print*,(akar1(i,j),j=1,nr3)
c         enddo

         if (iwerk.eq.2) return

C        Calculate vibrational frequencies
C        Construct AKAR1.B = B(T).A(T).H.A.B matrix in AMAT
         do i=1,nr3
            do j=1,nr3
               amat(i,j)=0
               do m=1,nr3
ccccc                  amat(i,j)=amat(i,j)+bmat(m,i)*akar1(m,j)
                  amat(i,j)=amat(i,j)+akar1(i,m)*bmat(m,j)
               enddo
            enddo
         enddo
         do i=1,nrpt
            do m=1,3
               scr1(3*i-3+m)=amas(i)
            enddo
         enddo
         do i=1,nr3
            do j=1,nr3
               amat(i,j)=amat(i,j)/sqrt(scr1(i)*scr1(j))
               bmat(i,j)=0
            enddo
         enddo
         call ch(maxpar,nr3,amat,bmat,ew,0,scr,scr,scr1,scr2,scr3,ierr)
         pi=4*atan(1.0d0)
         Clight=0.0299792d0
         do i=1,nr3
            scr2(i)=sqrt(abs(ew(i)))/(2*pi*Clight)
            if (ew(i).lt.0) scr2(i)=-scr2(i)
         enddo
         print*,'Vibrational frequencies of the free molecule(s):'
         write(16,*) 'Vibrational frequencies of the free molecule(s):'
         if (nspec.lt.3) then
            print*,' ***** Frequencies unreliable: no mass-effects'
            write(16,*)' ***** Frequencies unreliable: no mass-effects'
         endif
         print 40,(scr2(i),i=1,nr3)
         write(16,40) (scr2(i),i=1,nr3)
         nneg=0
         do i=6*nmols+1,nr3
c         do i=7,nr3
            if (scr2(i).lt.10) nneg=nneg+1
         enddo
         print*,nneg,' frequencies below 10 cm-1'
         write(16,*) nneg,' frequencies below 10 cm-1'
      endif
      return

  900 continue
      print*,' ***** Error reading data from IN.INT'
      print*,naib,' bonds'
      print*,naih,' angles'
      print*,naid,' dihedrals'
      stop

  910 print*,' ***** Error trying to read artificial impropers'
      stop

   30 format(' Internal coordinate information found:',
     +     i4,' bonds    ',i4,' angles    ',i4,' dihedrals')
   40 format(9f12.3)
c   41 format(12f9.5)
   42 format(i4,' coordinates found on file IN.INT for',i4,' atoms')
   43 format(i4,' identical group(s) of',i4,' molecules assumed')
   74 format(' Bonds (A): ',50f7.3)
   75 format(' Angles   : ',50f7.1)
   76 format(' Dihedrals: ',50f7.1)
      end




      subroutine bbond(kpar,ibn,jbn,x,bmat,sref)
C     B. P. van Eijck, January 2000
C     Calculates bond length and derivatives
C     Is based on the GROMOS procedure COBOND by W. F. van Gunsteren.

C     Input through parameters:
C        KPAR: number of internal coordinate (bond)
C        IBN, JBN: atom numbers
C        X(1..): atom coordinates

C     Output through parameters:
C        BMAT(KPAR,..): derivatives
C        SREF(KPAR): value of the bond length (A)

      implicit none
      include 'params.i'

      integer kpar,ibn,jbn,i3,j3,m
      double precision x,bmat,sref,rij,rij1,rij2,xh
      dimension x(*),sref(*),bmat(maxpar,maxpar)
      dimension rij(3)

      i3=3*ibn-3
      j3=3*jbn-3
      rij2=0
      do m=1,3
         rij(m)=x(i3+m)-x(j3+m)
         rij2=rij2+rij(m)**2
      enddo
      rij1=sqrt(rij2)
      sref(kpar)=10*rij1
      do m=1,3
         xh=rij(m)/rij1
         bmat(kpar,i3+m)= xh
         bmat(kpar,j3+m)=-xh
      enddo
      end




      subroutine bhoek(kpar,itn,jtn,ktn,x,bmat,sref)
C     B. P. van Eijck, January 2000
C     Calculates angle and derivatives
C     Is based on the GROMOS procedure ANGLE by W. F. van Gunsteren.

C     Input through parameters:
C        KPAR: number of internal coordinate (angle)
C        ITN, JTN, KTN: atom numbers
C        X(1..): atom coordinates

C     Output through parameters:
C        BMAT(KPAR,..): derivatives
C        SREF(KPAR): value of the angle (rad)

      implicit none
      include 'params.i'
      integer kpar,itn,jtn,ktn,k3,m,i3,j3
      double precision x,bmat,sref,cosa,dti,dtk,rij,rij1,rij2,
     +     rijk,rkj,rkj1,rkj2,vi,vi2,vk,vk2,at
      dimension x(*),sref(*),bmat(maxpar,maxpar)
      dimension rij(3),rkj(3),vi(3),vk(3),dti(3),dtk(3)

      i3=3*itn-3
      j3=3*jtn-3
      k3=3*ktn-3
      rij2=0
      rkj2=0
      rijk=0
      do m=1,3
         rij(m)=x(i3+m)-x(j3+m)
         rkj(m)=x(k3+m)-x(j3+m)
         rij2=rij2+rij(m)**2
         rkj2=rkj2+rkj(m)**2
         rijk=rijk+rij(m)*rkj(m)
      enddo
      rij1=sqrt(rij2)
      rkj1=sqrt(rkj2)
      cosa=rijk/(rij1*rkj1)
      at=acos(cosa)
      sref(kpar)=at*45/atan(1.0d0)

      vi2=0
      vk2=0
      do m=1,3
         vi(m)=rkj(m)/rkj1-cosa*rij(m)/rij1
         vk(m)=rij(m)/rij1-cosa*rkj(m)/rkj1
         vi2=vi2+vi(m)**2
         vk2=vk2+vk(m)**2
      enddo
      do m=1,3
         dti(m)=-vi(m)/sqrt(rij2*vi2)
         dtk(m)=-vk(m)/sqrt(rkj2*vk2)
      enddo
      do m=1,3
         bmat(kpar,i3+m)= dti(m)
         bmat(kpar,j3+m)=-dti(m)-dtk(m)
         bmat(kpar,k3+m)= dtk(m)
      enddo
      end




      subroutine bdiha(kpar,ipn,jpn,kpn,lpn,x,bmat,sref)
C     B. P. van Eijck,  January 2000
C     Calculates dihedral angle and derivatives
C     Is based on the GROMOS procedure DIHANG by W. F. van Gunsteren.

C     Input through parameters:
C        KPAR: number of internal coordinate (dihedral)
C        IPN, JPN, KPN, LPN: atom numbers
C        X(1..): atom coordinates

C     Output through parameters:
C        BMAT(KPAR,..): derivatives
C        SREF(KPAR): value of the dihedral (rad)

C     Calls subroutine PROD

      implicit none
      include 'params.i'
      double precision aj,ak,ap,bmat,cosa,dpi,dpj,dpk,dpl,
     +     rij,rijk,rjkl,rkj,rkj1,rkj2,rkl,sina,sj,sj1,sj2,
     +     sjsk,sk,sk1,sk2,sref,x
      integer i3,ipn,j3,jpn,k3,kpar,kpn,l3,lpn,m
      dimension x(*),sref(*),bmat(maxpar,maxpar)
      dimension rij(3),rkj(3),rkl(3),
     +     sj(3),sk(3),sjsk(3),dpi(3),dpj(3),dpk(3),dpl(3)

      i3=3*ipn-3
      j3=3*jpn-3
      k3=3*kpn-3
      l3=3*lpn-3

      do m=1,3
         rij(m)=x(i3+m)-x(j3+m)
         rkj(m)=x(k3+m)-x(j3+m)
         rkl(m)=x(k3+m)-x(l3+m)
      enddo

      call prod(rij,rkj,sj)
      call prod(rkj,rkl,sk)
      call prod(sj,sk,sjsk)
      sj2=0
      sk2=0
      rkj2=0
      cosa=0
      sina=0
      do m=1,3
         sj2=sj2+sj(m)**2
         sk2=sk2+sk(m)**2
         rkj2=rkj2+rkj(m)**2
         cosa=cosa+sj(m)*sk(m)
         sina=sina+rkj(m)*sjsk(m)
      enddo
      if (sj2.lt.1.0d-12) then
         print*,' Division by',sj2,' in CDIHA'
         print 60,ipn,jpn,jpn,kpn
         print*,(x(i3+m),m=1,3)
         print*,(x(j3+m),m=1,3)
         print*,(x(k3+m),m=1,3)
         print*,(x(l3+m),m=1,3)
         print*,(sj(m),m=1,3)
         print*,(sk(m),m=1,3)
         print*,(sjsk(m),m=1,3)
         stop
      endif
      if (sk2.lt.1.0d-12) then
         print*,' Division by',sk2,' in CDIHA'
         print 60,jpn,kpn,kpn,lpn
         print*,(x(i3+m),m=1,3)
         print*,(x(j3+m),m=1,3)
         print*,(x(k3+m),m=1,3)
         print*,(x(l3+m),m=1,3)
         print*,(sj(m),m=1,3)
         print*,(sk(m),m=1,3)
         print*,(sjsk(m),m=1,3)
         stop
      endif
      sj1=sqrt(sj2)
      sk1=sqrt(sk2)
      rkj1=sqrt(rkj2)
      cosa=cosa/(sj1*sk1)
      sina=sina/(sj1*sk1*rkj1)
      ap=atan2(sina,cosa)
      sref(kpar)=ap*45/atan(1.0d0)

      rijk=0
      rjkl=0
      do m=1,3
         rijk=rijk+rij(m)*rkj(m)
         rjkl=rjkl+rkj(m)*rkl(m)
      enddo
      aj=1/(sj2*rkj1)
      ak=1/(sk2*rkj1)
      do m=1,3
         dpi(m)= aj*sj(m)*rkj2
         dpj(m)=-aj*sj(m)*(rkj2-rijk) + ak*sk(m)*rjkl
         dpk(m)=-aj*sj(m)*rijk + ak*sk(m)*(rkj2-rjkl)
         dpl(m)=-ak*sk(m)*rkj2
      enddo
      do m=1,3
         bmat(kpar,i3+m)=dpi(m)
         bmat(kpar,j3+m)=dpj(m)
         bmat(kpar,k3+m)=dpk(m)
         bmat(kpar,l3+m)=dpl(m)
      enddo
   60 format(' ***** Parallel bonds ',i4,' --',i4,' and',i4,' --',i4)
      end



      subroutine matinv(a,b,n,nmax,scr)
C     Calculates b=a(inv)
      implicit none
      double precision a,b,scr,det
      integer i,n,nmax,j
      dimension a(nmax,nmax),b(nmax,nmax),scr(nmax,nmax)
      integer indx(1000)
      do i=1,n
         do j=1,n
            scr(i,j)=a(i,j)
            b(i,j)=0
         enddo
         b(i,i)=1
      enddo
      call ludcmp(scr,n,nmax,indx,det)
      do j=1,n
         det=det*scr(j,j)
         call lubksb(scr,n,nmax,indx,b(1,j))
      enddo
      end
      
      subroutine lubksb(A,N,NP,INDX,B)
      implicit none
      double precision a,b,sum
      integer i,ii,n,np,j,ll,indx
      DIMENSION A(NP,NP),INDX(N),B(N)
      II=0
      DO 12 I=1,N
        LL=INDX(I)
        SUM=B(LL)
        B(LL)=B(I)
        IF (II.NE.0)THEN
          DO 11 J=II,I-1
            SUM=SUM-A(I,J)*B(J)
11        CONTINUE
        ELSE IF (SUM.NE.0.) THEN
          II=I
        ENDIF
        B(I)=SUM
12    CONTINUE
      DO 14 I=N,1,-1
        SUM=B(I)
        IF(I.LT.N)THEN
          DO 13 J=I+1,N
            SUM=SUM-A(I,J)*B(J)
13        CONTINUE
        ENDIF
        B(I)=SUM/A(I,I)
14    CONTINUE
      RETURN
      END
      
      subroutine ludcmp(A,N,NP,INDX,D)
      implicit none
      integer n,np,nmax,indx,i,imax,j,k
      double precision tiny,a,d,aamax,dum,sum,vv
      PARAMETER (NMAX=1000,TINY=1.0d-20)
      DIMENSION A(NP,NP),INDX(N),VV(NMAX)
      character*8 regel
   10 format(a)      
      
      D=1
      DO 12 I=1,N
        AAMAX=0
        DO 11 J=1,N
          IF (ABS(A(I,J)).GT.AAMAX) AAMAX=ABS(A(I,J))
11      CONTINUE
        IF (AAMAX.EQ.0.) then
           print*,' *****  Singular matrix'
           print*,' Type ENTER if you wish to continue anyhow'
           read 10, regel
        endif
        VV(I)=1/AAMAX
12    CONTINUE
      DO 19 J=1,N
        IF (J.GT.1) THEN
          DO 14 I=1,J-1
            SUM=A(I,J)
            IF (I.GT.1)THEN
              DO 13 K=1,I-1
                SUM=SUM-A(I,K)*A(K,J)
13            CONTINUE
              A(I,J)=SUM
            ENDIF
14        CONTINUE
        ENDIF
        AAMAX=0
        DO 16 I=J,N
          SUM=A(I,J)
          IF (J.GT.1)THEN
            DO 15 K=1,J-1
              SUM=SUM-A(I,K)*A(K,J)
15          CONTINUE
            A(I,J)=SUM
          ENDIF
          DUM=VV(I)*ABS(SUM)
          IF (DUM.GE.AAMAX) THEN
            IMAX=I
            AAMAX=DUM
          ENDIF
16      CONTINUE
        IF (J.NE.IMAX)THEN
          DO 17 K=1,N
            DUM=A(IMAX,K)
            A(IMAX,K)=A(J,K)
            A(J,K)=DUM
17        CONTINUE
          D=-D
          VV(IMAX)=VV(J)
        ENDIF
        INDX(J)=IMAX
        IF(J.NE.N)THEN
          IF(A(J,J).EQ.0.)A(J,J)=TINY
          DUM=1/A(J,J)
          DO 18 I=J+1,N
            A(I,J)=A(I,J)*DUM
18        CONTINUE
        ENDIF
19    CONTINUE
      IF(A(N,N).EQ.0.)A(N,N)=TINY
      RETURN
      END
