      subroutine intre(ngr,nrp,box,x,iprint,
     +     enbi,eeli,eatti,erepi,epoli)

C     Version of NONBE for intramolecular energies only. No derivatives.
C     B.P. van Eijck, November 1999.
C     NOTE: Polarization is not yet well tested!!!


C     Can include polarization. References:
C     B.T. Thole, Chem. Phys. 59 (1981) 341-350.
C     D.N. Bernardo, Y. Ding, K. Krogh-Jespersen and R.M. Levy,
C          J. Phys. Chem. 98 (1994) 4180-4187

C     Input through parameters:
C     NGR: number of groups
C     NRP: number of atoms
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     X(1..NGR*NRP): atom coordinates.
C     IPRINT regulates output to unit 16 and to screen.

C     Input through common blocks:
C     /NONP/ gives information on nonbonded parameters:
C        IDO(1..NRP) (0 or 1) disables/enables atoms
C        IAC(1..NRP): atom codes, taking values 1..Ncod
C        IEXCL(1..NRP,1..NRP): excluded neighbors
C        C6(1..Ncod,1..Ncod): attractive C6 parameters
C        CS6(1..Ncod,1..Ncod): same for Lennard-Jones 1..4 interactions
C        C12(1..Ncod,1..Ncod),CS12(1..Ncod,1..Ncod):
C           for Lennard-Jones potential: same for repulsive terms
C           for Buckingham potential: repulsion C12 * exp(-CS12 * R)
C        SC14C, SC14LJ: multiply Lennard-Jones 1..4 Coulomb and other
C           terms with these factors, respectively.
C     /NONB/ gives various information:
C        CG(1..NRP): charges
C        EBIG: if any repulsion term exceeds EBIG, stop immediately and
C           set ENBI=90000.
C        BUCK: if TRUE, then choose Buckingham potential
C        DDC: if TRUE, use distance-dependent dielectric constant R (A).
C        TOLE: omit atom pairs which contribute less than TOLE to
C              any of the main three energy contributions (disabled!!)
C     /POLA/ gives polarization data:
C        NPP=0: no polarization
C        NPP=1: intermolecular polarization, so no contribution here
C        NPP>1: call full polarization calculation,
C               with a maximum number of NPP iteration cycles
C        ALFPOL(1..): polarizabilities (in low-field limit)
C        PMAX(1..): maximum dipole moment (not used by Bernardo et al)
C        SCHAAL: scaling distance for damping in T-tensor (Thole: 1.662)
C        SIJ(1..,1..): SIJ(i,j) = {(alfpol(i)*alfpol(j)}^(1/6)
C        POLTOL: stop iteration if energy changes less than POLTOL.
C        GAMPOL: damping coefficient in zero field equation (Bernardo: 0.85)
C        NNN: damping exponent in zero field equation (Bernardo: 3)
C           NNN = 0: no such damping.
C        MMM: damping exponent in T-tensor (Bernardo: 32)
C           MMM = 0: no such damping.
C        NTE = 1: include 1-2 and 1-3 neighbors in electric field
C        NTTT = 1: include 1-2 and 1-3 neighbors in T-tensor

C     Output through parameters:
C     ENBI: the nonbonded energy
C     Special cases:
C        ENBI = 70000: a cell axis is less than 0.5 Angstrom.
C        ENBI = 90000: a repulsion term exceeds EBIG
C     EELI: the coulomb energy
C     EATTI: the attractive dispersion term
C     EREPI: the repulsive term
C     EPOLI: the polarization energy.

C     Output through common blocks:
C     /POLA/: polarization output if NPP>1:
C        DIP(1..): induced dipole moments
C        NCYCP: number of iteration to make Epoli converge

      implicit none
      include 'params.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cpola.i'

      integer i,iaci,iacj,ii,ii3,iim,intra,iprint,j,jj,jj3,jjm,m,
     +     n,ngr,nr3,nrp,nrpt
      double precision box,c12def,c6def,cbu,cgi,cgij,ch12,ch6,
     +     denuli,denulj,eatti,ebu,eeli,enb1,enbi,enul,eold,
     +     epoli,erepi,euit,f1,f23,f3,rij1,rij2,rij3,rij6,
     +     rr,rs,vda,vdr,vel,wij,x,xij,xx,yij,yy,zij
      dimension box(*),x(*)
      logical skip
      dimension Euit(maxxco),enul(maxxco)

      nrpt=nrp*ngr
      nr3=3*nrpt
      if (iprint.gt.1) then
         print 40, ngr,nrp,npp
         if (buck) print*,'   Buckingham potential used'
         if (ddc) print*,
     +        '   Distance-dependent dielectric constant used'
         print 90,(box(m),m=1,6)
      endif
      do m=1,3
         if (box(m).lt.0.05d0) then
            print*,' ***** INTRE: Cel axes less than 0.5 Angstrom.'
            write(16,*)
     +           ' ***** INTRE: Cel axes less than 0.5 Angstrom.'
            enbi=70000
            return
         endif
      enddo
c      ax=box(1)
c      by=box(2)
c      cz=box(3)
c      bx=box(4)
c      cx=box(5)
c      cy=box(6)
      enbi=0
      eeli=0
      eatti=0
      erepi=0
      Epoli=0
      do i=1,nr3
         enul(i)=0
      enddo

      do ii=1,nrpt
C        Start the next atom II
         iim=mod(ii-1,nrp)+1
         if (ido(ii).eq.0) goto 390
         CGI=CG(II)
         IACI=IAC(II)
         II3=3*ii-3
         do jj=ii+1,(1+(ii-1)/nrp)*nrp
C           Start the next atom JJ within the same molecule
            jjm=mod(jj-1,nrp)+1
            if (ido(jj).eq.0) goto 250
            jj3=3*jj-3
            iacj=iac(jj)
            c6def=c6(iaci,iacj)
            c12def=c12(iaci,iacj)
            cgij=cgi*cg(jj)
            if (buck) then
               cbu=c12def
               ebu=cs12(iaci,iacj)
               c12def=0
            endif
            ch6=c6def
            ch12=c12def
            xij=x(ii3+1)-x(jj3+1)
            yij=x(ii3+2)-x(jj3+2)
            zij=x(ii3+3)-x(jj3+3)

            intra=iexcl(iim,jjm)
            skip=(intra.le.1)
            if (skip) goto 250
C           Skip interaction for excluded neighbors and intermolecular pairs
            if (intra.eq.2) then
C              Change and/or scale 1..4 interaction parameters
               cgij=cgij*sc14c
               if (.not. buck) then
                  ch12=cs12(iaci,iacj)*sc14lj
                  ch6=cs6(iaci,iacj)*sc14lj
               endif
            endif
            rij2=xij**2+yij**2+zij**2
            if (rij2.lt.1.0e-6) goto 250
            rij1=sqrt(rij2)
            if (iprint.gt.1 .and. rij2.lt.0.003) then
               print*,'***** INTRE: short distance: ',ii,jj,rij1
            endif

C           Calculate Coulomb energy
C           No distance dependent dielectric constant if DDC=TRUE.
            vel=cgij/rij1
            if (ddc) vel=vel/rij1

C           Calculate attractive r^(-6) terms
            rij6=rij2*rij2*rij2
            vda=ch6/rij6

C           Calculate R^(-12) or exponential repulsion
            if (buck) then
               vdr=cbu*exp(-ebu*rij1)
            else
               vdr=ch12/(rij6*rij6)
            endif
C           Throw away contributions smaller than TOLE
            if (abs(vel).lt.tole .and. abs(vda).lt.tole .and.
     +           abs(vdr).lt.tole) goto 250
            eeli=eeli+vel
            eatti=eatti-vda
            erepi=erepi+vdr
            if (erepi.gt.ebig) then
               enbi=90000
               return
            endif
            if (iprint.gt.6) then
               if (buck) then
                  print 88,ii,jj,xij,yij,zij,rij1,
     +                 cgij,ch6,cbu,ebu,vel,-vda,vdr
               else
                  print 89,ii,jj,xij,yij,zij,rij1,
     +                 cgij,ch6,ch12,vel,-vda,vdr
               endif
            endif
            if (npp.lt.2) goto 250

C           Calculate electric field for polarization calculation
C           Use damping if NNN>0.
            rs=rij1/(sij(iim,jjm)*schaal)
            if (nnn.gt.0) then
               f1=1-exp(-gampol*(rs**nnn))
            else
               f1=1
            endif
            rij3=rij1*rij2
            if (skip .and. nte.eq.0) goto 2300
            denuli=cg(jj)*f1/rij3
            denulj=cg(ii)*f1/rij3
            enul(ii3+1)=enul(ii3+1)+xij*denuli
            enul(ii3+2)=enul(ii3+2)+yij*denuli
            enul(ii3+3)=enul(ii3+3)+zij*denuli
            enul(jj3+1)=enul(jj3+1)-xij*denulj
            enul(jj3+2)=enul(jj3+2)-yij*denulj
            enul(jj3+3)=enul(jj3+3)-zij*denulj
 2300       continue

C           Calculate T matrix, with damping if MMM>0.
            if (skip .and. nttt.eq.0) goto 250
            if (mmm.gt.0 .and. rs.lt.1) then
               wij=rs+(1-rs**mmm)/mmm
               f23=3*(wij**4)
               f3=4*(wij**3)-f23
            else
               f23=3
               f3=1
            endif

C           Make pair list for polarization
            npla=npla+1
            ipla1(npla)=ii3
            if (skip) ipla1(npla)=-ii3-1
C           Negative IPLA1 markes an atom pair to be skipped
            if (npla.gt.maxap) then
               print*, ' ***** INTRE: Increase maxap',npla
               stop
            endif
            ipla2(npla)=jj3
            T11(npla)=((f23*xij*xij/rij2)-f3)/rij3
            T22(npla)=((f23*yij*yij/rij2)-f3)/rij3
            T33(npla)=((f23*zij*zij/rij2)-f3)/rij3
            T12(npla)=(f23*xij*yij/rij2)/rij3
            T13(npla)=(f23*xij*zij/rij2)/rij3
            T23(npla)=(f23*yij*zij/rij2)/rij3
  250       continue
         enddo
c  300    continue
  390    continue
      enddo

      if (npp.lt.2) goto 660

C     Start calculation of polarisation
      do i=1,nr3
         j=(i-1)/3+1
         xx=abs(Enul(i)*alfpol(j)/pmax(j))
         if (xx.lt.0.01) then
            dip(i)=alfpol(j)*Enul(i)*(1-xx/2)
         else
            dip(i)=alfpol(j)*Enul(i)*(1-exp(-xx))/xx
         endif
         Epoli=Epoli-dip(i)*Enul(i)/2
C        This is an extension of the standard equation:
C               dip(i)=alfpol(j)*Enul(i)
C        for using a maximum allowed dipole moment PMAX.
      enddo
c      print*,'Enul'
c      print*,(enul(i),i=1,nr3)
c      print*,'Dip'
c      print*,(dip(i),i=1,nr3)

      if (iprint.gt.1) then
         print*,' Results before polarization loop:'
         print*,' npla = ',npla
         enb1=enbi+eeli+eatti+erepi+Epoli
         print 96, enb1,eeli,eatti,erepi,Epoli
      endif

      ncycp=0
C     Next cycle in polarization loop
  620 continue
      ncycp=ncycp+1
      do i=1,nr3
         Euit(i)=Enul(i)
      enddo
      Eold=Epoli
      Epoli=0
      do n=1,npla
         i=ipla1(n)
         skip=(i.lt.0)
         if (skip .and. nttt.eq.0) goto 640
         if (skip) i=-(i+1)
         j=ipla2(n)
c         is1=1
c         is2=1
c         is3=1
         Euit(i+1)=Euit(i+1)+(T11(n)*dip(j+1)
     +        +T12(n)*dip(j+2)+T13(n)*dip(j+3))
         Euit(i+2)=Euit(i+2)+(T12(n)*dip(j+1)
     +        +T22(n)*dip(j+2)+T23(n)*dip(j+3))
         Euit(i+3)=Euit(i+3)+(T13(n)*dip(j+1)
     +        +T23(n)*dip(j+2)+T33(n)*dip(j+3))
         if (i.ne.j) then
            Euit(j+1)=Euit(j+1)+(T11(n)*dip(i+1)
     +           +T12(n)*dip(i+2)+T13(n)*dip(i+3))
            Euit(j+2)=Euit(j+2)+(T12(n)*dip(i+1)
     +           +T22(n)*dip(i+2)+T23(n)*dip(i+3))
            Euit(j+3)=Euit(j+3)+(T13(n)*dip(i+1)
     +           +T23(n)*dip(i+2)+T33(n)*dip(i+3))
         endif
  640    continue
      enddo
      do i=1,nr3
C     Nonlinear equations with PMAX:
         j=(i-1)/3+1
         xx=abs(Euit(i)*alfpol(j)/pmax(j))
         if (xx.lt.0.01) then
            yy=-xx/12
            dip(i)=alfpol(j)*Euit(i)*(1-xx/2)
         else
            rr=(1-exp(-xx))/xx
            yy=0.5d0+(rr-1)/(rr*xx)
            dip(i)=alfpol(j)*Euit(i)*rr
         endif
C        Iterate first, then update dipole moments.
         Epoli=Epoli+dip(i)*(-Enul(i)/2+Euit(i)*yy)
C        This is an extension of the standard equations:
C            Epoli=Epoli-dip(i)*Enul(i)/2
C            dip(i)=alfpol((i-1)/3+1)*Euit(i)
C        when using a maximum allowed dipole moment PMAX.
      enddo
c      print*,' epol = ',Epoli,eold
      if (ncycp.lt.npp .and. abs(Eold-Epoli).gt.poltol) goto 620
c      print*,' Dipoles NONBEP:'
c      do 670 i=1,5
c         print*,i,enul(i),dip(i)
c 670  continue
      if (iprint.gt.1) print*,' Results after polarization loop:'
 
  660 continue

      enbi=enbi+eeli+eatti+erepi+Epoli
      if (iprint.gt.1) print 96, enbi,eeli,eatti,erepi,Epoli

   40 format(' Start INTRE: ngr, nrp, npp',4i5)
   88 format(2i4,2x,4f8.3,3x,f8.3,2e13.4,f8.3,4x,8f8.3)
   89 format(2i4,2x,4f8.3,3x,f8.3,2e13.4,12x,8f8.3)
   90 format(' box: ',6f10.4)
   96 format(' Energy INTRE: ',6f9.3)
      end
