      subroutine introt(i3,j3,k3,x,a,b)

C     Sets up data for internal rotation of atom I about the axis K-J.

C     Input through parameters:
C        I3, J3, K3: the coordinates of these atoms are found in X(I3+M),
C            X(J3+M), X(K3+M) with M=1,2,3.
C        X: array of atom coordinates

C     Output through parameters:
C        A(1..3), B(1..3): if atom I is to be rotated over an angle OMEGA,
C        its coordinates are displaced over SHIFT(M) (M=1..3 for x,y,z):
C        SHIFT(M) = B(M)*sin(OMEGA) + A(M)*(cos(OMEGA)-1).
C        These coordinates are found in X(I3+M).

      implicit none
      include 'params.i'

      dimension x(*),a(*),b(*)
      double precision a,b,r,s,x,lambda(3),roh(3)
      integer i3,j3,k3,m

      r=0
      do m=1,3
         lambda(m)=x(j3+m)-x(k3+m)
         roh(m)=x(i3+m)-x(j3+m)
         r=r+lambda(m)**2
      enddo
      r=sqrt(r)
      s=0
      do m=1,3
         lambda(m)=lambda(m)/r
         s=s+lambda(m)*roh(m)
      enddo
C     LAMBDA is the unit vector from K to J, ROH is the vector from J to I.
      b(1)=lambda(2)*roh(3)-lambda(3)*roh(2)
      b(2)=lambda(3)*roh(1)-lambda(1)*roh(3)
      b(3)=lambda(1)*roh(2)-lambda(2)*roh(1)
      do m=1,3
         a(m)=roh(m)-s*lambda(m)
      enddo
      end
