      subroutine latvib(nq,flist,ntemp,temp,box,ivamp,ihess,
     +     einter,nmols,iprint,ifiles,ispgr,
     +     nneg,nsum,sumcv,sumev,sumsv,sumsc,sumuqm,sumucl,
     +     sumzpm,sumzpt,jjsh,n1sh,n2sh,n3sh,msh,sh)

C     Performs calculation of lattice vibrations
C     B. P. van Eijck, November 1999.

C     Output files:
C        fort.22: Data for plotting the vibrational spectrum
C        fort.23: Data for plotting the integrated vibrational spectrum

C     Input through parameters:
C     NQ: number of reciprocal lattice vectors
C     FLIST(0:NQ,3): the relevant vectors divided by 2 pi (range 0...1)
C     NTEMP: number of temperatures for thermodynamical quantities
C     TEMP(1..NTEMP): the relevant temperatures
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     IVAMP = 1: calculate vibrational amplitudes. This requires 
C                eigenvectors, and increases the computational effort
C     EINTER: lattice energy for estimate of Cv-Cp correction
C     NMOLS: the number of discrete molecules in the asymmetric unit
C     IPRINT regulates output to unit 16 and to screen.
C     IFILES = 1: output of spectral density to fort.22 and fort.23
C     ISPGR = 0 for a free molecule

C     Input through common block:
C     /PARS/:
C        NGR: number of molgroups
C        NRP: number of atoms in one molecule
C        X(1..3*NRP): coordinates
C     /NONB/:
C        AMASS(1..): atomic masses
C     /ATTP/
C        PANM(1..): atom names in topology file
C     /NBAL/ gives pair list information:
C        NPLA: number of pairs, including vectors to other cells
C        IPLA1(1...NPLA): number of atom (I) in a pair
C        IPLA2(1...NPLA): number of atom (J) in a pair
C        PHI(m1,m2,1..NPLA)=d2U/dr(m1)dr(m2)
C        RLIST(m,1..NPLA)= m-component of vector R(1...NPLA)
C        ISHIFT1..3(1...NPLA): fractional shifts
C     /SPGR/ gives space group information:
C        NSPGR, MSP, TSP: see subroutine LEESR

C     Output through parameters:
C     NNEG: number of vectors that generate at least one imaginary frequency
C     NSUM: the other ones, that behave decently
C     SUMCV(1..NTEMP): Cv
C     SUMEV(1..NTEMP): E (vibrational, excluding zero-point energy)
C     SUMSV(1..NTEMP): S (quantummechanical)
C     SUMSC(1..NTEMP): S (classical)
C     SUMUQM(1..NTEMP): Sum M(k)u(k)**2 (quantummechanical)
C     SUMUCL(1..NTEMP): Sum M(k)u(k)**2 (classical)
C     SUMZPM is the zero-point energy, including intramolecular vibrations
C     SUMZPT is the zero-point energy, only from intermolecular vibrations

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cnonb.i'
c      include 'cnbal.i'
      include 'cpars.i'
      include 'cspgr.i'

      integer i,i1,i2,id,idens,ierr,ifiles,ifirst,ihess,ii,ios,
     +     iprint,ispgr,itemp,ivamp,j,j1,j2,jj,jjsh,k,k1,k2,
     +     listq,listu,m,m1,m2,maxden,method,mj,moltot,msh,
     +     n1,n1sh,n2,n2sh,n3,n3sh,ngrtot,nmols,nneg,npla,nq,
     +     nr3,nr3tot,nrpt,nrptot,nsum,ntemp
      double precision a,acl,ai,aqm,ar,astx,asty,astz,ax,
     +     box,bsty,bstz,bx,by,c,clight,co,cpvib,cstz,cvib,cx,cy,cz,
     +     d,dens,dim,dre,dsigma,e,eb,edum,eh,ei,einter,enb,ep,evi,
     +     evib,evr,ew,f1,f2,f3,fact,factor,flist,flr,fm1,freq,
     +     fv1,fv2,fvib,hessian,hnukt,p,phi,pi,planck,qr,qvec,
     +     rgas,rlist,rmsder,s,s1,s2,sclas,scr,sh,si,st,step,
     +     sumcv,sumev,sumsc,sumsv,sumucl,sumuqm,sumzpm,sumzpt,svib,
     +     temp,tol,tol1,tvib,uamp,ucl,uqm,vol,w,xij,yij,
     +     zepmol,zeptot,zij,zpe

C     Common block with pair list data. Note difference with CNBAL.I
      common /nbal/phi(3,3,mxlist),
     +     ishift1(mxlist),ishift2(mxlist),ishift3(mxlist),
     +     isig1(mxlist),isig2(mxlist),isig3(mxlist),
     +     ipla1(mxlist),ipla2(mxlist),npla
      integer*2 ishift1,ishift2,ishift3,isig1,isig2,isig3,ipla1,ipla2
C     Scratch arrays for nonbe, nonbep and secder
      common /scrt/ flr(3*maxcag),ing(3*maxnsj)
      integer*2 ing,regel

      parameter(maxden=10000)
      dimension box(*),sumsc(maxtmp),sumucl(maxtmp),sumuqm(maxtmp),
     +     temp(maxtmp),sumcv(maxtmp),sumev(maxtmp),sumsv(maxtmp)
      dimension hessian(3*maxmat,3*maxmat),
     +     dre(3*maxmat,3*maxmat),dim(3*maxmat,3*maxmat),
     +     evr(3*maxmat,3*maxmat),evi(3*maxmat,3*maxmat),
     +     ew(3*maxmat),fv1(3*maxmat),fv2(3*maxmat),fm1(2,3*maxmat),
     +     uamp(maxmat,12,maxtmp),idens(maxden+500),flist(0:maxq,3),
     +     qvec(3),listu(2,6),rlist(3,mxlist),scr(3,3)

      nrpt=nrp*ngr
      nr3=3*nrpt
      nr3tot=nr3*(nspgr+1)
      nrptot=nrpt*(nspgr+1)
      ngrtot=ngr*(nspgr+1)
      moltot=nmols*(nspgr+1)
      if (nrptot.gt.maxmat) then
         print*,'***** Increase MAXMAT',nrptot
         stop
      endif
      do i=1,nr3tot
         do j=1,nr3tot
            hessian(i,j)=0
         enddo
      enddo
      dsigma=0.5d0
C     This is the bin width (cm-1) for the density of states.
      dsigma=5
C     This is the bin width used in PROACF.
C     The spectrum is smoother but the results are the same.
      listu(1,1)=1
      listu(2,1)=1
      listu(1,2)=1
      listu(2,2)=2
      listu(1,3)=2
      listu(2,3)=2
      listu(1,4)=1
      listu(2,4)=3
      listu(1,5)=2
      listu(2,5)=3
      listu(1,6)=3
      listu(2,6)=3
      pi=4*atan(1.0d0)
      Rgas=0.00831451d0
      Clight=0.0299792d0
      Planck=0.39903d0
      factor=Planck/(4*pi**2)

      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)

      if (jjsh.gt.0) then
C        Test LATVIB
         print*,' Checking second derivatives with shift',sh
         print*,'    For jj, m, n1, n2, n3 =',jjsh,msh,n1sh,n2sh,n3sh
         call secder(ngr,nrp,enb,box,x,fv1,hessian,rlist,0,
     +        jjsh,n1sh,n2sh,n3sh,msh,sh)
         call secder(ngr,nrp,enb,box,x,fv2,hessian,rlist,iprint,
     +        0,0,0,0,0,0.0d0)
c         do i=1,nr3*(nspgr+1)
c            print*,i,fv1(i),fv2(i),(fv2(i)-fv1(i))/sh
c         enddo
         do i=1,npla
            j=ipla1(i)
            ii=ipla1(i)/3+1
            jj=ipla2(i)/3+1
c            print*,i,ii,jj,ishift1(i),ishift2(i),ishift3(i),phi(1,1,i)
            if (jj.eq.jjsh .and. ishift1(i).eq.n1sh .and.
     +           ishift2(i).eq.n2sh .and. ishift3(i).eq.n3sh) then
               do m=1,3
                  qvec(m)=(fv2(j+m)-fv1(j+m))/sh
               enddo
               print 20,i,ii,jj,(phi(m,msh,i),m=1,3),(qvec(m),m=1,3),
     +              (phi(m,msh,i)-qvec(m),m=1,3)
            endif
         enddo
      endif

      tol=0.001d0
      if (ihess.eq.1) then
C        Read hessian from file prepared in this calculation
         print*,'Try reading ab-initio Hessian as prepared in this job'
         open(unit=46,form='UNFORMATTED',status='OLD',iostat=ios)
         if (ios.eq.0) then
            tol=20
            do m=1,nr3
               read(46) i
               read(46) fv1(i)
               read(46) (hessian(i,j),j=1,nr3)
            enddo
            close(46)
            goto 130
         else
            print*,' ***** Error reading ab-initio Hessian!'
         endif
c  120    continue
      endif
      if (ihess.eq.2) then
C        Read hessian from file prepared in a precious calculation
         print*,'Try reading ab-initio Hessian as previously prepared'
         open(unit=47,status='OLD',iostat=ios)
         if (ios.eq.0) then
            tol=20
            do m=1,nr3
               read(47,*) i
               read(47,*) fv1(i)
               read(47,7777) (hessian(i,j),j=1,nr3)
 7777          format(15f9.0)
            enddo
            close(47)
            goto 130
         else
            print*,' ***** Error reading external Hessian file!'
         endif
      endif

C     Calculate hessian for bonds, angles, impropers and torsions
      step=5.0d-9
      do i=1,nr3
         fv1(i)=0
      enddo
      call cbond(ngr,nrp,x,fv1,1,eb,0)
      call choek(ngr,nrp,x,fv1,1,eh,0)
      call cdiha(ngr,nrp,x,fv1,1,1,ei,0)
      call cdiha(ngr,nrp,x,fv1,1,2,ep,0)
      do i=1,nr3
         x(i)=x(i)+step
         do j=1,nr3
            fv2(j)=0
         enddo
         call cbond(ngr,nrp,x,fv2,1,edum,0)
         call choek(ngr,nrp,x,fv2,1,edum,0)
         call cdiha(ngr,nrp,x,fv2,1,1,edum,0)
         call cdiha(ngr,nrp,x,fv2,1,2,edum,0)
         do j=1,nr3
            hessian(i,j)=(-fv2(j)+fv1(j))/step
C           Simulate rigid molecule. Problem: librations get also large 
C           frequencies, caused by deformation in rectangular coordinates?
ccc            hessian(i,j)=1000000.0*hessian(i,j)
         enddo
         x(i)=x(i)-step
      enddo
  130 continue
      tol1=max(0.1d0,tol)

C     Expand hessian according to space group symmetry.
C     Enforce sum rule and symmetry (only cosmetic, is redone later)
      do m1=1,3
         do m2=1,3
            do k1=1,nrpt
               if (iprint.gt.6) print 25,(hessian(3*k1-3+m1,3*k2-3+m2),
     +              k2=1,nrptot)
               s=0
               do k2=1,nrpt
                  if (k2.ne.k1) s=s+hessian(3*k1-3+m1,3*k2-3+m2)
               enddo
               d=hessian(3*k1-3+m1,3*k1-3+m2)+s
               if (iprint.gt.3 .and. abs(d).gt.tol) 
     +              print 31,m1,m2,k1,s,d
               hessian(3*k1-3+m1,3*k1-3+m2)=-s
               do k2=1,nrpt
                  do k=1,nspgr
                     hessian(k*nr3+3*k1-3+m1,k*nr3+3*k2-3+m2) =
     +                    hessian(3*k1-3+m1,3*k2-3+m2)
     +                    *msp(3*k-3+m1)*msp(3*k-3+m2)
                  enddo
               enddo
            enddo
         enddo
      enddo
      do i=1,nr3tot
         do j=1,i-1
            s1=hessian(i,j)
            s2=hessian(j,i)
            if (iprint.gt.3 .and. abs(s1-s2).gt.tol1) 
     +           print 31,i,j,0,s1,s1-s2
            s=(s1+s2)/2
            hessian(i,j)=s
            hessian(j,i)=s
         enddo
      enddo
C     Print hessian matrix
      if (iprint.gt.5) then
         print*,' Hessian for bonds, angles, impropers and torsions'
         do i=1,nr3tot
            print 27,i
            print 25,(hessian(i,j),j=1,nr3tot)
         enddo
      endif

C     Calculate second derivatives for nonbonded interactions
      call secder(ngr,nrp,enb,box,x,ew,hessian,rlist,iprint,
     +     0,0,0,0,0,0.0d0)
      if (iprint.gt.0) print*,npla,' terms needed from SECDER'
      if (npla.gt.mxlist) then
         print*,' ***** Nonbonded interactions through file'
         open(unit=49,form='UNFORMATTED',status='OLD',iostat=ios)
         if (ios.ne.0) then
            print*,' ***** No interaction data file'
            stop
         endif
      endif

C     Print hessian matrix
      if (iprint.gt.5) then
         print*,' Complete hessian'
         do i=1,nr3tot
            print 27,i
            print 25,(hessian(i,j),j=1,nr3tot)
         enddo
      endif

      rmsder=0
      do i=1,nr3
         fv2(i)=ew(i)+fv1(i)
         if (iprint.gt.1) print 33,i,fv1(i),ew(i),fv2(i)
         rmsder=rmsder+fv2(i)**2
      enddo
      if (iprint.gt.0) then
         print 34,sqrt(rmsder/nr3)
         write(16,34) sqrt(rmsder/nr3)
         print*,' Calculation of second derivatives finished.'
         write(16,*) ' Calculation of second derivatives finished.'
      endif

C     HESSIAN contains constant contributions
C     RLIST(m,1..NPLA)= m-component of vector R(1...NPLA)
      if (enb.gt.69000) then
         print*,' ***** SECDER delivers nonbonded energy',enb
         print*,' Type ENTER if you wish to continue anyhow'
         read 10, regel
      endif

      method=1
C     METHOD=1: equations from Willis and Prior, containing r(lk)
C          This must be used if Ewald summaton will be implemented.
C     METHOD=2: equations from Eckold, containing only r(l)

C     Calculate reciprocal lattice components in orthogonal X,Y,Z system
      if (method.eq.1) then
         if (iprint.gt.0) then
            print*,' Using method of Willis/Prior'
            write(16,*)' Using method of Willis/Prior'
         endif
         vol=ax*by*cz
         astx=1/ax
         asty=-bx*cz/vol
         astz=(bx*cy-by*cx)/vol
         bsty=1/by
         bstz=-cy*ax/vol
         cstz=1/cz
      else
         if (iprint.gt.0) then
            print*,' Using method of Eckold'
            write(16,*) ' Using method of Eckold'
         endif
      endif

C     Set arrays to zero
      do itemp=1,ntemp
         sumev(itemp)=0
         sumsv(itemp)=0
         sumcv(itemp)=0
         sumsc(itemp)=0
         sumuqm(itemp)=0
         sumucl(itemp)=0
      enddo
      nsum=0
      nneg=0
      sumzpm=0
      sumzpt=0
      do i=1,nrptot
         do j=1,12
            do k=1,maxtmp
               uamp(i,j,k)=0
            enddo
         enddo
      enddo
      do i=1,maxden
         idens(i)=0
      enddo

C     Start loop over the q-vectors
      do listq=0,nq
         f1=2*pi*flist(listq,1)
         f2=2*pi*flist(listq,2)
         f3=2*pi*flist(listq,3)
         if (method.eq.1) then
            qvec(1)=f1*astx
            qvec(2)=f1*asty+f2*bsty
            qvec(3)=f1*astz+f2*bstz+f3*cstz
         endif
         if (iprint.gt.4) print 24,listq,(flist(listq,m),m=1,3)

C        Initialize dynamical matrix
         do i=1,nr3tot
            do j=1,nr3tot
               dre(i,j)=0
               dim(i,j)=0
            enddo
         enddo

C        Add the real and imaginary parts of direct terms.
C        Start with the terms already contained in the hessian:
         do k1=1,nrptot
            j1=3*k1-3
            do k2=1,nrptot
               j2=3*k2-3
               if (method.eq.1) then
                  qr=0
                  do m=1,3
                     xij=x(j2+m)-x(j1+m)
                     qr=qr+qvec(m)*xij
                  enddo
                  co=cos(qr)
                  si=sin(qr)
                  do m1=1,3
                     do m2=1,3
                        dre(j1+m1,j2+m2)=dre(j1+m1,j2+m2)+
     +                       hessian(j1+m1,j2+m2)*co
                        dim(j1+m1,j2+m2)=dim(j1+m1,j2+m2)+
     +                       hessian(j1+m1,j2+m2)*si
                     enddo
                  enddo
               else
                  do m1=1,3
                     do m2=1,3
                        dre(j1+m1,j2+m2)=dre(j1+m1,j2+m2)+
     +                       hessian(j1+m1,j2+m2)
                     enddo
                  enddo
               endif
            enddo
         enddo

C        Now add the terms from the list made in SECDER:
         
C        Transfer through common block
         do i=1,min(npla,mxlist)
            j1=ipla2(i)
            j2=ipla1(i)
            if (method.eq.1) then
               qr=0
               do m=1,3
                  qr=qr+qvec(m)*rlist(m,i)
               enddo
            else
               qr=f1*ishift1(i)+f2*ishift2(i)+f3*ishift3(i)
            endif
            co=cos(qr)
            si=sin(qr)
            do m1=1,3
               do m2=1,3
                  p=phi(m1,m2,i)
                  dre(j1+m1,j2+m2)=dre(j1+m1,j2+m2)+p*co
                  dim(j1+m1,j2+m2)=dim(j1+m1,j2+m2)+p*si
               enddo
            enddo
         enddo
         if (npla.le.mxlist) goto 220

C        Transfer trough unformatted file necessary (slow!)
         rewind(49)
  210    continue
         read(49,end=220) m1,j2,j1,n1,n2,n3,xij,yij,zij,scr
         if (method.eq.1) then
            qr=qvec(1)*xij+qvec(2)*yij+qvec(3)*zij
         else
            qr=f1*n1+f2*n2+f3*n3
         endif
         co=cos(qr)
         si=sin(qr)
         do m1=1,3
            do m2=1,3
               p=scr(m1,m2)
               dre(j1+m1,j2+m2)=dre(j1+m1,j2+m2)+p*co
               dim(j1+m1,j2+m2)=dim(j1+m1,j2+m2)+p*si
            enddo
         enddo
         goto 210

  220    continue
C        Add the real and imaginary parts of the reciprocal Ewald terms
C           (if and when successfully implemented ...)
c         call lwald(nrpt,f1,f2,f3,box,x,dre,dim)
  
C        Enforce Hermiticity and introduce mass-weighting
         do i=1,nrptot
            do m=1,3
               fv1(3*i-3+m)=amass(i)
            enddo
         enddo
         tol=0.1d0
         do i=1,nr3tot
            do j=1,i
               w=sqrt(fv1(i)*fv1(j))
C              Real part
               s1=dre(i,j)
               s2=dre(j,i)
               if (iprint.gt.4 .and. abs(s1-s2).gt.tol) 
     +              print 11,i,j,s1,s1-s2
               if (abs(s1-s2).gt.1) print 11,i,j,s1,s1-s2
               s=(s1+s2)/(2*w)
               dre(i,j)=s
               dre(j,i)=s
C              Imaginary part
               s1=dim(i,j)
               s2=dim(j,i)
               if (iprint.gt.4 .and. abs(s1+s2).gt.tol) 
     +              print 11,-i,-j,s1,s1+s2
               if (abs(s1+s2).gt.1) print 11,i,j,s1,s1+s2
               s=(s1-s2)/(2*w)
               dim(i,j)=s
               dim(j,i)=-s
            enddo
         enddo

C        Print dynamical matrix
         if (iprint.gt.6) then
            print*,' Real part of dynamical matrix'
            do i=1,nr3tot
               print 27,i
               print 25,(dre(i,j),j=1,nr3tot)
            enddo
            print*,' Imaginary part of dynamical matrix'
            do i=1,nr3tot
               print 27,i
               print 25,(dim(i,j),j=1,nr3tot)
            enddo
            print*,' '
         endif

C        Diagonalize matrix. Eigenvectors only if ivamp=1.
         call ch(3*maxmat,nr3tot,dre,dim,ew,ivamp,evr,evi,
     +        fv1,fv2,fm1,ierr)
         if (iprint.gt.4) print 25,(ew(i),i=1,nr3tot)
         if (ivamp.gt.0 .and. iprint.gt.6) then
            do j=1,nr3tot
               print*,' Real part of eigenvector',j
               print 30,(evr(i,j),i=1,nr3tot)
               print*,' Imaginary part of  eigenvector',j
               print 30,(evi(i,j),i=1,nr3tot)
            enddo
            print*,' '
         endif

C        Calculate frequencies in various units:
C        FV1 is the frequency in THz, FV2 is the frequency in cm-1
         zepmol=0
         zeptot=0
         do j=1,nr3tot
            fv1(j)=sqrt(abs(ew(j)))/(2*pi)
            if (ew(j).lt.0) fv1(j)=-fv1(j)
            fv2(j)=fv1(j)/Clight
            zeptot=zeptot+Planck*fv1(j)/2
            if (j.le.6*moltot) zepmol=zepmol+Planck*fv1(j)/2
         enddo

C        Write frequencies and check for negative frequencies
         if (ispgr.eq.0) then
C           Calculation for a free molecule.
C           There should be six zero frequencies.
            ifirst=7
            print 23,(fv2(j),j=1,6)
            print 29
            print 25,(fv2(j),j=7,nr3tot)
            write(16,23) (fv2(j),j=1,6)
            write(16,29)
            write(16,29) (fv2(j),j=7,nr3tot)
            if (fv2(7).lt.0.1d0) then 
               print*,' Negative frequency!'
               write(16,*) ' Negative frequency!'
               nneg=1
               return
            else
               nsum=1
            endif
         else
            if (listq.eq.0) then
C              The q=0 vector. There should be three zero frequencies.
               ifirst=4
               if (iprint.gt.0) then
                  print*,' Calculating the special vector q = 0'
                  write(16,*) ' Calculating the special vector q = 0'
               endif
               if (iprint.gt.2) print 25,(fv2(j),j=1,nr3tot)
               if (fv2(4).lt.0.1d0) then
                  if (iprint.gt.0) then
                     print*,'Negative frequency for q = 0!'
                     write(16,*) 'Negative frequency for q = 0!'
                  endif
                  goto 720
               else
                  nsum=1
               endif
            else
C              All other q-vectors
               ifirst=1
               if (iprint.gt.2) print 28,listq,(flist(listq,m),m=1,3),
     +              fv2(1)
               if (iprint.gt.3) print 25,(fv2(j),j=1,nr3tot)
               if (fv2(1).lt.0.1d0) then
                  nneg=nneg+1
                  if (iprint.gt.0) print 21,listq,
     +                 (flist(listq,m),m=1,3),(fv2(m),m=1,3)
                  if (ivamp.gt.0) then
C              Try and find out what is happening
c               print*,' Real part of wavefunction:'
c               print 30,(evr(i,1),i=1,nr3tot)
c               print*,' Imaginary part of wavefunction:'
c               print 30,(evi(i,1),i=1,nr3tot)
                     fact=2
C              Moet eigenlijk 1 zijn voor q=0, dan is er geen imaginair deel.
                     print*,' Vibrational amplitudes from real part:'
                     print 30,(evr(i,1)*sqrt(fact/amass((i+2)/3)),
     +                    i=1,nr3tot)
                     print*,
     +                    ' Vibrational amplitudes from imaginary part:'
                     print 30,(evi(i,1)*sqrt(fact/amass((i+2)/3)),
     +                    i=1,nr3tot)
C              Nog nadenken over normalisering en lineaire afhankelijkheid.
                  endif
                  goto 720
               endif
               nsum=nsum+1
            endif
         endif

C        ZEPMOL is the zero-point energy, only from intermolecular vibrations
C        ZEPTOT is the zero-point energy, including intramolecular vibrations
         sumzpm=sumzpm+zepmol/ngrtot
         sumzpt=sumzpt+zeptot/ngrtot
         if (iprint.gt.4) then
            print 18,moltot,zepmol/ngrtot
            print 32,zeptot/ngrtot
         endif

C        Calculate thermodynamic functions for NTEMP values of T
         do itemp=1,ntemp
            Tvib=temp(itemp)
            evib=0
            svib=0
            fvib=0
            cvib=0
            sclas=0
            uqm=0
            ucl=0
            if (iprint.gt.3) print*,' Temperature',Tvib
            if (Tvib.gt.0.1d0) then
               do j=ifirst,nr3tot
                  hnukT=Planck*fv1(j)/(Rgas*Tvib)
                  e=exp(-hnukT)
                  a=Rgas*Tvib*hnukT*e/(1-e)
                  evib=evib+a
                  svib=svib+a/Tvib-Rgas*log(1-e)
                  fvib=fvib+Rgas*Tvib*log(1-e)
                  cvib=cvib+Rgas*e*(hnukT/(1-e))**2
                  sclas=sclas+Rgas*(1-log(hnukT))
                  uqm=uqm+(0.5d0+e/(1-e))/fv1(j)
                  ucl=ucl+1/(hnukT*fv1(j))
                  if (iprint.gt.3) print 27,j,fv2(j),e,a,
     +                 a/Tvib-Rgas*log(1-e),Rgas*Tvib*log(1-e)
               enddo
            else
               do j=ifirst,nr3tot
                  uqm=uqm+0.5d0/fv1(j)
               enddo
            endif
            sumev(itemp)=sumev(itemp)+evib/ngrtot
            sumsv(itemp)=sumsv(itemp)+svib/ngrtot
            sumsc(itemp)=sumsc(itemp)+sclas/ngrtot
            sumcv(itemp)=sumcv(itemp)+cvib/ngrtot
            sumuqm(itemp)=sumuqm(itemp)+uqm/ngrtot
            sumucl(itemp)=sumucl(itemp)+ucl/ngrtot
            if (iprint.gt.3) print 26,Tvib,cvib/ngrtot,
     +           evib/ngrtot,svib/ngrtot,sclas/ngrtot,fvib/ngrtot
         enddo

         if (ispgr.eq.0 .or. nq.eq.0) return
         if (listq.eq.0) then
C           Exclude the q=0 vector from the following calculations
            do itemp=1,ntemp
               sumev(itemp)=0
               sumsv(itemp)=0
               sumcv(itemp)=0
               sumsc(itemp)=0
               sumuqm(itemp)=0
               sumucl(itemp)=0
            enddo
            nsum=0
            nneg=0
            sumzpm=0
            sumzpt=0
            do i=1,nrptot
               do j=1,12
                  do k=1,maxtmp
                     uamp(i,j,k)=0
                  enddo
               enddo
            enddo
            do i=1,maxden
               idens(i)=0
            enddo
            goto 720
         endif

C        Calculate density of states, in bins of DSIGMA
         do i=1,nr3tot
            j=nint(fv2(i)/dsigma)
            if (j.gt.0 .and. j.lt.maxden) then
               idens(j)=idens(j)+1
            endif
            if (j.gt.maxden) print*,' ***** Increase MAXDEN'
         enddo

         if (ivamp.gt.0) then
C           The following calculations need the eigenvectors (EVR, EVI)
            if (iprint.gt.4) then
C              Check normalization
               do j1=1,nr3tot
                  do j2=1,nr3tot
                     ar=0
                     ai=0
                     do i=1,nr3tot
                        ar=ar+evr(i,j1)*evr(i,j2)+evi(i,j1)*evi(i,j2)
                        ai=ai+evr(i,j1)*evi(i,j2)-evi(i,j1)*evr(i,j2)
                     enddo
                     if (j1.eq.j2) ar=ar-1
                     if (abs(ar).gt.1.0d-14 .or. abs(ai).gt.1.0d-14)
     +                    print*,' Error in normalization:',j1,j2,ar,ai
                  enddo
               enddo
            endif

C           Calculate vibrational amplitudes (100 Angstrom**2)
            do itemp=1,ntemp
               Tvib=temp(itemp)
               do j=1,nr3tot
                  hnukT=Planck*fv1(j)/(Rgas*Tvib)
                  e=exp(-hnukT)
                  aqm=(0.5d0+e/(1-e))/fv1(j)
                  acl=1/(hnukT*fv1(j))
                  do k=1,nrptot
                     w=factor*10000/amass(k)
                     do m=1,6
                        i1=3*k-3+listu(1,m)
                        i2=3*k-3+listu(2,m)
                        c=evr(i1,j)*evr(i2,j)+evi(i1,j)*evi(i2,j)
                        uamp(k,m,itemp)=uamp(k,m,itemp)+w*c*aqm
                        uamp(k,m+6,itemp)=uamp(k,m+6,itemp)+w*c*acl
                     enddo
                  enddo
               enddo
            enddo
         endif
  720    continue
      enddo
C     End of loop over the q-vectors

C     Calculation for all q-vectors finished, print results.
      if (nsum.ge.1) then
         mj=0
         do j=1,maxden
            if (idens(j).gt.0) mj=j
         enddo
         if (iprint.gt.4) then
            print*,'      Frequency in:         Density:'
            print*,'     cm-1         THz'
         endif
         id=0
         s=0
         st=0
         do j=1,mj+400
            dens=0
            if (idens(j).gt.0) then
               id=1
               dens=idens(j)/(dsigma*dfloat(nsum*ngrtot))
               s=s+dens*dsigma
               st=st+dens*dsigma
               if (iprint.gt.4) print 15,j*dsigma,j*dsigma*Clight,dens
            elseif (id.eq.1) then
               id=0
               if (iprint.gt.5) print 13,s,st
               s=0
            endif
            if (ifiles.eq.1) then
ccc               write(22,*) j*dsigma,dens
ccc               write(23,*) j*dsigma,st
            endif
            if (iprint.gt.5) print 13,s,st
         enddo

C        Calculate thermodynamic functions directly
         if (iprint.gt.3) print 12,nsum
         do i=1,ntemp
            Tvib=temp(i)
            sumcv(i)=sumcv(i)/nsum
            sumev(i)=sumev(i)/nsum
            sumsv(i)=sumsv(i)/nsum
            sumsc(i)=sumsc(i)/nsum
            sumuqm(i)=sumuqm(i)*factor/nsum
            sumucl(i)=sumucl(i)*factor/nsum
            fvib=sumev(i)-Tvib*sumsv(i)
            cpvib=sumcv(i)-1.13d0*36*Rgas**2*Tvib/(Einter/ngr)
            if (iprint.gt.3) print 22,Tvib,sumcv(i),cpvib,
     +           sumev(i),sumsv(i),sumsc(i),fvib,sumuqm(i),sumucl(i)
         enddo
         sumzpt=sumzpt/nsum
         sumzpm=sumzpm/nsum
         if (iprint.gt.3) then 
            print 32,sumzpt
            print 18,moltot,sumzpm
         endif

C        Calculate thermodynamic functions from frequency distribution
         if (iprint.gt.3) then
            zpe=0
            do j=1,mj
               zpe=zpe+idens(j)*Planck*j*dsigma*Clight/2
            enddo
            zpe=zpe/(nsum*ngrtot)
            print 19
            do itemp=1,ntemp
               Tvib=temp(itemp)
               evib=0
               svib=0
               fvib=0
               cvib=0
               sclas=0
               uqm=0
               ucl=0
               if (Tvib.gt.0.1d0) then
                  do j=1,mj
                     freq=j*dsigma*Clight
                     hnukT=Planck*freq/(Rgas*Tvib)
                     id=idens(j)
                     e=exp(-hnukT)
                     a=id*Rgas*Tvib*hnukT*e/(1-e)
                     evib=evib+a
                     svib=svib+a/Tvib-id*Rgas*log(1-e)
                     cvib=cvib+id*Rgas*e*(hnukT/(1-e))**2
                     sclas=sclas+id*Rgas*(1-log(hnukT))
                     uqm=uqm+id*(0.5d0+e/(1-e))/freq
                     ucl=ucl+id/(hnukT*freq)
                  enddo
               else
                  do j=1,mj
                     freq=j*dsigma*Clight
                     uqm=uqm+idens(j)*0.5d0/freq
                  enddo
               endif
               cvib=cvib/(nsum*ngrtot)
               evib=evib/(nsum*ngrtot)
               svib=svib/(nsum*ngrtot)
               sclas=sclas/(nsum*ngrtot)
               fvib=evib-Tvib*svib
               cpvib=cvib-1.13d0*36*Rgas**2*Tvib/(Einter/ngr)
               uqm=uqm*factor/(nsum*ngrtot)
               ucl=ucl*factor/(nsum*ngrtot)
               print 22,Tvib,cvib,cpvib,evib,svib,sclas,fvib,uqm,ucl
            enddo
            print 32,zpe
         endif
         
         if (ivamp.gt.0) then
            print 14,nsum
            write(16,14) nsum
            do itemp=1,ntemp
               Tvib=temp(itemp)
               uqm=0
               ucl=0
               do k=1,nrptot
                  m=1
                  uqm=uqm+amass(k)*uamp(k,m,itemp)/10000
                  ucl=ucl+amass(k)*uamp(k,m+6,itemp)/10000
                  m=3
                  uqm=uqm+amass(k)*uamp(k,m,itemp)/10000
                  ucl=ucl+amass(k)*uamp(k,m+6,itemp)/10000
                  m=6
                  uqm=uqm+amass(k)*uamp(k,m,itemp)/10000
                  ucl=ucl+amass(k)*uamp(k,m+6,itemp)/10000
               enddo
               print 16,Tvib
               write(16,16) Tvib
               do k=1,nrptot
                  print 17,k,panm(k),(uamp(k,m,itemp)/nsum,m=1,12)
                  write(16,17) k,panm(k),(uamp(k,m,itemp)/nsum,m=1,12)
               enddo
               print*,' uqm, ucl ',uqm/(nsum*ngrtot),ucl/(nsum*ngrtot)
               write(16,*) ' uqm, ucl ',
     +              uqm/(nsum*ngrtot),ucl/(nsum*ngrtot)
            enddo
         endif
      endif

   10 format(a)
   11 format(' ***** Hermicity of dynamical matrix doubtful:',
     +     2i4,2e15.5)
   12 format(/,' Averages (from',i6,' q-vectors) for the contributions',
     +     ' of the lattice vibrations to:',/,7x,'T',8x,
     +     'Cv        Cp           E     S(qm)     S(cl)           F',
     +     6x,'u(qm)      u(cl)')
   13 format(12x,'Sum for this band:',f15.3,'       Total:',f8.3)
   14 format(/,' Vibrational amplitudes (100 Angstrom**2) in ',
     +     'orthogonal axes from',i6,' q-vectors:',/,40x,
     +     'Quantum Mechanical',38x,'Classical')
   15 format(f10.1,f12.3,f15.3)
   16 format(12x,'T =',f8.2,4x,
     +     'U11     U12     U22     U13     U23     U33',9x,
     +     'U11     U12     U22     U13     U23     U33')
   17 format(' Atom',i5,3x,a5,5x,6f8.3,4x,6f8.3)
   18 format(' Lattice zero-point energy based on',i4,
     +     ' molecules per cell:',f8.3)
   19 format(/,' Averages (via frequency distribution) for the',
     +     ' contributions of the lattice vibrations to:',/,7x,'T',8x,
     +     'Cv        Cp           E         S     S(cl)           F',
     +     6x,'u(qm)      u(cl)')
   20 format(i8,2i4,2(4x,3f10.2),4x,3f10.4)
   21 format(' ***** Negative or very small wave number(s):',i10,2x,
     +     3f8.4,5x,10f10.3)
   22 format(f8.2,2f10.5,f12.3,2f10.5,f12.3,2f11.4)
   23 format('   Translational and rotational frequencies:',6f10.3)
   24 format(/,' q-vector:',i10,2x,3f8.4)
   25 format(12f10.1)
   26 format(' Temperature',f12.2,'   Contributions to Cvib, ',
     +     'Evib, Svib, Sclas, Fvib:',f10.6,f12.3,2f10.6,3f12.3)
   27 format(i5,f10.3,f10.5,3x,6f10.5)
   28 format(' q-vector:',i10,2x,3f8.4,3x,'Lowest frequency',f10.3)
   29 format('   Vibrational frequencies and thermodynamical',
     +     ' quantities for the free molecule:')
   30 format(12f10.5)
   31 format(' ***** Intramolecular hessian precision doubtful:',
     +     3i4,2e15.5)
   32 format(' Total zero-point energy:',f10.3)
   33 format(i3,3f10.3)
   34 format('RMS force on atoms ',f10.2)
      end
