      subroutine leesg(iai,nm,i1,i2,iprint,
     +     xfound,amafnd,xarr,grad,akar,egtot,title)

C     Reads data from GAMESS-output on unit 25.

C     Input through parameters:
C        IAI=0: searches for energy only (no derivatives)
C           =1: searches for energy, 1st and 2nd derivatives
C        NM: thenumber of the molecule
C        I1, I2: numbers of first and last atoms in molecule NM
C        IPRINT regulates output to unit 16 and to screen.
C     Output through parameters:
C        XFOUND(1..): atomic coordinates (nm) as found from unit 25
C        XARR(1..): same as from calling program, to be checked
C        AMAFND(1..): atomic masses as found on unit 25
C        GRAD(1..) vector of first derivatives (atomic units)
C        AKAR(1..,1..) matrix of second derivatives (atomic units)
C            For AKAR and GRAD only the entries for the molecule under
C            investigation are filled.
C        EGTOT: the energy as found on unit 25 (atomic units)
C        TITLE: the basis set used; or an error message.

      implicit none
      include 'params.i'

      integer i,i1,i2,iai,ierr,iprint,j,k,kmax,kmin,m,nat,nco,nm
      double precision akar,ama,amafnd,basis,grad,scr,xarr,xfound,
     +     escf,emp2,egtot
      dimension xfound(maxxco),xarr(*),
     +     akar(maxpar,maxpar),grad(maxpar),amafnd(maxat),scr(maxpar)
      character regel*200,title*30,ch*1

C     Analyze GAMESS-output
      escf=0
      emp2=0
      egtot=0
      title=' '
      rewind(25)
  100 continue
      read (25,10,end=300,err=300) regel
c      print*,regel

C     Search for basis set (as given under TITLE)
      if (regel(1:25).eq.'          *    basis set') then
         do i=26,80
            if (regel(i:i).ne.' ') then
               title=regel(i:i+29)
               goto 105
            endif
         enddo
  105    continue
C        Change to upper case
         do i=1,30
            ch=title(i:i)
            k=ichar(ch)
            if (ch.ge.'a' .and. ch.le.'z') k=k-32
            title(i:i)=char(k)
         enddo
      endif

C     Search for SCF + MP2 energy
      if (regel(11:23).eq.'total energy') then
         read(regel,51) escf
      endif
      if (regel(11:32).eq.'mp2 correlation energy') then
         read(regel,51) emp2
         egtot=egtot+escf+emp2
         print 80,nm,i1,i2,egtot,basis
         write(16,80) nm,i1,i2,egtot,basis
         print*,'SCF and MP2 contributions: ',escf,emp2
         write(16,*) 'SCF and MP2 contributions: ',escf,emp2
         if (iai.eq.0) goto 250
      endif

C     Search for SCF energy
      if (regel(21:33).eq.'total energy') then
         read(regel,50) escf
         egtot=egtot+escf
         print 80,nm,i1,i2,escf
         write(16,80) nm,i1,i2,escf
         if (iai.eq.0) goto 250
      endif

C     Search for geometry, in order to check on the calculation
      if (regel(31:59).ne.'molecular geometry (angstrom)') 
     +     goto 100
C     Read molecular geometry and atomic masses
      do i=1,4
         read (25,10,end=300,err=300) regel
      enddo
      if (iprint.gt.3) 
     +     print*,' Masses and coordinates from GAMESS output'
  110 continue
      read (25,40) j,(scr(m),m=1,3),ama
      if (j.ne.0) then
         do m=1,3
            xfound(3*(i1+j)-6+m)=scr(m)/10
         enddo
         amafnd(i1-1+j)=ama
         if (iprint.gt.3) print 41,j,ama,(scr(m),m=1,3)
         nat=j
         goto 110
      endif
      if (iprint.gt.2) print*,' The molecular geometry has been ',
     +     'found with',nat,' atoms'
      if (nat.ne.i2-i1+1) then
         print*,' ***** Error in number of atoms',
     +        nat,nm,i2,i1
         title='Wrong number of molecules'
         return
      endif
      nco=3*nat
      ierr=0
      do i=3*i1-2,3*i2
         if (abs(xarr(i)-xfound(i)).gt.1.0e-5) ierr=1
c         if (abs(xarr(i)-xfound(i)).gt.1.0e-4) ierr=1
      enddo
      if (ierr.eq.1) then
         print*,' ***** GAMESS atom coordinates are inconsistent:'
         do i=3*i1-2,3*i2
            print*,i,xarr(i),xarr(i)-xfound(i)
         enddo
         title='Inconsistent coordinates'
         return
      endif

C     Search for gradient of the energy
      rewind(25)
  120 continue
      read (25,10,end=300,err=300) regel
      if (regel(2:23).eq.'gradient of the energy') then
         if (iprint.gt.2) 
     +        print*,' The gradient of the energy has been found.'
      else
         goto 120
      endif
C     Read gradient of the energy
      kmin=1
      kmax=8
      read (25,10,end=300,err=300) regel
  170 continue
      do i=1,5
         read (25,10,end=300,err=300) regel
      enddo
      if (kmax.gt.nat) kmax=nat
      do m=1,3
         read(25,60,end=320,err=320)
     +        (grad(3*(i1+k)-6+m),k=kmin,kmax)
      enddo
      if (kmax.lt.nat) then
         kmin=kmax+1
         kmax=kmax+8
         goto 170
      endif

  200 continue
C     Search for force constant matrix
      read (25,10,end=300,err=300) regel
      if (regel(23:59).eq.'total cartesian 2nd derivative matrix')
     +     then
         if (iprint.gt.2) 
     +        print*,' The force constant matrix has been found.'
      else
         goto 200
      endif
C     Read force constant matrix
      kmin=1
      kmax=9
      read (25,10,end=300,err=300) regel
  220 continue
      do i=1,11
         read (25,10,end=300,err=300) regel
      enddo
      if (kmax.gt.nco) kmax=nco
      do j=1,nco
         read(25,20,end=310,err=310) 
     +        (akar(3*i1-3+j,3*i1-3+k),k=kmin,kmax)
      enddo
      if (kmax.lt.nco) then
         kmin=kmax+1
         kmax=kmax+9
         goto 220
      endif
  250 continue
c      close(25)
      return

  300 continue
      print*,' ***** Error on unit 25'
      title='Reading error'
      return
  310 continue
      print*,' ***** Error on unit 25 in force constants',j,kmin,kmax
      title='Force constant error'
      return
  320 continue
      print*,' ***** Error on unit 25 in gradients',m,kmin,kmax
      title='Gradient error'
      return


   10 format(a)
   20 format(20x,9f9.5)
   40 format(i9,4x,3f21.9,10x,f20.9)
   41 format(i5,f10.6,4x,3f11.6)
   50 format(40x,f30.10)
   51 format(32x,f30.10)
   60 format(7x,8f15.7)
   80 format(' Gamess energy for molecule',i4,
     +     ' (atoms from',i4,' to',i4,') in a.u.:',f15.6)
      end
