      subroutine leesmd(cutoff,alphac,alpha6,hmaxc,hmax6,tole,tolk,
     +     tempi,temp0,tautp,pres0,comp,taup,t,dt,tol,
     +     ig,nrun,ntt,ntp,nscm,nstlim,ntc,nsnb,
     +     ntpr,ntwx,ntwv,ntwe,ntpw)

C     Reads data for MD run, using function INPT. All parameters are
C     for output to PACK3 or CRYSMD; for their description see RUNMD.

C     It is assumed that that INPT has been called before LEESMD, so
C     the input is fetched from the data stored there.
C     If NSTLIM (number of MD steps) is zero, it is assumed that something
C     is wrong. Then it is tried to read a standard GROMOS input file
C     called MD.IN from unit 15.

C     If PRES0 (atmospheres) is not read, it should have a value on input.

C     Calls subroutines REALST and INTSTR

      implicit none
      integer idum,ig,inpt,ispgr1,n,npm1,nrun,nscm,nsnb,nstlim,ntc,
     +     ntcm,ntp,ntpr,ntpw,ntt,ntwe,ntwem,ntwv,ntwvm,ntwx,ntwxm
      double precision alpha6,alphac,comp,cutoff,dt,dum,hmax6,hmaxc,
     +     pres0,t,taup,tautp,temp0,tempi,tol,tole,tolk
      character regel*200,titel*80
      ig=0
      t=0
      nstlim=0
      nrun=1
      ntt=1
      tempi=0
      temp0=298
      tautp=0.10d0
      ntp=4
      comp=0.00076d0
      taup=5
      ntc=3
      tol=0.00001d0
      dt=0.002d0

      nscm=500
      nsnb=10
      ntpr=0
      ntwx=0
      ntwv=0
      ntwe=0
      ntpw=2

      regel=' '
      n=inpt('MDRUN ',regel)
      if (n.ge.2) call intstr(regel,2,nstlim)
      if (n.ge.3) call intstr(regel,3,nrun)
      if (nstlim.eq.0) then
C        If INPT does not work, try reading GROMOS-type file MD.IN
C        but neglect ISPGR and NPM. And use IG for MDSKIP!
         print 40
         write(16,40)
         open(unit=15,file='in.md',status='OLD',err=200)
         read(15,30) titel
         read(15,45) npm1,idum,idum,idum,ig,tempi
         read(15,41) idum
         read(15,42) nrun,ntt,temp0,dum,tautp
         read(15,42) ntp,idum,pres0,comp,taup
         pres0=pres0*16.389d0
C        This sets PRES0 to atmospheres.
         read(15,48) idum
         read(15,43) idum,ntcm,nscm
         read(15,43) nstlim,idum,idum,t,dt
         read(15,42) ntc,idum,tol
         read(15,48) idum,idum,idum,idum,idum,ispgr1
         read(15,42) idum,nsnb,cutoff
         read(15,48) ntpr
         read(15,48) ntwx,ntwxm,ntwv,ntwvm,ntwe,ntwem,ntpw
         read(15,48) idum
         read(15,48) idum
         read(15,48) idum
         read(15,48) idum
         read(15,49,err=110,end=110) alphac,alpha6,hmaxc,hmax6
         if (hmaxc.lt.1.0) hmaxc=sqrt(100000*hmaxc)/6.28381d0
         if (hmax6.lt.1.0) hmax6=sqrt(100000*hmax6)/6.28381d0
C        this is to ensure compatibility with older X-programs
 110     continue
         close(15)
         if (ntwxm.eq.0) ntwx=0
         if (ntwvm.eq.0) ntwv=0
         if (ntwem.eq.0) ntwe=0
      else
         n=inpt('MDCUT ',regel)
         if (n.ge.2) call realst(regel,2,cutoff)
         if (n.ge.3) then
            if (n.lt.8) then
               print*,' ***** Wrong number of items in CUTOFF',n
               stop
            endif
            call realst(regel,3,alphac)
            call realst(regel,4,alpha6)
            call realst(regel,5,hmaxc)
            call realst(regel,6,hmax6)
            call realst(regel,7,tole)
            call realst(regel,8,tolk)
         endif
         n=inpt('MDSKIP',regel)
         if (n.ge.2) then
            call intstr(regel,2,ig)
         endif
         n=inpt('MDTSC ',regel)
         if (n.ge.4) then
            call intstr(regel,2,ntt)
            call realst(regel,3,tempi)
            call realst(regel,4,tautp)
         endif
         n=inpt('MDTEMP',regel)
         if (n.ge.2) then
            call realst(regel,2,temp0)
         endif
         n=inpt('MDPRES',regel)
         if (n.ge.2) then
            call realst(regel,2,pres0)
         endif
         n=inpt('MDPSC ',regel)
         if (n.ge.4) then
            call intstr(regel,2,ntp)
            call realst(regel,3,comp)
            call realst(regel,4,taup)
         endif
         n=inpt('MDSHAK',regel)
         if (n.ge.2) call intstr(regel,2,ntc)
         if (n.ge.3) call realst(regel,3,tol)
         n=inpt('MDDT  ',regel)
         if (n.ge.2) then
            call realst(regel,2,dt)
         endif
         n=inpt('MDNSCM',regel)
         if (n.ge.2) then
            call intstr(regel,2,nscm)
         endif
         n=inpt('MDNSNB',regel)
         if (n.ge.2) then
            call intstr(regel,2,nsnb)
         endif
         n=inpt('MDPRI ',regel)
         if (n.ge.6) then
            call intstr(regel,2,ntpr)
            call intstr(regel,3,ntwx)
            call intstr(regel,4,ntwv)
            call intstr(regel,5,ntwe)
            call intstr(regel,6,ntpw)
         endif
      endif

      write(16,50)
      write(16,*) '     nrun     nskip    nstlim        dt'
      write(16,51) nrun,ig,nstlim,dt
      write(16,*) '      ntt     tempi     temp0     tau-T'
      write(16,52) ntt,tempi,temp0,tautp
      write(16,*) '      ntp     pres0     compr     tau-P'
      write(16,53) ntp,pres0,comp,taup
      write(16,*) '      ntc       tol      nscm      nsnb'
      write(16,54) ntc,tol,nscm,nsnb
      write(16,*) '     ntpr      ntwx      ntwv      ntwe      ntpw'
      write(16,57) ntpr,ntwx,ntwv,ntwe,ntpw
      write(16,*) ' cutoff  alphac  alpha6   hmaxc   hmax6',
     +     '      tole      tolk'
      write(16,55) cutoff,alphac,alpha6,hmaxc,hmax6,tole,tolk
      pres0=0.061018d0*pres0
C     This sets PRES0 to GROMOS units for pressure scaling.
C     But for NONBE we must set DRUK=PRES0/(NSPGR+1).

      if (temp0.lt.0.01d0) then
         print*,' ***** TEMP0 too small', temp0
         stop
      endif
      print 63
      write(16,63)
      return
 200  continue
      print*,' ***** No file IN.MD present'
      stop

 30   format(a80)
c 31   format(1x,a79)
 40   format(' ***** NSTLIM is zero!',
     +     ' Try reading MD data from GROMOS-type file MD.IN')
 41   format(i10,7f10.5)
 42   format(2i10,6f10.5)
 43   format(3i10,5f10.5)
 45   format(5i10,2f10.5,I10)
c 47   format(1x,2i10,e10.2)
 48   format(8i10)
 49   format(2f10.5,30x,2f10.5)
 50   format (/,'Data characterizing the MD run:')
 51   format(3i10,f10.4)
 52   format(i10,2f10.2,f10.5)
 53   format(i10,3f10.6)
 54   format(i10,f10.6,4i10)
 55   format(f8.3,4f8.2,2f10.5)
c 56   format(2i10,f10.2)
 57   format(10i10)
c 59   format(4f10.2)
 63   format(' End reading of data for MD-run')
      end
