      subroutine leess(nummer,nsp,nrsp,nsym,tsp,msp,spnaam,
     +     nfound,nspgr,titel,panm,box,x,ener,ierr,iwrite,iprint)

C     Reads structural data (cell parameters and atomic coordinates) from
C        an SPF-type file containing TITL, CELL, SPGR/SYMM and ATOM cards.
C     Expands the coordinates according to space group symmetry.
C     Then reduces the expanded coordinates by eliminating coincident ones.
      
C     If one or more SPGR cards are found, they are used to generate the
C     full group of symmetry operators. If not, an SPGR card is looked for
C     and symmetry information is taken from input parameters.
C     In both cases the atoms found are expanded to equivalent groups
C     with the same order of atoms.
C     Possibly coinciding atoms (due to special positions) are removed.

C     Input through parameters:
C     NUMMER: the unit number from which the SPF-type file is to be read.
C         If symmetry data are found from an SPGR card, we need the following:
C       NSP: number of entries for general space group information
C       NRSP(1...NSP): space group number
C       NSYM(1...NSP): number of space group operations (E excluded)
C       TSP(1...3*NSYM(NSP),1...NSP) and
C       MSP(1...3*NSYM(NSP),1...NSP):
C           oblique coordinates are expanded following
C           r'= MSP.r + TSP, so each MSP = +1 or -1 and each TSP = 0 or 0.5.
C       SPNAAM(1...NSP): space group name
C     IWRITE = 0: read, interpret and return data on the SPF file
C            = 3: read, interpret and return data on tape
C            = 2: read and copy this entry to file 13 (removed  entries)
C            = 1: read and copy this entry to file 15 (selected entries)
C     IPRINT regulates output to unit 16 and to screen.

C     Output through parameters:
C     NFOUND: the number of atoms after expansion and reduction
C     NSPGR: the number of space group operations (E excluded)
C     TITEL: the contents of the TITL card (preceded by the space group name)
C     PANM(1...NFOUND): atom names from ATOM cards
C     BOX(1...6): ax, by, cz, bx, cx, cy
C     X(1...3*NFOUND): the atomic coordinates (nm)
C     ENER: the energy (-1.0e10 if not given on ENER card)
C     IERR=0: no error status
C         =1: no first line found
C         =2: no cell data found
C         =3: given space group not found

      implicit none
      include 'cdist.i'
      integer i,ierr,iop,iprint,isp,ispgr,
     +     iwrite,j,k,m,m1,m2,msp,mspg,
     +     nfile,nfound,nop,nop3,nrptot,nrsp,nscrd,nsp,nspgr,nsym,
     +     nummer,nword
      double precision bb,box,bx,by,cc,cosa,cosb,cosc,
     +     cx,cy,cz,d,ener,pi,rad,tsp,tspg,x,xscr

      character*5 panm(maxnrp)
      character code*4,titel*80,ch*1,sptit*8
      character*20 spnm,spnaam(mxspin),upspnm,uplist,noslnm
      dimension box(12),x(3*maxnrp),
     +     nrsp(mxspin),nsym(mxspin),
     +     tsp(3*maxspg,mxspin),msp(3*maxspg,mxspin)

      dimension
     +     mspg(1000,3,3),tspg(1000,3),xscr(3)
      character*80 regel,regels(128)

      pi=4*atan(1.0d0)
      rad=180/pi
      nfile=0
      isp=1
      titel=' '
      ierr=1
      nscrd=0
      read(nummer,11,err=260,end=260) regel
      if (iwrite.eq.1) write(15,11) regel
      if (iwrite.eq.2) write(13,11) regel
      ierr=2
      ener=-1.0d+10
      spnm=' '
      sptit='Unknown'
 510  continue
      code=regel(1:4)
C     Change card code to upper case
      if (code(1:2).ne.'--') then
         do i=1,4
            ch=code(i:i)
c           print*,'ch ',ch
            if (ch.ne.' ' .and. (ch.gt.'z' .or. ch.lt.'A')) then
               print*,' ***** Wrong line in SPF file:'
               print*,regel
               stop
            endif
            k=ichar(ch)
            if (ch.ge.'a' .and. ch.le.'z') k=k-32
            code(i:i)=char(k)
         enddo
      endif
         
      if (code.eq.'TITL') titel=regel(5:80)
      if (code.eq.'SPGR') then
         call strext(regel,2,spnm)
         sptit=spnm(1:8)
      endif
      if (code.eq.'ENER') call realst(regel,2,ener)
      if (code(1:3).eq.'END') then
         if (nfile.eq.0) goto 530
         goto 540
      endif
      if (code.eq.'CELL') then
         ierr=0
         j=nword(regel)-6
         do m=1,6
            call realst(regel,j+m,box(m))
         enddo
      endif
      if (code.eq.'SYMM' .or. code.eq.'symm') then
         nscrd=nscrd+1
         regels(nscrd)=regel
      endif
      if (code.eq.'ATOM') then
         nfile=nfile+1
         if (nfile.gt.maxnrp) then
            print*,' ***** Too many entries in file'
            print*,' Last line: ',regel
            stop
         endif
         call strext(regel,2,panm(nfile))
         do m=1,3
            call realst(regel,m+2,x(3*(nfile-1)+m))
         enddo
      endif
 530  continue
      read(nummer,11,end=540) regel
      if (iwrite.eq.1) write(15,11) regel
      if (iwrite.eq.2) write(13,11) regel
      goto 510
 540  continue

C     The data have now been read from the SPF file
      if (iwrite.eq.1 .or. iwrite.eq.2) return
      if (iprint.gt.1) then
         print*,' '
         write(16,*) ' '
      endif
      if (iprint.gt.1) then
         print 23, nfile,titel(1:40)
         write(16,23) nfile,titel(1:40)
      endif
      if (ierr.eq.2) then
         print*,' ***** No cell given for this entry!'
         write(16,*) ' ***** No cell given for this entry!'
         return
      endif

C     Analyze symmetry and expand atomic coordinates
      nrptot=nfile
      if (nscrd.eq.0 .and. spnm(1:1).eq.' ') then
         nspgr=0
         if (iprint.gt.0) then
            print*,' ***** No symmetry data given for this entry, ',
     +           'P1 assumed.'
            write(16,*)' ***** No symmetry data given for this entry, ',
     +           'P1 assumed.'
         endif
      else
         if (nscrd.gt.0) then
C           Analyze and expand symmetry operators
            call expsym(nscrd,regels,nop,mspg,tspg,iprint)
C           NOP: the total number of generated operations (E included)
C           MSPG: the rotation components
C           TSPG: the translation components
            nspgr=nop-1
            if (nop*nfile.gt.maxnrp) then
               print*,' ***** Increase MAXNRP',nfile,nop
               stop
            endif
C           Expand fractional coordinates following space group symmetry
            do iop=2,nop
               do i=1,nfile
                  panm(i+(iop-1)*nfile)=panm(i)
                  j=3*i-3
                  k=j+3*nrptot
                  do m1=1,3
                     xscr(m1)=x(j+m1)
                  enddo
                  do m1=1,3
                     x(k+m1)=tspg(iop,m1)
                     do m2=1,3
                        x(k+m1)=x(k+m1)+mspg(iop,m1,m2)*xscr(m2)
                     enddo
                  enddo
               enddo
               nrptot=nrptot+nfile
            enddo
         else
C           Search space group name SPNM (changed to upper case in UPSPNM)
            do i=1,20
               ch=spnm(i:i)
               k=ichar(ch)
               if (ch.ge.'a' .and. ch.le.'z') k=k-32
               upspnm(i:i)=char(k)
            enddo
C           Search in list of space group names (changed to upper case: UPLIST
            do isp=1,nsp
               do i=1,20
                  ch=spnaam(isp)(i:i)
                  k=ichar(ch)
                  if (ch.ge.'a' .and. ch.le.'z') k=k-32
                  uplist(i:i)=char(k)
               enddo
C              Maybe a slash was absent, change UPLIST to NOSLNM
               noslnm=uplist
               if (.not. upspnm.eq.uplist) then
                  k=0
                  do i=1,20
                     ch=uplist(i:i)
                     if (ch.ne.'/') then
                        k=k+1
                        noslnm(k:k)=ch
                     endif
                  enddo
               endif
               if (upspnm.eq.noslnm) then
C                 Space group found in the list
                  ispgr=nrsp(isp)
                  nspgr=nsym(isp)
                  if (iprint.gt.0) then
                     print 40,ispgr,spnaam(isp)
                     write(16,40) ispgr,spnaam(isp)
                  endif
                  nrptot=(nspgr+1)*nfile
C                 Expand fractional coordinates following space group symmetry
                  do iop=1,nspgr
                     nop3=3*iop-3
                     do i=1,nfile
                        panm(iop*nfile+i)=panm(i)
                        j=3*i-3
                        k=j+iop*3*nfile
                        x(k+1)=msp(nop3+1,isp)*x(j+1)+tsp(nop3+1,isp)
                        x(k+2)=msp(nop3+2,isp)*x(j+2)+tsp(nop3+2,isp)
                        x(k+3)=msp(nop3+3,isp)*x(j+3)+tsp(nop3+3,isp)
                     enddo
                  enddo
                  goto 550
               endif
            enddo
C           The space group is not present in the list
            print 90,spnm
            write(16,90) spnm
            sptit='Absent'
            ierr=3
            return
 550        continue
         endif
      endif

C     Eliminate equivalent atoms
      do i=1,nrptot
         do j=i+1,nrptot
            if (panm(j).ne.'xxxxx') then
               do m=1,3
                  d=abs(x(3*i-3+m)-x(3*j-3+m))
                  if (abs(nint(d)-d).gt.0.01) goto 415
               enddo
               panm(j)='xxxxx'
               if (iprint.gt.2 .and. i.gt.3) 
     +              print*,' Coinciding atoms:',i,j
 415           continue
            endif
         enddo
      enddo
      nfound=0
      do i=1,nrptot
         if (panm(i).ne.'xxxxx') then
            nfound=nfound+1
            panm(nfound)=panm(i)
            do m=1,3
               x(3*nfound-3+m)=x(3*i-3+m)
            enddo
         endif
      enddo
      if (iprint.gt.0) then
         print*,nfile, ' atoms found on SPF file'
         print*,(nspgr+1)*nfile,' atoms after symmetry expansion'
         print*,nfound,' non-coinciding atoms left'
         write(16,*) nfile, ' atoms found on SPF file'
         write(16,*) (nspgr+1)*nfile,' atoms after symmetry expansion'
         write(16,*) nfound,' non-coinciding atoms left'
      endif

      if (iwrite.eq.3) goto 260
C     Convert to nanometer coordinates
      do m=1,3
         box(m)=box(m)*0.10d0
      enddo
      do i=1,nfound
         panm(i)=panm(mod(i-1,nfile)+1)
         if (iprint.gt.3) then
            print 30,i,panm(i),(x(3*i-3+m),m=1,3)
            write(16,30) i,panm(i),(x(3*i-3+m),m=1,3)
         endif
         do m=1,3
            x(3*i-3+m)=x(3*i-3+m)*box(m)
         enddo
      enddo
       
C     Convert from oblique to rectangular coordinates
      bb=box(2)
      cc=box(3)
      cosa=cos(box(4)/rad)
      cosb=cos(box(5)/rad)
      cosc=cos(box(6)/rad)
      call tri(nfound,0,x,-1,cosa,cosb,cosc)
      cx=cc*cosb
      bx=bb*cosc
      by=sqrt(bb*bb-bx*bx)
      cy=(bb*cc*cosa-bx*cx)/by
      cz=sqrt(cc*cc-cx*cx-cy*cy)
      box(2)=by
      box(3)=cz
      box(4)=bx
      box(5)=cx
      box(6)=cy

 260  continue
      regel=titel
c      titel=sptit//regel(1:70)

 11   format(a80)
 23   format(' New entry read with',i5,' coordinates ',
     +     'from SPF-type coordinate file: ',a)
 30   format(i5,2x,a5,2x,3f10.5)
 40   format('   Space group',i5,2x,a10)
 90   format(' ***** Space group not found: ',a)
      end
