      subroutine leesx(nummer,nfound,nrp,ngr,ireset,nstruc,
     +     titel,spname,ispgr,x,box,iprint)

C     Gets, ordens and (possibly) adds coordinates

C     Input through parameters:
C     NUMMER: unit number of the file to be read. However, if NUMMER=0 no file
C     is read but input data come from parameters NFOUND, NGR, NAAM and X.
C     If NUMMER>0 input format may be SPF, MDI, CSSR, CIF or XYZ.
C     NFOUND: number of atoms in array X (NUMMER=0)
C     NRP: number of atoms in the topology file
C     NGR: the number of topology groups known at moment of calling
C     IRESET=1: reset H-distances to standard value B0.
C     NSTRUC: identity number of structure to be read
C     SPNAME: space group name ('nospgr' for free molecule)
C     ISPGR:  space group number (-1: unknown; 0: free molecule)
C     The latter two data are overridden by information found here.
C     IPRINT regulates the quantity of output.

C---------------------------------------------------------------------------
C     Reads coordinate file on unit NUMMER. A - indicates a new line.

C     SPF format.
C     This format is compatible with the programs PLATON and PLUTON.
C     - TITL followed by arbitrary text
C     - CELL a, b, c (Angstrom), alpha, beta, gamma (degrees)
C     - ATOM atom name, x, y, z (oblique fractional coordinates)
C     As many of these cards as necessary.
C     - END
C     All other code words are disregarded.

C     MDI format.
C     This format is compatible with the GROMOS program package.
C     - arbitrary text
C     - format (i5): number of atoms
C     - format (i5,5x,a5,i5,3f8.5): I1, atom name, I2,
C     x, y, z (Cartesian nanometer coordinates)
C     - NOTE: this is fixed format, so the two integers I1 and I2
C     (which are never used) may be absent.
C     - a, b, c (nanometers), alpha, beta, gamma (degrees)

C     XYZ format
C     - number of atoms
C     - arbitrary text
C     x, y, z (Cartesian coordinates, Angstrom) [, charge]

C     CSSR and CIF formats can now also be read.
C---------------------------------------------------------------------------

C     Input from common block /BIND/:
C     IB,JB(1..NBOND): code numbers for bonded atoms
C     B0(1..NBOND): distances to bonded H-atoms
C     Input from common block /ATTP/:
C     NAAM(1..NRP): atom names corresponding to given coordinates (NUMMER=0)
C     PANM(1..): atom names in topology file, used if NGR=1

C     Output parameters:
C     NFOUND: number of atoms on coordinate file after eliminating special
C     positions and after adding H atoms
C     TITEL: title from coordinate file
C     X(1..): coordinates (rectangular, nm)
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     NGR: the number of topology groups
C     SPNAME: space group name. Special possibilities:
C     'P1' if cell was given, but no symmetry information
C     'nospgr' for free molecule 
C     'unknown' if only symmetry operators are found
C     'r-spname' if symmetry has been artificially reduced
C     for special positions or high symmetry
C     ISPGR: space group number (-1: unknown; 0: free molecule)

C     Output through common block /ATTP/:
C     NAAM(1..): atom names found on coordinate file (not ordered!)

C     Output through common block /NONP/:
C     CGCOR(1..): is default set to 999. But if charges are found
C     on a CSSR file, they are stored in CGCOR

C     Calls subroutines STREXT, REALST, NORM, PROD and function NWORD.
C     And further:
C     Free molecule: calls CONN ---> IDENT
C     Crystal: calls TRI, GETSYM ---> LEESR, [SPEPOS ---> IDENT], TRI
      
      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cnonp.i'
      integer nummer,nfound,nrp,ngr,ireset,nstruc,ispgr,iprint
      double precision x,box

      dimension x(*),box(*),jhb(10),ipres(10),ireqdh(10),iadd(10),
     +     cor(maxat,3),v1(3),v2(3),v3(3),v4(3),v5(3),v6(3),
     +     pmat(3,3),umat(3,3),tmat(3,3),smat(3,3),uinv(3,3)
      character*80 regel,titel,code*4,cifsp*11,titel1,regels(128)
      character*8 spname,naam1(maxat),tekst*12
      character*180 regel2
      double precision bb,bx,by,cx,cy,cz,cc,cor,cosa,cosb,cosc,cth,cth2,
     +     pi,rad,r2,rch,scale,sig,sth,sth2,th,th2,v1,v2,v3,v4,v5,v6,
     +     a,ax,pmat,umat,tmat,smat,uinv,vol
      integer i,i1,i2,i3,i1a,iadd,iaddh,icheck,igr,ihit,ilabel,iplaats,
     +     inew,ipres,ireqdh,ispf,ix,iy,iz,
     +     j,jhb,k,m,n,n1,nadd,ncorh,npres,nreqdh,nscrd
      integer nword

      data inew /0/
      save inew

      pi=4*atan(1.0d0)
      rad=180/pi
      do i=1,maxat
         cg1(i)=999
         cgcor(i)=999
      enddo
      nscrd=0

      if (nummer.eq.0) then
         ispgr=0
         if (iprint.gt.0) print*,
     +        ' LEESX. Reading coordinates from internal data'
         do i=1,ngr
            do j=1,nfound/ngr
               naam(j+(i-1)*nfound/ngr)=naam(j)
            enddo
         enddo
         goto 500
      endif

      do i=1,maxat
         naam(i)='unknown '
      enddo
      do m=1,3
         box(m)=1
         box(m+3)=90
      enddo

C     Undocumented option: include text from fort.28 in title
      tekst=' '
      open(unit=28,status='OLD',err=110)
      read(28,12,end=110,err=110) i,tekst
      print*,i,nstruc,tekst
      if (i.ne.nstruc) then
         print*,'***** Structure number disgrees with fort.28'
         stop
      endif

 110  continue
      read(nummer,11,end=990,err=990) regel
      if (regel(1:80).eq.' ') goto 110
      if (nstruc.gt.1) then
         ispgr=0
         spname='nospgr'
         print 38,nstruc
         write(16,38) nstruc
      endif
      print*,'LEESX. Reading coordinate file, first line:'
      write(16,*) 'LEESX. Reading coordinate file, first line:'
      print*,regel
      write(16,*) regel
      nscrd=0
 112  continue
      if (regel(1:1).eq.'#' .or. regel(1:80).eq.' ') then
         read(nummer,13,end=990,err=980) regel2
         regel=regel2(1:80)
         goto 112
      endif

      if (regel(1:4).eq.'data') then
C     Try reading CIF-type file
         nfound=0
         titel=regel(6:80)
         if (tekst.ne.' ') titel=tekst//'  '//regel(6:70)
         print 23,'CIF',titel
         write(16,23) 'CIF',titel 

         ihit=0
 120     continue
C        Read cell and space group symmetry
         read(nummer,11,end=990,err=980) regel
 1201    continue
         if (regel(1:1).eq.'#') goto 120
         do i=1,6
            if (regel(i:i).ne.' ') then
               regel=regel(i:80)
               goto 1203
            endif
         enddo
         goto 120
 1203    continue
         if (regel(1:11).eq.'_atom_site_') goto 125
         if (regel(1:14).eq.'_cell_length_a') then
            ihit=ihit+1
            call realst(regel,2,box(1))
            goto 120
         endif
         if (regel(1:14).eq.'_cell_length_b') then
            ihit=ihit+1
            call realst(regel,2,box(2))
            goto 120
         endif
         if (regel(1:14).eq.'_cell_length_c') then
            ihit=ihit+1
            call realst(regel,2,box(3))
            goto 120
         endif
         if (regel(1:17).eq.'_cell_angle_alpha') then
            ihit=ihit+1
            call realst(regel,2,box(4))
            goto 120
         endif
         if (regel(1:16).eq.'_cell_angle_beta') then
            ihit=ihit+1
            call realst(regel,2,box(5))
            goto 120
         endif
         if (regel(1:17).eq.'_cell_angle_gamma') then
            ihit=ihit+1
            call realst(regel,2,box(6))
            goto 120
         endif
         if (regel(1:27).eq.'_symmetry_equiv_pos_as_xyz') then
            ispgr=-1
 1202       continue
            read(nummer,11,end=990,err=980) regel
            do i=1,30
               if (regel(i:i).ne.' ') then
                  regel=regel(i:80)
                  goto 1204
               endif
            enddo
            goto 1202
 1204       continue
            if (regel(1:1).ne.'_' .and. regel(1:4).ne.'loop') then
               nscrd=nscrd+1
               regels(nscrd)='SYMM '//regel(1:75)
               goto 1202
            else
               if (iprint.gt.0) then
                  do i=1,nscrd
                     print*,regels(i)
                  enddo
               endif
               goto 1201
            endif
         endif
         if (regel(1:30).eq.'_symmetry_space_group_name_H-M') then
            ispgr=-1
            regel2=regel(31:80)
            j=0
            i=30
            cifsp=' '
 121        continue
            i=i+1
            if (i.eq.80) then
               call strext(regel,2,cifsp)
               print*, ' ***** No quotes in space group name: ',cifsp
               spname=cifsp(1:8)
               goto 124
            endif
            if (regel(i:i).eq.' ' .or. regel(i:i).eq.'_') goto 121
            if (regel(i:i).eq.'''' .and. j.eq.0) then
               j=j+1
               goto 121
            endif
            cifsp(j:j)=regel(i:i)
            j=j+1
            if (regel(i:i).ne.'''') goto 121
            spname=cifsp(1:j-2)
            if (ispgr.eq.-1 .and. spname.eq.'nospgr') then
               print*,' ***** No space group information on CIF file'
               stop
            endif
         endif
 124     continue
         goto 120

C        Determine structure of coordinate cards
 125     continue
         if (ihit.lt.6) then
            print*,' Incomplete set of cell parameters, free molecule'
            ispgr=0
         endif
         iplaats=1
 130     continue
         if (regel(1:16).eq.'_atom_site_label') ilabel=iplaats
         if (regel(1:18).eq.'_atom_site_fract_x') ix=iplaats
         if (regel(1:18).eq.'_atom_site_fract_y') iy=iplaats
         if (regel(1:18).eq.'_atom_site_fract_z') iz=iplaats
         iplaats=iplaats+1
 135     continue
         read(nummer,11,end=990,err=980) regel
         do i=1,6
            if (regel(i:i).ne.' ') then
               regel=regel(i:80)
               goto 1205
            endif
         enddo
         goto 135
 1205    continue

         if (regel(1:10).eq.'_atom_site') goto 130
         if (regel(1:4).eq.'data') then
            print*,' ***** Premature new data on CIF file'
            stop
         endif

 140     continue
C        Read coordinates
         if (regel(1:80).eq.' ') then
            read(nummer,11,end=200,err=980) regel
            goto 140
         endif
         if (regel(1:4).eq.'data') goto 200
         if (regel(1:1).eq.'#' .or. regel(1:5).eq.'loop_') goto 150
         nfound=nfound+1
         call strext(regel,ilabel,naam(nfound))
         call realst(regel,ix,x(3*nfound-3+1))
         call realst(regel,iy,x(3*nfound-3+2))
         call realst(regel,iz,x(3*nfound-3+3))
         read(nummer,11,end=200,err=980) regel
         goto 140
 150     continue
         read(nummer,13,end=200,err=980) regel2
         regel=regel2(1:80)
         if (regel(1:4).ne.'data') goto 150

 200     continue
         backspace(nummer)
         do m=1,3
            box(m)=box(m)*0.10d0
         enddo
         do i=1,nfound
            do m=1,3
               x(3*(i-1)+m)=x(3*(i-1)+m)*box(m)
            enddo
         enddo
         goto 500
      endif
C     End reading CIF file

      if (regel(1:4).eq.'TITL') then
C     Try reading SPF-type file
         nfound=0
         titel=regel(5:80)
         print 23,'SPF',titel
         write(16,23) 'SPF',titel 
 300     continue
         read(nummer,11,end=340) regel
 310     continue
         code=regel(1:4)
         if (code(1:3).eq.'END') then
            if (nfound.eq.0) goto 300
            goto 340
         endif
         if (code.eq.'SPGR') then
            ispgr=-1
            call strext(regel,2,spname)
         endif
         if (code.eq.'CELL') then
            ispgr=-1
            j=nword(regel)-6
            do m=1,6
               call realst(regel,j+m,box(m))
            enddo
         endif
         if (code.eq.'SYMM' .or. code.eq.'symm') then
            ispgr=-1
            if (nword(regel).gt.1) then
               nscrd=nscrd+1
               regels(nscrd)=regel
            else
 320           continue
               read(nummer,11,end=340) regel
               if (regel(1:1).lt.'A' .or. regel(1:).gt.'Z') then
                  nscrd=nscrd+1
                  regels(nscrd)='SYMM '//regel(1:75)
                  goto 320
               else
                  goto 310
               endif
            endif
         endif
         if (code.eq.'ATOM') then
            nfound=nfound+1
            call strext(regel,2,naam(nfound))
            do m=1,3
               call realst(regel,m+2,x(3*(nfound-1)+m))
            enddo
         endif
         goto 300
 340     continue
         do m=1,3
            box(m)=box(m)*0.10d0
         enddo
         do i=1,nfound
            do m=1,3
               x(3*(i-1)+m)=x(3*(i-1)+m)*box(m)
            enddo
         enddo
         if (iprint.gt.0) then
            do i=1,nscrd
               print*,regels(i)
            enddo
         endif
         goto 500
      endif
C     End reading SPF file

C     Investigate the first two lines of the file: MDI, CSSR or XYZ
      read(nummer,13,end=990,err=980) regel2
      if (nword(regel).eq.1) then
C        No CSSR file
         read(regel,92,err=360) nfound
C        Probably XYZ file
         if (nword(regel2).eq.1) then
            read(regel2,92,err=350) i
            print*,' Cannot distinguish between MDI and XYZ!'
            print*,' Expand the text line to at least two words.'
            stop
         endif
 350     continue
         print 23,'XYZ',regel2
         write(16,23) 'XYZ',regel2
         goto 450
      endif

 360  continue
      if (nword(regel2).eq.1) then
         read(regel2,92,err=370) nfound
         print 23,'MDI',regel
         write(16,23) 'MDI',regel
         titel=regel
         do i=1,nfound
            read(nummer,92,err=430) k,naam(i),k,(x(3*(i-1)+m),m=1,3)
         enddo
         ispgr=0
         read(nummer,*,end=500,err=500) (box(m),m=1,6)
         if (inew.eq.0) then
            print*,' Box given at the end of MDI file'
         endif
         ispgr=-2
         goto 500
 430     continue
         print*,' ***** Error reading MDI file'
         stop
      endif

 370  continue
C     Try reading CSSR-type file
      read(regel,41,end=425,err=425) (box(m),m=1,3) 
C     Defuse problems with unreadable space group number:
      read(regel2,42,end=425,err=425) (box(m),m=4,6),cifsp,titel
      ispgr=-10
      read(cifsp,90,end=421,err=421) i
      ispgr=i
 421  continue
      call strext(titel,1,spname)
      print*,'Second line gives space group ',spname
      read(nummer,43,end=425,err=425) nfound,ispf,titel
      print*,'Third line:',nfound,ispf,titel
      nscrd=0
 422  continue
      read(nummer,11,end=425,err=425) titel1
      print*,titel1
      if (titel1(1:4).eq.'SYMM' .or. titel1(1:4).eq.'symm ') then
         nscrd=nscrd+1
         regels(nscrd)=titel1
         goto 422
      endif
      titel=titel1(1:25)//titel(1:30)
      print 24, titel(1:60)
      write(16,24) titel(1:60)
      print 35,(box(m),m=1,6),ispgr,spname
      write(16,35) (box(m),m=1,6),ispgr,spname
      do i=1,nfound
         read(nummer,45,end=425,err=425) k,naam(i),
     +        (x(3*(i-1)+m),m=1,3),cg1(i)
         cgcor(i)=cg1(i)
      enddo
      do m=1,3
         box(m)=box(m)*0.10d0
      enddo
      do i=1,nfound
         do m=1,3
            x(3*(i-1)+m)=x(3*(i-1)+m)*box(m)
         enddo
      enddo
C     Adjust CSSR nomenclature to our peculiar one
      if (ispgr.eq.14) ispgr=1014
      if (ispgr.eq.15) ispgr=1015
      if (ispgr.ne.0 .and. spname(1:3).eq.'   ') then
         print*,' No space group name given, but ISPGR =',ispgr
         write(16,*) 'No space group name given, but ISPGR =',ispgr
         write(spname,77) ispgr
      endif
      if (spname.eq.'unknown' .or. spname(1:2).eq.'r-')ispgr=-1 
      if (ispgr.eq.0) then
         if (spname(1:3).eq.'nos' .or. spname(1:3).eq.'   ') then
            print*,' No space group information, free molecule'
            write(16,*) 'No space group information, free molecule'
         else
            ispgr=-1
            print*,' No space group number given, but name = ',spname
            write(16,*) ' No space group number given, but name = ',
     +           spname
         endif
      endif
      goto 500
c     endif
 425  continue
      print*,' ***** Error reading CSSR file'
      stop

 450  continue
C     Try reading (extended) XYZ-type file
      titel=regel
      read(titel,92) nfound
      print*,regel2
      if (regel2(1:7).eq.'Lattice') then
         print 47,nfound
         write(titel,47) nfound
         regel2=regel2(10:180)
         do i=1,180
            if (regel2(i:i).eq.'"') regel2(i:i)=' '
         enddo
         call realst(regel2,1,smat(1,1))
         call realst(regel2,2,smat(1,2))
         call realst(regel2,3,smat(1,3))
         call realst(regel2,4,smat(2,1))
         call realst(regel2,5,smat(2,2))
         call realst(regel2,6,smat(2,3))
         call realst(regel2,7,smat(3,1))
         call realst(regel2,8,smat(3,2))
         call realst(regel2,9,smat(3,3))
         do i=1,3
c        print*,(smat(i,j),j=1,3)
            do j=1,3
               smat(i,j)=smat(i,j)/10.0d0
            enddo
         enddo
         m=0
         do i=1,nfound
            read(nummer,11,end=995,err=995) regel
            call strext(regel,1,naam(i))
            call realst(regel,2,cor(i,1))
            call realst(regel,3,cor(i,2))
            call realst(regel,4,cor(i,3))
            m=m+1
            if (m.gt.nrp) m=1
c           print*,'---',naam(i),'----',panm(m),'----',(cor(i,j),j=1,3)
         enddo

C        Transform!!!!!
         do i=1,3
            do j=1,3
               a=0
               do k=1,3
                  a=a+smat(i,k)*smat(j,k)
               enddo
               pmat(i,j)=a
            enddo
c        print*,(pmat(i,k),k=1,3)
         enddo

         ax=sqrt(pmat(1,1))
         bb=sqrt(pmat(2,2))
         cc=sqrt(pmat(3,3))
c        print*,ax,bb,cc
         cosc=pmat(1,2)/(ax*bb)
         cosb=pmat(1,3)/(ax*cc)
         cosa=pmat(2,3)/(bb*cc)
c        print*,rad*acos(cosa),rad*acos(cosb),rad*acos(cosc)
         bx=bb*cosc
         by=sqrt(bb*bb-bx*bx)
         cx=cc*cosb
         cy=cc*(cosa-cosb*cosc)/(sqrt(1-cosc*cosc))
         cz=sqrt(cc*cc-cx*cx-cy*cy)
         vol=ax*by*cz
c        print*,ax,by,cz, bx, cx, cy
         umat(1,1)=ax
         umat(1,2)=0
         umat(1,3)=0
         umat(2,1)=bx
         umat(2,2)=by
         umat(2,3)=0
         umat(3,1)=cx
         umat(3,2)=cy
         umat(3,3)=cz
C        BOX(1..6): ax, by, cz, bx, cx, cy
         box(1)=ax
         box(2)=bb
         box(3)=cc
         box(4)=rad*acos(cosa)
         box(5)=rad*acos(cosb)
         box(6)=rad*acos(cosc)
         print*,'Volume',vol
         write(16,*) 'Volume',vol
c     print*,'umat '
c     do i=1,3
c     print*,(umat(i,k),k=1,3)
c     enddo

C        Bereken UINV(Tr)
         uinv(1,1)=1/ax
         uinv(1,2)=-bx*cz/vol
         uinv(1,3)=(bx*cy-by*cx)/vol
         uinv(2,1)=0.0d0
         uinv(2,2)=1/by
         uinv(2,3)=-ax*cy/vol
         uinv(3,1)=0.0d0
         uinv(3,2)=0.0d0
         uinv(3,3)=1/cz
c     print*,'U-1Tr '
c     do i=1,3
c     print*,(uinv(i,k),k=1,3)
c     enddo

C     Check inversie
c     print*,'check U.UINV: unit matrix'
c     do i=1,3
c     do j=1,3
c     a=0
c     do k=1,3
c     a=a+umat(i,k)*uinv(j,k)
c     enddo
c     pmat(i,j)=a
c     enddo
c     print*,(pmat(i,k),k=1,3)
c     enddo

C     Find T-matrix, U = T.S and T = (UINV)Tr . (SMAT)Tr
c     print*,'tmat'
         do i=1,3
            do j=1,3
               a=0
               do k=1,3
                  a=a+uinv(k,i)*smat(k,j)
               enddo
               tmat(i,j)=a
            enddo
c        print*,(tmat(i,k),k=1,3)
         enddo

C     Check T matrix
c     print*,'check T.T(tr): unit matrix'
c     do i=1,3
c     do j=1,3
c     a=0
c     do k=1,3
c     a=a+tmat(i,k)*tmat(j,k)
c     enddo
c     pmat(i,j)=a
c     enddo
c     print*,(pmat(i,k),k=1,3)
c     enddo

         m=0
C        Check transformatie U = T.S
         do i=1,3
            do j=1,3
               a=0
               do k=1,3
                  a=a+tmat(j,k)*smat(i,k)
               enddo
               pmat(i,j)=a
               if (a-umat(i,j).gt.1.0d-8) then
                  print*,i,j,a-umat(i,j)
                  m=1
               endif
            enddo
c           print*,(pmat(i,k),k=1,3)
         enddo
         if (m.eq.1) then
            print*,' ***** Error in transformatiematrix'
            print*,'umat '
            do i=1,3
               print*,(umat(i,k),k=1,3)
            enddo
            stop
         endif

C        Transform coordinates
         m=0
         do i=1,nfound
            do j=1,3
               a=0
               do k=1,3
                  a=a+tmat(j,k)*cor(i,k)
               enddo
               m=m+1
               x(m)=a/10.0d0
            enddo
c           print*,i,m,'   ',panm(i),(x(k),k=m-2,m)
         enddo
c        print*,(box(m),m=1,6)
c        spname='P1'
         ispgr=-2
         goto 500
      else
         print 48,nfound
         write(titel,48) nfound
         do i=1,nfound
            read(nummer,11,end=995,err=995) regel
            call strext(regel,1,naam(i))
            call realst(regel,2,cor(i,1))
            call realst(regel,3,cor(i,2))
            call realst(regel,4,cor(i,3))
         enddo
         m=0
         do i=1,nfound
            do j=1,3
               m=m+1
               x(m)=cor(i,j)/10.0d0
            enddo
         enddo
c        print*,i,m,'   ',naam(i),(x(k),k=m-2,m)
      endif

 500  continue
C     Reading the coordinate file is finished. Data in nanometers: cell (BOX)
C     and NFOUND coordinates (X) of atoms (NAAM) in non-orthogonal system.
      if (nummer.ne.0) then
         print 40,nfound,nrp
         write(16,40) nfound,nrp
      endif
      do i=1,nfound
 510     continue
         if (naam(i)(1:1).eq.' ') then
            naam(i)=naam(i)(2:7)//' '
            goto 510
         endif
         if (iprint.gt.3) print 44,naam(i),(x(3*i-3+m),m=1,3)
      enddo

C     Check whether coordinates and names must be rearranged.
C     Note that only the first character is checked (e.g. C30=C#17)
C     If that is unsatisfactory, interchange comment signs between 
C         7 and 8 lines below, starting with: if (naam(i)......      
      icheck=1
      ngr=nfound/nrp
      if (ngr*nrp.eq.nfound) then
         icheck=0
         do i=1,nfound
c           if (naam(i).ne.panm(i)) then
            if (naam(i)(1:1).ne.panm(i)(1:1)) then
               icheck=i
               if (iprint.gt.1) print 46,i,panm(i),naam(i)
            endif
         enddo
      endif
      
      if (ispgr.eq.0) then
C        Free molecule found from coordinates
         if (inew.eq.0) then
            print*,' No cell given, ',
     +           'non-fractional coordinates assumed'
            write(16,*) ' No cell given, ',
     +           'non-fractional coordinates assumed'
            if (icheck.gt.0) then
               print*,' Atom order differs from topology file, ',
     +              'check through CONN and IDENT'
               write(16,*) ' Atom order differs from topology file, ',
     +              'check through CONN and IDENT'
            endif
         endif
         inew=1
         if (nummer.ne.0) spname='nospgr'
         ngr=nfound/nrp
         if (icheck.eq.0 .and. ireset.eq.0) return
C        Atom names inconsistent with topology file, try to rearrange.
         call conn(nfound,nrp,ngr,naam,x,0.0d0,0.0d0,0.0d0,iprint)
         if (ngr.eq.0) goto 985
      else
C        Crystal structure found from coordinates
         bb=box(2)
         cc=box(3)
         cosa=cos(box(4)/rad)
         cosb=cos(box(5)/rad)
         cosc=cos(box(6)/rad)

         if (ispgr.eq.-2) then
C           Exception: coordinates were MDI-type, transform to oblique
            ispgr=-1
            call tri(nfound,0,x,1,cosa,cosb,cosc)
         endif

C        Read space group information and check for special positions.
         call getsym(ispgr,spname,nscrd,regels,iprint)
C        If a known space group is given nscrd is set to zero.
C        Check for special positions and find connectivity
         if (nscrd.gt.0) then
            print*,' Since nscrd =',nscrd,', SPEPOS will be called'
            write(16,*) ' Since nscrd =',nscrd,', SPEPOS will be called'
         endif
         if (icheck.gt.0) then
            print*,' Atom order differs from topology file, ',
     +           'check through SPEPOS and IDENT'
            write(16,*) ' Atom order differs from topology file, ',
     +           'check through SPEPOS and IDENT'
         endif
         if (icheck.gt.0 .or. nscrd.ne.0) then
            call spepos(nfound,ngr,box,
     +        naam,spname,cg1,cgcor,ispgr,iprint)
            if (ngr.eq.0) goto 985
C           If ngr=0 IDENT has failed. Probably atoms in one group
c           have different equivalent positions.
         endif
         print*,'Space group ',spname,ispgr
         write(16,*)'Space group ',spname,ispgr
         print*,ngr,' topology groups found after symmetry analysis'
         write(16,*)ngr,' topology groups found after symmetry analysis'
         if (iprint.gt.1) then
            print*,' Fractional coordinates'
            do igr=1,ngr
               do n=1,nrp
                  i=(igr-1)*nrp+n
                  print 37,igr,n,i,cgcor(i),panm(i),naam(i),
     +                 (x(3*i-3+m)/box(m),m=1,3)
               enddo
            enddo
         endif

C        Convert from (a,b,c,alfa,beta,gamma) to (ax,by,cz,bx,cx,cy)
         if (ispgr.ne.0) call tri(ngr*nrp,0,x,-1,cosa,cosb,cosc)
         cx=cc*cosb
         bx=bb*cosc
         by=sqrt(bb*bb-bx*bx)
         cy=(bb*cc*cosa-bx*cx)/by
         cz=sqrt(cc*cc-cx*cx-cy*cy)
         box(2)=by
         box(3)=cz
         box(4)=bx
         box(5)=cx
         box(6)=cy
      endif

C     End reading coordinate input
      do i=1,nrp*ngr
         naam1(i)=naam(i)
         cor(i,1)=x(3*i-2)
         cor(i,2)=x(3*i-1)
         cor(i,3)=x(3*i)
      enddo

      do i=1,nrp*ngr
c        print*,naam(i),'---',panm(i)
c        if (naam(i).ne.panm(i)) goto 550
         if (naam(i)(1:1).ne.panm(i)(1:1)) goto 550
      enddo
      if (ireset.eq.0) then
         if (iprint.gt.0) then
            print*,' No change of hydrogen atom input necessary'
            write(16,*)' No change of hydrogen atom input necessary'
         endif
         goto 970
      endif

 550  continue
      if (iprint.gt.1) then
         print*,' Nanometer coordinates'
         do i=1,nrp*ngr
            print 39,i,naam1(i),panm(i),(cor(i,m),m=1,3)
         enddo
      endif

      
C     Add and/or reset hydrogen atoms
      do igr=1,ngr
         if (iprint.gt.2) print 33,igr
         if (ireset.eq.1) then
            print*, 'RESET H ATOMS'
            write(16,*) 'RESET H ATOMS'
         endif
         iaddh=0
         do n=1,nrp
            i=(igr-1)*nrp+n
            if (panm(i)(1:1).ne.'H') then
               npres=0
               nreqdh=0
               ncorh=0
               nadd=0
               do j=1,nbond
                  k=0
                  if (ib(j).eq.n) k=jb(j)
                  if (jb(j).eq.n) k=ib(j)
                  if (k.gt.0) then
                     k=(igr-1)*nrp+k
                     if (panm(k)(1:1).eq.'H') then
                        nreqdh=nreqdh+1
                        ireqdh(nreqdh)=k
                        rch=b0(j)
                        if (naam(k)(1:1).eq.'H') then
                           if (iprint.gt.2) print*,' Present ',naam(k)
                           ncorh=ncorh+1
                           jhb(ncorh)=k
                        else
                           if (iprint.gt.1) then
                              print*,' To be added ',k,'   ',naam(k)
                              write(16,*)' To be added ',k,'   ',naam(k)
                           endif
                           nadd=nadd+1
                           iadd(nadd)=k
                        endif
                     else
                        npres=npres+1
                        ipres(npres)=k
                     endif
                  endif
               enddo
C              The number of non-H atoms around atom I is NPRES
C              The number of required H atoms is NREQDH
               if (nreqdh.gt.0) then
                  if (iprint.gt.3) print 20,panm(i),
     +                 (panm(ipres(k)),k=1,npres),
     +                 (panm(ireqdh(k)),k=1,nreqdh)
C                 The number of H-atoms already present is NCORH
                  if (iprint.gt.3) print 21,ncorh
                  if (nadd.ne.nreqdh-ncorh) then
                     print*,' ***** Error in NADD',
     +                    i,nadd,nreqdh,ncorh,npres
                     stop
                  endif
                  if (nadd.ge.0) then
                     do k=1,ncorh
                        if (iprint.gt.3) print 25,jhb(k),
     +                       naam(jhb(k)),panm(ireqdh(k))
                        naam1(ireqdh(k))=naam(jhb(k))
                        npres=npres+1
                        ipres(npres)=ireqdh(k)
                        if (ireset.gt.0) then
                           r2=0
                           do m=1,3
                              v1(m)=x(3*ireqdh(k)-3+m)
     +                             -x(3*i-3+m)
                              r2=r2+v1(m)**2
                           enddo
                           scale=rch/sqrt(r2)
                           do m=1,3
                              x(3*ireqdh(k)-3+m)=
     +                             x(3*i-3+m)+scale*v1(m)
                           enddo
                        endif
                     enddo
                  endif
                  if (nadd.gt.0) then
                     if (iprint.gt.4) print 22,npres,nreqdh,ncorh,
     +                    (iadd(k),k=1,nadd)

C                    Add NADD hydrogen atoms to the known NPRES number of atoms
                     cth=-1/3.0d0
                     th=acos(cth)
                     sth=sin(th)
                     th2=th/2
                     cth2=cos(th2)
                     sth2=sin(th2)
                     i1=ipres(1)
                     i2=ipres(2)
                     i3=ipres(3)
                     n1=i1-(igr-1)*nrp
                     iaddh=iaddh+nadd
                     if (npres.eq.3 .and. nadd.eq.1) then
C                       Add one H-atom to a tetrahedral C-atom
                        do m=1,3
                           v2(m)=x(3*i1-3+m)-x(3*i-3+m)
                           v3(m)=x(3*i2-3+m)-x(3*i-3+m)
                           v4(m)=x(3*i3-3+m)-x(3*i-3+m)
                        enddo
                        call norm(v2)
                        call norm(v3)
                        call norm(v4)
                        do m=1,3
                           v6(m)=-(v2(m)+v3(m)+v4(m))
                           v1(m)=v3(m)-v2(m)
                           v5(m)=v4(m)-v3(m)
                        enddo
                        call prod(v1,v5,v3)
                        call norm(v3)
                        call norm(v6)
                        sig=0
                        do m=1,3
                           sig=sig+v3(m)*v6(m)
                        enddo
                        if (sig.gt.0) sig=1
                        if (sig.lt.0) sig=-1
                        if (iprint.gt.1) then
                           print*,'Adding',iadd(1),i,sig,rch,i1,i2,i3
                           write(16,*) 'Adding',
     +                           iadd(1),i,sig,rch,i1,i2,i3
                        endif
                        do m=1,3
                           x(3*iadd(1)-3+m)=
     +                          x(3*i-3+m)+sig*rch*v3(m)
C                               This algorithm uses direction V3.
C                               It appears to give slightly better angles
C                                  than simply taking the direction V6.
                        enddo

                     elseif (npres.eq.2 .and. nadd.eq.1) then
C                       Add one H-atom to a trigonal C-atom
c                       print*,' CH-1 ',i1,'   ',panm(i1)
c                       print*,' CH-2 ',i2,'   ',panm(i2)
                        do m=1,3
                           v1(m)=x(3*i-3+m)-x(3*i1-3+m)
                           v2(m)=x(3*i-3+m)-x(3*i2-3+m)
                        enddo
                        call norm(v1)
                        call norm(v2)
                        do m=1,3
                           v3(m)=v1(m)+v2(m)
                        enddo
                        call norm(v3)
                        do m=1,3
                           x(3*iadd(1)-3+m)=x(3*i-3+m)+rch*v3(m)
                        enddo

                     elseif (npres.eq.2 .and. nadd.eq.2) then
C                       Add two H-atoms to a tetrahedral C-atom
c                       print*,' CH2-1 ',i1,'   ',panm(i1)
c                       print*,' CH2-2 ',i2,'   ',panm(i2)
                        do m=1,3
                           v4(m)=x(3*i1-3+m)-x(3*i-3+m)
                           v5(m)=x(3*i2-3+m)-x(3*i-3+m)
                        enddo
                        call norm(v4)
                        call norm(v5)
                        call prod(v4,v5,v2)
                        do m=1,3
                           v1(m)=-(v4(m)+v5(m))
                        enddo
                        call norm(v1)
                        call norm(v2)
                        do m=1,3
                           x(3*iadd(1)-3+m)=x(3*i-3+m)+
     +                          rch*(cth2*v1(m)+sth2*v2(m))
                           x(3*iadd(2)-3+m)=x(3*i-3+m)+
     +                          rch*(cth2*v1(m)-sth2*v2(m))
c                               print*,m,(x(3*iadd(1)-3+m)
c     +                         -x(3*i-3+m))**2,
c     +                         (x(3*iadd(2)-3+m)-x(3*i-3+m))**2
                        enddo

                     elseif (npres.eq.1 .and. nadd.eq.3) then
C                       Add three H-atoms to a tetrahedral C-atom. Find a bond
C                       I1a at I and put the H atom trans to that bond.
C                       In searching for I1a the topology list is followed.
C                       If it should be followed in reverse order, replace the
C                       next statement "do j=1,nbond" by the commented one.
                        do j=1,nbond
                           if (ib(j).eq.n1.and.jb(j).ne.n) then
                              i1a=jb(j)+(igr-1)*nrp
                              if (panm(i1a)(1:1).ne.'H') then
                                 goto 830
                              endif
                           endif
                           if (jb(j).eq.n1.and.ib(j).ne.n) then
                              i1a=ib(j)+(igr-1)*nrp
                              if (panm(i1a)(1:1).ne.'H') then
                                 goto 830
                              endif
                           endif
                        enddo
                        print*, '***** Connected atom not found'
                        stop
 830                    continue
                        if (iprint.gt.2) print*,'    Dihedral defined ',
     +                       'by connected atom',  i1a,'   ',panm(i1a)  
                        do m=1,3
                           v1(m)=x(3*i1a-3+m)-x(3*i1-3+m)
                           v2(m)=x(3*i1 -3+m)-x(3*i -3+m)
                        enddo
                        call norm(v1)
                        call norm(v2)
                        call prod(v1,v2,v3)
                        call norm(v3)
                        call prod(v3,v2,v4)
                        do m=1,3
                           v5(m)=cth*v2(m)+sth*v4(m)
                           v6(m)=-(v5(m)+v2(m))
                        enddo
                        call norm(v6)
                        do m=1,3
                           x(3*iadd(1)-3+m)=x(3*i-3+m)+rch*v5(m)
                           x(3*iadd(2)-3+m)=x(3*i-3+m)+
     +                          rch*(cth2*v6(m)+sth2*v3(m))
                           x(3*iadd(3)-3+m)=x(3*i-3+m)+
     +                          rch*(cth2*v6(m)-sth2*v3(m))
                        enddo

                     elseif (npres.eq.1 .and. nadd.eq.1) then
C                       Add one H-atom to an O-atom. Find a bond
C                       I1a at I and put the H atom trans to that bond.
C                       In searching for I1a the topology list is followed.
C                       If it should be followed in reverse order, replace the
C                          statement "do j=1,nbond" by "do j=nbond,1,-1"
                        do j=1,nbond
                           if (ib(j).eq.n1.and.jb(j).ne.n) then
                              i1a=jb(j)+(igr-1)*nrp
                              goto 880
                           endif
                           if (jb(j).eq.n1.and.ib(j).ne.n) then
                              i1a=ib(j)+(igr-1)*nrp
                              goto 880
                           endif
                        enddo
                        print*, '***** Connected atom not found'
                        stop
 880                    continue
                        if (iprint.gt.2) print*,'    Dihedral defined ',
     +                       'by connected atom',  i1a,'   ',panm(i1a)  
                        do m=1,3
                           v1(m)=x(3*i1a-3+m)-x(3*i1-3+m)
                           v2(m)=x(3*i1 -3+m)-x(3*i -3+m)
                        enddo
                        call norm(v1)
                        call norm(v2)
                        call prod(v1,v2,v3)
                        call norm(v3)
                        call prod(v3,v2,v4)
                        do m=1,3
                           v5(m)=cth*v2(m)+sth*v4(m)
                           v6(m)=-(v5(m)+v2(m))
                        enddo
                        call norm(v6)
                        do m=1,3
                           x(3*iadd(1)-3+m)=x(3*i-3+m)+rch*v5(m)
                        enddo

                     else
                        print 26,i,npres,nadd
                        stop
                     endif

                     do j=1,nadd
                        naam1(iadd(j))='Added'
                     enddo
                     if (iprint.gt.4) then
                        print 31,rch,(ipres(k),k=1,npres)
                        do j=1,nadd
                           print 32,iadd(j),panm(iadd(j)),
     +                          (x(3*iadd(j)-3+m),m=1,3)
                        enddo
                     endif
                  endif
               endif
            endif
         enddo

         print 30,iaddh
         write(16,30) iaddh
         if (iprint.gt.3) then
            do i=1,nrp*ngr
               j=i-nrp*((i-1)/nrp)
               if (naam1(i)(1:5).eq.'Added') print 88,i,panm(j),
     +              naam1(i),(x(3*(i-1)+m),m=1,3)
            enddo
         endif
         nfound=nfound+iaddh
      enddo
      
 970  continue
C     End of the calculations
      if (iprint.gt.1) then
         print 73,(box(m),m=1,6)
         if (ispgr.ne.0) then
            print*,' Found space group ',spname,' number',ispgr
         endif
      endif
      if (iprint.gt.1) then
         do i=1,abs(nrp*ngr)
            print 88,i,panm(i),naam1(i),(x(3*(i-1)+m),m=1,3)
         enddo
      endif
      if (iprint.gt.1) then
         print*,' End LEESX: found',nfound,' atoms in',ngr,' groups'
         write(16,*) ' End LEESX: found',nfound,
     +         ' atoms in',ngr,' groups'
      endif
      return

 980  continue
      print*,' ***** Error on coordinate file'
      write(16,*) ' ***** Error on coordinate file'
      nfound=-2
      return

 985  continue
      print*,' ***** Error in LEESX, structure', nstruc
      write(16,*)' ***** Error in LEESX, structure',nstruc
      return
      
 990  continue
      print*,' End reached on coordinate file'
      write(16,*) ' End reached on coordinate file'
      nfound=-1
      return
      
 995  continue
      write(16,*) '***** Incorrect number of atoms',i,nfound,nrp
      print*, '***** Incorrect number of atoms',i,nfound,nrp
      stop

 11   format(a80)
 12   format(i5,a)
 13   format(a180)
 20   format(' Atom ',a5,' must be surrounded by   ',6a5)
 21   format(i6,' hydrogen atom(s) found in coordinate file')
 22   format(5x,'With',3i5,'  adding atom(s): ',10i5)
 23   format(' Reading coordinates from ',a4,' type file: ',a)
 24   format(' Reading coordinates from CSSR type file: ',a,3x,a)
 25   format(5x,'Assigning atom',i4,' called ',a6,' to ',a5)
 26   format(' ***** Unknown H-addition algorithm for atom',4i4)
c     27 format(i5,' groups of',i5,' atoms found from',i5, 
c     +     ' atoms on ordered coordinate file')
c     28 format(i5,' atoms found in',i5,' groups of',i5,' atoms.',
c     +     '  Compare NRP =',i5)
c     29 format(i5,' groups of',i5,' atoms found from',i5, 
c     +     ' atoms on random coordinate file')
 30   format(' Created',i3,' H-atoms not found in coordinate file')
 31   format(5x,'H-distance',f8.4,5x,'Reference atoms:',10i5)
 32   format(5x,'Added atom',i5,2x,a5,' with coordinates ',10f10.5)
 33   format(' Assigning coordinates for molecule',i5)
c     34 format(i5,' groups of', i4,' atoms have equivalent names')
 35   format(' Cell: ',3f8.4,3f8.2,'     Space group',i5,'   ',a)
c     36 format('  ***** LEESX: Inconsistency in equivalent atom names:',
c     +     3i4,5x,2a5)
 37   format(3i4,f10.3,3x,2a6,3f10.5)
 38   format(//,'------------------ N E W    S T R U C T U R E',i5,
     +     ' ------------------')
 39   format(i4,3x,2a6,3f10.5)
 40   format(i4,' atoms found on coordinate file,',i4,' in topology')
 41   format(38x,3f8.3)
 42   format(21x,3f8.3,9x,a3,a20)
 43   format(2i4,1x,a80)
 44   format(a,3f12.5)
 45   format(i4,1x,a4,2x,3f10.5,33x,f7.3)
 46   format(' Inadequate input for atom',i3,':  ',a,
     +     ' Found: ',a)
 47   format(' Extended XYZ file found with',i4,' atoms')
 48   format(' Single molecule XYZ file found with',i4,' atoms')
 73   format(' Found box: ',6f10.5)
 77   format(i5,3f8.3,f10.3)
 88   format(i5,5x,2a8,3x,3f10.5)
 90   format(i3)
 92   format(i5,5x,a5,i5,6f8.5)
      end
