C     This set of routines ("line-to-word") serves to provide free format
C     input while storing the input line for further examination.
C     Each input line must to be built from substrings separated
C     by one or more spaces.
C     Courtesy of A. M. M. Schreurs.

      subroutine realst(str,n,x)
C     Fetch the N-th substring from STR and converts to real X.
      implicit none
      integer n,i,nword
      double precision x
      character str*(*)
      character*30 param
      if (n.gt.nword(str) .or. n.le.0) then
         print*,' ***** Realstr: wrong substring number',n,
     +        ' in string: ', str
         stop
      endif
      call strext(str,n,param)
C     Eliminate error limits given in parentheses as in 13.788(15)
      do i=1,30
         if (param(i:i).eq.'(') then
            param(i:30)='                     '
            goto 100
         endif
      enddo
  100 continue
      call getrl(x,param)
      end

      subroutine intstr(str,n,i)
C     Fetch the N-th substring from STR and converts to integer I.
      implicit none
      integer n,i,nword
      character str*(*)
      character*30 param
      if (n.gt.nword(str) .or. n.le.0) then
         print*,' ***** Intstr: wrong substring number',n,
     +        ' in string: ',str
         stop
      endif
      call strext(str,n,param)
      call getint(i,param)
      end

      function nword(str)
C     Returns the number of words in string STR.
      implicit none
      integer n,i,ll,nchar,nword
      character str*(*)
      character line*255
      call strcmp(str,line)
      ll = nchar(line)
      n = 0
      if (ll.gt.0) then
         do i=1,nchar(line)
            if (line(i:i).eq.' ') n = n + 1
         enddo
         n = n + 1
      endif
      nword=n
      end

      subroutine strext(source,n,destin)
C     Put N-th substring from SOURCE into string DESTIN
C     destin = ' ' if none found
      implicit none
      integer n,i
      character source*(*)
      character destin*(*)
      character line*255
      integer   p1,p2
      call strcmp(source,line)
      destin = ' '
      if (n.le.0) return
      p1 = 1
      p2 = index(line(p1:),' ')
      do i=2,n
         p1 = p1+p2
         p2 = index(line(p1:),' ')
      enddo
      if (p2.gt.0) then
         destin = line(p1:p1+p2-1)
      else
         destin = ' '
      endif
      end

C     The following functions and subroutines are only used locally in YLTW,
C     and not in other routines of UPACK.

      function nchar(str)
C     Returns number of characters in the string
      implicit none
      integer i,nchar
      character str*(*)
      do i=len(str),1,-1
         if (str(i:i).ne.' ') then
            nchar = i
            return
         endif
      enddo
      nchar = 0
      end

      subroutine getint(i,dd)
C     Tries to convert the string dd into an integer number i
      implicit none
      integer i
      character dd*(*)
      read(dd,10,err=125,end=125) i
      return
 125  print*,' Trying to do internal read from non-integer string: ',dd
      stop
 10   format(i20)
      end

      subroutine getrl(x,dd)
C     Tries to convert the string dd into a real number x
      implicit none
      integer i
      double precision x
      character dd*(*)
c     read(dd,*,err=115) x
c     This works well, but is no standard Fortran. So:
c     try to read integer...
      read(dd,10,err=130,end=130) i
      x=i
      return
 130  continue
c     try to read real ..
      read(dd,20,err=115,end=115) x
      return
 115  continue
c     no real can be obtained
      print*,' Trying to do internal read from non-real string: ',dd
      stop
 10   format(i20)
 20   format(f20.10)
      end

      subroutine strcmp(source,destin)
C     replaces tabs and multiple spaces by single spaces
      implicit none
      integer ldn,ldo,nchar,i,n
      character source*(*)
      character destin*(*)
      n=nchar(source)
      do i=1,n
c         if (ichar(source(i:i)).eq.9) then
c            print*,'TAB replaced'
c         endif
         if (ichar(source(i:i)).eq.9) source(i:i)=' ' 
      enddo
      call strld(source,destin)
      ldn = nchar(destin)
      ldo = 0
 140  if (ldo.ne.ldn) then
         call strsbs(destin,'  $',' $',destin)
         ldo = ldn
         ldn = nchar(destin)
         goto 140
      endif
      end

      subroutine strld(source,destin)
C     remove all leading spaces
      implicit none
      integer i,nchar
      character source*(*)
      character destin*(*)
      i = 1
      if (nchar(source).gt.0) then
 150     if (source(i:i).eq.' ') then
            i = i + 1
            goto 150
         endif
      endif
      destin = source(i:)
      return
      end

      subroutine strsbs(source,old,new,destin)
C     substitute all substrings old in source to new in destin
C     old and new are strings ended with a dummy character
C     old: string to change.. (not empty, nchar KL)
C     new: into this one (possibly empty, nchar KN)
      implicit none
      integer i,k,kl,kn,l,ls,nchar
      character source*(*)
      character old*(*)
      character new*(*)
      character destin*(*)
      character temp*255
      ls = nchar(source)
      if (ls.eq.0) then
         destin = source(1:ls)
         return
      endif
      KL = len(old)-1
      if (KL.le.0) then
         destin = source(1:ls)
         return
      endif
      i = index(source(1:ls),old(1:KL))
      if (i.eq.0) then
         destin = source(1:ls)
         return
      endif
      KN = len(new) - 1
      if (KN.lt.0) then
         destin = source(1:ls)
         return
      endif

      K = 1
      L = 1
 160  if (k.le.ls) then
         if (source(K:K+KL-1).eq.old(1:KL)) then
            if (kn.gt.0) then
               temp(L:L+KN-1) = new
            endif
            L = L + KN
            K = K + KL
         else
            temp(L:L) = source(K:K)
            L = L + 1
            K = K + 1
         endif
         goto 160
      endif
      if (L.eq.1) then
         destin = temp(1:1)
      else
         destin = temp(1:L-1)
      endif
      end
