      program mktop

C     Makes UPACK topology and part of ALL.KEY input for XTINKER.
C     B. P. van Eijck, February 2001.
C     Input files:  IN.TP (unit 20), molecular topology
C                   IN.FF (unit 12), force field
C     Output files: OUT.PRI (unit 16), general results
C                   OUT.XT  (unit 19), output for all.key in XTINKER
C                   OUT.UP  (unit 29), output for topology in UPACK

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cinds.i'

      integer i,m,nrp
      double precision a,phi
      open(unit=16,file='out.pri')
      open(unit=19,file='out.xt')
      open(unit=29,file='out.up')

      call leest1(nrp)

c      print 20,'ref-atms',(ireftr(m),m=1,nref)
      do i=1,ndiha
         print 20,'prt-tors',iome(i),jome(i),kome(i),lome(i)
      enddo
      print 10,'#bonp', nbond
      print 10,'#angp', nhoek
      print 10,'#torp', ntors

      write(19,20) 'parameters /home/vaneyck/xtinker/params/pw'
      write(19,20) '#chargeterm'
      write(19,20) '#no-intra-nonbe'
      write(19,20) 'cutoff 20'
      write(19,20) 'rep-cutoff 10'
      write(19,20) 'writeout 100'
      write(19,20) 'printout 1'
      write(19,20) 'aintra -1000'
      write(19,20) 'pairlist-shift 0.1'
      write(19,20) 'maxcyc 10'
      write(19,20) '#basis-intra 6-31g*  6-31g*'
      write(19,20) '#basis-inter 6-31g** 6-31g**'
      write(19,20) ' '
      write(19,20) 'UNSAFE'
      write(19,20) 'ref-atms',(ireftr(m),m=1,nref)
      do i=1,ndiha
         write(19,20) 'prt-tors',iome(i),jome(i),kome(i),lome(i)
      enddo
      write(19,10) '#bonp'
      do i=1,nbond
         write(19,11) ib(i),jb(i),0.5d0*cb(i),b0(i)
      enddo
      write(19,10) '#angp'
      do i=1,nhoek
         write(19,12) it(i),jt(i),kt(i),0.5d0*ct(i),t0(i)
      enddo
      write(19,10) '#torp'
      do i=1,ntors
         a=2*cp(i)
         phi=0
         if (a.lt.0) phi=180
         write(19,13) ip(i),jp(i),kp(i),lp(i),abs(a),phi,np(i)
      enddo

   10 format(a5,i5)
   11 format('bondr',2i8,f10.1,f10.4)
   12 format('angr', 3i8,f10.1,f10.3)
   13 format('torsr',4i8,f10.4,f10.1,i4)
   20 format(a,12i5)
      end

      subroutine nospac(nfile,regel)
C     Writes content of REGEL to file NFILE without trailing spaces
      implicit none
      integer i,nfile
      character*(*) regel
      character form*60,line*200
      line=' '
      line=regel
      do i=200,1,-1
         if (line(i:i).ne.' ') goto 100
      enddo
  100 continue
      if (i.lt.10) then
         write(form,10) i
      elseif (i.lt.100) then
         write(form,20) i
      else
         write(form,30) i
      endif
      write(nfile,form) line(1:i)
   10 format('(a',i1,')')
   20 format('(a',i2,')')
   30 format('(a',i3,')')
      end

      subroutine leest1(nrploc)
C     B. P. van Eijck, June 2001. Adjusted for ZMKPRM.
C     Input files: IN.TP (unit 20), molecular topology
C                  IN.FF (unit 12), force field
C        Optional: CHARGES.IN (CSSR or XYZ format) to supersede
C                  the charges from the toplogy and force field

C---------------------------------------------------------------------------
C     Reads file IN.TP which contains a formatted topology file,
C     created by progrsam CODES and/or by hand.
C     Input units: Angstrom, kcal/mole, atomic charge units
C     Output units: nanometers, different charge units, and kJ/mole.
C---------------------------------------------------------------------------
C     A force field file IN.FF must be present:
C     - text
C     - IBUCK = 0: use Lennard-Jones parameters (BUCK set to FALSE)
C                  with Gromos-type combination rule
C     -       =-1: use Lennard-Jones parameters (BUCK set to FALSE)
C                  with Charmm-type combination rule
C             = 1: use Buckingham parameters (BUCK set to TRUE)
C     - CUTOFF (not used)
C     - NPP (not used)
C     - FACT: factor to obtain kJ/mol from input energy units
C       FACR: factor to obtain nanometers from input length units
C     - Optional (not used)
C
C     Now the force field data are read.
C     On each line (possible mixed with blank lines):

C     codes 100-199, bond force constant, bond length
C     codes 200-299, angle force constant, angle value
C     codes 300-399, improper force constant, angle value
C     codes 400-499, torsional force constant, periodicity
C        If the periodicity is negative, the force constant is divided
C           by the multiplicity (which is the number of times that the
C           central atoms occur in torsional angles.
C           This usually presumes equal force constants.
C        If the periodicity is positive, the multiplicity is always 1.
C     Following data not used.
C---------------------------------------------------------------------------

C     Output trough parameters:
C        NRPLOC: number of atoms in one molecule

C     Output through various common blocks:
C        /NONP/ gives information on nonbonded parameters:
C           NCAG: number of charge groups in one molecule
C           IAC(1..NRP): atom codes, taking values 1..Ncod
C           IDO(I)=0: atom I does not contribute to nonbonded energy
C           IEXCL(1..NRP,1..NRP): excluded neighbors.
C           The elements of array IEXCL(i,j) are:
C               =0: intermolecular interaction
C               >0: intramolecular interaction:
C               =1: 1..1, 1..2, 1..3 excluded neighbors
C               =2: 1..4 neighbors
C               =3: 1..5 and further neighbors
C           INC(0..NCAG): charge group pointer list: INC(I) specifies the
C               position of the last atom of the I-th group in the
C               atom sequence.
C               (This algorithm is taken from the GROMOS procedure NONBML)
C        /ATTP/:
C           PANM(1..NRPLOC): atom names
C           TYPE(1..NRPLOC): atom types
C        /BIND/:
C           NBOND: number of bonds
C           IB, JB (1..): atom numbers in pairs for each bond
C           CB, B0 (1..): force constants and reference bond lengths
C        /HOEK/:
C           NHOEK: number of angles
C           IT, JT, KT (1..): atom numbers for each angle
C           CT, T0 (1..): force constants and reference angles
C        /IMPR/:
C           NIMPR: number of improper dihedrals
C           IQ, JQ, KQ, LQ (1..): atom numbers for each dihedral
C           CQ, Q0 (1..): force constants and reference bond lengths
C        /TORS/:
C           NTORS: number of proper dihedrals
C           IP, JP, KP, LP (1..): atom numbers for each dihedral
C           CP, NP (1..): force constants and periodicities
C        /INDS/ gives information on special dihedrals:
C           NDIHZ:  number of dihedral angles that determine the separate
C              conformations
C           NDIHH: number of less essential dihedrals
C           NDIHA=NDIHZ + NDIHH
C           IOME, JOME, KOME, LOME(1..NDIHA):  atomic numbers to define
C              the dihedral
C           IREFTR(1...), the reference atoms (see TRANS)

C     Calls subroutines STREXT, REALST, NOSPAC and function NWORD.

      implicit none
      include 'params.i'
      include 'cbond.i'
      include 'cnonb.i'
      include 'cnonp.i'
      include 'cinds.i'
      include 'cpola.i'

      integer i,i1,i2,i3,i4,ibuck,id,ioutd,ireadc,isav,
     +     j,k,m,maxff,mc,mul,n,nchg,npt,nratt,nrploc,ntor,nvdw,nword
      double precision c,cha,cpt,facb,facr,fact,ff

      parameter(maxff=900)
      dimension ff(maxff,3),isav(maxat),id(maxat)
      character*8 typein(maxnrt),woord,scr*5
      character*80 regel,titel,topver,fftors(400:499)

C     Read force field file
      do i=400,499
         fftors(i)='notread'
      enddo
      do i=1,maxff
         ff(i,1)=-999
         ff(i,2)=-999
         ff(i,3)=-999
      enddo
      open(unit=12,file='in.ff',status='OLD',err=860)
      read(12,11) regel
      print 28,regel
      write(16,28) regel
      read(12,*) ibuck
      buck=(ibuck.eq.1)
      if (buck) then
         print*,'   Buckingham-potential'
         write(16,*) '   Buckingham-potential'
      else
         if (ibuck.eq.0) then
            print*,'   Lennard-Jones potential,',
     +           ' Gromos-recipe for combination rule'
            write(16,*) '   Lennard-Jones potential,',
     +           ' Gromos-recipe for combination rule'
         else
            print*,'   Lennard-Jones potential,',
     +           ' Charmm-recipe for combination rule'
            write(16,*) '   Lennard-Jones potential,',
     +           ' Charmm-recipe for combination rule'
         endif
      endif
      read(12,11) regel
      read(12,11) regel
      read(12,*) fact,facr
      facb=fact/(facr**2)
      nvdw=0
      read(12,11) regel

C     Read force field codes
  106 read(12,11,end=108,err=108) regel
      n=nword(regel)
      if (n.eq.0) goto 106
      call strext(regel,1,woord)
      read(woord,25,err=107) i
      if (i.ge.400 .and. i.lt.500) then
         fftors(i)=regel
      else
         if (n.gt.4) n=4
         do m=2,n
            call realst(regel,m,ff(i,m-1))
         enddo
      endif
      goto 106
  107 continue
C     Read next line with Van der Waals data
      nvdw=nvdw+1
      goto 106
  108 continue
      close(12)
      print*,nvdw,' lines with van der Waals parameters found'
      write(16,*) nvdw,' lines with van der Waals parameters found'

C     Start reading topology file
      open(unit=20,file='in.tp',status='OLD',err=840)
      read(20,11) titel
      print 29,titel
      write(16,29) titel
      read(20,11) topver
      if (topver(8:17) .eq. 'atom types') then
         print*,' ***** Outdated version of topology file'
         write(16,*) ' ***** Outdated version of topology file'
         ioutd=1
         read(topver,25) nratt
         if (nratt.gt.maxnrt) then
            print*,' ***** Increase maxnrt'
            stop
         endif
         read(20,11) regel
         if (nratt.ne.nword(regel)) then
            print*,' ***** Error in LEEST:',nratt,nword(regel)
            stop
         endif
         do i=1,nratt
            call strext(regel,i,typein(i))
         enddo
         read(20,11) regel
         do i=1,nratt
            read(20,11) regel
         enddo
         read(20,*) nrploc
      elseif (topver(8:17) .eq. 'atoms in a') then
C        New version of topology file and force field file
         ioutd=0
         read(topver,25) nrploc
         nratt=0
      else
         print*,' ***** Error in topology file: ',topver(8:17)
         stop
      endif

C     Read information on atoms
      if (nrploc.gt.maxnrp) then
         print*,'  ***** Increase maxnrp',nrploc
         stop
      endif
      read(20,11) regel
      do i=1,nrploc
         read(20,11) regel
         call intstr(regel,1,isav(i))
         if (i.ne.abs(isav(i))) then
            print*,' ***** Error in atom numbering in LEEST',i,j
            stop
         endif
         call strext(regel,2,panm(i))
         call intstr(regel,3,id(i))
         call strext(regel,4,type(i))
         if (ioutd.eq.1) then
            iac(i)=id(i)
            if (type(i).ne.typein(iac(i))) then
               print*,' ***** Error in atom codes ',
     +              type(i),i,typein(iac(i))
               stop
            endif
         else
            do j=1,i-1
               if (type(j).eq.type(i)) then
                  iac(i)=iac(j)
                  goto 120
               endif
            enddo
            nratt=nratt+1
            if (nratt.gt.maxnrt) then
               print*,' ***** Increase maxnrt'
               stop
            endif
            iac(j)=nratt
            typein(nratt)=type(i)
  120       continue
            if (iac(i).ne.id(i)) then
               print*,' ***** Inconsistency in atom codes ',
     +              type(i),i,id(i),iac(i)
               stop
            endif
         endif
         call realst(regel,5,amass(i))
         call realst(regel,6,c)
C        Determine what kind of charge input there is
         if (i.eq.1) then
            if (c.ge.499.5 .and. c.lt.599.5) then
               ireadc=0
               print*,'Charge input through codes'
            else
               ireadc=1
               print*,'Charge input through values'
            endif
         endif
         if (c.ge.499.5 .and. c.lt.599.5) then
            if (ireadc.eq.1) then
               print*,'***** Inconsistent charge input', ireadc
               stop
            endif
            mc=nint(c)
            cg(i)=ff(mc,1)
            if (cg(i).lt.-900) then
               print*,' ***** Charge code',mc,' missing for atom',i
               stop
            endif
         else
            if (ireadc.eq.0) then
               print*,'***** Inconsistent charge input',ireadc
               stop
            endif
            cg(i)=c
            if (abs(cg(i)).gt.100) then
               print*,' ***** Wrong charge for atom',i,cg(i)
               stop
            endif
         endif
         call realst(regel,7,c)
         if (c.ge.799.5 .and. c.lt.899.5) then
            m=nint(c)
            alfpol(i)=ff(m,1)
            pmax(i)=ff(m,2)
            if (cg(i).lt.-900) then
               print*,' ***** Charge code',m,' missing for atom',i
               stop
            endif
         else
            alfpol(i)=c
            pmax(i)=1000000
         endif
         if (abs(alfpol(i)+999).lt.0.001d0) then
c            print*, 'Polarizability ',i,' set to zero'
            alfpol(i)=0
         endif
         if (abs(pmax(i)+999).lt.0.001d0) then
c            print*, 'Maximum dipole ',i,' set to 1000000'
            pmax(i)=1000000
         endif
      enddo
      print*,nratt,' atom types found'
      write(16,*) nratt,' atom types found'


C     Read exclusion matrix
      read(20,11) regel
      read(20,11) regel
      do i=1,nrploc
         read(20,*) (iexcl(j,i),j=1,i)
         do j=1,i
            iexcl(i,j)=iexcl(j,i)
         enddo
      enddo

C     Read charge group information
      read(20,*) ncag
      if (ncag.gt.maxcag) then
         print*,' ***** Increase maxcag',ncag,maxcag
         stop
      endif
      read(20,*) (inc(i),i=1,ncag)
      inc(0)=0
      if (ncag.eq.0) then
         print*,' ***** No charge groups'
         stop
      endif
      if (inc(ncag).ne.nrploc) then
         print*, ' ***** Charge group not closed'
         stop
      endif

C     Read bond information
      read(20,*) nbond
      if (nbond.gt.maxbon) then
         print*,' ***** Increase maxbon',nbond
         stop
      endif
      do i=1,nbond
         read(20,11) regel
         n=nword(regel)
         call intstr(regel,1,ib(i))
         call intstr(regel,2,jb(i))
         if (n.ge.4) then
            call strext(regel,4,scr)
            if (scr.eq.'*****') then
               n=3
            else
               call realst(regel,3,cb(i))
               call realst(regel,4,b0(i))
            endif
         endif 
         if (n.eq.3) then
            call intstr(regel,3,m)
            if (m/100.ne.1) then
               print*,' ***** Error in bond code',i,m
               cb(i)=0
               b0(i)=0
            else
               ibcod(i)=m
               cb(i)=ff(m,1)*facb
               b0(i)=ff(m,2)*10*facr
            endif
         endif
C        Read bond increments
         cha=0
         if (ireadc.eq.0 .and. ff(m,3).gt.-900) cha=ff(m,3)
C        Apply bond increments
         cg(ib(i))=cg(ib(i))-cha
         cg(jb(i))=cg(jb(i))+cha
         if (cb(i).lt.0 .or. b0(i).lt.0) then
            print 39,' ***** Error in bond ',i,m,cb(i),b0(i)
            stop
         endif
      enddo

C     Read angle information
      read(20,*) nhoek
      if (nhoek.gt.maxang) then
         print*, ' ***** Increase maxang',nhoek
         stop
      endif
      do i=1,nhoek
         read(20,11) regel
         n=nword(regel)
         call intstr(regel,1,it(i))
         call intstr(regel,2,jt(i))
         call intstr(regel,3,kt(i))
         if (n.eq.5) then
            call strext(regel,5,scr)
            if (scr.eq.'*****') then
               n=4
            else
               call realst(regel,4,ct(i))
               call realst(regel,5,t0(i))
            endif
         endif 
         if (n.eq.4) then
            call intstr(regel,4,m)
            if (m/100.ne.2) then
               print*,' ***** Error in angle code',i,m
               ct(i)=0
               t0(i)=0
            else
               ihcod(i)=m
               ct(i)=ff(m,1)*fact
               t0(i)=ff(m,2)
            endif
         endif
         if (ct(i).lt.0 .or. t0(i).lt.0) then
            print 39,' ***** Error in angle ',i,m,ct(i),t0(i)
            stop
         endif
      enddo

C     Read improper dihedral information
      read(20,*) nimpr
      if (nimpr.ne.0) then
         read(20,11) regel
      endif
      if (nimpr.gt.maximp) then
         print*, ' ***** Increase maximp',nimpr
         stop
      endif
      do i=1,nimpr
         read(20,11) regel
         n=nword(regel)
         call intstr(regel,1,iq(i))
         call intstr(regel,2,jq(i))
         call intstr(regel,3,kq(i))
         call intstr(regel,4,lq(i))
         if (n.eq.6) then
            call strext(regel,6,scr)
            if (scr.eq.'*****') then
               n=5
            else
               call realst(regel,5,cq(i))
               call realst(regel,6,q0(i))
            endif
         endif 
         if (n.eq.5) then
            call intstr(regel,5,m)
            if (m/100.ne.3) then
               print*,' ***** Error in improper code',i,m
               cq(i)=0
               q0(i)=0
            else
               iicod(i)=m
               cq(i)=ff(m,1)*fact
               q0(i)=ff(m,2)
            endif
         endif
         if (cq(i).lt.0) then
            print 39,' ***** Error in improper ',i,m,cq(i),q0(i)
            stop
         endif         
      enddo

C     Read torsional information
      read(20,*) ntor
      read(20,11) regel
      ntors=0
      do i=1,ntor
         read(20,11) regel
         n=nword(regel)
         mul=1
         if (n.lt.5) then
            print*,' ***** Error in torsional code',i,n
            stop
         endif
         call intstr(regel,1,i1)
         call intstr(regel,2,i2)
         call intstr(regel,3,i3)
         call intstr(regel,4,i4)
         if (n.ge.7) then
            call strext(regel,7,scr)
            if (scr.eq.'*****') then
               n=6
            else
               ntors=ntors+1
               ip(ntors)=i1
               jp(ntors)=i2
               kp(ntors)=i3
               lp(ntors)=i4
               call intstr(regel,5,mul)
C              The multiplicity has now already been taken care of.
C              So is the number of torsional periodicities.
               if (mul.ne.1) then
                  print*,'Multiplicity must be 1!!!'
                  stop
               endif
               call realst(regel,6,cp(ntors))
               call intstr(regel,7,np(ntors))
            endif
         endif
         if (n.lt.7) then
            call intstr(regel,5,m)
            if (n.eq.6) call intstr(regel,6,mul)
            if (m.eq.0 .or. regel(1:7).eq.'notread') then
               print*,' ***** Error in torsion ',i,m,mul
               regel='0    0     0'
            else
               regel=fftors(m)
            endif
            n=nword(regel)
            k=n/2
            if (2*k+1.ne.n) then
               print*,' ***** Even number of entries for torsion',m
               stop
            endif
            do j=1,k
               ntors=ntors+1
               if (ntors.gt.maxdih) then
                  print*, ' ***** Increase maxdih',ntors
                  stop
               endif
ccccc???               itcod(ntors)=m
               ip(ntors)=i1
               jp(ntors)=i2
               kp(ntors)=i3
               lp(ntors)=i4
               call realst(regel,2*j,cpt)
               call intstr(regel,2*j+1,npt)
               cp(ntors)=cpt*fact
               if (npt.lt.0) cp(ntors)=cpt/mul
               if (m.eq.0) np(ntors)=mul
               np(ntors)=abs(npt)
               if (m.eq.0) np(ntors)=mul
               if (np(ntors).eq.0) print 23
             enddo
         endif
      enddo

C     Take charges from CHARGES.IN, if present
      open(unit=13,file='charges.in',status='OLD',err=220)
      print*,'***** Taking charges from CHARGES.IN'
      read(13,11) regel
      n=nword(regel)
      if (n.eq.1) then
         print*,'Assume XYZ-type file'
         read(13,11) titel
         print 11,titel
         call intstr(regel,1,nchg)
         if (nchg.ne.nrploc) goto 810
         do i=1,nchg
            read(13,11,end=810,err=810) regel
            n=nword(regel)
            call realst(regel,n,cg(i))
         enddo
      endif
      if (n.eq.3) then
         print*,'Assume CSSR-type file'
         read(13,10,end=820,err=820) regel
         read(13,10,end=820,err=820) regel
         call intstr(regel,1,nchg)
         if (nchg.ne.nrploc) goto 820
         read(13,10,end=820,err=820) regel
         do i=1,nchg
            read(13,11,end=820,err=820) regel
            call realst(regel,14,cg(i))
         enddo
      endif
  220 continue

C     Start output of topolopy file
      call nospac(29,titel)
      call nospac(29,topver)
      write(29,74)
      do i=1,nrploc
         write(29,77) isav(i),panm(i),id(i),type(i),amass(i),
     +        cg(i),alfpol(i)
      enddo
      write(29,*) ' Exclusion matrix: 0: intermolecular interaction'
      write(29,*) '   Intramolecular: 1: 1-2,1-3;   2: 1-4;   3: 1-5,..'
      do i=1,nrploc
         write(29,'(25i3)') (iexcl(j,i),j=1,i)
      enddo
      write(29,42) ncag,'charge groups, ending at:'
      write(29,'(15i5)') (inc(i),i=1,ncag)
      write(29,42) nbond,'bonds'
      do i=1,nbond
         scr=' '
         if (b0(i).lt.0.00001d0) scr='*****'
         write(29,51) ib(i),jb(i),cb(i),b0(i),scr
      enddo
      write(29,42) nhoek,'angles'
      do i=1,nhoek
         scr=' '
         if (t0(i).lt.0.00001d0) scr='*****'
         write(29,52) it(i),jt(i),kt(i),ct(i),t0(i),scr
      enddo
      write(29,42) nimpr,'improper dihedrals'
      if (nimpr.ne.0) then
         print*,' ***** Check assignment of',nimpr,' improper dihedrals'
         write(29,*) '***** Change code if needed!'
      endif
      do i=1,nimpr
         scr=' '
c         if (cq(i).lt.0.00001) scr='*****'
         write(29,53) iq(i),jq(i),kq(i),lq(i),cq(i),q0(i),scr
      enddo
      write(29,42) ntors,'torsions'
      write(29,26)
      do i=1,ntors
         scr=' '
         if (np(i).lt.1 .or. abs(cp(i)).lt.0.00001d0) scr='*****'
         write(29,54) ip(i),jp(i),kp(i),lp(i),1,cp(i),np(i),scr
      enddo

C     Read information on special dihedrals
      read(20,*,end=890,err=890) ndihz
      write(29,42) ndihz,'essential dihedrals'
      do i=1,ndihz
         read(20,*) iome(i),jome(i),kome(i),lome(i)
         write(29,43) iome(i),jome(i),kome(i),lome(i)
      enddo
      read(20,*) ndihh
      write(29,42) ndihh,'unimportant dihedrals'
      print 17,ndihz,ndihh
      write(16,17) ndihz,ndihh
      if (ndihh.gt.maxtau) then
         print*, ' ***** Increase maxtau'
         stop
      endif
      ndiha=ndihz+ndihh
      if (ndiha.gt.maxome) then
         print*, ' ***** Increase maxome'
         stop
      endif
      do j=1,ndihh
         k=j+ndihz
         read(20,*) i1,i2,i3,i4
         write(29,43) i1,i2,i3,i4
         do i=1,ntors
            if ((i1.eq.ip(i) .and. i2.eq.jp(i) .and. i3.eq.kp(i)
     +           .and. i4.eq.lp(i)) .or.
     +           (i4.eq.ip(i) .and. i3.eq.jp(i) .and. i2.eq.kp(i)
     +           .and. i1.eq.lp(i))) then
               if (panm(i1)(1:1).eq.'H') then
                  iome(k)=i1
                  jome(k)=i2
                  kome(k)=i3
                  lome(k)=i4
               elseif (panm(i4)(1:1).eq.'H') then
                  iome(k)=i4
                  jome(k)=i3
                  kome(k)=i2
                  lome(k)=i1
               else
                  print*,' ***** No H atom in dihedral',i1,i2,i3,i4
                  stop
               endif
               goto 440
            endif
         enddo
         print*,' ***** Not found dihedral',i1,i2,i3,i4
         stop
  440    continue
      enddo
      do k=1,ndiha
         if (iome(k).le.0 .or. iome(k).gt.nrploc .or.
     +        jome(k).le.0 .or. jome(k).gt.nrploc .or.
     +        kome(k).le.0 .or. kome(k).gt.nrploc .or.
     +        lome(k).le.0 .or. lome(k).gt.nrploc) then
            print*,' ***** Error in atom number for dihedral',k
            stop
         endif
      enddo

C     Read information on reference atoms
      read(20,11,end=890,err=890) regel
      call nospac(29,regel)
      read(20,11,end=890,err=890) regel
      call nospac(29,regel)
      nref=nword(regel)
c      if (n.gt.3) n=3
      do i=1,nref
         call intstr(regel,i,ireftr(i))
      enddo
c      if (n.lt.3) ireftr(1)=-ireftr(1)
      do i=1,nref
         if (ireftr(i).lt.1) then
            print*,' Wrong reference atom',i,ireftr(i)
            stop
         endif
      enddo
      print 18,(ireftr(m),m=1,nref)
      write(16,18) (ireftr(m),m=1,nref)
      close(20)

C     Change units of parameters for XTINKER
      do i=1,nbond
         cb(i)=cb(i)/facb
      enddo
      do i=1,nhoek
         ct(i)=ct(i)/fact
      enddo
      do i=1,nimpr
         cq(i)=cq(i)/fact
      enddo
      do i=1,ntors
         cp(i)=cp(i)/fact
      enddo

      return

  810 print*,'Error on file XHARGES.XYZ'
      stop
  820 print*,'Error on file XHARGES.CSSR'
      stop
  840 print 34
      stop
  860 print 35
      stop
  890 print 36
      stop

   10 format(a5,i5)
   11 format(a80)
   17 format(i4,' conformation-determining dihedrals without H',
     +     ' and ',i4,' with H')
   18 format(' Reference atoms for rotational matrices:',/,4(3i4,3x))
   23 format(' ***** Torsional term with frequency zero!',2i5)
   25 format(i5,5x,a5)
   26 format
     +     (' Note redundancy between aromatic torsions and impropers!')
   28 format(' LEEST: Reading force field data from: ',a80)
   29 format(' LEEST: Reading topology data from: ',a80)
   34 format(' ***** No topology file IN.TP present')
   35 format(' ***** No force field file IN.FF present')
   36 format(' ***** Error in topology file')
   39 format(a,i5,'  code',i5,2x,2f12.4)
   42 format(i5,2x,a)
   43 format(4i5,3x,20i5)
   51 format(2i5,f12.1,f10.4,3x,a)
   52 format(3i5,f10.1,f10.3,3x,a)
   53 format(4i5,f10.3,f10.1,3x,a)
   54 format(4i5,i8,f10.4,i4,3x,a)
   74 format(7x,'Name',8x,'Type',9x,'Mass',6x,'Charge',5x,'Polar')
   77 format(i4,4x,a5,i6,4x,a5,f10.5,2f10.4)
      end
