      subroutine newchg(icoul,nat)
C     Reads and expands individual charges for a certain structure.

C     Input through parameters:
C        ICOUL = 2: use atom charges already found on CSSR file
C        ICOUL = 3: use atom charges from file CHARGES.CSSR
C        ICOUL = 4: reads monopoles from file CHARGES.XYZ
C                output from MOLDEN-ESP
C        ICOUL = 5: reads monopoles from fort.21 
C                output from MOLDEN or CONVERTMPOLES
C        NAT: number of charges to be expected
C        ***** Note: Atoms must be in the order of the topology file!

C     Input through common block /NONB/:
C        CGCOR(1..): atom charges already found on CSSR file

C     Output through common block /NONB/:
C        CG(1..): new atom charges found
C     Output through common block /NONP/:
C        IDO(1..): is (re)set to 1 (no exclusions) 

      implicit none
      include 'params.i'
      include 'cnonb.i'
      include 'cnonp.i'
      integer i,icoul,im,k1,k2,k3,m,n,nat,nchg,nword
      double precision a,facq,totabs,totcg,x,y,z
      character regel*80,atsy(maxat)*6

      do i=1,nat
         atsy(i)='  '
      enddo
      totabs=0
      nchg=0

      if (icoul.eq.2) then
C        Use atom charges already found on CSSR file
         do i=1,nat
            if (abs(cgcor(i)).gt.998) then
C              Incomplete information on CSSR file
               print 18
               write(16,18)
               stop
            endif
            totabs=totabs+abs(cgcor(i))
            if (totabs.lt.0.0001) then
C              Zero charges in CSSR file
               print 17
               write(16,17)
               stop
            else
C              Use charges already found in CSSR file
               print*,' Using charges from coordinate file'
               write(16,*)' Using charges from coordinate file'
               goto 920
            endif
         enddo
      endif

      if (icoul.eq.3) then
C        Charges from external CSSR file
         open(unit=24,file='charges.cssr',status='OLD',err=820)
         read(24,10,end=810,err=810) regel
         read(24,10,end=810,err=810) regel
         read(24,10,end=810,err=810) regel
         read(24,10,end=810,err=810) regel
  110    continue
         read(24,45,end=120,err=810) i,atsy(i),x,y,z,a
         nchg=nchg+1
         cgcor(nchg)=a
         totabs=totabs+abs(a)
         if (i.ne.nchg) then
            print*,' ***** Warning: i.ne.nchg in CSSR-file',i,nchg
         endif
         goto 110

  120    continue
         close(24)
         if (totabs.lt.0.0001) then
C           Zero charges in CSSR file
            print 17
            write(16,17)
            stop
         else
C           Use charges already found in CSSR file
            print*,' Using charges from external file'
            write(16,*)' Using charges from external file'
            goto 900
         endif
      endif

      if (icoul.eq.4) then
C        Charges from external XYZ file
         open(unit=24,file='charges.xyz',status='OLD',err=820)
         read(24,*) nchg
         read(24,10,end=810,err=810) regel
         print 10, regel
         i=0
  330    continue
         read(24,10,end=350,err=810) regel
         n=nword(regel)
         if (n.eq.1) then
            read(24,10,end=350,err=810) regel
c            print 10,regel
            goto 330
         endif
         i=i+1
         call realst(regel,n,cgcor(i))
         goto 330
  350    continue
         nchg=i
         close(24)
      endif

      if (icoul.eq.5) then
C        Charges from external non-CSSR file
         open(unit=21,status='OLD',err=820)
  150    continue
         read(21,10,end=900,err=800) regel
         if (regel(1:9).eq.'multipole') goto 400
         if (regel(6:21).eq.'ATOM NO.    TYPE') goto 200
         goto 150

  200    continue
C        Read input ESP.OUT from MOLDEN
         read(21,10,end=900,err=800) regel
  210    continue
         read(21,20,end=900,err=800) i,atsy(i),a
         if (i.eq.0) goto 900
         nchg=nchg+1
         cgcor(nchg)=a
         if (i.ne.nchg) then
            print*,' ***** Warning: i.ne.nchg in MOLDEN-output',i,nchg
         endif
         read(21,10,end=900,err=800) regel
         read(21,10,end=900,err=800) regel
         goto 210
         
  400    continue
C        Read input from CONVERTMPOLES
         read(regel,30,end=900,err=800) k1,k2,k3,a
         nchg=nchg+1
         cgcor(nchg)=a
         if (nchg.ne.abs(k1)) then
            print*,' ***** Warning: k1.ne.nchg in CONVERTMPOLES-output',
     +           k1,nchg
         endif
         print 30,k1,k2,k3,cgcor(nchg)
  410    continue
         read(21,10,end=900,err=800) regel
         if (regel(1:9).eq.'multipole') goto 400
         goto 410
      endif

  900 continue
C     New charges have been read. Check and expand.
      print*,nchg,' lines with charges found on external file.'
      write(16,*) nchg,' lines with charges found on external file.'
      if (nchg.ne.nat) then
         print*,'***** But',nat,' charges were expected'
         stop
      endif

  920 continue
      totcg=0
      totabs=0
      do i=1,nat
         totcg=totcg+cgcor(i)
         totabs=totabs+abs(cgcor(i))
      enddo
      if (totabs.lt.0.0001) then
         print*,' But total absolute charge zero, retain old values'
         write(16,*)'But total absolute charge zero, retain old values'
      else
C        Nonzero charges found; apply.
         do i=1,maxat
            ido(i)=1
         enddo
         facq=sqrt(332.0636d0*0.4184d0/epsil)
         if (abs(totcg).gt.0.0001d0) then
            print 19, totcg
            write(16,19) totcg
         endif
c         print*,' Charges from file as found   vs.   reset'
c         write(16,*)' Charges from file as found   vs.   reset'
         do i=1,nat
c            print 20,i,atsy(i),cgcor(i),cgcor(i)-totcg/nat
c            write(16,20) i,atsy(i),cgcor(i),cgcor(i)-totcg/nat
            cgcor(i)=cgcor(i)-totcg/nat
         enddo
         do m=1,maxat/nat
            im=(m-1)*nat
            do i=1,nat
               cg(i+im)=cgcor(i)*facq
            enddo
         enddo
      endif
      return

  800 print*,' ***** Error on unit 21'
      stop
  810 print*,' ***** Error on file CHARGES.CSSR'
      stop
  820 print*,' ***** No file with charges present!'
      stop

   10 format(a)
   17 format(' Total absolute charge from CSSR file zero')
   18 format(' No usable charge from CSSR file')
   19 format(' ***** Total charge not zero:',f8.3,'. Reset to zero!')
   20 format(i7,a11,f10.5,f14.5)
   30 format(14x,3i5,f20.5)
   45 format(i4,1x,a6,3f10.5,33x,f7.3)
      end
