      subroutine nonbep(ngr,nrp,enb,nf,der,nl,box,x,f,iprint)

C     Version of NONBE to include polarization.
C        R. Milot, December 1994 - February 1995
C        adding of damping B.P. van Eijck, February 1995
C     No virial calculation (NF=2 not implemented)
C     No effect of U2-inner on polarization included

C     References:
C     B.T. Thole, Chem. Phys. 59 (1981) 341-350.
C     D.N. Bernardo, Y. Ding, K. Krogh-Jespersen and R.M. Levy,
C          J. Phys. Chem. 98 (1994) 4180-4187

C     Input through common block /POLA/:
C        NPP: maximum number of iteration cycles
C        ALFPOL(1..): polarizabilities (in low-field limit)
C        PMAX(1..): maximum dipole moment (not used by Bernardo et al)
C        SCHAAL: scaling distance for damping in T-tensor (Thole: 1.662)
C        SIJ(1..,1..): SIJ(i,j) = {(alfpol(i)*alfpol(j)}^(1/6)
C        POLTOL: stop iteration if energy changes less than POLTOL.
C        GAMPOL: damping coefficient in zero field equation (Bernardo: 0.85)
C        NNN: damping exponent in zero field equation (Bernardo: 3)
C           NNN = 0: no such damping.
C        MMM: damping exponent in T-tensor (Bernardo: 32)
C           MMM = 0: no such damping.
C        NTE = 1: include 1-2 and 1-3 neighbors in electric field
C        NTTT = 1: include 1-2 and 1-3 neighbors in T-tensor

C     Output via common block /POLA/:
C        DIP(1..): induced dipole moments
C        NCYCP: number of iteration to make Epol converge
C        Epol: the polarization energy.

C     See NONBE for further description.
C     Note: for testing derivatives with procedure TEST use a very small
C           value of POLTOL with a sufficiently large value of NPP
C           to ensure that the energy is converged.

C     Calls subroutine KWALD.

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cvira.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cnbhl.i'
      include 'cinds.i'
      include 'cwald.i'
      include 'cpola.i'
      include 'chadd.i'

      double precision al6kw,atsqpi,ax,box,bx,by,
     +     c12def,c6def,c6r,cbu,cgdef,cgi,cgij,
     +     ch12,ch6,cx,cy,cz,de1,de2,de3,denuli,denulj,der,df,
     +     dip1,dip2,dip3,djp1,djp2,djp3,ebu,enb,enul,erfcc,
     +     ewco6,ewcoc,exa,f,facq,fact,factor,fi,fj,
     +     p,pi,pir,pjr,q,qi,qj,rcutp2,rij,rij1,rij2,rij3,rij6,rklrij,
     +     stuk2,vda,vdr,vel,velcc,vol,vp3vn,x,xd,xij,xij0,xsqc,xsqlj,
     +     yd,yij,yij0,zd,zij,zij0,
     +     bbb,edispa1,eel1,emacht,enb1,eold,euit,f1,f23,f3,pipj,rij5,
     +     rr,rs,rsm,rsn,w2,w3,wij,xx,yy
      integer i,iaci,iacj,ibox,icj,ii,ii0,ii3,iim,iimax,intra,iprint,
     +     is1,is2,is3,j,j3,jd,jj,jj0,jj3,jjm,jjmax,jpt,
     +     k,m,m1,mh,n,n1,n2,n3,nag,nbl,ncagt,nf,ngr,
     +     ni,ni3,nim,nj,nj3,njm,nl,nnj,nop3,nr3,nragt,nrp,nrpt,nsj

      dimension der(*),box(*),x(*),f(*),rij(3)
      logical skip
      dimension Euit(maxxco),enul(maxxco)

      facq=sqrt(332.0636d0*0.4184d0)
      ncagt=ncag*ngr
      nrpt=nrp*ngr
      if (iprint.gt.1) then
         print 40, ngr,nrp,nspgr,npp,cutoff,ebig,druk
         print 41, poltol,schaal,gampol,nnn,mmm,nte,nttt 
         if (noint) print*,
     +        '   No calculation of intramolecular nonbonded energy'
         if (buck) print*,'   Buckingham potential used'
         if (ddc) print*,
     +        '   Distance-dependent dielectric constant used'
         if (inner.gt.0) print*,'   U2-inner included'
         print 90,(box(m),m=1,6)
      endif
      do m=1,3
         if (box(m).lt.0.05) then
            print*,' ***** NONBEP: Cell axes less than 0.5 Angstrom.'
            write(16,*)
     +           ' ***** NONBEP: Cell axes less than 0.5 Angstrom.'
            enb=70000
            return
         endif
      enddo
      if (nl.eq.2) then
         do m=1,ndihh
            nplh(m)=0
         enddo
      endif
      if (nf.eq.2) then
         do m=1,3
            do n=1,3
               vira(m,n)=0
            enddo
         enddo
      endif

      npla=0
      rcutp2=cutoff**2
      nragt=(nspgr+1)*ncagt
      if (nragt.gt.maxcag) then
         print*,' ***** NONBEP: Increase maxcag',maxcag,nragt
         stop
      endif
      nr3=3*nrpt
      pi=4*atan(1.0d0)
      atsqpi=2*alphac/sqrt(pi)
      al6kw=alpha6**2
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      edc=0
      ed6=0
      ecc=0
      ec6=0
      ekc=0
      ek6=0
      eel=0
      edispa=0
      edispr=0
      ewcoc=0
      ewco6=0
      Epol=0
      do i=1,nr3
         enul(i)=0
      enddo

C     Calculate cell dipole moment and U2-inner; initialize derivatives
      uinner=0
      vol=ax*by*cz
      enb=druk*vol
c      vp3v=4*pi/(3.0d0*vol)
      vp3vn=finner/vol
C     This is twice the energy factor 2 pi / [3 V (nspgr+1)]
      do m=1,3
         p=0
         do i=1,nrpt
            p=p+x(3*i-3+m)*msom(m)*cg(i)
         enddo
         uinner=uinner-vp3vn*p*p/2
         pmol(m)=p
         if (inner.eq.0) p=0
         if (nf.gt.0) then
            do i=1,nrpt
               f(3*i-3+m)=vp3vn*msom(m)*cg(i)*p
            enddo
         endif
      enddo
      if (inner.ne.0 .and. iprint.gt.0) then
         print 93,(4.0745d0*pmol(m),m=1,3)
         write(16,93) (4.0745d0*pmol(m),m=1,3)
      endif
C     Decide whether to include U2-inner in EEL
      if (inner.ne.0) eel=uinner
C     Initialize field strengths with 4 pi dipole / [3 V]
ccc      do i=1,nrpt
ccc         do m=1,3
ccc            enul(3*i-3+m)=vp3v*pmol(m)
ccc         enddo
ccc      enddo
C     Iinitialize cell derivatives
      if (nf.eq.1) then
         do m=1,3
            der(m)=(eel-enb)/box(m)
            der(m+3)=0
         enddo
      endif
      if (iprint.gt.0) then
         print 86,uinner
         print 93,(pmol(m),m=1,3)
c         print 94,(vp3v*pmol(m),m=1,3)
         write(16,93) (pmol(m),m=1,3)
c         write(16,94) (vp3v*pmol(m),m=1,3)
      endif

      if (doew) then
C        Calculate reciprocal part of Ewald summation. This includes
C        the expansion with equivalent positions
         call kwald(nrpt,nf,der,box,x,f)
         if (ekc.gt.59999) then
            enb=60000
            return
         endif
      else
         if (nspgr.gt.0) then
C           Do expansion with equivalent positions
            do nop=1,nspgr
               nop3=3*nop-3
               do i=1,nrpt
                  j=3*i-3
                  k=j+nop*nr3
                  x(k+1)=msp(nop3+1)*x(j+1)+
     +                 tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
                  x(k+2)=msp(nop3+2)*x(j+2)+
     +                 tsp(nop3+2)*by+tsp(nop3+3)*cy
                  x(k+3)=msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
               enddo
            enddo
         endif
      endif
      if (iprint.gt.4) then
         print*,'Coordinates (after expansion):'
         do i=1,nrpt*(nspgr+1)
            print 91,i,(x(3*i-3+m),m=1,3)
         enddo
      endif

C     Calculate the centers of gravity of the charge groups
      i=0
      DO N=1,NRAGT
         DO M=1,3
            rij(M)=X(3*i+M)
         enddo
         i=i+1
         q=cg(i)
         nag=inc(n)-inc(n-1)
         DO J=2,NAG
            DO M=1,3
               RIJ(M)=RIJ(M)+X(3*i+M)
            enddo
            i=i+1
            q=q+cg(i)
         enddo
         DO M=1,3
            flr(3*n-3+m)=rij(m)/nag
         enddo
         m1=indmol(inc(n))
         if (iprint.gt.2) print 42,n,inc(n),m1,q/facq,
     +        (flr(3*n-3+m),m=1,3)
      enddo

      DO NI=1,ncagt
C        Start the next charge group NI in one asymmetric unit
         if (iprint.gt.8) print 81
         if (iprint.gt.3) print 80,ni
         NI3=3*NI-3
         ii0=inc(ni-1)+1
         iimax=inc(ni)
         DO NJ=ni,NRAGT
C           Start the next charge group NJ, including equivalent positions.
C           Make a list of translations needed to find all charge groups
C           combinations within CUTOFF. The translations are stored in array
C           ING in groups (N1, N2, N3) for every translation 1...NSJ.
            if (iprint.gt.9) print*,' Pair list for charge groups',ni,nj
            NJ3=3*NJ-3
            nsj=0
            xij0=flr(ni3+1)-flr(nj3+1)
            yij0=flr(ni3+2)-flr(nj3+2)
            zij0=flr(ni3+3)-flr(nj3+3)
C           Collect charge group distances within CUTOFF.
            n3=int((cutoff-zij0)/cz)
            zij=zij0+n3*cz
            n2=int((cutoff-yij0-n3*cy)/by)
            yij=yij0+n2*by+n3*cy
            n1=int((cutoff-xij0-n2*bx-n3*cx)/ax)
            xij=xij0+n1*ax+n2*bx+n3*cx
  350       continue
            rij2=xij**2+yij**2+zij**2
            if (rij2.lt.rcutp2) then
               nsj=nsj+1
               if (iprint.gt.9) print 155,nsj,ni,nj,n1,n2,n3,
     +              xij,yij,zij,sqrt(rij2)
               if (nsj.gt.maxnsj) then
                  print*,' ***** NONBEP: Increase mxlist',nsj
                  stop
               endif
               jpt=3*(nsj-1)
               ing(jpt+1)=n1
               ing(jpt+2)=n2
               ing(jpt+3)=n3
            endif
            n1=n1-1
            xij=xij-ax
  370       if (xij.lt.-cutoff) then
               n2=n2-1
               yij=yij-by
  380          if (yij.lt.-cutoff) then
                  n3=n3-1
                  zij=zij-cz
                  if (zij.lt.-cutoff) goto 390
                  n2=int((cutoff-yij0-n3*cy)/by)
                  yij=yij0+n2*by+n3*cy
                  goto 380
               endif
               n1=int((cutoff-xij0-n2*bx-n3*cx)/ax)
               xij=xij0+n1*ax+n2*bx+n3*cx
               goto 370
            endif
            goto 350
  390       continue
C           End of making list ING(1..NSJ)

C           Start the actual calculation for charge groups NI and NJ
            velcc=0
            IF (nsj.EQ.0) GOTO 480
            ICJ=(NJ-1)/NCAGT
C           ICJ is the number of the equivalent group, 0..NSPGR.
            NNJ=NJ-NCAGT*ICJ
            jj0=inc(nnj-1)+1
            jjmax=inc(nnj)
            if (iprint.gt.4) print 152,ni,ii0,iimax,nj,jj0,jjmax,icj

            do ii=ii0,iimax
C              Start the next atom II, if it is within charge group NI
C              and allowed by IDO
               nim=indmol(ii)
               iim=mod(ii-1,nrp)+1
               if (ido(ii).eq.0) goto 475
               CGI=CG(II)
               IACI=IAC(II)
               II3=3*ii-3

               do jj=jj0,jjmax
C                 Start the next atom JJ, if it is within charge group NJ
C                 and allowed by IDO.
C                 JJ0 and JJMAX were reduced to the non-expanded group
                  njm=indmol(jj+nrpt*icj)
                  jjm=mod(jj-1,nrp)+1
C                 NIM and NJM are molecule numbers, negative for virtual atoms.
C                 IIM and JJM indicate the atom numbers in the main group.
                  if (iprint.gt.7) print 92,ni,nj,ii,jj,jj+nrpt*icj,
     +                 nim,njm,iim,jjm,ido(jj)
                  if (jj.lt.ii .or. ido(jj).eq.0) goto 470
                  factor=1
                  if (jj.eq.ii) factor=0.5d0
                  jj3=3*jj-3
                  jd=3*icj
                  j3=jj3+nrpt*jd
                  if (jd.eq.0) then
                     is1=1
                     is2=1
                     is3=1
                     de1=0
                     de2=0
                     de3=0
                  else
                     is1=msp(jd-2)
                     is2=msp(jd-1)
                     is3=msp(jd)
                     de1=-tsp(jd-2)
                     de2=-tsp(jd-1)
                     de3=-tsp(jd)
                  endif
                  iacj=iac(jj)
                  c6def=c6(iaci,iacj)*factor
                  c12def=c12(iaci,iacj)*factor
                  cgdef=cgi*cg(jj)*factor
                  if (buck) then
                     cbu=c12def
                     ebu=cs12(iaci,iacj)
                     c12def=0
                  endif
                  xij0=x(ii3+1)-x(j3+1)
                  yij0=x(ii3+2)-x(j3+2)
                  zij0=x(ii3+3)-x(j3+3)

                  do ibox=1,nsj
C                    Start the next cell translation from ING
                     ch6=c6def
                     ch12=c12def
                     cgij=cgdef
                     intra=0
                     skip=.false.
                     n1=ing((ibox-1)*3+1)
                     n2=ing((ibox-1)*3+2)
                     n3=ing((ibox-1)*3+3)
                     if (icj.eq.0 .and. 
     +                    n1.eq.0 .and. n2.eq.0 .and. n3.eq.0) then
C                       Possibly an intramolecular distance
                        if (abs(nim).eq.abs(njm)) then
                           intra=1
                           if (njm.gt.0) intra=iexcl(iim,jjm)
                        endif
                        if (intra.eq.2) then
C                          Change and/or scale 1..4 interaction parameters
                           cgij=cgij*sc14c
                           if (.not. buck) then
                              ch12=cs12(iaci,iacj)*factor*sc14lj
                              ch6=cs6(iaci,iacj)*factor*sc14lj
                           endif
                        endif
                        skip=(intra.eq.1 .or. (noint.and.intra.ne.0))
                     endif
C                    Skip interaction for excluded neighbors
C                    Skip intramolecular interaction if NOINT is true.
                     if (.not. doew .and. skip) goto 450
CCC Is dit goed? Dan geen veldbijdrage van 1..2 en 1..3 intercaties
                     xij=xij0+n1*ax+n2*bx+n3*cx
                     yij=yij0+n2*by+n3*cy
                     zij=zij0+n3*cz
                     rij2=xij**2+yij**2+zij**2
                     if (n1.eq.0 .and. n2.eq.0 .and. n3.eq.0 .and.
     +                    ii.eq.jj .and. icj.eq.0) goto 450
                     rij1=sqrt(rij2)
                     if (rij1.lt.1.0d-6) then
                        print*,' ***** Unacceptably small distance ',
     +                       'in NONBE:',ii,jj,rij1
                        enb=80000
                        return
                     endif
                     if (rij1.lt.0.05 .and. (abs(nim).ne.abs(njm))) then
                        if (iprint.gt.1) print*,
     +                       '***** NONBE: short distance: ',ii,jj,rij1
                        enb=80000
                        return
                     endif

C                    Calculate Coulomb energy
C                    Distance dependent dielectric constant if DDC=TRUE.
                     vel=cgij/rij1
                     if (skip) vel=0
                     if (nf.gt.0) df=-vel/rij2
                     if (ddc) then
                        vel=vel/rij1
                        df=2*df/rij1
                     endif
                     if (doewc) then
C                       Calculate Ewald correction
                        rklrij=alphac*rij1
                        xsqc=rklrij*rklrij
                        ewcoc=cgdef*(erfcc(rklrij)-1)/rij1
                        if (nf.gt.0) df=df-(ewcoc+cgdef*atsqpi
     +                       *exp(-xsqc))/rij2
                     endif

C                    Calculate attractive r^(-6) terms
                     rij6=rij2*rij2*rij2
                     vda=ch6/rij6
                     if (skip) vda=0
                     if (nf.gt.0) df=df+6*vda/rij2
                     if (doew6) then
C                       Calculate Ewald correction
                        c6r=c6def/rij6
                        xsqlj=al6kw*rij2
                        exa=exp(-xsqlj)
                        fact = (1 + xsqlj * (1 + xsqlj/2)) * exa - 1
                        ewco6=c6r*fact
                        if (nf.gt.0) df=df+(c6r/rij2)*
     +                       (6*fact+exa*xsqlj*xsqlj*xsqlj)
                     endif

C                    Calculate R^(-12) or exponential repulsion
                     if (skip) then
                        vdr=0
                     else
                        if (buck) then
                           vdr=cbu*exp(-ebu*rij1)
                           if (nf.gt.0) df=df-ebu*vdr/rij1
                        else
                           vdr=ch12/(rij6*rij6)
                           if (nf.gt.0) df=df-12*vdr/rij2
                        endif
                     endif

c  400                continue
C                    Throw away contributions smaller than TOLE
                     if (abs(vel+ewcoc).lt.tole .and.
     +                    abs(vda+ewco6).lt.tole .and.
     +                    abs(vdr).lt.tole .and. npp.eq.0) goto 450
c                        print 555,tole,vel,vda,vel+ewcoc,vda+ewco6,vdr
c 555                    format(10f10.4)
                     eel=eel+vel
                     edispa=edispa-vda
                     edispr=edispr+vdr
                     edc=edc+ewcoc
                     ed6=ed6-ewco6
                     if (edispr.gt.ebig) then
                        if (iprint.gt.1) print*,
     +                       'NONBEP: energy > EBIG',rij1,vdr
                        enb=90000
                        return
                     endif
c                    if (abs(eel).gt.ebig) then
c                       print*,'eel in nonbep te groot ',vel,eel
c                       enb=80000
c                       return
c                    endif
CCCCC     Dit komt wel voor, terwijl toch de som goed is! Voldoende precisie?
                     if (iprint.gt.8) then
                        if (buck) then
                           print 88,n1,n2,n3,xij,yij,zij,rij1,
     +                          cgij,ch6,cbu,ebu,vel,-vda,vdr
                        else
                           print 89,n1,n2,n3,xij,yij,zij,rij1,
     +                          cgij,ch6,ch12,vel,-vda,vdr
                        endif
                     endif
                     velcc=velcc+vel

C                    Calculate electric field for polarization calculation
C                    Use damping if NNN>0.
                     rs=rij1/(sij(iim,jjm)*schaal)
                     if (nnn.gt.0) then
                        f1=1-exp(-gampol*(rs**nnn))
                     else
                        f1=1
                     endif
                     rij3=rij1*rij2
                     if (skip .and. nte.eq.0) goto 2300
                     denuli=cg(jj)*f1/rij3
                     denulj=cg(ii)*f1/rij3
                     enul(ii3+1)=enul(ii3+1)+xij*denuli
                     enul(ii3+2)=enul(ii3+2)+yij*denuli
                     enul(ii3+3)=enul(ii3+3)+zij*denuli
                     if (ii3.ne.jj3) then
                        enul(jj3+1)=enul(jj3+1)-xij*denulj*is1
                        enul(jj3+2)=enul(jj3+2)-yij*denulj*is2
                        enul(jj3+3)=enul(jj3+3)-zij*denulj*is3
                     endif
 2300                continue

C                    Increase pair list number
                     npla=npla+1
                     if (npla.gt.mxlist) then
                        print*, ' ***** NONBEP: Increase mxlist',npla
                        stop
                     endif

C                    Calculate T matrix, with damping if MMM>0.
                     if (skip .and. nttt.eq.0) goto 2400
                     if (mmm.gt.0 .and. rs.lt.1) then
                        wij=rs+(1-rs**mmm)/mmm
                        f23=3*(wij**4)
                        f3=4*(wij**3)-f23
                     else
                        f23=3
                        f3=1
                     endif
c                  if (rij1.lt.0.18) print*,rij1
                     if (npla.gt.maxap) then
                        print*, ' ***** NONBEP: Increase maxap',npla
                        stop
                     endif
                     T11(npla)=((f23*xij*xij/rij2)-f3)/rij3
                     T22(npla)=((f23*yij*yij/rij2)-f3)/rij3
                     T33(npla)=((f23*zij*zij/rij2)-f3)/rij3
                     T12(npla)=(f23*xij*yij/rij2)/rij3
                     T13(npla)=(f23*xij*zij/rij2)/rij3
                     T23(npla)=(f23*yij*zij/rij2)/rij3
 2400                continue
                     r1(npla)=xij
                     r2(npla)=yij
                     r3(npla)=zij
c                     rabs(npla)=rij1

C                    Make pair list for LIST (NL=1) and for polarization
c                     if (nl.eq.1) then
                     ipla1(npla)=ii3
                     if (skip) ipla1(npla)=-ii3-1
C                       Negative IPLA1 markes an atom pair to be skipped
                     ipla2(npla)=jj3
c                        shift1(npla)=n1+de1
c                        shift2(npla)=n2+de2
c                        shift3(npla)=n3+de3
                     ishift1(npla)=nint(2*(n1+de1))
                     ishift2(npla)=nint(2*(n2+de2))
                     ishift3(npla)=nint(2*(n3+de3))
                     isig1(npla)=is1
                     isig2(npla)=is2
                     isig3(npla)=is3
                     plj6(npla)=ch6
                     plj12(npla)=ch12
                     if (buck) then
                        plj12(npla)=cbu
                        pexp(npla)=ebu
                     endif
                     pcg(npla)=cgij
                     if (doew6) plj6ew(npla)=c6def
                     if (doewc) pcgew(npla)=cgdef
c                        qi(npla)=cg(ii)
c                        qj(npla)=cg(jj)
c                     endif

C                    Make hydrogen atom pair list for LISTH (NL=2).
C                    No Ewald summation in this case
                     if (nl.eq.2 .and. .not. skip) then
                        do mh=1,ndihh
                           n=3*iome(ndihz+mh)-3
                           if (ii3.eq.n .or. jj3.eq.n) then
                              nplh(mh)=nplh(mh)+1
                              nbl=nplh(mh)
                              if (nbl.gt.maxh) then
                                 print*, ' NONBEP: Increase maxh'
                                 stop
                              endif
                              iplh1(mh,nbl)=ii3
                              iplh2(mh,nbl)=jj3
                              shifh1(mh,nbl)=n1+de1
                              shifh2(mh,nbl)=n2+de2
                              shifh3(mh,nbl)=n3+de3
                              isigh1(mh,nbl)=is1
                              isigh2(mh,nbl)=is2
                              isigh3(mh,nbl)=is3
                              pcgh(mh,nbl)=cgij
                           endif
                        enddo
                     endif

                     if (nf.gt.0) then
C                       Calculate derivatives
                        xd=xij*df
                        yd=yij*df
                        zd=zij*df
                        f(ii3+1)=f(ii3+1)-xd
                        f(ii3+2)=f(ii3+2)-yd
                        f(ii3+3)=f(ii3+3)-zd
                        f(jj3+1)=f(jj3+1)+xd*is1
                        f(jj3+2)=f(jj3+2)+yd*is2
                        f(jj3+3)=f(jj3+3)+zd*is3
                        if (nf.eq.1) then
C                          Calculate derivatives to cell axes
                           der(1)=der(1)-xd*(de1+n1)
                           der(2)=der(2)-yd*(de2+n2)
                           der(3)=der(3)-zd*(de3+n3)
                           der(4)=der(4)-xd*(de2+n2)
                           der(5)=der(5)-xd*(de3+n3)
                           der(6)=der(6)-yd*(de3+n3)
c                           print 8888,ii3,jj3,de1+n1,de2+n2,de3+n3,
c     +                          xij,yij,zij,
c     +                          xd*(de2+n2),xd*(de3+n3),yd*(de3+n3)
c 8888                       format(2i3,5x,3f6.2,3f10.5,5x,3f10.5)
                        endif
                        if (nf.eq.2) then
C                          Calculate virial
                           rij(1)=xij
                           rij(2)=yij
                           rij(3)=zij
                           do m=1,3
                              do k=1,3
                                 vira(m,k)=vira(m,k)+rij(m)*df*rij(k)
                              enddo
                           enddo
                        endif
                     endif
  450                continue
                  enddo
  470             continue
               enddo
  475          continue
            enddo
  480       continue
            if (iprint.gt.4) then
               print 142,ni,nj,velcc
               print 141,eel
            endif
c  490       continue
         enddo
      enddo

C     Start calculation of polarisation
      do i=1,nr3
         j=(i-1)/3+1
         xx=abs(Enul(i)*alfpol(j)/pmax(j))
         if (xx.lt.0.01) then
            dip(i)=alfpol(j)*Enul(i)*(1-xx/2)
         else
            dip(i)=alfpol(j)*Enul(i)*(1-exp(-xx))/xx
         endif
         Epol=Epol-dip(i)*Enul(i)/2
C        This is an extension of the standard equation:
C               dip(i)=alfpol(j)*Enul(i)
C        for using a maximum allowed dipole moment PMAX.
      enddo
c      print*,'Enul'
c      print*,(enul(i),i=1,nr3)
c      print*,'Dip'
c      print*,(dip(i),i=1,nr3)
      if (npp.eq.1) goto 660
C     dipole field omitted if NPP=1
      if (iprint.gt.1) then
         print*,' Results before polarization loop:'
         print*,' npla = ',npla
         eel1=eel+ecc+ekc+edc
         edispa1=edispa+ec6+ek6+ed6
         enb1=enb+eel1+edispa1+edispr+Epol
         print 96, enb1,eel1,edispa1,edispr,uinner,Epol
         print 97, (der(i),i=1,6)
         print 98, (vira(1,m)/2,m=1,3)
         print 98, (vira(2,m)/2,m=1,3)
         print 98, (vira(3,m)/2,m=1,3)
         print 99, (f(i),i=1,nr3)
      endif

      ncycp=0
C     Next cycle in polarization loop
  620 continue
      ncycp=ncycp+1
      do i=1,nr3
         Euit(i)=Enul(i)
      enddo
      Eold=Epol
      Epol=0
      do n=1,npla
         i=ipla1(n)
         skip=(i.lt.0)
         if (skip .and. nttt.eq.0) goto 640
         if (skip) i=-(i+1)
         j=ipla2(n)
         is1=isig1(n)
         is2=isig2(n)
         is3=isig3(n)
         Euit(i+1)=Euit(i+1)+(T11(n)*dip(j+1)*is1
     +        +T12(n)*dip(j+2)*is2+T13(n)*dip(j+3)*is3)
         Euit(i+2)=Euit(i+2)+(T12(n)*dip(j+1)*is1
     +        +T22(n)*dip(j+2)*is2+T23(n)*dip(j+3)*is3)
         Euit(i+3)=Euit(i+3)+(T13(n)*dip(j+1)*is1
     +        +T23(n)*dip(j+2)*is2+T33(n)*dip(j+3)*is3)
         if (i.ne.j) then
            Euit(j+1)=Euit(j+1)+is1*(T11(n)*dip(i+1)
     +           +T12(n)*dip(i+2)+T13(n)*dip(i+3))
            Euit(j+2)=Euit(j+2)+is2*(T12(n)*dip(i+1)
     +           +T22(n)*dip(i+2)+T23(n)*dip(i+3))
            Euit(j+3)=Euit(j+3)+is3*(T13(n)*dip(i+1)
     +           +T23(n)*dip(i+2)+T33(n)*dip(i+3))
         endif
  640    continue
      enddo
      do i=1,nr3
C     Nonlinear equations with PMAX:
         j=(i-1)/3+1
         xx=abs(Euit(i)*alfpol(j)/pmax(j))
         if (xx.lt.0.01) then
            yy=-xx/12
            dip(i)=alfpol(j)*Euit(i)*(1-xx/2)
         else
            rr=(1-exp(-xx))/xx
            yy=0.5d0+(rr-1)/(rr*xx)
            dip(i)=alfpol(j)*Euit(i)*rr
         endif
C        Iterate first, then update dipole moments.
         Epol=Epol+dip(i)*(-Enul(i)/2+Euit(i)*yy)
C        This is an extension of the standard equations:
C            Epol=Epol-dip(i)*Enul(i)/2
C            dip(i)=alfpol((i-1)/3+1)*Euit(i)
C        when using a maximum allowed dipole moment PMAX.
      enddo
      print*,'ncyc=',ncycp,' epol = ',epol,eold
      if (ncycp.lt.npp .and. abs(Eold-Epol).gt.poltol) goto 620
  660 continue
c      print*,' Dipoles NONBEP:'
c      do 670 i=1,5
c         print*,i,enul(i),dip(i)
c 670  continue


C     Calculate intermolecular polarization forces
CCCCC (virial yet to be done!!)
CCCCC Ook Uinner nog in veld opnemen
      if (nf.eq.0) goto 700
      do n=1,npla
c         print*,n,r1(n),r2(n),r3(n)
         i=ipla1(n)
         skip=(i.lt.0)
         if (skip .and. nte.eq.0 .and. nttt.eq.0) goto 690
         if (skip) i=-(i+1)
         j=ipla2(n)
         ii=i/3+1
         jj=j/3+1
         qi=cg(ii)
         qj=cg(jj)
         is1=isig1(n)
         is2=isig2(n)
         is3=isig3(n)
         dip1=dip(i+1)
         dip2=dip(i+2)
         dip3=dip(i+3)
         djp1=dip(j+1)*is1
         djp2=dip(j+2)*is2
         djp3=dip(j+3)*is3
c         rij1=rabs(n)
c         rij2=rij1*rij1
         rij2=r1(n)**2+r2(n)**2+r3(n)**2
         rij1=sqrt(rij2)
         rij3=rij1*rij2
         rij5=rij2*rij3
         ii=i/3+1
         jj=j/3+1
         iim=mod(ii-1,nrp)+1
         jjm=mod(jj-1,nrp)+1
         rs=rij1/(sij(iim,jjm)*schaal)
c         print*,iim,jjm,sij(iim,jjm)
         if (skip .and. nte.eq.0) then
            f1=0
            bbb=0
         else
            if (nnn.gt.0 .and. abs(rs).gt.1.0d-10) then
               rsn=gampol*rs**nnn
               emacht=exp(-rsn)
               f1=1-emacht
               bbb=3*f1-nnn*rsn*emacht
            else
               f1=1
               bbb=3
            endif
         endif
c         print*,f1,bbb,rsn,nnn,gampol,rs,schaal
c         print*,'*****',n,r1(n),r2(n),r3(n)
         pjr=djp1*r1(n)+djp2*r2(n)+djp3*r3(n)
         pir=dip1*r1(n)+dip2*r2(n)+dip3*r3(n)
         pipj=dip1*djp1+dip2*djp2+dip3*djp3

         if (skip .and. nttt.eq.0) then
            stuk2=0
            f23=0
            f3=0
         else
            if (mmm.gt.0 .and. rs.lt.1) then
               rsm=rs**mmm
               wij=rs+(1-rsm)/mmm
               w2=wij*wij
               w3=wij*w2
               f23=3*w2*w2
               f3=4*w3-f23
               stuk2=-12*(pipj*(w2-w3)-(w3*pir*pjr/rij2))*(rs-rsm)
c               print*,i,j,sij(ii,jj)*schaal,rij1,wij,stuk2
            else
               f23=3
               f3=1
               stuk2=0
            endif
         endif

         stuk2=((-qj*pir+qi*pjr)*bbb+3*pipj*f3
     +        -(5*f23*pjr*pir/rij2)+stuk2)/rij5
         fj=qj*f1/rij3+pjr*f23/rij5
         fi=qi*f1/rij3-pir*f23/rij5
         xd=dip1*fj-djp1*fi+stuk2*r1(n)
         yd=dip2*fj-djp2*fi+stuk2*r2(n)
         zd=dip3*fj-djp3*fi+stuk2*r3(n)

         if (i.eq.j) then
            xd=xd/2
            yd=yd/2
            zd=zd/2
         endif
         if (nf.eq.1) then
            der(1)=der(1)+xd*ishift1(n)/2
            der(2)=der(2)+yd*ishift2(n)/2
            der(3)=der(3)+zd*ishift3(n)/2
            der(4)=der(4)+xd*ishift2(n)/2
            der(5)=der(5)+xd*ishift3(n)/2
            der(6)=der(6)+yd*ishift3(n)/2
         endif
         f(i+1)=f(i+1)+xd
         f(i+2)=f(i+2)+yd
         f(i+3)=f(i+3)+zd
         f(j+1)=f(j+1)-xd*is1
         f(j+2)=f(j+2)-yd*is2
         f(j+3)=f(j+3)-zd*is3
  690    continue
      enddo
  700 continue

      if (doewc) eel=eel+ecc+ekc+edc
      if (doew6) edispa=edispa+ec6+ek6+ed6
      enb=enb+eel+edispa+edispr+Epol
      if (nf.eq.2) then
         do m=1,3
            do n=1,3
               vira(m,n)=vira(m,n)/2
            enddo
         enddo
      endif
      if (iprint.gt.1) then
         print*,' Results after polarization loop:'
         print 96, enb,eel,edispa,edispr,uinner,Epol
         print 97, (der(i),i=1,6)
         print 98, (vira(1,m),m=1,3)
         print 98, (vira(2,m),m=1,3)
         print 98, (vira(3,m),m=1,3)
         print 99, (f(i),i=1,nr3)
      endif

   40 format(' Start NONBEP: ngr, nrp, nspgr, npp, cutoff, ebig,', 
     +     ' pressure: ',4i5,f8.2,2e12.4)
   41 format('     poltol schaal gamma nnn mmm nte nttt',3f10.6,4i4)
   42 format(' Charge group ',i3,'  last atom',i4,'   molecule',
     +     i4,'   charge',f10.5,4x,4f10.5)
   80 format(/,' Start calculation for charge group',i4)
   81 format(12x,'Translation     dx      dy      dz      r',8x,'cgij',
     +     8x,'ch6',10x,'cbu',6x,'ebu',10x,'coul    disp     rep')
   86 format(' U2-inner: ',3f8.3)
   88 format (12x,3i3,2x,4f8.3,3x,f8.3,2e13.4,f8.3,4x,8f8.3)
   89 format (12x,3i3,2x,4f8.3,3x,f8.3,2e13.4,12x,8f8.3)
   90 format(' box: ',6f10.4)
   91 format(i5,3f9.4,6x,9f8.2)
   92 format(7x,'charge groups NI, NJ:',2i4,8x,
     +     'atoms II, JJ, JJ-exp:',3i4,8x,'molecules:',6i4)
   93 format(' Dipole moment (* 4.0745 for Debye):  ',3f8.3)
c   94 format(' Creating field strength:             ',3f8.3)
c   95 format('  Induced dipole moments',18x,'from field strengths')
   96 format(' Energy NONBE: ',6f9.3)
   97 format(' - Der:  ',6f12.3)
   98 format(' Virial: ',3f12.3)
   99 format(9f8.1)
  141 format(6x,' Cumulative results so far:',f12.5)
  142 format(6x,' Contribution from charge groups',2i4,f12.5)
  152 format('    Starting charge group combination',i4,' range',
     +     2i4,'  and',i4,' range', 2i4,'  with eq. pos.',8i4)
  155 format(i5,3x,2i5,3x,3i5,10f8.3)
      end
