      subroutine opth(nang,euit,iprint,par)

C     B.P. van Eijck, januari 1995.
C     Rotates hydroxyl H atoms and calculates nonbonded energies.
C     It is supposed that the Van der Waals radius of a hydrogen atom
C     is zero, so only the Coulomb energy changes upon internal rotation
C     of a hydroxyl group. This Coulomb energy is calculated by LISTH.
C     The algorithm is very efficient and, therefore, hardly understandable.

C     Input through parameters:
C        NANG: largest occurring number of rotations required to rotate
C              any hydroxyl group through 360 degrees.
C        PAR(1..12): crystallographic parameters to be refined
C        IPRINT regulates output to unit 16 and to screen.

C     Input through common blocks:
C        /PARS/:
C           NRP: number of atoms
C           XR(1..3*NRP): reference coordinates for the free molecule
C           CEL(1..12): ax, by, cz, bx, cx, cy, X, Y, Z, phi, theta, psi
C           IREF(1..12)=0: take BOX from CEL
C                      =1: take BOX from PAR
C        /INDS/ gives hydroxyl dihedral information:
C           NDIHH: number of hydroxyl groups to be rotated
C           NDIHZ: number of dihedral that preceeds the first
C                  hydroxyl dihedral
C           VN(1..NDIHH), NTAL(1..NDIHH) define the potential energy:
C              each dihedral contributes VN(1+cos(NTAL*PAR))
C           IOME, JOME, KOME, NDIHZ: specify the hydroxyl dihedral angles,
C              which are initially rotated about an angle PAR.
C              See procedure INTROT for details.
C           NSTEPH(1..NDIHH): number of steps for rotation over 360 degrees

C     Output through parameters:
C        EUIT(KK): energy values for all combinations of dihedral angles.
C        The index KK is a combination of values ITAU in the
C        NANG-based system:
C           KK = ITAU(1) + ITAU(2)*NANG + ITAU(3)*NANG**2 + .....
C        where ITAU(I) defines the dihedral angle I:
C        dihedral I = starting angle (from PAR) + ITAU(i)*360/NSTEPH(I).

C     Output through common block /PARS/:
C        X(1..3*NRP): coordinates calculated from PAR, CEL and XR.

C     Calls subroutines INTROT, NONBE, LISTH and function DIHAN.

      implicit none
      include 'params.i'
      include 'cinds.i'
      include 'cpars.i'
      integer i,i3,index,indih,iprint,itau,j,j3,jxx,k,k3,kk,kpar,kxx,
     +     m,n,nang
      double precision arot,box,brot,ca,cb,cg,co,cum,dihan,din,
     +     eh,eh1,ein,enb,euit,euler,omega,par,pi2,ref,sa,sb,scr,sg,
     +     shift,si,xrot
      dimension itau(maxtau),shift(maxtau,0:maxint,3),arot(3),brot(3),
     +     ref(maxtau),cum(0:maxtau),index(maxtau)
      dimension par(*),euit(*),xrot(3*maxnrp),euler(3,3),box(12),scr(12)

      do i=1,ndihh
         index(i)=3*iome(ndihz+i)-3
      enddo
      if (nang.le.1) return
      kpar=0
      do i=1,12
         if (iref(i).gt.0) then
            kpar=kpar+1
            box(i)=par(kpar)
         else
            box(i)=cel(i)
         endif
      enddo

      do i=1,nrp
         do m=1,3
            xrot(3*i-3+m)=xr(3*i-3+m)
         enddo
      enddo

      ca=cos(box(10))
      sa=sin(box(10))
      cb=cos(box(11))
      sb=sin(box(11))
      cg=cos(box(12))
      sg=sin(box(12))
      euler(1,1)= cg*ca-sg*sa*cb
      euler(1,2)= cg*sa+sg*ca*cb
      euler(1,3)= sg*sb
      euler(2,1)=-sg*ca-cg*sa*cb
      euler(2,2)=-sg*sa+cg*ca*cb
      euler(2,3)= cg*sb
      euler(3,1)= sa*sb
      euler(3,2)=-ca*sb
      euler(3,3)= cb
      do i=1,nrp
         j=3*(i-1)
         do m=1,3
            x(j+m)=box(m+6)
            do n=1,3
               x(j+m)=x(j+m)+euler(m,n)*xrot(j+n)
            enddo
         enddo
      enddo

      call nonbe(1,nrp,ein,0,scr,2,box,x,xrot,0)
      if (iprint.gt.2) then
         print*,' OPTH, after NONBE: ',ein
      endif

      do i=1,nrp
         do m=1,3
            xrot(3*i-3+m)=x(3*i-3+m)
         enddo
      enddo

      pi2=8*atan(1.0d0)
      do i=1,ndihh
         do m=1,3
            shift(i,0,m)=0
         enddo
         i3=3*iome(ndihz+i)-3
         j3=3*jome(ndihz+i)-3
         k3=3*kome(ndihz+i)-3
         do k=1,nsteph(i)-1
            omega=k*pi2/nsteph(i)
            si=sin(omega)
            co=cos(omega)-1
            call introt(i3,j3,k3,x,arot,brot)
            do m=1,3
               shift(i,k,m)=si*brot(m)+co*arot(m)
            enddo
         enddo
      enddo

      enb=ein
      eh=ein
      cum(0)=ein
      indih=1
      itau(1)=0
 500  continue
      kxx=itau(indih)
      do m=1,3
         xrot(index(indih)+m)=x(index(indih)+m)+shift(indih,kxx,m)
         do jxx=indih+1,ndihh
            xrot(index(jxx)+m)=x(index(jxx)+m)
         enddo
      enddo
      call listh(enb,box,xrot,indih)
c      print*,' OPTH, after LISTH: ',enb

      if (kxx.eq.0) then
         ref(indih)=enb
         cum(indih)=cum(indih-1)
      else
         eh=cum(indih-1)+enb-ref(indih)
         cum(indih)=eh
      endif

C     CUM(I) is the cumulative contribution up to and including dihedral I.
C     REF(I) is the zero level of dihedral I.

      if (indih.eq.ndihh) then
         eh1=eh
         kk=1
         m=1
         do i=1,ndihh
            din=ntal(i)*dihan(i+ndihz,xrot)
            eh1=eh1+Vn(i)*(1+cos(din))
            kk=kk+itau(i)*m
            m=m*nang
c            print*,'OPTH',i,Vn(i),ntal(i),57.29*dihan(i+ndihz,xrot),
c     +           Vn(i)*(1+cos(din))
         enddo
         euit(kk)=eh1
         if (iprint.gt.2) then
            print 90, kk,eh,eh1,(itau(m),m=1,ndihh)
c            write(16,90) kk,eh,eh1,(itau(m),m=1,ndihh)
         endif
 540     itau(indih)=itau(indih)+1
         if (itau(indih).lt.nsteph(indih)) goto 500
         indih=indih-1
         if (indih.eq.0) return
         goto 540
      endif
      indih=indih+1
      itau(indih)=0
      goto 500
 90   format(i6,f13.3,f10.3,3x,15i3)
      end





      subroutine listh(eel,box,x,nh)

C     B.P. van Eijck, januari 1995
C     Calculates EEL, the contribution of hydrogen atom number NH to the
C     Coulomb energy. The calculation is done using a pair list prepared
C     by NONBE (option NL=2), stored in common block /NBHL/.

C     Input through parameters:
C     BOX(1..6): ax, by, cz, bx, cx, cy
C     X(1..): atom coordinates. X must be big enough to accomodate
C        all symmetry-expanded coordinates.
C     NH is the number of the hydrogen atom to be considered.

C     Input through common block:
C     /NBHL/ gives pair list information:
C        NPLH: numbers of atoms that form a pair with NH
C        IPLH1, IPLH2: coordinates are located in X(IPLH+M) (M=1,2,3)
C        PCGH: Coulomb coefficicient (the product of the two charges)
C        SHIFH1..3: fractional shifts required for symmetry operation
C        ISIGH1..3: signs for required symmetry operation

C     Output through parameter:
C     EEL is the Coulomb contribution from hydrogen atom NH.

      implicit none
      include 'params.i'
      include 'cnbhl.i'
      integer i,is1,is2,is3,j,n,ndist,nh
      double precision ax,box,bx,by,cx,cy,cz,eel,rij2,s1,s2,s3,
     +     x,xij,yij,zij
      dimension box(*),x(*)

      eel=0
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)

      ndist=nplh(nh)
      do n=1,ndist
         i=iplh1(nh,n)
         j=iplh2(nh,n)
         s1=shifh1(nh,n)
         s2=shifh2(nh,n)
         s3=shifh3(nh,n)
         is1=isigh1(nh,n)
         is2=isigh2(nh,n)
         is3=isigh3(nh,n)

         xij=x(i+1)-is1*x(j+1)+s1*ax+s2*bx+s3*cx
         yij=x(i+2)-is2*x(j+2)+s2*by+s3*cy
         zij=x(i+3)-is3*x(j+3)+s3*cz
         rij2=xij**2+yij**2+zij**2
c        if (rij2.lt.1.0e-6) goto 300
         eel=eel+pcgh(nh,n)/sqrt(rij2)
      enddo

c      write(94,297) ' Listh: ',eel
c      write(94,298) (der(i),i=1,6)
c      write(94,299) (f(i),i=1,nr3)
c  297 format(a20,3f12.6)
c  298 format(' Der: ',6f12.6)
c  299 format(6f12.6)
      end
