      program pack1

C     First stage of crystal packing prediction.
C     Varies a maximum of 12 parameters for rigid molecules. The parameters
C        are divided in four groups:
C     1: AX, BY, CZ: projections of A, B, C on X, Y, Z.
C     2: BX, CX, CY: likewise (zero for orthorhombic systems)
C     3: X, Y, Z: coordinates of the center of gravity
C     4: PHI, THETA, PSI: Euler angles for orientation.

C     B. P. van Eijck, November 1997.
C     Input files:  IN.TP (unit 20), molecular topology
C                   IN.SPG (unit 11), space group data
C                   IN.FF (unit 12), force field
C                   IN.PA (unit 13), code-directed input
C                   COR.001, COR.002, ...(unit 14), coordinates
C     Output files: OUT.PRI (unit 16), general results
C                   PACK.001, PACK.002,...(unit 10), structure parameters
C     These can be clustered by program CLUS and are there transformed
C        into files PACK.101, PACK.102, .... for input to PACK2.

C     File IN.TP (read by subroutine LEEST) is the formatted topology file
C        created by program CODES and/or by hand.
C     File IN.SPG (read by subroutine LEESR) contains the space group data
C        for triclinic, monoclinic, or orthorhombic space groups.
C     File IN.FF (read by subroutine LEEST) contains the force field
C     Files COR.001, COR.002,... (read by subroutine LEESX) contain
C        reference atomic coordinates for the various conformations.
C        Allowed formats are SPF or MDI.
C     File IN.PA contains data specific for this program.
C     See manual for details.

      implicit none
      external func1
      include 'params.i'
      include 'cspgr.i'
      include 'cbond.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cwald.i'
      include 'cnbkl.i'
      include 'cpars.i'
      include 'cinds.i'
      include 'color.i'
      include 'caidt.i'
      character*200 regel

      integer i,iclus,icoul,idih,idum,indeul,inpt,intra,ios,iprint,
     +     iprx,iseed,ispgr,istage,istat,itest,itest1,itest2,itest3,
     +     itot,k,m,n,ncl,nclus,ncy1cg,ncy1sd,ncy2cg,ncy2sd,ncy3cg,
     +     ncy3sd,ncyc,neuler,nheul,nhit1,nhit2,nhit3,nhit4,nmax,
     +     npar,npm,nrand,nre1sd,nre2sd,nre3sd,nreul,nstart,ntry,ntryt
      double precision amax,amin,amin0,ax,bclus,bmax,bmin,bmin0,box,
     +     bx,bxmax,bxmin,by,cmax,cmin,cmin0,coord,cphi,cpsi,cthe0,
     +     ctheta,cx,cxmax,cxmin,cy,cymax,cymin,cz,da0,dax,db0,dbx,
     +     dby,dc0,dclin,dclus,dconf,dcthe0,dcx,dcy,dcz,dep1cg,dep2cg,
     +     dep3cg,der,dexp,dihan,dmax,dphi,dphi0,dpsi,dpsi0,dpsi1,
     +     dtest,dthe,dx,dy,dz,eclus,emax,en1,en2,en3,en4,estmax,
     +     estopp,euler,frac,par,phi,phi0,phimax,phimin,pi,pres,psi,
     +     psi0,psimax,psimin,r,rad,random,rmsder,scr,shm1cg,shm1sd,
     +     shm2cg,shm2sd,shm3cg,shm3sd,sphi,spsi,stcin,stdc,stheta,
     +     stp1cg,stp1sd,stp2cg,stp2sd,stp3cg,stp3sd,tau,themax,
     +     themin,theta,tol1cg,tol1sd,tol2cg,tol2sd,tol3cg,tol3sd,
     +     totm,vexp,vol,volcon,volmin,xcm,xmax,xmin,xx,ymax,ymin,
     +     yy,zmax,zmin,zz
      dimension coord(maxnrp,3),box(12),indeul(3),
     +     der(12),stcin(4),stdc(12),dclin(4),dclus(12),par(12),
     +     euler(3,3),xcm(3),itest(3),nmax(0:3),frac(1:3),
     +     emax(3),istat(imin:imax),scr(maxpar),tau(maxome),
     +     bclus(maxcl1,12),eclus(maxcl1,3),iclus(maxcl1,2)

      character*8 spname,dumnam
      character fin*8,fuit*9,titel*80,form*60
      logical dorand,estvol

      ngr=1
      pi=4*atan(1.0d0)
      rad=180/pi
      rmsder=-9.99d0
      ispgr=0
      volmin=10000

      amin=0.4d0
      dax=0.15d0
      amax=2.5d0
      bmin=0.4d0
      dby=0.15d0
      bmax=2.5d0
      cmin=0.4d0
      dcz=0.15d0
      cmax=2.5d0
      amin0=amin
      bmin0=bmin
      cmin0=cmin
      da0=amax-amin
      db0=bmax-bmin
      dc0=cmax-cmin
      estopp=-1

      phimin=0
      dphi=20
      phimax=359
      themin=0
      dthe=20
      themax=89
      psimin=0
      dpsi=20
      psimax=179
      cthe0=1
      phi0=0
      psi0=0
      dcthe0=-2
      dphi0=360
      dpsi0=180

      bxmin=0
      dbx=0.2d0
      bxmax=0.999d0
      cxmin=0
      dcx=0.2d0
      cxmax=0.999d0
      cymin=0
      dcy=0.2d0
      cymax=0.999d0

      xmin=0
      dx=0.1d0
      xmax=0.499d0
      ymin=0
      dy=0.1d0
      ymax=0.499d0
      zmin=0
      dz=0.1d0
      zmax=0.499d0

      ncy1sd=5
      tol1sd=10
      stp1sd=0.005d0
      nre1sd=50
      shm1sd=1.0d0
      ncy1cg=0
      tol1cg=10
      stp1cg=0.05d0
      dep1cg=1
      shm1cg=1

      ncy2sd=10
      tol2sd=1
      stp2sd=0.002d0
      nre2sd=50
      shm2sd=1
      ncy2cg=10
      tol2cg=1
      stp2cg=0.05d0
      dep2cg=1
      shm2cg=1

      ncy3sd=10
      tol3sd=0.1d0
      stp3sd=0.001d0
      nre3sd=10
      shm3sd=1
      ncy3cg=10
      tol3cg=0.1d0
      stp3cg=0.01d0
      dep3cg=1
      shm3cg=0.05d0

      nmax(1)=20000
      nmax(2)=15000
      nmax(3)=10000
      frac(1)=0.020d0
      frac(2)=0.015d0
      frac(3)=0.014d0
      ebig=2000
      dexp=-100
c      dexp=1.5
      nrand=0

      inner=0
      iseed=281197
      intra=0
      icoul=0
      pres=0
      iprint=0
      dtest=0
      nstart=0

      do i=1,4
         stcin(i)=1
         dclin(i)=0.05d0
      enddo
      dclin(4)=10

      open(unit=16,file='out.pri')
      open(unit=13,file='in.pa',status='OLD',iostat=ios)
      print*,geel,' P A C K 1',wit
      write(16,*) ' '
      write(16,*) ' P A C K 1'
      if (ios.ne.0) then
         print*,' No file IN.PA present'
         write(16,*) ' No file IN.PA present'
      else
         regel=' '
         n=inpt('INPUT ',regel)
         close(13)
      endif
      n=inpt('SPGR  ',regel)
      spname='noname'
 115  continue
      if (n.gt.1) then
         call strext(regel,2,spname)
         ispgr=-1
         read(spname,58,err=120) i
         ispgr=i
 120     continue
      endif
      if (ispgr.eq.0) then
         print*,' ***** ISPGR must not be zero!'
         print*,' Give space group name or number: '
         read 11,spname
         write(16,*) 'Space group read from keyboard: ',spname
         print*, 'Space group read from keyboard: ',spname
         regel='SPGR  '//spname
         print*,regel
         n=2
         goto 115
      endif
      n=inpt('START ',regel)
      if (n.ge.2) call intstr(regel,2,nstart)
C     Undocumented option: select grid point NSTART for debugging
      n=inpt('INNER ',regel)
      if (n.ge.2) call intstr(regel,2,inner)
c      n=inpt('INTRA ',regel)
c      if (n.ge.2) call intstr(regel,2,intra)
      noint=(intra.eq.0)
      n=inpt('COUL  ',regel)
      if (n.ge.2) call intstr(regel,2,icoul)
      n=inpt('PRES  ',regel)
      if (n.ge.2) call realst(regel,2,pres)
      n=inpt('DEXP  ',regel)
      if (n.ge.2) call realst(regel,2,dexp)
      n=inpt('EBIG  ',regel)
      if (n.ge.2) call realst(regel,2,ebig)
      n=inpt('PRINT ',regel)
      if (n.ge.2) call intstr(regel,2,iprint)
      iprx=iprint
      n=inpt('TEST  ',regel)
      if (n.ge.2) call realst(regel,2,dtest)
      n=inpt('SEED  ',regel)
      if (n.ge.1) call intstr(regel,2,iseed)
      n=inpt('RAND  ',regel)
      if (n.ge.2) call intstr(regel,2,nrand)
      n=inpt('AXES  ',regel)
      if (n.ge.10) then
         call realst(regel,2,amin)
         call realst(regel,3,dax)
         call realst(regel,4,amax)
         call realst(regel,5,bmin)
         call realst(regel,6,dby)
         call realst(regel,7,bmax)
         call realst(regel,8,cmin)
         call realst(regel,9,dcz)
         call realst(regel,10,cmax)
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in AXES',n
         stop
      endif
      n=inpt('MINARE',regel)
      if (n.ge.2) call realst(regel,2,estopp)
C     Undocumented option: select minimum area for automatic AMAX, BMAX, CMAX
      n=inpt('GRAVS ',regel)
      if (n.ge.10) then
         call realst(regel,2,xmin)
         call realst(regel,3,dx)
         call realst(regel,4,xmax)
         call realst(regel,5,ymin)
         call realst(regel,6,dy)
         call realst(regel,7,ymax)
         call realst(regel,8,zmin)
         call realst(regel,9,dz)
         call realst(regel,10,zmax)
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in GRAVS',n
         stop
      endif
      n=inpt('EULERS',regel)
      if (n.ge.10) then
         call realst(regel,2,phimin)
         call realst(regel,3,dphi)
         call realst(regel,4,phimax)
         call realst(regel,5,themin)
         call realst(regel,6,dthe)
         call realst(regel,7,themax)
         call realst(regel,8,psimin)
         call realst(regel,9,dpsi)
         call realst(regel,10,psimax)
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in EULERS',n
         stop
      endif
      n=inpt('ANGLES',regel)
      if (n.ge.10) then
         call realst(regel,2,bxmin)
         call realst(regel,3,dbx)
         call realst(regel,4,bxmax)
         call realst(regel,5,cxmin)
         call realst(regel,6,dcx)
         call realst(regel,7,cxmax)
         call realst(regel,8,cymin)
         call realst(regel,9,dcy)
         call realst(regel,10,cymax)
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in ANGLES',n
         stop
      endif
      n=inpt('STC   ',regel)
      if (n.ge.5) then
         call realst(regel,2,stcin(1))
         call realst(regel,3,stcin(2))
         call realst(regel,4,stcin(3))
         call realst(regel,5,stcin(4))
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in STC',n
         stop
      endif
      n=inpt('CLUS  ',regel)
      if (n.ge.5) then
         call realst(regel,2,dclin(1))
         call realst(regel,3,dclin(2))
         call realst(regel,4,dclin(3))
         call realst(regel,5,dclin(4))
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in STC',n
         stop
      endif
      n=inpt('SD1   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy1sd)
      if (n.ge.3) call realst(regel,3,tol1sd)
      if (n.ge.4) call realst(regel,4,stp1sd)
      if (n.ge.5) call intstr(regel,5,nre1sd)
      if (n.ge.6) call realst(regel,6,shm1sd)
      n=inpt('CG1   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy1cg)
      if (n.ge.3) call realst(regel,3,tol1cg)
      if (n.ge.4) call realst(regel,4,stp1cg)
      if (n.ge.5) call realst(regel,5,dep1cg)
      if (n.ge.6) call realst(regel,6,shm1cg)
      n=inpt('SD2   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy2sd)
      if (n.ge.3) call realst(regel,3,tol2sd)
      if (n.ge.4) call realst(regel,4,stp2sd)
      if (n.ge.5) call intstr(regel,5,nre2sd)
      if (n.ge.6) call realst(regel,6,shm2sd)
      n=inpt('CG2   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy2cg)
      if (n.ge.3) call realst(regel,3,tol2cg)
      if (n.ge.4) call realst(regel,4,stp2cg)
      if (n.ge.5) call realst(regel,5,dep2cg)
      if (n.ge.6) call realst(regel,6,shm2cg)
      n=inpt('SD3   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy3sd)
      if (n.ge.3) call realst(regel,3,tol3sd)
      if (n.ge.4) call realst(regel,4,stp3sd)
      if (n.ge.5) call intstr(regel,5,nre3sd)
      if (n.ge.6) call realst(regel,6,shm3sd)
      n=inpt('CG3   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy3cg)
      if (n.ge.3) call realst(regel,3,tol3cg)
      if (n.ge.4) call realst(regel,4,stp3cg)
      if (n.ge.5) call realst(regel,5,dep3cg)
      if (n.ge.6) call realst(regel,6,shm3cg)

      call leesr(ispgr,spname,iref,itest,msom,inner)
C     LEESR may set INNER to zero
      itest1=itest(1)
      itest2=itest(2)
      itest3=itest(3)
      call leest(nrp,icoul,iprint)
      nmols=1
      imols(1)=1
      imols(2)=nrp+1
C     This gives NRP, the number of atoms in one molecule.
      doewc=.false.
      doew6=.false.
      doew=.false.
C     If desired, override standard cutoff radius from topology file
      n=inpt('CUTOFF',regel)
      if (n.ge.2) call realst(regel,2,cutoff)
      if (n.ge.3) call realst(regel,3,tole)

      totm=0
      do i=1,nrp
         totm=totm+amass(i)
      enddo

      write(16,*) ' cutoff    tole   pressure(atm)'
      write(16,51) cutoff,tole,pres
      write(16,*) '  inner   intra   icoul  iprint'
      write(16,55) inner,intra,icoul,iprint
      write(16,*) '    nrp   ndihz   ndihh  ireftr...'
      write(16,55) nrp,ndihz,ndihh,(ireftr(m),m=1,3)
      write(16,*) ' Dihedrals:'
      do i=1,ndihz
         write(16,50) iome(i),jome(i),kome(i),lome(i)
      enddo
      write(16,*) '    nrp  itest1  itest2  itest3'
      write(16,55) nrp,itest1,itest2,itest3
      write(16,*) ' ncycsd   tolsd   stpsd   nresd   shmsd       dtest'
      write(16,33) ncy1sd,tol1sd,stp1sd,nre1sd,shm1sd,dtest
      write(16,33) ncy2sd,tol2sd,stp2sd,nre2sd,shm2sd
      write(16,33) ncy3sd,tol3sd,stp3sd,nre3sd,shm3sd
      write(16,*) ' ncyccg   tolcg   stpcg   depcg   shmcg'
      write(16,38) ncy1cg,tol1cg,stp1cg,dep1cg,shm1cg
      write(16,38) ncy2cg,tol2cg,stp2cg,dep2cg,shm2cg
      write(16,38) ncy3cg,tol3cg,stp3cg,dep3cg,shm3cg

      finner=16*atan(1.0d0)/(3*(nspgr+1))
      druk=0.061018d0*pres/(nspgr+1)
      npar=0
      nreul=0
      dclin(4)=dclin(4)/rad
      do i=1,12
         k=(i+2)/3
         dclus(i)=dclin(k)
         if (iref(i).gt.0) then
            npar=npar+1
            stdc(npar)=stcin(k)
            if (i.gt.9) then
               nreul=nreul+1
               indeul(nreul)=npar
            endif
         endif
      enddo
      write(form,99) npar
      write(16,*) 'Number of parameters ', npar
      write(16,*) 'Steepest descent parameters:'
      write(16,54) (stdc(m),m=1,npar)
      write(16,*) 'Clustering limits:'
      write(16,54) (dclus(m),m=1,12)
      write(16,*) 'Total mass:',totm
      if (dexp.gt.0.001d0) then
         vexp=0.0016605d0*(nspgr+1)*totm/dexp
C        dexp in g.cm-3, vexp in nm3.
         volmin=vexp
      endif

 180  continue
      istage=0
      dorand=(nrand.gt.0)
      nmax(0)=nrand
C     Check whether we have to estimate an expected volume
      estvol=(dexp.lt.-0.001d0)
      if (estvol) then
         dorand=.true.
         nmax(0)=nint(-dexp)
         vexp=10000
         emax(1)=-1
         emax(2)=-1
         emax(3)=-1
      else
C        Check whether we have to determine EMAX values
         n=inpt('EMAX  ',regel)
         if (n.ge.4) then
C           Skip statistical determination of EMAX
            call realst(regel,2,emax(1))
            call realst(regel,3,emax(2))
            call realst(regel,4,emax(3))
         elseif (n.gt.0) then
            print*,' ***** Wrong number of items in EMAX',n
            stop
         else
C           Do statistical determination of EMAX
            n=inpt('STAT  ',regel)
            if (n.ge.7) then
               call intstr(regel,2,nmax(1))
               call intstr(regel,3,nmax(2))
               call intstr(regel,4,nmax(3))
               call realst(regel,5,frac(1))
               call realst(regel,6,frac(2))
               call realst(regel,7,frac(3))
            elseif (n.gt.0) then
               print*,' ***** Wrong number of items in STAT',n
               stop
            endif
            print 66
            write(16,66)
            print*, 'Statistical determination of EMAX'
            write(16,*) 'Statistical determination of EMAX:'
            write(16,*)
     +           '  nmax1   nmax2   nmax3     frac1   frac2   frac3'
            write(16,52) nmax(1),nmax(2),nmax(3),frac(1),frac(2),frac(3)
            if (abs(dexp).lt.0.001d0) then
               print*,' Search not limited by estimated volume'
               write(16,*) ' Search not limited by estimated volume'
            else
               print 60,vexp,dexp
               write(16,60) vexp,dexp
            endif
            dorand=.true.
            istage=1
c            frac=frac(1)
c            nmax=nmax(1)
         endif
      endif

 190  continue
C     Start a new stage in or after the EMAX determination
      idih=0
      iprx=1
      if (istage.ne.0) then
C        A stage in the EMAX determination
         print 63,istage,nmax(istage),frac(istage)
         write(16,63) istage,nmax(istage),frac(istage)
         do i=imin,imax
            istat(i)=0
         enddo
      else
C        The actual calculation (or the volume estimation)
         print 66
         write(16,66)
         if (estvol) then
C           The volume estimation
            print 32,nmax(0)
            write(16,32) nmax(0)
         else
C           The actual calculation
            if (iprint.gt.1) iprx=2
            if (estopp.gt.0) then
               estmax=vexp/estopp
               print*,' Estimated maximum cell length', estmax
               write(16,*)' Estimated maximum cell length', estmax
               amax=estmax
               bmax=estmax
               cmax=estmax
            endif
            if (dorand) then
               print 35,nmax(0)
               write(16,35) nmax(0)
C              Now limit the random search to chosen limits
               amin0=amin
               bmin0=bmin
               cmin0=cmin
               da0=amax-amin
               db0=bmax-bmin
               dc0=cmax-cmin
               cthe0=cos(themin/rad)
               phi0=phimin
               psi0=psimin
               dcthe0=cos(themax/rad)-cthe0
               dphi0=phimax-phimin
               dpsi0=psimax-psimin
            else
               print*, 'Start grid search'
               write(16,*) 'Start grid search'
            endif
            write(16,*) 'Search limits:'
            write(16,56) iref(1),amin,dax,amax
            write(16,56) iref(2),bmin,dby,bmax
            write(16,56) iref(3),cmin,dcz,cmax
            write(16,56) iref(4),bxmin,dbx,bxmax
            write(16,56) iref(5),cxmin,dcx,cxmax
            write(16,56) iref(6),cymin,dcy,cymax
            write(16,56) iref(7),xmin,dx,xmax
            write(16,56) iref(8),ymin,dy,ymax
            write(16,56) iref(9),zmin,dz,zmax
            write(16,56) iref(10),phimin,dphi,phimax
            write(16,56) iref(11),themin,dthe,themax
            write(16,56) iref(12),psimin,dpsi,psimax
            if (abs(dexp).lt.0.001d0) then
               print*,' Search not limited by estimated volume'
               write(16,*) ' Search not limited by estimated volume'
            else
               print 60,vexp,dexp
               write(16,60) vexp,dexp
            endif
         endif
         print 31,ebig,emax(1),emax(2),emax(3)
      endif

 200  continue
C     Start the next coordinate file
      idih=idih+1
      write(fin,26) 'cor.',idih
      open(unit=14,file=fin,status='OLD',err=840)
      volcon=10000
      if (istage.eq.0 .and. .not. estvol) then
         print 25, fin
         write(16,25) fin
      endif
      call leesx(14,n,nrp,npm,1,0,titel,dumnam,idum,x,box,iprx)
      close(14)

C     Transform to reference coordinates XR, start PACK file
      call trans(nrp,x,ireftr,xcm,xr,phi,theta,psi)
      do i=1,ndihz
         tau(i)=rad*dihan(i,x)
      enddo
      if (istage.eq.0 .and. .not. estvol) then
         write(fuit,22) 'pack.',idih
         open(unit=10,file=fuit)
         write(10,94) ispgr,1
         write(10,94) ndihz,0
         do i=1,ndihz
            write(10,95) iome(i),jome(i),kome(i),lome(i),tau(i)
         enddo
         write(10,94) nrp,(ireftr(i),i=1,3)
         do i=1,nrp
            write(10,72) i,panm(i)(1:5),iac(i),(xr(3*(i-1)+m),m=1,3)
            write(16,72) i,panm(i)(1:5),iac(i),(xr(3*(i-1)+m),m=1,3)
         enddo
         write(10,93) amin,bmin,cmin,bxmin,cxmin,cymin,
     +        xmin,ymin,zmin,phimin,themin,psimin
         write(10,94) (iref(i),i=1,12)
      endif
      print 59, idih,ndihz,(tau(i),i=1,ndihz)
      write(16,59) idih,ndihz,(tau(i),i=1,ndihz)

      neuler=0
      ntry=0
      nhit1=0
      nhit2=0
      nhit3=0
      nhit4=0

C     Start search: if (DORAND) random, else grid.
      theta=themin
 300  continue
C     Start the next THETA value
      if (dorand) then
C        Take random values for cos(THETA) rather than for THETA
         ctheta=0.999999d0*(cthe0+random(iseed)*dcthe0)
         theta=acos(ctheta)*rad
      else
C        Increment in PSI depends on THETA
         dpsi1=180.0d0/(int((180/dpsi)*sin(theta/rad))+1)
         ctheta=cos(theta/rad)
      endif
      stheta=sin(theta/rad)
      psi=psimin
 302  continue
C     Start the next PSI value
      if (dorand) psi=psi0+random(iseed)*dpsi0
      cpsi=cos(psi/rad)
      spsi=sin(psi/rad)
      phi=phimin
 304  continue
C     Start the next PHI value
      if (dorand) phi=phi0+random(iseed)*dphi0
      cphi=cos(phi/rad)
      sphi=sin(phi/rad)
      cel(10)=phi/rad
      cel(11)=theta/rad
      cel(12)=psi/rad
      neuler=neuler+1
      if (.not. dorand .and. iprint.gt.0) then
         print 23,neuler
         print 24,dpsi1,phi,theta,psi
      endif

C     Rotate reference geometry XR over Euler angles (phi,theta,psi) to COORD.
      euler(1,1)= cpsi*cphi-spsi*sphi*ctheta
      euler(1,2)= cpsi*sphi+spsi*cphi*ctheta
      euler(1,3)= spsi*stheta
      euler(2,1)=-spsi*cphi-cpsi*sphi*ctheta
      euler(2,2)=-spsi*sphi+cpsi*cphi*ctheta
      euler(2,3)= cpsi*stheta
      euler(3,1)= sphi*stheta
      euler(3,2)=-cphi*stheta
      euler(3,3)= ctheta
      do i=1,nrp
         do m=1,3
            coord(i,m)=0
            do n=1,3
               coord(i,m)=coord(i,m)+euler(m,n)*xr(3*(i-1)+n)
            enddo
         enddo
      enddo

C     Set cell parameters
      nclus=0
      nheul=0
      by=bmin
 340  continue
C     Start the next BY value
      cz=cmin
 342  continue
C     Start the next theta value
      ax=amin
 344  continue
C     Start the next AX value
      if (dorand) then
         ax=amin0+random(iseed)*da0
         by=bmin0+random(iseed)*db0
         cz=cmin0+random(iseed)*dc0
      endif
C     Unlimited search only if DEXP=0, otherwise use volume for AX.
      if (dexp.gt.0.001d0) then
         ax=vexp/(by*cz)
         if (ax.lt.amin .or. ax.gt.amax) goto 590
      endif
      if (itest1.eq.1 .and. (ax.gt.by+dby)) goto 590
      if (itest2.eq.1 .and. (by.gt.cz+dcz)) goto 590
      if (itest3.eq.1 .and. (ax.gt.cz+dcz)) goto 590
      cel(1)=ax
      cel(2)=by
      cel(3)=cz
      bx=bxmin*ax
 350  continue
C     Start the next BX value
      cx=cxmin*ax
 352  continue
C     Start the next CX value
      cy=cymin*by
 354  continue
C     Start the next CY value

C     Set molecular centers
      xx=xmin*ax
 360  continue
C     Start the next X value
      zz=zmin*cz
 362  continue
C     Start the next Z value
      yy=ymin*by
 364  continue
C     Start the next Y value
      if (dorand) then
         if (iref(4).gt.0) bx=ax*random(iseed)
         if (iref(5).gt.0) cx=ax*random(iseed)
         if (iref(6).gt.0) cy=by*random(iseed)
         if (iref(7).gt.0) xx=ax*random(iseed)
         if (iref(8).gt.0) yy=by*random(iseed)
         if (iref(9).gt.0) zz=cz*random(iseed)
      endif
      cel(4)=bx
      cel(5)=cx
      cel(6)=cy
      cel(7)=xx
      cel(8)=yy
      cel(9)=zz
      do i=1,nrp
         do m=1,3
            x(3*(i-1)+m)=coord(i,m)+cel(m+6)
         enddo
      enddo

C     First energy calculation (no minimization), gives EN1.
      ntry=ntry+1
      if (nstart.gt.0 .and. ntry.ne.nstart) goto 570
      call nonbe(1,nrp,en1,0,der,0,cel,x,scr,0)
      if (iprint.gt.3) print 27,ntry,(cel(m),m=1,9),phi,theta,psi,en1
      if (istage.eq.1) then
         k=nint(en1)
         if (k.lt.imin) then
            print*,' ***** Change IMIN',k
            stop
         endif
         if (k.le.imax) istat(k)=istat(k)+1
         goto 570
      endif
      if (en1.gt.emax(1)) goto 570
      nhit1=nhit1+1

      k=0
      do i=1,12
         if (iref(i).gt.0) then
            k=k+1
            par(k)=cel(i)
         endif
      enddo

C     Second energy calculation, first minimization, gives EN2
      if (iprint.gt.2) print 44, ntry,en1,(par(m),m=1,npar)
      if (dtest.gt.1.0e-10) call test(par,npar,func1,dtest)
      en2=en1
      if (ncy1sd.gt.0) then
         call stedes(npar,par,func1,ncy1sd,tol1sd,ntry,stdc,stp1sd,
     +        nre1sd,shm1sd,iprint,en2,ncyc,rmsder)
         if (iprint.gt.2) print 48,' SD1:',ncyc,en2,(par(m),m=1,npar)
      endif
      if (ncy1cg.gt.0) then
         call congra(npar,par,func1,ncy1cg,tol1cg,ntry,stp1cg,
     +        dep1cg,shm1cg,iprint,en2,ncyc,rmsder)
         if (iprint.gt.2) print 48,' CG1:',ncyc,en2,(par(m),m=1,npar)
      endif
      if (istage.eq.2) then
         k=nint(en2)
         if (k.lt.imin) then
            print*,' ***** Change IMIN',k
            stop
         endif
         if (k.le.imax) istat(k)=istat(k)+1
         goto 570
      endif
      if (en2.gt.emax(2)) goto 570
      nhit2=nhit2+1

C     Third energy calculation, second minimization, gives EN3
      en3=en2
      if (ncy2sd.gt.0) then
         call stedes(npar,par,func1,ncy2sd,tol2sd,ntry,stdc,stp2sd,
     +        nre2sd,shm2sd,iprint,en3,ncyc,rmsder)
         if (iprint.gt.2) print 48,' SD2:',ncyc,en3,(par(m),m=1,npar)
      endif
      if (ncy2cg.gt.0) then
         call congra(npar,par,func1,ncy2cg,tol2cg,ntry,stp2cg,
     +        dep2cg,shm2cg,iprint,en3,ncyc,rmsder)
         if (iprint.gt.2) print 48,' CG2:',ncyc,en3,(par(m),m=1,npar)
      endif
      if (istage.eq.3) then
         k=nint(en3)
         if (k.lt.imin) then
            print*,' ***** Change IMIN',k
            stop
         endif
         if (k.le.imax) istat(k)=istat(k)+1
         goto 570
      endif
      if (en3.gt.emax(3)) goto 570
      nhit3=nhit3+1
      nheul=nheul+1

C     Do clustering for a grid search
      k=0
      do i=1,12
         if (iref(i).gt.0) then
            k=k+1
            box(i)=par(k)
         else
            box(i)=cel(i)
         endif
      enddo
      if (dorand) then
         nclus=1
         do m=1,12
            bclus(nclus,m)=box(m)
         enddo
         iclus(nclus,1)=1
         iclus(nclus,2)=ntry
         eclus(nclus,1)=en1
         eclus(nclus,2)=en2
         eclus(nclus,3)=en3
         goto 600
      endif
      do i=1,nclus
         do m=1,12
            der(m)=box(m)-bclus(i,m)
         enddo
C     Het volgende maar niet doen, want gaat mis bij A centering.
c        if (abs(der(4)).gt.box(1)/2) then
c          der(4)=der(4)-sign(box(1),der(4))
c        endif
c        if (abs(der(5)).gt.box(1)/2) then
c          der(5)=der(5)-sign(box(1),der(5))
c        endif
c        if (abs(der(6)).gt.box(2)/2) then
c          der(6)=der(6)-sign(box(2),der(6))
c          der(5)=der(5)-sign(box(4),der(6))
c        endif
         if (abs(der(7)).gt.box(1)/4) then
            der(7)=der(7)-sign(box(1)/2,der(7))
         endif
         if (abs(der(8)).gt.box(2)/4) then
            der(7)=der(7)-sign(box(4)/2,der(8))
            der(8)=der(8)-sign(box(2)/2,der(8))
         endif
         if (abs(der(9)).gt.box(3)/4) then
            der(7)=der(7)-sign(box(5)/2,der(9))
            der(8)=der(8)-sign(box(6)/2,der(9))
            der(9)=der(9)-sign(box(3)/2,der(9))
         endif
         do m=1,12
            if (abs(der(m)).gt.dclus(m)) goto 500
         enddo
C        This solution was already found, take the one with lowest energy
         if (en3.gt.eclus(i,3)) then
            if (iprint.gt.2) print*,en3,' removed against ',i,eclus(i,3)
            goto 520
         endif
         if (iprint.gt.2) print*,
     +        ' previous ',i,eclus(i,3),' replaced by',en3
         do m=1,12
            bclus(i,m)=box(m)
         enddo
         iclus(i,1)=iclus(i,1)+1
         iclus(i,2)=ntry
         eclus(i,1)=en1
         eclus(i,2)=en2
         eclus(i,3)=en3
         goto 520
 500     continue
      enddo
C     This is a new solution
      nclus=nclus+1
      if (nclus.gt.maxcl1) then
         print*, ' ***** Increase maxcl1'
         stop
      endif
      do m=1,12
         bclus(nclus,m)=box(m)
      enddo
      iclus(nclus,1)=1
      iclus(nclus,2)=ntry
      eclus(nclus,1)=en1
      eclus(nclus,2)=en2
      eclus(nclus,3)=en3
 520  continue

 570  continue
      if (dorand) goto 590
C     Apply shifts to start the next set of centers of gravity
      if (iref(8).gt.0) then
         yy=yy+dy
         if (yy.le.ymax*by) goto 364
      endif
      if (iref(9).gt.0) then
         zz=zz+dz
         if (zz.le.zmax*cz) goto 362
      endif
      if (iref(7).gt.0) then
         xx=xx+dx
         if (xx.le.xmax*ax) goto 360
      endif
C     Apply shifts to start the next set of axis projections
      if (iref(6).gt.0) then
         cy=cy+dcy
         if (cy.le.cymax*by) goto 354
      endif
      if (iref(5).gt.0) then
         cx=cx+dcx
         if (cx.le.cxmax*ax) goto 352
      endif
      if (iref(4).gt.0) then
         bx=bx+dbx
         if (bx.le.bxmax*ax) goto 350
      endif
 590  continue
      if (dorand) then
         if (neuler.gt.50000 .and. ntry.eq.0) then
            print*,'***** Apparently no solutions, inspect conditions'
            stop
         endif
         if (istage.gt.0) then
            if (ntry.lt.nmax(istage)) goto 300
         else
            if (nhit4.lt.nmax(istage)) goto 300
         endif
         goto 700
      endif
C     Apply shifts to start the next set of cell axes.
C     Unlimited search only if DEXP=0, otherwise use volume for AX.
      if (iref(1).ne.0 .and. abs(dexp).lt.0.001d0) then
         ax=ax+dax
         if (ax.le.amax+0.001d0) goto 344
      endif
      if (iref(3).gt.0) then
         cz=cz+dcz
         if (cz.le.cmax+0.001d0) goto 342
      endif
      if (iref(2).gt.0) then
         by=by+dby
         if (by.le.bmax+0.001d0) goto 340
      endif
C     End of search for this set of Euler angles

C     Continue with clustered results
      if (iprint.gt.0) print 57,nheul,nclus
 600  continue
C     Final energy minimization, after clustering, gives EN4
      do ncl=1,nclus
         k=0
         do i=1,12
            if (iref(i).gt.0) then
               k=k+1
               par(k)=bclus(ncl,i)
            endif
         enddo
         if (iprint.gt.2) print 43, ncl,iclus(ncl,2),iclus(ncl,1),
     +           eclus(ncl,3),(par(m),m=1,npar)
         en4=eclus(ncl,3)
         if (ncy3sd.gt.0) then
            call stedes(npar,par,func1,ncy3sd,tol3sd,ntry,stdc,stp3sd,
     +           nre3sd,shm3sd,iprint,en4,ncyc,rmsder)
            if (iprint.gt.2) print 48,' SD3:',ncyc,en4,(par(m),m=1,npar)
         endif
         if (ncy3cg.gt.0) then
            call congra(npar,par,func1,ncy3cg,tol3cg,ntry,stp3cg,
     +           dep3cg,shm3cg,iprint,en4,ncyc,rmsder)
            if (iprint.gt.2) print 48,' CG3:',ncyc,en4,(par(m),m=1,npar)
         endif
         if (en4.gt.emax(3)) goto 650
C        This occurs, unexpectedly, from time to time.
         nhit4=nhit4+1
         vol=par(1)*par(2)*par(3)
         if (vol.lt.volcon) then
C           Recalculate energy, just to be sure
            call func1(npar,par,der,en4,1)
            if (en4.gt.emax(3)) goto 650
            volcon=vol
            if (vol.lt.volmin) then
               volmin=vol
               dmax=0.0016605d0*(nspgr+1)*totm/volmin
               print 47,vol,dmax,en4,nhit4
               write(16,47) vol,dmax,en4,nhit4
               if (estvol) then
Ccc                 Improve density: use the smallest volume encountered + 10
ccc                  vexp=1.10*volmin
C                 Improve density: use the smallest volume encountered
                  vexp=volmin
                  dexp=0.0016605d0*(nspgr+1)*totm/vexp
               endif
            endif
         endif
         if (istage.eq.0) then
            do i=1,nreul
               par(indeul(i))=par(indeul(i))*rad/100
            enddo
            if (abs(en4).gt.99999) en4=99999
            if (.not. estvol) write(10,form) iclus(ncl,1),nhit4,en4,
     +           (nint(1000*par(m)),m=1,npar),iclus(ncl,2),
     +           (nint(eclus(ncl,m)),m=1,3),nint(1000*vol)
            if (iprint.gt.1) print form, iclus(ncl,1),nhit4,en4,
     +           (nint(1000*par(m)),m=1,npar),iclus(ncl,2),
     +           (nint(eclus(ncl,m)),m=1,3),nint(1000*vol)
         endif
 650     continue
      enddo

C     End of clustering and energy minimization
      if (.not. dorand .and. iprint.gt.0) print 58, 
     +     ntry,nhit1,nhit2,nhit3,nhit4
C     Start a new random calculation if NTRY < NMAX
      if (dorand) then
c         if (ntry.lt.nmax(istage)) goto 300
         if (nhit4.lt.nmax(istage)) goto 300
      else
C     Apply shifts to start the next set of Euler angles
         if (iref(10).gt.0) then
            phi=phi+dphi
            if (phi.le.phimax) goto 304
         endif
         if (iref(11).gt.0) then
            psi=psi+dpsi1
            if (psi.le.psimax) goto 302
         endif
         if (iref(12).gt.0) then
            theta=theta+dthe
            if (theta.le.themax) goto 300
         endif
         if (istage.eq.0) close(10)
      endif

 700  continue
C     End of loop over Euler angles

C     Output for this conformation
      if (istage.eq.0) then
         print*, neuler,' Euler angles investigated'
         write(16,*) neuler,' Euler angles investigated'
         if (dorand) then
            print*, ntry, ' random points investigated'
            write(16,*) ntry, ' random points investigated'
         else
            print*, ntry, ' grid points investigated'
            write(16,*) ntry, ' grid points investigated'
         endif
         print*, nhit1,' remain after initial energy calculation'
         print*, nhit2,' remain after first optimization'
         print*, nhit3,' remain after second optimization'
         write(16,*) nhit1,' remain after initial energy calculation'
         write(16,*) nhit2,' remain after first optimization'
         write(16,*) nhit3,' remain after second optimization'
         if (.not. dorand) then
            print*, nhit4,' remain after clustering'
            write(16,*) nhit4,' remain after clustering'
         else
            print*, nhit4,' remain after third optimization'
            write(16,*) nhit4,' remain after third optimization'
         endif
         if (nhit4.gt.0) then
            dconf=0.0016605d0*(nspgr+1)*totm/volcon
            print 65,volcon,dconf
            write(16,65) volcon,dconf
         endif
      endif
C     Start the next conformation
      goto 200
 840  continue
      if (idih.eq.1) then
         print*,' ***** No coordinate file COR.001 present'
         stop
      endif
      print*,'No further coordinate files COR present'
      write(16,*) 'No further coordinate files COR present'

      if (istage.ne.0) then
C        Determine the EMAX values corresponding to FRAC
         ntryt=(idih-1)*nmax(istage)
         itot=0
         do i=imin,imax
            itot=itot+istat(i)
         enddo
         r=itot/dfloat(ntryt)
         print 64,ntryt,itot,r
         write(16,64) ntryt,itot,r
         call stats(imin,imax,istat,ntryt,frac(istage),emax(istage))
         print 62,istage,frac(istage),ntryt,emax(istage)
         write(16,62) istage,frac(istage),ntryt,emax(istage)
         istage=istage+1
         if (istage.eq.4) then
            istage=0
            print 31,ebig,(emax(m),m=1,3)
            write(16,31) ebig,(emax(m),m=1,3)
            dorand=(nrand.gt.0)
            nmax(0)=nrand
         endif
         goto 190
      endif

      if (estvol) goto 180
C     The density should now be estimated, continue the procedure

      print 67,volmin,dmax
      write(16,67) volmin,dmax
      print 31,ebig,(emax(m),m=1,3)
      write(16,31) ebig,(emax(m),m=1,3)
      i=inpt('CHECK ',regel)

c 10   format(7(1x,e10.5))
 11   format(a)
c 12   format(16a5)
c 13   format(25i3)
c 14   format(5x,a5,i5,2f10.5)
c 15   format(15i5)
c 20   format(a80)
c 21   format(1x,a78)
 22   format(a5,i3.3)
 23   format(/,' E U L E R   S E T',i6)
 24   format(' dpsi=',f8.3,'      phi, theta, psi: ',3f8.3)
 25   format(/,' N E X T    G E O M E T R Y    ',a8)
 26   format(a4,i3.3)
 27   format(i8,3x,9f6.3,3f6.1,f12.3)
c 29   format(' M O L E C U L A R   T O P O L O G Y   F I L E')
 31   format(' Thresholds: ebig, emax1, emax2, emax3: ',4f10.1)
 32   format(' Determine cell volume by random search ',
     +        'for',i8,' structures')
 33   format(i8,2f8.4,i8,f8.4,f12.7)
 35   format(' Start random search for',i5,' structures')
 38   format(i8,4f8.4)
c 42   format (' Next trial: ',i8,f12.1,3x,9f6.3,3f6.1)
 43   format(i5,i8,i4,3x,f8.1,3x,12f7.3)
 44   format(i8,f8.1,1x,12f7.3)
 47   format(' Smaller volume found:',f10.4,'   (density',f10.4,')',
     +     '    with energy',f10.3,' at structure',i6)
 48   format(a5,i4,f8.1,1x,12f7.3)
 50   format(4i4,f7.1,2i4,3x,10i4)
 51   format(f8.3,f8.4,f16.1)
 52   format(3i8,2x,3f8.4)
 54   format(12f10.4)
 55   format(12i8)
 56   format(i8,3x,10f10.3)
 57   format(i6,' structures,',i6,' remain after clustering')
 58   format(10i10)
 59   format(' Geometry',i4,' with',i4,' dihedral(s): ',10f10.3)
 60   format(' Search limited by expected volume',f10.4,
     +     '   (density',f10.4,')')
c 61   format(2i10,f10.4)
 62   format(' Statistics from stage',i3,' with frac =',f10.4,' and',
     +     i8,' entries: emax =',f8.1)
 63   format(/,' Now starting statistics, stage',i3,
     +     ' with nmax, frac:',i8,f9.4)
 64   format(i8,' trials done,',i8,' within range, ratio ',f10.4)
 65   format(' Smallest volume for this conformation',f10.4,
     +     '   (density',f10.4,')')
 66   format(/,'---------------------------------------------------',
     +     '---------------------------------')
 67   format(' Smallest volume encountered:',f10.4,
     +     '   (density',f10.4,')')
 72   format(i5,5x,a5,i5,6f8.4)
c 92   format(20x,6f10.6)
 93   format(6f10.4)
 94   format(12i5)
 95   format(4i5,f10.3)
 99   format('(2i7,f8.1,1x,',i2,'i5,i12,10i5)')
      end


      subroutine stats(imin,imax,istat,ninpt,frac,emax)
      implicit none
      integer i,ilast,imax,imin,istat,ninpt,nstat
      double precision emax,frac,r,ruitp,ruitw
      dimension istat(imin:imax)
      nstat=0
      ruitp=0
      ruitw=0
      do i=imin,imax
         if (istat(i).ne.0) then
            ilast=i
            nstat=nstat+istat(i)
            r=nstat/dfloat(ninpt)
            if (r.le.frac) emax=i+1
            if (r.gt.ruitp) then
               ruitp=ruitp+frac/5
               if (r.lt.2*frac) print 60,i,nstat,r
            endif
            if (r.gt.ruitw) then
               ruitw=ruitw+frac/20
               if (r.lt.2*frac) write(16,60) i,nstat,r
            endif
         endif
         if (i.eq.imax) then
            print 60,ilast,nstat,r
            print 60,i,nstat,r
            write(16,60) ilast,nstat,r
            write(16,60) i,nstat,r
         endif
      enddo
 60   format(2i10,f10.4)
      end
