      program pack12

C     Complete generation of possible crystal structures.
C     Takes random variables for the following parameters:
C     -  AX, BY, CZ: projections of A, B, C on X, Y, Z.
C     -  BX, CX, CY: likewise (zero for orthorhombic systems)
C     For each independent molecule also:
C     -  PHI, THETA, PSI: Euler angles for orientation.
C     -  X, Y, Z: coordinates of the center of gravity
C     -  TAU(1...NDIHH), torsional angles

C     B. P. van Eijck, October 1998.
C     Input files:  IN.TP (unit 20), molecular topology
C     IN.SPG (unit 11), space group data
C     IN.FF (unit 12), force field
C     IN.PA (unit 13), code-directed input
C     COR.001, COR.002, ...(unit 14), coordinates
C     Output files: OUT.PRI (unit 16), general results
C     PACK.10 (unit 10), structure parameters
C     PACK.19 (unit 19), generated coordinates
C     STOP (unit 15) stops calculation manually
C          An integer in STOP updates NRAND.

C     File IN.TP (read by subroutine LEEST) is the formatted topology file
C     created by program CODES and/or by hand.
C     File IN.SPG (read by subroutine LEESR) contains the space group data
C     for triclinic, monoclinic, or orthorhombic space groups.
C     File IN.FF (read by subroutine LEEST) contains the force field
C     Files COR.001, COR.002,... (read by subroutine LEESX) contain
C     reference atomic coordinates for the various conformations.
C     Allowed formats are SPF or MDI.
C     File IN.PA contains data specific for this program.
C     See manual for details.

      implicit none
      external func1
      external func3
      include 'params.i'
      include 'cspgr.i'
      include 'cbond.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cwald.i'
      include 'cnbkl.i'
      include 'cpars.i'
      include 'cinds.i'
      include 'color.i'
      include 'chadd.i'
      include 'caidt.i'

      integer i,i1,i2,i3,iadd,iapp,ibl,iconn,icopt,icoul,idih,idum,
     +     iflex,iim,imol,inpt,intra,ios,iprd,iprint,iprs,iseed,ispgr,
     +     itest,itest1,itest2,itest3,j,j3,jbl,jjm,jlist,k,k3,kkm,
     +     klist,l,m,n,nc1,ncyc,ncycg,ncysd,nfound,ng2,nhit,nhit1,
     +     nhit2,nim,njlist,njm,nklist,nkm,nmax,nn,npar,npar1,npm,nr3,
     +     nrand,nresd,nrpt,ntest,ntry,iknot,nknot,doknot
      double precision amax,amin,arot,autmax,ax,bb,bmax,bmin,box,brot,
     +     bx,bxmax,bxmin,by,cc,cmax,cmin,co,cosa,cosb,cosc,cphi,cpsi,
     +     ctheta,cx,cxmax,cxmin,cy,cymax,cymin,cz,da,db,dbx,dc,dconf,
     +     dcx,dcy,depcg,dexp,dihan,dmax,dtest,eb,eh,ei,emax1,emax2,
     +     emin,en1,enb,ep,epot,estmax,euler,hydfix,omega,par,phi,
     +     phi1,phifix,phimax,pi,pos,pres,psi,psi1,psifix,psimax,rad,
     +     random,rmsder,scr,shmcg,shmsd,si,sphi,spsi,stcin,stdc,
     +     stheta,stpcg,stpsd,tau,thefix,theta,theta1,tolcg,tolsd,totm,
     +     vexp,vol,volcon,volcor,volmin,volmol,xcm,xscr,xxfix,yyfix,
     +     zzfix,den

      dimension box(6),stcin(4),arot(3),brot(3),xcm(3),pos(3),
     +     par(maxpar),xscr(maxpar),stdc(maxpar),scr(maxpar),
     +     euler(3,3),tau(maxome*maxmol),itest(3),
     +     phifix(maxmol),thefix(maxmol),psifix(maxmol),xxfix(maxmol),
     +     yyfix(maxmol),zzfix(maxmol),hydfix(maxmol*maxome),
     +     jlist(maxmol*maxome,maxnrp),klist(maxnrp),
     +     njlist(maxmol*maxome)

c      character*200 regel
      character (len=200) regel
      character*8 spname,dumnam,keynam
c      character (len=8) spname,dumnam.keynam     mag niet!!
      character fin*8,titel*80,form10*60
      logical estvol

      pi=4*atan(1.0d0)
      rad=180/pi
      rmsder=-9.99d0
      ispgr=0
      volmin=10000
      autmax=3.7d0
      irfat1=0
      irfat2=0
      nknot=0
      nfound=0
      idum=0
      dumnam='   '

      amin=0.4d0
      amax=2.5d0
      bmin=0.4d0
      bmax=2.5d0
      cmin=0.4d0
      cmax=2.5d0

      bxmin=0
      bxmax=1
      cxmin=0
      cxmax=1
      cymin=0
      cymax=1

      phimax=360
      psimax=360

      ncysd=100
      tolsd=1
      stpsd=0.001d0
      nresd=10
      shmsd=1
      ncycg=1000
      tolcg=1
      stpcg=0.01d0
      depcg=1
      shmcg=0.05d0

c      ebig=2000
      dexp=-100
      volcor=1.10d0
      nrand=5000
      ngr=1

      inner=1
      iseed=281197
      intra=1
      iflex=1
      icoul=1
      pres=0
      iprint=0
      dtest=0
      stcin(1)=1
      stcin(2)=1
      stcin(3)=1
c      emax1=40
      emax2=10000
      emin=1.0d20
      nhit=0
      epsil=-10000
      open(unit=15,file='STOP',status='OLD',err=100)
      close(unit=15,status='DELETE')

  100 continue
      open(unit=16,file='out.pri')
      open(unit=13,file='in.pa',status='OLD',iostat=ios)
      print*,geel,' P A C K 1 2',wit
      write(16,*) ' '
      write(16,*) ' P A C K 1 2'
      if (ios.ne.0) then
         print*,' No file IN.PA present'
         write(16,*) ' No file IN.PA present'
      else
         regel=' '
         n=inpt('INPUT ',regel)
         close(13)
      endif

      spname='noname'
      n=inpt('SPGR  ',regel)
      if (n.lt.2) then
         print*,' Give space group name or number: '
         read*,keynam
         write(16,*) 'Space group read from keyboard: ',keynam
         print*, 'Space group read from keyboard: ',keynam
         regel='SPGR  '//keynam
      endif
      call strext(regel,2,spname)
      ispgr=-1
      read(spname,58,err=120) i
      ispgr=i
  120 continue
      if (ispgr.eq.0) then
         print*,' ***** ISPGR must not be zero!'
         stop
      endif
      n=inpt('NGR   ',regel)
      if (n.ge.2) call intstr(regel,2,ngr)
      n=inpt('INNER ',regel)
      if (n.ge.2) call intstr(regel,2,inner)
      n=inpt('INTRA ',regel)
      if (n.ge.2) call intstr(regel,2,intra)
      noint=(intra.eq.0)
      do i=1,10
         icn(i)=0
      enddo
      n=inpt('CONN  ',regel)
      do i=2,n
         call intstr(regel,i,m)
         icn(i-1)=m
      enddo
      n=inpt('FLEX  ',regel)
      if (n.ge.2) call intstr(regel,2,iflex)
      n=inpt('COUL  ',regel)
      if (n.ge.2) call intstr(regel,2,icoul)
      n=inpt('EPSIL ',regel)
      if (n.gt.0) call realst(regel,2,epsil)
      n=inpt('PRES  ',regel)
      if (n.ge.2) call realst(regel,2,pres)
      n=inpt('DEXP  ',regel)
      if (n.ge.2) call realst(regel,2,dexp)
      if (n.ge.3) call realst(regel,3,volcor)
      n=inpt('PRINT ',regel)
      if (n.ge.2) call intstr(regel,2,iprint)
      iprs=iprint
      iprd=iprint
      n=inpt('TEST  ',regel)
      if (n.ge.2) call realst(regel,2,dtest)
      n=inpt('SEED  ',regel)
      if (n.ge.1) call intstr(regel,2,iseed)
      n=inpt('APPEND',regel)
      if (n.gt.0) then
         if (n.gt.1) call intstr(regel,2,iseed)
         if (n.eq.1) then
            print*,'***** Appending new calculation, ',
     +           'add a different SEED'
            stop
         endif
         iapp=1
         open(unit=19,file='pack.19',access='APPEND')
      else
         iapp=0
         open(unit=19,file='pack.19')
      endif
      n=inpt('RAND  ',regel)
      if (n.ge.2) call intstr(regel,2,nrand)
      n=inpt('AUTMAX',regel)
      if (n.ge.2) call realst(regel,2,autmax)
C     Change factor for automatic AMAX, BMAX, CMAX (-1 is disable)
      doknot=0
      n=inpt('KNOT  ',regel)
      if (n.gt.0) doknot=1
      n=inpt('AXES  ',regel)
      if (n.ge.7) then
         call realst(regel,2,amin)
         call realst(regel,3,amax)
         call realst(regel,4,bmin)
         call realst(regel,5,bmax)
         call realst(regel,6,cmin)
         call realst(regel,7,cmax)
         autmax=-1
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in AXES',n
         stop
      endif
      da=amax-amin
      db=bmax-bmin
      dc=cmax-cmin
      n=inpt('ANGLES',regel)
      if (n.ge.7) then
         call realst(regel,2,bxmin)
         call realst(regel,3,bxmax)
         call realst(regel,4,cxmin)
         call realst(regel,5,cxmax)
         call realst(regel,6,cymin)
         call realst(regel,7,cymax)
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in ANGLES',n
         stop
      endif
      dbx=bxmax-bxmin
      dcx=cxmax-cxmin
      dcy=cymax-cymin
      n=inpt('STC   ',regel)
      if (n.ge.4) then
         call realst(regel,2,stcin(1))
         call realst(regel,3,stcin(2))
         call realst(regel,4,stcin(3))
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in STC',n
         stop
      endif
      ichad=0
      nvir=0
      n=inpt('CHADD ',regel)
      if (n.ge.2) call intstr(regel,2,ichad)
      n=inpt('SD    ',regel)
      if (n.ge.2) call intstr(regel,2,ncysd)
      if (n.ge.3) call realst(regel,3,tolsd)
      if (n.ge.4) call realst(regel,4,stpsd)
      if (n.ge.5) call intstr(regel,5,nresd)
      if (n.ge.6) call realst(regel,6,shmsd)
      n=inpt('CG    ',regel)
      if (n.ge.2) call intstr(regel,2,ncycg)
      if (n.ge.3) call realst(regel,3,tolcg)
      if (n.ge.4) call realst(regel,4,stpcg)
      if (n.ge.5) call realst(regel,5,depcg)
      if (n.ge.6) call realst(regel,6,shmcg)

      call leesr(ispgr,spname,iref,itest,msom,inner)
      if (ispgr.eq.-3) then
         print*,' ***** No acceptable space group found'
         stop
      endif
      if (iref(4).eq.0) then
         bxmin=0
         dbx=0
      endif
      if (iref(5).eq.0) then
         cxmin=0
         dcx=0
      endif
      if (iref(6).eq.0) then
         cymin=0
         dcy=0
      endif
      
C     LEESR may set INNER to zero
      itest1=itest(1)
      itest2=itest(2)
      itest3=itest(3)
      call leest(nrp,icoul,iprint)
C     This gives NRP, the number of atoms in one group
      emax1=nrp*ngr+20
      if (emax1.lt.40) emax1=40
      n=inpt('EMAX  ',regel)
      if (n.ge.2) call realst(regel,2,emax1)
      if (n.ge.3) call realst(regel,3,emax2)
      ebig=(nrp*ngr-20)*2500
      if (ebig.lt.2000) ebig=2000
      if (ebig.gt.80000) ebig=80000
      n=inpt('EBIG  ',regel)
      if (n.ge.2) call realst(regel,2,ebig)

      nrpt=nrp*ngr
      if (nrpt.gt.maxat) then
         print*,' ***** Increase maxat',ngr,nrp,maxat
         stop
      endif
      nr3=3*nrpt
      if (nr3+12.gt.maxpar) then
         print*,' ***** Increase maxpar',nr3+12,maxpar
         stop
      endif

      nmols=1
      imols(1)=1
C     Check for disconnected sets of atoms
      do i=2,nrpt
         nim=(i-1)/nrp
         iim=mod(i-1,nrp)+1
         do j=1,i-1
            njm=(j-1)/nrp
            if (nim.eq.njm) then
               jjm=mod(j-1,nrp)+1
               if (iexcl(jjm,iim).ne.0) goto 130
            endif
         enddo
         do k=i+1,nrpt
            nkm=(k-1)/nrp
            kkm=mod(k-1,nrp)+1
            do j=1,i-1
               njm=(j-1)/nrp
               if (nkm.eq.njm) then
                  jjm=mod(j-1,nrp)+1
                  if (iexcl(jjm,kkm).ne.0) goto 130
               endif
            enddo
         enddo
         print*,' Atom',i,' is not connected to previous ones'
         nmols=nmols+1
         if (nmols.ge.maxmol) then
            print*,' ***** Increase MAXMOL'
            stop
         endif
         imols(nmols)=i
  130    continue
      enddo
C     There are NMOLS separate molecules, coordinates starting at IMOLS
      imols(nmols+1)=nrpt+1
      do i=1,nmols
         print*,' Molecule',i,' from', imols(i),' to',imols(i+1)-1
         write(16,*) ' Molecule',i,' from', imols(i),' to',imols(i+1)-1
      enddo
      
C     Read available crystal data to be fixed, for all molecules
      k=0
      do i=1,maxmol
         thefix(i)=1000
         psifix(i)=1000
         phifix(i)=1000
         xxfix(i)=1000
         yyfix(i)=1000
         zzfix(i)=1000
         do j=1,ndihz
            k=k+1
            hydfix(k)=1000
         enddo
         do j=1,ndihh
            k=k+1
            hydfix(k)=1000
         enddo
      enddo

      n=inpt('GRVFIX',regel)
      if (n.gt.3*nmols) then
         do i=1,nmols
            call realst(regel,3*i-1,xxfix(i))
            call realst(regel,3*i  ,yyfix(i))
            call realst(regel,3*i+1,zzfix(i))
         enddo
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in GRVFIX',n
         stop
      endif
      n=inpt('EULFIX',regel)
      if (n.gt.3*nmols) then
         do i=1,nmols
            call realst(regel,3*i-1,phifix(i))
            call realst(regel,3*i  ,thefix(i))
            call realst(regel,3*i+1,psifix(i))
         enddo
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in EULFIX',n
         stop
      endif
      n=inpt('HYDFIX',regel)
      if (n.gt.ngr*ndiha) then
         k=0
         do i=1,ngr
            do j=1,ndiha
               k=k+1
               call realst(regel,k+1,hydfix(k))
c               print*,k,hydfix(k)
            enddo
         enddo
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in HYDFIX',n-1
         stop
      endif
      icopt=1
      n=inpt('CELOPT',regel)
      if (n.ge.2) call intstr(regel,2,icopt)
      if (icopt.le.0) then
         do m=1,6
            iref(m)=0
         enddo
      endif

      doewc=.false.
      doew6=.false.
      doew=.false.
C     If desired, override standard cutoff radius from topology file
      n=inpt('CUTOFF',regel)
      if (n.ge.2) call realst(regel,2,cutoff)
      if (n.ge.3) then
         if (n.lt.8) then
            print*,' ***** Wrong number of items in CUTOFF',n
            stop
         endif
         call realst(regel,3,alphac)
         call realst(regel,4,alpha6)
         call realst(regel,5,hmaxc)
         call realst(regel,6,hmax6)
         call realst(regel,7,tole)
         call realst(regel,8,tolk)
      endif
      doewc=(icoul.gt.0 .and. alphac.gt.1.0d-9)
      doew6=(alpha6.gt.1.0d-9)
      doew=(doewc .or. doew6)
      if (doewc) inner=0
      if (abs(tole).gt.0.1d0 .or. abs(tolk).gt.0.1d0) then
         print*,' ***** Warning: TOLE, TOLK = ',tole,tolk
         write(16,*) ' ***** Warning: TOLE, TOLK = ',tole,tolk
      endif

      totm=0
      do i=1,nrp
         totm=totm+amass(i)
      enddo
      totm=totm*ngr

      write(16,*) ' cutoff    tole   pressure(atm)'
      write(16,51) cutoff,tole,pres
      write(16,*) '  inner   intra   icoul  iprint'
      write(16,55) inner,intra,icoul,iprint
      write(16,*) '  nrand   emax1   emax2    ebig'
      write(16,18) nrand,emax1,emax2,ebig
      write(16,*) '   nrpt   ndihz   ndihh  ireftr...'
      write(16,55) nrpt,ndihz,ndihh,(ireftr(m),m=1,nref)
      write(16,*) ' Dihedrals:'
      do i=1,ndiha
         write(16,50) iome(i),jome(i),kome(i),lome(i)
      enddo
      write(16,*) '   nrpt  itest1  itest2  itest3'
      write(16,55) nrpt,itest1,itest2,itest3
      write(16,*) ' ncycsd   tolsd   stpsd   nresd   shmsd       dtest'
      write(16,33) ncysd,tolsd,stpsd,nresd,shmsd,dtest
      write(16,*) ' ncyccg   tolcg   stpcg   depcg   shmcg'
      write(16,38) ncycg,tolcg,stpcg,depcg,shmcg
      write(16,36) (stcin(m),m=1,3)

      emax1=emax1*ngr
      emax2=emax2*ngr
      finner=16*atan(1.0d0)/(3*(nspgr+1))
      druk=0.061018d0*pres/(nspgr+1)
      do i=1,maxpar
         stdc(i)=stcin(3)
      enddo
      npar=0
      do i=1,6
         if (iref(i).gt.0) then
            npar=npar+1
            k=(i+2)/3
            stdc(npar)=stcin(k)
         endif
      enddo
      npar1=npar
      write(form10,99) npar1
      write(16,*) 'Number of cell parameters ', npar1
      write(16,*) 'Steepest descent parameters:'
      write(16,54) (stdc(m),m=1,npar)
      write(16,43) totm
      estvol=(dexp.lt.-0.001d0)
      if (dexp.gt.0.001d0) dexp=dexp/volcor
      if (abs(dexp).gt.0.001d0) vexp=0.0016605d0*(nspgr+1)*totm/dexp
C     dexp in g.cm-3, vexp in nm3.

  150 continue
      nmax=nrand
C     Check whether we have to determine an expected volume
      if (estvol) then
         nmax=nint(-dexp)
         call dens(nrp,amass,vexp)
         vexp=2*vexp*(nspgr+1)*ngr
         dexp=0.0016605d0*(nspgr+1)*totm/vexp
C        Start with twice the estimated volume
      endif
      
C     Determine for each dihedral the two rotatable fragments of the molecule
      k=0
      do i=1,ngr
         do j=(i-1)*ndiha+1,i*ndiha
            k=k+1
            print 41,iome(j),jome(j),kome(j),lome(j)
            write(16,41) iome(j),jome(j),kome(j),lome(j)

C           Search for atoms linked to atom JOME(J), not through KOME(J)
            njlist(k)=1
            jlist(k,1)=jome(j)
  160       continue
            iadd=0
            do m=1,njlist(k)
               do l=1,nbond
                  ibl=ib(l)+(i-1)*nrp
                  jbl=jb(l)+(i-1)*nrp
                  if (ibl.eq.jlist(k,m)) then
                     iconn=jbl
                  elseif (jbl.eq.jlist(k,m)) then
                     iconn=ibl
                  else
                     goto 170
                  endif
                  if (iconn.eq.kome(j)) goto 170
                  do n=1,njlist(k)
                     if (iconn.eq.jlist(k,n)) goto 170
                  enddo
                  iadd=1
                  njlist(k)=njlist(k)+1
                  jlist(k,njlist(k))=iconn
  170             continue
               enddo
            enddo
c            print*,(jlist(k,n),n=1,njlist(k))
            if (iadd.eq.1) goto 160
            print 40,jome(j),(jlist(k,n),n=2,njlist(k))
            write(16,40) jome(j),(jlist(k,n),n=2,njlist(k))

C           Search for atoms linked to atom KOME(J), not through JOME(J)
            nklist=1
            klist(1)=kome(j)
  180       continue
            iadd=0
            do m=1,nklist
               do l=1,nbond
                  ibl=ib(l)+(i-1)*nrp
                  jbl=jb(l)+(i-1)*nrp
                  if (ibl.eq.klist(m)) then
                     iconn=jbl
                  elseif (jbl.eq.klist(m)) then
                     iconn=ibl
                  else
                     goto 185
                  endif
                  if (iconn.eq.jome(j)) goto 185
                  do n=1,nklist
                     if (iconn.eq.klist(n)) goto 185
                  enddo
                  iadd=1
                  nklist=nklist+1
                  klist(nklist)=iconn
  185             continue
               enddo
            enddo
c            print*,(klist(n),n=1,nklist)
            if (iadd.eq.1) goto 180
            print 40,kome(j),(klist(n),n=2,nklist)
            write(16,40) kome(j),(klist(n),n=2,nklist)

C           Check that the dihedral is not part of a ring
            do n=1,njlist(k)
               do m=1,nklist
                  if (jlist(k,n).eq.klist(m)) then
                     print 42,iome(j),jome(j),kome(j),lome(j)
                     write(16,42) iome(j),jome(j),kome(j),lome(j)
                     hydfix(k)=-2000
                     goto 190
                  endif
               enddo
            enddo
C           Check that the sum of the entries is correct
C           (A trivial error would occur for more than one molecule)
            if (nmols.eq.1 .and. njlist(k)+nklist.ne.nrp) then
               print*,' ***** Sumcheck error:',njlist(k),nklist,nrp
            endif
  190       continue
         enddo
      enddo

      idih=0
      print 66
      write(16,66)
      if (estvol) then
C        The volume estimation
         print 32,nmax
         write(16,32) nmax
         print 30,vexp
         write(16,30) vexp
      else
C        The actual calculation
         if (autmax.gt.0) then
            estmax=sqrt(vexp)*autmax
            print 52, estmax
            write(16,52) estmax
            if (estmax.gt.amax) amax=estmax
            if (estmax.gt.bmax) bmax=estmax
            if (estmax.gt.cmax) cmax=estmax
            da=amax-amin
            db=bmax-bmin
            dc=cmax-cmin
         endif
         print 35,nmax
         write(16,35) nmax
         write(16,*) 'Search limits:'
         write(16,56) iref(1),amin,amax
         write(16,56) iref(2),bmin,bmax
         write(16,56) iref(3),cmin,cmax
         write(16,56) iref(4),bxmin,bxmax
         write(16,56) iref(5),cxmin,cxmax
         write(16,56) iref(6),cymin,cymax
         if (abs(dexp).lt.0.001d0) then
            print*,' Search not limited by estimated volume'
            write(16,*) ' Search not limited by estimated volume'
         else
            print 60,dexp,vexp,volcor
            write(16,60) dexp,vexp,volcor
         endif
      endif

  200 continue
C     Start the next coordinate file
      idih=idih+1
      emin=1.0d20
C     Take a new minimum, otherwise there may be no end ....
      write(fin,26) 'cor.',idih
      open(unit=14,file=fin,status='OLD',err=840)
      volcon=10000
      if (.not. estvol) then
         print 25, fin
         write(16,25) fin
      endif
      npm=0
      call leesx(14,nfound,nrp,npm,0,0,titel,dumnam,idum,x,box,iprint)
      close(14)
c      ng1=nfound/nrp
c      print*,npm,' groups found on coordinate file'
c      write(16,*) npm,' groups found on coordinate file'
      print 28,npm,nrp,nfound
      write(16,28) npm,nrp,nfound
      ng2=ngr/npm
      if (npm*ng2.ne.ngr) then
         print*,' ***** This is incompatible with NGR =',ngr
         stop
      endif
      nc1=3*nrp*npm
C     If necessary, expand to NGR groups of NRP atoms each
      do j=2,ng2
         do i=1,nc1
            x(i+(j-1)*nc1)=x(i)
         enddo
      enddo

C     Transform to standard geometry for a free molecule
      do imol=1,nmols
         i1=imols(imol)
         i2=imols(imol+1)-1
         nn=i2-i1+1
         k=0
         do j=i1,i2
            do m=1,3
               k=k+1
               xscr(k)=x(3*j-3+m)
            enddo
c            print*,imol,j,(x(3*j-3+m),m=1,3)
         enddo
C        IREFTR refers to the same atom numbers in each independent molecule
C        This is not meaningful if these molecules are not identical
         call trans(nn,xscr,ireftr,xcm,scr,phi1,theta1,psi1)
         k=0
         do j=i1,i2
            do m=1,3
               k=k+1
               xr(3*j-3+m)=scr(k)
            enddo
         enddo
         print 79,imol,(xcm(m),m=1,3),phi1,theta1,psi1
         write(16,79) imol,(xcm(m),m=1,3),phi1,theta1,psi1
      enddo
c      print 93,(xr(m),m=1,nr3)

      if (iprint.gt.2) then
         do i=1,nrpt
            print 72, i,panm(i)(1:5),iac(i),(xr(3*(i-1)+m),m=1,3)
         enddo
      endif
      do i=1,ngr
         do k=1,ndiha
            tau(k)=rad*dihan((i-1)*ndiha+k,xr)
         enddo
         print 53, i,(tau(k),k=1,ndiha)
         write(16,53) i,(tau(k),k=1,ndiha)
      enddo

      if (.not. estvol .and. idih.eq.1) then
         open(unit=10,file='pack.10')
         if (iapp.eq.1) then
C           Read old file PACK10 and append new data
            read(10,11) regel
            read(10,94) i1,i2
            if (i1.ne.ndihz .or. i2.ne.ndihh) then
               print*,' ***** Error in pack.10: NDIH ',i1,ndihz,i2,ndihh
               stop
            endif
            do i=1,ndiha
               read(10,11) regel
            enddo
            read(10,94) i1
            if (i1.ne.nrp) then
               print*,' ***** Error in pack.10: NRP ',i1,nrp
               stop
            endif
            do i=1,nrp+3
               read(10,11) regel
            enddo
  220       continue
            read(10,form10,end=240,err=240) i1,i2
            ntry=i1
            nhit=i2
            goto 220
  240       continue
            print 16,ntry,nhit
            write(16,16) ntry,nhit
            if (iapp.eq.1) backspace(10)
         else
C           Create new file PACK10
            write(10,94) ispgr,-1
            write(10,94) ndihz,ndihh
            do i=1,ndihz
               write(10,95) iome(i),jome(i),kome(i),lome(i)
            enddo
            do i=ndihz+1,ndiha
               write(10,95) iome(i),jome(i),kome(i),lome(i)
            enddo
            write(10,94) nrp,(ireftr(i),i=1,nref)
            do i=1,nrp
               write(10,72) i,panm(i)(1:5),iac(i),(xr(3*(i-1)+m),m=1,3)
               write(16,72) i,panm(i)(1:5),iac(i),(xr(3*(i-1)+m),m=1,3)
            enddo
C           So coordinates on outputfile refer to the first conformation only!
            write(10,93) amin,bmin,cmin,bxmin,cxmin,cymin,
     +           0.0d0,0.0d0,0.0d0,0.0d0,0.0d0,0.0d0
            write(10,94) (iref(i),i=1,12)
c            write(16,94) (iref(i),i=1,12)
c            print 94, (iref(i),i=1,12)
         endif
      endif

C     Start random search
      ntest=0
      if (iapp.eq.0) ntry=0
      nhit1=0
      nhit2=0

  300 continue
C     Select the next random point, do a volume test first
      ntest=ntest+1
      if (estvol) then
         if (ntest.gt.1000000) then
            print*,'***** Very few solutions, ntest =',ntest,
     +           '   nhit =',nhit
            write(16,*) '***** Very few solutions, ntest =',ntest,
     +           '   nhit =',nhit
            estvol=.false.
            nhit=0
C           Stop density estimation and continue the procedure
            goto 150
         endif
         if (ntest.gt.50000 .and. ntry.eq.0) then
            print*,'***** Apparently no solutions, ntest =',ntest
            print*,'For a small molecule, decrease AMIN, BMIN, CMIN'
            stop
         endif
      endif
      ax=amin+random(iseed)*da
      by=bmin+random(iseed)*db
      cz=cmin+random(iseed)*dc
      if (dexp.gt.0.001d0) then
         ax=vexp/(by*cz)
         if (iprint.gt.3) print 17,dexp,vexp,ax,by,cz
         if (ax.lt.amin .or. ax.gt.amax) goto 800
      endif
C     Density test passed, this will become a new trial structure.
C     No test for specified order of cell axes necessary.
      ntry=ntry+1
      cel(1)=ax
      cel(2)=by
      cel(3)=cz
      cel(4)=(bxmin+random(iseed)*dbx)*ax
      cel(5)=(cxmin+random(iseed)*dcx)*ax
      cel(6)=(cymin+random(iseed)*dcy)*by
      npar=0
      do i=1,6
         if (iref(i).gt.0) then
            npar=npar+1
            par(npar)=cel(i)
c            k=(i+2)/3
c            stdc(npar)=stcin(k)
         endif
      enddo
      if ((ntry/1000)*1000 .eq.ntry) then
         if (iprint.gt.1) print 78,ntry,(cel(m),m=1,6)
         open(unit=15,file='STOP',status='OLD',err=305)
         read(15,*,err=304) nmax
         close(unit=15,status='DELETE')
         print*,' '
         print*,' NRAND changed to',nmax
         goto 305
  304    continue
         print*,' Stopped because of manual intervention'
         nmax=0
         goto 800
      endif
  305 continue

C     Loop over the separate molecules
      do imol=1,nmols
         theta=thefix(imol)
         if (abs(theta).gt.999) then
            ctheta=1.999999d0*(random(iseed)-0.5d0)
C           Take random values for cos(THETA) rather than for THETA
            theta=acos(ctheta)*rad
         else
            ctheta=cos(theta/rad)
         endif
         stheta=sin(theta/rad)
         psi=psifix(imol)
         if (abs(psi).gt.999) psi=random(iseed)*psimax
         cpsi=cos(psi/rad)
         spsi=sin(psi/rad)
         phi=phifix(imol)
         if (abs(phi).gt.999) phi=random(iseed)*phimax
         cphi=cos(phi/rad)
         sphi=sin(phi/rad)
         euler(1,1)= cpsi*cphi-spsi*sphi*ctheta
         euler(1,2)= cpsi*sphi+spsi*cphi*ctheta
         euler(1,3)= spsi*stheta
         euler(2,1)=-spsi*cphi-cpsi*sphi*ctheta
         euler(2,2)=-spsi*sphi+cpsi*cphi*ctheta
         euler(2,3)= cpsi*stheta
         euler(3,1)= sphi*stheta
         euler(3,2)=-cphi*stheta
         euler(3,3)= ctheta
c         if (abs(euler(2,1)).gt.0.01) goto 800
c         if (abs(euler(2,1)).lt.0.999) goto 800

C        Set molecular centers
         pos(1)=xxfix(imol)
         pos(2)=yyfix(imol)
         pos(3)=zzfix(imol)
         if (abs(pos(1)).gt.999) pos(1)=ax*random(iseed)
         if (abs(pos(2)).gt.999) pos(2)=by*random(iseed)
         if (abs(pos(3)).gt.999) pos(3)=cz*random(iseed)

         if (imol.eq.1) then
            do m=1,3
               cel(m+6)=pos(m)
               if (iref(m+6).ne.0) then
                  npar=npar+1
                  par(npar)=pos(m)
               endif
            enddo
         else
            par(npar+1)=pos(1)
            par(npar+2)=pos(2)
            par(npar+3)=pos(3)
            npar=npar+3
         endif
         par(npar+1)=phi/rad
         par(npar+2)=theta/rad
         par(npar+3)=psi/rad
         npar=npar+3

         i1=imols(imol)
         i2=imols(imol+1)-1
C        Determine individual centers of gravity
         nn=i2-i1+1
         k=0
         do i=i1,i2
            do m=1,3
               k=k+1
               xscr(k)=xr(3*i-3+m)
            enddo
         enddo
C        IREFTR refers to the same atom numbers in each independent molecule
C        This is not meaningful if these molecules are not identical
         call trans(nn,xscr,ireftr,xcm,scr,phi1,theta1,psi1)
c         print 79,imol,(xcm(m),m=1,3),phi1,theta1,psi1
c         write(16,79) imol,(xcm(m),m=1,3),phi1,theta1,psi1

C        Rotate reference geometry XR over Euler angles (phi,theta,psi) to X
         do i=i1,i2
            j=3*(i-1)
            do m=1,3
               xr(j+m)=xr(j+m)-xcm(m)
            enddo
            do m=1,3
               x(j+m)=pos(m)
               do n=1,3
                  x(j+m)=x(j+m)+euler(m,n)*xr(j+n)
               enddo
            enddo
         enddo
         if (iprint.gt.3) print 79,imol,(pos(m),m=1,3),phi,theta,psi
      enddo
C     End of loop over separate molecules

C     Set dihedral angles to random values if HYDFIX > 999, or
C     set them to prescribed values HYDFIX if ABS(HYDFIX) < 1000.
      k=0
ccc      if (iflex.ne.0) then
         do i=1,ngr
            do j=(i-1)*ndiha+1,i*ndiha
               k=k+1
               if (hydfix(k).lt.-999) goto 310
               if (hydfix(k).gt.999) then
                  omega=(random(iseed)*2*pi)
               else
                  omega=hydfix(k)/rad-dihan(j,xr)
               endif
               if (iprint.gt.4) print*,iome(j),jome(j),kome(j),lome(j),
     +              hydfix(k),omega*rad
               co=cos(omega)-1
               si=sin(omega)
               j3=3*jome(j)-3
               k3=3*kome(j)-3
               do n=1,njlist(k)
                  i3=3*jlist(k,n)-3
                  call introt(i3,j3,k3,x,arot,brot)
                  do m=1,3
                     x(i3+m)=x(i3+m)+si*brot(m)+co*arot(m)
                  enddo
               enddo
  310          continue
            enddo
         enddo
ccc      endif

      if (iprint.gt.6) call files(2,nhit+1,
     +     ' Starting random geometry',0.0d0,spname,nrpt,panm,x,cel)
C     This introduces minor changes in the coordinates X!
      if (iprint.gt.3 .and. ndiha.gt.0) then
         do i=1,ngr
            do k=1,ndiha
               tau(k)=rad*dihan((i-1)*ndiha+k,x)
            enddo
            print 77,i,(tau(k),k=1,ndiha)
         enddo
      endif

C     First energy calculation (no minimization), gives EN1.
      call nonbe(ngr,nrp,en1,0,scr,0,cel,x,scr,0)
      if (iprint.gt.3) then
         print 27,ntry,en1,(cel(m),m=1,6)
      endif
      if (en1.gt.ebig) goto 800

      nhit1=nhit1+1
C     Try this structure
      if (iflex.eq.0) then
C        Rigid-body energy minimization
         noint=.true.
         if (iprint.gt.2) then
            print 31,(par(m),m=1,npar)
            print 29,(cel(m),m=1,9)
            do m=1,ndiha*ngr
               tau(m)=rad*dihan(m,x)
            enddo
            print 39, (tau(m),m=1,ndiha*ngr)
         endif

         k=0
         do i=1,6
            if (iref(i).gt.0) then
               k=k+1
               box(i)=par(k)
            else
               box(i)=cel(i)
            endif
         enddo

C        Energy minimization, gives EPOT
c        call nonbe(ngr,nrp,en1,0,scr,0,cel,x,scr,5)
         if (dtest.gt.1.0d-10) call test(par,npar,func1,dtest)
         epot=en1
         if (ncysd.gt.0) then
            call stedes(npar,par,func1,ncysd,tolsd,ntry,stdc,stpsd,
     +           nresd,shmsd,iprint,epot,ncyc,rmsder)
            if (iprint.gt.2) print 49,'  SD:',ncyc,epot,
     +           (par(m),m=1,npar1)
         endif
         if (ncycg.gt.0) then
            call congra(npar,par,func1,ncycg,tolcg,ntry,stpcg,
     +           depcg,shmcg,iprint,epot,ncyc,rmsder)
            if (iprint.gt.2) print 49,'  CG:',ncyc,epot,
     +           (par(m),m=1,npar1)
         endif
         if (epot/ngr.lt.emin) emin=epot/ngr
         if (epot/ngr.gt.emin+emax1 .or. epot/ngr.gt.emax2) goto 650

         k=0
         do i=1,6
            if (iref(i).gt.0) then
               k=k+1
               box(i)=par(k)
            else
               box(i)=cel(i)
            endif
         enddo
      else

C        Full geometry relaxation
         npar=0
         do i=1,6
            if (iref(i).gt.0) then
               npar=npar+1
               par(npar)=cel(i)
            endif
         enddo
         if (iprint.gt.2) then
            print 31,(par(m),m=1,npar)
            do m=1,ndiha*ngr
               tau(m)=rad*dihan(m,x)
            enddo
            print 39, (tau(m),m=1,ndiha*ngr)
         endif
         do i=1,3
            if (iref(i+6).gt.0) then
               npar=npar+1
               par(npar)=x(i)
            endif
         enddo
         do i=4,nr3
            npar=npar+1
            par(npar)=x(i)
         enddo

         k=0
         do i=1,6
            if (iref(i).gt.0) then
               k=k+1
               box(i)=par(k)
            else
               box(i)=cel(i)
            endif
         enddo

C        Energy minimization, gives EPOT
c        call nonbe(ngr,nrp,en1,0,scr,0,cel,x,scr,5)
         if (dtest.gt.1.0d-10) call test(par,npar,func3,dtest)
         epot=en1
         if (ncysd.gt.0) then
            call stedes(npar,par,func3,ncysd,tolsd,ntry,stdc,stpsd,
     +           nresd,shmsd,iprs,epot,ncyc,rmsder)
            if (iprint.gt.2) print 49,'  SD:',ncyc,epot,
     +           (par(m),m=1,npar1)
         endif
         if (ncycg.gt.0) then
            call congra(npar,par,func3,ncycg,tolcg,ntry,stpcg,
     +           depcg,shmcg,iprd,epot,ncyc,rmsder)
            if (iprint.gt.2) print 49,'  CG:',ncyc,epot,
     +           (par(m),m=1,npar1)
         endif
         if (epot/ngr.lt.emin) emin=epot/ngr
         if (epot/ngr.gt.emin+emax1 .or. epot/ngr.gt.emax2) goto 650

         k=0
         do i=1,6
            if (iref(i).gt.0) then
               k=k+1
               box(i)=par(k)
            else
               box(i)=cel(i)
            endif
         enddo
      endif
      
      vol=par(1)*par(2)*par(3)
      volmol=vol/(ngr*(nspgr+1))


C     Look for knotted structures
      if (ispgr.gt.0 .and. doknot.gt.0) then
         open(unit=49,file='rings.def',status='OLD',iostat=ios)
         if (ios.ne.0) then
            print*,' ***** No file RINGS.DEF present'
            print*,' Trying to create one:' 
            write(16,*) ' ***** No file RINGS.DEF present'
            write(16,*) ' Trying to create one...' 
            call rings(nrp,nbond,ib,jb,iprint)
            print*,' Check the prepared file rings.def carefully!'
            print*,' First line should contain the number of rings'
            print*,' Then for every ring a line with:'
            print*,' number of atoms (NATR), atom numbers 1....NATR, 1'
            print*,' '
            print*,' If correct, then rerun the program'
            stop
         endif
         call catena(nrp,ngr,nspgr,box,x,panm,iprint,iknot)
         close(49)
         if (iknot.gt.0) then
            nknot=nknot+1
            print 61,iknot,nhit2
            write(16,61) iknot,nhit2
            goto 800
         endif
         if (iknot.eq.-100) then
            print*,' ***** Unrealistic bond distance: in Catena'
            write(16,*)'  ***** Unrealistic bond distance:'
            goto 800
         endif
      endif

C     An acceptable structure is found

C     Recalculate energy, including bonded terms
cc      call nonbe(ngr,nrp,epot,0,scr,0,box,x,scr,0)
cc      call cbond(ngr,nrp,x,scr,0,eb,0)
cc      call choek(ngr,nrp,x,scr,0,eh,0)
cc      call cdiha(ngr,nrp,x,scr,0,1,ei,0)
cc      call cdiha(ngr,nrp,x,scr,0,2,ep,0)
ccc         print*,enb,eb,eh,ei,ep
cc      epot=epot+eb+eh+ei+ep

      if (vol.lt.volcon) then
cc         if (epot/ngr.gt.emin+emax1 .or. epot/ngr.gt.emax2) goto 650
         volcon=vol
         if (vol.lt.volmin .and. epot/ngr.lt.50000) then
            volmin=vol
            dmax=0.0016605d0*(nspgr+1)*totm/volmin
            print 47,vol,dmax,epot/ngr,nhit
            write(16,47) vol,dmax,epot/ngr,nhit
            if (estvol) then
C              Improve density: use the smallest volume encountered 
C              multiplied by a volume correction of about 1.1.
               vexp=volmin*volcor
               dexp=0.0016605d0*(nspgr+1)*totm/vexp
            endif
         endif
      endif

C     Reduce fractional coordinates
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      call tri(nrpt,0,x,1,cosa,cosb,cosc)
      do i=1,nrpt
         x(3*i-2)=x(3*i-2)/ax
         x(3*i-1)=x(3*i-1)/bb
         x(3*i)=x(3*i)/cc
      enddo
      do imol=1,nmols
         i1=imols(imol)
         i2=imols(imol+1)-1
         do m=1,3
            scr(m)=x(3*i1-3+m)-x(m)
         enddo
         do j=i1,i2
            do m=1,3
               x(3*j-3+m)=x(3*j-3+m)-NINT(scr(m))
            enddo
         enddo
      enddo
      do i=1,nrpt
         x(3*i-2)=x(3*i-2)*ax
         x(3*i-1)=x(3*i-1)*bb
         x(3*i)=x(3*i)*cc
      enddo
      call tri(nrpt,0,x,-1,cosa,cosb,cosc)

      nhit2=nhit2+1
      nhit=nhit+1
      do m=1,ndiha*ngr
         tau(m)=rad*dihan(m,x)
      enddo
C     Print and write results
      if (.not. estvol) then
         if (abs(epot/ngr).gt.99999) epot=99999*ngr
         write(10,form10) mod(ntry,10000000),nhit,epot/ngr,
     +        (nint(1000*par(m)),m=1,npar1),
     +        nint(en1),nint(1000*volmol),
     +        (nint(tau(m)),m=1,ndiha*ngr),1
         write(19,97) nhit,(box(m),m=1,6)
         write(19,98) (x(i),i=1,nr3)
      endif

      eb=0
      eh=0
      ei=0
      ep=0
      if (iprint.gt.2) then
         call nonbe(ngr,nrp,enb,0,scr,0,box,x,scr,0)
         if (iflex.ne.0) then
            call cbond(ngr,nrp,x,scr,0,eb,0)
            call choek(ngr,nrp,x,scr,0,eh,0)
            call cdiha(ngr,nrp,x,scr,0,1,ei,0)
            call cdiha(ngr,nrp,x,scr,0,2,ep,0)
         endif
         epot=enb+eb+eh+ei+ep
      endif

c      if (nhit.gt.1323) then
c         iprs=7
c         iprd=7
c      endif

      if (iprint.gt.0) then
         den=0.0016605d0*totm/(ngr*volmol)
         if (rmsder.gt.999.99) rmsder=999.0
         print 34, ntest,ntry,nhit,geel,epot/ngr,wit,
     +        den,rmsder,(par(m),m=1,npar1)
c         write(16,34) ntest,ntry,nhit,geel,epot/ngr,wit,
c     +        volmol,rmsder,(par(m),m=1,npar1)
      endif
      if (iprint.gt.1) then
         print 37, eb/ngr,eh/ngr,ei/ngr,ep/ngr,enb/ngr,uinner/ngr
         do imol=1,nmols
            i1=imols(imol)
            i2=imols(imol+1)-1
            nn=i2-i1+1
            k=0
            do j=i1,i2
               do m=1,3
                  k=k+1
                  xscr(k)=x(3*j-3+m)
               enddo
            enddo
c            print*,(xscr(m),m=1,3*nn)
C           IREFTR refers to the same atom numbers in each independent molecule
C           This is not meaningful if these molecules are not identical
            call trans(nn,xscr,ireftr,xcm,scr,phi1,theta1,psi1)
            print 79,imol,(xcm(m),m=1,3),phi1,theta1,psi1
         enddo
      endif
      if (iprint.gt.1 .and.ndihh+ndihz.gt.0) then
         do i=1,ngr
            do k=1,ndiha
               tau(k)=rad*dihan((i-1)*ndiha+k,x)
            enddo
            print 80,epot/ngr,i,(tau(k),k=1,ndiha)
         enddo
      endif
      goto 800

  650 continue
      if (iprint.gt.1) print 62,emin,epot

C     End of this structure
  800 continue
C     Start a new random calculation if NHIT2 < NMAX
      if (nhit2.lt.nmax .and. nhit.lt.nmax) goto 300

C     Output for this conformation
      print*, ntest,' random cells tried'
      write(16,*) ntest,' random cells tried'
      print*, ntry, ' accepted and randomly filled'
      write(16,*) ntry, ' accepted and randomly filled'
      if (doknot.gt.0) then
         print*, nknot, ' knotted structures refused'
         write(16,*) nknot, ' knotted structures refused'
      endif
      print*, nhit1,' remain after initial energy calculation'
      print*, nhit2,' remain after energy optimization'
      write(16,*) nhit1,' remain after initial energy calculation'
      write(16,*) nhit2,' remain after energy optimization'
      if (nhit2.gt.0) then
         dconf=0.0016605d0*(nspgr+1)*totm/volcon
         print 65,volcon,dconf
         write(16,65) volcon,dconf
      endif
C     Start the next conformation
      goto 200
  840 print*,'No (further) coordinate files COR present'
      write(16,*) 'No (further) coordinate files COR present'

      if (estvol) then
         if (nhit.eq.0) then
            print*,' Volume estimation failed'
            stop
         endif
         estvol=.false.
         nhit=0
C        The density should now be estimated, continue the procedure
         goto 150
      endif

      print*,nhit,' structures found from all conformations'
      write(16,*) nhit,' structures found from all conformations'
      print 67,volmin,dmax
      write(16,67) volmin,dmax
      i=inpt('CHECK ',regel)

c   10 format(7(1x,e10.5))
   11 format(a)
c   12 format(16a5)
c   13 format(25i3)
c   14 format(5x,a5,i5,2f10.5)
c   15 format(15i5)
   16 format(' Previous file PACK10 will be extended after ntry =',
     +     i8,' and nhit =',i8)
   17 format(' Trial values for dexp, vexp, ax, by, cz',8f7.3)
   18 format(i8,5f8.1)
c   20 format(a80)
c   21 format(1x,a78)
c   22 format(a5,i3.3)
c   23 format(/,' E U L E R   S E T',i6)
c   24 format(' dpsi=',f8.3,'      phi, theta, psi: ',3f8.3)
   25 format(/,' N E X T    G E O M E T R Y    ',a8)
   26 format(a4,i3.3)
   27 format('   Initial energy',i10,f15.3,5x,6f6.3)
   28 format(i5,' groups of',i5,' atoms found from',i5, 
     +     ' atoms on ordered coordinate file')
   29 format('CEL ',9f10.4)
   30 format(' Start with twice the estimated volume:',f10.4)
   31 format(/,' Try the next structure: ',60f10.4)
   32 format(' Determine cell volume by random search ',
     +     'for',i5,' structures')
   33 format(i8,2f8.4,i8,f8.4,f12.7)
   34 format(i12,i12,i6,a,f12.3,a,' Dens =',f9.4,'  R =',f9.4,
     +     '   Parameters',6f8.4)
   35 format(' Start random search for',i10,' structures')
   36 format('  stcin: ',12f10.2)
   37 format(' Eb...enb, E-inner, E-pol: ', 4f7.2,f8.2,2f7.2)
   38 format(i8,4f8.4)
   39 format(' Dihedrals: ',15f8.1)
   40 format('    Connected to atom',i4,' are atoms',50i4)
   41 format(' Searching connections for dihedral',4i4)
   42 format(' ***** But this dihedral cannot take a random value ',
     +     ' since it is part of a ring:',4i4)
   43 format('Total mass:',f10.3)
   47 format(' Smaller volume found:',f10.4,'   (density',f10.4,')',
     +     '    with energy',f10.3,' at structure',i6)
   49 format(a5,i6,' cycles, energy',f8.2,'    Parameters:',10f10.4)
   50 format(4i4,f7.1,2i4,3x,10i4)
   51 format(f8.3,f8.4,f16.1)
   52 format(' Estimated maximum cell length', f12.5)
   53 format('    Group',i4,5x,' Dihedrals from COR file:',30f10.3)
   54 format(12f10.4)
   55 format(12i8)
   56 format(i8,3x,10f10.3)
   58 format(10i10)
   60 format(' Search limited by expected density',f8.4,
     +     ' and volume ',f8.4,' which is an expansion by ',f5.2)
   61 format(' Refused:',i3,' crossings found after trial',4i10)
   62 format(' Energy',2f12.3,' too high: refused')
   65 format(' Smallest volume for this conformation',f10.4,
     +     '   (density',f10.4,')')
   66 format(/,'---------------------------------------------------',
     +     '----------------------------')
   67 format(' Smallest volume encountered:',f10.4,
     +     '   (density',f10.4,')')
   72 format(i5,5x,a5,i5,6f8.4)
   77 format('   Group',i5,'    Dihedrals:', 15f7.1)
   78 format(' Trial',i10,5x,' Cell:',6f10.3)
   79 format(12x,'Molecule',i4,'     Center',3f8.4,'     Eulers',3f7.1)
   80 format(8x,f12.3,'   Group',i4,'   Dihedrals:',30f7.1)
c   92 format(4(3i5,3x))
   93 format(6f10.4)
   94 format(15i5)
   95 format(4i5,f10.3)
   97 format(i7,2x,7f8.4)
   98 format(15f8.5)
   99 format('(2i7,f10.3,1x,',i2,'i5,2x,i6,i5,1x,30i5)')
      end
