      program pack2

C     Second stage of crystal packing prediction.
C     Varies the same parameters as PACK1, plus NDIHH dihedral angles
C        for H-containing dihedrals.
C     Ends with full structure relaxation.

C     B. P. van Eijck, November 1997.
C     Input files:  IN.TP (unit 20), molecular topology
C                   IN.SPG (unit 11), space group
C                   IN.FF (unit 12), force field
C                   IN.PA (unit 13), code-directed input
C                   IN.PAR (unit 10), structure parameters
C                   PACK.101, PACK.102, ...(unit 10), structure parameters
C     Output files: OUT.PRI (unit 16), general results
C                   OUT.PAR (unit 20), generated structure parameters
C                   OUT.COR (unit 29), generated coordinates
C     OUT.PAR can be clustered by program CLUS to be used as input
C        for PACK3 or continued (e.g.rigid-body) treatment by PACK2.

C     File IN.TP (read by subroutine LEEST) is the formatted topology file
C        created by program CODES and/or by hand.
C     File IN.SPG (read by subroutine LEESR) contains the space group data
C        for triclinic, monoclinic, or orthorhombic space groups.
C     File IN.FF (read by subroutine LEEST) contains the force field
C     Files FORT.001, FORT.002,... contain structure parameters generated
C        by PACK1. If input is used from a previous calculation by PACK2
C        only FORT.001 is used.
C     File IN.PA contains data specific for this program.
C     See manual for details.
C     If IN.PAR is present, no hydroxyl positions are generated.

      implicit none
      external func2,func3
      include 'params.i'
      include 'cspgr.i'
      include 'cbond.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cnbhl.i'
      include 'cnbkl.i'
      include 'cpars.i'
      include 'cinds.i'
      include 'cwald.i'
      include 'cpola.i'
      include 'color.i'

      integer i,i3,icod1,icod2,icode,icoul,idih,idum,idum1,idum2,
     +     ifilc,ifild,ifilm,ifils,imult,indeul,indihh,inp21,inpt,
     +     intra,ionce,ios,ipar,iprint,irefin,iscr,ispgr,istage,
     +     istat1,istat2,istat3,j,j3,k,k3,k6,kk,kpar,m,maxinp,maxstt,
     +     mdih,n,n1,n2,nang,nangt,nclus,ncy1cg,ncy1sd,ncy2cg,ncy2sd,
     +     ncy3cg,ncy3sd,ncyc1,ncyc2,ndubb,nhit1,nhit1t,nhit2,nhit2t,
     +     nhit3,nhit3t,nhit4,nhit4t,ninp,ninpt,nintr,nintr1,nintr2,
     +     nmax1,nmax2,nmax3,npar0,npar1,npar2,nr1,nr3,nre1sd,nre2sd,
     +     nre3sd,nreul,nroth,nrpxx,ntot,nw,nword
      double precision arot,box,brot,co,d,dep1cg,dep2cg,dep3cg,der,
     +     dihan,dihin,dtest1,dtest2,e1,e2,e3,eb,eclus,ee,eh,ei,emax1,
     +     emax2,emax3,en0,en1,en2,en3,enb,eopth,ep,epot,frac1,frac2,
     +     frac3,omega,par,par0,parar,phi,pi,pres,psi,rad,rmsder,
     +     scr,shm1cg,shm1sd,shm2cg,shm2sd,shm3cg,shm3sd,si,stc,stcin,
     +     stdc,stp1cg,stp1sd,stp2cg,stp2sd,stp3cg,stp3sd,tau,tauar,
     +     tauinp,tclus,theta,tol1cg,tol1sd,tol2cg,tol2sd,tol3cg,
     +     tol3sd,versch,xcm,xstore

      character*8 spname
      character filenm*9,form20*60,regel*255
      dimension tauar(maxcl2,maxtau),parar(maxcl2,12),eclus(maxcl2,3),
     +     ndubb(maxcl2),eopth(maxpre),
     +     nr1(100),irefin(12),der(6),box(12),stcin(6),
     +     xcm(3),indeul(3),iscr(3),arot(3),brot(3),
     +     istat1(imin:imax),istat2(imin:imax),istat3(imin:imax),
     +     stdc(maxpar),par0(maxpar),ipar(maxpar),stc(maxpar),
     +     scr(maxpar),par(maxpar),xstore(maxxco,maxcnf),tau(maxome),
     +     tauinp(maxome,0:maxcnf),indihh(maxome),dihin(maxome)

      pi=4*atan(1.0d0)
      rad=180/pi
      rmsder=-9.99d0
      ngr=1

      do i=1,maxtau
         nsteph(i)=3
         Vn(i)=0.3d0
         ntal(i)=3
      enddo
      tclus=10
      irfat1=0
      irfat2=0

      ncy1sd=25
      tol1sd=0.6d0
      stp1sd=0.005d0
      nre1sd=8
      shm1sd=10
      ncy1cg=10
      tol1cg=0.1d0
      stp1cg=0.01d0
      dep1cg=0.3d0
      shm1cg=0.02d0

c      ncy2sd=10
      ncy2sd=0
      tol2sd=0.1d0
      stp2sd=0.001d0
      nre2sd=10
      shm2sd=0.02d0
c      ncy2cg=10
      ncy2cg=0
      tol2cg=0.5d0
      stp2cg=0.01d0
      dep2cg=0.3d0
      shm2cg=0.02d0

      ncy3sd=10
      tol3sd=0.1d0
      stp3sd=0.001d0
      nre3sd=10
      shm3sd=0.02d0
      ncy3cg=10
      tol3cg=0.01d0
      stp3cg=0.01d0
      dep3cg=0.3d0
      shm3cg=0.02d0

      nmax1=1000
      nmax2=500
      nmax3=100
      frac1=10
      frac2=0.1d0
      frac3=0.9d0

      inner=0
      intra=1
      icoul=1
      pres=0
      iprint=0
      maxinp=2000

      dtest1=0
      dtest2=0
      do i=1,6
         stcin(i)=1
      enddo
      stcin(5)=100
      ifils=0
      ifilm=0
      ifild=0
      ifilc=0
      maxstt=10
      ionce=1


      open(unit=16,file='out.pri')
      open(unit=13,file='in.pa',status='OLD',iostat=ios)
      print*,geel,' P A C K 2',wit
      write(16,*) ' '
      write(16,*) ' P A C K 2'
      open(unit=21,file='in.par',status='OLD',iostat=ios)
      if (ios.ne.0) then
         inp21=0
         print*,' Read data from PACK1 run'
         write(16,*) ' Read data from PACK1 run'
         open(unit=10,file='pack.101',status='OLD',iostat=ios)
         if (ios.ne.0) then
            print*, ' ***** No file PACK.101'
            stop
         endif
         read(10,23) regel
         close(10)
      else
         inp21=1
         print*,' Read data from previous PACK2 run'
         write(16,*) ' Read data from previous PACK2 run'
         read(21,23) regel
      endif
      close(21)
      spname='noname'
      call intstr(regel,1,ispgr)
      print*,' Space group',ispgr
      write(16,*) ' Space group',ispgr
      if (ios.ne.0) then
         print*,' No file IN.PA present'
         write(16,*) ' No file IN.PA present'
      else
         regel=' '
         n=inpt('INPUT ',regel)
         close(13)
      endif

      n=inpt('INNER ',regel)
      if (n.ge.2) call intstr(regel,2,inner)
      n=inpt('INTRA ',regel)
      if (n.ge.2) call intstr(regel,2,intra)
      noint=(intra.eq.0)
      n=inpt('COUL  ',regel)
      if (n.ge.2) call intstr(regel,2,icoul)
      n=inpt('MAXINP',regel)
      if (n.ge.2) call intstr(regel,2,maxinp)
      if (maxinp.lt.0) then
         nw=n-2
         do i=1,nw
            call intstr(regel,i+2,nr1(i))
         enddo
      endif
      n=inpt('CLUS  ',regel)
      if (n.ge.2) call realst(regel,2,tclus)
      n=inpt('PRINT ',regel)
      if (n.ge.2) call intstr(regel,2,iprint)
      n=inpt('FILES ',regel)
      if (n.ge.2) call intstr(regel,2,ifils)
      if (n.ge.3) call intstr(regel,3,ifilm)
      if (n.ge.4) call intstr(regel,4,ifild)
      if (n.ge.5) call intstr(regel,5,ifilc)
      istage=1
      n=inpt('EMAX  ',regel)
      if (n.ge.4) then
         istage=0
         call realst(regel,2,emax1)
         call realst(regel,3,emax2)
         call realst(regel,4,emax3)
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in EMAX',n
         stop
      endif
      n=inpt('STAT  ',regel)
      if (n.ge.7) then
         call intstr(regel,2,nmax1)
         call intstr(regel,3,nmax2)
         call intstr(regel,4,nmax3)
         call realst(regel,5,frac1)
         call realst(regel,6,frac2)
         call realst(regel,7,frac3)
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in STAT ',n
         stop
      endif
      n=inpt('STC   ',regel)
      if (n.ge.7) then
         call realst(regel,2,stcin(1))
         call realst(regel,3,stcin(2))
         call realst(regel,4,stcin(3))
         call realst(regel,5,stcin(4))
         call realst(regel,6,stcin(5))
         call realst(regel,7,stcin(6))
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in STC',n
         stop
      endif
      n=inpt('PRES  ',regel)
      if (n.ge.2) call realst(regel,2,pres)
      n=inpt('TEST  ',regel)
      if (n.ge.2) call realst(regel,2,dtest1)
      if (n.ge.3) call realst(regel,3,dtest2)
      n=inpt('SD1   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy1sd)
      if (n.ge.3) call realst(regel,3,tol1sd)
      if (n.ge.4) call realst(regel,4,stp1sd)
      if (n.ge.5) call intstr(regel,5,nre1sd)
      if (n.ge.6) call realst(regel,6,shm1sd)
      n=inpt('CG1   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy1cg)
      if (n.ge.3) call realst(regel,3,tol1cg)
      if (n.ge.4) call realst(regel,4,stp1cg)
      if (n.ge.5) call realst(regel,5,dep1cg)
      if (n.ge.6) call realst(regel,6,shm1cg)
      n=inpt('SD2   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy2sd)
      if (n.ge.3) call realst(regel,3,tol2sd)
      if (n.ge.4) call realst(regel,4,stp2sd)
      if (n.ge.5) call intstr(regel,5,nre2sd)
      if (n.ge.6) call realst(regel,6,shm2sd)
      n=inpt('CG2   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy2cg)
      if (n.ge.3) call realst(regel,3,tol2cg)
      if (n.ge.4) call realst(regel,4,stp2cg)
      if (n.ge.5) call realst(regel,5,dep2cg)
      if (n.ge.6) call realst(regel,6,shm2cg)
      n=inpt('SD3   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy3sd)
      if (n.ge.3) call realst(regel,3,tol3sd)
      if (n.ge.4) call realst(regel,4,stp3sd)
      if (n.ge.5) call intstr(regel,5,nre3sd)
      if (n.ge.6) call realst(regel,6,shm3sd)
      n=inpt('CG3   ',regel)
      if (n.ge.2) call intstr(regel,2,ncy3cg)
      if (n.ge.3) call realst(regel,3,tol3cg)
      if (n.ge.4) call realst(regel,4,stp3cg)
      if (n.ge.5) call realst(regel,5,dep3cg)
      if (n.ge.6) call realst(regel,6,shm3cg)

      call leesr(ispgr,spname,iref,iscr,msom,inner)
      call leest(nrp,icoul,iprint)
C     This gives NRP, the number of atoms in one molecule.
      nr3=3*nrp
C     If desired, override standard settings from topology file
      n=inpt('POL   ',regel)
      if (n.ge.2) call intstr(regel,2,npp)
      if (n.ge.3) then
         if (n.lt.9) then
            print*,' ***** Wrong number of items in POL',n
            stop
         endif
         call realst(regel,3,poltol)
         call realst(regel,4,schaal)
         call realst(regel,5,gampol)
         call intstr(regel,6,nnn)
         call intstr(regel,7,mmm)
         call intstr(regel,8,nte)
         call intstr(regel,9,nttt)
      endif
      n=inpt('CUTOFF',regel)
      if (n.ge.2) call realst(regel,2,cutoff)
      if (n.ge.3) then
         if (n.lt.8) then
            print*,' ***** Wrong number of items in CUTOFF',n
            stop
         endif
         call realst(regel,3,alphac)
         call realst(regel,4,alpha6)
         call realst(regel,5,hmaxc)
         call realst(regel,6,hmax6)
         call realst(regel,7,tole)
         call realst(regel,8,tolk)
      endif
      doewc=(icoul.gt.0 .and. alphac.gt.1.0d-9)
      doew6=(alpha6.gt.1.0d-9)
      doew=(doewc .or. doew6)
      if (doewc) inner=0
      if (abs(tole).gt.0.1d0 .or. abs(tolk).gt.0.1d0) then
         print*,' ***** Warning: TOLE, TOLK = ',tole,tolk
         write(16,*) ' ***** Warning: TOLE, TOLK = ',tole,tolk
      endif
      n=inpt('STEPH ',regel)
      if (n.gt.0) then
         if (n.lt.ndihh+1) then
            print*,' Wrong number of entries in STEPH',n,ndihh
            stop
         endif
         do i=1,ndihh
            call intstr(regel,i+1,nsteph(i))
         enddo
      endif
      n=inpt('VN    ',regel)
      if (n.gt.0) then
         if (n.lt.ndihh+1) then
            print*,' Wrong number of entries in VN',n,ndihh
            stop
         endif
         do i=1,ndihh
            call realst(regel,i+1,Vn(i))
         enddo
      endif
C     Change input from kcal to kJ
      do i=1,ndihh
         Vn(i)=Vn(i)*4.184d0
      enddo
      n=inpt('NTAL  ',regel)
      if (n.gt.0) then
         if (n.lt.ndihh+1) then
            print*,' Wrong number of entries in NTAL',n,ndihh
            stop
         endif
         do i=1,ndihh
            call intstr(regel,i+1,ntal(i))
         enddo
      endif
      nroth=inpt('ROTH  ',regel)
      if (nroth.gt.0) then
         if (nroth.lt.ndihh+1) then
            print*,' Wrong number of entries in ROTH',n,ndihh
            stop
         endif
C        Starting values for hydroxyl dihedral angles
         do i=1,ndihh
            call realst(regel,i+1,dihin(i))
         enddo
      endif

      nang=0
      nangt=1
      do i=1,ndihh
         if (nsteph(i).lt.1 .or. inp21.eq.1) nsteph(i)=1
         nangt=nangt*nsteph(i)
         if (nsteph(i).gt.nang) nang=nsteph(i)
      enddo

      frac1=frac1*ndihh
      if (ndihh.eq.0 .or. inp21.eq.1) then
         istage=0
         emax1=10000
         emax2=10000
         emax3=10000
         n=inpt('EMAX  ',regel)
         if (n.ge.4) then
            call realst(regel,2,emax1)
            call realst(regel,3,emax2)
            call realst(regel,4,emax3)
         endif
      endif
      write(16,*) '  inner   intra   icoul     npp'
      write(16,34) inner,intra,icoul,npp
      write(16,36) pres
      write(16,*) ' cutoff  alphac  alpha6   hmaxc   hmax6',
     +     '      tole      tolk'
      write(16,32) cutoff,alphac,alpha6,hmaxc,hmax6,tole,tolk
      if (npp.gt.1) then
         write(16,*) '    npp  poltol  schaal   gamma     nnn     mmm',
     +        '     nte    nttt'
         write(16,39) npp,poltol,schaal,gampol,nnn,mmm,nte,nttt
      endif
      write(16,*) '    nrp   ndihz   ndihh  ireftr...'
      write(16,34) nrp,ndihz,ndihh,(ireftr(m),m=1,3)
      write(16,*) ' maxinp    nang   nangt   tclus'
      write(16,40) maxinp,nang,nangt,tclus
      if (inp21.eq.0) then
         write(16,*) '  nmax1   nmax2   nmax3   frac1   frac2   frac3'
         write(16,57) nmax1,nmax2,nmax3,frac1,frac2,frac3
      endif
      write(16,*) '  ifils   ifilm   ifild   ifilc  iprint'
      write(16,34) ifils,ifilm,ifild,ifilc,iprint
      write(16,*) ' ncycsd   tolsd   stpsd   nresd   shmsd       dtest'
      write(16,33) ncy1sd,tol1sd,stp1sd,nre1sd,shm1sd,dtest1
      write(16,33) ncy2sd,tol2sd,stp2sd,nre2sd,shm2sd
      write(16,33) ncy3sd,tol3sd,stp3sd,nre3sd,shm3sd,dtest2
      write(16,*) ' ncyccg   tolcg   stpcg   depcg   shmcg'
      write(16,38) ncy1cg,tol1cg,stp1cg,dep1cg,shm1cg
      write(16,38) ncy2cg,tol2cg,stp2cg,dep2cg,shm2cg
      write(16,38) ncy3cg,tol3cg,stp3cg,dep3cg,shm3cg
      if (nroth.gt.0) then
         write(16,*) ' Reset starting hydroxyl angles to:'
         write(16,51) (dihin(i),i=1,ndihh)
      endif

      if (nang.gt.maxint) then
         print*,' ***** Increase MAXINT',nang,maxint
         stop
      endif
      if (nang.gt.0) then
         j=nang**ndihh
         if (j.gt.maxpre) then
            print*,j,' ***** Increase MAXPRE'
            stop
         endif
      endif
      tclus=tclus/rad
      ebig=10000

 185  continue
      do i=imin,imax
         istat1(i)=0
         istat2(i)=0
         istat3(i)=0
      enddo

 190  continue
C     Start a new stage in the calculation
      if (istage.eq.0) then
         print*, 'Start full calculation'
         write(16,*) 'Start full calculation'
      endif
      ntot=0
      ninpt=0
      nhit1t=0
      nhit2t=0
      nhit3t=0
      nhit4t=0
      idih=0

 300  idih=idih+1
      if (idih.gt.maxcnf .and. inp21.gt.0) then
         print*,' ***** Increase MAXCNF'
         stop
      endif
C     Open the next PACK file with a different molecular conformation
      write(filenm,22) 'pack.',idih+100
      open(unit=10,file=filenm,status='OLD',iostat=ios)
      if (ios.ne.0) then
         print*,' No file ',filenm
         write(16,*) ' No file ',filenm
         if (inp21.eq.0) goto 910
C        End of reading stage for files pack.1##
         if (idih.eq.1 .and. ndihz.gt.0) then
            print 29
            stop
         endif
         inp21=2
         idih=idih-1
         filenm='in.par'
         open(unit=10,file=filenm,status='OLD',iostat=ios)
         print 26
         write(16,26)
         if (ios.ne.0) then
            print*,' ***** No file IN.PAR'
            stop
         endif
      endif

      read(10,23) regel
      call intstr(regel,1,idum)
      if (nword(regel).gt.1) then
         call intstr(regel,2,icode)
      else
         print*,' Outdated PACK-file, adjust manually!'
         stop
      endif
C     icode=1: file made by PACK1
C     icode>1: repeated rigid-body calculation; in every calculation
C              with PACK2 or PACK3 icode is increased by 1.
C     If icode>1 the structure numbering remains unchanged.
      if (ionce.eq.1) then
         ionce=0
         finner=16*atan(1.0d0)/(3*(nspgr+1))
         druk=0.061018d0*pres/(nspgr+1)
         kpar=0
         nreul=0
         do i=1,12
            if (iref(i).gt.0) then
               kpar=kpar+1
               m=(i+2)/3
               stdc(kpar)=stcin(m)
               if (i.gt.9) then
                  nreul=nreul+1
                  indeul(nreul)=kpar
               endif
            endif
         enddo
         do i=1,nr3+12
            stc(i)=stcin(6)
         enddo
         npar0=0
         do i=1,6
            if (iref(i).gt.0) then
               npar0=npar0+1
               stc(npar0)=stdc(npar0)
            endif
         enddo
         do m=kpar+1,kpar+ndihh
            stdc(m)=stcin(5)
         enddo
         npar1=kpar+ndihh
         write(form20,96) kpar+ndiha
         write(16,*) ' Numbers of parameters ', kpar,ndihh
         write(16,*) ' Steepest descent parameters:'
         write(16,53) (stdc(m),m=1,kpar)
         write(16,53) (stdc(m),m=kpar+1,npar1)
         write(16,53) stcin(6)
      endif
      if (idih.eq.1) then
         if (istage.eq.1) then
            print 63,1,nmax1,frac1
            write(16,63) 1,nmax1,frac1
         endif
         if (istage.eq.2) then
            if (frac2.gt.1.0) then
               emax2=imax
               print 64,2,emax2
               write(16,64) 2,emax2
               istage=3
            else
               print 63,2,nmax2,frac2
               write(16,63) 2,nmax2,frac2
            endif
         endif
         if (istage.eq.3) then
            if (frac3.gt.1.0) then
               emax3=imax
               print 64,3,emax3
               write(16,63) 2,emax3
               istage=0
               do i=imin,imax
                  istat1(i)=0
                  istat2(i)=0
                  istat3(i)=0
               enddo
               print 31,emax1,emax2,emax3
               write(16,31) emax1,emax2,emax3
               if (maxinp.eq.0) stop
               print*, 'Start full calculation'
               write(16,*) 'Start full calculation'
            else
               print 63,3,nmax3,frac3
               write(16,63) 3,nmax3,frac3
            endif
         endif
      endif

      read(10,*) n1,n2
      if (n1.ne.ndihz) then
         print*,' ***** Error in PACK-file',n1,ndihz
      endif
      do i=1,n1+n2
         read(10,*) idum
      enddo
      read(10,94) nrpxx
      do i=1,nrpxx
         read(10,72) k,naam(i),j,(xr(3*i-3+m),m=1,3)
      enddo
      if (inp21.eq.1) then
         do i=1,nr3
            xstore(i,idih)=xr(i)
         enddo
      endif
      read(10,93) (cel(i),i=1,12)
      read(10,94) (irefin(i),i=1,12)
      do i=1,12
         if (irefin(i).ne.iref(i)) then
            print*,' iref',i,' inconsistent:',irefin(i),iref(i)
            print*,' ******* RESULTS UNRELIABLE !!'
         endif
      enddo

      if (istage.eq.0 .and. inp21.lt.2) then
         print 21,filenm
         write(16,21) filenm
      endif
      if (ndiha.gt.0) then
         do m=1,ndiha
            tau(m)=rad*dihan(m,xr)
            if (inp21.gt.0) tauinp(m,idih)=tau(m)
         enddo
         if (istage.eq.0) then
            print*,' Dihedrals from input coordinates:'
            print 51,(tau(m),m=1,ndiha)
            write(16,*) ' Dihedrals from input coordinates:'
            write(16,51) (tau(m),m=1,ndiha)
         endif
      endif

      if (inp21.eq.1) goto 300


      if (nroth.gt.0) then
C        Reset hydroxyl dihedral angles to given values DIHIN
         do i=1,ndihh
            omega=(dihin(i)-tau(ndihz+i))/rad
            tau(ndihz+i)=dihin(i)
            co=cos(omega)-1
            si=sin(omega)
            i3=3*iome(ndihz+i)-3
            j3=3*jome(ndihz+i)-3
            k3=3*kome(ndihz+i)-3
            call introt(i3,j3,k3,xr,arot,brot)
            do m=1,3
               xr(i3+m)=xr(i3+m)+si*brot(m)+co*arot(m)
            enddo
         enddo
      endif
      if (istage.eq.0 .and. ndihh.gt.0) then
         print*, ' Hydroxyl dihedrals      nsteph      Vn  ntal'
         write(16,*) ' Hydroxyl dihedrals      nsteph      Vn  ntal'
         do i=1,ndihh
            j=ndihz+i
            print 50, iome(j),jome(j),kome(j),lome(j),
     +           tau(ndihz+i),nsteph(i),Vn(i),ntal(i)
            write(16,50) iome(j),jome(j),kome(j),lome(j),
     +           tau(ndihz+i),nsteph(i),Vn(i),ntal(i)
         enddo
      endif

      ninp=0
      nhit1=0
      nhit2=0
      nhit3=0
      nhit4=0
      if (idih.eq.1 .or. inp21.eq.2) then
         do i=1,80
            regel(i:i)=' '
         enddo
         if (istage.eq.0) then
            open(unit=29,file='out.cor')
            open(unit=20,file='out.par')
            write(20,94) ispgr,icode+1
            write(20,94) ndihz,ndihh
            do m=1,ndiha
               write(20,94) iome(m),jome(m),kome(m),lome(m)
            enddo
            write(20,94) nrp,(ireftr(i),i=1,3)
            do i=1,nrp
               write(20,72) i,panm(i)(1:5),iac(i),(xr(3*i-3+m),m=1,3)
            enddo
C           So coordinates on outputfile refer to the first conformation only!
            write(20,93) (cel(i),i=1,12)
            write(20,94) (iref(i),i=1,12)
         endif
      endif

C     Read a new line from the PACK file
 500  continue
      read(10,28,end=900) regel
      if (inp21.eq.0) then
         read(regel,99,err=502) idum1,idum2,en0,(ipar(m),m=1,kpar)
         imult=1
      else
         read(regel,99,err=502) idum1,idum2,en0,
     +        (ipar(m),m=1,kpar+ndiha)
         n=nword(regel)
         if (iprint.gt.2) 
     +        print 99, idum1,idum2,en0,(ipar(m),m=1,kpar+ndiha)
         call intstr(regel,n,imult)
      endif
      goto 504
 502  print*,' ***** Error reading file ',idih,idum1,idum2
      write(16,*) ' ***** Error reading file ',idih,idum1,idum2
      goto 500
 504  continue
      if (idum1.eq.0 .or. idum2.eq.0) then
         print*,' Suspect code numbers in ',idih,idum1,idum2
         write(16,*) ' Suspect code numbers in ',idih,idum1,idum2
         goto 500
      endif
      if (istage.eq.0) then
         if (maxinp.lt.0) then
            do i=1,nw
               if (nr1(i).eq.idum2) goto 508
            enddo
            goto 500
         else
            if (ninp.ge.maxinp) goto 900
         endif
      endif
      if (istage.eq.1 .and. ninp.ge.nmax1) goto 900

 508  continue
C     Start the calculation
      ninp=ninp+1
      if (iprint.gt.2) then
         print*, ' '
         print*,'++++++++++++++++++++++++++++++++++++++++++++++++++++++'
         print*,' New entry read from file ',filenm
         if (inp21.eq.0) then
            print 99, idum1,idum2,en0,(ipar(m),m=1,kpar)
         else
            print 99, idum1,idum2,en0,(ipar(m),m=1,kpar+ndiha)
         endif
      endif
      do m=1,kpar
         par0(m)=dfloat(ipar(m))/1000
      enddo
      do i=1,nreul
         par0(indeul(i))=par0(indeul(i))*100/rad
      enddo
      do i=1,ndihh
         indihh(i)=0
      enddo
      nclus=0
      nintr=0
      nintr1=0
      nintr2=0

      if (inp21.eq.0) then
C        Do a fast energy calculation for all hydroxyl dihedrals
         call opth(nang,eopth,iprint,par0)
         if (iprint.gt.2) print*, (nplh(m),m=1,ndihh)
         m=0
         do i=1,nangt
            k=nint(eopth(i))
            if (k.lt.imin) then
               print*,' ***** Change IMIN',1,k
               stop
            endif
            if (k.le.imax) istat1(k)=istat1(k)+1
            if (k.gt.ebig) m=k
         enddo
         if (m.gt.0) then
            print*,' ***** Note large EOPTH:',m,idih,idum1,idum2
            write(16,*) ' ***** Note large EOPTH:',m,idih,idum1,idum2
         endif
         if (istage.eq.1) goto 500
      endif

 560  nintr=nintr+1
C     Consider the next hydroxyl dihedral (defined by array INDIHH)
      do k=1,kpar
         par(k)=par0(k)
      enddo
      m=1
      kk=1
      do i=1,ndihh
         par(kpar+i)=indihh(i)*2*pi/nsteph(i)
         kk=kk+m*indihh(i)
         m=m*nang
      enddo

      if (inp21.eq.0) then
         en1=eopth(kk)
         if (en1.gt.emax1) goto 690
         if (iprint.gt.2 .or. ncy1sd+ncy1cg.eq.0)
     +        call func2(npar1,par,scr,ee,1)
C        This is necessary to fill array X under all circumstances
         if (iprint.gt.2) then
            print*,' New dihedral angle combination'
            print 42, nintr,en0,en1,ee,(indihh(m),m=1,ndihh)
C           EN1 and EE may be different due to change in pair list
         endif
      else
         mdih=0
         if (idih.gt.0) then
C           Use individual dihedral angles from input line,
C           selecting the proper conformation file pack.1##
            do i=1,idih
               do m=1,ndihz
                  d=abs(dfloat(ipar(kpar+m))-tauinp(m,i))
                  if (d.gt.180) d=d-360
                  if (abs(d).gt.45) goto 600
               enddo
               mdih=i
  600          continue
            enddo
            if (mdih.eq.0) then
               print*,' No suitable coordinate set for',ninp,
     +              (ipar(kpar+m),m=1,ndihz)
               goto 500
            endif
            do i=1,nr3
               xr(i)=xstore(i,mdih)
            enddo
         else
C           Use geometry from file IN.PAR for rigid molecule
         endif
C        Individual hydroxyl dihedral angles from input line
         do m=1,ndihh
            par(kpar+m)=(ipar(kpar+ndihz+m)-tauinp(ndihz+m,mdih))/rad
         enddo
         call func2(npar1,par,scr,ee,1)
         if (iprint.gt.2) then
            print 27,mdih,en0,ee,
     +           (dfloat(ipar(kpar+m)),rad*dihan(m,x),m=1,ndihz)
C           EN0 contains intramolecular energy, EE does not.
         endif
      endif
      if (iprint.gt.2) then
         print 43, (rad*par(kpar+m),rad*dihan(ndihz+m,x),m=1,ndihh)
C        PAR is the shift from the starting angle, DIHAN is the actual angle
      endif
      nhit1=nhit1+1
      nintr1=nintr1+1

      en2=en1
C     Do semi-rigid body energy minimization
      if (dtest1.gt.1.0e-10) call test(par,npar1,func2,dtest1)
      ncyc1=0
      ncyc2=0
      if (ncy1sd.gt.0) then
         call stedes(npar1,par,func2,ncy1sd,tol1sd,idum2,stdc,stp1sd,
     +        nre1sd,shm1sd,iprint,en2,ncyc1,rmsder)
         if (iprint.gt.2) print 45,' SD1:',ncyc1,en2,(par(m),m=1,npar1)
         k=nint(en2)
         if (k.lt.imin) then
            print*,' ***** Change IMIN',2,k
            stop
         endif
         if (k.le.imax) istat2(k)=istat2(k)+1
         if (istage.eq.2) then
            if (nhit1.ge.nmax2) goto 900
            goto 690
         endif
      endif
      en3=en2
      if (inp21.eq.0 .and. en2.gt.emax2) goto 690
      nhit2=nhit2+1
      nintr2=nintr2+1
      if (ncy1cg.gt.0) then
         call congra(npar1,par,func2,ncy1cg,tol1cg,idum2,stp1cg,
     +        dep1cg,shm1cg,iprint,en3,ncyc2,rmsder)
         if (iprint.gt.2) print 45,' CG1:',ncyc2,en3,(par(m),m=1,npar1)
         k=nint(en3)
         if (k.lt.imin) then
            print*,' ***** Change IMIN',3,k
            stop
         endif
         if (k.le.imax) istat3(k)=istat3(k)+1
         if (istage.eq.3) then
            if (nhit2.ge.nmax3) goto 900
            goto 690
         endif
      endif
      if (inp21.eq.0 .and. en3.gt.emax3) goto 690
      if (iprint.gt.2) print 43,
     +     (rad*par(kpar+m),rad*dihan(ndihz+m,x),m=1,ndihh)
      nhit3=nhit3+1
      k=0
      do i=1,ndihh
         tau(i)=par(kpar+i)
      enddo

C     Was this combination of dihedrals already found?
      do i=1,nclus
         do j=1,ndihh
            versch=abs(tau(j)-tauar(i,j))
            if (versch.gt.pi) versch=abs(versch-2*pi)
            if (versch.gt.tclus) goto 670
         enddo

C        Yes, it was: choose the one with lowest energy.
         ndubb(i)=ndubb(i)+1
         if (eclus(i,3).lt.en3) goto 690
         eclus(i,1)=en1
         eclus(i,2)=en2
         eclus(i,3)=en3
         do k=1,ndihh
            tauar(i,k)=tau(k)
         enddo
         do k=1,kpar
            parar(i,k)=par(k)
         enddo
         goto 690
 670     continue
      enddo

C     No, it was not: we have a new one here
      nclus=nclus+1
      if (nclus.gt.maxcl2) then
         print*,' ***** Increase maxcl2'
         stop
      endif
      ndubb(nclus)=1
      eclus(nclus,1)=en1
      eclus(nclus,2)=en2
      eclus(nclus,3)=en3
      do i=1,ndihh
         tauar(nclus,i)=tau(i)
      enddo
      do i=1,kpar
         parar(nclus,i)=par(i)
      enddo

C     Start working on the next set of hydroxyl dihedrals
 690  continue
      i=1
 692  continue
      if (i.gt.ndihh) goto 694
      indihh(i)=indihh(i)+1
      if (indihh(i).ge.nsteph(i)) then
         indihh(i)=0
         i=i+1
         goto 692
      endif
      if (istage.gt.1 .and. nintr.gt.maxstt) goto 500
C     Avoid too many data from one basis structure in the statistics.
      goto 560
 694  continue
C     End of loop over all OH dihedrals for this structure

C     Do energy minimization for clustered structures
      if (inp21.eq.0) then
         if (iprint.gt.2) print*,
     +        '          ------------------------------------'
         if (iprint.gt.0 .and. nintr1.gt.0) print 35,
     +        idum1,idum2,nintr1,nintr2,nclus
      endif

      do k6=1,nclus
         nhit4=nhit4+1
         ntot=ntot+1

C        Do second semi-rigid body energy minimization
         do i=1,kpar
            par(i)=parar(k6,i)
         enddo
         do i=1,ndihh
            par(kpar+i)=tauar(k6,i)
         enddo
         ncyc1=0
         ncyc2=0
         if (iprint.gt.2 .or. ncy2sd+ncy2cg.eq.0)
     +        call func2(npar1,par,scr,ee,1)
C        This is necessary to fill array X under all circumstances
         if (iprint.gt.2) print 24,ee
         en2=ee
         if (ncy2sd.gt.0) then
            call stedes(npar1,par,func2,ncy2sd,tol2sd,idum2,stdc,stp2sd,
     +           nre2sd,shm2sd,iprint,en2,ncyc1,rmsder)
            if (iprint.gt.2) print 45,' SD2:',
     +           ncyc1,en2,(par(m),m=1,npar1)
         endif
         en3=en2
         if (ncy2cg.gt.0) then
            call congra(npar1,par,func2,ncy2cg,tol2cg,idum2,stp2cg,
     +           dep2cg,shm2cg,iprint,en3,ncyc2,rmsder)
            if (iprint.gt.2) print 45,' CG2:',
     +           ncyc2,en3,(par(m),m=1,npar1)
         endif

C        Do fully flexible energy minimization
         npar2=0
         do i=1,6
            if (iref(i).gt.0) then
               npar2=npar2+1
               par(npar2)=par(npar2)
            endif
         enddo
         do i=1,3
            if (iref(i+6).gt.0) then
               npar2=npar2+1
               par(npar2)=x(i)
            endif
         enddo
         do i=4,nr3
            npar2=npar2+1
            par(npar2)=x(i)
         enddo

         ncyc1=0
         ncyc2=0
         if (dtest2.gt.1.0e-10) call test(par,npar2,func3,dtest2)
         if (iprint.gt.2) print 25,en3
         if (ncy3sd.gt.0) then
            call stedes(npar2,par,func3,ncy3sd,tol3sd,idum2,stc,stp3sd,
     +           nre3sd,shm3sd,iprint,epot,ncyc1,rmsder)
            if (iprint.gt.2) print 45,' SD3:',
     +           ncyc1,epot,(par(m),m=1,npar0)
         endif
         if (ncy3cg.gt.0) then
            call congra(npar2,par,func3,ncy3cg,tol3cg,idum2,stp3cg,
     +           dep3cg,shm3cg,iprint,epot,ncyc2,rmsder)
            if (iprint.gt.2) print 45,' CG3:',
     +           ncyc2,epot,(par(m),m=1,npar0)
         endif

C        Do a final energy calculation
         k=0
         do i=1,6
            if (iref(i).gt.0) then
               k=k+1
               box(i)=par(k)
            else
               box(i)=cel(i)
            endif
         enddo
         do i=1,3
            if (iref(i+6).gt.0) then
               k=k+1
               x(i)=par(k)
            endif
         enddo
         do i=4,nr3
            k=k+1
            x(i)=par(k)
         enddo
         call nonbe(1,nrp,enb,0,der,0,box,x,scr,0)
         call cbond(1,nrp,x,scr,0,eb,0)
         call choek(1,nrp,x,scr,0,eh,0)
         call cdiha(1,nrp,x,scr,0,1,ei,0)
         call cdiha(1,nrp,x,scr,0,2,ep,0)
         epot=enb+eb+eh+ei+ep

C        Determine position and orientation parameters in the crystal
         call trans(nrp,x,ireftr,xcm,scr,phi,theta,psi)
         do i=1,3
            box(i+6)=xcm(i)
         enddo
         box(10)=phi
         box(11)=theta
         box(12)=psi

         do m=1,ndiha
            tau(m)=rad*dihan(m,x)
         enddo
         k=0
         do i=1,12
            if (iref(i).gt.0) then
               k=k+1
               par(k)=box(i)
               if (i.gt.9) par(k)=par(k)/100
            endif
         enddo
         if (icode.le.1) then
C           From now on NTOT remains the code number for this structure
            icod1=idum2
            icod2=ntot
         else
            icod1=ntot
            icod2=idum2
         endif

C        Write parameters to unit 20 and coordinates to unit 29
         write(29,97) icod2,(box(i),i=1,6)
         write(29,98) (x(i),i=1,nr3)
         if (abs(epot).gt.99999) epot=99999
         if (inp21.eq.0) then
            write(20,form20) icod1,icod2,epot,
     +           (nint(1000*par(m)),m=1,kpar),(nint(tau(m)),m=1,ndiha),
     +           idum1,(nint(eclus(k6,m)),m=1,3),imult
         else
            write(20,form20) icod1,icod2,epot,
     +           (nint(1000*par(m)),m=1,kpar),(nint(tau(m)),m=1,ndiha),
     +           imult
         endif
         if (iprint.gt.1) then
            print 46, icod2,geel,epot,wit,rmsder,
     +           eb,eh,ei,ep,enb,uinner,epol
            print 47, (box(m),m=1,12)
            if (ndiha.gt.0) print 44, (tau(m),m=1,ndiha)
         endif
         if (iprint.gt.2) print*, '         npla = ',npla
         if (ifilm.ne.0) call files(1,icod2,regel,
     +        epot,spname,nrp,panm,x,box)
         if (ifils.ne.0) call files(ifils+1,icod2,regel,
     +        epot,spname,nrp,panm,x,box)
         if (ifild.eq.1) call fdat(1,ispgr,icod2,regel,
     +        epot,spname,nrp,ngr,x,box)
         if (ifild.eq.2) call fdat(4,ispgr,icod2,regel,
     +        epot,spname,nrp,ngr,x,box)
         if (ifilc.ne.0) call fdat(ifilc+1,ispgr,icod2,regel,
     +        epot,spname,nrp,ngr,x,box)
C        End of loop over structures in PARAR and TAUAR
      enddo
C     Go to the next structure on the PACK file
      goto 500

 900  continue
      close(10)
C     End reading that PACK file
      ninpt=ninpt+ninp
      nhit1t=nhit1t+nhit1
      nhit2t=nhit2t+nhit2
      nhit3t=nhit3t+nhit3
      nhit4t=nhit4t+nhit4
      print 56,ninp,filenm
      write(16,56) ninp,filenm
      if (inp21.eq.2) then
         print*,nhit4t,' structures written to file'
         write(16,*) nhit4t,' structures written to file'
         stop
      endif
      if (ndihh.eq.0 .or. inp21.eq.1) goto 300

      if (istage.eq.1) goto 300
      print*,nhit1,' structures after rigid OH rotations'
      write(16,*) nhit1,' structures after rigid OH rotations'
      if (istage.eq.2) goto 300
      print*,nhit2,' structures after SD optimalisation'
      write(16,*) nhit2,' structures after SD optimalisation'
      if (istage.eq.3) goto 300
      print*,nhit3,' structures after CG optimalisation'
      print*,nhit4,' structures left after clustering'
      write(16,*) nhit3,' structures after CG optimalisation'
      write(16,*) nhit4,' structures left after clustering'
      goto 300
C     End of this conformation, start next PACK file

 910  continue
      if (istage.eq.0) then
         print 65
         write(16,65)
         if (ndihh.eq.0 .or. inp21.eq.1) then
            print*,nhit4t,' structures energy minimized'
            write(16,*) nhit4t,' structures energy minimized'
            stop
         endif
      endif
      print*,ninpt,' lines used for statistics'
      write(16,*) ninpt,' lines used for statistics'
      if (istage.eq.0) then
C        End of calculation, print statistics for complete run
         call stats(-1,imin,imax,istat1,ninpt,frac1,e1)
         call stats(-2,imin,imax,istat2,nhit1t,frac2,e2)
         call stats(-3,imin,imax,istat3,nhit2t,frac3,e3)
         i=inpt('CHECK ',regel)
         print*,nhit1t,' structures after rigid OH rotations'
         print*,nhit2t,' structures after SD optimalisation'
         print*,nhit3t,' structures after CG optimalisation'
         print*,nhit4t,' structures left after clustering'
         write(16,*) nhit1t,' structures after rigid OH rotations'
         write(16,*) nhit2t,' structures after SD optimalisation'
         write(16,*) nhit3t,' structures after CG optimalisation'
         write(16,*) nhit4t,' structures left after clustering'
         print 31, emax1,emax2,emax3
         write(16,31) emax1,emax2,emax3
         print 37, e1,e2,e3
         write(16,37) e1,e2,e3
         stop
      endif

C     Determine the EMAX values corresponding to FRAC
      if (istage.eq.1) then
         call stats(1,imin,imax,istat1,ninpt,frac1,emax1)
         if (ninpt.lt.nmax1) then
            maxstt=maxstt*nmax1/ninpt
            print*,'maxstt increased to',maxstt
            write(16,*) 'maxstt increased to',maxstt
         endif
         istage=2
         goto 190
      endif
      if (istage.eq.2) then
         call stats(2,imin,imax,istat2,nhit1t,frac2,emax2)
         istage=3
         goto 190
      endif
      if (istage.eq.3) then
         call stats(3,imin,imax,istat3,nhit2t,frac3,emax3)
         istage=0
         print 31,emax1,emax2,emax3
         write(16,31) emax1,emax2,emax3
         if (maxinp.gt.0) goto 185
      endif
C     Go to the next stage

 21   format(/,' N E X T   G E O M E T R Y:   ',a10)
 22   format(a5,i3.3)
 23   format(a80)
 24   format(' Energy before second semi-rigid refinement',f10.3)
 25   format(' Energy before fully flexible refinement',f10.3)
 26   format(/,' Reading parameters from file IN.PAR')
 27   format(i7,f8.1,f10.3,3x,10(f8.1,f6.1))
 28   format(a)
   29 format(' ***** Dihedral information needed, but no files ',
     +     'pack.1## are present.')
 31   format(5x,' emax1, emax2, emax3: ',3f10.2)
 32   format(f8.3,4f8.2,2f10.5)
 33   format(i8,2f8.4,i8,f8.4,f12.7)
 34   format(12i8)
 35   format(' Structure',i6,i8,5x,'Passing EMAX1, EMAX2, EMAX3:',3i5)
 36   format('   pressure (atm)',f18.1)
 37   format(8x,' Suggested values: ',3f10.2)
 38   format(i8,4f8.4)
 39   format(i8,3f8.4,4i8)
 40   format(3i8,f8.1)
 42   format(i8,3f10.3,10i5)
 43   format(10(f8.1,f6.1))
 44   format(10x,'Dihedrals: ',15f8.1)
 45   format(a5,i4,f8.1,1x,20f7.3)
 46   format(i7,a,f12.3,a,'R =',f9.4,4x,'Eb...enb, E-inner, E-pol: ',
     +     4f7.2,f8.2,2f7.2)
 47   format(10x,'Box: ',3(3f6.3,2x),3f6.1)
c 48   format(' SD:',i4,f8.1,1x,20f7.3)
c 49   format(' CG:',i4,f8.1,1x,20f7.3)
 50   format(4i4,f7.1,i9,f8.3,i6)
 51   format(20f8.1)
 53   format(12f10.2)
 56   format(i13,' lines read from file',a10)
 57   format(3i8,3f8.2)
c 59   format(' Geometry',i4,' with',i4,' dihedral(s): ',10f10.3)
 63   format(/,' Now starting statistics, stage',i3,
     +     ' with nmax, frac:',i8,f8.3)
 64   format(/,' No statistics for stage',i3,'.    Set emax to',f8.1)
 65   format(/,' C U M U L A T I V E   R E S U L T S')
 72   format(i5,5x,a5,i5,6f8.4)
 93   format(6f10.4)
 94   format(12i5)
c 95   format(3i5,f8.3)
 96   format('(2i7,f8.1,1x,',i2,'i5,i7,4i5,i7)')
 97   format(i7,2x,6f8.4)
 98   format(15f8.5)
 99   format(2i7,f8.1,1x,25i5)
      end


      subroutine stats(istage,imin,imax,istat,ninpt,frac,emax)
C     Delivers EMAX, which is the threshold to retain a fraction FRAC
C     out of NINPT data whose histogram is in ISTAT(MIN:IMAX).
C     The stage ISTAGE is only used for output.

      implicit none
      integer i,ilast,imax,imin,istage,istat,ninpt,nstat
      double precision emax,frac,r,ruitw
      dimension istat(imin:imax)
      if (ninpt.eq.0) then
         print*,' ***** No input for statistics'
         write(16,*) ' ***** No input for statistics'
         stop
      endif
      nstat=0
c      ruitp=0
      ruitw=0
      if (istage.lt.0) then
         print 64,-istage
         write(16,64) -istage
      endif
      do i=imin,imax
         if (istat(i).ne.0) then
            ilast=i
            nstat=nstat+istat(i)
            r=nstat/dfloat(ninpt)
            if (r.le.frac) emax=i+1
c            if (r.gt.ruitp) then
c               ruitp=ruitp+frac/5.0
c               if (r.lt.2.0*frac) print 60,i,nstat,r
c            endif
            if (r.gt.ruitw) then
               ruitw=ruitw+frac/20
               if (r.lt.2*frac) write(16,60) i,nstat,r
            endif
         endif
         if (i.eq.imax) then
            write(16,60) ilast,nstat,r
            write(16,60) i,nstat,r
         endif
      enddo
      print 62,abs(istage),frac,ninpt,emax
      write(16,62) abs(istage),frac,ninpt,emax

 60   format(2i10,f10.4)
 62   format(' Statistics from stage',i3,' with frac =',f7.1,' and',
     +     i8,' entries: emax =',f8.1)
 64   format(/,' Statistics from complete run for stage',i3)
      end
