      program pack3

C     Third stage of crystal packing prediction.
C     Varies 6 cell parameters (groups 1 and 2) and atomic coordinates.
C     Can expand united atoms to all H atoms.
C     Can read additional SPF or MDI files
C     Can expand to P1; afterwards the geometry is reduced by
C         symmetry operations to the original space group.
C         But the energy still refers to P1 (scaled to one molecule).
C     Can perform MD-run before (or instead of) the energy minimization.
C        Only one independent molecule, expanded with equivalent positions into
C        one unit cell, can be studied. This is, of course, a very unrealistic
C        simulation, only meant to "shake up" the system to leave a possible
C        saddle point in the potential energy.
C     Can perform lattice vibration calculations.

C     B. P. van Eijck, November 1997.
C     Input files:  IN.TP (unit 20), molecular topology
C                   IN.SPG (unit 11), space group
C                   IN.FF (unit 12), force field
C                   IN.PA (unit 13), code-directed input
C                   IN.PAR (unit 21), structure parameters
C                   IN.COR (unit 29), coordinates
C                   IN.ADD (unit 14), additional coordinates
C                   STOP (unit 15) stops calculation manually
C     Output files: OUT.PRI (unit 16), general results
C                   OUT.PAR (unit 30), generated structure parameters
C                   OUT.COR (unit 39), generated coordinates
C     Input for MD: IN.MD (unit 15, in subroutine LEESMD)
C     Output from MD:

C     File IN.TP (read by subroutine LEEST) is the formatted topology file
C        created by program CODES and/or by hand.
C     File IN.SPG (read by subroutine LEESR) contains the space group data
C        for triclinic, monoclinic, or orthorhombic space groups.
C     File IN.FF (read by subroutine LEEST) contains the force field
C     File IN.PA reads data specific for this program. All distances in nm.
C     File IN.PAR contains parameters.
C     File IN.COR contains also atomic coordinates.
C        If IN.COR is absent the coordinates are calculated from IN.PAR.
C     File IN.ADD (unit 14) contains an arbitrarily long set of
C        additional coordinates (SPF or MDI; SPF sets separated by END).

      implicit none
      external func3,func1
      include 'params.i'
      include 'cspgr.i'
      include 'cbond.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cpola.i'
      include 'cinds.i'
      include 'cnbkl.i'
      include 'cpars.i'
      include 'cwald.i'
      include 'color.i'
      include 'chadd.i'
      include 'caidt.i'

      integer i,i1,i2,i3,iadd,icode,iconn,icopt,icoul,idum,idummy,
     +     ientr,iexp,iextra,ifilc,ifild,ifilm,ifils,ig,imult,inp29,
     +     inpt,insav,intra,ios,ipar,iprint,ipw,irefin,iscr,iseed,
     +     ispgr,ivamp,j,j2,j3,jjsh,jlist,k,k3,klist,kmax,kpar,kpar1,
     +     l,m,maxinp,md,msh,n,n1,n1sh,n2,n2sh,n3,n3sh,nat29,
     +     ncyc1,ncyc2,ncycg,ncysd,ndih1,nexpd,nfile,ngr21,nhit4,nhit5,
     +     ninp,njlist,nklist,nm,nneg,nop3,npa,npar0,npar2,
     +     npm,npos,nq,nr1,nr3,nread,nresd,nrp21,nrpt,nrun,nscm,nsnb,
     +     nstlim,nstruc,ntc,ntemp,ntp,ntpr,ntpw,ntt,ntwe,ntwv,
     +     ntwx,number,nw,nword,nsporg,iflex,iim,jjm,kkm,nim,njm,nkm,
     +     mbox,maxbox,irefs,nbox,ncyc,nn,npar1,numm,iknot,nknot,doknot
      double precision a,alf6em,alf6md,alfcem,alfcmd,arot,ax,bb,box,
     +     brot,bx,by,cc,co,comp,cosa,cosb,cosc,cut6em,cut6md,cutcem,
     +     cutcmd,cvib,cx,cy,cz,dd,depcg,der,dihan,dt,dtest,dx,
     +     eb,eh,ei,ein,einter,emax1,emax2,emin,enb,ep,epot,etot,euler,
     +     evib,expand,f1,f2,f3,fe,flist,fvib,omega,par,phi,pi,
     +     pres,pres0,psi,rad,random,rcutem,rcutmd,rmsder,rmsrat,scl,
     +     scr,sh,shmcg,shmsd,si,sina,sinb,sinc,sqm,stc,stcin,stpcg,
     +     stpsd,t,tau,taup,tautp,temp,temp0,tempi,theta,tol,tolcg,
     +     toleem,tolemd,tolkem,tolkmd,tolsd,tvib,ucl,uqm,
     +     vol,vtemp,xcm,zpemol,zpetot,ete,scr2,totm,dens
      dimension flist(0:maxq,3),vtemp(maxtmp),scl(maxtmp),sqm(maxtmp),
     +     ucl(maxtmp),uqm(maxtmp),cvib(maxtmp),evib(maxtmp),
     +     jlist(maxome,maxnrp),klist(maxnrp),njlist(maxome),
     +     irefs(6),scr2(maxpar)

      character*8 spname,ndum,naam23(maxnrp)
      character titel*80,filenm*10,regel*255,form30*60,num*5

      parameter(maxbox=1000)
      dimension nr1(mxdata),iscr(3),euler(3,3),arot(3),brot(3),
     +     der(6),stcin(4),box(maxbox),xcm(3),irefin(12),
     +     stc(maxpar),scr(maxpar),par(maxpar),
     +     tau(maxome*maxmol),dx(maxxco),ipar(maxpar)
      integer*2 mult(mxdata)

      iseed=190996
      pi=4*atan(1.0d0)
      rad=180/pi
      rmsder=-9.99d0
      titel=' '
      nfile=90000
      ebig=9999

      iexp=0
      expand=0
      maxinp=5000
c      emax1=30
      emax2=10000
      nhit5=0
      irfat1=0
      irfat2=0
      nknot=0

      ncysd=100
      tolsd=0.01d0
      stpsd=0.0001d0
      nresd=5
      shmsd=0.02d0
      ncycg=5000
      tolcg=0.0005d0
      stpcg=0.005d0
      depcg=0.3d0
      shmcg=0.02d0

      iextra=0
      inner=2
      intra=1
      iflex=1
      icoul=1
      pres=0
      iprint=0
      dtest=0
      ifils=0
      ifilm=0
      ifild=0
      ifilc=0
      md=0
      mbox=0
      do i=1,3
         stcin(i)=1
      enddo
      cutlv=1.5d0
      tollv=0
      epsil=-10000
      nsporg=0
      open(unit=15,file='STOP',status='OLD',err=100)
      close(unit=15,status='DELETE')

  100 continue
      open(unit=16,file='out.pri')
      open(unit=21,file='in.par',status='OLD',iostat=ios)
      print*,geel,' P A C K 3',wit
      write(16,*) ' '
      write(16,*) ' P A C K 3'
      if (ios.ne.0) then
         print*,' No parameter file IN.PAR present'
         stop
      endif
      open(unit=30,file='out.par')
      open(unit=39,file='out.cor')
      open(unit=29,file='in.cor',status='OLD',iostat=ios)
      if (ios.ne.0) then
         inp29=0
         ngr21=1
         print*,'No coordinate file IN.COR present, ',
     +        'assume semi-rigid molecule'
         write(16,*) ' No coordinate file IN.COR present ',
     +        'assume semi-rigid molecule'
      else
         inp29=1
C        Determine the number of atoms on this file
         j2=-1
         read(29,97,end=115,err=115) nhit4,(box(i),i=1,6)
         j2=0
  242    continue
         i=1
         read(29,13,end=247) regel
         do i=132,1,-1
            if (regel(i:i).ne.' ') goto 245
         enddo
  245    n=i/8
         if (8*n.eq.i) then
            j2=j2+n
            goto 242
         endif
  247    continue
         nat29=j2/3
C        NAT29 is the number of atoms on the coordinate file
  115    continue
         if (3*nat29.ne.j2 .or. j2.eq.0) then
            print*,' ***** Wrong number of coordinates on file ',
     +           'IN.COR:',j2
            stop
         endif
         print 28,nat29
         write(16,28) nat29
         close(29)
         open(unit=29,file='in.cor',status='OLD')
      endif

      open(unit=13,file='in.pa',status='OLD',iostat=ios)
      if (ios.ne.0) then
         print*,' No file IN.PA present'
         write(16,*) ' No file IN.PA present'
      else
         n=inpt('INPUT ',regel)
         close(13)
      endif

      nq=-1
      n=inpt('LATVIB',regel)
      if (n.ge.1) then
C        Prepare for calculation of lattice vibrations
         ntemp=1
         vtemp(1)=298
         ivamp=0
         do i=1,3
            flist(0,i)=0
         enddo

         if (n.eq.1) nq=0
         if (n.eq.2) then
C           NQ random points
            call intstr(regel,2,nq)
            if (nq.gt.maxq) then
               print*,' ***** Increase MAXQ',nq
               stop
            endif
            do i=1,nq
               do m=1,3
                  flist(i,m)=random(iseed)
               enddo
            enddo
         endif
         if (n.eq.4) then
C           N1 * N2 * N3 cells
            call intstr(regel,2,n1)
            call intstr(regel,3,n2)
            call intstr(regel,4,n3)
            if (n1*n2*n3.gt.maxq) then
               print*,' ***** Increase MAXQ',nq
               stop
            endif
            nq=0
            do i1=1,n1
               do i2=1,n2
                  do i3=1,n3
                     nq=nq+1
                     flist(nq,1)=i1/dfloat(n1)
                     flist(nq,2)=i2/dfloat(n2)
                     flist(nq,3)=i3/dfloat(n3)
                  enddo
               enddo
            enddo
         endif
         if (n.eq.5) then
C           One branch of q-values, divided into NQ steps
            call realst(regel,2,f1)
            call realst(regel,3,f2)
            call realst(regel,4,f3)
            call intstr(regel,5,nq)
            if (nq.gt.maxq) then
               print*,' ***** Increase MAXQ',nq
               stop
            endif
            do i=1,nq
               flist(i,1)=f1*i/dfloat(nq)
               flist(i,2)=f2*i/dfloat(nq)
               flist(i,3)=f3*i/dfloat(nq)
            enddo
         endif
         n=inpt('VIBAMP',regel)
         if (n.ge.1) ivamp=1
         n=inpt('CUTLV ',regel)
         if (n.ge.2) call realst(regel,2,cutlv)
         if (n.ge.3) call realst(regel,3,tollv)
         n=inpt('TEMP  ',regel)
         if (n.gt.1) then
            ntemp=n-1
            do i=1,ntemp
               call realst(regel,i+1,vtemp(i))
            enddo
         endif
C        Check double precision
         a=(1.0d0/3)
         if (abs(a-(1.0d0/3)).gt.1.0d-14) then
            print*,' ***** LATVIB requires double precision'
            stop
         endif
      endif

      n=inpt('INNER ',regel)
      if (n.ge.2) call intstr(regel,2,inner)
      n=inpt('INTRA ',regel)
      if (n.ge.2) call intstr(regel,2,intra)
      noint=(intra.eq.0)
      n=inpt('FLEX  ',regel)
      if (n.ge.2) call intstr(regel,2,iflex)
      if (iflex.eq.0) noint=.true.
      n=inpt('COUL  ',regel)
      if (n.ge.2) call intstr(regel,2,icoul)
      n=inpt('EPSIL ',regel)
      if (n.gt.0) call realst(regel,2,epsil)
      n=inpt('MDRUN ',regel)
      if (n.ge.1) md=1
      n=inpt('EXPAND',regel)
      if (n.gt.0) iexp=1
      if (n.gt.1) call realst(regel,2,expand)
      nw=-1
      n=inpt('MAXINP',regel)
      if (n.ge.2) call intstr(regel,2,maxinp)
      if (maxinp.lt.0) then
         if (maxinp.eq.-999) then
C           Undocumented option: read structure numbers from file "list".
C           This must be a file pack.pr2, pack.pr3, ...
C           Do only structures with rmsder > a.
            call realst(regel,3,a)
            open(unit=99,file='list',status='OLD',iostat=ios)
            if (ios.ne.0) then
               print*,' No parameter file LIST present'
               stop
            endif
            print*,' Reading structure numbers from file LIST'
            print 52,a
            write(16,*) ' Reading structure numbers from file LIST'
            write(16,52) a
            ninp=0
  246       continue
            read(99,12,end=249) regel
            if (nword(regel).ne.15) goto 246
            read(regel,35,err=246) j,rmsder            
            if (rmsder.gt.a) then
               ninp=ninp+1
               nr1(ninp)=j
               print 97,nr1(ninp),rmsder
            endif
            goto 246
  249       continue
            rmsder=-9.99d0
            print*,ninp,' structure numbers from file LIST'
            write(16,*) ninp,' structure numbers from file LIST'
         else
            ninp=n-2
            nw=ninp
            do i=1,ninp
               call intstr(regel,i+2,nr1(i))
            enddo
         endif
      endif
      n=inpt('PRES  ',regel)
      if (n.ge.2) call realst(regel,2,pres)
      doknot=0
      n=inpt('KNOT  ',regel)
      if (n.gt.0) doknot=1
      n=inpt('PRINT ',regel)
      if (n.ge.2) call intstr(regel,2,iprint)
      n=inpt('FILES ',regel)
      if (n.ge.2) call intstr(regel,2,ifils)
      if (n.ge.3) call intstr(regel,3,ifilm)
      if (n.ge.4) call intstr(regel,4,ifild)
      if (n.ge.5) call intstr(regel,5,ifilc)
      n=inpt('TEST  ',regel)
      if (n.ge.2) call realst(regel,2,dtest)
      n=inpt('SD    ',regel)
      if (n.ge.2) call intstr(regel,2,ncysd)
      if (n.ge.3) call realst(regel,3,tolsd)
      if (n.ge.4) call realst(regel,4,stpsd)
      if (n.ge.5) call intstr(regel,5,nresd)
      if (n.ge.6) call realst(regel,6,shmsd)
      n=inpt('CG    ',regel)
      if (n.ge.2) call intstr(regel,2,ncycg)
      if (n.ge.3) call realst(regel,3,tolcg)
      if (n.ge.4) call realst(regel,4,stpcg)
      if (n.ge.5) call realst(regel,5,depcg)
      if (n.ge.6) call realst(regel,6,shmcg)
      n=inpt('STC   ',regel)
      if (n.ge.4) then
         call realst(regel,2,stcin(1))
         call realst(regel,3,stcin(2))
         call realst(regel,4,stcin(3))
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in STC',n
         stop
      endif
      ichad=0
      nvir=0
      n=inpt('CHADD ',regel)
      if (n.ge.2) call intstr(regel,2,ichad)

C     Find space group 
      ngr21=1
      read(21,11) regel
      if (nword(regel).le.0) goto 930
      spname='noname'
      call intstr(regel,1,ispgr)
      insav=inner
      call leesr(ispgr,spname,iref,iscr,msom,inner)
      if (nword(regel).gt.1) then
         call intstr(regel,2,icode)
C        If ICODE < 0, the data file was generated by the program PACK12.
         if (icode.lt.0) icode=2
cc         if (nword(regel).gt.2) then
cc            call intstr(regel,3,ngr21) 
cc         else
cc            n=inpt('NGR21   ',regel)
cc            if (n.ge.2) call intstr(regel,2,ngr21)
cc         endif
cc         print*,ngr21,' groups expected on coordinate file'
cc         write(16,*) ngr21,' groups expected on coordinate file'
      else
         print*,' Outdated PACK-file, adjust manually!'
         stop
      endif

      icopt=1
      n=inpt('CELOPT',regel)
      if (n.ge.2) call intstr(regel,2,icopt)
      if (icopt.le.0) then
         do m=1,6
            iref(m)=0
         enddo
      endif

C     Read further general input from parameter file
      read(21,94,err=930) n1,n2
      do m=1,n1+n2
         read(21,94,err=930) idummy
      enddo
      read(21,94) nrp21
C     The number of atoms on the parameter file is NRP21
      do i=1,nrp21
         read(21,72) k,naam23(i),k,(xr(3*i-3+m),m=1,3)
      enddo
      read(21,93,err=930,end=930) (cel(i),i=1,12)
      read(21,94,err=930,end=930) (irefin(i),i=1,12)

C     Find NGR21 (the number of groups on the coordinate file)
      if (inp29.eq.1) then
         ngr21=nat29/nrp21
         if (nat29.ne.nrp21*ngr21) then
            print*,' ***** Number of atoms on file IN.COR',
     +           ' incorrect:',nat29,nrp21
            stop
         endif
      endif
      print*, ngr21,' group(s) found on coordinate file'
      write(16,*) ngr21,' group(s) found on coordinate file'

      nexpd=0
      if (iexp.ne.0) then
C        Expand to P1
         print 34, ngr21*(nspgr+1),expand
         write(16,34) ngr21*(nspgr+1),expand
         nexpd=nspgr
         ispgr=1
         nspgr=0
         spname='P1'
         do m=1,3
            msom(m)=1
            inner=insav
            if (doewc) inner=0
         enddo
      endif
      ngr=ngr21*(nexpd+1)
C     NGR is the total number of groups in the asymmetric unit
      call leest(nrp,icoul,iprint)
C     This gives NRP, the number of atoms in one group
      nrpt=nrp*ngr
C     NRPT is the total number of atoms in the asymmetric unit
      if (nrpt.gt.maxat) then
         print*,' ***** Increase maxat',ngr,nrp,maxat
         stop
      endif
      emax1=20+nrp*ngr21/2
      if (emax1.lt.30) emax1=30
      n=inpt('EMAX  ',regel)
      if (n.ge.2) call realst(regel,2,emax1)
      if (n.ge.3) call realst(regel,3,emax2)
      nr3=3*nrpt
      finner=16*atan(1.0d0)/(3*(nspgr+1))
      druk=0.061018d0*pres/(nspgr+1)
C     PRES is in atmospheres, DRUK in adapted units
C     These quantities refer to the asymmetric unit, as corresponding to NONBE.

C     If desired, override standard settings from topology file
      n=inpt('POL   ',regel)
      if (n.ge.2) call intstr(regel,2,npp)
      if (n.ge.3) then
         if (n.lt.9) then
            print*,' ***** Wrong number of items in POL',n
            stop
         endif
         call realst(regel,3,poltol)
         call realst(regel,4,schaal)
         call realst(regel,5,gampol)
         call intstr(regel,6,nnn)
         call intstr(regel,7,mmm)
         call intstr(regel,8,nte)
         call intstr(regel,9,nttt)
      endif
      n=inpt('CUTOFF',regel)
      if (n.ge.2) call realst(regel,2,cutoff)
      if (n.ge.3) then
         if (n.lt.8) then
            print*,' ***** Wrong number of items in CUTOFF',n
            stop
         endif
         call realst(regel,3,alphac)
         call realst(regel,4,alpha6)
         call realst(regel,5,hmaxc)
         call realst(regel,6,hmax6)
         call realst(regel,7,tole)
         call realst(regel,8,tolk)
      endif
      if (cutlv.lt.cutoff) cutlv=cutoff
      doewc=(icoul.gt.0 .and. alphac.gt.1.0d-9)
      doew6=(alpha6.gt.1.0d-9)
      doew=(doewc .or. doew6)
      if (doewc) inner=0
      if (abs(tole).gt.0.1d0 .or. abs(tolk).gt.0.1d0) then
         print*,' ***** Warning: TOLE, TOLK = ',tole,tolk
         write(16,*) ' ***** Warning: TOLE, TOLK = ',tole,tolk
      endif
      rcutem=cutoff
      alfcem=alphac
      alf6em=alpha6
      cutcem=hmaxc
      cut6em=hmax6
      toleem=tole
      tolkem=tolk

C     Test for compatibilities
      if (nq.ge.0 .and. npp.gt.0) then
         print*,' ***** Do not combine LATVIB with polarization'
         stop
      endif
      if (nq.ge.0 .and. iflex.ne.1) then
         print*,' ***** Do not combine LATVIB with IFLEX<>1'
         stop
      endif
      if (iexp.ne.0 .and. iflex.ne.1) then
         print*,' ***** Do not combine P1-expansion with IFLEX<>1'
         stop
      endif
      if (ichad.ne.0 .and. npp.ne.0) then
         print*,' ***** Do not combine CHADD with polarization'
         stop
      endif
      if (maxinp.gt.mxdata) then
         print*,' Increase MXDATA in YPACK3',maxinp,mxdata
         stop
      endif

      totm=0
      do i=1,nrp
         totm=totm+amass(i)
      enddo
      
      print*, '    nrp    ngr21    nrpt     ngr'
      print 37, nrp,ngr21,nrpt,ngr
      write(16,*) '    nrp    ngr21    nrpt     ngr'
      write(16,37) nrp,ngr21,nrpt,ngr
      write(16,*) ' cutoff  alphac  alpha6   hmaxc   hmax6',
     +     '      tole      tolk'
      write(16,32) rcutem,alphac,alpha6,hmaxc,hmax6,tole,tolk
      write(16,26) pres
      if (npp.gt.1) then
         write(16,*) '    npp  poltol  schaal   gamma     nnn     mmm',
     +        '     nte    nttt'
         write(16,29) npp,poltol,schaal,gampol,nnn,mmm,nte,nttt
      endif
      write(16,*) '  inner   intra   icoul     npp    iexp    expand'
      write(16,21) inner,intra,icoul,npp,iexp,expand
      write(16,*) ' maxinp   emax1   emax2'
      write(16,23) maxinp,emax1,emax2
      write(16,*) '  ifils   ifilm   ifild   ifilc  iprint'
      write(16,37) ifils,ifilm,ifild,ifilc,iprint
      write(16,*) '  ncysd   tolsd   stpsd   nresd   shmsd       dtest'
      write(16,33) ncysd,tolsd,stpsd,nresd,shmsd,dtest
      write(16,*) '  ncycg   tolcg   stpcg   depcg   shmcg'
      write(16,38) ncycg,tolcg,stpcg,depcg,shmcg
      write(16,36) (stcin(m),m=1,3)

C     Write new parameter file
C     ICODE is incremented by 1 in every calculation with PACK2 or PACK3
c      write(30,94) ispgr,icode+1,ngr
      write(30,94) ispgr,icode+1
      write(30,94) ndihz,ndihh
      write(16,*) '  ndihz   ndihh    ireftr...'
      write(16,39) ndihz,ndihh,(ireftr(i),i=1,nref)
      if (n1.ne.ndihz .or. n2.ne.ndihh) then
         print 31,ngr,n1,ndihz,n2,ndihh
         write(16,31) ngr,n1,ndihz,n2,ndihh
c        stop
C        The number of special dihedrals may have been changed, no problem
      endif
      write(16,*) ' Dihedrals:'
      do m=1,ndihz+ndihh
         write(16,94) iome(m),jome(m),kome(m),lome(m)
         write(30,94) iome(m),jome(m),kome(m),lome(m)
      enddo
      if (nrp21.ne.nrp) then
         print 40,nrp,nrp21
         write(16,40) nrp,nrp21
c        stop
         print*,' Type ENTER if you wish to continue anyhow'
         read 11, regel
      endif
      write(30,94) nrp,(ireftr(i),i=1,nref)

      npm=1
      n=nrp21
cc      isave=ispgr
cc      ispgr=0
      do i=1,nrp21
         naam(i)=naam23(i)
      enddo
cc      call leesx(0,n,nrp,npm,0,nstruc,titel,spname,ispgr,xr,box,iprint)
      call leesx(0,n,nrp,npm,0,nstruc,titel,ndum,idum,xr,box,iprint)
cc      print*,ispgr,spname
      do i=1,nrp
         write(30,72) i,panm(i),iac(i),(xr(3*(i-1)+m),m=1,3)
         print 72, i,panm(i),iac(i),(xr(3*(i-1)+m),m=1,3)
      enddo

      write(30,93) (cel(i),i=1,12)
      kpar=0
      do i=1,12
         if (irefin(i).ne.iref(i)) then
            print*,' ***** IREF',i,' inconsistent:',irefin(i),iref(i)
            write(16,*)
     +           ' ***** IREF',i,' inconsistent:',irefin(i),iref(i)
            stop
         endif
         if (iref(i).gt.0) kpar=kpar+1
      enddo
      
      if (iexp.ne.0) then
C        Energy minimization in P1
         do m=1,3
            iref(m+3)=1
            iref(m+6)=0
         enddo
      endif
      write(30,94) (iref(i),i=1,12)
      if (nr3+12.gt.maxpar) then
         print*,' ***** Increase maxpar',nr3+12,maxpar
         stop
      endif
      do i=1,maxpar
         stc(i)=stcin(3)
      enddo
      npar0=0
      do i=1,6
         if (iref(i).gt.0) then
            npar0=npar0+1
            m=(i+2)/3
            stc(npar0)=stcin(m)
         endif
      enddo

      if (md.ne.0) then
C        Read and print MD data
C        Use CUTOFF etc from EM unless changed by LEESMD
         rcutmd=cutoff
         alfcmd=alphac
         alf6md=alpha6
         cutcmd=hmaxc
         cut6md=hmax6
         tolemd=tole
         tolkmd=tolk
         pres0=pres
         call leesmd(rcutmd,alfcmd,alf6md,cutcmd,cut6md,tolemd,tolkmd,
     +        tempi,temp0,tautp,pres0,comp,taup,t,dt,tol,
     +        ig,nrun,ntt,ntp,nscm,nstlim,ntc,nsnb,
     +        ntpr,ntwx,ntwv,ntwe,ntpw)
         druk=pres0/(nspgr+1)
      endif

C     Read parameter file 21, select required entries into file NR1(1..NINP)
      if (maxinp.eq.0) then
         goto 850
      else
         nread=1
         if (iprint.gt.1) print*,
     +        'Reading structure numbers from PACK-file'
  230    continue
         read(21,13,end=250,err=240) regel
         if (iprint.gt.3) print 12, regel
         n=nword(regel)
         if (n.lt.3) goto 230
         if (inp29.eq.0 .and. n.lt.kpar+ndiha) then
            print*,' ***** No coordinate file and insufficient ',
     +           'entries on parameter file:',n
            stop
         endif
         call intstr(regel,2,nhit4)
         call realst(regel,3,ein)
         mult(nhit4)=1
         if (n.gt.3) then
            call intstr(regel,n,m)
            mult(nhit4)=m
         endif
C        call intstr(regel,n,mult(nhit4)) gives bus error because of integer*2
         if (nhit4.gt.mxdata) then
            print*,'Structure number ',nhit4,' too large '
            write(16,*) 'Structure number ',nhit4,' too large '
            stop
         endif
         if (maxinp.gt.0) then
            nr1(nread)=nhit4
            if (nread.eq.1) emin=ein
            if (ein-emin.le.emax1 .and. ein.le.emax2) nread=nread+1
            if (nread.gt.maxinp) goto 250
         else
            nread=nread+1
         endif
         goto 230
  240    print*,' ***** Error reading data parameter file:'
         print 12,regel
         write(16,*) ' ***** Error reading data parameter file:'
         write(16,12) regel
         goto 230
  250    continue
         nread=nread-1
         if (maxinp.gt.0) then
            ninp=nread
            print*,ninp,' structure(s) read from parameter file'
            write(16,*) ninp,' structure(s) read from parameter file'
         else
            if (nw.eq.0) then
               print*,' All structures in IN.COR selected'
               write(16,*) ' All structures in IN.COR selected'
            else
               print*,nread,' structure(s) read from parameter file'
               write(16,*) nread,
     +              ' structure(s) read from parameter file'
               print*,ninp,' structures selected in MAXINP card'
               write(16,*) ninp,' structures selected in MAXINP card'
            endif
         endif
      endif

      if (inp29.eq.0) then
C        No coordinate file present, reconstruct coordinates from tape21
C        Determine for each dihedral the two rotatable fragments
         do k=1,ndiha
            print 61,iome(k),jome(k),kome(k),lome(k)
            write(16,61) iome(k),jome(k),kome(k),lome(k)

C           Search for atoms linked to atom JOME(K), not through KOME(K)
            njlist(k)=1
            jlist(k,1)=jome(k)
  260       continue
            iadd=0
            do m=1,njlist(k)
               do l=1,nbond
                  if (ib(l).eq.jlist(k,m)) then
                     iconn=jb(l)
                  elseif (jb(l).eq.jlist(k,m)) then
                     iconn=ib(l)
                  else
                     goto 270
                  endif
                  if (iconn.eq.kome(k)) goto 270
                  do n=1,njlist(k)
                     if (iconn.eq.jlist(k,n)) goto 270
                  enddo
                  iadd=1
                  njlist(k)=njlist(k)+1
                  jlist(k,njlist(k))=iconn
  270             continue
               enddo
            enddo
c            print*,(jlist(k,n),n=1,njlist(k))
            if (iadd.eq.1) goto 260
            print 60,jome(k),(jlist(k,n),n=2,njlist(k))
            write(16,60) jome(k),(jlist(k,n),n=2,njlist(k))

C           Search for atoms linked to atom KOME(K), not through JOME(K)
            nklist=1
            klist(1)=kome(k)
  280       continue
            iadd=0
            do m=1,nklist
               do l=1,nbond
                  if (ib(l).eq.klist(m)) then
                     iconn=jb(l)
                  elseif (jb(l).eq.klist(m)) then
                     iconn=ib(l)
                  else
                     goto 290
                  endif
                  if (iconn.eq.jome(k)) goto 290
                  do n=1,nklist
                     if (iconn.eq.klist(n)) goto 290
                  enddo
                  iadd=1
                  nklist=nklist+1
                  klist(nklist)=iconn
  290             continue
               enddo
            enddo
c            print*,(klist(n),n=1,nklist)
            if (iadd.eq.1) goto 280
            print 60,kome(k),(klist(n),n=2,nklist)
            write(16,60) kome(k),(klist(n),n=2,nklist)

C           Check that the dihedral is not part of a ring
            do n=1,njlist(k)
               do m=1,nklist
                  if (jlist(k,n).eq.klist(m)) then
                     print 62,iome(k),jome(k),kome(k),lome(k)
                     write(16,62) iome(k),jome(k),kome(k),lome(k)
                     goto 295
                  endif
               enddo
            enddo
C           Check that the sum of the entries is correct
            if (njlist(k)+nklist.ne.nrp) then
               print*,' ***** Sumcheck error:',njlist(k),nklist,nrp
               stop
            endif
  295       continue
         enddo
         rewind(21)
         do m=1,ndiha+nrp21+6
            read(21,11) regel
         enddo
      else
         close(21)
      endif

C     Read geometry information for next calculation
  300 continue
      if (doewc) inner=0
      if (iextra.ne.0) then
C        Read entry from additional coordinate file IN.ADD (SPF or MDI).
C        There have already NINP structures been treated from PACK-files.
         imult=1
         if (iprint.gt.0) print*,' '
         do i=1,nrp21
            naam(i)=naam23(i)
         enddo
         call leesx(14,n,nrp,ngr21,0,0,titel,ndum,idum,x,box,iprint)
         if (n.lt.0) goto 900
         read(titel,24,err=330) number
         if (number.le.0) goto 330
         do i=1,ninp
            if (number.eq.nr1(i)) goto 330
         enddo
C        The number in the file title has not been used before;
C            assign it to the new entry in the PACK-files.
         nhit4=number
         ninp=ninp+1
         nr1(ninp)=number
         goto 340
  330    continue
C        The number in the file title is absent or has been used before;
C           assign new numbers consecutively from 90001 onwards.
         nfile=nfile+1
         nhit4=nfile
  340    continue
         if (iprint.gt.0) print 22,nhit4,titel
         if (iprint.gt.2) then
            print 73,(box(m),m=1,6)
            do i=1,nrp
               print 72,i,panm(i),i,(x(3*(i-1)+m),m=1,3)
            enddo
         endif
      else
         if (inp29.eq.1) then
C           Read the next coordinate set from unit 29
            read(29,97,end=850,err=430) nhit4,(box(i),i=1,6)
            read(29,98,end=850,err=430) (x(i),i=1,3*nrp21*ngr21)
            goto 440
 430        continue
            print*,'       Check EXPAND (if applicable)'
            print*,' ***** Error reading coordinate file',
     +           nhit4,nrp21,ngr21,nat29
            write(16,*)' ***** Error reading coordinate file',
     +           nhit4,nrp21,ngr21,nat29
C           Skip this one, try the next
            goto 300
 440        continue
            if (nw.eq.0) then
               imult=1
               ninp=ninp+1
               nr1(ninp)=nhit4
               n=nrp21*ngr21
               do i=1,nrp21
                  naam(i)=naam23(i)
               enddo
               call leesx(0,n,nrp,ngr21,0,0,titel,ndum,idum,x,box,0)
               goto 500
            endif
         else
C           Read the next parameter set from unit 21
            read(21,*,end=850,err=940) idummy,nhit4,ein,
     +           (ipar(m),m=1,kpar+ndiha)
         endif

C        Check whether this entry occurs in the selected set NR1(1..NW).
         do ientr=1,ninp
            if (nhit4.eq.nr1(ientr)) then
               imult=mult(nhit4)
               if (inp29.eq.1) then
C                 Analyze and complete coordinate information from unit 29
                  n=nrp21*ngr21
                  do i=1,nrp21
                     naam(i)=naam23(i)
                  enddo
                  call leesx(0,n,nrp,ngr21,0,9,titel,ndum,idum,x,box,0)
                  goto 500
               else
C                 Construct coordinates using information from unit 21
                  npa=0
                  do i=1,12
                     if (iref(i).gt.0) then
                        npa=npa+1
                        box(i)=dfloat(ipar(npa))/1000
                     else
                        box(i)=cel(i)
                     endif
                     if (i.gt.9) box(i)=box(i)*100/rad
                  enddo
                  do i=1,nr3
                     x(i)=xr(i)
                  enddo

C                 Set dihedral angles to required values
                  do j=1,ndiha
                     omega=dfloat(ipar(kpar+j))/rad
                     if (iprint.gt.4) print*,'Setting dihedral',j,':  ',
     +                    iome(j),jome(j),kome(j),lome(j),omega*rad
                     omega=omega-dihan(j,x)
                     co=cos(omega)-1
                     si=sin(omega)
                     j3=3*jome(j)-3
                     k3=3*kome(j)-3
                     do n=1,njlist(j)
                        i3=3*jlist(j,n)-3
                        call introt(i3,j3,k3,x,arot,brot)
                        do m=1,3
                           x(i3+m)=x(i3+m)+si*brot(m)+co*arot(m)
                        enddo
                     enddo
                  enddo

                  call trans(nrp,x,ireftr,xcm,dx,phi,theta,psi)
                  cosa=cos(box(10))
                  sina=sin(box(10))
                  cosb=cos(box(11))
                  sinb=sin(box(11))
                  cosc=cos(box(12))
                  sinc=sin(box(12))
                  euler(1,1)= cosc*cosa-sinc*sina*cosb
                  euler(1,2)= cosc*sina+sinc*cosa*cosb
                  euler(1,3)= sinc*sinb
                  euler(2,1)=-sinc*cosa-cosc*sina*cosb
                  euler(2,2)=-sinc*sina+cosc*cosa*cosb
                  euler(2,3)= cosc*sinb
                  euler(3,1)= sina*sinb
                  euler(3,2)=-cosa*sinb
                  euler(3,3)= cosb
                  do i=1,nrp
                     j=3*(i-1)
                     do m=1,3
                        x(j+m)=box(m+6)
                        do k=1,3
                           x(j+m)=x(j+m)+euler(m,k)*dx(j+k)
                        enddo
                     enddo
                  enddo
               endif
               goto 500
            endif
         enddo
C        Skip this one, try the next
         goto 300
      endif

  500 continue
      open(unit=15,file='STOP',status='OLD',err=305)
      close(unit=15,status='DELETE')
      print*,' '
      print*,' Stopped because of manual intervention'
      goto 900
  305 continue
      nhit5=nhit5+1

C     Look for knotted structures
      if (ispgr.gt.0 .and. doknot.gt.0) then
C        Do expansion with equivalent positions
         ax=box(1)
         by=box(2)
         cz=box(3)
         bx=box(4)
         cx=box(5)
         cy=box(6)
         do nop=1,nspgr
            nop3=3*nop-3
            do i=1,nrpt
               j=3*i-3
               k=j+nop*nr3
               x(k+1)=msp(nop3+1)*x(j+1)+
     +              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               x(k+2)=msp(nop3+2)*x(j+2)+
     +              tsp(nop3+2)*by+tsp(nop3+3)*cy
               x(k+3)=msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
            enddo
         enddo
         if (iprint.gt.1) then
            print 55,nhit5   
            write(16,55) nhit5
         endif
         open(unit=49,file='rings.def',status='OLD',iostat=ios)
         if (ios.ne.0) then
            print*,' ***** No file RINGS.DEF present'
            print*,' Trying to create one:' 
            write(16,*) ' ***** No file RINGS.DEF present'
            write(16,*) ' Trying to create one...' 
            call rings(nrp,nbond,ib,jb,iprint)
            print*,' Check the prepared file rings.def carefully!'
            print*,' First line should contain the number of rings'
            print*,' Then for every ring a line with:'
            print*,' number of atoms (NATR), atom numbers 1....NATR, 1'
            print*,' '
            print*,' If correct, then rerun the program'
            stop
         endif
         call catena(nrp,ngr,nspgr,box,x,panm,-iprint,iknot)
         close(49)
         if (iknot.gt.0) then
            nknot=nknot+1
            print 54,iknot,nhit4
            write(16,54) iknot,nhit4
            goto 300
         endif
         if (iknot.eq.-100) then
            print*,'   Unrealistic bond distance in ',nhit4
            write(16,*)'   Unrealistic bond distance in ',nhit4
            goto 300
         endif
      endif

C     Begin a new calculation
      if (iprint.gt.1) print 41,nhit5,nhit4,(box(i),i=1,6)
C     Expand equivalent positions if required
      if (abs(iexp).eq.1) then
         if (iprint.gt.1) print*,' Expand and calculate in P1'
         ax=box(1)
         by=box(2)
         cz=box(3)
         bx=box(4)
         cx=box(5)
         cy=box(6)
         if (iprint.gt.3) print*,' Expanded coordinates:'
         do nop=1,nexpd
            nop3=3*nop-3
            do i=1,nrp*ngr21
               j=3*i-3
               k=j+3*nop*nrp*ngr21
               dd=2*expand*(random(iseed)-0.5d0)
               x(k+1)=dd+msp(nop3+1)*x(j+1)+
     +              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               dd=2*expand*(random(iseed)-0.5d0)
               x(k+2)=dd+msp(nop3+2)*x(j+2)+
     +              tsp(nop3+2)*by+tsp(nop3+3)*cy
               dd=2*expand*(random(iseed)-0.5d0)
               x(k+3)=dd+msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
               if (iprint.gt.3) print 74,i,(x(k+m),m=1,3)
            enddo
         enddo
      endif

C     Do MD run
      if (md.ne.0) then
         cutoff=rcutmd
         alphac=alfcmd
         alpha6=alf6md
         hmaxc=cutcmd
         hmax6=cut6md
         tole=tolemd
         tolk=tolkmd
         doewc=(alphac.gt.1.0d-9)
         doew6=(alpha6.gt.1.0d-9)
         doew=(doewc .or. doew6)
         inner=insav
         if (doewc) inner=0
         ipw=iprint
         if (iprint.gt.5) ipw=5
C        WRITE TITLE TO TAPES WHEN REQUIRED
         if (ntwx.ne.0) then
            write(filenm,25) nhit4,'.co '
            if (ntpw.eq.0) then
               open(unit=17,file=filenm,form='UNFORMATTED')
               write(17) titel
            else
               open(unit=17,file=filenm)
               write(17,11) titel
            endif
         endif
         if (ntwv.ne.0) then
            write(filenm,25) nhit4,'.ve '
            if (ntpw.eq.0) then
               open(unit=18,file=filenm,form='UNFORMATTED')
               write(18) titel
            else
               open(unit=18,file=filenm)
               write(18,11) titel
            endif
         endif
         if (ntwe.ne.0) then
            write(filenm,25) nhit4,'.en '
            open(unit=15,file=filenm)
            write(15,11) titel
         endif
         do i=1,nr3
            xr(i)=0
         enddo

         call runmd(ngr,nrp,nbond,box,x,xr,panm,
     +        temp,epot,etot,scr,dx,ipw,1,1,1,nsporg,
     +        tempi,temp0,tautp,pres0,comp,taup,t,dt,tol,
     +        ig,nrun,ntt,ntp,nscm,nstlim,ntc,nsnb,
     +        ntpr,ntwx,ntwv,ntwe,ntpw)

C        Reset coordinates using cell translations,
C        for each independent molecule
         ax=box(1)
         by=box(2)
         cz=box(3)
         bx=box(4)
         cx=box(5)
         cy=box(6)
         bb=sqrt(bx**2+by**2)
         cc=sqrt(cx**2+cy**2+cz**2)
         cosa=(bx*cx+by*cy)/(bb*cc)
         cosb=cx/cc
         cosc=bx/bb
         call tri(nrpt,0,x,1,cosa,cosb,cosc)
         do k=1,ngr
            nm=3*(k-1)*nrp
            n1=-nint(x(nm+1)/ax)
            n2=-nint(x(nm+2)/bb)
            n3=-nint(x(nm+3)/cc)
            do i=1,nrp
               j=nm+3*i-3
               x(j+1)=x(j+1)+n1*ax
               x(j+2)=x(j+2)+n2*bb
               x(j+3)=x(j+3)+n3*cc
            enddo
            if (iprint.gt.1) print 51,k,n1,n2,n3
         enddo
         call tri(nrpt,0,x,-1,cosa,cosb,cosc)

         if (ifilm.ne.0) then
            write(filenm,25) nhit4,'.mdo'
            open(unit=19,file=filenm)
            write(19,*) ' MDO file number',nhit4
            write(19,95) nrpt
            do j=1,nrpt
               write(19,72) j,panm(j),iac(j),
     +              (x(3*j-3+m),m=1,3),(xr(3*j-3+m),m=1,3)
            enddo
            write(19,93) ax,bb,cc,rad*acos(cosa),
     +           rad*acos(cosb),rad*acos(cosc)
            close(19)
         endif
         if (ntwx.ne.0) close(17)
         if (ntwv.ne.0) close(18)
         if (ntwe.ne.0) close(15)

C        End MD run, reset parameters for EM-run
         druk=0.061018d0*pres/(nspgr+1)
         cutoff=rcutem
         alphac=alfcem
         alpha6=alf6em
         hmaxc=cutcem
         hmax6=cut6em
         tole=toleem
         tolk=tolkem
         doewc=(alphac.gt.1.0d-9)
         doew6=(alpha6.gt.1.0d-9)
         doew=(doewc .or. doew6)
         inner=insav
         if (doewc) inner=0
         if (ncysd.eq.0 .and. ncycg.eq.0) goto 700
      endif
C     End of MD run

      if (iflex.eq.0) then
C     Perform rigid-body optimization
         nmols=1
         imols(1)=1
C        Check for disconnected sets of atoms
         do i=2,nrpt
            nim=(i-1)/nrp
            iim=mod(i-1,nrp)+1
            do j=1,i-1
               njm=(j-1)/nrp
               if (nim.eq.njm) then
                  jjm=mod(j-1,nrp)+1
                  if (iexcl(jjm,iim).ne.0) goto 170
               endif
            enddo
            do k=i+1,nrpt
               nkm=(k-1)/nrp
               kkm=mod(k-1,nrp)+1
               do j=1,i-1
                  njm=(j-1)/nrp
                  if (nkm.eq.njm) then
                     jjm=mod(j-1,nrp)+1
                     if (iexcl(jjm,kkm).ne.0) goto 170
                  endif
               enddo
            enddo
            if (mbox.eq.0) print*,
     +           ' Atom',i,' is not connected to previous ones'
            nmols=nmols+1
            if (nmols.ge.maxmol) then
               print*,' ***** Increase MAXMOL'
               stop
            endif
            imols(nmols)=i
  170       continue
         enddo
C        There are NMOLS separate molecules, coordinates starting at IMOLS
         imols(nmols+1)=nrpt+1
         if (mbox.eq.0) then
            do i=1,nmols
               print 30,i,imols(i),imols(i+1)-1
               write(16,30) i,imols(i),imols(i+1)-1
            enddo
         endif
         mbox=6*nmols+6
         if (mbox.gt.maxbox) then
            print*,' ***** Increase MAXBOX',mbox,maxbox
            stop
         endif
         nbox=0
         do i=1,6
            irefs(i)=iref(i)
            if (iref(i).gt.0) then
               nbox=nbox+1
            endif
         enddo
C        Transform to reference geometry XR
         do i=1,nmols
            nn=imols(i+1)-imols(i)
            k=0
            do j=imols(i),imols(i+1)-1
               do m=1,3
                  k=k+1
                  scr(k)=x(3*j-3+m)
               enddo
            enddo
C        IREFTR refers to the same atom numbers in each independent molecule.
C        This is not meaningful if these molecules are not identical
c         print*,'Calling TRANS for molecule',i,imols(i),imols(i+1)-1
            call trans(nn,scr,ireftr,xcm,scr2,phi,theta,psi)
            k=0
            do j=imols(i),imols(i+1)-1
               do m=1,3
                  k=k+1
                  xr(3*j-3+m)=scr2(k)
               enddo
cc            print*,k,scr2(k)
            enddo
            box(6*i+1)=xcm(1)
            box(6*i+2)=xcm(2)
            box(6*i+3)=xcm(3)
            box(6*i+4)=phi
            box(6*i+5)=theta
            box(6*i+6)=psi
            if (nn.eq.1) then
               do m=4,6
                  box(6*i+m)=0
               enddo
            endif
         enddo
         
C        Rigid-body energy minimization
         epot=enb
         npar1=0
         do i=1,12
            cel(i)=box(i)
            if (iref(i).gt.0) then
               npar1=npar1+1
               par(npar1)=box(i)
               m=(i+2)/3
               if (m.eq.4) par(npar1)=box(i)/rad
               stc(npar1)=stcin(m)
            endif
         enddo
         do i=13,6*(1+nmols)
            npar1=npar1+1
            par(npar1)=box(i)
            m=(i+2)/3
            m=mod(m+1,2)+3
            if (m.eq.4) par(npar1)=box(i)/rad
            stc(npar1)=stcin(m)
         enddo
         if (dtest.gt.1.0e-10) then
            call func1(npar1,par,scr,ete,1)
            print*,' Direct test func1 ',ete
            call test(par,npar1,func1,dtest)
         endif
         if (ncysd.gt.0) then
            call stedes(npar1,par,func1,ncysd,tolsd,nhit4,stc,
     +           stpsd,nresd,shmsd,iprint,epot,ncyc,rmsder)
            if (iprint.gt.1) print 42,
     +           ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
         if (ncycg.gt.0) then
            call congra(npar1,par,func1,ncycg,tolcg,nhit4,
     +           stpcg,depcg,shmcg,iprint,epot,ncyc,rmsder)
            if (iprint.gt.1) print 49,
     +           ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
         call func1(npar1,par,scr,epot,1)
cc            print*,' Final energy',epot
         k=0
         do i=1,6
            if (iref(i).gt.0) then
               k=k+1
               box(i)=par(k)
            else
               box(i)=cel(i)
            endif
         enddo
cc         if (iprint.gt.2) print*, ' npla = ',npla
         goto 700
      endif

C     Perform all-atom optimization
      npar2=0
      do i=1,6
         if (iref(i).gt.0) then
            npar2=npar2+1
            par(npar2)=box(i)
         endif
      enddo
      do i=1,3
         if (iref(i+6).gt.0) then
            npar2=npar2+1
            par(npar2)=x(i)
         endif
      enddo
      do i=4,nr3
         npar2=npar2+1
         par(npar2)=x(i)
      enddo
      ncyc1=0
      ncyc2=0
      if (dtest.gt.1.0d-10) call test(par,npar2,func3,dtest)
      if (ncysd.gt.0) then
         call stedes(npar2,par,func3,ncysd,tolsd,nhit4,stc,
     +        stpsd,nresd,shmsd,iprint,epot,ncyc1,rmsder)
         rmsrat=rmsder/tolsd
         if (iprint.gt.1) print 48, ' SD: ',ncyc1,epot,rmsder,
     +        (par(m),m=1,6)
      endif
c      call func3(npar2,par,scr,xx,1)
c      print*,' tussenresultaat ',xx
      if (ncycg.gt.0) then
         call congra(npar2,par,func3,ncycg,tolcg,nhit4,
     +        stpcg,depcg,shmcg,iprint,epot,ncyc2,rmsder)
         rmsrat=rmsder/tolcg
         if (iprint.gt.1) print 48, ' CG: ',ncyc2,epot,rmsder,
     +        (par(m),m=1,6)
      endif

C     Perform a final energy calculation
      if (md.ne.0 .and. ncysd.eq.0 .and. ncycg.eq.0) goto 700
      k=0
      do i=1,6
         if (iref(i).gt.0) then
            k=k+1
            box(i)=par(k)
         else
            box(i)=cel(i)
         endif
      enddo
      do i=1,3
         if (iref(i+6).gt.0) then
            k=k+1
            x(i)=par(k)
         endif
      enddo
      do i=4,nr3
         k=k+1
         x(i)=par(k)
      enddo
      call nonbe(ngr,nrp,enb,0,der,0,box,x,scr,0)
      call cbond(ngr,nrp,x,scr,0,eb,0)
      call choek(ngr,nrp,x,scr,0,eh,0)
      call cdiha(ngr,nrp,x,scr,0,1,ei,0)
      call cdiha(ngr,nrp,x,scr,0,2,ep,0)
      epot=enb+eb+eh+ei+ep
      if (iprint.gt.2) print*, ' npla = ',npla

  700 continue
C     Write coordinates for expanded molecules
      if (iexp.gt.0) then
         write(39,97) nhit4,(box(m),m=1,6)
         write(39,98) (x(i),i=1,nr3)
         if (ifilm.ne.0) call files(1,nhit4,titel,
     +        epot/ngr,spname,nrpt,panm,x,box)
         if (ifils.ne.0) call files(ifils+1,nhit4,titel,
     +        epot/ngr,spname,nrpt,panm,x,box)
         if (ifild.eq.1) call fdat(1,ispgr,nhit4,titel,
     +        epot/ngr,spname,nrpt,ngr,x,box)
         if (ifild.eq.2) call fdat(4,ispgr,nhit4,titel,
     +        epot/ngr,spname,nrpt,ngr,x,box)
         if (ifilc.ne.0) call fdat(ifilc+1,ispgr,nhit4,titel,
     +        epot/ngr,spname,nrpt,ngr,x,box)
      endif

C     Check whether coordinates fit in format
      do i=1,nr3      
         if (x(i).lt.-9.99999 .or. x(i).gt.99.99999) then
            print*,'***** Error in structure', nhit5,nhit4,etot
            print*,'      Coordinates too large for format f8.5'
            write(16,*) '***** Error in structure', nhit5,nhit4,etot
            write(16,*) '      Coordinates too large for format f8.5'
            goto 300
         endif
      enddo

C     Contract results according to space group symmetry
      if (iexp.lt.0) then
         do m=1,3
            der(m)=0
         enddo
         ax=box(1)
         by=box(2)
         cz=box(3)
         bx=box(4)
         cx=box(5)
         cy=box(6)
         do i=1,3*nrp
            dx(i)=x(i)**2
         enddo
         
         do nop=1,nexpd
            nop3=3*nop-3
            do i=1,nrp*ngr21
               j=3*i-3
               k=j+3*nop*nrp*ngr21
               scr(1)=msp(nop3+1)*(x(k+1)-
     +              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx)
ccc?               x(j+1)=x(j+1)+xtr
               scr(2)=msp(nop3+2)*(x(k+2)-
     +              tsp(nop3+2)*by+tsp(nop3+3)*cy)
               scr(3)=msp(nop3+3)*(x(k+3)-tsp(nop3+3)*cz)
               do m=1,3
                  x(j+m)=x(j+m)+scr(m)
                  dx(j+m)=dx(j+m)+scr(m)**2
               enddo
            enddo
         enddo
         if (iprint.gt.1) print 50
         do i=1,nrp
            j=3*i-3
            do m=1,3
               x(j+m)=x(j+m)/ngr
               dx(j+m)=sqrt(abs(dx(j+m)/ngr-x(j+m)**2))
               der(m)=der(m)+dx(j+m)
            enddo
            if (iprint.gt.1) print 43,(x(j+m),m=1,3),(dx(j+m),m=1,3)
         enddo
         if (iprint.gt.1) print 45, (der(m)/nrp,m=1,3)
      endif

      open(unit=15,file='filename')
      read(15,11,err=720,end=720)titel
      n=0
      do i=1,80
         n=n+1
         if (titel(i:i).eq.' ') goto 710
      enddo
 710  continue
      titel(n:n)='-'
      write(num,25) nhit4
      titel=titel(1:n)//num
 720  continue
      close(15)
      
C     Write coordinates for one (possibly contracted) molecule
      if (iexp.le.0) then
         write(39,97) nhit4,(box(m),m=1,6)
         write(39,98) (x(i),i=1,nr3)
         if (ifilm.ne.0) call files(1,nhit4,titel,
     +        epot/ngr,spname,nrp*ngr21,panm,x,box)
         if (ifils.ne.0) call files(ifils+1,nhit4,titel,
     +        epot/ngr,spname,nrp*ngr21,panm,x,box)
         if (ifild.eq.1) call fdat(1,ispgr,nhit4,titel,
     +        epot/ngr,spname,nrp*ngr21,ngr,x,box)
         if (ifild.eq.2) call fdat(4,ispgr,nhit4,titel,
     +        epot/ngr,spname,nrp*ngr21,ngr,x,box)
         if (ifilc.ne.0) call fdat(ifilc+1,ispgr,nhit4,titel,
     +        epot/ngr,spname,nrp*ngr21,ngr,x,box)
      endif

C     Write results to parameter file
      if (ngr.gt.1) then
C        More than one molecule, no position and orientation parameters
         kmax=6
         ndih1=0
      else
C        Determine position and orientation parameters for one molecule
         call trans(nrp,x,ireftr,xcm,scr,phi,theta,psi)
         do i=1,3
            box(i+6)=xcm(i)
         enddo
         box(10)=phi
         box(11)=theta
         box(12)=psi
         kmax=12
         ndih1=ndiha
         do m=1,ndih1
            tau(m)=rad*dihan(m,x)
         enddo
      endif
      kpar1=0
      do i=1,kmax
         if (iref(i).gt.0) then
            kpar1=kpar1+1
            par(kpar1)=box(i)
            if (i.gt.9) par(kpar1)=par(kpar1)/100
         endif
      enddo
      write(form30,19) kpar1+ndih1
      vol=box(1)*box(2)*box(3)/(ngr*(nspgr+1))
      if (abs(epot/ngr).gt.99999) epot=99999*ngr

      etot=epot/ngr
      if (etot .gt. 50000) then
         print*,'***** Error in structure', nhit5,nhit4,etot
         print*,'      Probably something wrong in stedes or congra'
         write(16,*)'***** Error in structure', nhit5,nhit4,etot
         write(16,*)'      Probably something wrong in stedes or congra'
C        Try the next structure
         goto 300
      endif
      if (nq.ge.0) then
         if (iprint.gt.1) print 20,etot
         nmols=0
         einter=1.0d30
         inner=insav
         call latvib(nq,flist,ntemp,vtemp,box,ivamp,0,
     +        einter,nmols,iprint,0,1,
     +        nneg,npos,cvib,evib,sqm,scl,uqm,ucl,zpemol,zpetot,
     +        jjsh,n1sh,n2sh,n3sh,msh,sh)
         print 17,nhit4,nneg,npos
         write(16,17) nhit4,nneg,npos
         if (iprint.gt.1 .and. npos.gt.1) then
            print 14
            do i=1,ntemp
               Tvib=vtemp(i)
               fvib=evib(i)-Tvib*sqm(i)
               print 16,Tvib,cvib(i),evib(i),sqm(i),
     +              scl(i),fvib,uqm(i),ucl(i)
               if (nmols.gt.0) print 18,nmols*(nspgr+1),zpemol
            enddo
         endif
         fe=evib(1)-vtemp(1)*sqm(1)
         etot=etot+fe+zpetot
         print 15, nhit4,epot/ngr,zpetot,vtemp(1),fe,fe+zpetot,etot,nneg
         write(16,15) nhit4,epot/ngr,zpetot,vtemp(1),fe,fe+zpetot,
     +        etot,nneg
      endif

      write(30,form30) nhit5,nhit4,etot,
     +     (nint(1000*par(m)),m=1,kpar1),(nint(tau(m)),m=1,ndih1),
     +     imult
      dens=0.0016605d0*totm/vol
      if (rmsder.gt.999.99) rmsder=999.0
      if (rmsrat.gt.10) print*,' Large value of final shifts:',rmsder
      if (iprint.gt.0 .or. rmsrat.gt.10) print 46, 
     +     nhit4,geel,etot,wit,dens,rmsder,(box(m),m=1,6)
      write(16,53) nhit4,etot,dens,rmsder,(box(m),m=1,6)
      if (iprint.gt.1) print 47,
     +     eb/ngr,eh/ngr,ei/ngr,ep/ngr,enb/ngr,uinner/ngr,epol/ngr
      if (iprint.gt.0 .and. ndiha.gt.0) then
         do m=1,ndiha*ngr
            tau(m)=rad*dihan(m,x)
         enddo
         do i=1,ngr
            print 44,etot,i,(tau(m),m=(i-1)*ndiha+1,i*ndiha)
         enddo
      endif

C     End of calculation for this entry, try the next
      goto 300

  850 continue
C     End of PACK file reached
      print*,nhit5,' coordinate sets from PACK-file'
      write(16,*) nhit5,' coordinate sets from PACK-file'
      if (doknot.gt.0) then
         print*, nknot, ' knotted structures refused'
         write(16,*) nknot, ' knotted structures refused'
      endif
C     If desired, read file IN.ADD with additional coordinate sets.
C     (MDI or SPF; SPF sections must now be closed with an END card)
      n=inpt('EXTRA ',regel)
      if (n.ge.1) then
         if (n.ge.2) call intstr(regel,2,ncysd)
         if (n.ge.3) call realst(regel,3,tolsd)
         if (n.ge.4) call intstr(regel,4,ncycg)
         if (n.ge.5) call realst(regel,5,tolcg)
      endif
      open(unit=14,file='in.add',err=900,status='OLD')
      iextra=1
      print 27
      write(16,27)
      goto 300

  900 continue
      print*,nhit5,' coordinate sets done'
      write(16,*) nhit5,' coordinate sets done'
      i=inpt('CHECK ',regel)
      stop

  930 print*,' ***** Error reading header pack-file'
      stop
  940 print*,' ***** Error reading data pack-file'
      stop

   11 format(a80)
   12 format(a130)
   13 format(a)
   14 format(7x,'T',8x,'Cv           E     S(qm)     ',
     +     'S(cl)           F      u(qm)      u(cl)')
   15 format(' Structure',i7.5,'   Basis',f10.3,'   Zero-point',f9.3,
     +     '   Free (Temp ',f6.1,')',f9.3,'  Corr',f9.3,
     +     '   Total',f10.3,'   Imag',i5)
   16 format(f8.2,f10.5,f12.3,2f10.5,f12.3,2f11.4)
   17 format(' Structure',i7,'   LATVIB found',i5,' imaginary and',
     +     i6,' regular lattice vibrational modes')
   18 format(' Lattice zero-point energy based on',i4,
     +     ' molecules per cell:',f8.3)
   19 format('(2i7,f10.3,1x,',i2,'i5,i7)')
   20 format('  Energy without LATVIB contribution',f9.3)
   21 format(5i8,f10.5)
   22 format(' Additional structure',i8,' with title:  ',a80)
   23 format(i8,5f8.1)
   24 format(i8)
   25 format(i5.5,a4)
   26 format('   pressure (atm)',f18.1)
   27 format('------------------------------------------------------',
     +     /,'Start reading extra files from file IN.ADD')
   28 format(i4,' atoms found on coordinate file')
   29 format(i8,3f8.4,4i8)
   30 format(' Molecule',i4,' from atoms',i4,' to',i4)
   31 format(' ***** Inconsistency in PACK-file',10i5)
   32 format(f8.3,4f8.2,2f10.5)
   33 format(i8,2f8.4,i8,f8.4,f12.7)
   34 format(' Expand to',i4,' groups and calculate in P1, ', 
     +     'with maximum disturbance',f10.5)
   35 format(i7,34x,f9.4)
   36 format('  stcin: ',12f10.2)
   37 format(10i8)
   38 format(i8,4f8.4)
   39 format(2i8,2x,4(3i5,3x))
   40 format(' ***** Number of atoms on topology file is',i4,
     +     ', but',i4,' atoms are found in the parameter file')
   41 format(8x,'Starting calculation',i6,' on structure',i8,
     +     ' with box:',2x,6f7.3)
   42 format(' End of STEDES:',i4,i8,f14.3,f12.4,3x,12f7.3)
   43 format(3f10.5,7x,3f10.5)
   44 format(10x,f12.3,'   Dihedrals for molecule',i3,': ',30f8.1)
   45 format(' RMS for space group symmetry contraction: ',3f10.3)
   46 format(i7,a,f12.3,a,' Dens =',f9.4,'  R =',f9.4,
     +     '   Parameters',6f8.4)
   47 format(10x,'Eb...enb, E-inner, E-pol: ',4f7.2,f8.2,2f7.2)
   48 format(a4,i6,' cycles, energy',f11.3,'  rmsder',f8.2,
     +     '   Parameters',6f8.4)
   49 format(' End of CONGRA:',i4,i8,f14.3,f12.4,3x,12f7.3)
   50 format(' Average coordinates and rms deviations ',
     +     'after space group symmetry contraction',/,
     +     ' NOTE: energy does NOT refer to these coordinates',
     +     ' but to expanded ones!')
   51 format(i5,'-th independent molecule, coordinate shifts',
     +     ' in cell axes:' ,3i5)
   52 format('  Extracting only structures with RMSDER >',f11.5)
   53 format(i7,f12.3,'  Dens =',f9.4,'  R =',f9.4,
     +     '   Parameters',6f8.4)
   54 format(' Refused:',i3,' crossings found for structure',4i10)
   55 format(/,i7,' Checking for knotted structure')
   60 format('    Connected to atom',i4,' are atoms',50i4)
   61 format(' Searching connections for dihedral',4i4)
   62 format(' ***** But this dihedral cannot take a random value ',
     +     ' since it is part of a ring:',4i4)
   72 format(i5,5x,a5,i5,6f8.4)
   73 format(3f10.5,3f10.3)
   74 format(i5,3f10.5)
   93 format(6f10.4)
   94 format(15i5)
   95 format(i5,2x,a40)
   97 format(i7,2x,7f8.4)
   98 format(15f8.5)
      end
