      program prep

C     Calculates energy of a single geometry, and optimizes this geometry.
C     This can be done for the crystal or for the free molecule.
C     In the latter case selected dihedral angles can be restrained.
C     Can expand united atoms to all H atoms.
C     Can expand to space group P1.
C     Can perform lattice vibration calculations.
C     B. P. van Eijck, November 1997.
C     Input files:  IN.TP (unit 20), molecular topology
C                   IN.SPG (unit 11), space group
C                   IN.FF (unit 12), force field
C                   IN.PP (unit 13), code-directed input
C                   IN.COR (unit 14), coordinates (MDI or SPF)
C     Output files: OUT.PRI (unit 16), general results
C                   OUT.EXP (unit 18), crystal data before minimalization
C                   OUT.TGT (unit 19), crystal data after minimalization
C                   00000.SPF, 00000.MDI, 00000.DAT, coordinates
C                       (if ISPGR=0 in the reference axes system)
C                   PARFILE (unit 15), parameter summary

C     File IN.TP (read by subroutine LEEST) is the formatted topology file
C        created by program CODES and/or by hand.
C     File IN.SPG (read by subroutine LEESR) contains the space group data
C        for triclinic, monoclinic, or orthorhombic space groups.
C     File IN.COR (read by subroutine LEESX) contains the coordinates
C        Allowed formats are SPF or MDI.
C     File IN.PP contains data specific for this program.

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cbond.i'
      include 'cwald.i'
      include 'cvira.i'
      include 'cnonp.i'
      include 'cnonb.i'
      include 'cnbal.i'
      include 'cnbkl.i'
      include 'cpars.i'
      include 'cinds.i'
      include 'cpola.i'
      include 'color.i'
      include 'caidt.i'
      include 'chadd.i'

      double precision a, alpha,arot,ax,bb,beta,box,boxlist,brot,
     +     bx,by,cbu,cc,co,cosa,cosb,cosc,cvib,cvib1,cvp,cx,cy,cz,
     +     db,dbox,depcg,der,dihan,dihin,din,dscan,dsh,dtest,dx,
     +     eb,ebond,ebu,edispa1,edispa2,edispr1,edispr2,eel1,eel2,eg2,
     +     eh,ei,einter,enb,enb1,enb2,eoud1,eoud2,ep,epol1,epot,
     +     eps,escan,estat,ete,evib,evib1,expand,f1,f12,f2,f3,f6,
     +     facq,facr,fact,flist,free,fvib,gamma,omega,p12,p6,par,
     +     phi,pi,pres,psi,pv,rad,random,refbox,rmd,rmin,rmsder,
     +     scl,scr,scr1,scr2,shmcg,shmsd,si,sigma,sqm,sqm1,stc,stcin,
     +     stpcg,stpsd,tau,theta,tolcg,tolsd,totm,tvib,ucl,uqm,
     +     vimp,vol,vtemp,xcm,y1,y2,y3,zpemol,zpet1,zpetot,dens,
     +     eb1,eh1,ei1,ep1,ebond1
      integer i,i1,i2,i3,i4,iaci,iadd,ibl,iconn,icopt,icoul,icycb,
     +     iexp,ifilc,ifild,ifilm,ifils,iflex,igrid,ihess,iim,inpt,
     +     insav,intra,ios,ioscor,iout29,ipr,ipr1,ipr2,ipr3,ipr4,ipr5,
     +     iprint,irefs,ireset,iscan,iseed,ispgr,itemp,ivamp,ix,iy,iz,
     +     j,j3,jbl,jjm,jjsh,jlist,k,k3,kkm,klist,l,m,maxbox,maxcy,
     +     maxlis,mbox,mold,msh,n,n1,n1sh,n2,n2sh,n3,n3sh,nas,ireft,
     +     nbox,ncyc,ncycb,ncycg,ncysd,ndt,ndum,nexpd,nfix,ngr1,ngrid,
     +     nim,nimprs,nimtot,njlist,njm,nklist,nkm,nn,nneg,nop3,npar0,
     +     npar1,npar2,npos,nq,nr3,nresd,nrps,nrpt,nstruc,ntemp,numm,
     +     nx,ny,nz,nintra,ninter,iknot,doknot
      parameter(maxbox=1000,maxlis=100)
      dimension flist(0:maxq,3),vtemp(maxtmp),scl(maxtmp),sqm(maxtmp),
     +     ucl(maxtmp),uqm(maxtmp),cvib(maxtmp),evib(maxtmp),
     +     sqm1(maxtmp),cvib1(maxtmp),evib1(maxtmp),
     +     free(maxtmp,0:maxlis),boxlist(maxlis,6),dbox(20)

      character*8 naam1,spname
      character titel*80,regel*200,filenm*40
      character*30 btra1,btra3,bter1,bter3

      dimension xcm(3),arot(3),brot(3),irefs(6),ireft(3),
     +     der(6),box(maxbox),refbox(6),stcin(6),
     +     stc(maxpar),scr(maxpar),scr2(maxpar),par(maxpar),
     +     tau(maxome*maxmol),dihin(maxome*maxmol),klist(maxnrp),
     +     njlist(maxmol*maxome),jlist(maxmol*maxome,maxnrp),
     +     Vimp(maxmol*maxome)
      external func1,func2,func0,func3g

      btra1='NONE'
      btra2='NONE'
      btra3='NONE'
      bter1='NONE'
      bter2='NONE'
      bter3='NONE'
      iai=0
      idftb=0
      aiofst=0
      saib=0.1d0
      saih=5
      said=5
      maxcy=15
      ncycga=0
      ncyc=0
      icopt=1
      igrid=0
      dbox(1)=0.01d0
      iscan=0
      dscan=0
      itemp=0
      ncycb=0
      iout29=0
C     Set iout29 = 1 if a file with grid point data is required
      nas=0
      irfat1=0
      irfat2=0
      jjsh=0

      iseed=120297
      pi=4*atan(1.0d0) 
      rad=180/pi
      fact=4.184d0
      facr=0.10d0
      facq=sqrt(332.0636d0*0.4184d0)
      f6=fact*(facr**6)
      f12=fact*(facr**12)
      do i=1,12
         cel(i)=0
      enddo
      do i=1,maxome
         Vn(i)=0.3d0
         ntal(i)=3
      enddo
      ispgr=0
      spname='nospgr'
      inner=2
      iflex=1
      icoul=1
      intra=1
      pres=0
      iexp=0
      nx=1
      ny=1
      nz=1
      expand=0
      do i=1,6
         stcin(i)=1
      enddo
      stcin(5)=100
      do i=1,maxome*maxmol
         dihin(i)=0
      enddo
      ncysd=100
      tolsd=0.01d0
      stpsd=0.0001d0
      nresd=5
      shmsd=0.02d0
      ncycg=5000
      tolcg=0.0005d0
      stpcg=0.005d0
      depcg=0.3d0
      shmcg=0.02d0
      iprint=0
      dtest=0
      ireset=0
      numm=0
      nvir=0
      ifils=1
      ifilm=1
      ifild=1
      ifilc=2
      ichad=0
      cutlv=1.5d0
      tollv=0
      epsil=1
      nintra=0
      ninter=0

      open(unit=13,file='filename',status='OLD',err=110)
      read(13,*) filenm
      close(13)
  110 continue
      open(unit=16,file='out.pri')
      open(unit=15,file='parfile')
      open(unit=13,file='in.pp',status='OLD',iostat=ios)
      print*,geel,' P R E P',wit
      write(16,*) ' '
      write(16,*) ' P R E P'
      regel=' '
      if (ios.ne.0) then
         print*,' No file IN.PP present'
         write(16,*) ' No file IN.PP present'
      else
         n=inpt('INPUT ',regel)
         close(13)
      endif
      n=inpt('SPGR  ',regel)
      if (n.gt.1) then
         call strext(regel,2,spname)
         ispgr=-1
         read(spname,34,err=120) i
         ispgr=i
  120    continue
      endif

      nq=-1
      n=inpt('LATVIB',regel)
      if (n.ge.1) then
C        Prepare for calculation of lattice vibrations
         ntemp=1
         vtemp(1)=298
         ivamp=0
         do i=1,3
            flist(0,i)=0
         enddo

         if (n.eq.1) nq=0
         if (n.eq.2) then
C           NQ random points
            call intstr(regel,2,nq)
            if (nq.gt.maxq) then
               print*,' ***** Increase MAXQ',nq
               stop
            endif
            do i=1,nq
               do m=1,3
                  flist(i,m)=random(iseed)
               enddo
            enddo
         endif
         if (n.eq.4) then
C           N1 * N2 * N3 cells
            call intstr(regel,2,n1)
            call intstr(regel,3,n2)
            call intstr(regel,4,n3)
            if (n1*n2*n3.gt.maxq) then
               print*,' ***** Increase MAXQ',nq
               stop
            endif
            nq=0
            do i1=1,n1
               do i2=1,n2
                  do i3=1,n3
                     nq=nq+1
                     flist(nq,1)=i1/dfloat(n1)
                     flist(nq,2)=i2/dfloat(n2)
                     flist(nq,3)=i3/dfloat(n3)
                  enddo
               enddo
            enddo
         endif
         if (n.eq.5) then
C           One branch of q-values, divided into NQ steps
            call realst(regel,2,f1)
            call realst(regel,3,f2)
            call realst(regel,4,f3)
            call intstr(regel,5,nq)
            if (nq.gt.maxq) then
               print*,' ***** Increase MAXQ',nq
               stop
            endif
            do i=1,nq
               flist(i,1)=f1*i/dfloat(nq)
               flist(i,2)=f2*i/dfloat(nq)
               flist(i,3)=f3*i/dfloat(nq)
            enddo
         endif
         if (n.gt.5) then
            print*,' Too many entries on LATVIB card'
            stop
         endif
         n=inpt('VIBAMP',regel)
         if (n.ge.1) ivamp=1
         n=inpt('CUTLV ',regel)
         if (n.ge.2) call realst(regel,2,cutlv)
         if (n.ge.3) call realst(regel,3,tollv)
         n=inpt('TEMP  ',regel)
         if (n.gt.1) then
            ntemp=n-1
            do i=1,ntemp
               call realst(regel,i+1,vtemp(i))
            enddo
         endif
C        Check double precision
         a=(1/3.0d0)
         if (abs(a-(1/3.0d0)).gt.1.0d-14) then
            print*,' ***** LATVIB requires double precision'
            stop
         endif
      endif

      n=inpt('INNER ',regel)
      if (n.ge.2) call intstr(regel,2,inner)
      n=inpt('INTRA ',regel)
      if (n.ge.2) call intstr(regel,2,intra)
      noint=(intra.eq.0)
      n=inpt('CHADD ',regel)
      if (n.ge.2) call intstr(regel,2,ichad)
      n=inpt('MOLC  ',regel)
      if (n.ge.3) then
         call intstr(regel,2,irfat1)
         call intstr(regel,3,irfat2)
      endif

C     Data for ab-initio calculations
      ninter=inpt('BINTER ',regel)
      if (ninter.ge.4) then
         call strext(regel,2,bter1)
         call strext(regel,3,bter2)
         call strext(regel,4,bter3)

         if ((bter1.eq.'IMPORT' .and. bter2.eq.'NONE') .or.
     +        bter2.eq.'IMPORT' .or. bter3.eq.'IMPORT') then
            print*,'***** Code IMPORT not allowed here'
            stop
         endif
         if (bter1.ne.'IMPORT' .and. bter1.ne.bter2) then
            print 101,bter1,bter2
            stop
         endif
         if (bter1.eq.'NONE') then
            print*,' ***** No ab-initio intermolecular energy!'
         else
ccccc            iai=2
            if (ichad.eq.0) ichad=1
         endif
      else
         ninter=0
      endif
      nintra=inpt('BINTRA ',regel)
      if (nintra.ge.4)then
         call strext(regel,2,btra1)
         call strext(regel,3,btra2)
         call strext(regel,4,btra3)
         if ((btra1.eq.'IMPORT' .and. btra2.eq.'NONE') .or.
     +        btra2.eq.'IMPORT' .or. btra3.eq.'IMPORT') then
            print*,'***** Code IMPORT not allowed here'
            stop
         endif
         if (btra1.ne.'IMPORT' .and. btra1.ne.btra2) then
            print 101,btra1,btra2
            stop
         endif
         if (btra1.eq.'NONE') then
            print*,' ***** No ab-initio intramolecular energy!'
         else
ccccc            iai=2
            noint=.true.
         endif
      else
         nintra=0
      endif
      if (bter1.eq.'DFTB'.or. btra1.eq.'DFTB') then
         idftb=1
         btra1='NONE'
         bter1='NONE'
      endif
      if (bter2.eq.'DFTB'.or. btra2.eq.'DFTB') then
         idftb=1
         btra2='NONE'
         bter2='NONE'
      endif      
      if (bter3.eq.'DFTB'.or. btra3.eq.'DFTB') then
         idftb=1
         btra3='NONE'
         bter3='NONE'
      endif
      n=inpt('AINTRA',regel)
      if (n.gt.1) call realst(regel,2,aiofst)
C     Offset for initio intramolecular energy (atomic units)
      n=inpt('MAXSHI',regel)
      if (n.ge.4) then
         call realst(regel,2,saib)
         call realst(regel,3,saih)
         call realst(regel,4,said)
      endif
      n=inpt('MAXCYC',regel)
      if (n.ge.2) call intstr(regel,2,maxcy)
      ihess=0
      n=inpt('HESS  ',regel)
      if (n.ge.2) call intstr(regel,2,ihess)

C     Other data
      do i=1,10
         icn(i)=0
      enddo
      n=inpt('CONN  ',regel)
      do i=2,n
         call intstr(regel,i,m)
         icn(i-1)=m
      enddo
      n=inpt('FLEX  ',regel)
      if (n.ge.2) call intstr(regel,2,iflex)
      if (iflex.eq.0) noint=.true.
      n=inpt('COUL  ',regel)
      if (n.ge.2) call intstr(regel,2,icoul)
      n=inpt('EPSIL ',regel)
      if (n.gt.0) call realst(regel,2,epsil)
      n=inpt('PRINT ',regel)
      if (n.ge.2) call intstr(regel,2,iprint)
      ipr1=iprint
      ipr2=iprint
      ipr3=iprint
      ipr4=iprint
      ipr5=iprint
      if (n.ge.3) call intstr(regel,3,ipr1)
      if (n.ge.4) call intstr(regel,4,ipr2)
      if (n.ge.5) call intstr(regel,5,ipr3)
      if (n.ge.6) call intstr(regel,6,ipr4)
      if (n.ge.7) call intstr(regel,7,ipr5)
      n=inpt('FILES ',regel)
      if (n.ge.2) call intstr(regel,2,ifils)
      if (n.ge.3) call intstr(regel,3,ifilm)
      if (n.ge.4) call intstr(regel,4,ifild)
      if (n.ge.5) call intstr(regel,5,ifilc)
      n=inpt('EXPAND',regel)
      if (n.gt.0) iexp=1
      if (n.eq.2) call realst(regel,2,expand)
      if (n.eq.5) then
         call intstr(regel,3,nx)
         call intstr(regel,4,ny)
         call intstr(regel,5,nz)
      endif
      n=inpt('PRES  ',regel)
      if (n.ge.2) call realst(regel,2,pres)
      doknot=0
      n=inpt('KNOT  ',regel)
      if (n.gt.0) doknot=1
      n=inpt('SD    ',regel)
      if (n.ge.2) call intstr(regel,2,ncysd)
      if (n.ge.3) call realst(regel,3,tolsd)
      if (n.ge.4) call realst(regel,4,stpsd)
      if (n.ge.5) call intstr(regel,5,nresd)
      if (n.ge.6) call realst(regel,6,shmsd)
      n=inpt('CG    ',regel)
      if (n.ge.2) call intstr(regel,2,ncycg)
      if (n.ge.3) call realst(regel,3,tolcg)
      if (n.ge.4) call realst(regel,4,stpcg)
      if (n.ge.5) call realst(regel,5,depcg)
      if (n.ge.6) call realst(regel,6,shmcg)
      n=inpt('TEST  ',regel)
      if (n.ge.2) call realst(regel,2,dtest)
      if (n.gt.7) then
C        Undocumented option: check SECDER elements (ii,m1;jj,m2,n1,n2,n3)
C           for all ii and all m, with jj=jjsh, m2=msh, etc; step dsh.
         call intstr(regel,3,jjsh)
         call intstr(regel,4,n1sh)
         call intstr(regel,5,n2sh)
         call intstr(regel,6,n3sh)
         call realst(regel,7,dsh)
         call intstr(regel,8,msh)
      endif
      n=inpt('STC   ',regel)
      if (n.ge.7) then
         call realst(regel,2,stcin(1))
         call realst(regel,3,stcin(2))
         call realst(regel,4,stcin(3))
         call realst(regel,5,stcin(4))
         call realst(regel,6,stcin(5))
         call realst(regel,7,stcin(6))
      elseif (n.gt.0) then
         print*,' ***** Wrong number of items in STC',n
         stop
      endif
      n=inpt('RESET ',regel)
      if (n.gt.0) ireset=1
      if (ispgr.eq.0 .and. iflex.eq.2) then
         print*,' ***** Do not combine free molecule with IFLEX=2'
         stop
      endif
      if (iexp.ne.0 .and. iflex.ne.1) then
         print*,' ***** Do not combine P1-expansion with IFLEX<>1'
         stop
      endif

      call leest(nrp,icoul,iprint)
      open(unit=14,file='in.cor',status='OLD',iostat=ioscor)
      if (ioscor.ne.0) then
         if (ispgr.eq.0) then
            filenm=' Constructed by subroutine CART'
            inner=0
            ngr=1
            nrpt=nrp
            nr3=3*nrp
            print 31
            write(16,31)
            call cart(nrp,x,iprint)
c            do i=1,nrp
c               print*,i,panm(i),(x(3*i-3+m),m=1,3)
c            enddo
            call leesx(0,nrp,nrpt,ngr,0,nstruc,
     +           titel,spname,ispgr,x,box,iprint)
c            do i=1,nrpt
c               print*,i,panm(i),(x(3*i-3+m),m=1,3)
c            enddo
            ngr1=1
            ndum=nrp
            ispgr=0
            spname='nospgr'
            nstruc=1
            goto 140
         else
            print*,' ***** No coordinate file IN.COR present'
            stop
         endif
      endif
      
      nstruc=0
  130 continue
C     Read new entry from coordinate file
      maxcyc=maxcy
      naam1=spname
      insav=inner
      nstruc=nstruc+1
      call leesx(14,ndum,nrp,ngr1,ireset,nstruc,
     +     titel,spname,ispgr,x,box,ipr1)
      if (ngr1.eq.0) then
         print*,' ***** Error in PREP'
         write(16,*)' ***** Error in PREP'
         print*,' Type GO if you accept this and wish to continue'
         read*, regel
         if (regel.ne.'GO' .and. regel.ne.'go') stop
         goto 130
      endif
      if (ndum.lt.0) then
         print*,' No (more) data on file IN.COR,',
     +        nstruc-1,' structures done'
         write(16,*) ' No (more) data on file IN.COR,',
     +        nstruc-1,' structures done'
         i=inpt('CHECK ',regel)
         stop
      endif
      if (icoul.gt.1) then
c         print 90
c         write(16,90)
         do i=1,nrp*ngr1
            cg1(i)=cg(i)
         enddo
         call newchg(icoul,nrp*ngr1)
C        New charges are delivered in CG
         print 90
         write(16,90)
         do i=1,nrp*ngr1
            print 21,i,panm(i),cg1(i)/facq,cg(i)/facq
            write(16,21) i,panm(i),cg1(i)/facq,cg(i)/facq
         enddo
      endif
  140 continue
      do i=1,3*ndum
         xr(i)=x(i)
      enddo
c      nstruc=nstruc+1
c      if (nstruc.gt.1) then
c        print 29,nstruc
c         write(16,29) nstruc
c      endif
      print 61,ndum,ngr1,titel(1:80)
      write(16,61) ndum,ngr1,titel(1:80)
c      ngr1=abs(ngr1)
c      if (spname(1:2).ne.'no') then
      print 60,spname,ispgr
      write(16,60) spname,ispgr
      if (naam1.ne.spname) then
         if (naam1(1:2).ne.'no') then
c               ispgr=-1
c               spname=naam1
            print 64,naam1
            write(16,64) naam1
         else
c               ispgr=0
         endif
      endif 
c         ispgr=-1
c         read(spname,34,err=150) i
c         ispgr=i
c  150 continue
c      endif
      
c      insav=inner
c      call leesr(ispgr,spname,iref,iscr,msom,inner)
C     LEESR may set INNER to zero
      if (nq.ge.0 .and. iflex.ne.1) then
         print*,' ***** Do not combine LATVIB with IFLEX<>1'
         stop
      endif

C     Read data for grid search around optimized geometry
      n=inpt('CELOPT',regel)
      if (n.ge.2) call intstr(regel,2,icopt)
      if (n.ge.3) then
         ncycb=n-2
         do i=1,ncycb
            call realst(regel,i+2,dbox(i))
         enddo
      endif
      icycb=1
      db=dbox(1)

      nexpd=0
      if (iexp.ne.0) then
C        Expand to P1
         print 35, ngr1*(nspgr+1),expand
         write(16,35) ngr1*(nspgr+1),expand
         nexpd=nspgr
         ispgr=1
         nspgr=0
         spname='P1'
         inner=insav
         do m=1,3
            msom(m)=1
            iref(m+3)=1
            iref(m+6)=0
         enddo
      endif
      ngr=ngr1*(nexpd+1)
      nrps=nrp*ngr
      ngr=ngr1*(nexpd+1)*nx*ny*nz
C     NGR is the total number of groups in the asymmetric unit
      nrpt=nrp*ngr
      aiofst=aiofst*ngr

C     NRPT is the total number of atoms in the asymmetric unit
      if (nrpt.gt.maxat) then
         print*,' ***** Increase MAXAT',ngr,nrp,maxat
         stop
      endif
      nr3=3*nrpt
      finner=16*atan(1.0d0)/(3.0d0*(nspgr+1))
      druk=0.061018d0*pres/(nspgr+1)
C     PRES is in atmospheres, DRUK in adapted units
C     These quantities refer to the asymmetric unit, as corresponding to NONBE.
      
c  160 continue
      nmols=1
      imols(1)=1
C     Check for disconnected sets of atoms
      do i=2,nrpt
         nim=(i-1)/nrp
         iim=mod(i-1,nrp)+1
         do j=1,i-1
            njm=(j-1)/nrp
            if (nim.eq.njm) then
               jjm=mod(j-1,nrp)+1
               if (iexcl(jjm,iim).ne.0) goto 170
            endif
         enddo
         do k=i+1,nrpt
            nkm=(k-1)/nrp
            kkm=mod(k-1,nrp)+1
            do j=1,i-1
               njm=(j-1)/nrp
               if (nkm.eq.njm) then
                  jjm=mod(j-1,nrp)+1
                  if (iexcl(jjm,kkm).ne.0) goto 170
               endif
            enddo
         enddo
         print*,' Atom',i,' is not connected to previous ones'
         write(16,*) ' Atom',i,' is not connected to previous ones'
         nmols=nmols+1
         if (nmols.ge.maxmol) then
            print*,' ***** Increase MAXMOL'
            stop
         endif
         imols(nmols)=i
  170    continue
      enddo
C     There are NMOLS separate molecules, coordinates starting at IMOLS
      imols(nmols+1)=nrpt+1
      do i=1,nmols
         print 29,i,imols(i),imols(i+1)-1
         write(16,29) i,imols(i),imols(i+1)-1
      enddo
      mbox=6*nmols+6
      if (mbox.gt.maxbox) then
         print*,' ***** Increase MAXBOX',mbox,maxbox
         stop
      endif

      if (ispgr.ne.0) then      
         print*,'Box from input coordinates:'
         print 30,(box(m),m=1,6)
         write(16,*) 'Box from input coordinates:'
         write(16,30) (box(m),m=1,6)
         ax=box(1)
         by=box(2)
         cz=box(3)
         bx=box(4)
         cx=box(5)
         cy=box(6)
         bb=sqrt(bx**2+by**2)
         cc=sqrt(cx**2+cy**2+cz**2)
         cosa=(bx*cx+by*cy)/(bb*cc)
         cosb=cx/cc
         cosc=bx/bb
         alpha=rad*acos(cosa)
         beta=rad*acos(cosb)
         gamma=rad*acos(cosc)
         print*,'Cell from input coordinates:'
         print 54,ax,bb,cc,alpha,beta,gamma
         write(16,*) 'Cell from input coordinates:'
         write(16,54) ax,bb,cc,alpha,beta,gamma
      endif
      if (ispgr.ne.0 .and. iexp.ne.0) then
C        Expand asymmetric unit to P1
         ax=box(1)
         by=box(2)
         cz=box(3)
         bx=box(4)
         cx=box(5)
         cy=box(6)
         if (iprint.gt.3) print*,' Expanded coordinates:'
         do nop=1,nexpd
            nop3=3*nop-3
            do i=1,nrp*ngr1
               j=3*i-3
               k=j+3*nop*nrp*ngr1
               scr1=2*expand*(random(iseed)-0.5d0)
               x(k+1)=scr1+msp(nop3+1)*x(j+1)+
     +              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               scr1=2*expand*(random(iseed)-0.5d0)
               x(k+2)=scr1+msp(nop3+2)*x(j+2)+
     +              tsp(nop3+2)*by+tsp(nop3+3)*cy
               scr1=2*expand*(random(iseed)-0.5d0)
               x(k+3)=scr1+msp(nop3+3)*x(j+3)+tsp(nop3+3)*cz
               if (iprint.gt.3) print 74,i,(x(k+m),m=1,3)
            enddo
            if (ichad.eq.2) then
               do j=1,nvir*ngr
                  k=(jvir(j,1)-1)/(nrp*ngr1)
                  if (k.eq.nop) then
C                    Adjust sign for virtual atom containing vector product
                     schad(j)=schad(j)*
     +                    msp(nop3+1)*msp(nop3+2)*msp(nop3+3)
                  endif
c                  print*,nop,j,jvir(j,1),nrp,ngr1,k,schad(j)
               enddo
            endif
         enddo
      endif

      if (nx*ny*nz.gt.1) then
C        Expand equivalent cells
         print*,' Expand equivalent cells:',nx,ny,nz
         write(16,*) ' Expand equivalent cells:',nx,ny,nz
         box(1)=ax*nx
         box(2)=by*ny
         box(3)=cz*nz
         box(4)=bx*ny
         box(5)=cx*nz
         box(6)=cy*nz
         k=nrps
         do ix=0,nx-1
            do iy=0,ny-1
               do iz=0,nz-1
                  if (ix.ne.0 .or. iy.ne.0 .or. iz.ne.0) then
                     do i=1,nrps
                        k=k+1
                        i3=3*i-3
                        k3=3*k-3
                        x(k3+1)=x(i3+1)+ix*ax+iy*bx+iz*cx
                        x(k3+2)=x(i3+2)+iy*by+iz*cy
                        x(k3+3)=x(i3+3)+iz*cz
                        if (iprint.gt.3)
     +                       print*,ix,iy,iz,i,k,(x(k3+m),m=1,3)
                     enddo
                  endif
               enddo
            enddo
         enddo
      endif

C     Voor alleen expansie zonder energieberekening:
c      goto 670

      if (ndiha.gt.0 .and. iexp.eq.0) then
         do i=1,ngr
            do k=1,ndiha
               tau(k)=rad*dihan((i-1)*ndiha+k,x)
            enddo
            print 62, i,'Initial dihedrals:',(tau(k),k=1,ndiha)
            write(16,62) i,'Initial dihedrals:',(tau(k),k=1,ndiha)
         enddo

         n=inpt('ROTH  ',regel)
         if (n.gt.0) then
C           Set hydroxyl dihedral angles to given values
            print*,'  Reset hydroxyl dihedrals to ROTH input'
            write(16,*) '  Reset hydroxyl dihedrals to ROTH input'
            if (n.lt.ndihh+1) then
               print*,' Wrong number of entries in ROTH',n,ndihh
               stop
            endif
            k=0
            do i=1,ngr
c               do j=(i-1)*ndiha+ndihz+1,(i-1)*ndiha+ndihz+ndihh
               do j=(i-1)*ndiha+ndihz+1,i*ndiha
                  k=k+1
                  call realst(regel,k+1,omega)
                  omega=omega/rad-dihan(j,x) 
                  co=cos(omega)-1
                  si=sin(omega)
                  i3=3*iome(j)-3
                  j3=3*jome(j)-3
                  k3=3*kome(j)-3
                  call introt(i3,j3,k3,x,arot,brot)
                  do m=1,3
                     x(i3+m)=x(i3+m)+si*brot(m)+co*arot(m)
                  enddo
               enddo
            enddo
            do i=1,ngr
               do k=1,ndiha
                  tau(k)=rad*dihan((i-1)*ndiha+k,x)
               enddo
               print 62, i,'Reset dihedrals:',(tau(k),k=1,ndiha)
               write(16,62) i,'Reset dihedrals:',(tau(k),k=1,ndiha)
            enddo
         endif
      endif

c  250 continue
C     If desired, override standard settings from topology file
      npp=0
      n=inpt('POL   ',regel)
      if (n.ge.2) call intstr(regel,2,npp)
      if (n.ge.3) then
         if (n.lt.9) then
            print*,' ***** Wrong number of items in POL',n
            stop
         endif
         call realst(regel,3,poltol)
         call realst(regel,4,schaal)
         call realst(regel,5,gampol)
         call intstr(regel,6,nnn)
         call intstr(regel,7,mmm)
         call intstr(regel,8,nte)
         call intstr(regel,9,nttt)
      endif
      n=inpt('CUTOFF',regel)
      if (n.ge.2) call realst(regel,2,cutoff)
      if (n.ge.3) then
         if (n.lt.8) then
            print*,' ***** Wrong number of items in CUTOFF',n
            stop
         endif
         call realst(regel,3,alphac)
         call realst(regel,4,alpha6)
         call realst(regel,5,hmaxc)
         call realst(regel,6,hmax6)
         call realst(regel,7,tole)
         call realst(regel,8,tolk)
      endif
      if (cutlv.lt.cutoff) cutlv=cutoff
      doewc=(icoul.gt.0 .and. alphac.gt.1.0e-9)
      doew6=(alpha6.gt.1.0e-9)
      doew=(doewc .or. doew6)
      if (doewc) inner=0
      if (abs(tole).gt.0.1 .or. abs(tolk).gt.0.1) then
         print*,' ***** Warning: TOLE, TOLK = ',tole,tolk
         write(16,*) ' ***** Warning: TOLE, TOLK = ',tole,tolk
      endif

C     Test for compatibility with polarization
      if (nq.ge.0 .and. npp.gt.0) then
         print*,' ***** Do not combine LATVIB with polarization'
         stop
      endif
      if (ichad.eq.2 .and. npp.ne.0) then
         print*,' ***** Do not combine virtual atoms with polarization'
         stop
      endif

      if (ispgr.eq.0) then
         doewc=.false.
         doew6=.false.
         doew=.false.
         cutoff=10
         alphac=0
         alpha6=0
         hmaxc=0
         hmax6=0
         tole=0
         tolk=0
         do m=1,3
            cel(m)=100
            box(m)=100
            cel(m+3)=0
            box(m+3)=90
         enddo
         nspgr=0
         do i=1,9
            iref(i)=0
         enddo
         if (ndiha.gt.0) then
            nfix=inpt('TORS  ',regel)-1
            if (nfix.gt.0) then
C              Target values for dihedrals will be enforced
               if (nfix.ne.2*ndiha) then
                  print*,' ***** Wrong number of entries in TORS',
     +                 nfix,ndiha
                  stop
               endif
               do i=1,ndiha
                  call realst(regel,2*i,dihin(i))
                  call realst(regel,2*i+1,Vimp(i))
                  print 20,i,Vimp(i),dihin(i)
                  write(16,20) i,Vimp(i),dihin(i)
               enddo
               n=inpt('SCAN  ',regel)
               if (n.ge.5) then
C                 One of these will be scanned (superseding the TORS value).
                  call intstr(regel,2,iscan)
                  call realst(regel,3,dihin(iscan))
                  call realst(regel,4,dscan)
                  call realst(regel,5,escan)
                  print*, 'But scan dihedral',iscan,' in range',
     +                 dihin(iscan),dscan,escan              
                  write(16,*) 'But scan dihedral',iscan,' in range',
     +                 dihin(iscan),dscan,escan              
               endif
            endif
            if (nimpr+ndiha.gt.maximp) then
               print*,' ***** Increase MAXIMP'
               stop
            endif
            do i=1,ndiha
               iq(nimpr+i)=iome(i)
               jq(nimpr+i)=jome(i)
               kq(nimpr+i)=kome(i)
               lq(nimpr+i)=lome(i)
               q0(nimpr+i)=dihin(i)/rad
            enddo
         endif
      endif

      totm=0
      do i=1,nrp
         totm=totm+amass(i)
      enddo
      print 51,totm
      write(16,51) totm

      if (ispgr.eq.0 .and. btra1.ne.'NONE  ') then
         print*,' Calculating in P1 with large cell and celopt = 0'
         ispgr=1
         spname='free-ai'
      endif

      write(16,*) '  inner   iflex   icoul  ireset'
      write(16,34) inner,iflex,icoul,ireset
      write(16,*) '   pressure (atm)'
      write(16,37) pres
      write(16,*) ' cutoff  alphac  alpha6   hmaxc   hmax6',
     +     '      tole      tolk'
      write(16,32) cutoff,alphac,alpha6,hmaxc,hmax6,tole,tolk
      if (npp.ne.0) then
         write(16,*) '    npp  poltol  schaal   gamma     nnn     mmm',
     +        '     nte    nttt'
         write(16,39) npp,poltol,schaal,gampol,nnn,mmm,nte,nttt
      endif
      write(16,*) '    nrp   ndihz   ndihh  ireftr...'
      write(16,34) nrp,ndihz,ndihh,(ireftr(m),m=1,nref)
      write(16,*) '    ngr     nrp    iexp     expand'
      write(16,44) ngr,nrp,iexp,expand
      write(16,*) '  ifils   ifilm   ifild   ifilc  iprint'
      write(16,28) ifils,ifilm,ifild,ifilc,iprint
      write(16,*) '  ncysd   tolsd   stpsd   nresd   shmsd       dtest'
      write(16,33) ncysd,tolsd,stpsd,nresd,shmsd,dtest
      write(16,*) '  ncycg   tolcg   stpcg   depcg   shmcg'
      write(16,40) ncycg,tolcg,stpcg,depcg,shmcg
      write(16,36) (stcin(m),m=1,6)
      write(16,94) (iref(i),i=1,12)
      if (ninter+nintra.gt.0) then
         write(16,96) aiofst
         write(16,92) maxcyc
         write(16,93) saib,saih,said
         write(16,102) 'Intra-',btra1,btra2,btra3
         write(16,102) 'Inter-',bter1,bter2,bter3
      endif
      if (nq.ge.0) then
         write(16,97) nq
         write(16,98) (vtemp(i),i=1,ntemp)
         if (ivamp.gt.0) write(16,*) '   Also vibrational amplitudes'
      endif

      if (iflex.eq.2) then
         n=inpt('VN    ',regel)
         if (n.gt.0) then
            if (n.lt.ndihh+1) then
               print*,' Wrong number of entries in VN',n,ndihh
               stop
            endif
            do i=1,ndihh
               call realst(regel,i+1,Vn(i))
            enddo
         endif
         n=inpt('NTAL  ',regel)
         if (n.gt.0) then
            if (n.lt.ndihh+1) then
               print*,' Wrong number of entries in NTAL',n,ndihh
               stop
            endif
            do i=1,ndihh
               call intstr(regel,i+1,ntal(i))
            enddo
         endif
         do i=1,ndihh
            print 56,i,dihin(i),Vn(i),ntal(i)
            write(16,56) i,dihin(i),Vn(i),ntal(i)
            Vn(i)=Vn(i)*4.184d0
         enddo
      endif
      
C     Write starting values for dihedrals
      if (ndiha.gt.0) then
         write(16,43)
         print 43
         do m=1,ndiha
            i1=iome(m)
            i2=jome(m)
            i3=kome(m)
            i4=lome(m)
            do i=1,ngr
               tau(i)=rad*dihan((i-1)*ndiha+m,x)
            enddo
            write(16,50) i1,i2,i3,i4,
     +           panm(i1),panm(i2),panm(i3),panm(i4),(tau(i),i=1,ngr)
            print 50, i1,i2,i3,i4,
     +           panm(i1),panm(i2),panm(i3),panm(i4),(tau(i),i=1,ngr)
         enddo
      endif

      ebig=1.0d30
      if (nr3+12.gt.maxpar) then
         print*,' ***** Increase MAXPAR',maxpar,nr3+12
         stop
      endif
      do i=1,nr3+12
         stc(i)=stcin(6)
      enddo
      nbox=0
      do i=1,6
         irefs(i)=iref(i)
         if (iref(i).gt.0) then
            nbox=nbox+1
         endif
      enddo

C     Transform to reference geometry XR
      do i=1,nmols
         nn=imols(i+1)-imols(i)
         k=0
         do j=imols(i),imols(i+1)-1
            do m=1,3
               k=k+1
               scr(k)=x(3*j-3+m)
            enddo
         enddo
         do j=1,3
            ireft(j)=ireftr(j+3*i-3)-imols(i)+1
         enddo
         call trans(nn,scr,ireft,xcm,scr2,phi,theta,psi)
         k=0
         do j=imols(i),imols(i+1)-1
            do m=1,3
               k=k+1
               xr(3*j-3+m)=scr2(k)
            enddo
         enddo
         box(6*i+1)=xcm(1)
         box(6*i+2)=xcm(2)
         box(6*i+3)=xcm(3)
         box(6*i+4)=phi
         box(6*i+5)=theta
         box(6*i+6)=psi
         if (nn.eq.1) then
            do m=4,6
               box(6*i+m)=0
            enddo
         endif
      enddo

C     Make file OUT.EXP, containing experimental parameters.
      if (ispgr.ne.0 .and. nstruc.eq.1) then
         open(unit=18,file='out.exp')
         write(18,41) (box(m),m=1,mbox)
         if (ndiha.gt.0) then
            ndt=0
            do i=1,ngr
               do k=1,ndiha
                  ndt=ndt+1
                  tau(ndt)=rad*dihan((i-1)*ndiha+k,x)
               enddo
            enddo
            write(18,47) (tau(m),m=1,ndt)
         endif
         close(18)
      endif

C     Check for modified charges and/or virtual atoms
      if (nvir.ne.0 .and. intra.ne.0) then
         print*,'***** Virtual atoms present, INTRA superseded by zero'
ccc         intra=0
         noint=.true.
      endif
      ncags=ncag

      write(16,80)
      if (iprint.gt.1) print 80
      do i=1,nrps
         write(16,75) i,panm(i),type(i),cg(i)/facq,
     +        1000*alfpol(i),(x(3*(i-1)+m),m=1,3)
         if (iprint.gt.1) print 75,i,panm(i),type(i),cg(i)/facq,
     +        1000*alfpol(i),(x(3*(i-1)+m),m=1,3)
         do j=1,ncag
            if (inc(j).eq.mod(i-1,nrp)+1) write(16,*) '--- '
            if (iprint.gt.1 .and. inc(j).eq.mod(i-1,nrp)+1) 
     +           print*, '--- '
         enddo
      enddo
      if (buck) then
         write(16,79)
         if (iprint.gt.1) print 79
         do i=1,nrp
            iaci=iac(i)
            cbu=c12(iaci,iaci)/fact
            ebu=cs12(iaci,iaci)*facr
            p6=c6(iaci,iaci)/f6
            write(16,78) i,panm(i),type(i),iaci,cbu,p6,ebu
            if (iprint.gt.1) print 78,i,panm(i),type(i),iaci,cbu,p6,ebu
         enddo
      else
         write(16,77)
         if (iprint.gt.1) print 77
         do i=1,nrp
            iaci=iac(i)
            p12=c12(iaci,iaci)/f12
            p6=c6(iaci,iaci)/f6
            eps=p6**2/(4*p12)
            sigma=(p12/p6)**(1/6.0d0)
            rmin=(2*p12/p6)**(1/6.0d0)
            if (abs(p12).gt.1.0e-10 .and. abs(p6).gt.1.0e-10) then
               write(16,76) i,panm(i),type(i),iaci,p12,p6,eps,sigma,rmin
               if (iprint.gt.1) print 76,
     +              i,panm(i),type(i),iaci,p12,p6,eps,sigma,rmin
            else
               write(16,76) i,panm(i),type(i),iaci,p12,p6
               if (iprint.gt.1) print 76,i,panm(i),type(i),iaci,p12,p6
            endif
         enddo
      endif

C     Determine for each dihedral the two rotatable fragments of the molecule
      k=0
      do i=1,ngr
         do j=(i-1)*ndiha+1,i*ndiha
            k=k+1
            print 104,iome(j),jome(j),kome(j),lome(j)
            write(16,104) iome(j),jome(j),kome(j),lome(j)
C           Search for atoms linked to atom JOME(J), not through KOME(J)
            njlist(k)=1
            jlist(k,1)=jome(j)
 1160       continue
            iadd=0
            do m=1,njlist(k)
               do l=1,nbond
                  ibl=ib(l)+(i-1)*nrp
                  jbl=jb(l)+(i-1)*nrp
                  if (ibl.eq.jlist(k,m)) then
                     iconn=jbl
                  elseif (jbl.eq.jlist(k,m)) then
                     iconn=ibl
                  else
                     goto 1170
                  endif
                  if (iconn.eq.kome(j)) goto 1170
                  do n=1,njlist(k)
                     if (iconn.eq.jlist(k,n)) goto 1170
                  enddo
                  iadd=1
                  njlist(k)=njlist(k)+1
                  jlist(k,njlist(k))=iconn
 1170             continue
               enddo
            enddo
c            print*,(jlist(k,n),n=1,njlist(k))
            if (iadd.eq.1) goto 1160
            print 103,jome(j),(jlist(k,n),n=2,njlist(k))
            write(16,103) jome(j),(jlist(k,n),n=2,njlist(k))

C           Search for atoms linked to atom KOME(J), not through JOME(J)
            nklist=1
            klist(1)=kome(j)
 1180       continue
            iadd=0
            do m=1,nklist
               do l=1,nbond
                  ibl=ib(l)+(i-1)*nrp
                  jbl=jb(l)+(i-1)*nrp
                  if (ibl.eq.klist(m)) then
                     iconn=jbl
                  elseif (jbl.eq.klist(m)) then
                     iconn=ibl
                  else
                     goto 1185
                  endif
                  if (iconn.eq.jome(j)) goto 1185
                  do n=1,nklist
                     if (iconn.eq.klist(n)) goto 1185
                  enddo
                  iadd=1
                  nklist=nklist+1
                  klist(nklist)=iconn
 1185             continue
               enddo
            enddo
c            print*,(klist(n),n=1,nklist)
            if (iadd.eq.1) goto 1180
            print 103,kome(j),(klist(n),n=2,nklist)
            write(16,103) kome(j),(klist(n),n=2,nklist)

C           Check that the dihedral is not part of a ring
            do n=1,njlist(k)
               do m=1,nklist
                  if (jlist(k,n).eq.klist(m)) then
                     print 105,iome(j),jome(j),kome(j),lome(j)
                     write(16,105) iome(j),jome(j),kome(j),lome(j)
c                     stop
                  endif
               enddo
            enddo
C           Check that the sum of the entries is correct
C           (A trivial error would occur for more than one molecule)
            if (nmols.eq.1 .and. njlist(k)+nklist.ne.nrp) then
               print*,' ***** Sumcheck error:',njlist(k),nklist,nrp
            endif
         enddo
      enddo

C     Do initial calculation
ccccc      iai=1
ccccc      if (ncysd+ncycg.eq.0) then
ccccc         if (btra1.ne.'IMPORT') then
ccccc            iai=0
ccccc         else
ccccc            if (ihess.eq.0) ihess=1
ccccc         endif
ccccc      endif
c      if (ncysd+ncycg.gt.0 .or. ihess.eq.1 .or.
c     +    (ncysd+ncycg.eq.0 .and. btra1.eq.'IMPORT')) iai=1

      do i=1,nr3
         par(i)=0
      enddo
      mold=0
      if (bter2.eq.btra2) mold=1
      if (idftb.gt.0) then
         noint=.true.
         ichad=0
         call dftb(par,ebond,1,3)
      endif
      if (btra1(1:4).ne.'NONE') then
C        For optimization or hessian ab-initio derivatives will be needed
         if (ncysd+ncycg.gt.0.or.ihess.eq.1.or.dtest.gt.1.0e-11) iai=1
C        Start GAMESS calculation, for derivatives and force constants
         print*,'Initial GAMESS calculation BONDED, basis ',btra1
         write(16,*) 'Initial GAMESS calculation BONDED, basis ',btra1
         call games(par,ebond,ichad,mold,btra1,iprint)
      endif
      if (bter1(1:4).ne.'NONE' .and. mold.eq.0) then
         iai=0
C        Start GAMESS calculation, for charges only
         print*,'Initial GAMESS calculation NONBONDED, basis ',bter1
         write(16,*)'Initial GAMESS calculation NONBONDED, basis ',bter1
         ichad=-abs(ichad)
         do i=1,3*nrpt
            scr(i)=0
         enddo
         call games(scr,eg2,ichad,1,bter1,iprint)
C        New atomic charges have been written in file CHADD.XYZ
      endif

      enb1=0
      eel1=0
      edispa1=0
      edispr1=0
      epol1=0
      eb1=0
      eh1=0
      ei1=0
      ep1=0
      call nonbe(ngr,nrp,enb,2,der,0,box,x,scr,ipr3)
      if (iflex.eq.1) then
         ipr=1
         if (ipr2.gt.0) ipr=ipr2
         call intre(ngr,nrp,box,x,ipr2,enb1,eel1,edispa1,edispr1,epol1)
         call cbond(ngr,nrp,x,par,2,eb1,ipr)
         call choek(ngr,nrp,x,par,2,eh1,ipr)
         call cdiha(ngr,nrp,x,par,2,1,ei1,ipr)
         call cdiha(ngr,nrp,x,par,2,2,ep1,ipr)
         ebond1=eb1+eh1+ei1+ep1
      endif

      if (ipr3.gt.0) then
C        Output forces
         print 91
         do i=1,nrpt
            j=3*i-3
            print 87,i,(par(j+m),m=1,3),(scr(j+m),m=1,3),
     +           (par(j+m)+scr(j+m),m=1,3)
         enddo
      endif

      if (btra1(1:4).eq.'NONE' .and. idftb.eq.0) then
         ebond=ebond1
         eb=eb1
         eh=eh1
         ei=ei1
         ep=ep1
      else
         eb=0
         eh=0
         ei=0
         ep=0
      endif
      epot=enb+ebond
      vol=box(1)*box(2)*box(3)
      pv=vol*druk

      if (noint) then
         enb2=enb
         eel2=eel
         edispa2=edispa
         edispr2=edispr
      else
         enb2=enb-enb1
         eel2=eel-eel1
         edispa2=edispa-edispa1
         edispr2=edispr-edispr1
      endif
      eoud1=epot
      print 65
      print 68,' E-bonds        ',eb1/ngr,eb/ngr
      print 68,' E-angle        ',eh1/ngr,eh/ngr
      print 68,' E-improper     ',ei1/ngr,ei/ngr
      print 68,' E-torsion      ',ep1/ngr,ep/ngr
      print 68,' ETOT-bonded    ',ebond1/ngr,ebond/ngr
      print 66,' E-coulomb      ',eel1/ngr,eel2/ngr,eel/ngr
      print 66,' E-disp-attr    ',edispa1/ngr,edispa2/ngr,edispa/ngr
      print 66,' E-exch-rep     ',edispr1/ngr,edispr2/ngr,edispr/ngr
      print 66,' E-pol          ',epol1/ngr,(epol-epol1)/ngr,
     +     epol/ngr
      print 66,' ETOT-nonbonded ',enb1/ngr,(enb2-pv)/ngr,(enb-pv)/ngr
      if (ispgr.ne.0) print 67,' PV-term        ',pv/ngr,pv/ngr
      print 66,' ETOT           ',(enb1+ebond1)/ngr,enb2/ngr,epot/ngr

      write(16,65)
      write(16,68) ' E-bonds        ',eb1/ngr,eb/ngr
      write(16,68) ' E-angle        ',eh1/ngr,eh/ngr
      write(16,68) ' E-improper     ',ei1/ngr,ei/ngr
      write(16,68) ' E-torsion      ',ep1/ngr,ep/ngr
      write(16,68) ' ETOT-bonded    ',ebond1/ngr,ebond/ngr
      write(16,66) ' E-coulomb      ',eel1/ngr,eel2/ngr,eel/ngr
      write(16,66) ' E-disp-attr    ',edispa1/ngr,edispa2/ngr,edispa/ngr
      write(16,66) ' E-exch-rep     ',edispr1/ngr,edispr2/ngr,edispr/ngr
      write(16,66) ' E-pol          ',epol1/ngr,(epol-epol1)/ngr,
     +     epol/ngr
      write(16,66) ' ETOT-nonbonded ',enb1/ngr,(enb2-pv)/ngr,
     +     (enb-pv)/ngr
      if (ispgr.ne.0) write(16,67) ' PV-term        ',pv/ngr,pv/ngr
      write(16,66)' ETOT           ',(enb1+ebond1)/ngr,enb2/ngr,epot/ngr

      if (doewc) then
         print 57,uinner/ngr,' not included (Ewald)'
         write(16,57) uinner/ngr,' not included (Ewald)'
      else
         print 57,uinner/ngr,' included in E-coulomb'
         write(16,57) uinner/ngr,' included in E-coulomb'
      endif
      if (doew) then
         print 84,(ecc+ekc+edc)/ngr,(ec6+ek6+ed6)/ngr
         print 89,ecc/ngr,ec6/ngr
         print 85,edc/ngr,ed6/ngr
         print 86,ekc/ngr,ek6/ngr
         print 88,ktotc/ngr,ktot6/ngr
         write(16,84) (ecc+ekc+edc)/ngr,(ec6+ek6+ed6)/ngr
         write(16,89) ecc/ngr,ec6/ngr
         write(16,85) edc/ngr,ed6/ngr
         write(16,86) ekc/ngr,ek6/ngr
         write(16,88) ktotc/ngr,ktot6/ngr
      endif
      print 95, npla, nplk
      write(16,95) npla, nplk
      if (ispgr.ne.0) then
         print 70,vol/(ngr*(nspgr+1))
         write(16,70) vol/(ngr*(nspgr+1))
         cvp=-2/(0.061018d0*vol)
         print 58,((vira(m,n)*cvp,n=1,3),m=1,3)
         write(16,58) ((vira(m,n)*cvp,n=1,3),m=1,3)
C        Determine reference geometry parameters
         if (ncysd+ncycg.gt.0)
     +        call geom(1,nrp,ngr,x,amass,box,nmols,iref,ipr2)
      endif

  600 continue
ccccc      ncycga=0
      if (ispgr.eq.0) then
C        Free molecule(s). 
         npar0=nr3
C        Read atom parameters
         do i=1,nr3
            par(i)=x(i)
         enddo
         if (nfix.le.0) goto 620

C        Perform internal rotations using given torsional angles DIHIN
         k=0
         do i=1,ngr
            do j=(i-1)*ndiha+1,i*ndiha
               k=k+1
               omega=dihin(k)/rad-dihan(j,x)
               if (iprint.gt.3) print*,iome(j),jome(j),kome(j),
     +              lome(j),dihan(j,x)*rad,dihin(k),omega*rad
               co=cos(omega)-1
               si=sin(omega)
               j3=3*jome(j)-3
               k3=3*kome(j)-3
               do n=1,njlist(k)
                  i3=3*jlist(k,n)-3
                  call introt(i3,j3,k3,x,arot,brot)
                  do m=1,3
                     x(i3+m)=x(i3+m)+si*brot(m)+co*arot(m)
                  enddo
               enddo
c 1310          continue
            enddo
         enddo
         
C        Create temporarily strong extra impropers to fix DIHIN
         do i=1,nr3
            par(i)=x(i)
         enddo
         do i=1,ndiha
            cq(nimpr+i)=Vimp(i)
         enddo
         nimprs=nimpr
         nimpr=nimpr+ndiha

         if (ncysd.gt.0) then
            call stedes(npar0,par,func0,100000,0.001d0,numm,stc,
     +           stpsd,nresd,shmsd,ipr4,epot,ncyc,rmsder)
            if (iscan.eq.0) print 48,ncyc,npla,epot,rmsder
         endif
         if (ncycg.gt.0) then
            call congra(npar0,par,func0,1000000,0.00001d0,numm,
     +           stpcg,depcg,shmcg,ipr4,epot,ncyc,rmsder)
            if (iscan.eq.0) print 49,ncyc,npla,epot,rmsder
         endif
C        Reset normal improper interaction and do final calculation
         nimpr=nimprs
         goto 640
         
C        Minimize energy for a free molecule
  620    continue
         if (iscan.eq.0) print*,' '
         if (iscan.eq.0) write(16,*) ' '
         if (ncysd+ncycg.gt.0) then
            print*, 'Starting full energy minimization ...'
            write(16,*) 'Starting full energy minimization ...'
         else
            print*, 'No energy minimization'
            write(16,*) 'No energy minimization'
         endif
         if (dtest.gt.1.0e-10) then
            i=1
            call func0(npar0,par,scr,ete,i)
            print*,' Direct test func0 ',ete
            call test(par,npar0,func0,dtest)
         endif
         if (ncysd+ncycg.eq.0) goto 640
         if (ncysd.gt.0) then
            call stedes(npar0,par,func0,ncysd,tolsd,numm,stc,
     +           stpsd,nresd,shmsd,ipr4,epot,ncyc,rmsder)
            print 48,ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
         if (ncycg.gt.0) then
            call congra(npar0,par,func0,ncycg,tolcg,numm,
     +           stpcg,depcg,shmcg,ipr4,epot,ncyc,rmsder)
            print 49,ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
C        End of calculation for a free molecule
         goto 640
      endif

C     Prepare for energy minimization in crystal cell.
      if (icopt.le.0) then
         do m=1,6
            iref(m)=0
            cel(m)=box(m)
         enddo
      endif
      if (icopt.gt.0) then
         print*,' '
         write(16,*) ' '
         if (ncysd+ncycg.gt.0) then
            print*, 'Starting full energy minimization ...'
            write(16,*) 'Starting full energy minimization ...'
         else
            print*, 'No energy minimization'
            write(16,*) 'No energy minimization'
            if (dtest.lt.1.0e-10 .and. nintra+ninter.eq.0) goto 650
         endif
      endif
      if (icopt.eq.0) then
         print*,' '
         write(16,*) ' '
         print*, 'Starting energy minimization with fixed cell'
         write(16,*) 'Starting energy minimization with fixed cell'
      endif
      if (icopt.eq.-1) then
         print*,' '
         write(16,*) ' '
         print 22,igrid,(box(m),m=1,6)
         write(16,22) igrid,(box(m),m=1,6)
      endif

C     Minimization in a crystal
c  625 continue
      if (iflex.eq.0) then
C        Rigid-body energy minimization
         epot=enb
         npar1=0
         do i=1,12
            cel(i)=box(i)
            if (iref(i).gt.0) then
               npar1=npar1+1
               par(npar1)=box(i)
               m=(i+2)/3
               if (m.eq.4) par(npar1)=box(i)/rad
               stc(npar1)=stcin(m)
            endif
         enddo
         do i=13,6*(1+nmols)
            npar1=npar1+1
            par(npar1)=box(i)
            m=(i+2)/3
            m=mod(m+1,2)+3
            if (m.eq.4) par(npar1)=box(i)/rad
            stc(npar1)=stcin(m)
         enddo
         if (dtest.gt.1.0e-10) then
            call func1(npar1,par,scr,ete,1)
            print*,' Direct test func1 ',ete
            call test(par,npar1,func1,dtest)
            if (ncysd+ncycg.le.0) goto 640
         endif
         if (ncysd.gt.0) then
            call stedes(npar1,par,func1,ncysd,tolsd,numm,stc,
     +           stpsd,nresd,shmsd,max(ipr4,1),epot,ncyc,rmsder)
            print 48,ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
            write(16,48) ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
         if (ncycg.gt.0) then
            call congra(npar1,par,func1,ncycg,tolcg,numm,
     +           stpcg,depcg,shmcg,max(ipr4,1),epot,ncyc,rmsder)
            print 49,ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
            write(16,49) ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif

      elseif (iflex.eq.1) then
C        Full geometry relaxation
         npar2=0
         do i=1,6
            if (iref(i).gt.0) then
               npar2=npar2+1
               par(npar2)=box(i)
               m=(i+2)/3
               stc(npar2)=stcin(m)
            endif
         enddo
         do i=1,3
            if (iref(i+6).gt.0) then
               npar2=npar2+1
               par(npar2)=x(i)
            endif
         enddo
         do i=4,nr3
            npar2=npar2+1
            par(npar2)=x(i)
         enddo
         iai=2
         iprem=ipr4
         if (dtest.gt.1.0e-10) then
            i=1
            print*,'Testing derivatives'
            call func3g(npar2,par,scr,ete,i)
            print*,' Direct test func3g ',ete
            call test(par,npar2,func3g,dtest)
         endif
c         iprem=ipr4
c         rms0=0.0010
c  630    continue
         if (ncysd.gt.0) then
            call stedes(npar2,par,func3g,ncysd,tolsd,numm,stc,
     +           stpsd,nresd,shmsd,max(ipr4,1),epot,ncyc,rmsder)
            print 48,ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
            write(16,48) ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
         if (ncycg.gt.0) then
            call congra(npar2,par,func3g,ncycg,tolcg,numm,
     +           stpcg,depcg,shmcg,max(ipr4,1),epot,ncyc,rmsder)
            print 49,ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
            write(16,49) ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
c         if (rmsder.gt.rms0) then
c            rms0=rmsder
c            ncysd=1000
c            tolsd=0.00001
c            stpsd=0.00001
c            ncycg=5000
c            tolcg=0.00001
c            stpcg=0.00001
c            goto 630
c         endif
c         print*,' Call NEWTON-RAPHSON'
c         call newrap(npar2,par,func3,100,tolcg,numm,
c     +        0.000001,ipr4,epot,ncyc,rmsder)
c         if (iprint.gt.1) print 125,ncyc,npla,epot,rmsder,
c     +        (par(m),m=1,nbox)

      elseif (iflex.eq.2) then
C        Rigid body + rotatable hydroxyl groups
         noint=.false.
         epot=enb
         npar2=0
         do i=1,12
            if (iref(i).gt.0) then
               npar2=npar2+1
               par(npar2)=box(i)
               m=(i+2)/3
               stc(npar2)=stcin(m)
            endif
         enddo

         do i=1,ndihh
            npar2=npar2+1
            stc(npar2)=stcin(5)
            par(npar2)=0
C           These PAR elements give the deviation from the starting dihedrals
         enddo
         if (dtest.gt.1.0e-10) then
            call func2(npar2,par,scr,ete,1)
            print*,' Direct test func2 ',ete
            call test(par,npar2,func2,dtest)
         endif
         if (ncysd.gt.0) then
            call stedes(npar2,par,func2,ncysd,tolsd,numm,stc,
     +           stpsd,nresd,shmsd,max(ipr4,1),epot,ncyc,rmsder)
            print 48,ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
            write(16,48) ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
         if (ncycg.gt.0) then
            call congra(npar2,par,func2,ncycg,tolcg,numm,
     +           stpcg,depcg,shmcg,max(ipr4,1),epot,ncyc,rmsder)
            print 49,ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
            write(16,49) ncyc,npla,epot,rmsder,(par(m),m=1,nbox)
         endif
      endif

  640 continue
C     End of energy minimization
      k=0
      do i=1,6
         if (iref(i).gt.0) then
            k=k+1
            box(i)=par(k)
         else
            box(i)=cel(i)
         endif
      enddo
      if (ncycga.gt.0) then
         print*,'Number of ab initio cycles:',ncycga
         write(16,*) 'Number of ab initio cycles:',ncycga
      endif
c      ncycga=ncycga+1

C     Do a final energy calculation
      print*,'    Starting final energy calculation'
      write(16,*) '    Starting final energy calculation'
      noint=(intra.eq.0 .or. iflex.eq.0)
      ebond=0
      iai=0
      if (ihess.eq.1) iai=1
C     For LATVIB final ab-initio derivatives will be needed
      do i=1,nr3
         par(i)=0
      enddo
      mold=0

      if (bter3.eq.btra3) mold=1
      if (btra3(1:4).ne.'NONE') then
C        Start intramolecular GAMESS calculation
         noint=.true.
         print*,'Final GAMESS calculation BONDED, basis ',btra3
         write(16,*) 'Final GAMESS calculation BONDED, basis ',btra3
         ichad=-abs(ichad)
         if (ichad.eq.0) ichad=-1
         call games(par,ebond,ichad,mold,btra3,iprint)
ccccc         if (ihess.eq.0) ihess=1
      endif
      if (bter3(1:4).eq.'NONE') ichad=0
      if (bter3(1:4).ne.'NONE' .and. mold.eq.0) then
C        Start GAMESS calculation, for charges only
         print*,'Final GAMESS calculation NONBONDED, basis ',bter3
         write(16,*) 'Final GAMESS calculation NONBONDED, basis ',bter3
         ichad=-abs(ichad)
         if (ichad.eq.0) ichad=-1
         iai=0
         call games(scr,eg2,ichad,1,bter3,iprint)
C        New atomic charges have been written in file CHADD.XYZ
      endif

      eoud2=epot
      if (iflex.eq.1) then
         if (idftb.gt.0) then
            noint=.true.
            ichad=0
            call dftb(par,ebond,1,3)
         endif
      endif
      call nonbe(ngr,nrp,enb,1,der,0,box,x,scr,0)
      enb1=0
      eel1=0
      edispa1=0
      edispr1=0
      epol1=0
      eb1=0
      eh1=0
      ei1=0
      ep1=0
      if (iflex.eq.1) then
         call intre(ngr,nrp,box,x,0,enb1,eel1,edispa1,edispr1,epol1)
         call cbond(ngr,nrp,x,scr,1,eb1,0)
         call choek(ngr,nrp,x,scr,1,eh1,0)
         call cdiha(ngr,nrp,x,scr,1,1,ei1,0)
         call cdiha(ngr,nrp,x,scr,1,2,ep1,0)
         ebond1=eb1+eh1+ei1+ep1
      endif
      if (btra3(1:4).eq.'NONE' .and. idftb.eq.0) then
         ebond=ebond1
         eb=eb1
         eh=eh1
         ei=ei1
         ep=ep1
      else
         eb=0
         eh=0
         ei=0
         ep=0
      endif
      epot=ebond+enb

      if (ncysd+ncycg.gt.0) then
         print 106,eoud1/ngr,eoud2/ngr,epot/ngr
         write(16,106) eoud1,eoud2,epot
         if (eoud2.gt.eoud1) then
            print*,' ***** Energy increased upon minimization!'
            write(16,*) ' ***** Energy increased upon minimization!'
         endif
         if (btra2.eq.btra3 .and. bter2.eq.bter3) then
            rmd=0
            do i=1,nr3
               rmd=rmd+(scr(i)+par(i))**2
            enddo
            print 71,(epot-eoud2)/ngr,sqrt(rmd/nr3)
            write(16,71) (epot-eoud2)/ngr,sqrt(rmd/nr3)
            if (epot-eoud2.ge.1.0) then
               print*,107
               write(16,107)
               if (epot-eoud2.ge.2.0) then
cc               print*,' RESTART'
cc               write(16,*) ' RESTART'
cc               goto 625
               endif
            endif
         endif
      endif
      if (icopt.lt.0) goto 680
      if (iscan.ne.0) goto 650

  650 continue
      if (iflex.eq.2) then
         if (iprint.gt.1) print*,' Energy of hydroxyl torsions:'
         do i=1,ndihh
            din=dihan(i+ndihz,x)
            if (iprint.gt.1) print 38,
     +           i,rad*din,Vn(i)*(1+cos(ntal(i)*din))
            epot=enb+Vn(i)*(1+cos(din))
         enddo
      endif

C     Transform to reference geometry XR
      do i=1,nmols
         nn=imols(i+1)-imols(i)
         k=0
         do j=imols(i),imols(i+1)-1
            do m=1,3
               k=k+1
               scr(k)=x(3*j-3+m)
            enddo
         enddo
         do j=1,3
            ireft(j)=ireftr(j+3*i-3)-imols(i)+1
         enddo
C        IREFTR refers to the same atom numbers in each independent molecule.
C        This is not meaningful if these molecules are not identical
         call trans(nn,scr,ireft,xcm,xr,phi,theta,psi)
         box(6*i+1)=xcm(1)
         box(6*i+2)=xcm(2)
         box(6*i+3)=xcm(3)
         box(6*i+4)=phi
         box(6*i+5)=theta
         box(6*i+6)=psi
         if (nn.eq.1) then
            do m=4,6
               box(6*i+m)=0
            enddo
         endif
      enddo

C     Make file OUT.TGT, containing optimized parameters
      if (ispgr.ne.0 .and. nstruc.eq.1) then
         open(unit=19,file='out.tgt')
         write(19,41) (box(m),m=1,mbox)
         if (ndiha.gt.0) then
            ndt=0
            do i=1,ngr
               do k=1,ndiha
                  ndt=ndt+1
                  tau(ndt)=rad*dihan((i-1)*ndiha+k,x)
               enddo
            enddo
            write(19,47) (tau(m),m=1,ndt)
         endif
         close(19)
      endif

      if (iflex.eq.0) then
         print*,'Only intermolecular energy:'
         write(16,*) 'Only intermolecular energy:'
      endif
      if (iflex.eq.2) then
         print*,'Only nonbonded energy and hydroxyl torsions:'
         write(16,*) 'Only nonbonded energy and hydroxyl torsions:'
      endif
      vol=box(1)*box(2)*box(3)/(ngr*(nspgr+1))
      if (ispgr.eq.0 .or. spname.eq.'free-ai') then
         vol=0
      else
         dens=0.0016605d0*totm/vol
         print 46,ncyc,geel,epot/ngr,wit,vol,dens,rmsder
         write(16,46) ncyc,' ',epot/ngr,' ',vol,dens,rmsder
      endif         
      if (iscan.eq.0) then
         if (rmsder.gt.999.99) rmsder=999.0
cc         dens=0.0016605d0*totm/vol
cc         print 46,ncyc,geel,epot/ngr,wit,vol,dens,rmsder
cc         write(16,46) ncyc,' ',epot/ngr,' ',vol,dens,rmsder
         print 95, npla, nplk
         write(16,95) npla, nplk
      endif

      if (spname.eq.'free-ai') ispgr=0

      if (ispgr.eq.0) then
         if (iscan.eq.0) print 52,(box(m),m=7,mbox)
         if (iscan.eq.0) write(16,52) (box(m),m=7,mbox)
      else
         print 45
         print 53,(box(m),m=1,mbox)
         write(16,45)
         write(16,53) (box(m),m=1,mbox)
c         write(15,*) titel(1:65)
c         write(15,74) ispgr,epot/ngr
         write(15,59) nstruc,spname,ngr,geel,epot/ngr,wit,vol,rmsder,
     +        titel(1:65)
         write(15,55) (box(m),m=1,mbox)
         ax=box(1)
         by=box(2)
         cz=box(3)
         bx=box(4)
         cx=box(5)
         cy=box(6)
         bb=sqrt(bx**2+by**2)
         cc=sqrt(cx**2+cy**2+cz**2)
         cosa=(bx*cx+by*cy)/(bb*cc)
         cosb=cx/cc
         cosc=bx/bb
         alpha=rad*acos(cosa)
         beta=rad*acos(cosb)
         gamma=rad*acos(cosc)
         do i=1,nmols
            do m=1,3
               xcm(m)=box(6*i+m)
               scr(6*i+3+m)=box(6*i+3+m)
            enddo
            call tri(1,0,xcm,1,cosa,cosb,cosc)
            scr(6*i+1)=xcm(1)/ax
            scr(6*i+2)=xcm(2)/bb
            scr(6*i+3)=xcm(3)/cc
         enddo
         print 54,ax,bb,cc,alpha,beta,gamma,(scr(m),m=7,mbox)
         write(16,54) ax,bb,cc,alpha,beta,gamma,(scr(m),m=7,mbox)
         write(15,55) ax,bb,cc,alpha,beta,gamma,(scr(m),m=7,mbox)
      endif
      if (ndiha.gt.0) then
         do i=1,ngr
            do k=1,ndiha
               tau(k)=rad*dihan((i-1)*ndiha+k,x)
            enddo
            if (iscan.eq.0) then
               print 11,epot/ngr,i,'Final dihedrals:',(tau(k),k=1,ndiha)
               write(16,11) epot/ngr,i,'Final dihedrals:',
     +            (tau(k),k=1,ndiha)
            else
               print 63,geel,epot/ngr,wit,rmsder,(tau(k),k=1,ndiha)
               write(16,63) ' ',epot/ngr,' ',rmsder,(tau(k),k=1,ndiha)
            endif
         enddo
      endif

c  670 continue

c      print*,filenm
c      print*,titel
      if (filenm(1:12).ne.' Constructed') filenm=titel(1:40)
      
C     Output of coordinate files
      if (ispgr.eq.0) then
C        All molecules are referred to IREFTR of the first one
         call trans(nrp,x,ireftr,xcm,xr,phi,theta,psi)
         do m=1,3
            box(m)=0.1d0
            box(m+3)=0
         enddo
         if (ifilm.eq.1) call files(11,nstruc-1,
     +        filenm,epot/ngr,spname,nrpt,panm,xr,box)
         if (ifilm.eq.2) call files(15,nstruc-1,
     +        filenm,epot/ngr,spname,nrpt,panm,xr,box)
         if (ifils.ne.0) call files(ifils+11,nstruc-1,
     +        filenm,epot/ngr,spname,nrpt,panm,xr,box)
         if (ifild.eq.1) call fdat(11,ispgr,nstruc-1,
     +        filenm,epot/ngr,spname,nrpt,ngr,xr,box)
         if (ifild.eq.2) call fdat(14,ispgr,nstruc-1,
     +        filenm,epot/ngr,spname,nrpt,ngr,xr,box)
         if (ifilc.ne.0) call fdat(ifilc+11,ispgr,nstruc-1,
     +        filenm,epot/ngr,spname,nrpt,ngr,xr,box)
      else
         if (ncysd+ncycg.gt.0)
     +        call geom(2,nrp,ngr,x,amass,box,nmols,iref,ipr2)
         if (ifilm.eq.1) call files(11,nstruc,
     +        filenm,epot/ngr,spname,nrpt,panm,x,box)
         if (ifilm.eq.2) call files(15,nstruc,
     +        filenm,epot/ngr,spname,nrpt,panm,x,box)
         if (ifils.ne.0) call files(ifils+11,nstruc,
     +        filenm,epot/ngr,spname,nrpt,panm,x,box)
         if (ifild.eq.1) call fdat(11,ispgr,nstruc,
     +        filenm,epot/ngr,spname,nrpt,ngr,x,box)
         if (ifild.eq.2) call fdat(14,ispgr,nstruc,
     +        filenm,epot/ngr,spname,nrpt,ngr,x,box)
         if (ifilc.ne.0) call fdat(ifilc+11,ispgr,nstruc,
     +        filenm,epot/ngr,spname,nrpt,ngr,x,box)
      endif

C     Output final energies
ccc      call nonbe(ngr,nrp,enb,2,der,0,box,x,scr,ipr3)
      if (noint) then
         enb2=enb
         eel2=eel
         edispa2=edispa
         edispr2=edispr
      else
         enb2=enb-enb1
         eel2=eel-eel1
         edispa2=edispa-edispa1
         edispr2=edispr-edispr1
      endif
      print 165
      print 168,' E-bonds        ',eb1/ngr,eb/ngr
      print 168,' E-angle        ',eh1/ngr,eh/ngr
      print 168,' E-improper     ',ei1/ngr,ei/ngr
      print 168,' E-torsion      ',ep1/ngr,ep/ngr
      print 168,' ETOT-bonded    ',ebond1/ngr,ebond/ngr
      print 166,' E-coulomb      ',eel1/ngr,eel2/ngr,eel/ngr
      print 166,' E-disp-attr    ',edispa1/ngr,edispa2/ngr,
     +     edispa/ngr
      print 166,' E-exch-rep     ',edispr1/ngr,edispr2/ngr,
     +     edispr/ngr
      print 166,' E-pol          ',epol1/ngr,(epol-epol1)/ngr,
     +     epol/ngr
      print 166,' ETOT-nonbonded ',enb1/ngr,(enb2-pv)/ngr,(enb-pv)/ngr
      if (ispgr.ne.0) print 167,' PV-term        ',pv/ngr,pv/ngr
      print 166,' ETOT           ',(enb1+ebond1)/ngr,enb2/ngr,epot/ngr

      write(16,165)
      write(16,168) ' E-bonds        ',eb1/ngr,eb/ngr
      write(16,168) ' E-angle        ',eh1/ngr,eh/ngr
      write(16,168) ' E-improper     ',ei1/ngr,ei/ngr
      write(16,168) ' E-torsion      ',ep1/ngr,ep/ngr
      write(16,168) ' ETOT-bonded    ',ebond1/ngr,ebond/ngr
      write(16,166) ' E-coulomb      ',eel1/ngr,eel2/ngr,eel/ngr
      write(16,166) ' E-disp-attr    ',edispa1/ngr,edispa2/ngr,
     +     edispa/ngr
      write(16,166) ' E-exch-rep     ',edispr1/ngr,edispr2/ngr,
     +     edispr/ngr
      write(16,166) ' E-pol          ',epol1/ngr,(epol-epol1)/ngr,
     +     epol/ngr
      write(16,166) ' ETOT-nonbonded ',enb1/ngr,(enb2-pv)/ngr,
     +     (enb-pv)/ngr
      if (ispgr.ne.0) write(16,167) ' PV-term        ',pv/ngr,pv/ngr
      write(16,166)' ETOT           ',(enb1+ebond1)/ngr,enb2/ngr,
     +     epot/ngr

      if (doewc) then
         print 57,uinner/ngr,' not included (Ewald)'
         write(16,57) uinner/ngr,' not included (Ewald)'
      else
         print 57,uinner/ngr,' included in E-coulomb'
         write(16,57) uinner/ngr,' included in E-coulomb'
      endif
      if (doew) then
         print 84,(ecc+ekc+edc)/ngr,(ec6+ek6+ed6)/ngr
         print 89,ecc/ngr,ec6/ngr
         print 85,edc/ngr,ed6/ngr
         print 86,ekc/ngr,ek6/ngr
         print 88,ktotc/ngr,ktot6/ngr
         write(16,84) (ecc+ekc+edc)/ngr,(ec6+ek6+ed6)/ngr
         write(16,89) ecc/ngr,ec6/ngr
         write(16,85) edc/ngr,ed6/ngr
         write(16,86) ekc/ngr,ek6/ngr
         write(16,88) ktotc/ngr,ktot6/ngr
      endif
      print 95, npla, nplk
      write(16,95) npla, nplk
      if (ispgr.ne.0) then
         print 70,vol
         write(16,70) vol
      endif

      if (ispgr.ne.0) then
C        Calculate lattice energy (approximated by intermolecular energy)
C        These quantities are not really well defined.
ccc         noint=.true.
ccc         call nonbe(ngr,nrp,einter,0,der,1,box,x,scr,0)
ccc         print 83,einter/ngr
ccc         write(16,83) einter/ngr
         einter=enb2-pv
         print 83,einter/ngr
         write(16,83) einter/ngr
      endif
      noint=(intra.eq.0)

C     Calculate lattice vibrations

C     This section was meant to omit the virtual atoms, but that implies also 
C        an inconsistent geometry. Not recommended.
c      if (bter3(1:4).ne.'NONE') then
C        Start GAMESS calculation, for charges on real atoms only
c         print*,'GAMESS calculation for LATVIB, basis ',bter3
c         ichad=-1
c         nvir=0
c         iai=0
c         call games(scr,eg2,ichad,1,bter3,iprint)
C        New atomic charges have been written in file CHADD.XYZ
c         call nonbe(ngr,nrp,edum,0,der,0,box,x,scr,0)
c      endif

      fvib=0
      zpetot=0
      if (nq.ge.0) then
         if (ispgr.eq.0) then
            do m=1,3
               box(m)=100
               box(m+3)=90
            enddo
         endif
         inner=insav
C        For lattice vibrations the choice of INNER must be made
C        independently of the polarity of the space group.
         print 108,nq,cutlv
         write(16,108) nq,cutlv
         if (ipr5.eq.0) ipr5=1
         call latvib(nq,flist,ntemp,vtemp,box,ivamp,ihess,
     +        einter,nmols,ipr5,1,ispgr,
     +        nneg,npos,cvib,evib,sqm,scl,uqm,ucl,zpemol,zpetot,
     +        jjsh,n1sh,n2sh,n3sh,msh,dsh)
         print 17,nneg,npos
         write(16,17) nneg,npos
         estat=epot/ngr
         if (ispgr.eq.0 .or. npos.gt.0) then
            print 14
            write(16,14)
            do i=1,ntemp
               Tvib=vtemp(i)
               fvib=evib(i)-Tvib*sqm(i)
               free(i,0)=estat+zpetot+fvib
               scr1=cvib(i)-1.13d0*36*0.00831451d0**2*Tvib/(einter/ngr)
               if (ispgr.eq.0) scr1=cvib(i)
               print 16,Tvib,cvib(i),scr1,evib(i),sqm(i),
     +              scl(i),fvib,uqm(i),ucl(i)
               write(16,16) Tvib,cvib(i),scr1,evib(i),sqm(i),
     +              scl(i),fvib,uqm(i),ucl(i)
            enddo
            if (ispgr.gt.0) then
               print 18,nmols*(nspgr+1),zpemol
               write(16,18) nmols*(nspgr+1),zpemol
            endif
            fvib=evib(1)-vtemp(1)*sqm(1)
            print 15, estat,zpetot,vtemp(1),fvib,fvib+zpetot,
     +           estat+fvib+zpetot,nneg
            write(16,15) estat,zpetot,vtemp(1),fvib,fvib+zpetot,
     +           estat+fvib+zpetot,nneg
         endif
      endif
C     End of calculation for a single structure
      if (icopt.eq.0 .or. icopt.eq.1) goto 900

      if (icopt.gt.1) then
C        Prepare for grid search to determine the influence of cell parameters
         if (iout29.gt.0) then
            write(29,*) titel(1:65)
            write(29,*) ntemp,'   ',(irefs(m),m=1,6)
            write(29,19) (box(m),m=1,6),nneg,epot/ngr,zpetot
            do i=1,ntemp
               Tvib=vtemp(i)
               fvib=evib(i)-Tvib*sqm(i)
               scr1=cvib(i)-1.13d0*36*0.00831451d0**2*Tvib/(einter/ngr)
               write(29,16) Tvib,cvib(i),scr1,evib(i),sqm(i),
     +              scl(i),fvib,fvib+zpetot
            enddo
         endif
         icopt=-1
         nimtot=nneg
         do m=1,6
            refbox(m)=box(m)
         enddo
C        For optimized cell: increase cell axes (not angles)
C            to obtain center point of grid search.
         do m=1,3
            if (irefs(m).gt.0) box(m)=box(m)+db
         enddo

C        Check for fixed box input. If found, center around given values
C           and use only the first temperature.
         n=inpt('BOXFIX',regel)
         if (n.ge.7) then
            do m=1,6
               call realst(regel,m+1,box(m))
            enddo
            ntemp=1
            icycb=0
            goto 800
         endif
         print 12,db,(box(m),m=1,6)
         write(16,12) db,(box(m),m=1,6)

C        Prepare list of grid points
         ngrid=1
         do m=1,6
            do i=1,maxlis
               boxlist(i,m)=box(m)
            enddo
         enddo
         do i=1,6
            if (irefs(i).ne.0) then
               boxlist(ngrid+1,i)=box(i)-db
               boxlist(ngrid+2,i)=box(i)+db
               ngrid=ngrid+2
            endif
         enddo
         print 82
         write(16,82)
         do i=1,ngrid
            print 40,i,(boxlist(i,m),m=1,6)
            write(16,40) i,(boxlist(i,m),m=1,6)
         enddo
         igrid=1
         do m=1,6
            box(m)=boxlist(igrid,m)
         enddo
         goto 600
      endif

  680 continue
C     Calculate lattice vibrations for a grid point
      zpet1=0
      inner=insav
      call latvib(nq,flist,ntemp,vtemp,box,ivamp,0,
     +     einter,nmols,iprint,0,ispgr,
     +     nneg,npos,cvib1,evib1,sqm1,scl,uqm,ucl,zpemol,zpet1,
     +     0,0,0,0,0,0.0d0)
      print 72,epot-enb1,nneg
      write(16,72) epot-enb1,nneg
      do i=1,ntemp
         Tvib=vtemp(i)
         fvib=evib1(i)-Tvib*sqm1(i)
         if (itemp.eq.0) then
            free(i,igrid)=epot/ngr+zpet1+fvib
            print 73, vtemp(i),fvib,fvib+zpet1,
     +           epot/ngr+fvib+zpet1
            write(16,73) vtemp(i),fvib,fvib+zpet1,
     +           epot/ngr+fvib+zpet1
            scr1=cvib1(i)-1.13d0*36*0.00831451d0**2*Tvib/(einter/ngr)
            if (iout29.gt.0) write(29,16) Tvib,cvib1(i),scr1,evib1(i),
     +           sqm1(i),scl(i),fvib,fvib+zpet1
         endif
         if (itemp.eq.i) then
            y1=evib(i)-Tvib*sqm(i)
            y2=epot/ngr+zpet1+fvib
            print 23
            print 27,' Statically  optimized',cvib(i),sqm(i),evib(i),
     +           y1,estat,zpetot,free(i,0)
            print 27,' Free energy optimized',cvib1(i),sqm1(i),evib1(i),
     +           fvib,epot/ngr,zpet1,y2
            print 27,' Difference           ',cvib1(i)-cvib(i),
     +           sqm1(i)-sqm(i),evib1(i)-evib(i),fvib-y1,
     +           epot/ngr-estat,zpet1-zpetot,y2-free(i,0)
            write(16,23)
            write(16,27) ' Statically  optimized',cvib(i),sqm(i),
     +           evib(i),y1,estat,zpetot,free(i,0)
            write(16,27) ' Free energy optimized',cvib1(i),sqm1(i),
     +           evib1(i),fvib,epot/ngr,zpet1,y2
            write(16,27) ' Difference           ',cvib1(i)-cvib(i),
     +           sqm1(i)-sqm(i),evib1(i)-evib(i),fvib-y1,
     +           epot/ngr-estat,zpet1-zpetot,y2-free(i,0)
         endif
      enddo
      nimtot=nimtot+nneg
      if (nas.gt.6) goto 800
      if (icopt.eq.-3) goto 820

C     Select the next grid point from the list
      igrid=igrid+1
      if (igrid.gt.ngrid) goto 730
      do m=1,6
         box(m)=boxlist(igrid,m)
      enddo
      goto 600

  730 continue
C     Grid points done, estimate optimum box values
      itemp=itemp+1
      icopt=-2
      if (itemp.gt.ntemp) then
         do m=1,6
            box(m)=scr(m)
         enddo
         goto 800
      endif
      print 26,vtemp(itemp),icycb
      write(16,26) vtemp(itemp),icycb
      nas=0
      do m=1,6
         if (irefs(m).ne.0) then
            nas=nas+1
            y1=free(itemp,2*nas)
            y2=free(itemp,1)
            y3=free(itemp,2*nas+1)
            dx=db*(y3-y1)/(4*y2-2*y3-2*y1)
C           This is the shift with respect to the center of the grid
            box(m)=boxlist(1,m)+dx
            print 69,m,y1,y2,y3,dx,box(m)
            write(16,69) nas,y1,y2,y3,dx,box(m)
         endif
         if (itemp.eq.1) scr(m)=box(m)
      enddo
      nas=0
      print 24,(box(m)-refbox(m),m=1,6)
      write(16,24) (box(m)-refbox(m),m=1,6)
      print 25,(box(m),m=1,6)
      write(16,25) (box(m),m=1,6)
      goto 600

  800 continue
      if (icycb.ge.ncycb) goto 900
C     Start another cycle of thermal expansion, only for temperature #1.
c      print*,icycb,ncycb
      icycb=icycb+1
      ntemp=1
      itemp=0
      icopt=-3
      nas=1
      igrid=1
      db=dbox(icycb)
      print 13,vtemp(1),icycb,db,(box(m),m=1,6)
      write(16,13) vtemp(1),icycb,db,(box(m),m=1,6)
      box(1)=box(1)-db
      goto 600

  820 continue
C     Select the next grid point
      igrid=igrid+1
      box(nas)=box(nas)+db
      if (igrid.le.3) goto 600

C     Find minimum along the cell axis NAS
      y1=free(1,1)
      y2=free(1,2)
      y3=free(1,3)
      dx=db*(y3-y1)/(4*y2-2*y3-2*y1)
C     This is the shift with respect to the center of the grid
      box(nas)=box(nas)-2*db+dx
      print 69,nas,y1,y2,y3,dx,box(nas)
      write(16,69) nas,y1,y2,y3,dx,box(nas)

C     Start the next axis
      igrid=1
  840 continue
      nas=nas+1
      if (nas.eq.7) then
         itemp=1
C        All grid point calculations are finished, do final calculation
         print*,' '
         print*,'Recalculation at optimized geometry for cycle',icycb
         write(16,*) ' '
         write(16,*) 'Recalculation at optimized geometry for cycle',
     +        icycb
         print 24,(box(m)-refbox(m),m=1,6)
         write(16,24) (box(m)-refbox(m),m=1,6)
         print 25,(box(m),m=1,6)
         write(16,25) (box(m),m=1,6)
         icopt=-2
         goto 600
      endif
      if (irefs(nas).eq.0) goto 840
      box(nas)=box(nas)-db
      goto 600

  900 continue
C     Continue a dihedral scan in a free molecule
      if (abs(dscan).gt.0) then
         nstruc=nstruc+1
         dihin(iscan)=dihin(iscan)+dscan
         if (dihin(iscan).lt.escan+0.001d0) then
            q0(nimpr+iscan)=dihin(iscan)/rad
            goto 600
         endif
      endif

C     End of calculations for this structure
      if (icopt.lt.0) then
         print 81,nimtot
         write(16,81) nimtot
      endif

C     Try the next structure
      print 99,eoud2/ngr,epot/ngr,rmsder
      write(16,99) eoud2/ngr,epot/ngr,rmsder

C     Look for knotted structures
      if (ispgr.gt.0 .and. doknot.gt.0) then
         open(unit=49,file='rings.def',status='OLD',iostat=ios)
         if (ios.ne.0) then
            print*,' ***** No file RINGS.DEF present'
            print*,' Trying to create one:' 
            write(16,*) ' ***** No file RINGS.DEF present'
            write(16,*) ' Trying to create one...' 
            call rings(nrp,nbond,ib,jb,iprint)
            print*,' Check the prepared file rings.def carefully!'
            print*,' First line should contain the number of rings'
            print*,' Then for every ring a line with:'
            print*,' number of atoms (NATR), atom numbers 1....NATR, 1'
            print*,' '
            print*,' If correct, then rerun the program'
            stop
         endif
c         print*,(panm(i),i=1,10)
c         pause
         ipr=iprint
         if (iprint.lt.3) ipr=3 
         call catena(nrp,ngr,nspgr,box,x,panm,ipr,iknot)
         close(49)
         if (iknot.gt.0) then
            print 42,filenm(1:20)
            write(16,42) filenm(1:20)
         endif
      endif

      goto 130

   11 format(f14.3,'    Group',i4,6x,a,30f10.3)
   12 format(/,' Starting grid search with fixed grid of',f8.5,
     +     /,' Centering around',6f9.5)
   13 format(/,' Starting optimizing grid search at temperature',f6.1,
     +     '   Cycle',i4,' with steps of',f8.5,
     +     /,' Centering around',6f9.5)
   14 format(7x,'T',8x,'Cv        Cp           E     S(qm)     ',
     +     'S(cl)           F      u(qm)      u(cl)')
   15 format(' Basis energy',f10.3,'   Zero-point',f9.3,
     +     '   Free (Temp ',f6.1,')',f9.3,'  Corr',f9.3,
     +     '   Total',f10.3,'   Imag',i5)
   16 format(f8.2,2f10.5,f12.3,2f10.5,f12.3,2f11.4)
   17 format(' LATVIB found',i5,' imaginary and',
     +     i6,' regular lattice vibrational modes')
   18 format(' Lattice zero-point energy based on',i4,
     +     ' molecules per cell:',f8.3)
   19 format(3f9.5,2x,3f9.5,i5,2x,2f10.4)
   20 format(' Restrain dihedral',i3,' with Vimp',f10.1,
     +     '  at value',10f9.1)
   21 format(i4,3x,a5,f10.4,10x,f10.4)
   22 format(' Starting grid point',i4,5x,'Box',6f9.5)
   23 format(25x,'C-vib   S-vib   E-vib   F-vib      E-stat',
     +     '       ZPE     F-total')
   24 format(' Shifts with respect to static minimum',6f9.5)
   25 format(' BOXFIX',6f9.5)
   26 format(/,'Optimization at T =',f6.1,' for cycle',i5)
   27 format(a22,2f8.5,2f8.3,2x,2f10.3,f12.3)
   28 format(5i8,f8.2)
   29 format(' Molecule',i4,' from atoms',i4,' to',i4)
   30 format('    ax, by, cz',3f10.5,8x,'bx, cx, cy',3f10.5)
   31 format(' No file IN.COR present, construct coordinates ',
     +     'for a free molecule')
   32 format(f8.3,4f8.2,2f10.5)
   33 format(i8,2f8.4,i8,f8.4,f12.7)
   34 format(12i8)
   35 format(' Expand to',i4,' groups and calculate in P1, ', 
     +     'with maximum disturbance',f10.5)
   36 format('  stcin: ',12f10.2)
   37 format(f18.1)
   38 format(i5,3f10.3)
   39 format(i8,3f8.4,4i8)
   40 format(i8,6f8.4)
   41 format(2(3f8.4,2x),16(:/,3f8.4,2x,3f8.1))
   42 format('mv    ',a,'   knotsdump')
   43 format(' Dihedrals ',15f8.1)
   44 format(3i8,f11.5)
   45 format(12x,'ax      by      cz        bx      cx      cy')
   46 format(i7,' cycles     Energy',a,f12.3,a,' Vol =',f8.4,
     +     '  Density =',f8.4,'  R = ',f9.4)
   47 format(20f8.1)
   48 format(' End of STEDES:',i4,i8,f14.3,f12.4,3x,12f7.3)
   49 format(' End of CONGRA:',i4,i8,f14.3,f12.4,3x,12f7.3)
   50 format(4i5,8x,4a8,3x,20f10.3)
   51 format(' Molecular mass:',f10.4)
   52 format(' Center of gravity, Euler angles    ',3f8.4,2x,3f8.1,
     +     16(/,36x,3f8.4,2x,3f8.1))
   53 format(' Box  ',3f8.5,2x,3f8.5,2x,3f8.4,2x,3f8.1,
     +     16(/,58x,3f8.4,2x,3f8.1))
   54 format(' Cell ',3f8.5,2x,3f8.2,2x,3f8.4,2x,3f8.1,
     +     16(/58x,3f8.4,2x,3f8.1))
   55 format(3f9.5,1x,3f9.2,4(1x,3f9.5,1x,3f9.2))
   56 format(' Hydroxyl dihedral',i5,f10.5,
     +     '    potential (Vn,n):',f10.5,i5)
   57 format(' E-inner',f12.3,a)
   58 format(' Pressure (in atmosphere, for asymmetric unit only!) ',
     +     3(/,3f15.1))
   59 format(i5,2x,a8,'Ngr:',i3,a,f9.3,a,' Vol:',f8.4,'  R:',f10.4,
     +     2x,a65)
   60 format(' Space group ',a8,i6,' found on coordinate file IN.COR')
   61 format(i5,' atoms found in',i5,' groups in file IN.COR: ',a80)
   62 format('    Group',i4,6x,a,30f10.3)
   63 format(' Energy',a,f12.3,a,'    R = ',f10.4,
     +     '   Dihedrals ',15f8.1)
   64 format(' This supersedes previous space group information: ',a8)
   65 format(' Energies:       Intramolecular ',
     +     'Intermolecular          Total')
   66 format(a16,3f15.3)
   67 format(a16,15x,2f15.3)
   68 format(a16,f15.3,15x,f15.3)
   69 format(' Axis',i4,'  Values',3f10.4,8x,'Shift',f9.5,
     +     '    Optimum value',f9.5)
   70 format(' Volume for one molecular group',f9.5)
   71 format(' Energy change upon recalculation: ',f15.4,8x,
     +     'RMS force on atoms ',f10.4)
   72 format(' Energy change upon recalculation: ',f8.4,9x,'Imag',i5)
   73 format(' T =',f6.1,'   F-vib',f9.3,'   Corr',f9.3,'   Total',f9.3)
   74 format(i5,3f10.5)
   75 format(i4,4x,a5,2x,a5,2f8.4,2x,3f10.5)
   76 format(i4,4x,a5,2x,a5,i4,f14.1,f10.3,f9.5,3f9.4)
   77 format(5x,'Atom',12x,'Code',7x,
     +     '   C12        C6  epsilon    sigma     rmin  (kcal, A)')
   78 format(i4,4x,a5,2x,a5,i4,f14.3,f12.4,f10.3)
   79 format(5x,'Atom',12x,'Code',6x,
     +     'A(rep)         C6    Exponent  (kcal, A)')
   80 format(5x,'Atom',13x,'Charge   Alpha      Nanometer coordinates',
     +     11x,'Charge groups separated by ---')
   81 format(/,' Total number of imaginary frequencies: ',i5)
   82 format(' List of grid points')
   83 format(' Intermolecular energy        ',2f11.3)
   84 format(' Convergence acceleration:        Coulomb Dispersion',
     +     /,' Total Ewald-correction       ',2f11.3)
   85 format(' Direct contributions         ',2f11.3)
   86 format(' Reciprocal contributions     ',2f11.3)
   87 format(i4,4x,3(3f10.2,4x))
   88 format(' Number of reciprocal vectors ',2i11)
   89 format(' Constant contributions       ',2f11.3)
   90 format(' Charges other than from topology are required.',/,
     +     ' Charges from topology   vs.   coordinates')
   91 format(' Forces:',14x,'Bonded',25x,'Nonbonded',29x,'Total')
   92 format(' Maximum number of GAMESS iterations',i5)
   93 format(' Maximum allowed shifts for bonds (A), angles, and'
     +     ,' torsions (degrees): ',f7.2,2f6.1)
   94 format('  iref:',12i5)
   95 format(' Numbers of direct/reciprocal terms in the lists:',i8,i6)
   96 format(' Ab-initio calculation of conformational energy, ',
     +     'offset',f10.3)
   97 format(' Lattice vibration calculations with',i6,' vectors')
   98 format('    Temperatures ',30f6.1)
   99 format('E-min',f12.3,'    E-fin',f12.3,'    RMSDER ',f10.4)
  101 format(' Unequal basis sets for initial and subsequent ',
     +     'calculations: ',2a20)
  102 format(a7,'bases for initial, minimization, and final ',
     +     'calculations: ',3a20)
  103 format('    Connected to atom',i4,' are atoms',50i4)
  104 format(' Searching connections for dihedral',4i4)
  105 format(' ***** But this dihedral cannot be set',
     +     ' since it is part of a ring:',4i4)
  106 format(' Etot begin, after EM, after recalc:',3f12.3)
  107 format(' ***** Excessive energy increase upon recalculation')
  108 format('Starting LATVIB with',i5,' vectors, cutoff',f6.3)
c  125 format(' NR:',i4,i8,f14.3,f12.4,3x,12f7.3)
  165 format(' Final energies:       Intramolecular ',
     +     'Intermolecular          Total')
  166 format(' Final',a16,3f15.3)
  167 format(' Final',a16,15x,2f15.3)
  168 format(' Final',a16,f15.3,15x,f15.3)
      end
      
