      subroutine runmd(ngr,nrp,nbond,box,x,v,panm,
     +     temp,potav,totav,boxav,xav,iprint,nx,ny,nz,nsporg,
     +     tempi,temp0,tautp,pres0,comp,taup,t,dt,tol,
     +     ig,nrun,ntt,ntp,nscm,nstlim,ntc,nsnb,
     +     ntpr,ntwx,ntwv,ntwe,ntpw)

C     Performs molecular dynamics run.
C     Adapted from the GROMOS procedure RUNMD, for crystal simulation only.
C     Thanks are due to W. F. van Gunsteren for his permission to do so.

C     Only atomic virial, no restraining of positions, distances
C     or dihedrals, no solvent, no free energy calculations,
C     no switch or twin-range in energy.

C     Length units nm, time units ps, energy units kJ/mol,
C     pressure units 16.389 atm.

C     Input through parameters:
C        NGR: number of groups of molecules
C        NBOND: number of bonds in that molecule
C        BOX(1...6): orthogonal cell parameters AX, BY, CZ, BX, CX, CY.
C        X, V(1...3*NRP): coordinates and velocities
C        IPRINT rules output to unit 16 and to the screen:
C        IPRINT = 0,1: no output
C                 2: average T, ETOT, EPOT, BOX and average coordinates
C                 3: every NTPR steps short intermediate data
C                 4: also initial data; full average data to file
C                 5: also STOPCM data and full intermediate data to file
C        MSOM(m) = sum over MSP(m). If MSOM(m) = 0 the coordinates in the
C           direction M can be freely chosen.
C        TEMPI: if TEMPI > 10.e-6 take the initial velocities from
C           a Maxwellain distribution with temperature TEMPI
C        NTT=0/1: disable/ enable temperature scaling
C        TEMP0: reference temperature for the MD simulation
C        TAUTP: relaxation time for the temperature scaling
C        NTP=0: no pressure scaling
C           =1: isotropic pressure scaling
C           =2: orthorhombic pressure scaling
C           =3: monoclinic pressure scaling
C           =4: triclinic pressure scaling
C        PRES0: reference pressure
C        COMP: compressibility
C        TAUP: pressure relaxation time (a few picoseconds at least!)
C        T: starting time
C        DT: time step
C        NTC=1: disable the SHAKE procedure
C        TOL: tolerance for SHAKE
C        IG: random number seed
C        NRUN: number of complete runs to be done
C        NSTLIM: number of MD steps in each run
C        NSCM: stop translation of center of mass every NSCM steps.
C        NSTLIM: number of MD steps in each run
C        NSNB: Make a new pair list (by calling NONBE) every NSNB steps.
C           Between these calls LIST is used with constant pair list.
C        NTPR: print output every NTPR steps
C        NTWE: if NTWE > 0 write box, pressure, virial, energies and so on
C            to unit 15 every NTWE steps
C        NTWX: same for coordinates to unit 17.
C        NTWV: same for velocities to unit 18.
C        NTPW=0/2: write unformatted/formatted coordinates and velocities.

C     Input through common blocks:
C     /VIRA/:
C        VIRA(1..3,1..3): contribution to virial
C     /SPGR/ gives space group information:
C        NSPGR, MSP: see subroutine LEESR
C     /NONB/:
C        NRP: number of atoms in one molecule
C        PMOL(1..3): the cell dipole moment
C        UINNER: the calculated U2-inner (if INNER unequal zero)
C     /WALD/:
C        DOEW: if TRUE, then Ewald calculation has been done
C        KTOTC, KTOT6: number of reciprocal lattice vectors
C        ECC, EC6: Ewald contribution from constant terms (from KWALD)
C        EKC, EK6: Ewald contribution from reciprocal terms (from KWALD)
C        EDC, ED6: Ewald contribution from direct terms (this subroutine)
C     /ATTP/:
C        AMASS(1...NRP): atomic masses

C     Output through parameters: averages for the last run.
C        TEMP: average temperature
C        POTAV: average potential energy
C        TOTAV: avarage total energy
C        BOXAV: average box lengths
C        XAV: average coordinates

C     Calls subroutines SHAKE, STOPCM, NONBE, LIST, CBOND, CHOEK, CDIHA, TRI
C     and function RANDOM.

C     If file "stopmd" (optionally containing NNEW) is created during the run,
C     the number of cycles NSTLIM is changed to NNEW. If NNEW is missing
C     in STOPMD, the program will ask for it.

      implicit none
      integer nren
      parameter(nren=46)
      include 'params.i'
      include 'cspgr.i'
      include 'cwald.i'
      include 'cnonb.i'
      include 'cvira.i'

      integer i,i3,ig,iprint,ix,iy,iz,j,j3,k,k3,m,m1,n,nbond,ndf,
     +     nexpd,ngr,ngr0,nitp,nop3,nr3,nrp,nrpind,nrpt,nru,nrun,
     +     nscm,nskip,nsnb,nsporg,nstep,nstlim,nstt,ntc,ntp,ntpr,ntpw,
     +     ntt,ntwe,ntwv,ntwx,nul,nused,nx,ny,nz
      double precision am,anh,ast,astx,asty,astz,avkin,ax,b1,bb,boltz,
     +     boltz2,box,boxav,bst,bsty,bstz,bx,by,cc,comp,cosa,cosb,cosc,
     +     cst,cx,cy,cz,der,dt,dtcp,dttp,e0,e1,e2,eangl,ebond,eimpr,
     +     ekcm,ekin,ekin0,ekina,enb,ener,enew,eold,epot,etors,etot,
     +     f,fac,pi,potav,pres,pres0,presa,rad,random,rmu,rmua,scalp,
     +     scr,sd,step,t,taup,tautp,temp,temp0,tempi,tmas,tol,totav,
     +     tuij,u0,v,vcm,vol,winv,x,xav
      dimension box(*),x(*),v(*),xav(*),boxav(*),
     +     f(maxxco),winv(maxat),tuij(maxat,6),rmu(3),vcm(3),scr(6),
     +     presa(3,3),ekina(3,3),rmua(3,3),ENER(nren),nul(3,3)
c     +     ,xref(maxxco)
      double precision enert(nren),enert2(nren),
     +     xsom(maxxco),uij(maxat,6),x1,x2,x3
      double precision sekin,sepot,setot,sanh,tekin,tepot,tetot,tanh

      character*8 panm(*)

      nskip=ig
      ig=1042000

      if (nstlim.le.0) return
      dt2=2*dt*dt
      pi=4*atan(1.0d0)
      rad=180/pi
      nrpt=nrp*ngr
      nr3=3*nrpt
      nexpd=nx*ny*nz
      ngr0=ngr/(nexpd*(nsporg+1))
C     NGR0 is the number of independent molecules
      nrpind=ngr0*nrp
C     NRPIND is the number of independent atoms

      if (iprint.gt.1) then
         print*, nrpind, 'independent atoms in the cell'
         do i=1,nrpind
            print 72, 1,panm(i)(1:5),i,(x(3*i-3+m),m=1,3)
         enddo
         print*,'Expanding cell axes: ',nx,ny,nz
      endif

      do j=1,6
         boxav(j)=0
         do i=1,maxat
            tuij(i,j)=0
         enddo
      enddo
      do i=1,maxxco
         xav(i)=0
      enddo
      tekin=0
      tepot=0
      tetot=0
      tanh=0
      sekin=0
      sepot=0
      setot=0
      sanh=0
      do i=1,nrp
         do k=1,ngr
            j=(k-1)*nrp+i
            amass(j)=amass(i)
            winv(j)=1/amass(j)
         enddo
      enddo
      tmas=0
      do i=1,nrpt
         tmas=tmas+amass(i)
      enddo
      if (iprint.gt.1) then
         print*,' Total mass for',nrpt,' atoms: ',tmas
         write(16,*) ' Total mass for',nrpt,' atoms: ',tmas
      endif
      do m=1,3
         do n=1,3
            nul(m,n)=1
            do nop=1,nspgr
               nop3=3*nop-3
               nul(m,n)=nul(m,n)+msp(nop3+m)*msp(nop3+n)
            enddo
         enddo
      enddo
C     The virial is only calculated for the asymmetric unit and must be
C     multiplied with the number of equivalent molecules OR by zero,
C     as required by the space group symmetry. This factor is in NUL.

      ndf=nr3-3
      if (ntc.ne.1) ndf=ndf-nbond*ngr
      if (iprint.gt.1) then
         print*, ' Degrees of freedom', ndf
         write(16,*) ' Degrees of freedom', ndf
      endif
      BOLTZ=8.31441d-3
      FAC=BOLTZ*NDF/2
      EKIN0=FAC*TEMP0
      IF (NTT.GT.0) DTTP=DT/abs(TAUTP)
      IF (NTP.GT.0) DTCP=COMP*DT/(3*TAUP)
      NITP=0

C     TAKE THE VELOCITIES FROM A MAXWELLIAN, WHEN REQUIRED
      IF (TEMPI.gt.1.d-6) then
         if (iprint.gt.1) then
            print 177
            write(16,177)
         endif
         BOLTZ2=BOLTZ*TEMPI
         I=0
         do J=1,NRPT
            SD= SQRT(BOLTZ2/amass(j))
            do M=1,3
               I=I+1
               v(i)=sd*sqrt(-2*log(random(ig))) * cos(2*pi*random(ig))
C              Take v(i) from normal distribution with sigma=sd
            enddo
         enddo
      endif

C     Do a first dynamics step for initialization
      do I3=1,NR3
         F(I3)=X(I3)
      enddo
      CALL SHAKE (NRPT,WINV,1,0,tol,f,x,ntc,nitp)
C     Now X is the shaken coordinate at T=0
      if (iprint.gt.3) write(16,175) NITP
      IF (NITP.EQ.0) RETURN
      do I3=1,NR3
         F(I3)=X(I3)-DT*V(I3)
      enddo
      CALL SHAKE (NRPT,WINV,1,0,tol,x,f,ntc,nitp)
C     Now F is the shaken coordinate at T=-DT
      if (iprint.gt.3) write(16,176) NITP
      IF (NITP.EQ.0) RETURN
      do I3=1,NR3
         V(I3)=(X(I3)-F(I3))/DT
      enddo
C     Now V is the shaken velocity at T=-DT/2

C     Stops the center of mass movement, not the overall rotation.
      call stopcm(nrpt,tmas,amass,v,msom,vcm,ekcm)
      if (iprint.gt.3) then
         print 336, 0,(vcm(m),m=1,3),ekcm
         write(16,336) 0,(vcm(m),m=1,3),ekcm
      endif

C     Calculate kinetic energy, print the initial temperature
      eold=0
      I3=0
      do J=1,NRPT
         am=amass(j)
         do M=1,3
            I3=I3+1
            eold=eold+am*V(I3)**2
         enddo
      enddo
      eold=eold/2
      IF (eold.LT.1.d-4) eold=EKIN0
      if (iprint.gt.1) then
         print 310,(box(m),m=1,6)
         print 558, eold/fac
         write(16,310) (box(m),m=1,6)
         write(16,558) eold/fac
      endif

      nru=0
C     Initialize the next run
  300 continue
      nru=nru+1
      if (iprint.gt.1) then
         print 340, nru
         write(16,340) nru
      endif
      NSTEP=0
      do I=1,NREN
         ener(i)=0
         enert(i)=0
         enert2(i)=0
      enddo
      do i=1,maxxco
         xsom(i)=0
      enddo
      do j=1,6
         do i=1,maxat
            uij(i,j)=0
         enddo
      enddo

C     Next MD-step
  400 continue

C     Calculation of forces
      if (nsnb.le.1) then
         call nonbe(ngr,nrp,enb,2,scr,0,box,x,f,0)
      else
         if (mod(nstep,nsnb).eq.0) then
            if (nstep.ne.0 .and. iprint.gt.3)
     +           print*,'         new pair list made after step',nstep
c            call nonbe(ngr,nrp,enb,2,scr,1,box,x,f,iprint)
            call nonbe(ngr,nrp,enb,2,scr,1,box,x,f,0)
         else
            call list(ngr,nrp,enb,2,scr,box,x,f)
         endif
      endif
      call cbond(ngr,nrp,x,f,2,ebond,0)
      call choek(ngr,nrp,x,f,2,eangl,0)
      call cdiha(ngr,nrp,x,f,2,1,eimpr,0)
      call cdiha(ngr,nrp,x,f,2,2,etors,0)
      epot=ebond+eangl+eimpr+etors+enb

C     Test derivatives (IF -NR3 IS REPLACED BY NR3)
c   60 format('        parameter       der.low    der.high    der.aver')
   70 format (i6,f10.5,3x,8f12.4)
c      print 60
      step=0.000001d0
      e0=epot
      do i=1,-nr3
         x(i)=x(i)+step
         call nonbe(ngr,nrp,enb,2,scr,1,box,x,f,0)
         call cbond(ngr,nrp,x,f,2,ebond,0)
         call choek(ngr,nrp,x,f,2,eangl,0)
         call cdiha(ngr,nrp,x,f,2,1,eimpr,0)
         call cdiha(ngr,nrp,x,f,2,2,etors,0)
         e1=ebond+eangl+eimpr+etors+enb
         x(i)=x(i)-2*step
         call nonbe(ngr,nrp,enb,2,scr,1,box,x,f,0)
         call cbond(ngr,nrp,x,f,2,ebond,0)
         call choek(ngr,nrp,x,f,2,eangl,0)
         call cdiha(ngr,nrp,x,f,2,1,eimpr,0)
         call cdiha(ngr,nrp,x,f,2,2,etors,0)
         e2=ebond+eangl+eimpr+etors+enb
         x(i)=x(i)+step
         der=(e1-e2)/(2*step)
         print 70,i,x(i),(e1-e0)/step,(e0-e2)/step,
     +        der,der+f(i)
      enddo

      scalp=1
      if (ntt.gt.0) SCALP=SQRT(1+DTTP*(EKIN0/eold-1))
      I3=0
      do J=1,NRPT
         am=dt/amass(j)
         do M=1,3
            I3=I3+1
            v(I3)=(V(I3)+F(I3)*am)*SCALP
         enddo
      enddo
C     Now V is the new velocity, scaled but not yet shaken
      do I3=1,NR3
         F(I3)=X(I3)
         X(I3)=X(I3)+v(i3)*DT
C        Now F is the old position coordinate and X is the new one
      enddo

      enew=0
      i3=0
      IF (NTC.eq.1) then
C        no SHAKE
         do J=1,NRPT
            am=amass(j)
            do M=1,3
               I3=I3+1
               enew=enew+am*v(I3)**2
            enddo
         enddo
      else
         CALL SHAKE (NRPT,WINV,1,0,tol,f,x,ntc,nitp)
         IF (NITP.EQ.0) then
C           EMERGENCY STOP
            do I3=1,NR3
               X(I3)=F(I3)
            enddo
            nru=nrun
            goto 900
         endif
         do J=1,NRPT
            am=amass(j)
            do M=1,3
               I3=I3+1
               V(I3)=(X(I3)-F(I3))/DT
C              Now V is the new velocity, scaled and shaken
               enew=enew+am*V(I3)**2
            enddo
         enddo
      endif
      enew=enew/2

C     Calculate the pressure and perform pressure scaling if required.
      vol=box(1)*box(2)*box(3)
      do m=1,3
         do m1=1,3
            i3=m
            j3=m1
            ekina(m,m1)=0
            do j=1,nrpt
               ekina(m,m1)=ekina(m,m1)+v(i3)*v(j3)*amass(j)
               i3=i3+3
               j3=j3+3
            enddo
         enddo
      enddo
      do M=1,3
         do K=1,3
            presa(M,K)=(ekina(m,k)-2*vira(M,K))*nul(m,k)/vol
            rmua(m,k)=0
         enddo
      enddo
      pres=(presa(1,1)+presa(2,2)+presa(3,3))/3

      if (ntp.eq.0) goto 720
      if (ntp.gt.1) goto 620
C     isotropic (ntp=1)
      rmu(1)=1-DTCP*(PRES0-PRES)
      rmu(2)=rmu(1)
      rmu(3)=rmu(1)
      GOTO 640
  620 continue
C     orthorhombic (ntp=2)
      do M=1,3
         rmu(M)=1-DTCP*(PRES0-presa(m,m))
         rmua(m,m)=rmu(m)
      enddo
      if (ntp.gt.2) goto 680
  640 continue
      do M=1,3
         BOX(M)=BOX(M)*rmu(M)
      enddo
      I3=0
      do J=1,Nrpt
         do M=1,3
            X(I3+M)=X(I3+M)*rmu(M)
         enddo
         I3=I3+3
      enddo
      GOTO 720

C     monoclinic (ntp=3) and triclinic (ntp=4)
  680 continue
      rmua(1,3)=DTCP*(presa(1,3)+presa(3,1))
      if (ntp.eq.4) then
         rmua(2,3)=DTCP*(presa(2,3)+presa(3,2))
         rmua(1,2)=DTCP*(presa(1,2)+presa(2,1))
      endif
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      box(1)=rmua(1,1)*ax
      box(4)=rmua(1,1)*bx + rmua(1,2)*by
      box(2)=rmua(2,2)*by
      box(5)=rmua(1,1)*cx + rmua(1,2)*cy + rmua(1,3)*cz
      box(6)=rmua(2,2)*cy + rmua(2,3)*cz
      box(3)=rmua(3,3)*cz

      i3=0
      do j=1,nrpt
         do m=1,3
            b1=0
            do k=1,3
               b1=b1+rmua(m,k)*x(i3+k)
            enddo
            x(i3+m)=b1
         enddo
         i3=i3+3
      enddo

  720 continue
      ekin=(enew+eold)/2
      etot=ekin+epot
      eold=enew

C     Analyze results for this MD step.
      ener(1)=etot
      ener(2)=ekin
      ener(3)=epot
      do i=1,6
         ener(i+3)=box(i)
      enddo
      ener(10)=box(1)*box(2)*box(3)
      ener(11)=ebond
      ener(12)=eangl
      ener(13)=eimpr
      ener(14)=etors
      ener(15)=enb
      do i=1,3
         do j=1,3
            ener(17+3*i+j)=presa(i,j)
            ener(27+3*i+j)=vira(i,j)
         enddo
      enddo
      ener(30)=pres
      ener(40)=(vira(1,1)+vira(2,2)+vira(3,3))/3
C     The division by 3.0 is absent in the GROMOS procedure RUNMD.
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      ener(41)=ax
      ener(42)=bb
      ener(43)=cc
      ener(44)=rad*acos(cosa)
      ener(45)=rad*acos(cosb)
      ener(46)=rad*acos(cosc)
      NSTEP=NSTEP+1
      T=T+DT

C     Accumulate results for this run
      do M=1,NREN
         ENERT(M)=ENERT(M)+ENER(M)
         ENERT2(M)=ENERT2(M)+ENER(M)**2
      enddo

C     Calculate average fractional coordinates.
      do i=1,nr3
         f(i)=x(i)
      enddo
      call tri(nrpt,0,f,1,cosa,cosb,cosc)
C     F now contains the triclinic coordinates (not fractional!)

      k=0
      do ix=0,nx-1
         do iy=0,ny-1
            do iz=0,nz-1
               i=0
               do nop=0,nsporg
                  do j=1,nrpind
                     nop3=3*nop-3
                     i=i+1
                     i3=3*i-3
                     k=k+1
                     k3=3*k-3
C            Is this necessary?
Ccc                 Reset within half a box length to the values XREF found
Ccc                 for the initial coordinates. (Do not reset the actual 
Ccc                 coordinates, that would require a new pair list)
cc                  if (nstep.eq.1 .and. nru.eq.1) then
cc                     x1=f(k3+1)/ax
cc                     x2=f(k3+2)/bb
cc                     x3=f(k3+3)/cc
cc                     xref(j+1)=x1
cc                     xref(j+2)=x2
cc                     xref(j+3)=x3
cc                  else
cc                     x1=x1-nint(2.0*(x1-xref(j+1)))/2.0
cc                     x2=x2-nint(2.0*(x2-xref(j+2)))/2.0
cc                     x3=x3-nint(2.0*(x3-xref(j+3)))/2.0
cc                  endif
c                     xav(i3+1)=x1-(float(ix)/nx)
c                     xav(i3+2)=x2-(float(iy)/ny)
c                     xav(i3+3)=x3-(float(iz)/nz)

C                    Apply translational shifts over the expanded cell axes 
C                    reduced to the original ones.
                     x1=f(k3+1)/ax-(dfloat(ix)/nx)
                     x2=f(k3+2)/bb-(dfloat(iy)/ny)
                     x3=f(k3+3)/cc-(dfloat(iz)/nz)
                     if (nop.gt.0) then
C                       Apply space group symmetry in the original cell
                        x1=msp(nop3+1)*(x1-tsp(nop3+1)/nx)
                        x2=msp(nop3+2)*(x2-tsp(nop3+2)/ny)
                        x3=msp(nop3+3)*(x3-tsp(nop3+3)/nz)
                     endif
c  777                format('Atom',2i5,3x,3(3f9.5))
c                     print 777,i,k,x1,x2,x3
C                    Accumulate results (still fractional!)
C                    Sequence as in the GROMOS analysis program PROAVX
                     j3=3*j-3
                     xsom(j3+1)=xsom(j3+1)+x1
                     xsom(j3+2)=xsom(j3+2)+x2
                     xsom(j3+3)=xsom(j3+3)+x3
                     uij(j,1)=uij(j,1)+x1**2
                     uij(j,2)=uij(j,2)+x1*x2
                     uij(j,3)=uij(j,3)+x2**2
                     uij(j,4)=uij(j,4)+x1*x3
                     uij(j,5)=uij(j,5)+x2*x3
                     uij(j,6)=uij(j,6)+x3**2
                  enddo
               enddo
            enddo
         enddo
      enddo

C     If file "stopmd" is present, give a new number of cycles
      open(11,file='stopmd',status='OLD',err=870)
      print*,' Old number of cycles:',nstlim,'     done:',nstep
      read(11,*,err=850) nstlim
      goto 860
  850 continue
      print*,' Give new number of cycles: '
      read*, nstlim
  860 continue
      close(11,status='DELETE')
      print*,' New number of cycles:',nstlim,'   at step',nstep
      write(16,*) ' New number of cycles:',nstlim,'   at step',nstep
  870 continue

C     OUTPUT PER ... STEPS TO FILES 15, 17, 18.

C     Output to files for coordinates, velocities and energies
      if (ntwx.gt.0 .and. mod(nstep,ntwx).eq.0) then
         if (ntpw.eq.0) write(17) (x(i),i=1,nr3)
         if (ntpw.eq.2) write(17,304) (x(i),i=1,nr3)
         if (ntpw.eq.0) write(17) (ener(k),k=41,46)
         if (ntpw.eq.2) write(17,303) (ener(k),k=41,46)
      endif
      if (ntwv.gt.0 .and. mod(nstep,ntwv).eq.0) then
         if (ntpw.eq.0) write(18) (v(i),i=1,nr3)
         if (ntpw.eq.2) write(18,304) (v(i),i=1,nr3)
      endif
      if (ntwe.gt.0 .and. mod(nstep,ntwe).eq.0) then
         write(15,318) (ENER(K),K=1,NREN)
      endif

C     Output for first step
      if (nstep.eq.1 .and. nru.eq.1) u0=ener(3)/ngr
      if (nstep.eq.1 .and. iprint.gt.1) then
         print 308,ener(3)/ngr
         write(16,308) ener(3)/ngr
         if (inner.ne.0) then
            print 482,(pmol(m),m=1,3)
            print 483,uinner/ngr
            write(16,482) (pmol(m),m=1,3)
            write(16,483) uinner/ngr
         endif
         if (doew) then
            print 488,ktotc,ktot6
            print 484,(ecc+ekc+edc)/ngr,(ec6+ek6+ed6)/ngr
            print 489,ecc/ngr,ec6/ngr
            print 485,edc/ngr,ed6/ngr
            print 486,ekc/ngr,ek6/ngr
            write(16,488) ktotc,ktot6
            write(16,484) (ecc+ekc+edc)/ngr,(ec6+ek6+ed6)/ngr
            write(16,489) ecc/ngr,ec6/ngr
            write(16,485) edc/ngr,ed6/ngr
            write(16,486) ekc/ngr,ek6/ngr
         endif
      endif

C     Print output to screen and file every NTPR steps
      if (ntpr.gt.0 .and. mod(nstep,ntpr).eq.0
     +     .or. nstep.eq.1 .or. nstep.eq.nstlim) then
         if (iprint.gt.1) then
            if (nstep.eq.1) then
               print 307
               write(16,307)
            endif
            print 309,nstep,(ener(m),m=4,9),enert(2)/(fac*nstep),
     +           enert(1)/(ngr*nstep),enert(3)/(ngr*nstep),
     +           ener(2)/ngr,ener(1)/ngr,ener(3)/ngr
            write(16,309) nstep,(ener(m),m=4,9),enert(2)/(fac*nstep),
     +           enert(1)/(ngr*nstep),enert(3)/(ngr*nstep),
     +           ener(2)/ngr,ener(1)/ngr,ener(3)/ngr
         endif
         if (iprint.gt.3) then
            if (nstep.eq.1) write(16,311)
            write(16,312) NSTEP,NITP,T,(ENER(K),K=1,9)
            write(16,313) (ENER(K),K=10,20)
            write(16,314) (ENER(K),K=21,30)
            write(16,315) (ENER(K),K=31,40)
            write(16,316) (ener(k),k=41,46)
         endif
      endif

C     CALCULATE AND STOP CENTER OF MASS MOTION, ONCE IN NSCM STEPS
      if (nscm.ne.0) then
         if (mod(nstep,nscm).eq.0) then
            call stopcm(nrpt,tmas,amass,v,msom,vcm,ekcm)
            if (iprint.gt.3) then
c               print 336, nstep,(vcm(m),m=1,3),ekcm
               write(16,336) nstep,(vcm(m),m=1,3),ekcm
            endif
         endif
      endif
      IF (NSTEP.LT.NSTLIM) GOTO 400

C     End of this run. Calculate average energies.
  900 continue
      do M=1,NREN
         ENERT(M)=ENERT(M)/nstep
         ENERT2(M)= SQRT( ABS(ENERT2(M)/nstep-ENERT(M)**2))
      enddo
      temp=enert(2)/fac
      totav=enert(1)
      avkin=enert(2)
      potav=enert(3)
      anh=(totav-2*avkin)/ngr-u0
      ax=enert(4)
      by=enert(5)
      cz=enert(6)
      bx=enert(7)
      cx=enert(8)
      cy=enert(9)
      vol=ax*by*cz
      astx=1/ax
      asty=-bx*cz/vol
      astz=(bx*cy-by*cx)/vol
      ast=sqrt(astx**2+asty**2+astz**2)
      bsty=1/by
      bstz=-cy*ax/vol
      bst=sqrt(bsty**2+bstz**2)
      cst=1/cz
      do m=1,6
         scr(m)=0
      enddo

C     Calculate average fractional coordinates and U-values
      nstt=nstep*nexpd*(nsporg+1)
      do i=1,3*nrpind
         xsom(i)=xsom(i)/nstt
      enddo
      do i=1,nrpind
         x1=xsom(3*i-3+1)
         x2=xsom(3*i-3+2)
         x3=xsom(3*i-3+3)
         uij(i,1)=(uij(i,1)/nstt-x1**2)/ast**2
         uij(i,2)=(uij(i,2)/nstt-x1*x2)/(ast*bst)
         uij(i,3)=(uij(i,3)/nstt-x2**2)/bst**2
         uij(i,4)=(uij(i,4)/nstt-x1*x3)/(ast*cst)
         uij(i,5)=(uij(i,5)/nstt-x2*x3)/(bst*cst)
         uij(i,6)=(uij(i,6)/nstt-x3**2)/cst**2
         do m=1,6
            uij(i,m)=uij(i,m)*100
            scr(m)=scr(m)+uij(i,m)
         enddo
      enddo

      if (nru.gt.nskip) then
C        Collect results after skipping the first NSKIP runs.
         tetot=tetot+totav/ngr
         tekin=tekin+avkin/ngr
         tepot=tepot+potav/ngr
         tanh=tanh+anh
         setot=setot+(totav/ngr)**2
         sekin=sekin+(avkin/ngr)**2
         sepot=sepot+(potav/ngr)**2
         sanh=sanh+anh**2
         do m=1,6
            boxav(m)=boxav(m)+enert(m+3)
            do i=1,nrpind
               tuij(i,m)=tuij(i,m)+uij(i,m)
            enddo
         enddo
         do i=1,3*nrpind
            xav(i)=xav(i)+xsom(i)
         enddo
      endif

C     Output for this run
      if (iprint.gt.0) then
         if (nru.le.nskip) then
            print 330, nru
            write(16,330) nru
         else
            print 331, nru
            write(16,331) nru
         endif
         print 554,temp,enert2(2)/fac,avkin/ngr,enert2(2)/ngr,
     +        totav/ngr,enert2(1)/ngr,potav/ngr,enert2(3)/ngr
         print 555, (enert(m),m=4,9)
         print 556, anh
         write(16,554) temp,enert2(2)/fac,avkin/ngr,enert2(2)/ngr,
     +        totav/ngr,enert2(1)/ngr,potav/ngr,enert2(3)/ngr
         write(16,555) (enert(m),m=4,9)
         write(16,556) anh
         print 322, (scr(m)/nrpind,m=1,6)
         write(16,322) (scr(m)/nrpind,m=1,6)
      endif
      if (iprint.gt.3) then
         print 320
         print 321
         do i=1,nrpind
            print 319, i,panm(i)(1:5),
     +           (xsom(3*i-3+m),m=1,3),(uij(i,m),m=1,6)
         enddo
      endif
      if (iprint.gt.2) then
         write(16,320)
         write(16,321)
         do i=1,nrpind
            write(16,319) i,panm(i)(1:5),
     +           (xsom(3*i-3+m),m=1,3),(uij(i,m),m=1,6)
         enddo
      endif
      if (iprint.gt.2) then
         write(16,551)
         write(16,311)
         write(16,312) NSTEP,0,T,(ENERT(K),K=1,9)
         write(16,313) (ENERT(K),K=10,20)
         write(16,314) (ENERT(K),K=21,30)
         write(16,315) (ENERT(K),K=31,40)
         write(16,316) (enert(k),k=41,46)
         write(16,552)
         write(16,312) NSTEP,0,T,(ENERT2(K),K=1,9)
         write(16,313) (ENERT2(K),K=10,20)
         write(16,314) (ENERT2(K),K=21,30)
         write(16,315) (ENERT2(K),K=31,40)
         write(16,316) (enert2(k),k=41,46)
      endif
      if (nru.lt.nrun) goto 300

C     Calculate averages over the runs after the first NSKIP.
      nused=nrun-nskip
      if (nused.lt.2) then
         print 317, nused
         write(16,317) nused
         if (nused.eq.0) stop
      endif

C     Calculate orthogonal box parameters for output
      do i=1,6
         boxav(i)=boxav(i)/nused
      enddo
      if (nused.gt.1) then
C        Calculate energies and standard deviations
         sekin=sqrt((sekin-tekin**2/nused)/(nused*(nused-1)))
         setot=sqrt((setot-tetot**2/nused)/(nused*(nused-1)))
         sepot=sqrt((sepot-tepot**2/nused)/(nused*(nused-1)))
         sanh=sqrt((sanh-tanh**2/nused)/(nused*(nused-1)))
         print 301, nused
         print 305, (boxav(m),m=1,6)
         print 306, tekin*ngr/(fac*nused),
     +        tekin/nused,tetot/nused,tepot/nused,tanh/nused
         print 302, sekin*ngr/fac,sekin,setot,sepot,sanh
         print 557, u0
         write(16,301) nused
         write(16,305) (boxav(m),m=1,6)
         write(16,306) tekin*ngr/(fac*nused),
     +        tekin/nused,tetot/nused,tepot/nused,tanh/nused
         write(16,302) sekin*ngr/fac,sekin,setot,sepot,sanh
         write(16,557) u0
      endif
      if (iprint.gt.3) then
         print 320
         print 321
         do i=1,nrpind
            print 319, i,panm(i)(1:5),
     +           (xav(3*i-3+m)/nused,m=1,3),(tuij(i,m)/nused,m=1,6)
         enddo
      endif
      if (iprint.gt.1) then
         write(16,320)
         write(16,321)
         do i=1,nrpind
            write(16,319) i,panm(i)(1:5),
     +           (xav(3*i-3+m)/nused,m=1,3),(tuij(i,m)/nused,m=1,6)
         enddo
      endif

C     Calculate final atomic Cartesian coordinates (nm) for output
      ax=boxav(1)
      by=boxav(2)
      cz=boxav(3)
      bx=boxav(4)
      cx=boxav(5)
      cy=boxav(6)
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      do i=1,nrpind
         j=3*i-3
         xav(j+1)=xav(j+1)*ax/nused
         xav(j+2)=xav(j+2)*bb/nused
         xav(j+3)=xav(j+3)*cc/nused
      enddo
      call tri(nrpt,0,xav,-1,cosa,cosb,cosc)
      if (iprint.gt.3) then
         print*,' Average Cartesian coordinates:'
         write(16,*) ' Average Cartesian coordinates:'
         do i=1,nrpind
            print 319, i,panm(i)(1:5),(xav(3*i-3+m),m=1,3)
            write(16,319) i,panm(i)(1:5),(xav(3*i-3+m),m=1,3)
         enddo
      endif
      return

   72 format(i5,5x,a5,i5,6f8.4)
  175 format(27H SHAKE COORDINATES, NITER =,I4)
  176 format(27H SHAKE VELOCITIES,  NITER =,I4)
  177 format('  Velocities were taken from a Maxwellian distribution')
  301 format(/,'  Final results over the last',i4,' runs:')
  302 format('  StDev Temp',f8.1,'  Ekin',f10.3,
     +     '  Etot',f10.3,'  Epot',f10.3,'  Eanh',f10.3)
  303 format(3f8.5,3f8.3)
  304 format(10f8.4)
  305 format('  Final Box',6f10.4)
  306 format('  Final Temp',f8.1,'  Ekin',f10.3,
     +     '  Etot',f10.3,'  Epot',f10.3,'  Eanh',f10.3)
  307 format(10x,'Box',39x,'T-cum   Etot-cum   Epot-cum',
     +     '       Ekin       Etot       Epot')
  308 format(' First molecular dynamics step, ', 
     +     'potential energy = ',3f10.3)
  309 format(i8,6f7.3,f7.1,6f11.2)
  310 format('  Starting Box',6f10.4)
  311 format(/,4x,'STEP NITP      TIME     E-TOT     E-KIN     E-POT',
     +     '        AX      BY      CZ      BX      CX      CY',
     +     /,10x,'VOLUME    E-BOND    E-ANGL    E-IMPR    E-TORS',
     +     '    E-NONB',
     +     /,5x,'PRESSURE   XX        XY        XZ           YX       ',
     +     ' YY        YZ           ZX        ZY        ZZ',
     +     10x,'TOT',/,7x,'VIRIAL',
     +     /,15x,'A         B         C     ALPHA      BETA     GAMMA')
  312 format(/,i8,i5,f10.4,3f10.3,2x,6f8.3)
  313 format(6X,f10.5,10f10.3)
  314 format(4X,3(3x,3f10.2),f13.2)
  315 format(6X,3(3x,3f10.4),f13.4)
  316 format(6X,3f10.4,3f10.2)
  317 format(/,'  Insufficient usable runs for block averages:',i5)
  318 format(5E16.8)
  319 format(i6,2x,a5,2x,3f9.4,4x,6f9.4)
  320 format(' Fractional coordinates and U-values averaged over ',
     +     'all symmetry-equivalent molecules:')
  321 format(8x,'Fractional   Xav      Yav      Zav',10x,'U11      U12',
     +     '      U22      U13      U23      U33  (Angstrom**2)')
  322 format('  Average U-values ',6f9.4)
  330 format(/,' Results for run ',i4,6x,'These data will be skipped')
  331 format(/,' Results for run ',i4,6x,
     +     'These data are used for the averages')
  340 format(/,' Starting the next run: ',i4)
  336 format('  Step',i8,'  Center-of-mass movement stopped, ',
     +     'velocities were',3f9.5,5x,'kinetic energy was',f10.5)
  482 format(' Cell dipole moment (* 4.0745 for Debye):  ',3f8.3)
  483 format(' U2-inner: ',3f10.3)
  484 format(' Total Ewald corrections   ',2f13.3)
  485 format(' Direct contributions      ',2f13.3)
  486 format(' Reciprocal contributions  ',2f13.3)
  488 format(' Number of recip. vectors  ',2i13)
  489 format(' Constant contributions    ',2f13.3)
  551 format(/,' AVERAGES ARE:')
  552 format(/,' ROOT MEAN SQUARE FLUCTUATIONS:')
  554 format('  Average and rms temperature: ',2f8.1, '    Ekin',f10.3,
     +     f8.3,'    Etot',f10.3,f8.3,'    Epot',f10.3,f8.3)
  555 format('  Average box  ',6f10.4)
  556 format('  Average anharmonic energy',f10.4)
  557 format('  NOTE: Correct the anharmonic energy if the value',f10.4,
     +     ' for U(0) does not correspond to the energy minimum!')
  558 format('  Initial temperature:',f8.3)
      end





      subroutine stopcm(nrp,tmas,amass,v,msom,vcm,ekcm)
C     Stops the center of mass-movement, considering crystal symmetry
C     Input parameters: NRP, TMAS, AMASS, V, MSOM
C     Output parameters:
C     VCM is velocity of the center of mass before it was stopped
C     EKCM is the kinetic energy before CM movement was stopped

      implicit none
      dimension v(*),amass(*),msom(*),vcm(*)
      integer i,j,m,msom,nrp
      double precision am,amass,ekcm,tmas,v,vcm
      ekcm=0
      DO M=1,3
         vcm(m)=0
      enddo
      I=0
      DO J=1,NRP
         am=amass(j)
         DO M=1,3
            I=I+1
            if (msom(m).ne.0) VCM(M)=VCM(M)+am*V(I)
         enddo
      enddo
      DO M=1,3
         VCM(M)=VCM(M)/TMAS
         EKCM=EKCM+VCM(M)**2
      enddo
      EKCM=EKCM*TMAS/2

      I=0
      DO J=1,NRp
         DO M=1,3
            I=I+1
            V(I)=V(I)-VCM(M)
         enddo
      enddo
      end
