      subroutine sort(n,arr,ind)
C     Sorts array ARR of N reals, in ascending order. The array is unchanged,
C     the place of the sorted element I is found in array IND(I).
C     To increase the performance a set of subgroups is ordered first.
C     A better routine (INDEXX) can be found in Numerical Recipes p. 233,
C     but is not used here for copyright reasons.

      implicit none
      include 'params.i'
      integer i,ihulp,ind,ipl,j,jmin,k,kmin,knp1,m,maxgrp,mnp2,n,np1,
     +     np2,ns,ns1,ns2,nt1,nt2,ntt
      double precision abig,amin,arr,x

      parameter(maxgrp=1000)
      dimension arr(*),ind(*),ipl(maxgrp),ihulp(mxdata)

C     The values NP1, NP2 optimize the performance, and may differ according
C        to the computer used; NP2 should be divisible by NP1.
C        Orders of magnitude: NP1 = N**(1/3)/5, NP2 = NP1**2/5.
      np1=25
      np2=250
      if (n.gt.10000) then
         np1=50
         np2=1000
      endif
      if (n.gt.30000) then
         np1=125
         np2=5000
      endif
      if (n.gt.100000) then
         np1=200
         np2=10000
      endif
      if (n.gt.300000) then
         np1=500
         np2=50000
      endif
      ns=np2/np1
      ns1=(n+np1-1)/np1
      ns2=(n+np2-1)/np2
      nt1=ns1*np1
      nt2=ns2*np2
      ntt=nt2
      if (nt1.gt.ntt) ntt=nt1
c      print*,' Elements in subgroups:',np1
c      print*,' Number of subgroups:  ',ns1,nt1
c      print*,' Elements in clustered subgroups:',np2
c      print*,' Number of clustered subgroups:  ',ns2,nt2
c      print*,' Cluster size', ns
      if (np1*ns.ne.np2) then
         print*,' SORT: wrong constants:', np1, np2
         stop
      endif
      if (ns1.gt.maxgrp) then
         print*,' SORT: increase maxgrp', ns1
         stop
      endif
      if (ntt.gt.mxdata) then
         print*,' SORT: increase mxdata',ntt
         stop
      endif

C     Sort all subgroups using straight insertion
      do i=1,ntt
         ind(i)=i
      enddo
      abig=1.0d+30
      do i=n+1,ntt
         arr(i)=abig
      enddo
      abig=2*abig
      do k=1,ns1
         knp1=(k-1)*np1
         do i=knp1+2,k*np1
            x=arr(ind(i))
            do j=i-1,knp1+1,-1
               if (arr(ind(j)).le.x) goto 140
               ind(j+1)=ind(j)
            enddo
            j=knp1
 140        continue
            ind(j+1)=i
         enddo
C        Commented lines are bubblesort, slightly slower and strictly
C        forbidden by the authors of Numerical Recipes.
c         do 150 i=knp1+1,k*np1
c            amin=abig
c            do 140 j=i,k*np1
c               if (arr(ind(j)).lt.amin) then
c                  amin=arr(ind(j))
c                  jmin=j
c               endif
c 140        continue
c            j=ind(i)
c            ind(i)=ind(jmin)
c            ind(jmin)=j
c 150     continue
      enddo
c      print*,' Subgroups sorted'

C     Sort NS2 subgroups
C     Only the upper ones of the entries left for each group are considered.
      do m=1,ns2
c         do 210 k=1,ns
c           print 20, (arr(ind((m-1)*np2+(k-1)*np1+i)),i=1,np1)
c 210     continue
         mnp2=(m-1)*np2
         do k=1,ns
            ipl(k)=1
         enddo
         do i=1,np2
            amin=abig
            do k=1,ns
               if (ipl(k).le.np1) then
                  j=ind(mnp2+(k-1)*np1+ipl(k))
                  if (arr(j).lt.amin) then
                     amin=arr(j)
                     kmin=k
                     jmin=j
                  endif
               endif
            enddo
            ihulp(mnp2+i)=jmin
            ipl(kmin)=ipl(kmin)+1
         enddo
      enddo
c      print*,' First sorting cycle finished'

C     Sort the entire array, same principle
      do k=1,ns2
         ipl(k)=1
      enddo
c      do 315 k=1,ns2
c        print 20, (arr(ihulp((k-1)*np2+i)),i=1,np2)
c 315  continue
      do i=1,n
         amin=abig
         do k=1,ns2
            if (ipl(k).le.np2) then
               j=ihulp((k-1)*np2+ipl(k))
               if (arr(j).lt.amin) then
                  amin=arr(j)
                  kmin=k
                  jmin=j
               endif
            endif
         enddo
         ind(i)=jmin
         ipl(kmin)=ipl(kmin)+1
      enddo
c 20   format(10f7.3)
      end
