      subroutine spepos(nfound,ngr1,box,naam,spname,
     +     cg1,cgcor,ispgr,iprint)
C     Checks for special positions
C     Finds connectivity from coordinates      
      
C     Input through parameters:
C        NFOUND: number of atoms on coordinate file
C        BOX(1..6): a, b, c (nm), alfa, beta, gamma
C        NAAM(1..): atom names from coordinates
C        CG1(1..): atom charges from CSSR file
C        SPNAME: space group name.
C        IPRINT regulates the quantity of output.

C     Input from common block /CPARS/:
C        NRP: number of atoms in the topology file
C        X(1..): coordinates (nm, non-rectangular)

C     Input from common block /CSPGR/:
C        NOP: number of operators
C        NSPGR: NOP-1

C     Output through parameters:
C        NGR1: the number of topology groups
C        NAAM(..): the reshuffeled atomic names
C        CGCOR(1..): the reshuffeled atom charges
C        ISPGR > 0: space group number
C              < 0: space group not found in file ALL.SPG
C        SPNAME: space group name. Special possibilities:
C               'unknown' if only symmetry operators are found
C               'r-spname' if symmetry has been artificially reduced
C                          for special positions or high symmetry

C     Output to common block /CPARS/:
C        IREF: see YGETSYM
C        X(1..): reshuffeled coordinates (nm, non-rectangular)
C        XR(1..): scratch array for relative coordinates

      implicit none
      include 'params.i'
      include 'cspgr.i'
      include 'cnonb.i'
      include 'cpars.i'

      integer i,ielop,igr,imem,index,iop,iop3,iprint,iscr,ispgr,
     +     j,k,k0,k1,k2,k3,m,m1,m2,m3,maxop,ms,
     +     n1,n2,nconn,nfound,ngr1,ngr2,nind,nmol,nrest,nspgr2
      double precision a11,a12,a22,a23,a31,a33,box,cg1,cgcor,
     +     cosa,cosb,cosc,d,d1,d2,d3,dx,dy,dz,r,rad,rij,scr,ts

      parameter(maxop=64)
      dimension box(*),cg1(*),cgcor(*),scr(3),iscr(6),
     +     ielop(0:mxspgr),at1(maxat),at2(maxat),index(maxat),
     +     igr(0:maxat),table(0:maxop,maxop),imem(0:mxspgr)
      integer at1,at2,table

      character*8 spname,naam(*),naam1(0:maxat),tekst*12

      print 37,ispgr,spname,nop
      write(16,37) ispgr,spname,nop

C     Expand fractional coordinates following space group symmetry
      do i=1,nfound
         do m=1,3
            xr(3*i-3+m)=x(3*i-3+m)/box(m)
         enddo
c         write(20,*) 'ATOM  ',naam(i),(xr(3*i-3+m),m=1,3)
c         print*,i,(xr(3*i-3+m),m=1,3)
      enddo
      do iop=2,nop
         do m=1,3
            print 38,iop,m,tspg(iop,m),(mspg(iop,m,m1),m1=1,3)
         enddo
         do i=1,nfound
            naam((iop-1)*nfound+i)=naam(i)
            j=3*i-3
            k=j+(iop-1)*3*nfound
            do m1=1,3
               scr(m1)=xr(j+m1)
            enddo
            do m1=1,3
               xr(k+m1)=tspg(iop,m1)
               do m2=1,3
                  xr(k+m1)=xr(k+m1)+mspg(iop,m1,m2)*scr(m2)
               enddo
            enddo
            if (iprint.gt.3) print*,iop,i,k,'  ',naam(i),(xr(k+m),m=1,3)
         enddo
      enddo

C     Eliminate equivalent atoms
      do i=1,nfound*nop
         do j=i+1,nfound*nop
            if (naam(j).ne.'xxxxx') then
               do m=1,3
                  d=abs(xr(3*i-3+m)-xr(3*j-3+m))
                  if (abs(nint(d)-d).gt.0.01) goto 150
               enddo
               naam(j)='xxxxx'
               if (iprint.gt.2) then 
                  print*,' Coinciding atoms:',i,j
                  print*,(xr(3*i-3+m),m=1,3)
                  print*,(xr(3*j-3+m),m=1,3)
               endif
  150          continue
            endif
         enddo
      enddo

C     Collect remaining atoms
      nrest=0
      do i=1,nfound*nop
         if (naam(i).ne.'xxxxx') then
            nrest=nrest+1
            naam(nrest)=naam(i)
            do m=1,3
               xr(3*nrest-3+m)=xr(3*i-3+m)
            enddo
         endif
      enddo

      print 41,nop,nfound,nrest
      write(16,41) nop,nfound,nrest

C     Determine connectivity from coordinates
      if (iprint.gt.0) print*,' Determine connectivity from coordinates'
      a11=box(1)**2
      a22=box(2)**2
      a33=box(3)**2
      rad=45/atan(1.0d0)
      cosa=cos(box(4)/rad)
      cosb=cos(box(5)/rad)
      cosc=cos(box(6)/rad)
      a12=2*box(1)*box(2)*cosc
      a23=2*box(2)*box(3)*cosa
      a31=2*box(3)*box(1)*cosb
      nconn=0
      do i=1,nrest
         naam1(i)=naam(i)
c         print*,'Checking atom',i
         do j=i+1,nrest
C           Determine shortest distance i,j, possibly after translating j
            dx=xr(3*i-2)-xr(3*j-2)
            dy=xr(3*i-1)-xr(3*j-1)
            dz=xr(3*i  )-xr(3*j  )
            dx=dx-nint(dx)
            dy=dy-nint(dy)
            dz=dz-nint(dz)
            rij=1000
            do k1=-2,2
               do k2=-2,2
                  do k3=-2,2
                     d1=dx+k1
                     d2=dy+k2
                     d3=dz+k3
                     r=sqrt(a11*d1**2+a22*d2**2+a33*d3**2
     +                    +a12*d1*d2+a23*d2*d3+a31*d3*d1)
                     if (r.lt.rij) rij=r
                  enddo
               enddo
            enddo
            if (naam(i)(1:1).eq.'H' .and. naam(j)(1:1).eq.'H')
     +           goto 190
            if ((naam(i)(1:1).eq.'H' .or. naam(j)(1:1).eq.'H') .and. 
     +           rij.gt.0.130) goto 190
            if ((naam(i)(1:1).eq.'F' .or. naam(j)(1:1).eq.'F') .and. 
     +           rij.gt.0.160) goto 190
            if ((naam(i)(1:1).eq.'I' .or. naam(j)(1:1).eq.'I') .and. 
     +           rij.lt.0.220) goto 180
            if ((naam(i)(1:2).eq.'Cu' .or. naam(j)(1:2).eq.'Cu') .and. 
     +           rij.lt.0.255) goto 180
            if (rij.gt.0.198) goto 190
  180       continue
            nconn=nconn+1
            at1(nconn)=i
            at2(nconn)=j
            if (iprint.gt.3) print 43,nconn,naam(i),naam(j),i,j,rij
  190       continue
         enddo
      enddo
      print*,nconn,' bonds found from coordinates'
      write(16,*) nconn,' bonds found from coordinates'

C     Check for separate molecules and translate atoms so as to make
C         connected groups for each molecule.
      do i=1,nrest
         igr(i)=0
      enddo
      nmol=1
      k=0
      igr(1)=1
  200 continue
      k0=k
      do k=1,nconn
         do i=1,nrest
            if (igr(i).eq.nmol) then
C              Search for atoms j to be attached to i
               j=0
               if (at1(k).eq.i) j=at2(k)
               if (at2(k).eq.i) j=at1(k)
               if (j.gt.0 .and. igr(j).eq.0) then
                  igr(j)=nmol
C                 Determine translation of atom j
c                     print*,i,naam(i),j,naam(j)
c                     print*,(xr(3*i-3+m)-xr(3*j-3+m),m=1,3)
                  dx=xr(3*i-2)-xr(3*j-2)
                  dy=xr(3*i-1)-xr(3*j-1)
                  dz=xr(3*i  )-xr(3*j  )
                  dx=dx-nint(dx)
                  dy=dy-nint(dy)
                  dz=dz-nint(dz)
                  rij=1000
                  do k1=-2,2
                     do k2=-2,2
                        do k3=-2,2
                           d1=dx+k1
                           d2=dy+k2
                           d3=dz+k3
                           r=sqrt(a11*d1**2+a22*d2**2+a33*d3**2
     +                          +a12*d1*d2+a23*d2*d3+a31*d3*d1)
                           if (r.lt.rij) then
                              rij=r
                              scr(1)=d1
                              scr(2)=d2
                              scr(3)=d3
                           endif
                        enddo
                     enddo
                  enddo
                  do m=1,3
                     xr(3*j-3+m)=xr(3*i-3+m)-scr(m)
                  enddo
c                  print*,(xr(3*i-3+m)-xr(3*j-3+m),m=1,3)
               endif
            endif
         enddo
      enddo
      k=0
      do i=1,nrest
         if (igr(i).eq.nmol) k=k+1
      enddo
      if (k.gt.k0) goto 200
      print*,'Molecule ',nmol,' found with',k,' atoms'
      write(16,*) 'Molecule ',nmol,' found with',k,' atoms'
c      print*,(igr(i),i=1,nrest)
      do i=1,nrest
         if (igr(i).eq.0) then
            k=0
            nmol=nmol+1
            igr(i)=nmol
            goto 200
         endif
      enddo
      print*,nmol,' separate molecules found from coordinates'
      write(16,*) nmol,' separate molecules found from coordinates'
      if (nmol.gt.maxmol) then
         print*,' Increase MAXMOL, now ',maxmol
         write(16,*) ' Increase MAXMOL, now ',maxmol
         stop
      endif

C     Find connectivity and shells from topology
      call ident(nrp,nrest,nconn,naam1,igr,at1,at2,
     +     ngr1,nmol,index,iprint)
      if (ngr1.eq.0) then
         print*,' SPEPOS found error in IDENT'
         write(16,*) ' SPEPOS found error in IDENT'
         return
      endif
      nind=ngr1*nrp
      if (iprint.gt.0) then
         print*,nrest,' atoms found'
         print*,nmol,' molecules found'
         print*,ngr1,' topology groups found'
         print*,nind,' atoms should be present'
      endif

C     Check presence of H atoms and rearrange coordinates
      naam1(0)='absent'
      do i=1,nind
         igr(i)=0
         j=index(i)
         naam(i)=naam1(j)
         cgcor(i)=cg1(j)
         if (j.eq.0) then
            if (iprint.gt.1) print*,' H atom',i,' absent'
            do m=1,3
               x(3*i-3+m)=0
            enddo
         else
            do m=1,3
               x(3*i-3+m)=xr(3*j-3+m)
            enddo
         endif
         if (iprint.gt.3) print 36,i,j,naam(i),(x(3*i-3+m),m=1,3)
      enddo
C     End creating NIND atoms arranged in NGR1 topology groups

C     Check for additional operators that must be eliminated
      do i=1,maxop
         table(0,i)=i
         do j=1,maxop
            table(i,j)=0
         enddo
      enddo
      do iop=1,nspgr
         iop3=3*iop-3
         if (iprint.gt.1) print 45,iop,
     +        (msp(iop3+m),m=1,3),(tsp(iop3+m),m=1,3)
         do n1=1,ngr1
            do n2=1,nrp
               i=(n1-1)*nrp+n2
               if (naam(i).ne.'absent') then
                  do m=1,3
                     scr(m)=msp(iop3+m)*x(3*i-3+m)+tsp(iop3+m)
                  enddo
                  do m1=1,ngr1
                     do m2=1,nrp
                        j=(m1-1)*nrp+m2
                        do m=1,3
                           d=abs(x(3*j-3+m)-scr(m))
                           if (abs(d-nint(d)).gt.0.01) goto 220
                        enddo
                        igr(i)=m1
  220                   continue
                     enddo
                  enddo
               endif
            enddo
c            print*,'Group',n1,' transforms to',(igr(i),i=1,nind)
            do m1=1,ngr1
               k=0
               do m2=1,nrp
                  m=igr((m1-1)*nrp+m2)
                  if (m.ne.0 .and. k.eq.0) k=m
                  if (m.ne.0 .and. m.ne.k) 
     +                 print*,'SPEPOS ERROR',m1,m2,m,k
               enddo
               table(iop,m1)=k
            enddo
         enddo
      enddo

C     Study the transformation table
      if (iprint.gt.1) print*,' Transformation table: groups  ',
     +     '1   2   3   4 ... transform into:'
      ielop(0)=-10
      do iop=1,nspgr
         ielop(iop)=-10
         do j=1,ngr1
            do m=1,ngr1
               if (table(iop,m).eq.j) goto 240
            enddo
            STOP'ERROR in SPEPOS'
  240       continue
         enddo
C        Check for special positions
         k=-10
         do i=0,iop-1
            if (ielop(i).eq.-10) then
               do m=1,ngr1
                  if (table(iop,m).eq.table(i,m)) ielop(iop)=i
               enddo
            endif
         enddo
         if (iprint.gt.1) print 42,iop,(table(iop,m),m=1,ngr1)
      enddo
C     Collect remaining operators
      nspgr2=0
      do iop=1,nspgr
         if (ielop(iop).eq.-10) then
            do m=1,3
               msp(3*nspgr2+m)=msp(3*iop-3+m)
               tsp(3*nspgr2+m)=tsp(3*iop-3+m)
            enddo
            nspgr2=nspgr2+1
            imem(nspgr2)=iop
         else
            if (iprint.gt.1) print*,' Operator',iop,' brings a group',
     +           ' transformed by operator',ielop(iop),' into itself'
         endif
      enddo

      if (iprint.gt.-1) then
         do i=1,nspgr2
            m3=0
            k3=0
            do m=1,3
               iscr(m)=msp(3*i-3+m)
               m3=m3+iscr(m)
               iscr(m+3)=abs(nint(2*tsp(3*i-3+m)))
               k3=k3+iscr(m+3)
            enddo
            if (m3.eq.-3) tekst='   i'
            if (m3.eq. 3) then
               if (k3.eq.3) then 
                  tekst='I'
               elseif (k3.eq.2) then
                  if (iscr(4).eq.0) tekst='A'
                  if (iscr(5).eq.0) tekst='B'
                  if (iscr(6).eq.0) tekst='C'
               else
                  tekst='X - unknown'
               endif
            endif
            if (m3.eq.-1) then
               if (iscr(1).eq.1) then
                  if (iscr(4).eq.0) tekst='   2  .  .'
                  if (iscr(4).eq.1) tekst='  21  .  .'
               endif
               if (iscr(2).eq.1) then
                  if (iscr(5).eq.0) tekst='   .  2  .'
                  if (iscr(5).eq.1) tekst='   . 21  .'
               endif
               if (iscr(3).eq.1) then
                  if (iscr(6).eq.0) tekst='   .  .  2'
                  if (iscr(6).eq.1) tekst='   .  . 21'
               endif
            endif
            if (m3.eq.1) then
               if (iscr(1).eq.-1) then
                  if (iscr(5).eq.0.and.iscr(6).eq.0) tekst='   m  .  .'
                  if (iscr(5).eq.0.and.iscr(6).eq.1) tekst='   c  .  .'
                  if (iscr(5).eq.1.and.iscr(6).eq.0) tekst='   b  .  .'
                  if (iscr(5).eq.1.and.iscr(6).eq.1) tekst='   n  .  .'
               endif
               if (iscr(2).eq.-1) then
                  if (iscr(4).eq.0.and.iscr(6).eq.0) tekst='   .  m  .'
                  if (iscr(4).eq.0.and.iscr(6).eq.1) tekst='   .  c  .'
                  if (iscr(4).eq.1.and.iscr(6).eq.0) tekst='   .  a  .'
                  if (iscr(4).eq.1.and.iscr(6).eq.1) tekst='   .  n  .'
               endif
               if (iscr(3).eq.-1) then
                  if (iscr(4).eq.0.and.iscr(5).eq.0) tekst='   .  .  m'
                  if (iscr(4).eq.0.and.iscr(5).eq.1) tekst='   .  .  b'
                  if (iscr(4).eq.1.and.iscr(5).eq.0) tekst='   .  .  a'
                  if (iscr(4).eq.1.and.iscr(5).eq.1) tekst='   .  .  n'
               endif
            endif
            print 44,imem(i),(iscr(m),m=1,3),(tsp(3*i-3+m),m=1,3),tekst
            write(16,44)imem(i),(iscr(m),m=1,3),(tsp(3*i-3+m),m=1,3),
     +           tekst
         enddo
      endif

C     Collect remaining groups
      ngr2=0
      do i=1,ngr1
         igr(i)=0
      enddo
      do i=1,ngr1
         if (igr(i).eq.0) then
            do iop=0,nspgr
c                if (iprint.gt.1) print*,i,iop,table(iop,i)
               igr(table(iop,i))=i
            enddo
         endif
         if (iprint.gt.1) print*,(igr(j),j=1,ngr1)
         do j=1,ngr1
            if (igr(j).eq.i) then
               ngr2=ngr2+1
               if (iprint.gt.1) print*,'Keeping group',i,
     +              ' as independent group',ngr2
               do k=1,nrp
                  do m=1,3
                     xr(3*(nrp*(ngr2-1)+k-1)+m)=x(3*(nrp*(i-1)+k-1)+m)
                  enddo
c                  print*,k,(xr(3*(nrp*(ngr2-1)+k-1)+m),m=1,3)
               enddo
               goto 280
            endif
         enddo
  280    continue
      enddo
      ngr1=ngr2
      nspgr=nspgr2
c      print*,nop,nspgr,ispgr,'---',spname,'---'
c      pause
      
      if (nop.ne.nspgr+1 .or. ispgr.lt.0) then
C        Prepare additional symmetry information for unknown space group
         if (spname.ne.'unknown' .and. spname(1:2).ne.'r-') 
     +        spname='r-'//spname(1:6)
         if (ispgr.eq.-10) ispgr=1
         k=0
         do m=1,3
            msom(m)=1
            iscr(m)=-10
            if (nspgr.eq.0) iscr(m)=1
         enddo
         do iop=1,nspgr
            iop3=3*iop-3
            m3=0
            do m=1,3
               ts=tsp(iop3+m)
               ms=msp(iop3+m)
               msom(m)=msom(m)+ms
               m3=m3+ms
c               print*,iop,m,ms,ts
               ts=2*ts
               ts=ts-nint(ts)
               if (abs(ts).gt.0.001) then
                  print*,' ***** Unacceptable shift',ts/2,
     +                 ' in space group ',spname
                  stop
               endif
            enddo
            do m=1,3
               if (iscr(m).ne.0) then
                  ms=msp(iop3+m)
                  if (abs(m3).eq.3) iscr(m)=1
                  if (abs(m3).eq.1) iscr(m)=abs((ms-m3)/2)
               endif
            enddo
c            print*,iop,'    ',(iscr(m),m=1,3)
         enddo

         do m=1,12
            iref(m)=1
         enddo
         iref(4)=iscr(3)
         iref(5)=iscr(2)
         iref(6)=iscr(1)
C        a unique - cy free; b unique - cx free; c unique - bx free
         do m=1,3
            if (iscr(m).eq.-10) then
               print 34,spname,m,ms,m3,iscr(m)
               stop
            endif
            if (msom(m).ne.0) then
C              No translational constraint in direction m
               if (msom(m).ne.nspgr+1) then
                  print 32,spname,m,msom(m),nspgr+1,iref(m+6)
                  stop
               endif
               iref(m+6)=0
               k=k+msom(m)
            endif
         enddo
         if (k.eq.0) inner=0
         if (iprint.gt.0) print 33,(msom(m),m=1,3)
         if (inner.eq.0 .and. iprint.gt.0) print 31
      endif
      print 35,(iref(m),m=1,12)
      write(16,35) (iref(m),m=1,12)
c      pause

      do i=1,nind
         do m=1,3
            x(3*i-3+m)=xr(3*i-3+m)*box(m)
         enddo
c         print*,i,'   ',naam(i),(x(3*i-3+m),m=1,3)
c         write(29,*) 'ATOM    ',naam(i),(x(3*i-3+m),m=1,3)
      enddo

      nfound=ngr1*nrp
      if (iprint.gt.1) print*,'End SPEPOS'

c   20 format(a8)
   31 format(' No calculation of U2-inner')
   32 format(' ***** Error in translational constraint for',
     +     ' space group ',a8,6i4)
   33 format(' Dipole coefficients',3i5)
   34 format(' ***** Error in cel angle constraint for',
     +     ' space group ',a8,6i4)
   35 format('IREF ',12i4)
   36 format(2i4,3x,a,3f10.5)
   37 format(' SPEPOS checks for special positions in space group',i5,
     +     ' named ',a,' with', i4,' symmetry operators')
   38 format(2i4,f6.1,3x,3i4)
   41 format(i4,' symmetry operations on',i4,
     +     ' atoms leave',i4,' non-coinciding atoms')
   42 format(4x,'Operation',i4,12x,64i4)
   43 format(i4,4x,2a5,2i4,f8.3)
   44 format('Remaining operator',i4,3x,3i4,3x,3f6.1,4x,a)
   45 format('  Operator',i4,3x,3i4,3x,3f5.1)
      end
