      subroutine test(par,n,func,step)

      implicit none
      include 'params.i'
      external func
      dimension par(*),g(maxpar),gn(maxpar),p1(maxpar),scr(maxpar)

C     Numerical test on the correctness of the calculation of energy
C     and forces by a function FUNC(n,par,g,en,isubr).
C        This function depends on N parameters PAR;
C        EN delivers the value of the function;
C        G delivers the "forces" (negative derivatives of EN to PAR(1..N))
C        ISUBR=1: call NONBE to make pair list(s);
C             =2: call LIST to use previously made pair list(s).
C     FUNC is first called with ISUBR=1, then with ISUBR=2 for comparison
C     between NONBE and LIST.
C     (If ISUBR=1 is un-commented, only NONBE is tested)
C     Also, the forces are calculated by increasing and decreasing the
C     parameters by STEP and determining the derivatives numerically.
C     Averaging these two results should give a good approximation to
C     the analytical derivatives delivered by FUNC.
C     STEP should be about 10^(-3) in single precision and 10^(-6) in
C     double precision.

      integer i,isubr,n
      double precision der,e0,e1,e2,en,g,gn,p1,par,scr,step
      isubr=2
c      isubr=1
      print*,' Calling TEST with ISUBR =',isubr
      call func(n,par,gn,en,1)
      call func(n,par,g,e0,isubr)
      do i=1,n
         p1(i)=par(i)
      enddo
      print 10, step
      print 20, en
      if (isubr.eq.2) print 30, e0
      if (isubr.eq.1) print 20, e0
      print 50
      print 60
      do i=1,n
         p1(i)=par(i)+step
         call func(n,p1,scr,e1,isubr)
         p1(i)=par(i)-step
         call func(n,p1,scr,e2,isubr)
         der=(e1-e2)/(2*step)
         print 70,i,par(i),(e1-e0)/step,(e0-e2)/step,
     +        der,(der+gn(i)),(der+g(i)),e1,e2
         p1(i)=par(i)
      enddo
      call func(n,par,g,e0,isubr)
C     This last call restores the original coordinate array X.

 10   format(' Test forces (= negative derivatives), step',f12.8)
 20   format('    Function value from Nonbe:',f14.7)
 30   format('    Function value from List: ',f14.7)
 50   format(60x,'Difference with:')
 60   format('Check       parameter        der-low    der-high    ',
     +     'der-aver   der-nonbe    der-list',11x,'e-low',15x,'e-high')
 70   format('Check',i6,f10.5,3x,5f12.4,2x,2f20.10)
      end
