      subroutine transf(box,nin,boxpp,icond,iref,wrot,iprint)
C     Transforms the system A (upper trianglar) to A' = A.N,
C     then search for a rotation matrix W which transforms A' into A"
C     through A" = W.A' in such a way that A" is again upper triangular.
C     Does not work for F-centering.

C     Input through parameters:
C        BOX: BOX(1..6) contains A,
C             BOX(7..9) contains A.t (zwaartepuntsvector)
C        NIN(1..3,1..3) contains N
C        IREF(4..9) =  0/1: parameters constrained/free (cf. LEESR)
C           cell angles:              4: BX    5: CX      6: CY
C           molecular centers:        7: X     8: Y       9: Z
C        IPRINT regulates output to unit 16 and to the screen.
C            Output starts if IPRINT>4, up to and including IPRINT=8.

C     Output through parameters: :
C        BOXPP(1..9) contains A" and A".t"
C        WROT(1..3,1..3) contains W
C        ICOND(M), with M=1..3, gives the status of the transformation:
C           ICOND(M)= 0: no condition, choose TAUP(M)=0. This must be
C                        compatible with the space group symmetry.
C           ICOND(M)= 1: translation obligatory,
C                        see TAUP(M) (+/- multiple of 1/2)
C           ICOND(M)=10: contradiction, operation impossible.
C        ICOND(4) reports the error status:
C           ICOND(4)= 0: no problems
C           ICOND(4)> 0: various error messages;
C                        no calculation of BOXPP and WROT.
C           ICOND(4)=1: elements of N deviate too much from TOL
C           ICOND(4)=2: det N not equal 1
C           ICOND(4)=3: M1, M2 not equivalent
C           ICOND(4)=4: No solution for the shift vector
C        The following errors should not occur:
C           ICOND(4)=5: More then one shift vector
C           ICOND(4)=6: shift should not be free in this space group
C           ICOND(4)=7: shift should be free in this space group
C           ICOND(4)=8: angle should be 90 degrees in this space group
C           ICOND(4)=9: volumes different after transformation

C     Input through common block /SPGR/:
C        NSPGR, MSP, TSP: space group information, see subroutine LEESR

C     Calls subroutines MATMAT and INVBOX.

      implicit none
      include 'params.i'
      include 'cspgr.i'
      integer i,i3,icond,idet,ih,ihit,ind,iprint,iref,ish,ishi,itau,
     +     iused,j,j3,k,m,mi,mj,n,ngoed,nhit,nin
      double precision aptild,as,atild,box,boxpp,co,s,scr,taup,tr,
     +     vol0,vol1,wrot
      dimension box(*),nin(3,3),boxpp(*),icond(*)
      dimension atild(3,3),aptild(3,3),as(3),co(3),
     +     scr(3,3),wrot(3,3),taup(3),tr(3),iref(12),
     +     nhit(mxspgr),ihit(mxspgr,8),itau(mxspgr,8,3),ishi(3),
     +     ind(mxspgr),iused(mxspgr)

      icond(4)=0
      idet=nin(1,1)*(nin(2,2)*nin(3,3)-nin(2,3)*nin(3,2))
     +     +nin(1,2)*(nin(2,3)*nin(3,1)-nin(2,1)*nin(3,3))
     +     +nin(1,3)*(nin(2,1)*nin(3,2)-nin(3,1)*nin(2,2))
      if (idet.ne.1) then
         if (iprint.gt.6) then
            print*,' ***** det N is not equal to 1: ' ,idet
            write(16,*) ' ***** det N is not equal to 1: ' ,idet
         endif
         icond(4)=2
         icond(4)=2
         return
      endif

C     Find out which M-operations can correspond to each other
      do i=1,nspgr
         i3=3*i-3
         ih=0
         do j=1,nspgr
            j3=3*j-3
            do mi=1,3
               do mj=1,3
                  if (nin(mi,mj).ne.0 .and.
     +                 msp(i3+mi).ne.msp(j3+mj)) then
                     if (iprint.gt.8) print*,i,j,' NK not equal KN'
                     goto 170
                  endif
               enddo
            enddo

            do mi=1,3
               s=tsp(i3+mi)
               do mj=1,3
                  s=s-nin(mi,mj)*tsp(j3+mj)
               enddo
               s=(s-nint(s))
               if (msp(i3+mi).eq.1) then
C                 s must be zero, then tau is free
                  if (abs(s).gt.0.001d0) then
                     if (iprint.gt.8) print*,i,j,s,mi,
     +                    ' K=1, but s'' not integer'
                     goto 170
                  endif
                  itau(i,ih+1,mi)=-80
               else
C                 tau is not free
                  if (iprint.gt.8) print*,i,j,s,mi
                  itau(i,ih+1,mi)=abs(nint(4*s))
               endif
            enddo
            ih=ih+1
            ihit(i,ih)=j
 170        continue
         enddo
         if (ih.eq.0) then
            if (iprint.gt.6) print*,
     +           ' ***** No equivalent operation for', i
            icond(4)=3
            return
         endif
         nhit(i)=ih
      enddo

      if (iprint.gt.7) then
         print*,' Resulting shifts (-10 means no condition):'
         do i=1,nspgr
            do ih=1,nhit(i)
               print*,i,ihit(i,ih),(itau(i,ih,m)/8.0d0,m=1,3)
            enddo
         enddo
      endif

C     Find out whether a combination of these shifts is possible
      ngoed=0
      do i=1,nspgr
         ind(i)=1
      enddo
 220  continue
C     Eliminate wrong combinations
      do i=1,nspgr
         iused(i)=0
      enddo
      do i=1,nspgr
         k=ihit(i,ind(i))
         if (iused(k).eq.1) goto 300
         iused(k)=1
      enddo
      if (iprint.gt.7) print 50,(ihit(i,ind(i)),i=1,nspgr)
C     Find the shifts
      do i=1,3
         ishi(i)=-80
      enddo
      do i=1,nspgr
c         k=ihit(i,ind(i))
         do m=1,3
            ish=itau(i,ind(i),m)
            if (ish.ge.0) then
               if (ishi(m).lt.0) ishi(m)=ish
               if (ishi(m).ne.ish) then
                  if (iprint.gt.7) print*,' No shift possible'
                  goto 300
               endif
            endif
         enddo
      enddo
C     A good shift vector TAUP (=tau') found
      ngoed=ngoed+1
      do m=1,3
         if (ishi(m).ge.0) then
            taup(m)=ishi(m)/8.0d0
            icond(m)=1
         else
            taup(m)=0
            icond(m)=0
         endif
      enddo
      if (iprint.gt.7) then
         print*,' shift = ',(taup(m),m=1,3)
         print*,' icond = ',(icond(m),m=1,3)
         do i=1,nspgr
            print 60,i,ihit(i,ind(i))
         enddo
      endif

 300  continue
C     Try the next possibility for combination (see program RECURSION)
      i=1
 330  continue
      if (i.gt.nspgr) goto 350
      ind(i)=ind(i)+1
      if (ind(i).gt.nhit(i)) then
         ind(i)=1
         i=i+1
         goto 330
      endif
      goto 220
 350  continue

C     Evaluate results.
      if (ngoed.eq.0) then
         if (iprint.gt.6) print*,' ***** No solution'
         icond(4)=4
         return
      endif
      if (ngoed.gt.1) then
         print*,' ***** More than one good solution!!!!!'
         print*,' ***** The program needs to be extended'
C        This may occur in F-centered space groups.
         icond(4)=5
         stop
      endif
      if (iprint.gt.7) print*,'A solution found'
      do m=1,3
         if (icond(m).eq.0 .and. iref(m+6).ne.0) then
            print*,m,' ***** This shift not free in this space group'
            icond(4)=6
            stop
         endif
         if (icond(m).eq.1 .and. iref(m+6).eq.0) then
            print*,m,' ***** This shift free in this space group'
            icond(4)=7
            stop
         endif
      enddo

C     Carry out the transformation
      atild(1,1)=box(1)
      atild(1,2)=0
      atild(1,3)=0
      atild(2,1)=box(4)
      atild(2,2)=box(2)
      atild(2,3)=0
      atild(3,1)=box(5)
      atild(3,2)=box(6)
      atild(3,3)=box(3)
      vol0=box(1)*box(2)*box(3)
      do m=1,3
         co(m)=0
         as(m)=0
         do n=1,3
            scr(m,n)=nin(n,m)
         enddo
      enddo
      call matmat(aptild,scr,atild)
C     A'(transp) = N(transp).A(transp), so A'=A.N
      do m=1,3
         co(1)=co(1)+aptild(2,m)*aptild(3,m)
         co(2)=co(2)+aptild(1,m)*aptild(3,m)
         co(3)=co(3)+aptild(1,m)*aptild(2,m)
         do n=1,3
            as(n)=as(n)+aptild(n,m)**2
         enddo
      enddo
      do m=1,3
         as(m)=sqrt(as(m))
      enddo
      co(1)=co(1)/(as(2)*as(3))
      co(2)=co(2)/(as(1)*as(3))
      co(3)=co(3)/(as(1)*as(2))
      boxpp(1)=as(1)
      boxpp(4)=as(2)*co(3)
      boxpp(2)=sqrt(as(2)**2-boxpp(4)**2)
      boxpp(5)=as(3)*co(2)
      boxpp(6)=(as(2)*as(3)*co(1)-boxpp(4)*boxpp(5))/boxpp(2)
      boxpp(3)=sqrt(as(3)**2-boxpp(5)**2-boxpp(6)**2)
      vol1=boxpp(1)*boxpp(2)*boxpp(3)

      call invbox(boxpp,scr)
      call matmat(wrot,scr,aptild)
C     W = A"(transp)(inv).A'(transp), so W = W(transp)(inv)=A".A'(inv)

C     Do some tests
      do m=4,6
         if (iref(m).eq.0 .and. abs(boxpp(m)).gt.0.001d0) then
            print*,m,' ***** This angle should be 90 degrees'
            icond(4)=8
            stop
         endif
      enddo
      if (abs(vol0-vol1).gt.0.001d0) then
         print*,' ***** Volumes different: ',vol0,vol1
         icond(4)=9
         stop
      endif

C     Calculate the new centers of gravity.
      do m=1,3
         tr(m)=box(m+6)
         do j=1,3
            tr(m)=tr(m)-atild(j,m)*taup(j)
         enddo
         if (iprint.gt.7) print*,taup(m),tr(m)
      enddo
C     TR = A.t-A.tau'
      do m=1,3
         boxpp(m+6)=0
         do n=1,3
            boxpp(m+6)=boxpp(m+6)+wrot(m,n)*tr(n)
         enddo
      enddo
C     A".t" = W.A.(t-tau') = W.A.t - A".tau, or t" = N(inv).t - tau

 50   format(' Try combination ',8i4)
 60   format('  To operation',i3,' in structure 1 ',
     +     'corresponds operation',i3,' in structure 2')
      end


      subroutine matmat(a,b,c)
C     Calculates matrix product A = B.C for 3*3 matrices.
      implicit none
      integer i,j,k
      double precision a,b,c
      dimension a(3,3),b(3,3),c(3,3)
      do i=1,3
         do j=1,3
            a(i,j)=0
            do k=1,3
               a(i,j)=a(i,j)+b(i,k)*c(k,j)
            enddo
         enddo
      enddo
      end


      subroutine invbox(box,atrinv)
C     Inverts the matrix A corresponding to BOX(1..6), giving ATRINV
C     which is the transpose of the inverse (so lower diagonal).
      implicit none
      double precision atrinv,ax,box,bx,by,cx,cy,cz
      dimension box(*),atrinv(3,3)
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      atrinv(1,1)=1/ax
      atrinv(2,1)=-bx/(ax*by)
      atrinv(3,1)=(bx*cy-by*cx)/(ax*by*cz)
      atrinv(1,2)=0
      atrinv(2,2)=1/by
      atrinv(3,2)=-cy/(by*cz)
      atrinv(1,3)=0
      atrinv(2,3)=0
      atrinv(3,3)=1/cz
      end
