      subroutine tri(nr,npa,x,it,cosa,cosb,cosc)

C     Transforms Cartesian to triclinic coordinates and back.
C     This is an extension of the Gromos routine TRACO (W.F. van Gunsteren).

C     NR: number of atoms to be transformed
C     NPA: number of preceeding atoms to be skipped
C     X(1..) contains the coordinates
C     IT rules the transformation:
C        IT > 0: Cartesian to triclinic
C        IT < 0: triclinic to Cartesian
C     COSA, COSB, COSC: the cosines of the three cell angles

C     As always, the a-axis coincides with the x-axis and the b-axis
C     lies in the x-y plane.

      implicit none
      integer i3,it,j,npa,nr
      double precision cccs,cosa,cosb,cosc,sinc,vabc
      double precision x(*)
      i3=3*npa+1
      sinc=sqrt(1-cosc**2)
      vabc=1-cosa**2-cosb**2-cosc**2+2*cosa*cosb*cosc
      if (vabc.lt.0) then
         print*,'***** Error in TRI: unacceptable cosines'
         print*,cosa,cosb,cosc,'   giving ',vabc
         stop
      endif
      vabc=sqrt(vabc)
      cccs=(cosa-cosb*cosc)/sinc
C     IT > 0: Cartesian to triclinic
      if (it.gt.0) then
         do j=1,nr
            x(i3)=x(i3)-(cosc/sinc)*x(i3+1)+
     +           (cosc*cccs-sinc*cosb)*x(i3+2)/vabc
            x(i3+1)=x(i3+1)/sinc-(cccs/vabc)*x(i3+2)
            x(i3+2)=(sinc/vabc)*x(i3+2)
            i3=i3+3
         enddo
      endif

C     IT < 0: triclinic to Cartesian
      if (it.lt.0) then
         do j=1,nr
            x(i3)=x(i3)+cosc*x(i3+1)+cosb*x(i3+2)
            x(i3+1)=sinc*x(i3+1)+cccs*x(i3+2)
            x(i3+2)=(vabc/sinc)*x(i3+2)
            i3=i3+3
         enddo
      endif
      end
